(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@wm/transpiler'), require('@wm/core'), require('@angular/compiler')) :
    typeof define === 'function' && define.amd ? define('@wm/buildTask', ['exports', '@wm/transpiler', '@wm/core', '@angular/compiler'], factory) :
    (global = global || self, factory((global.wm = global.wm || {}, global.wm.buildTask = {}), global.wm.transpiler, global.wm.core, global.ng.compiler));
}(this, (function (exports, transpiler, core, compiler) { 'use strict';

    var carouselTagName = 'carousel';
    var dataSetKey = 'dataset';
    var idGen = new core.IDGenerator('wm_carousel_ref_');
    var isDynamicCarousel = function (node) { return node.attrs.find(function (attr) { return attr.name === 'type' && attr.value === 'dynamic'; }); };
    var ɵ0 = isDynamicCarousel;
    transpiler.register('wm-carousel', function () {
        return {
            pre: function (attrs, shared) {
                // generating unique Id for the carousel
                var counter = idGen.nextUid();
                shared.set('carousel_ref', counter);
                return "<div class=\"app-carousel carousel\"><" + carouselTagName + " wmCarousel #" + counter + "=\"wmCarousel\"  " + transpiler.getAttrMarkup(attrs) + " interval=\"0\" [ngClass]=\"" + counter + ".navigationClass\">";
            },
            post: function () { return "</" + carouselTagName + "></div>"; },
            template: function (node) {
                // check if the carousel is dynamic
                if (isDynamicCarousel(node)) {
                    var datasetAttr = node.attrs.find(function (attr) { return attr.name === dataSetKey; });
                    var widgetNameAttr = node.attrs.find(function (attr) { return attr.name === 'name'; });
                    if (!datasetAttr) {
                        return;
                    }
                    var boundExpr = transpiler.getBoundToExpr(datasetAttr.value);
                    if (!boundExpr) {
                        return;
                    }
                    core.updateTemplateAttrs(node, boundExpr, widgetNameAttr.value);
                }
            },
            // To provide parent carousel reference for children
            provide: function (attrs, shared) {
                var provider = new Map();
                provider.set('carousel_ref', shared.get('carousel_ref'));
                return provider;
            }
        };
    });
    var carousel_build = (function () { });

    var carousel_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': carousel_build,
        ɵ0: ɵ0
    });

    var carouselContentTagName = 'slide';
    var counter = 1;
    // For static carousel
    transpiler.register('wm-carousel-content', function () {
        return {
            pre: function (attrs) { return "<" + carouselContentTagName + " wmCarouselTemplate  " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + carouselContentTagName + ">"; }
        };
    });
    // For dynamic carousel
    transpiler.register('wm-carousel-template', function () {
        return {
            requires: ['wm-carousel'],
            pre: function (attrs, shared, parentCarousel) {
                var carouselRef = parentCarousel.get('carousel_ref');
                return "<div *ngIf=\"!" + carouselRef + ".fieldDefs\">{{" + carouselRef + ".nodatamessage}}</div>\n                    <" + carouselContentTagName + " wmCarouselTemplate  " + transpiler.getAttrMarkup(attrs) + " *ngFor=\"let item of " + carouselRef + ".fieldDefs; let i = index;\">\n                        <ng-container [ngTemplateOutlet]=\"carouselTempRef" + counter + "\" [ngTemplateOutletContext]=\"{item:item, index:i}\"></ng-container>\n                    </" + carouselContentTagName + ">\n                    <ng-template #carouselTempRef" + counter++ + " let-item=\"item\" let-index=\"index\">";
            },
            post: function () { return "</ng-template>"; }
        };
    });
    var carouselTemplate_build = (function () { });

    var carouselTemplate_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': carouselTemplate_build
    });

    var tagName = 'div';
    transpiler.register('wm-login', function () {
        return {
            pre: function (attrs) { return "<" + tagName + " wmLogin " + transpiler.getAttrMarkup(attrs) + " eventsource.bind=\"Actions.loginAction\">"; },
            post: function () { return "</" + tagName + ">"; },
            provide: function () {
                var provider = new Map();
                provider.set('isLogin', true);
                return provider;
            }
        };
    });
    var login_build = (function () { });

    var login_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': login_build
    });

    var tagName$1 = 'marquee';
    transpiler.register('wm-marquee', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1 + " onmouseover=\"this.stop();\" onmouseout=\"this.start();\" wmMarquee role=\"marquee\" aria-live=\"off\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1 + ">"; }
        };
    });
    var marquee_build = (function () { });

    var marquee_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': marquee_build
    });

    var tagName$2 = 'a';
    transpiler.register('wm-anchor', function () {
        return {
            pre: function (attrs) { return "<" + tagName$2 + " wmAnchor role=\"link\" data-identifier=\"anchor\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$2 + ">"; }
        };
    });
    var anchor_build = (function () { });

    var anchor_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': anchor_build
    });

    var tagName$3 = 'div';
    transpiler.register('wm-audio', function () {
        return {
            pre: function (attrs) { return "<" + tagName$3 + " wmAudio " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$3 + ">"; }
        };
    });
    var audio_build = (function () { });

    var audio_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': audio_build
    });

    var tagName$4 = 'div';
    transpiler.register('wm-html', function () {
        return {
            pre: function (attrs) { return "<" + tagName$4 + " wmHtml " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$4 + ">"; }
        };
    });
    var html_build = (function () { });

    var html_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': html_build
    });

    var tagName$5 = 'span';
    transpiler.register('wm-icon', function () {
        return {
            pre: function (attrs) { return "<" + tagName$5 + " wmIcon aria-hidden=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$5 + ">"; }
        };
    });
    var icon_build = (function () { });

    var icon_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': icon_build
    });

    var tagName$6 = 'div';
    transpiler.register('wm-iframe', function () {
        return {
            pre: function (attrs) { return "<" + tagName$6 + " wmIframe " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$6 + ">"; }
        };
    });
    var iframe_build = (function () { });

    var iframe_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': iframe_build
    });

    var tagName$7 = 'label';
    transpiler.register('wm-label', function () {
        return {
            pre: function (attrs) { return "<" + tagName$7 + " wmLabel " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$7 + ">"; }
        };
    });
    var label_build = (function () { });

    var label_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': label_build
    });

    var tagName$8 = 'img';
    transpiler.register('wm-picture', function () {
        return {
            pre: function (attrs) { return "<" + tagName$8 + " wmPicture alt=\"image\" wmImageCache=\"" + (attrs.get('offline') || 'true') + "\" " + transpiler.getAttrMarkup(attrs) + ">"; }
        };
    });
    var picture_build = (function () { });

    var picture_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': picture_build
    });

    var tagName$9 = 'div';
    transpiler.register('wm-spinner', function () {
        return {
            pre: function (attrs) { return "<" + tagName$9 + " wmSpinner role=\"loading\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$9 + ">"; }
        };
    });
    var spinner_build = (function () { });

    var spinner_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': spinner_build
    });

    var tagName$a = 'div';
    var getAttr = function (node, attrName) { return node.attrs.find(function (attr) { return attr.name === attrName; }); };
    var ɵ0$1 = getAttr;
    var getAttrValue = function (node, attrName) {
        var match = getAttr(node, attrName);
        if (match) {
            return match.value;
        }
    };
    var ɵ1 = getAttrValue;
    var getReplaceRegex = function (v) { return new RegExp("bind:(" + v + "|" + v + "\\[\\$i])\\.", 'g'); };
    var ɵ2 = getReplaceRegex;
    transpiler.register('wm-progress-bar', function () {
        return {
            template: function (node) {
                var dataset = getAttrValue(node, 'dataset');
                var boundExpr = transpiler.getBoundToExpr(dataset);
                if (boundExpr) {
                    var type = getAttrValue(node, 'type');
                    var datavalue = getAttrValue(node, 'datavalue');
                    var replaceRegex = getReplaceRegex(boundExpr);
                    if (type && type.includes(boundExpr)) {
                        type = type.replace(replaceRegex, '');
                        getAttr(node, 'type').value = type;
                    }
                    if (datavalue && datavalue.includes(boundExpr)) {
                        datavalue = datavalue.replace(replaceRegex, '');
                        getAttr(node, 'datavalue').value = datavalue;
                    }
                }
            },
            pre: function (attrs) { return "<" + tagName$a + " wmProgressBar " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$a + ">"; }
        };
    });
    var progressBar_build = (function () { });

    var progressBar_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': progressBar_build,
        ɵ0: ɵ0$1,
        ɵ1: ɵ1,
        ɵ2: ɵ2
    });

    var tagName$b = 'div';
    transpiler.register('wm-progress-circle', function () {
        return {
            pre: function (attrs) { return "<" + tagName$b + " wmProgressCircle " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$b + ">"; }
        };
    });
    var progressCircle_build = (function () { });

    var progressCircle_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': progressCircle_build
    });

    var tagName$c = 'div';
    transpiler.register('wm-richtexteditor', function () {
        return {
            pre: function (attrs) { return "<" + tagName$c + " wmRichTextEditor role=\"textbox\" " + transpiler.getFormMarkupAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$c + ">"; }
        };
    });
    var richTextEditor_build = (function () { });

    var richTextEditor_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': richTextEditor_build
    });

    var tagName$d = 'div';
    transpiler.register('wm-search', function () {
        return {
            pre: function (attrs) { return "<" + tagName$d + " wmSearch " + transpiler.getFormMarkupAttr(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$d + ">"; }
        };
    });
    var search_build = (function () { });

    var search_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': search_build
    });

    var tagName$e = 'div';
    transpiler.register('wm-tree', function () {
        return {
            pre: function (attrs) { return "<" + tagName$e + " wmTree redrawable " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$e + ">"; }
        };
    });
    var tree_build = (function () { });

    var tree_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tree_build
    });

    var tagName$f = 'div';
    transpiler.register('wm-card', function () {
        return {
            pre: function (attrs) { return "<" + tagName$f + " wmCard " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$f + ">"; }
        };
    });
    var card_build = (function () { });

    var card_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': card_build
    });

    var tagName$g = 'div';
    transpiler.register('wm-card-content', function () {
        return {
            pre: function (attrs) { return "<" + tagName$g + " wmCardContent partialContainer " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$g + ">"; }
        };
    });
    var cardContent_build = (function () { });

    var cardContent_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': cardContent_build
    });

    var tagName$h = 'div';
    transpiler.register('wm-card-actions', function () {
        return {
            pre: function (attrs) { return "<" + tagName$h + " wmCardActions " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$h + ">"; }
        };
    });
    var cardActions_build = (function () { });

    var cardActions_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': cardActions_build
    });

    var tagName$i = 'div';
    transpiler.register('wm-card-footer', function () {
        return {
            pre: function (attrs) { return "<" + tagName$i + " wmCardFooter " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$i + ">"; }
        };
    });
    var cardFooter_build = (function () { });

    var cardFooter_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': cardFooter_build
    });

    var tagName$j = 'div';
    transpiler.register('wm-chart', function () {
        return {
            pre: function (attrs) { return "<" + tagName$j + " wmChart redrawable " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$j + ">"; }
        };
    });
    var chart_build = (function () { });

    var chart_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': chart_build
    });

    var tagName$k = 'div';
    transpiler.register('wm-accordion', function () {
        return {
            pre: function (attrs) { return "<" + tagName$k + " wmAccordion role=\"tablist\" aria-multiselectable=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$k + ">"; }
        };
    });
    var accordion_build = (function () { });

    var accordion_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': accordion_build
    });

    var tagName$l = 'div';
    transpiler.register('wm-accordionpane', function () {
        return {
            pre: function (attrs) { return "<" + tagName$l + " wmAccordionPane partialContainer wm-navigable-element=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$l + ">"; }
        };
    });
    var accordionPane_build = (function () { });

    var accordionPane_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': accordionPane_build
    });

    var tagName$m = 'div';
    transpiler.register('wm-container', function () {
        return {
            pre: function (attrs) { return "<" + tagName$m + " wmContainer partialContainer wmSmoothscroll=\"" + (attrs.get('smoothscroll') || 'false') + "\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$m + ">"; }
        };
    });
    var container_build = (function () { });

    var container_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': container_build
    });

    var tagName$n = 'div';
    transpiler.register('wm-gridcolumn', function () {
        return {
            pre: function (attrs) { return "<" + tagName$n + " wmLayoutGridColumn " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$n + ">"; }
        };
    });
    var layoutGridColumn_build = (function () { });

    var layoutGridColumn_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': layoutGridColumn_build
    });

    var tagName$o = 'div';
    transpiler.register('wm-gridrow', function () {
        return {
            pre: function (attrs) { return "<" + tagName$o + " wmLayoutGridRow " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$o + ">"; }
        };
    });
    var layoutGridRow_build = (function () { });

    var layoutGridRow_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': layoutGridRow_build
    });

    var tagName$p = 'div';
    transpiler.register('wm-layoutgrid', function () {
        return {
            pre: function (attrs) { return "<" + tagName$p + " wmLayoutGrid " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$p + ">"; }
        };
    });
    var layoutGrid_build = (function () { });

    var layoutGrid_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': layoutGrid_build
    });

    var tagName$q = 'div';
    transpiler.register('wm-panel', function () {
        return {
            pre: function (attrs) { return "<" + tagName$q + " wmPanel partialContainer aria-label=\"panel\" wm-navigable-element=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$q + ">"; }
        };
    });
    transpiler.register('wm-panel-footer', function () {
        return {
            pre: function (attrs) { return "<" + tagName$q + " wmPanelFooter aria-label=\"panel footer\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$q + ">"; }
        };
    });
    var panel_build = (function () { });

    var panel_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': panel_build
    });

    var tagName$r = 'div';
    transpiler.register('wm-segmented-control', function () {
        return {
            pre: function (attrs) { return "<" + tagName$r + " wmSegmentedControl " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$r + ">"; }
        };
    });
    var segmentedControl_build = (function () { });

    var segmentedControl_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': segmentedControl_build
    });

    var tagName$s = 'li';
    transpiler.register('wm-segment-content', function () {
        return {
            pre: function (attrs) { return "<" + tagName$s + " wmSegmentContent partialContainer wmSmoothscroll=" + (attrs.get('smoothscroll') || 'true') + " wm-navigable-element=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$s + ">"; }
        };
    });
    var segmentContent_build = (function () { });

    var segmentContent_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': segmentContent_build
    });

    var tagName$t = 'div';
    transpiler.register('wm-tabs', function () {
        return {
            pre: function (attrs) { return "<" + tagName$t + " wmTabs " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$t + ">"; }
        };
    });
    var tabs_build = (function () { });

    var tabs_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tabs_build
    });

    var tagName$u = 'div';
    transpiler.register('wm-tabpane', function () {
        return {
            pre: function (attrs) { return "<" + tagName$u + " wmTabPane  partialContainer " + transpiler.getAttrMarkup(attrs) + " wm-navigable-element=\"true\" role=\"tabpanel\">"; },
            post: function () { return "</" + tagName$u + ">"; }
        };
    });
    var tabPane_build = (function () { });

    var tabPane_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tabPane_build
    });

    var tagName$v = 'div';
    transpiler.register('wm-tile', function () {
        return {
            pre: function (attrs) { return "<" + tagName$v + " wmTile aria-describedby=\"Tile\" wm-navigable-element=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$v + ">"; }
        };
    });
    var tile_build = (function () { });

    var tile_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tile_build
    });

    var tagName$w = 'div';
    transpiler.register('wm-wizard', function () {
        return {
            pre: function (attrs) { return "<" + tagName$w + " wmWizard role=\"tablist\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$w + ">"; }
        };
    });
    var wizard_build = (function () { });

    var wizard_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': wizard_build
    });

    var tagName$x = 'form';
    var idGen$1 = new core.IDGenerator('wizard_step_id_');
    transpiler.register('wm-wizardstep', function () {
        return {
            pre: function (attrs) {
                var counter = idGen$1.nextUid();
                return "<" + tagName$x + " wmWizardStep #" + counter + "=\"wmWizardStep\" " + transpiler.getAttrMarkup(attrs) + ">\n                       <ng-template [ngIf]=\"" + counter + ".isInitialized\">";
            },
            post: function () { return "</ng-template></" + tagName$x + ">"; }
        };
    });
    var wizardStep_build = (function () { });

    var wizardStep_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': wizardStep_build
    });

    var tagName$y = 'button';
    transpiler.register('wm-barcodescanner', function () {
        return {
            pre: function (attrs) { return "<" + tagName$y + " wmBarcodescanner " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$y + ">"; }
        };
    });
    var barcodeScanner_build = (function () { });

    var barcodeScanner_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': barcodeScanner_build
    });

    var tagName$z = 'button';
    transpiler.register('wm-camera', function () {
        return {
            pre: function (attrs) { return "<" + tagName$z + " type='button' wmCamera " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$z + ">"; }
        };
    });
    var camera_build = (function () { });

    var camera_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': camera_build
    });

    var tagName$A = 'div';
    transpiler.register('wm-alertdialog', function () {
        return {
            pre: function (attrs) { return "<" + tagName$A + " wmAlertDialog role=\"alertdialog\" wm-navigable-element=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$A + ">"; }
        };
    });
    var alertDialog_build = (function () { });

    var alertDialog_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': alertDialog_build
    });

    var tagName$B = 'div';
    transpiler.register('wm-confirmdialog', function () {
        return {
            pre: function (attrs) { return "<" + tagName$B + " wmConfirmDialog wm-navigable-element=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$B + ">"; }
        };
    });
    var confirmDialog_build = (function () { });

    var confirmDialog_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': confirmDialog_build
    });

    var tagName$C = 'div';
    transpiler.register('wm-dialogactions', function () {
        return {
            pre: function (attrs) { return "<ng-template #dialogFooter><" + tagName$C + " wmDialogFooter data-identfier=\"actions\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$C + "></ng-template>"; }
        };
    });
    var dialogFooter_build = (function () { });

    var dialogFooter_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': dialogFooter_build
    });

    var tagName$D = 'div';
    transpiler.register('wm-dialog', function () {
        return {
            pre: function (attrs) { return "<" + tagName$D + " wmDialog " + transpiler.getAttrMarkup(attrs) + " wm-navigable-element=\"true\"><ng-template #dialogBody>"; },
            post: function () { return "</ng-template></" + tagName$D + ">"; }
        };
    });
    // Todo:vinay remove wm-view in migration
    transpiler.register('wm-view', function () {
        return {
            pre: function (attrs) { return ''; },
            post: function () { return ''; }
        };
    });
    var dialog_build = (function () { });

    var dialog_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': dialog_build
    });

    var tagName$E = 'div';
    transpiler.register('wm-iframedialog', function () {
        return {
            pre: function (attrs) { return "<" + tagName$E + " wmIframeDialog wm-navigable-element=\"true\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$E + ">"; }
        };
    });
    var iframeDialog_build = (function () { });

    var iframeDialog_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': iframeDialog_build
    });

    var tagName$F = 'div';
    transpiler.register('wm-logindialog', function () {
        return {
            pre: function (attrs) { return "<" + tagName$F + " wmDialog wmLoginDialog " + transpiler.getAttrMarkup(attrs) + " eventsource.bind=\"Actions.loginAction\" wm-navigable-element=\"true\"><ng-template #dialogBody>"; },
            post: function () { return "</ng-template></" + tagName$F + ">"; }
        };
    });
    var loginDialog_build = (function () { });

    var loginDialog_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': loginDialog_build
    });

    var tagName$G = 'div';
    transpiler.register('wm-pagedialog', function () {
        return {
            pre: function (attrs, shared) {
                var content = attrs.get('content');
                attrs.delete('content');
                var boundContent = attrs.get('content.bind');
                attrs.delete('content.bind');
                var onLoad = attrs.get('load.event');
                attrs.delete('load.event');
                var onLoadEvtMarkup = '';
                var contentMarkup = '';
                if (onLoad) {
                    onLoadEvtMarkup = "load.event=\"" + onLoad + "\"";
                }
                if (boundContent) {
                    contentMarkup = "content.bind=\"" + boundContent + "\"";
                }
                else if (content) {
                    contentMarkup = "content=\"" + content + "\"";
                }
                var containerMarkup = '';
                if (contentMarkup) {
                    shared.set('hasPartialContent', true);
                    containerMarkup += "<ng-template><div wmContainer #partial partialContainer " + contentMarkup + " width=\"100%\" height=\"100%\" " + onLoadEvtMarkup + ">";
                }
                return "<" + tagName$G + " wmPartialDialog " + transpiler.getAttrMarkup(attrs) + ">" + containerMarkup;
            },
            post: function (attrs, shared) {
                var preContent = '';
                if (shared.get('hasPartialContent')) {
                    preContent = "</div></ng-template>";
                }
                return preContent + "</" + tagName$G + ">";
            }
        };
    });
    var partialDialog_build = (function () { });

    var partialDialog_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': partialDialog_build
    });

    var ALLFIELDS = 'All Fields';
    var EDIT_MODE = {
        QUICK_EDIT: 'quickedit',
        INLINE: 'inline',
        FORM: 'form',
        DIALOG: 'dialog'
    };
    var fieldTypeWidgetTypeMap = {
        'integer': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
        'big_integer': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
        'short': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
        'float': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
        'big_decimal': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
        'number': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
        'double': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
        'long': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
        'byte': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
        'string': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'switch', 'currency', 'autocomplete', 'chips', 'colorpicker'],
        'character': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'switch', 'currency', 'autocomplete', 'chips'],
        'text': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'switch', 'currency', 'autocomplete', 'chips', 'colorpicker'],
        'date': ['date', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
        'time': ['time', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
        'timestamp': ['timestamp', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
        'datetime': ['datetime', 'text', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
        'boolean': ['checkbox', 'radioset', 'toggle', 'select'],
        'list': ['select', 'radioset', 'checkboxset', 'switch', 'autocomplete', 'chips'],
        'clob': ['text', 'textarea', 'richtext'],
        'blob': ['upload'],
        'file': ['upload'],
        'custom': ['text', 'number', 'textarea', 'password', 'checkbox', 'toggle', 'slider', 'richtext', 'currency', 'switch',
            'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'rating', 'datetime', 'autocomplete', 'chips', 'colorpicker']
    };
    var DATASET_WIDGETS = new Set([core.FormWidgetType.SELECT, core.FormWidgetType.CHECKBOXSET, core.FormWidgetType.RADIOSET,
        core.FormWidgetType.SWITCH, core.FormWidgetType.AUTOCOMPLETE, core.FormWidgetType.CHIPS, core.FormWidgetType.TYPEAHEAD, core.FormWidgetType.RATING]);
    var isDataSetWidget = function (widget) {
        return DATASET_WIDGETS.has(widget);
    };
    // Get filter widget applicable to the given type
    var getDataTableFilterWidget = function (type) {
        var widget = fieldTypeWidgetTypeMap[type] && fieldTypeWidgetTypeMap[type][0];
        if (type === core.DataType.BOOLEAN) {
            widget = core.FormWidgetType.SELECT;
        }
        var filterWidgets = [core.FormWidgetType.TEXT, core.FormWidgetType.NUMBER, core.FormWidgetType.SELECT, core.FormWidgetType.AUTOCOMPLETE,
            core.FormWidgetType.DATE, core.FormWidgetType.TIME, core.FormWidgetType.DATETIME];
        if (filterWidgets.includes(widget)) {
            return widget;
        }
        return core.FormWidgetType.TEXT;
    };
    /**
     * @ngdoc function
     * @name wm.widgets.live.getEditModeWidget
     * @methodOf wm.widgets.live.LiveWidgetUtils
     * @function
     *
     * @description
     * This function returns the default widget for grid
     *
     * @param {object} colDef field definition
     */
    var getEditModeWidget = function (colDef) {
        if (colDef['related-entity-name'] && colDef['primary-key']) {
            return core.FormWidgetType.SELECT;
        }
        return (fieldTypeWidgetTypeMap[colDef.type] && fieldTypeWidgetTypeMap[colDef.type][0]) || core.FormWidgetType.TEXT;
    };

    var tagName$H = 'div';
    var idGen$2 = new core.IDGenerator('formfield_');
    var getEventsTemplate = function (attrs) {
        var eventAttrs = new Map();
        if (!attrs.has('focus.event')) {
            attrs.set('focus.event', '');
        }
        if (!attrs.has('blur.event')) {
            attrs.set('blur.event', '');
        }
        attrs.forEach(function (value, key) {
            if (key.endsWith('.event')) {
                if (key === 'focus.event') {
                    value = "_onFocusField($event);" + value;
                }
                else if (key === 'blur.event') {
                    value = "_onBlurField($event);" + value;
                }
                eventAttrs.set(key, value);
                attrs.delete(key);
            }
        });
        return transpiler.getFormMarkupAttr(eventAttrs);
    };
    var ɵ0$2 = getEventsTemplate;
    var DEFAULT_PLACEHOLDERS = new Map([
        [core.FormWidgetType.SELECT, ['Select Min value', 'Select Max value', 'Select value']],
        [core.FormWidgetType.DATETIME, ['Select Min date time', 'Select Max date time', 'Select date time']],
        [core.FormWidgetType.TIME, ['Select Min time', 'Select Max time', 'Select time']],
        [core.FormWidgetType.DATE, ['Select Min date', 'Select Max date', 'Select date']],
        [core.FormWidgetType.TEXTAREA, ['', '', 'Enter value']],
        [core.FormWidgetType.RICHTEXT, ['', '', 'Enter value']],
        [core.FormWidgetType.COLORPICKER, ['Select Color', 'Select Color', 'Select Color']],
        [core.FormWidgetType.CHIPS, ['', '', 'Type here...']],
        [core.FormWidgetType.PASSWORD, ['Enter Min value', 'Enter Max value', 'Enter value']],
        [core.FormWidgetType.NUMBER, ['Enter Min value', 'Enter Max value', 'Enter value']],
        [core.FormWidgetType.TEXT, ['Enter Min value', 'Enter Max value', 'Enter value']],
        [core.FormWidgetType.CURRENCY, ['Enter Min value', 'Enter Max value', 'Enter value']],
        [core.FormWidgetType.AUTOCOMPLETE, ['', '', 'Search']],
    ]);
    var setDefaultPlaceholder = function (attrs, widgetType, index) {
        var prop = index === 1 ? 'maxplaceholder' : 'placeholder';
        var placeholder = attrs.get(prop);
        if (placeholder || placeholder === '') {
            return;
        }
        placeholder = DEFAULT_PLACEHOLDERS.get(widgetType) && DEFAULT_PLACEHOLDERS.get(widgetType)[index];
        if (placeholder) {
            attrs.set(prop, placeholder);
        }
    };
    var ɵ1$1 = setDefaultPlaceholder;
    var getWidgetTemplate = function (attrs, options) {
        var name = attrs.get('name');
        var fieldName = (attrs.get('key') || name || '').trim();
        var formControl = options.isMaxWidget ? "formControlName=\"" + fieldName + "_max\"" : (options.isInList ? "[formControlName]=\"" + options.counter + "._fieldName\"" : "formControlName=\"" + fieldName + "\"");
        var tmplRef = options.isMaxWidget ? "#formWidgetMax" : "#formWidget";
        var widgetName = name ? (options.isMaxWidget ? "name=\"" + name + "_formWidgetMax\"" : "name=\"" + name + "_formWidget\"") : '';
        var conditionalClass = "[ngClass]=\"" + attrs.get('ngclass') + "\"";
        var defaultTmpl = "[class.hidden]=\"!" + options.pCounter + ".isUpdateMode && " + options.counter + ".viewmodewidget !== 'default'\" " + formControl + " " + options.eventsTmpl + " " + conditionalClass + " " + tmplRef + " " + widgetName;
        return core.getFormWidgetTemplate(options.widgetType, defaultTmpl, attrs, { counter: options.counter, pCounter: options.pCounter });
    };
    var ɵ2$1 = getWidgetTemplate;
    var getTemplate = function (attrs, widgetType, eventsTmpl, counter, pCounter, isInList) {
        var isRange = attrs.get('is-range') === 'true';
        if (!isRange) {
            return getWidgetTemplate(attrs, { widgetType: widgetType, eventsTmpl: eventsTmpl, counter: counter, pCounter: pCounter, isInList: isInList });
        }
        var layoutClass = core.isMobileApp() ? 'col-xs-6' : 'col-sm-6';
        return "<div class=\"" + layoutClass + "\">" + getWidgetTemplate(attrs, { widgetType: widgetType, eventsTmpl: eventsTmpl, counter: counter, pCounter: pCounter }) + "</div>\n                <div class=\"" + layoutClass + "\">" + getWidgetTemplate(attrs, { widgetType: widgetType, eventsTmpl: eventsTmpl, counter: counter, pCounter: pCounter, isMaxWidget: true }) + "</div>";
    };
    var ɵ3 = getTemplate;
    var getCaptionByWidget = function (attrs, widgetType, counter) {
        if (attrs.get('is-related') === 'true') {
            return counter + ".getDisplayExpr()";
        }
        if (widgetType === core.FormWidgetType.PASSWORD) {
            return '\'********\'';
        }
        var caption = counter + ".value";
        if (widgetType === core.FormWidgetType.DATETIME || widgetType === core.FormWidgetType.TIMESTAMP) {
            caption += " | toDate:" + counter + ".formWidget.datepattern || 'yyyy-MM-dd hh:mm:ss a'";
            return caption;
        }
        if (widgetType === core.FormWidgetType.TIME) {
            caption += " | toDate:" + counter + ".formWidget.timepattern || 'hh:mm a'";
            return caption;
        }
        if (widgetType === core.FormWidgetType.DATE) {
            caption += " | toDate:" + counter + ".formWidget.datepattern ||  'yyyy-MMM-dd'";
            return caption;
        }
        if (widgetType === core.FormWidgetType.RATING || widgetType === core.FormWidgetType.UPLOAD) {
            return '';
        }
        if (isDataSetWidget(widgetType) && attrs.get('datafield') === ALLFIELDS) {
            return counter + ".getDisplayExpr()";
        }
        return counter + ".getCaption()";
    };
    var ɵ4 = getCaptionByWidget;
    var registerFormField = function (isFormField) {
        return {
            requires: ['wm-form', 'wm-liveform', 'wm-livefilter', 'wm-list'],
            pre: function (attrs, shared, parentForm, parentLiveForm, parentFilter, parentList) {
                var counter = idGen$2.nextUid();
                var parent = parentForm || parentLiveForm || parentFilter;
                var pCounter = (parent && parent.get('form_reference')) || 'form';
                var widgetType = attrs.get('widget') || core.FormWidgetType.TEXT;
                var dataRole = isFormField ? 'form-field' : 'filter-field';
                var validationMsg = isFormField ? "<p *ngIf=\"" + counter + "._control?.invalid && " + counter + "._control?.touched && " + pCounter + ".isUpdateMode\"\n                                   class=\"help-block text-danger\"\n                                   [textContent]=\"" + counter + ".validationmessage\"></p>" : '';
                var eventsTmpl = widgetType === core.FormWidgetType.UPLOAD ? '' : getEventsTemplate(attrs);
                var controlLayout = core.isMobileApp() ? 'col-xs-12' : 'col-sm-12';
                var isInList = pCounter === (parentList && parentList.get('parent_form_reference'));
                attrs.set('__widgetType', widgetType);
                attrs.delete('widget');
                shared.set('counter', counter);
                if (attrs.get('is-range') === 'true') {
                    setDefaultPlaceholder(attrs, widgetType, 0);
                    setDefaultPlaceholder(attrs, widgetType, 1);
                }
                else {
                    setDefaultPlaceholder(attrs, widgetType, 2);
                }
                return "<" + tagName$H + " data-role=\"" + dataRole + "\" [formGroup]=\"" + pCounter + ".ngform\" wmFormField #" + counter + "=\"wmFormField\" widgettype=\"" + widgetType + "\" " + transpiler.getFormMarkupAttr(attrs) + ">\n                        <div class=\"live-field form-group app-composite-widget clearfix caption-{{" + pCounter + ".captionposition}}\" widget=\"" + widgetType + "\">\n                            <label [hidden]=\"!" + counter + ".displayname\" class=\"app-label control-label formfield-label {{" + pCounter + "._captionClass}}\" [title]=\"" + counter + ".displayname\"\n                                        [ngStyle]=\"{width: " + pCounter + ".captionsize}\" [ngClass]=\"{'text-danger': " + counter + "._control?.invalid && " + counter + "._control?.touched && " + pCounter + ".isUpdateMode,\n                                         required: " + pCounter + ".isUpdateMode && " + counter + ".required}\" [textContent]=\"" + counter + ".displayname\"> </label>\n                            <div [ngClass]=\"" + counter + ".displayname ? " + pCounter + "._widgetClass : '" + controlLayout + "'\">\n                                 <label class=\"form-control-static app-label\"\n                                       [hidden]=\"" + pCounter + ".isUpdateMode || " + counter + ".viewmodewidget === 'default' || " + counter + ".widgettype === 'upload'\" [innerHTML]=\"" + getCaptionByWidget(attrs, widgetType, counter) + "\"></label>\n                                " + getTemplate(attrs, widgetType, eventsTmpl, counter, pCounter, isInList) + "\n                                <span aria-hidden=\"true\" *ngIf=\"" + counter + ".showPendingSpinner\" class=\"form-field-spinner fa fa-circle-o-notch fa-spin form-control-feedback\"></span>\n                                <p *ngIf=\"!(" + counter + "._control?.invalid && " + counter + "._control?.touched) && " + pCounter + ".isUpdateMode\"\n                                   class=\"help-block\" [textContent]=\"" + counter + ".hint\"></p>\n                                " + validationMsg + "\n                            </div>\n                        </div>";
            },
            post: function () { return "</" + tagName$H + ">"; },
            provide: function (attrs, shared) {
                var provider = new Map();
                provider.set('form_reference', shared.get('counter'));
                return provider;
            },
            imports: function (attrs) {
                var requiredWidget = core.getRequiredFormWidget(attrs.get('__widgetType') || attrs.get('type'));
                attrs.delete('__widgetType');
                return [requiredWidget, 'wm-form'];
            }
        };
    };
    var ɵ5 = registerFormField;
    transpiler.register('wm-form-field', registerFormField.bind(this, true));
    transpiler.register('wm-filter-field', registerFormField.bind(this, false));
    var formField_build = (function () { });

    var formField_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': formField_build,
        ɵ0: ɵ0$2,
        ɵ1: ɵ1$1,
        ɵ2: ɵ2$1,
        ɵ3: ɵ3,
        ɵ4: ɵ4,
        ɵ5: ɵ5
    });

    var tagName$I = 'div';
    var registerAction = function (tmpl) {
        return {
            pre: function (attrs) { return "<" + tagName$I + " wmFormAction name=\"" + (attrs.get('name') || attrs.get('key')) + "\" " + transpiler.getAttrMarkup(attrs) + " " + tmpl + ">"; },
            post: function () { return "</" + tagName$I + ">"; }
        };
    };
    var ɵ0$3 = registerAction;
    transpiler.register('wm-form-action', registerAction.bind(this, ''));
    transpiler.register('wm-filter-action', registerAction.bind(this, " update-mode=\"true\" "));
    var formAction_build = (function () { });

    var formAction_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': formAction_build,
        ɵ0: ɵ0$3
    });

    var tagName$J = 'form';
    var idGen$3 = new core.IDGenerator('form_');
    var formWidgets = new Set([
        'wm-text',
        'wm-textarea',
        'wm-checkbox',
        'wm-slider',
        'wm-richtexteditor',
        'wm-currency',
        'wm-switch',
        'wm-select',
        'wm-checkboxset',
        'wm-radioset',
        'wm-date',
        'wm-time',
        'wm-timestamp',
        'wm-upload',
        'wm-rating',
        'wm-datetime',
        'wm-search',
        'wm-chips',
        'wm-colorpicker',
        'wm-table'
    ]);
    var numberOfFields;
    var addFormControlName = function (children) {
        if (children === void 0) { children = []; }
        children.forEach(function (childNode) {
            if (childNode.name === 'wm-form-field') {
                numberOfFields++;
            }
            if (formWidgets.has(childNode.name)) {
                var key = childNode.attrs.find(function (attr) { return attr.name === 'key' || attr.name === 'name'; });
                key = key && key.value;
                if (!key) {
                    // for checkbox inside table inside form, key and name attrs are not available. Using datavalue to construct formControlName instead.
                    var dataValue = childNode.attrs.find(function (attr) { return attr.name === 'datavalue'; });
                    if (dataValue && dataValue.value) {
                        var regex = /\((.*)\)/;
                        dataValue = dataValue.value.match(regex);
                        if (dataValue && dataValue.length > 1) {
                            key = dataValue[1].replace(/["']/g, '');
                        }
                    }
                }
                childNode.attrs.push(new compiler.Attribute('formControlName', key, 1, 1));
                childNode.attrs.push(new compiler.Attribute('wmFormWidget', '', 1, 1));
            }
            addFormControlName(childNode.children);
        });
    };
    var ɵ0$4 = addFormControlName;
    var updateFormDataSource = function (attrMap) {
        if (attrMap.get('formdata.bind')) {
            var formDataSource = transpiler.getDataSource(attrMap.get('formdata.bind'));
            if (formDataSource) {
                attrMap.set('formdatasource.bind', formDataSource);
            }
        }
    };
    var ɵ1$2 = updateFormDataSource;
    var buildTask = function (directiveAttr) {
        if (directiveAttr === void 0) { directiveAttr = ''; }
        return {
            requires: ['wm-livetable', 'wm-login'],
            template: function (node) {
                numberOfFields = 0;
                addFormControlName(node.children);
            },
            pre: function (attrs, shared, parentLiveTable, parentLoginWidget) {
                var tmpl;
                var dialogId;
                var role = parentLoginWidget && parentLoginWidget.get('isLogin') ? 'app-login' : '';
                var counter = idGen$3.nextUid();
                var dependsOn = attrs.get('dependson') ? "dependson=\"" + attrs.get('dependson') + "\"" : '';
                var classProp = attrs.get('formlayout') === 'page' ? 'app-device-liveform panel liveform-inline' : '';
                var dialogAttributes = ['title', 'title.bind', 'iconclass', 'iconclass.bind', 'width'];
                attrs.delete('dependson');
                var liveFormTmpl = "<" + tagName$J + " wmForm role=\"" + role + "\" " + directiveAttr + " #" + counter + " ngNativeValidate [formGroup]=\"" + counter + ".ngform\" [noValidate]=\"" + counter + ".validationtype !== 'html'\"\n                    class=\"" + classProp + "\" [class]=\"" + counter + ".captionAlignClass\" [autocomplete]=\"" + counter + ".autocomplete ? 'on' : 'off'\" captionposition=" + attrs.get('captionposition');
                attrs.set('numberOfFields', "" + numberOfFields);
                shared.set('counter', counter);
                updateFormDataSource(attrs);
                if (attrs.get('formlayout') === 'dialog') {
                    dialogId = parentLiveTable ? parentLiveTable.get('liveform_dialog_id') : "liveform_dialog_id_" + counter;
                    attrs.set('dialogId', dialogId);
                    var dialogAttrsMap_1 = new Map();
                    dialogAttributes.forEach(function (attr) {
                        if (attrs.get(attr)) {
                            dialogAttrsMap_1.set(attr, attrs.get(attr));
                        }
                    });
                    attrs.set('width', '100%');
                    tmpl = transpiler.getAttrMarkup(attrs);
                    return "<div data-identifier=\"liveform\" init-widget class=\"app-liveform liveform-dialog\" " + dependsOn + " dialogid=\"" + dialogId + "\">\n                            <div wmDialog class=\"app-liveform-dialog\" name=\"" + dialogId + "\" role=\"form\" " + transpiler.getAttrMarkup(dialogAttrsMap_1) + " modal=\"true\">\n                            <ng-template #dialogBody>\n                            " + liveFormTmpl + " " + tmpl + ">";
                }
                var mobileFormContentTmpl = '';
                var buttonTemplate = '';
                // Include mobile-navbar above the form when formlayout is set to page
                if (attrs.get('formlayout') === 'page') {
                    var name_1 = "device_liveform_header_" + counter;
                    var navbarAttrsMap = new Map();
                    navbarAttrsMap.set('title', attrs.get('title'));
                    navbarAttrsMap.set('backbtnclick.event', attrs.get('backbtnclick.event'));
                    buttonTemplate = "<ng-template #buttonRef let-btn=\"btn\">\n                                            <button  wmButton name=\"{{btn.key}}\" class=\"navbar-btn btn-primary btn-transparent\" iconclass.bind=\"btn.iconclass\" show.bind=\"btn.show\"\n                                                     (click)=\"" + counter + ".invokeActionEvent($event, btn.action)\" type.bind=\"btn.type\" hint.bind=\"btn.title\" shortcutkey.bind=\"btn.shortcutkey\" disabled.bind=\"btn.disabled\"\n                                                     tabindex.bind=\"btn.tabindex\" [class.hidden]=\"btn.updateMode ? !" + counter + ".isUpdateMode : " + counter + ".isUpdateMode\"></button>\n                                        </ng-template>";
                    mobileFormContentTmpl = "<header wmMobileNavbar name=\"" + name_1 + "\" " + transpiler.getAttrMarkup(navbarAttrsMap) + ">\n                                            <ng-container *ngFor=\"let btn of " + counter + ".buttonArray\" [ngTemplateOutlet]=\"buttonRef\" [ngTemplateOutletContext]=\"{btn:btn}\">\n                                            </ng-container>\n                                        </header>\n                                        <div class=\"form-elements panel-body\" >";
                }
                tmpl = transpiler.getAttrMarkup(attrs);
                return liveFormTmpl + " " + tmpl + " " + dependsOn + ">\n                       " + buttonTemplate + " " + mobileFormContentTmpl;
            },
            post: function (attrs) {
                if (attrs.get('formlayout') === 'dialog') {
                    return '</form></ng-template></div></div>';
                }
                if (attrs.get('formlayout') === 'page') {
                    return "</div></" + tagName$J + ">";
                }
                return "</" + tagName$J + ">";
            },
            provide: function (attrs, shared) {
                var provider = new Map();
                provider.set('form_reference', shared.get('counter'));
                return provider;
            }
        };
    };
    var ɵ2$2 = buildTask;
    transpiler.register('wm-form', buildTask);
    transpiler.register('wm-liveform', function () { return buildTask('wmLiveForm'); });
    transpiler.register('wm-livefilter', function () { return buildTask('wmLiveFilter'); });
    var form_build = (function () { });

    var form_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': form_build,
        ɵ0: ɵ0$4,
        ɵ1: ɵ1$2,
        ɵ2: ɵ2$2
    });

    var tagName$K = 'div';
    transpiler.register('wm-calendar', function () {
        return {
            pre: function (attrs) { return "<" + tagName$K + " wmCalendar redrawable style=\"width:100%\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$K + ">"; }
        };
    });
    var calendar_build = (function () { });

    var calendar_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': calendar_build
    });

    var tagName$L = 'ul';
    transpiler.register('wm-chips', function () {
        return {
            pre: function (attrs) { return "<" + tagName$L + " wmChips role=\"listbox\" " + transpiler.getAttrMarkup(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$L + ">"; }
        };
    });
    var chips_build = (function () { });

    var chips_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': chips_build
    });

    var tagName$M = 'div';
    transpiler.register('wm-colorpicker', function () {
        return {
            pre: function (attrs) { return "<" + tagName$M + " wmColorPicker " + transpiler.getAttrMarkup(attrs) + " role=\"input\" " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$M + ">"; }
        };
    });
    var colorPicker_build = (function () { });

    var colorPicker_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': colorPicker_build
    });

    var tagName$N = 'div';
    transpiler.register('wm-currency', function () {
        return {
            pre: function (attrs) { return "<" + tagName$N + " wmCurrency " + transpiler.getAttrMarkup(attrs) + " role=\"input\" " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$N + ">"; }
        };
    });
    var currency_build = (function () { });

    var currency_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': currency_build
    });

    var tagName$O = 'div';
    transpiler.register('wm-buttongroup', function () {
        return {
            pre: function (attrs) { return "<" + tagName$O + " wmButtonGroup role=\"group\" aria-labelledby=\"button group\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$O + ">"; }
        };
    });
    var buttonGroup_build = (function () { });

    var buttonGroup_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': buttonGroup_build
    });

    var tagName$P = 'button';
    transpiler.register('wm-button', function () {
        return {
            pre: function (attrs) { return "<" + tagName$P + " wmButton " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$P + ">"; }
        };
    });
    var button_build = (function () { });

    var button_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': button_build
    });

    var tagName$Q = 'div';
    transpiler.register('wm-checkbox', function () {
        return {
            pre: function (attrs) { return "<" + tagName$Q + " wmCheckbox role=\"input\" " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$Q + ">"; }
        };
    });
    var checkbox_build = (function () { });

    var checkbox_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': checkbox_build
    });

    var tagName$R = 'ul';
    transpiler.register('wm-checkboxset', function () {
        return {
            pre: function (attrs) { return "<" + tagName$R + " wmCheckboxset " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$R + ">"; }
        };
    });
    var checkboxset_build = (function () { });

    var checkboxset_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': checkboxset_build
    });

    var tagName$S = 'div';
    transpiler.register('wm-composite', function () {
        return {
            pre: function (attrs) { return "<" + tagName$S + " wmComposite role=\"group\" " + transpiler.setChildAttrs(attrs) + " " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$S + transpiler.clearChildAttrs() + ">"; }
        };
    });
    var composite_build = (function () { });

    var composite_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': composite_build
    });

    var tagName$T = 'div';
    transpiler.register('wm-number', function () {
        return {
            pre: function (attrs) { return "<" + tagName$T + " wmNumber " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$T + ">"; }
        };
    });
    var number_build = (function () { });

    var number_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': number_build
    });

    var tagName$U = 'ul';
    transpiler.register('wm-radioset', function () {
        return {
            pre: function (attrs) { return "<" + tagName$U + " wmRadioset role=\"radiogroup\" " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$U + ">"; }
        };
    });
    var radioset_build = (function () { });

    var radioset_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': radioset_build
    });

    var tagName$V = 'wm-select';
    transpiler.register('wm-select', function () {
        return {
            pre: function (attrs) { return "<" + tagName$V + " " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + "  " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$V + ">"; }
        };
    });
    var select_build = (function () { });

    var select_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': select_build
    });

    var tagName$W = 'div';
    transpiler.register('wm-switch', function () {
        return {
            pre: function (attrs) { return "<" + tagName$W + " wmSwitch " + transpiler.getFormMarkupAttr(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$W + ">"; }
        };
    });
    var switch_build = (function () { });

    var switch_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': switch_build
    });

    var tagName$X = 'wm-input';
    transpiler.register('wm-text', function () {
        return {
            pre: function (attrs) { return "<" + tagName$X + " " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$X + ">"; }
        };
    });
    var text_build = (function () { });

    var text_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': text_build
    });

    var tagName$Y = 'wm-textarea';
    transpiler.register('wm-textarea', function () {
        return {
            pre: function (attrs) { return "<" + tagName$Y + " " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$Y + ">"; }
        };
    });
    var textarea_build = (function () { });

    var textarea_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': textarea_build
    });

    var tagName$Z = 'div';
    transpiler.register('wm-datetime', function () {
        return {
            pre: function (attrs) { return "<" + tagName$Z + " wmDateTime " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$Z + ">"; }
        };
    });
    var dateTime_build = (function () { });

    var dateTime_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': dateTime_build
    });

    var tagName$_ = 'div';
    transpiler.register('wm-date', function () {
        return {
            pre: function (attrs) { return "<" + tagName$_ + " wmDate " + transpiler.getFormMarkupAttr(attrs) + " " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$_ + ">"; }
        };
    });
    var date_build = (function () { });

    var date_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': date_build
    });

    var tagName$$ = 'div';
    transpiler.register('wm-time', function () {
        return {
            pre: function (attrs) { return "<" + tagName$$ + " wmTime " + transpiler.getFormMarkupAttr(attrs) + " role=\"input\" " + transpiler.getChildAttrs(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$$ + ">"; }
        };
    });
    var time_build = (function () { });

    var time_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': time_build
    });

    var tagName$10 = 'div';
    transpiler.register('wm-fileupload', function () {
        return {
            pre: function (attrs) {
                if (attrs.get('select.event')) {
                    var onSelectBinding = transpiler.getDataSource(attrs.get('select.event'));
                    attrs.set('datasource.bind', onSelectBinding);
                }
                return "<" + tagName$10 + " wmFileUpload " + transpiler.getAttrMarkup(attrs) + " role=\"input\">";
            },
            post: function () { return "</" + tagName$10 + ">"; }
        };
    });
    var fileUpload_build = (function () { });

    var fileUpload_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': fileUpload_build
    });

    var tagName$11 = 'div';
    transpiler.register('wm-rating', function () {
        return {
            pre: function (attrs) { return "<" + tagName$11 + " wmRating " + transpiler.getFormMarkupAttr(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$11 + ">"; }
        };
    });
    var rating_build = (function () { });

    var rating_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': rating_build
    });

    var tagName$12 = 'div';
    transpiler.register('wm-slider', function () {
        return {
            pre: function (attrs) { return "<" + tagName$12 + " wmSlider " + transpiler.getAttrMarkup(attrs) + " " + core.getNgModelAttr(attrs) + ">"; },
            post: function () { return "</" + tagName$12 + ">"; }
        };
    });
    var slider_build = (function () { });

    var slider_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': slider_build
    });

    var wmlistTag = 'wm-list';
    var tagName$13 = 'div';
    var dataSetKey$1 = 'dataset';
    function copyAttribute(from, fromAttrName, to, toAttrName) {
        var fromAttr = from.attrs.find(function (a) { return a.name === fromAttrName; });
        if (fromAttr) {
            to.attrs.push(new compiler.Attribute(toAttrName, fromAttr.value, fromAttr.sourceSpan, fromAttr.valueSpan));
        }
    }
    transpiler.register('wm-media-list', function () {
        return {
            template: function (node) {
                var bindDataset;
                var attrObj = node.attrs.find(function (attr) { return attr.name === dataSetKey$1; }), 
                /**
                 *  Replacing binded property value with item
                 * @param children
                 */
                replaceBind = function (children) {
                    if (children === void 0) { children = []; }
                    children.forEach(function (childNode) {
                        if (childNode.name) {
                            // return if the child Element is of wm-list .
                            if (childNode.name !== wmlistTag) {
                                childNode.attrs.forEach(function (attr) {
                                    if (attr.value.startsWith("bind:" + bindDataset + ".data[$i]")) {
                                        attr.value = attr.value.replace(bindDataset + ".data[$i]", 'item');
                                    }
                                    else if (attr.value.startsWith("bind:" + bindDataset)) {
                                        attr.value = attr.value.replace(bindDataset, 'item');
                                    }
                                });
                                replaceBind(childNode.children);
                            }
                        }
                    });
                };
                if (attrObj && attrObj.value.startsWith('bind:')) {
                    bindDataset = attrObj.value.replace('bind:', '');
                }
                if (bindDataset) {
                    replaceBind(node.children);
                }
                var template = node.children
                    .find(function (e) { return e instanceof compiler.Element && e.name === 'wm-media-template'; });
                if (template != null) {
                    copyAttribute(template, 'width', node, 'thumbnailwidth');
                    copyAttribute(template, 'height', node, 'thumbnailheight');
                }
            },
            pre: function (attrs) { return "<" + tagName$13 + " wmMediaList " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$13 + ">"; }
        };
    });
    var mediaList_build = (function () { });

    var mediaList_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': mediaList_build
    });

    var tagName$14 = 'ng-template';
    transpiler.register('wm-media-template', function () {
        return {
            pre: function () { return "<" + tagName$14 + " #mediaListTemplate let-item=\"item\" let-index=\"index\">"; },
            post: function () { return "</" + tagName$14 + ">"; }
        };
    });
    var mediaListItem_build = (function () { });

    var mediaListItem_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': mediaListItem_build
    });

    var wmListTag = 'wm-list';
    var listTagName = 'div';
    var dataSetKey$2 = 'dataset';
    transpiler.register(wmListTag, function () {
        return {
            requires: ['wm-form', 'wm-liveform'],
            template: function (node) {
                var datasetAttr = node.attrs.find(function (attr) { return attr.name === dataSetKey$2; });
                var widgetNameAttr = node.attrs.find(function (attr) { return attr.name === 'name'; });
                if (!datasetAttr) {
                    return;
                }
                var boundExpr = transpiler.getBoundToExpr(datasetAttr.value);
                if (!boundExpr) {
                    return;
                }
                core.updateTemplateAttrs(node, boundExpr, widgetNameAttr.value, 'itemRef.');
            },
            pre: function (attrs, shared, parentForm, parentLiveForm) {
                var parent = parentForm || parentLiveForm;
                shared.set('form_reference', parent && parent.get('form_reference'));
                return "<" + listTagName + " wmList wmLiveActions " + transpiler.getAttrMarkup(attrs) + ">";
            },
            post: function () { return "</" + listTagName + ">"; },
            provide: function (attrs, shared) {
                var provider = new Map();
                provider.set('parent_form_reference', shared.get('form_reference'));
                return provider;
            }
        };
    });
    transpiler.register('wm-listtemplate', function () {
        return {
            pre: function () { return "<ng-template #listTemplate let-item=\"item\" let-$index=\"$index\" let-itemRef=\"itemRef\" let-$first=\"$first\" let-$last=\"$last\"  let-currentItemWidgets=\"currentItemWidgets\" >"; },
            post: function () { return "</ng-template>"; }
        };
    });
    function copyAttribute$1(from, fromAttrName, to, toAttrName) {
        var fromAttr = from.attrs.find(function (a) { return a.name === fromAttrName; });
        if (fromAttr) {
            to.attrs.push(new compiler.Attribute(toAttrName, fromAttr.value, fromAttr.sourceSpan, fromAttr.valueSpan));
        }
    }
    transpiler.register('wm-list-action-template', function () {
        return {
            template: function (node) {
                var position = node.attrs.find(function (attr) { return attr.name === 'position'; }).value;
                var btns = node.children
                    .filter(function (e) { return e instanceof compiler.Element && e.name === 'wm-button'; });
                // add swipe-position on button nodes to identify whether buttons are from left or right action templates
                btns.forEach(function (btnNode) {
                    copyAttribute$1(node, 'position', btnNode, 'swipe-position');
                });
            },
            pre: function (attrs, el) {
                if (attrs.get('position') === 'left') {
                    return "<ng-template #listLeftActionTemplate>\n                            <li class=\"app-list-item-action-panel app-list-item-left-action-panel actionMenu\" " + transpiler.getAttrMarkup(attrs) + ">";
                }
                if (attrs.get('position') === 'right') {
                    return "<ng-template #listRightActionTemplate>\n                            <li class=\"app-list-item-action-panel app-list-item-right-action-panel actionMenu\" " + transpiler.getAttrMarkup(attrs) + ">";
                }
            },
            post: function () { return "</li></ng-template>"; }
        };
    });
    var list_build = (function () { });

    var list_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': list_build
    });

    var tagName$15 = 'div';
    var idGen$4 = new core.IDGenerator('liveform_dialog_id_');
    transpiler.register('wm-livetable', function () {
        return {
            pre: function (attrs, shared) {
                var counter = idGen$4.nextUid();
                shared.set('counter', counter);
                return "<" + tagName$15 + " wmLiveTable role=\"table\" " + transpiler.getAttrMarkup(attrs) + " dialogid=\"" + counter + "\">";
            },
            post: function () { return "</" + tagName$15 + ">"; },
            provide: function (attrs, shared) {
                var provider = new Map();
                provider.set('liveform_dialog_id', shared.get('counter'));
                return provider;
            }
        };
    });
    var liveTable_build = (function () { });

    var liveTable_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': liveTable_build
    });

    var tagName$16 = 'p';
    transpiler.register('wm-message', function () {
        return {
            pre: function (attrs) { return "<" + tagName$16 + " wmMessage aria-label=\"Notification Alerts\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$16 + ">"; }
        };
    });
    var message_build = (function () { });

    var message_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': message_build
    });

    var tagName$17 = 'ol';
    transpiler.register('wm-breadcrumb', function () {
        return {
            pre: function (attrs) { return "<" + tagName$17 + " wmBreadcrumb " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$17 + ">"; }
        };
    });
    var breadcrumb_build = (function () { });

    var breadcrumb_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': breadcrumb_build
    });

    var tagName$18 = 'div';
    transpiler.register('wm-menu', function () {
        return {
            pre: function (attrs) { return "<" + tagName$18 + " wmMenu dropdown " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$18 + ">"; }
        };
    });
    var menu_build = (function () { });

    var menu_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': menu_build
    });

    var tagName$19 = 'li';
    transpiler.register('wm-nav-item', function () {
        return {
            pre: function (attrs) { return "<" + tagName$19 + " wmNavItem role=\"presentation\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$19 + ">"; }
        };
    });
    var navItem_build = (function () { });

    var navItem_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': navItem_build
    });

    var tagName$1a = 'ul';
    transpiler.register('wm-nav', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1a + " wmNav data-element-type=\"wmNav\" data-role=\"page-header\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1a + ">"; }
        };
    });
    var nav_build = (function () { });

    var nav_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': nav_build
    });

    var tagName$1b = 'nav';
    transpiler.register('wm-navbar', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1b + " wmNavbar data-element-type=\"wmNavbar\" role=\"navigation\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1b + ">"; }
        };
    });
    var navbar_build = (function () { });

    var navbar_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': navbar_build
    });

    var tagName$1c = 'wm-popover';
    transpiler.register('wm-popover', function () {
        return {
            requires: ['wm-table'],
            pre: function (attrs, shared, table) {
                var contentSource = attrs.get('contentsource');
                var popoverTemplate;
                if (contentSource !== 'inline') {
                    var content = attrs.get('content');
                    var bindContent = attrs.get('content.bind');
                    var contentMarkup = '';
                    if (content) {
                        contentMarkup = "content=\"" + content + "\"";
                    }
                    else if (bindContent) {
                        contentMarkup = "content.bind=\"" + bindContent + "\"";
                    }
                    popoverTemplate = "<div wmContainer #partial partialContainer " + contentMarkup + ">";
                    shared.set('hasPopoverContent', true);
                }
                var markup = "<" + tagName$1c + " wmPopover " + transpiler.getAttrMarkup(attrs) + ">";
                var contextAttrs = table ? "let-row=\"row\"" : "";
                markup += "<ng-template " + contextAttrs + "><button class=\"popover-start\"></button>";
                // todo keyboard navigation - tab
                if (popoverTemplate) {
                    markup += "" + (popoverTemplate ? popoverTemplate : '');
                }
                return markup;
            },
            post: function (attrs, shared) {
                var markup = '';
                if (shared.get('hasPopoverContent')) {
                    markup += "</div>";
                }
                return markup + "<button class=\"popover-end\"></button></ng-template></" + tagName$1c + ">";
            }
        };
    });
    var popover_build = (function () { });

    var popover_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': popover_build
    });

    var tagName$1d = 'div';
    var findChild = function (node, childName) {
        var child = node && node.children.find(function (e) { return (e instanceof compiler.Element && e.name === childName); });
        return child;
    };
    var ɵ0$5 = findChild;
    var createElement = function (name) {
        return new compiler.Element(name, [], [], noSpan, noSpan, noSpan);
    };
    var ɵ1$3 = createElement;
    var addAtrribute = function (node, name, value) {
        var attr = new compiler.Attribute(name, value, noSpan, noSpan);
        node.attrs.push(attr);
    };
    var ɵ2$3 = addAtrribute;
    var getElementNode = function (name, node) {
        var elementNode;
        if (!node) {
            return;
        }
        _.forEach(node.children, function (child) {
            if (child instanceof compiler.Element) {
                if (child.name === name) {
                    elementNode = child;
                }
                else if (child.children) {
                    elementNode = getElementNode(name, child);
                }
            }
            return !elementNode;
        });
        return elementNode;
    };
    var ɵ3$1 = getElementNode;
    var noSpan = {};
    transpiler.register('wm-page', function () {
        return {
            template: function (node) {
                var pageContentNode = findChild(findChild(node, 'wm-content'), 'wm-page-content');
                if (!pageContentNode) {
                    pageContentNode = getElementNode('wm-page-content', getElementNode('wm-content', node));
                }
                if (pageContentNode) {
                    var conditionalNode = createElement('ng-container');
                    addAtrribute(conditionalNode, '*ngIf', 'compilePageContent');
                    conditionalNode.children = conditionalNode.children.concat(pageContentNode.children);
                    conditionalNode.children.push(new compiler.Text('{{onPageContentReady()}}', null));
                    pageContentNode.children = [conditionalNode];
                    if (core.isMobileApp()) {
                        var loader = createElement('div');
                        addAtrribute(loader, 'wmPageContentLoader', '');
                        addAtrribute(loader, '*ngIf', '!showPageContent');
                        pageContentNode.children.push(loader);
                    }
                }
            },
            pre: function (attrs) { return "<" + tagName$1d + " wmPage data-role=\"pageContainer\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1d + ">"; }
        };
    });
    var page_build = (function () { });

    var page_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': page_build,
        ɵ0: ɵ0$5,
        ɵ1: ɵ1$3,
        ɵ2: ɵ2$3,
        ɵ3: ɵ3$1
    });

    var tagName$1e = 'nav';
    transpiler.register('wm-pagination', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1e + " wmPagination data-identifier=\"pagination\" aria-label=\"Page navigation\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1e + ">"; }
        };
    });
    var pagination_build = (function () { });

    var pagination_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': pagination_build
    });

    var tagName$1f = 'main';
    transpiler.register('wm-content', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1f + " wmContent data-role=\"page-content\" role=\"main\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1f + ">"; }
        };
    });
    var content_build = (function () { });

    var content_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': content_build
    });

    var tagName$1g = 'footer';
    transpiler.register('wm-footer', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1g + " wmFooter partialContainer data-role=\"page-footer\" role=\"contentinfo\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1g + ">"; }
        };
    });
    var footer_build = (function () { });

    var footer_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': footer_build
    });

    var tagName$1h = 'header';
    transpiler.register('wm-header', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1h + " wmHeader partialContainer data-role=\"page-header\" role=\"banner\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1h + ">"; }
        };
    });
    var header_build = (function () { });

    var header_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': header_build
    });

    var tagName$1i = 'aside';
    transpiler.register('wm-left-panel', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1i + " wmLeftPanel partialContainer data-role=\"page-left-panel\" role=\"complementary\" wmSmoothscroll=\"" + (attrs.get('smoothscroll') || 'true') + "\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1i + ">"; }
        };
    });
    var leftPanel_build = (function () { });

    var leftPanel_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': leftPanel_build
    });

    var tagName$1j = 'header';
    transpiler.register('wm-mobile-navbar', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1j + " wmMobileNavbar " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1j + ">"; }
        };
    });
    var mobileNavbar_build = (function () { });

    var mobileNavbar_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': mobileNavbar_build
    });

    var tagName$1k = 'aside';
    transpiler.register('wm-right-panel', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1k + " wmRightPanel partialContainer data-role=\"page-right-panel\" role=\"complementary\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1k + ">"; }
        };
    });
    var rightPanel_build = (function () { });

    var rightPanel_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': rightPanel_build
    });

    var tagName$1l = 'div';
    transpiler.register('wm-page-content', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1l + " wmPageContent wmSmoothscroll=\"" + (attrs.get('smoothscroll') || 'true') + "\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1l + ">"; }
        };
    });
    var pageContent_build = (function () { });

    var pageContent_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': pageContent_build
    });

    var tagName$1m = 'div';
    transpiler.register('wm-mobile-tabbar', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1m + " wmMobileTabbar " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1m + ">"; }
        };
    });
    var tabBar_build = (function () { });

    var tabBar_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tabBar_build
    });

    var tagName$1n = 'section';
    transpiler.register('wm-top-nav', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1n + " wmTopNav partialContainer data-role=\"page-topnav\" role=\"region\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1n + ">"; }
        };
    });
    var topNav_build = (function () { });

    var topNav_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': topNav_build
    });

    var tagName$1o = 'section';
    var noSpan$1 = {};
    var createElement$1 = function (name) {
        return new compiler.Element(name, [], [], noSpan$1, noSpan$1, noSpan$1);
    };
    var ɵ0$6 = createElement$1;
    var addAtrribute$1 = function (node, name, value) {
        var attr = new compiler.Attribute(name, value, noSpan$1, noSpan$1);
        node.attrs.push(attr);
    };
    var ɵ1$4 = addAtrribute$1;
    transpiler.register('wm-partial', function () {
        return {
            template: function (node) {
                var conditionalNode = createElement$1('ng-container');
                addAtrribute$1(conditionalNode, '*ngIf', 'compileContent');
                conditionalNode.children = conditionalNode.children.concat(node.children);
                node.children.length = 0;
                node.children.push(conditionalNode);
            },
            pre: function (attrs) { return "<" + tagName$1o + " wmPartial data-role=\"partial\" role=\"region\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1o + ">"; }
        };
    });
    var partial_build = (function () { });

    var partial_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': partial_build,
        ɵ0: ɵ0$6,
        ɵ1: ɵ1$4
    });

    var tagName$1p = 'div';
    transpiler.register('wm-param', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1p + " wmParam hidden " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1p + ">"; }
        };
    });
    var partialParam_build = (function () { });

    var partialParam_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': partialParam_build
    });

    var tagName$1q = 'section';
    transpiler.register('wm-prefab', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1q + " wmPrefab data-role=\"perfab\" " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1q + ">"; }
        };
    });
    var prefab_build = (function () { });

    var prefab_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': prefab_build
    });

    var noSpan$2 = {};
    var createElement$2 = function (name) {
        return new compiler.Element(name, [], [], noSpan$2, noSpan$2, noSpan$2);
    };
    var ɵ0$7 = createElement$2;
    var addAtrribute$2 = function (node, name, value) {
        var attr = new compiler.Attribute(name, value, noSpan$2, noSpan$2);
        node.attrs.push(attr);
    };
    var ɵ1$5 = addAtrribute$2;
    var tagName$1r = 'div';
    transpiler.register('wm-prefab-container', function () {
        return {
            template: function (node) {
                var conditionalNode = createElement$2('ng-container');
                addAtrribute$2(conditionalNode, '*ngIf', 'compileContent');
                conditionalNode.children = conditionalNode.children.concat(node.children);
                node.children.length = 0;
                node.children.push(conditionalNode);
            },
            pre: function (attrs) { return "<" + tagName$1r + " wmPrefabContainer " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1r + ">"; }
        };
    });
    var prefabContainer_build = (function () { });

    var prefabContainer_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': prefabContainer_build,
        ɵ0: ɵ0$7,
        ɵ1: ɵ1$5
    });

    var tagName$1s = 'div';
    transpiler.register('wm-table-action', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1s + " name=\"" + (attrs.get('name') || attrs.get('key')) + "\" wmTableAction " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1s + ">"; }
        };
    });
    var tableAction_build = (function () { });

    var tableAction_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tableAction_build
    });

    var tagName$1t = 'div';
    transpiler.register('wm-table-column-group', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1t + " wmTableColumnGroup " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1t + ">"; }
        };
    });
    var tableColumnGroup_build = (function () { });

    var tableColumnGroup_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tableColumnGroup_build
    });

    var tagName$1u = 'div';
    var idGen$5 = new core.IDGenerator('data_table_form_');
    var formWidgets$1 = new Set([
        'wm-text',
        'wm-textarea',
        'wm-checkbox',
        'wm-slider',
        'wm-currency',
        'wm-switch',
        'wm-select',
        'wm-checkboxset',
        'wm-radioset',
        'wm-date',
        'wm-time',
        'wm-timestamp',
        'wm-rating',
        'wm-datetime',
        'wm-search',
        'wm-chips',
        'wm-colorpicker'
    ]);
    // Add ngModelOptions standalone true as inner custom form widgets will be not part of table ngform
    var addNgModelStandalone = function (children) {
        if (children === void 0) { children = []; }
        var formControlAttr = [];
        children.forEach(function (childNode) {
            if (formWidgets$1.has(childNode.name)) {
                // angular build is failing when formControlName is undefined for checkbox and mgModelOptions attr is present
                if (childNode.name === 'wm-checkbox') {
                    formControlAttr = childNode.attrs.filter(function (attr) {
                        return attr.name === 'formControlName';
                    });
                    if (formControlAttr.length) {
                        return;
                    }
                }
                childNode.attrs.push(new compiler.Attribute('[ngModelOptions]', '{standalone: true}', 1, 1));
            }
            addNgModelStandalone(childNode.children);
        });
    };
    var ɵ0$8 = addNgModelStandalone;
    // get the filter template (widget and filter menu) to be displayed in filter row
    var getFilterTemplate = function (attrs, pCounter) {
        var widget = attrs.get('filterwidget') || getDataTableFilterWidget(attrs.get('type') || core.DataType.STRING);
        var fieldName = attrs.get('binding');
        var type = attrs.get('type') || 'string';
        var datasourceBinding, submitEventBinding;
        var datasetAttr = attrs.get('filterdataset.bind');
        // when multicolumn is selected and filterwidget as autocomplete is assigned to dataset.
        if (attrs.get('filterwidget') === 'autocomplete') {
            if (datasetAttr) {
                var binddatasource = transpiler.getDataSource(datasetAttr);
                datasourceBinding = "dataset.bind=\"" + datasetAttr + "\" datasource.bind=\"" + binddatasource + "\"";
            }
            submitEventBinding = "submit.event=\"changeFn('" + fieldName + "')\"";
        }
        var innerTmpl = "#filterWidget formControlName=\"" + (fieldName + '_filter') + "\" " + datasourceBinding + " " + submitEventBinding + " change.event=\"changeFn('" + fieldName + "')\"\n                        disabled.bind=\"isDisabled('" + fieldName + "')\"";
        var options = { inputType: 'filterinputtype' };
        var widgetTmpl = "" + core.getFormWidgetTemplate(widget, innerTmpl, attrs, options);
        return "<ng-template #filterTmpl let-changeFn=\"changeFn\" let-isDisabled=\"isDisabled\">\n    <div class=\"input-group " + widget + "\" data-col-identifier=\"" + fieldName + "\">\n        " + widgetTmpl + "\n        <span class=\"input-group-addon filter-clear-icon\" *ngIf=\"" + pCounter + ".showClearIcon('" + fieldName + "')\">\n            <button class=\"btn-transparent btn app-button\" aria-label=\"Clear button\" type=\"button\" (click)=\"" + pCounter + ".clearRowFilter('" + fieldName + "')\">\n                <i class=\"app-icon wi wi-clear\" aria-hidden=\"true\"></i>\n            </button>\n         </span>\n        <span class=\"input-group-addon\" dropdown container=\"body\" (onShown)=\"" + pCounter + ".adjustContainer('" + fieldName + "');\">\n            <button class=\"btn-transparent btn app-button\" type=\"button\" dropdownToggle><i class=\"app-icon wi wi-filter-list\"></i></button>\n            <ul class=\"matchmode-dropdown dropdown-menu\" *dropdownMenu>\n                   <li *ngFor=\"let matchMode of " + pCounter + ".matchModeTypesMap['" + type + "']\"\n                        [ngClass]=\"{active: matchMode === (" + pCounter + ".rowFilter['" + fieldName + "'].matchMode || " + pCounter + ".matchModeTypesMap['" + type + "'][0])}\">\n                        <a href=\"javascript:void(0);\" (click)=\"" + pCounter + ".onFilterConditionSelect('" + fieldName + "', matchMode)\" [innerText]=\"" + pCounter + ".matchModeMsgs[matchMode]\"></a>\n                    </li>\n             </ul>\n        </span>\n    </div></ng-template>";
    };
    var ɵ1$6 = getFilterTemplate;
    var getEventsTmpl = function (attrs) {
        var tmpl = '';
        attrs.forEach(function (val, key) {
            if (key.endsWith('.event')) {
                tmpl += key + "=\"" + val + "\" ";
            }
        });
        return tmpl;
    };
    var ɵ2$4 = getEventsTmpl;
    // Generate inline edit properties template. Properties requiring row instance are generated here.
    var getInlineEditRowPropsTmpl = function (attrs) {
        var propAttrs = new Map();
        var props = ['disabled', 'disabled.bind'];
        props.forEach(function (prop) {
            if (attrs.get(prop)) {
                propAttrs.set(prop, attrs.get(prop));
                attrs.delete(prop);
            }
        });
        return transpiler.getAttrMarkup(propAttrs);
    };
    var ɵ3$2 = getInlineEditRowPropsTmpl;
    // get the inline widget template
    var getInlineEditWidgetTmpl = function (attrs, isNewRow, errorstyle, pCounter) {
        var options = {};
        var fieldName = attrs.get('binding');
        var widget = attrs.get('edit-widget-type') || getEditModeWidget({
            'type': attrs.get('type'),
            'related-entity-name': attrs.get('related-entity-name'),
            'primary-key': attrs.get('primary-key')
        });
        var widgetRef = '';
        var formControl = '';
        var wmFormWidget = '';
        if (widget === core.FormWidgetType.UPLOAD) {
            options.uploadProps = {
                formName: idGen$5.nextUid(),
                name: fieldName
            };
            options.counter = pCounter;
        }
        else {
            widgetRef = isNewRow ? '#inlineWidgetNew' : '#inlineWidget';
            formControl = isNewRow ? "formControlName=\"" + fieldName + "_new\"" : "formControlName=\"" + fieldName + "\"";
            wmFormWidget = 'wmFormWidget';
        }
        options.inputType = 'editinputtype';
        var tmplRef = isNewRow ? '#inlineWidgetTmplNew' : '#inlineWidgetTmpl';
        var eventsTmpl = widget === core.FormWidgetType.UPLOAD ? '' : getEventsTmpl(attrs);
        var rowPropsTl = getInlineEditRowPropsTmpl(attrs);
        var innerTmpl = widgetRef + " " + wmFormWidget + " key=\"" + fieldName + "\" data-field-name=\"" + fieldName + "\" " + formControl + " " + eventsTmpl + " " + rowPropsTl;
        var widgetTmpl = core.getFormWidgetTemplate(widget, innerTmpl, attrs, options);
        var pendingSpinnerStatus = isNewRow ? 'getPendingSpinnerStatusNew' : 'getPendingSpinnerStatus';
        var validationErrorTmpl = getValiationErrorTemplate(errorstyle);
        /* This element with error message is used to run Automation checks if any error message is present */
        var errorMsgElementForAutomation = '<span class="sr-only" *ngIf="getValidationMessage()">{{getValidationMessage()}}</span>';
        return "<ng-template " + tmplRef + " let-row=\"row\" let-getControl=\"getControl\" let-getValidationMessage=\"getValidationMessage\" let-" + pendingSpinnerStatus + "=\"" + pendingSpinnerStatus + "\">\n                <div data-col-identifier=\"" + fieldName + "\" >\n                     " + widgetTmpl + "\n                     " + validationErrorTmpl + "\n                     <div class=\"overlay\" *ngIf=\"" + pendingSpinnerStatus + "()\"><span aria-hidden=\"true\" class=\"form-field-spinner fa fa-circle-o-notch fa-spin form-control-feedback\"></span></div>\n                     " + errorMsgElementForAutomation + "\n                 </div>\n            </ng-template>";
    };
    var ɵ4$1 = getInlineEditWidgetTmpl;
    var getValiationErrorTemplate = function (errorTmplType) {
        var validationErrorTmpl = '';
        switch (errorTmplType) {
            case 'bottom':
                validationErrorTmpl = "<span class=\"help-block text-danger\" *ngIf=\"getValidationMessage() && getControl() && getControl().invalid && getControl().touched\">{{getValidationMessage()}}</span>";
                break;
            case 'hover':
            default:
                validationErrorTmpl = "<span placement=\"top\" container=\"body\" tooltip=\"{{getValidationMessage()}}\" class=\"text-danger wi wi-error\"\n                                        *ngIf=\"getValidationMessage() && getControl() && getControl().invalid && getControl().touched\">\n                                    </span>";
        }
        return validationErrorTmpl;
    };
    var ɵ5$1 = getValiationErrorTemplate;
    var getFormatExpression = function (attrs) {
        var columnValue = "row.getProperty('" + attrs.get('binding') + "')";
        var formatPattern = attrs.get('custompipeformat') || attrs.get('formatpattern');
        var colExpression = '';
        // For date time data types, if format pattern is not applied, Apply default toDate format
        if (core.isDateTimeType(attrs.get('type')) && (!formatPattern || formatPattern === 'None')) {
            attrs.set('formatpattern', 'toDate');
            attrs.delete('datepattern');
            formatPattern = 'toDate';
        }
        switch (formatPattern) {
            case 'toDate':
                colExpression = "{{" + columnValue + " | toDate: colDef.datepattern}}";
                break;
            case 'toCurrency':
                if (attrs.get('currencypattern')) {
                    colExpression = "{{" + columnValue + " | toCurrency: '" + attrs.get('currencypattern');
                    if (attrs.get('fractionsize')) {
                        colExpression += "': " + attrs.get('fractionsize') + "}}";
                    }
                    else {
                        colExpression += "'}}";
                    }
                }
                break;
            case 'numberToString':
                if (attrs.get('fractionsize')) {
                    colExpression = "{{" + columnValue + " | numberToString: '" + attrs.get('fractionsize') + "'}}";
                }
                break;
            case 'stringToNumber':
                colExpression = "{{" + columnValue + " | stringToNumber}}";
                break;
            case 'timeFromNow':
                colExpression = "{{" + columnValue + " | timeFromNow}}";
                break;
            case 'prefix':
                if (attrs.get('prefix')) {
                    colExpression = "{{" + columnValue + " | prefix: '" + attrs.get('prefix') + "'}}";
                }
                break;
            case 'suffix':
                if (attrs.get('suffix')) {
                    colExpression = "{{" + columnValue + " | suffix: '" + attrs.get('suffix') + "'}}";
                }
                break;
            default:
                if (core.checkIsCustomPipeExpression(formatPattern)) {
                    colExpression = "{{" + columnValue + " |  " + formatPattern + " : " + "row" + "}}";
                }
                break;
        }
        return colExpression;
    };
    var ɵ6 = getFormatExpression;
    transpiler.register('wm-table-column', function () {
        return {
            requires: ['wm-table'],
            template: function (node, shared) {
                if (node.children.length) {
                    // If node has children, but an empty text node dont generate custom expression
                    if (node.children.length === 1 && node.children[0] instanceof compiler.Text && node.children[0].value.trim().length === 0) {
                        return;
                    }
                    shared.set('customExpression', true);
                    addNgModelStandalone(node.children);
                }
            },
            pre: function (attrs, shared, parentTable) {
                var rowFilterTmpl = '';
                var inlineEditTmpl = '';
                var inlineNewEditTmpl = '';
                var parentForm = '';
                if (parentTable) {
                    var pCounter = parentTable.get('table_reference');
                    rowFilterTmpl = (parentTable.get('filtermode') === 'multicolumn' && attrs.get('searchable') !== 'false') ? getFilterTemplate(attrs, pCounter) : '';
                    var editMode = parentTable.get('editmode');
                    var isInlineEdit = (editMode !== EDIT_MODE.DIALOG && editMode !== EDIT_MODE.FORM && attrs.get('readonly') !== 'true');
                    inlineEditTmpl = isInlineEdit ? getInlineEditWidgetTmpl(attrs, false, parentTable.get('errorstyle'), pCounter) : '';
                    inlineNewEditTmpl = isInlineEdit && editMode === EDIT_MODE.QUICK_EDIT && parentTable.get('shownewrow') !== 'false' ? getInlineEditWidgetTmpl(attrs, true, attrs.get('errorstyle')) : '';
                    parentForm = " [formGroup]=\"" + pCounter + ".ngform\" ";
                }
                var formatPattern = attrs.get('formatpattern');
                var customExpr = "<ng-template #customExprTmpl let-row=\"row\" let-colDef=\"colDef\" let-editRow=\"editRow\" let-deleteRow=\"deleteRow\" let-addNewRow=\"addNewRow\">";
                var customExprTmpl = '';
                var formatExprTmpl = '';
                if (shared.get('customExpression')) {
                    attrs.set('customExpression', 'true');
                    customExprTmpl = customExpr + "<div data-col-identifier=\"" + attrs.get('binding') + "\">";
                }
                else if ((formatPattern && formatPattern !== 'None') || core.isDateTimeType(attrs.get('type'))) {
                    formatExprTmpl = getFormatExpression(attrs);
                    if (formatExprTmpl) {
                        shared.set('customExpression', true);
                        attrs.set('customExpression', 'true');
                        customExprTmpl = customExpr + "<div data-col-identifier=\"" + attrs.get('binding') + "\" title=\"" + formatExprTmpl + "\">" + formatExprTmpl;
                    }
                }
                return "<" + tagName$1u + " wmTableColumn " + transpiler.getAttrMarkup(attrs) + " " + parentForm + ">\n                    " + rowFilterTmpl + "\n                    " + inlineEditTmpl + "\n                    " + inlineNewEditTmpl + "\n                    " + customExprTmpl;
            },
            post: function (attrs, shared) {
                var customExprTmpl = '';
                if (shared.get('customExpression')) {
                    customExprTmpl = "</div></ng-template>";
                }
                return customExprTmpl + "</" + tagName$1u + ">";
            },
            imports: function (attrs) {
                var editWidgetType = attrs.get('edit-widget-type');
                var filterWidgetType = attrs.get('filterwidget') || getDataTableFilterWidget(attrs.get('type') || core.DataType.STRING);
                return [core.getRequiredFormWidget(editWidgetType), core.getRequiredFormWidget(filterWidgetType), 'wm-table'];
            }
        };
    });
    var tableColumn_build = (function () { });

    var tableColumn_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tableColumn_build,
        ɵ0: ɵ0$8,
        ɵ1: ɵ1$6,
        ɵ2: ɵ2$4,
        ɵ3: ɵ3$2,
        ɵ4: ɵ4$1,
        ɵ5: ɵ5$1,
        ɵ6: ɵ6
    });

    var tagName$1v = 'div';
    var getRowExpansionActionTmpl = function (attrs) {
        var tag = attrs.get('widget-type') === 'anchor' ? 'a' : 'button';
        var directive = attrs.get('widget-type') === 'anchor' ? 'wmAnchor' : 'wmButton';
        return "<ng-template #rowExpansionActionTmpl let-row=\"row\">\n               <" + tag + " " + directive + "\n                    " + core.getRowActionAttrs(attrs) + "\n                    class=\"" + attrs.get('class') + " row-expansion-button\"\n                    iconclass=\"" + attrs.get('collapseicon') + "\"\n                    type=\"button\"></" + tag + ">\n            </ng-template>";
    };
    var ɵ0$9 = getRowExpansionActionTmpl;
    transpiler.register('wm-table-row', function () {
        return {
            pre: function (attrs) {
                return "<" + tagName$1v + " wmTableRow " + transpiler.getAttrMarkup(attrs) + ">\n                    " + getRowExpansionActionTmpl(attrs) + "\n                    <ng-template #rowExpansionTmpl let-row=\"row\" let-rowDef=\"rowDef\" let-containerLoad=\"containerLoad\">\n                        <div wmContainer partialContainer content.bind=\"rowDef.content\" load.event=\"containerLoad(widget)\"\n                            [ngStyle]=\"{'height': rowDef.height, 'overflow-y': 'auto'}\">\n                         <div *ngFor=\"let param of rowDef.partialParams | keyvalue\" wmParam hidden\n                            [name]=\"param.key\" [value]=\"param.value\"></div>";
            },
            post: function () { return "</div></ng-template></" + tagName$1v + ">"; }
        };
    });
    var tableRow_build = (function () { });

    var tableRow_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tableRow_build,
        ɵ0: ɵ0$9
    });

    var tagName$1w = 'div';
    var getSaveCancelTemplate = function () {
        return "<button type=\"button\" aria-label=\"Save edit icon\" class=\"save row-action-button btn app-button btn-transparent save-edit-row-button hidden\" title=\"Save\">\n                <i class=\"wi wi-done\" aria-hidden=\"true\"></i>\n            </button>\n            <button type=\"button\" aria-label=\"Cancel edit icon\" class=\"cancel row-action-button btn app-button btn-transparent cancel-edit-row-button hidden\" title=\"Cancel\">\n                <i class=\"wi wi-cancel\" aria-hidden=\"true\"></i>\n            </button>";
    };
    var ɵ0$a = getSaveCancelTemplate;
    // get the inline widget template
    var getRowActionTmpl = function (attrs) {
        var action = attrs.get('action');
        var actionTmpl = action ? " click.event.delayed=\"" + action + "\" " : '';
        var saveCancelTmpl = action && action.includes('editRow(') ? getSaveCancelTemplate() : '';
        var btnClass = action ? (action.includes('editRow(') ? 'edit edit-row-button' :
            (action.includes('deleteRow(') ? 'delete delete-row-button' : '')) : '';
        var tabIndex = attrs.get('tabindex') ? "tabindex=\"" + attrs.get('tabindex') + "\"" : '';
        var tag = attrs.get('widget-type') === 'anchor' ? 'a' : 'button';
        var directive = attrs.get('widget-type') === 'anchor' ? 'wmAnchor' : 'wmButton';
        return "<ng-template #rowActionTmpl let-row=\"row\">\n               <" + tag + " " + directive + " data-action-key=\"" + attrs.get('key') + "\"\n                    " + core.getRowActionAttrs(attrs) + "\n                    class=\"row-action row-action-button " + attrs.get('class') + " " + btnClass + "\"\n                    iconclass=\"" + attrs.get('iconclass') + "\"\n                    " + actionTmpl + "\n                    " + tabIndex + "\n                    type=\"button\"></" + tag + ">\n                " + saveCancelTmpl + "\n            </ng-template>";
    };
    var ɵ1$7 = getRowActionTmpl;
    transpiler.register('wm-table-row-action', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1w + " wmTableRowAction " + transpiler.getAttrMarkup(attrs) + ">\n                        " + getRowActionTmpl(attrs); },
            post: function () { return "</" + tagName$1w + ">"; }
        };
    });
    var tableRowAction_build = (function () { });

    var tableRowAction_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': tableRowAction_build,
        ɵ0: ɵ0$a,
        ɵ1: ɵ1$7
    });

    var tagName$1x = 'div';
    var dataSetKey$3 = 'dataset';
    var idGen$6 = new core.IDGenerator('table_');
    var columnIndex = 0;
    /**
     * This method assigns index to the table-column and column-groups in order to the maintain the columns in the same order
     * as specified in the configuration.
     * Scenario: ordering the columns when accessrole is specified on column.
     *
     * headerIndex:    ID  |      Name        |   Street  |  ==> top level header (ID has index 0, Name 1, Street 2)
     * index:         ---- |  fname | lname    |   ---    |  ==> sub level header ( ID 0, fname 1, lname 2, Street 3)
     * fname will have headerIndex as 1 and colIndex as 1. lname will have headerIndex as 1 and columnIndex as 2.
     * If suppose we have a group in the sub-level-header, then col index will be treated as the header index.
     *
     */
    function assignColumnIndex(node, parentIndex) {
        var headerIndex = 0;
        node.forEach(function (childNode) {
            var nodeName = childNode.name;
            var newheaderIndex = parentIndex !== undefined ? parentIndex : headerIndex;
            if (nodeName === 'wm-table-column' || nodeName === 'wm-table-column-group') {
                childNode.attrs.push(new compiler.Attribute('index', '' + columnIndex, 1, 1));
                childNode.attrs.push(new compiler.Attribute('headerIndex', '' + newheaderIndex, 1, 1));
                if (nodeName === 'wm-table-column-group') {
                    assignColumnIndex(childNode.children, headerIndex);
                }
                columnIndex++;
                headerIndex++;
            }
        });
    }
    transpiler.register('wm-table', function () {
        return {
            template: function (node, shared) {
                // If table does not have child columns, set isdynamictable to true
                if (node.children.length) {
                    var isColumnsPresent = node.children.some(function (childNode) {
                        return childNode.name === 'wm-table-column' || childNode.name === 'wm-table-column-group';
                    });
                    assignColumnIndex(node.children);
                    shared.set('isdynamictable', isColumnsPresent ? 'false' : 'true');
                }
                else {
                    shared.set('isdynamictable', 'true');
                }
                var datasetAttr = node.attrs.find(function (attr) { return attr.name === dataSetKey$3; });
                var widgetNameAttr = node.attrs.find(function (attr) { return attr.name === 'name'; });
                if (!datasetAttr) {
                    return;
                }
                var boundExpr = transpiler.getBoundToExpr(datasetAttr.value);
                if (!boundExpr) {
                    return;
                }
                core.updateTemplateAttrs(node, boundExpr, widgetNameAttr.value, '', 'row');
            },
            pre: function (attrs, shared) {
                var counter = idGen$6.nextUid();
                shared.set('counter', counter);
                attrs.set('isdynamictable', shared.get('isdynamictable'));
                return "<" + tagName$1x + " wmTable=\"" + counter + "\" wmTableFilterSort wmTableCUD #" + counter + " data-identifier=\"table\" role=\"table\" " + transpiler.getAttrMarkup(attrs) + ">";
            },
            post: function () { return "</" + tagName$1x + ">"; },
            provide: function (attrs, shared) {
                var provider = new Map();
                provider.set('table_reference', shared.get('counter'));
                provider.set('filtermode', attrs.get('filtermode'));
                provider.set('errorstyle', attrs.get('errorstyle'));
                provider.set('editmode', attrs.get('editmode'));
                provider.set('shownewrow', attrs.get('shownewrow'));
                return provider;
            }
        };
    });
    var table_build = (function () { });

    var table_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': table_build
    });

    var tagName$1y = 'div';
    transpiler.register('wm-video', function () {
        return {
            pre: function (attrs) { return "<" + tagName$1y + " wmVideo " + transpiler.getAttrMarkup(attrs) + ">"; },
            post: function () { return "</" + tagName$1y + ">"; }
        };
    });
    var video_build = (function () { });

    var video_build$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': video_build
    });

    var initComponentsBuildTask = function () { };

    Object.keys(transpiler).forEach(function (k) {
        if (k !== 'default') Object.defineProperty(exports, k, {
            enumerable: true,
            get: function () {
                return transpiler[k];
            }
        });
    });
    exports.accordionBuild = accordion_build$1;
    exports.accordionPaneBuild = accordionPane_build$1;
    exports.alertDlgBuild = alertDialog_build$1;
    exports.anchorBuild = anchor_build$1;
    exports.audioBuild = audio_build$1;
    exports.barcodeScannerBuild = barcodeScanner_build$1;
    exports.breadcrumbBuild = breadcrumb_build$1;
    exports.buttonBuild = button_build$1;
    exports.buttonGroupBuild = buttonGroup_build$1;
    exports.calendarBuild = calendar_build$1;
    exports.cameraBuild = camera_build$1;
    exports.cardActionsBuild = cardActions_build$1;
    exports.cardBuild = card_build$1;
    exports.cardContentBuild = cardContent_build$1;
    exports.cardFooterBuild = cardFooter_build$1;
    exports.carouselBuild = carousel_build$1;
    exports.carouselTemplateBuild = carouselTemplate_build$1;
    exports.chartBuild = chart_build$1;
    exports.checkboxBuild = checkbox_build$1;
    exports.checkboxSetBuild = checkboxset_build$1;
    exports.chipsBuild = chips_build$1;
    exports.colorPickerBuild = colorPicker_build$1;
    exports.compositeBuild = composite_build$1;
    exports.confirmDlgBuild = confirmDialog_build$1;
    exports.containerBuild = container_build$1;
    exports.contentBuild = content_build$1;
    exports.currencyBuild = currency_build$1;
    exports.dateBuild = date_build$1;
    exports.dateTimeBuild = dateTime_build$1;
    exports.dlgBuild = dialog_build$1;
    exports.dlgFooterBuild = dialogFooter_build$1;
    exports.fileUploadBuild = fileUpload_build$1;
    exports.footerBuild = footer_build$1;
    exports.formActionBuild = formAction_build$1;
    exports.formBuild = form_build$1;
    exports.formFieldBuild = formField_build$1;
    exports.headerBuild = header_build$1;
    exports.htmlBuild = html_build$1;
    exports.iconBuild = icon_build$1;
    exports.iframeBuild = iframe_build$1;
    exports.iframeDlgBuild = iframeDialog_build$1;
    exports.initComponentsBuildTask = initComponentsBuildTask;
    exports.labelBuild = label_build$1;
    exports.leftPanelBuild = leftPanel_build$1;
    exports.lgBuild = layoutGrid_build$1;
    exports.lgcolBuild = layoutGridColumn_build$1;
    exports.lgrowBuild = layoutGridRow_build$1;
    exports.listBuild = list_build$1;
    exports.liveTableBuild = liveTable_build$1;
    exports.loginBuild = login_build$1;
    exports.loginDlgBuild = loginDialog_build$1;
    exports.marqueeBuild = marquee_build$1;
    exports.mediaListBuild = mediaList_build$1;
    exports.mediaListItemBuild = mediaListItem_build$1;
    exports.menuBuild = menu_build$1;
    exports.messageBuild = message_build$1;
    exports.mobileNavbarBuild = mobileNavbar_build$1;
    exports.navBuild = nav_build$1;
    exports.navItemBuild = navItem_build$1;
    exports.navbarBuild = navbar_build$1;
    exports.numberBuild = number_build$1;
    exports.pageBuild = page_build$1;
    exports.pageContentBuild = pageContent_build$1;
    exports.paginationBuild = pagination_build$1;
    exports.panelBuild = panel_build$1;
    exports.partialBuild = partial_build$1;
    exports.partialDlgBuild = partialDialog_build$1;
    exports.partialParamBuild = partialParam_build$1;
    exports.pictureBuild = picture_build$1;
    exports.popoverBuild = popover_build$1;
    exports.prefabBuild = prefab_build$1;
    exports.prefabContainerBuild = prefabContainer_build$1;
    exports.progressBarBuild = progressBar_build$1;
    exports.progressCircleBuild = progressCircle_build$1;
    exports.radiosetBuild = radioset_build$1;
    exports.ratingBuild = rating_build$1;
    exports.rightPanelBuild = rightPanel_build$1;
    exports.rteBuild = richTextEditor_build$1;
    exports.searchBuild = search_build$1;
    exports.segContentBuild = segmentContent_build$1;
    exports.segControlBuild = segmentedControl_build$1;
    exports.selectBuild = select_build$1;
    exports.sliderBuild = slider_build$1;
    exports.spinnerBuild = spinner_build$1;
    exports.switchBuild = switch_build$1;
    exports.tabBarBuild = tabBar_build$1;
    exports.tabPaneBuild = tabPane_build$1;
    exports.tableActionBuild = tableAction_build$1;
    exports.tableBuild = table_build$1;
    exports.tableColumnBuild = tableColumn_build$1;
    exports.tableColumnGroupBuild = tableColumnGroup_build$1;
    exports.tableRowActionBuild = tableRowAction_build$1;
    exports.tableRowBuild = tableRow_build$1;
    exports.tabsBuild = tabs_build$1;
    exports.textAreaBuild = textarea_build$1;
    exports.textBuild = text_build$1;
    exports.tileBuild = tile_build$1;
    exports.timeBuild = time_build$1;
    exports.topNavBuild = topNav_build$1;
    exports.treeBuild = tree_build$1;
    exports.videoBuild = video_build$1;
    exports.wizardBuild = wizard_build$1;
    exports.wizardStepBuild = wizardStep_build$1;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.umd.js.map
