import { register, getAttrMarkup, getBoundToExpr, getFormMarkupAttr, getDataSource, getChildAttrs, setChildAttrs, clearChildAttrs } from '@wm/transpiler';
export * from '@wm/transpiler';
import { IDGenerator, updateTemplateAttrs, getNgModelAttr, FormWidgetType, DataType, getFormWidgetTemplate, isMobileApp, getRequiredFormWidget, isDateTimeType, checkIsCustomPipeExpression, getRowActionAttrs } from '@wm/core';
import { Attribute, Element, Text } from '@angular/compiler';

const carouselTagName = 'carousel';
const dataSetKey = 'dataset';
const idGen = new IDGenerator('wm_carousel_ref_');
const isDynamicCarousel = node => node.attrs.find(attr => attr.name === 'type' && attr.value === 'dynamic');
const ɵ0 = isDynamicCarousel;
register('wm-carousel', () => {
    return {
        pre: (attrs, shared) => {
            // generating unique Id for the carousel
            const counter = idGen.nextUid();
            shared.set('carousel_ref', counter);
            return `<div class="app-carousel carousel"><${carouselTagName} wmCarousel #${counter}="wmCarousel"  ${getAttrMarkup(attrs)} interval="0" [ngClass]="${counter}.navigationClass">`;
        },
        post: () => `</${carouselTagName}></div>`,
        template: (node) => {
            // check if the carousel is dynamic
            if (isDynamicCarousel(node)) {
                const datasetAttr = node.attrs.find(attr => attr.name === dataSetKey);
                const widgetNameAttr = node.attrs.find(attr => attr.name === 'name');
                if (!datasetAttr) {
                    return;
                }
                const boundExpr = getBoundToExpr(datasetAttr.value);
                if (!boundExpr) {
                    return;
                }
                updateTemplateAttrs(node, boundExpr, widgetNameAttr.value);
            }
        },
        // To provide parent carousel reference for children
        provide: (attrs, shared) => {
            const provider = new Map();
            provider.set('carousel_ref', shared.get('carousel_ref'));
            return provider;
        }
    };
});
var carousel_build = () => { };

var carousel_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': carousel_build,
    ɵ0: ɵ0
});

const carouselContentTagName = 'slide';
let counter = 1;
// For static carousel
register('wm-carousel-content', () => {
    return {
        pre: attrs => `<${carouselContentTagName} wmCarouselTemplate  ${getAttrMarkup(attrs)}>`,
        post: () => `</${carouselContentTagName}>`
    };
});
// For dynamic carousel
register('wm-carousel-template', () => {
    return {
        requires: ['wm-carousel'],
        pre: (attrs, shared, parentCarousel) => {
            const carouselRef = parentCarousel.get('carousel_ref');
            return `<div *ngIf="!${carouselRef}.fieldDefs">{{${carouselRef}.nodatamessage}}</div>
                    <${carouselContentTagName} wmCarouselTemplate  ${getAttrMarkup(attrs)} *ngFor="let item of ${carouselRef}.fieldDefs; let i = index;">
                        <ng-container [ngTemplateOutlet]="carouselTempRef${counter}" [ngTemplateOutletContext]="{item:item, index:i}"></ng-container>
                    </${carouselContentTagName}>
                    <ng-template #carouselTempRef${counter++} let-item="item" let-index="index">`;
        },
        post: () => `</ng-template>`
    };
});
var carouselTemplate_build = () => { };

var carouselTemplate_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': carouselTemplate_build
});

const tagName = 'div';
register('wm-login', () => {
    return {
        pre: attrs => `<${tagName} wmLogin ${getAttrMarkup(attrs)} eventsource.bind="Actions.loginAction">`,
        post: () => `</${tagName}>`,
        provide: () => {
            const provider = new Map();
            provider.set('isLogin', true);
            return provider;
        }
    };
});
var login_build = () => { };

var login_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': login_build
});

const tagName$1 = 'marquee';
register('wm-marquee', () => {
    return {
        pre: attrs => `<${tagName$1} onmouseover="this.stop();" onmouseout="this.start();" wmMarquee role="marquee" aria-live="off" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1}>`
    };
});
var marquee_build = () => { };

var marquee_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': marquee_build
});

const tagName$2 = 'a';
register('wm-anchor', () => {
    return {
        pre: attrs => `<${tagName$2} wmAnchor role="link" data-identifier="anchor" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$2}>`
    };
});
var anchor_build = () => { };

var anchor_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': anchor_build
});

const tagName$3 = 'div';
register('wm-audio', () => {
    return {
        pre: attrs => `<${tagName$3} wmAudio ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$3}>`
    };
});
var audio_build = () => { };

var audio_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': audio_build
});

const tagName$4 = 'div';
register('wm-html', () => {
    return {
        pre: attrs => `<${tagName$4} wmHtml ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$4}>`
    };
});
var html_build = () => { };

var html_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': html_build
});

const tagName$5 = 'span';
register('wm-icon', () => {
    return {
        pre: attrs => `<${tagName$5} wmIcon aria-hidden="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$5}>`
    };
});
var icon_build = () => { };

var icon_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': icon_build
});

const tagName$6 = 'div';
register('wm-iframe', () => {
    return {
        pre: attrs => `<${tagName$6} wmIframe ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$6}>`
    };
});
var iframe_build = () => { };

var iframe_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': iframe_build
});

const tagName$7 = 'label';
register('wm-label', () => {
    return {
        pre: attrs => `<${tagName$7} wmLabel ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$7}>`
    };
});
var label_build = () => { };

var label_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': label_build
});

const tagName$8 = 'img';
register('wm-picture', () => {
    return {
        pre: attrs => `<${tagName$8} wmPicture alt="image" wmImageCache="${attrs.get('offline') || 'true'}" ${getAttrMarkup(attrs)}>`
    };
});
var picture_build = () => { };

var picture_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': picture_build
});

const tagName$9 = 'div';
register('wm-spinner', () => {
    return {
        pre: attrs => `<${tagName$9} wmSpinner role="loading" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$9}>`
    };
});
var spinner_build = () => { };

var spinner_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': spinner_build
});

const tagName$a = 'div';
const getAttr = (node, attrName) => node.attrs.find(attr => attr.name === attrName);
const ɵ0$1 = getAttr;
const getAttrValue = (node, attrName) => {
    const match = getAttr(node, attrName);
    if (match) {
        return match.value;
    }
};
const ɵ1 = getAttrValue;
const getReplaceRegex = (v) => new RegExp(`bind:(${v}|${v}\\[\\$i])\\.`, 'g');
const ɵ2 = getReplaceRegex;
register('wm-progress-bar', () => {
    return {
        template: (node) => {
            const dataset = getAttrValue(node, 'dataset');
            const boundExpr = getBoundToExpr(dataset);
            if (boundExpr) {
                let type = getAttrValue(node, 'type');
                let datavalue = getAttrValue(node, 'datavalue');
                const replaceRegex = getReplaceRegex(boundExpr);
                if (type && type.includes(boundExpr)) {
                    type = type.replace(replaceRegex, '');
                    getAttr(node, 'type').value = type;
                }
                if (datavalue && datavalue.includes(boundExpr)) {
                    datavalue = datavalue.replace(replaceRegex, '');
                    getAttr(node, 'datavalue').value = datavalue;
                }
            }
        },
        pre: attrs => `<${tagName$a} wmProgressBar ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$a}>`
    };
});
var progressBar_build = () => { };

var progressBar_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': progressBar_build,
    ɵ0: ɵ0$1,
    ɵ1: ɵ1,
    ɵ2: ɵ2
});

const tagName$b = 'div';
register('wm-progress-circle', () => {
    return {
        pre: attrs => `<${tagName$b} wmProgressCircle ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$b}>`
    };
});
var progressCircle_build = () => { };

var progressCircle_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': progressCircle_build
});

const tagName$c = 'div';
register('wm-richtexteditor', () => {
    return {
        pre: attrs => `<${tagName$c} wmRichTextEditor role="textbox" ${getFormMarkupAttr(attrs)}>`,
        post: () => `</${tagName$c}>`
    };
});
var richTextEditor_build = () => { };

var richTextEditor_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': richTextEditor_build
});

const tagName$d = 'div';
register('wm-search', () => {
    return {
        pre: attrs => `<${tagName$d} wmSearch ${getFormMarkupAttr(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$d}>`
    };
});
var search_build = () => { };

var search_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': search_build
});

const tagName$e = 'div';
register('wm-tree', () => {
    return {
        pre: attrs => `<${tagName$e} wmTree redrawable ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$e}>`
    };
});
var tree_build = () => { };

var tree_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tree_build
});

const tagName$f = 'div';
register('wm-card', () => {
    return {
        pre: attrs => `<${tagName$f} wmCard ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$f}>`
    };
});
var card_build = () => { };

var card_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': card_build
});

const tagName$g = 'div';
register('wm-card-content', () => {
    return {
        pre: attrs => `<${tagName$g} wmCardContent partialContainer ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$g}>`
    };
});
var cardContent_build = () => { };

var cardContent_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': cardContent_build
});

const tagName$h = 'div';
register('wm-card-actions', () => {
    return {
        pre: attrs => `<${tagName$h} wmCardActions ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$h}>`
    };
});
var cardActions_build = () => { };

var cardActions_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': cardActions_build
});

const tagName$i = 'div';
register('wm-card-footer', () => {
    return {
        pre: attrs => `<${tagName$i} wmCardFooter ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$i}>`
    };
});
var cardFooter_build = () => { };

var cardFooter_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': cardFooter_build
});

const tagName$j = 'div';
register('wm-chart', () => {
    return {
        pre: attrs => `<${tagName$j} wmChart redrawable ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$j}>`
    };
});
var chart_build = () => { };

var chart_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': chart_build
});

const tagName$k = 'div';
register('wm-accordion', () => {
    return {
        pre: attrs => `<${tagName$k} wmAccordion role="tablist" aria-multiselectable="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$k}>`
    };
});
var accordion_build = () => { };

var accordion_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': accordion_build
});

const tagName$l = 'div';
register('wm-accordionpane', () => {
    return {
        pre: attrs => `<${tagName$l} wmAccordionPane partialContainer wm-navigable-element="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$l}>`
    };
});
var accordionPane_build = () => { };

var accordionPane_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': accordionPane_build
});

const tagName$m = 'div';
register('wm-container', () => {
    return {
        pre: attrs => `<${tagName$m} wmContainer partialContainer wmSmoothscroll="${attrs.get('smoothscroll') || 'false'}" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$m}>`
    };
});
var container_build = () => { };

var container_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': container_build
});

const tagName$n = 'div';
register('wm-gridcolumn', () => {
    return {
        pre: attrs => `<${tagName$n} wmLayoutGridColumn ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$n}>`
    };
});
var layoutGridColumn_build = () => { };

var layoutGridColumn_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': layoutGridColumn_build
});

const tagName$o = 'div';
register('wm-gridrow', () => {
    return {
        pre: attrs => `<${tagName$o} wmLayoutGridRow ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$o}>`
    };
});
var layoutGridRow_build = () => { };

var layoutGridRow_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': layoutGridRow_build
});

const tagName$p = 'div';
register('wm-layoutgrid', () => {
    return {
        pre: attrs => `<${tagName$p} wmLayoutGrid ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$p}>`
    };
});
var layoutGrid_build = () => { };

var layoutGrid_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': layoutGrid_build
});

const tagName$q = 'div';
register('wm-panel', () => {
    return {
        pre: attrs => `<${tagName$q} wmPanel partialContainer aria-label="panel" wm-navigable-element="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$q}>`
    };
});
register('wm-panel-footer', () => {
    return {
        pre: attrs => `<${tagName$q} wmPanelFooter aria-label="panel footer" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$q}>`
    };
});
var panel_build = () => { };

var panel_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': panel_build
});

const tagName$r = 'div';
register('wm-segmented-control', () => {
    return {
        pre: attrs => `<${tagName$r} wmSegmentedControl ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$r}>`
    };
});
var segmentedControl_build = () => { };

var segmentedControl_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': segmentedControl_build
});

const tagName$s = 'li';
register('wm-segment-content', () => {
    return {
        pre: attrs => `<${tagName$s} wmSegmentContent partialContainer wmSmoothscroll=${attrs.get('smoothscroll') || 'true'} wm-navigable-element="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$s}>`
    };
});
var segmentContent_build = () => { };

var segmentContent_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': segmentContent_build
});

const tagName$t = 'div';
register('wm-tabs', () => {
    return {
        pre: attrs => `<${tagName$t} wmTabs ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$t}>`
    };
});
var tabs_build = () => { };

var tabs_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tabs_build
});

const tagName$u = 'div';
register('wm-tabpane', () => {
    return {
        pre: attrs => `<${tagName$u} wmTabPane  partialContainer ${getAttrMarkup(attrs)} wm-navigable-element="true" role="tabpanel">`,
        post: () => `</${tagName$u}>`
    };
});
var tabPane_build = () => { };

var tabPane_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tabPane_build
});

const tagName$v = 'div';
register('wm-tile', () => {
    return {
        pre: attrs => `<${tagName$v} wmTile aria-describedby="Tile" wm-navigable-element="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$v}>`
    };
});
var tile_build = () => { };

var tile_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tile_build
});

const tagName$w = 'div';
register('wm-wizard', () => {
    return {
        pre: attrs => `<${tagName$w} wmWizard role="tablist" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$w}>`
    };
});
var wizard_build = () => { };

var wizard_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': wizard_build
});

const tagName$x = 'form';
const idGen$1 = new IDGenerator('wizard_step_id_');
register('wm-wizardstep', () => {
    return {
        pre: attrs => {
            const counter = idGen$1.nextUid();
            return `<${tagName$x} wmWizardStep #${counter}="wmWizardStep" ${getAttrMarkup(attrs)}>
                       <ng-template [ngIf]="${counter}.isInitialized">`;
        },
        post: () => `</ng-template></${tagName$x}>`
    };
});
var wizardStep_build = () => { };

var wizardStep_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': wizardStep_build
});

const tagName$y = 'button';
register('wm-barcodescanner', () => {
    return {
        pre: attrs => `<${tagName$y} wmBarcodescanner ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$y}>`
    };
});
var barcodeScanner_build = () => { };

var barcodeScanner_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': barcodeScanner_build
});

const tagName$z = 'button';
register('wm-camera', () => {
    return {
        pre: attrs => `<${tagName$z} type='button' wmCamera ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$z}>`
    };
});
var camera_build = () => { };

var camera_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': camera_build
});

const tagName$A = 'div';
register('wm-alertdialog', () => {
    return {
        pre: attrs => `<${tagName$A} wmAlertDialog role="alertdialog" wm-navigable-element="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$A}>`
    };
});
var alertDialog_build = () => { };

var alertDialog_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': alertDialog_build
});

const tagName$B = 'div';
register('wm-confirmdialog', () => {
    return {
        pre: attrs => `<${tagName$B} wmConfirmDialog wm-navigable-element="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$B}>`
    };
});
var confirmDialog_build = () => { };

var confirmDialog_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': confirmDialog_build
});

const tagName$C = 'div';
register('wm-dialogactions', () => {
    return {
        pre: attrs => `<ng-template #dialogFooter><${tagName$C} wmDialogFooter data-identfier="actions" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$C}></ng-template>`
    };
});
var dialogFooter_build = () => { };

var dialogFooter_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': dialogFooter_build
});

const tagName$D = 'div';
register('wm-dialog', () => {
    return {
        pre: attrs => `<${tagName$D} wmDialog ${getAttrMarkup(attrs)} wm-navigable-element="true"><ng-template #dialogBody>`,
        post: () => `</ng-template></${tagName$D}>`
    };
});
// Todo:vinay remove wm-view in migration
register('wm-view', () => {
    return {
        pre: attrs => '',
        post: () => ''
    };
});
var dialog_build = () => { };

var dialog_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': dialog_build
});

const tagName$E = 'div';
register('wm-iframedialog', () => {
    return {
        pre: attrs => `<${tagName$E} wmIframeDialog wm-navigable-element="true" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$E}>`
    };
});
var iframeDialog_build = () => { };

var iframeDialog_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': iframeDialog_build
});

const tagName$F = 'div';
register('wm-logindialog', () => {
    return {
        pre: attrs => `<${tagName$F} wmDialog wmLoginDialog ${getAttrMarkup(attrs)} eventsource.bind="Actions.loginAction" wm-navigable-element="true"><ng-template #dialogBody>`,
        post: () => `</ng-template></${tagName$F}>`
    };
});
var loginDialog_build = () => { };

var loginDialog_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': loginDialog_build
});

const tagName$G = 'div';
register('wm-pagedialog', () => {
    return {
        pre: (attrs, shared) => {
            const content = attrs.get('content');
            attrs.delete('content');
            const boundContent = attrs.get('content.bind');
            attrs.delete('content.bind');
            const onLoad = attrs.get('load.event');
            attrs.delete('load.event');
            let onLoadEvtMarkup = '';
            let contentMarkup = '';
            if (onLoad) {
                onLoadEvtMarkup = `load.event="${onLoad}"`;
            }
            if (boundContent) {
                contentMarkup = `content.bind="${boundContent}"`;
            }
            else if (content) {
                contentMarkup = `content="${content}"`;
            }
            let containerMarkup = '';
            if (contentMarkup) {
                shared.set('hasPartialContent', true);
                containerMarkup += `<ng-template><div wmContainer #partial partialContainer ${contentMarkup} width="100%" height="100%" ${onLoadEvtMarkup}>`;
            }
            return `<${tagName$G} wmPartialDialog ${getAttrMarkup(attrs)}>${containerMarkup}`;
        },
        post: (attrs, shared) => {
            let preContent = '';
            if (shared.get('hasPartialContent')) {
                preContent = `</div></ng-template>`;
            }
            return `${preContent}</${tagName$G}>`;
        }
    };
});
var partialDialog_build = () => { };

var partialDialog_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': partialDialog_build
});

const ALLFIELDS = 'All Fields';
const EDIT_MODE = {
    QUICK_EDIT: 'quickedit',
    INLINE: 'inline',
    FORM: 'form',
    DIALOG: 'dialog'
};
const fieldTypeWidgetTypeMap = {
    'integer': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
    'big_integer': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
    'short': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'float': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'big_decimal': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'number': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'double': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'long': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
    'byte': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'string': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'switch', 'currency', 'autocomplete', 'chips', 'colorpicker'],
    'character': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'switch', 'currency', 'autocomplete', 'chips'],
    'text': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'switch', 'currency', 'autocomplete', 'chips', 'colorpicker'],
    'date': ['date', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'time': ['time', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'timestamp': ['timestamp', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'datetime': ['datetime', 'text', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'boolean': ['checkbox', 'radioset', 'toggle', 'select'],
    'list': ['select', 'radioset', 'checkboxset', 'switch', 'autocomplete', 'chips'],
    'clob': ['text', 'textarea', 'richtext'],
    'blob': ['upload'],
    'file': ['upload'],
    'custom': ['text', 'number', 'textarea', 'password', 'checkbox', 'toggle', 'slider', 'richtext', 'currency', 'switch',
        'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'rating', 'datetime', 'autocomplete', 'chips', 'colorpicker']
};
const DATASET_WIDGETS = new Set([FormWidgetType.SELECT, FormWidgetType.CHECKBOXSET, FormWidgetType.RADIOSET,
    FormWidgetType.SWITCH, FormWidgetType.AUTOCOMPLETE, FormWidgetType.CHIPS, FormWidgetType.TYPEAHEAD, FormWidgetType.RATING]);
const isDataSetWidget = widget => {
    return DATASET_WIDGETS.has(widget);
};
// Get filter widget applicable to the given type
const getDataTableFilterWidget = type => {
    let widget = fieldTypeWidgetTypeMap[type] && fieldTypeWidgetTypeMap[type][0];
    if (type === DataType.BOOLEAN) {
        widget = FormWidgetType.SELECT;
    }
    const filterWidgets = [FormWidgetType.TEXT, FormWidgetType.NUMBER, FormWidgetType.SELECT, FormWidgetType.AUTOCOMPLETE,
        FormWidgetType.DATE, FormWidgetType.TIME, FormWidgetType.DATETIME];
    if (filterWidgets.includes(widget)) {
        return widget;
    }
    return FormWidgetType.TEXT;
};
/**
 * @ngdoc function
 * @name wm.widgets.live.getEditModeWidget
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * This function returns the default widget for grid
 *
 * @param {object} colDef field definition
 */
const getEditModeWidget = colDef => {
    if (colDef['related-entity-name'] && colDef['primary-key']) {
        return FormWidgetType.SELECT;
    }
    return (fieldTypeWidgetTypeMap[colDef.type] && fieldTypeWidgetTypeMap[colDef.type][0]) || FormWidgetType.TEXT;
};

const tagName$H = 'div';
const idGen$2 = new IDGenerator('formfield_');
const getEventsTemplate = (attrs) => {
    const eventAttrs = new Map();
    if (!attrs.has('focus.event')) {
        attrs.set('focus.event', '');
    }
    if (!attrs.has('blur.event')) {
        attrs.set('blur.event', '');
    }
    attrs.forEach((value, key) => {
        if (key.endsWith('.event')) {
            if (key === 'focus.event') {
                value = `_onFocusField($event);${value}`;
            }
            else if (key === 'blur.event') {
                value = `_onBlurField($event);${value}`;
            }
            eventAttrs.set(key, value);
            attrs.delete(key);
        }
    });
    return getFormMarkupAttr(eventAttrs);
};
const ɵ0$2 = getEventsTemplate;
const DEFAULT_PLACEHOLDERS = new Map([
    [FormWidgetType.SELECT, ['Select Min value', 'Select Max value', 'Select value']],
    [FormWidgetType.DATETIME, ['Select Min date time', 'Select Max date time', 'Select date time']],
    [FormWidgetType.TIME, ['Select Min time', 'Select Max time', 'Select time']],
    [FormWidgetType.DATE, ['Select Min date', 'Select Max date', 'Select date']],
    [FormWidgetType.TEXTAREA, ['', '', 'Enter value']],
    [FormWidgetType.RICHTEXT, ['', '', 'Enter value']],
    [FormWidgetType.COLORPICKER, ['Select Color', 'Select Color', 'Select Color']],
    [FormWidgetType.CHIPS, ['', '', 'Type here...']],
    [FormWidgetType.PASSWORD, ['Enter Min value', 'Enter Max value', 'Enter value']],
    [FormWidgetType.NUMBER, ['Enter Min value', 'Enter Max value', 'Enter value']],
    [FormWidgetType.TEXT, ['Enter Min value', 'Enter Max value', 'Enter value']],
    [FormWidgetType.CURRENCY, ['Enter Min value', 'Enter Max value', 'Enter value']],
    [FormWidgetType.AUTOCOMPLETE, ['', '', 'Search']],
]);
const setDefaultPlaceholder = (attrs, widgetType, index) => {
    const prop = index === 1 ? 'maxplaceholder' : 'placeholder';
    let placeholder = attrs.get(prop);
    if (placeholder || placeholder === '') {
        return;
    }
    placeholder = DEFAULT_PLACEHOLDERS.get(widgetType) && DEFAULT_PLACEHOLDERS.get(widgetType)[index];
    if (placeholder) {
        attrs.set(prop, placeholder);
    }
};
const ɵ1$1 = setDefaultPlaceholder;
const getWidgetTemplate = (attrs, options) => {
    const name = attrs.get('name');
    const fieldName = (attrs.get('key') || name || '').trim();
    const formControl = options.isMaxWidget ? `formControlName="${fieldName}_max"` : (options.isInList ? `[formControlName]="${options.counter}._fieldName"` : `formControlName="${fieldName}"`);
    const tmplRef = options.isMaxWidget ? `#formWidgetMax` : `#formWidget`;
    const widgetName = name ? (options.isMaxWidget ? `name="${name}_formWidgetMax"` : `name="${name}_formWidget"`) : '';
    const conditionalClass = `[ngClass]="${attrs.get('ngclass')}"`;
    const defaultTmpl = `[class.hidden]="!${options.pCounter}.isUpdateMode && ${options.counter}.viewmodewidget !== 'default'" ${formControl} ${options.eventsTmpl} ${conditionalClass} ${tmplRef} ${widgetName}`;
    return getFormWidgetTemplate(options.widgetType, defaultTmpl, attrs, { counter: options.counter, pCounter: options.pCounter });
};
const ɵ2$1 = getWidgetTemplate;
const getTemplate = (attrs, widgetType, eventsTmpl, counter, pCounter, isInList) => {
    const isRange = attrs.get('is-range') === 'true';
    if (!isRange) {
        return getWidgetTemplate(attrs, { widgetType, eventsTmpl, counter, pCounter, isInList });
    }
    const layoutClass = isMobileApp() ? 'col-xs-6' : 'col-sm-6';
    return `<div class="${layoutClass}">${getWidgetTemplate(attrs, { widgetType, eventsTmpl, counter, pCounter })}</div>
                <div class="${layoutClass}">${getWidgetTemplate(attrs, { widgetType, eventsTmpl, counter, pCounter, isMaxWidget: true })}</div>`;
};
const ɵ3 = getTemplate;
const getCaptionByWidget = (attrs, widgetType, counter) => {
    if (attrs.get('is-related') === 'true') {
        return `${counter}.getDisplayExpr()`;
    }
    if (widgetType === FormWidgetType.PASSWORD) {
        return '\'********\'';
    }
    let caption = `${counter}.value`;
    if (widgetType === FormWidgetType.DATETIME || widgetType === FormWidgetType.TIMESTAMP) {
        caption += ` | toDate:${counter}.formWidget.datepattern || 'yyyy-MM-dd hh:mm:ss a'`;
        return caption;
    }
    if (widgetType === FormWidgetType.TIME) {
        caption += ` | toDate:${counter}.formWidget.timepattern || 'hh:mm a'`;
        return caption;
    }
    if (widgetType === FormWidgetType.DATE) {
        caption += ` | toDate:${counter}.formWidget.datepattern ||  'yyyy-MMM-dd'`;
        return caption;
    }
    if (widgetType === FormWidgetType.RATING || widgetType === FormWidgetType.UPLOAD) {
        return '';
    }
    if (isDataSetWidget(widgetType) && attrs.get('datafield') === ALLFIELDS) {
        return `${counter}.getDisplayExpr()`;
    }
    return `${counter}.getCaption()`;
};
const ɵ4 = getCaptionByWidget;
const registerFormField = (isFormField) => {
    return {
        requires: ['wm-form', 'wm-liveform', 'wm-livefilter', 'wm-list'],
        pre: (attrs, shared, parentForm, parentLiveForm, parentFilter, parentList) => {
            const counter = idGen$2.nextUid();
            const parent = parentForm || parentLiveForm || parentFilter;
            const pCounter = (parent && parent.get('form_reference')) || 'form';
            const widgetType = attrs.get('widget') || FormWidgetType.TEXT;
            const dataRole = isFormField ? 'form-field' : 'filter-field';
            const validationMsg = isFormField ? `<p *ngIf="${counter}._control?.invalid && ${counter}._control?.touched && ${pCounter}.isUpdateMode"
                                   class="help-block text-danger"
                                   [textContent]="${counter}.validationmessage"></p>` : '';
            const eventsTmpl = widgetType === FormWidgetType.UPLOAD ? '' : getEventsTemplate(attrs);
            const controlLayout = isMobileApp() ? 'col-xs-12' : 'col-sm-12';
            const isInList = pCounter === (parentList && parentList.get('parent_form_reference'));
            attrs.set('__widgetType', widgetType);
            attrs.delete('widget');
            shared.set('counter', counter);
            if (attrs.get('is-range') === 'true') {
                setDefaultPlaceholder(attrs, widgetType, 0);
                setDefaultPlaceholder(attrs, widgetType, 1);
            }
            else {
                setDefaultPlaceholder(attrs, widgetType, 2);
            }
            return `<${tagName$H} data-role="${dataRole}" [formGroup]="${pCounter}.ngform" wmFormField #${counter}="wmFormField" widgettype="${widgetType}" ${getFormMarkupAttr(attrs)}>
                        <div class="live-field form-group app-composite-widget clearfix caption-{{${pCounter}.captionposition}}" widget="${widgetType}">
                            <label [hidden]="!${counter}.displayname" class="app-label control-label formfield-label {{${pCounter}._captionClass}}" [title]="${counter}.displayname"
                                        [ngStyle]="{width: ${pCounter}.captionsize}" [ngClass]="{'text-danger': ${counter}._control?.invalid && ${counter}._control?.touched && ${pCounter}.isUpdateMode,
                                         required: ${pCounter}.isUpdateMode && ${counter}.required}" [textContent]="${counter}.displayname"> </label>
                            <div [ngClass]="${counter}.displayname ? ${pCounter}._widgetClass : '${controlLayout}'">
                                 <label class="form-control-static app-label"
                                       [hidden]="${pCounter}.isUpdateMode || ${counter}.viewmodewidget === 'default' || ${counter}.widgettype === 'upload'" [innerHTML]="${getCaptionByWidget(attrs, widgetType, counter)}"></label>
                                ${getTemplate(attrs, widgetType, eventsTmpl, counter, pCounter, isInList)}
                                <span aria-hidden="true" *ngIf="${counter}.showPendingSpinner" class="form-field-spinner fa fa-circle-o-notch fa-spin form-control-feedback"></span>
                                <p *ngIf="!(${counter}._control?.invalid && ${counter}._control?.touched) && ${pCounter}.isUpdateMode"
                                   class="help-block" [textContent]="${counter}.hint"></p>
                                ${validationMsg}
                            </div>
                        </div>`;
        },
        post: () => `</${tagName$H}>`,
        provide: (attrs, shared) => {
            const provider = new Map();
            provider.set('form_reference', shared.get('counter'));
            return provider;
        },
        imports: (attrs) => {
            const requiredWidget = getRequiredFormWidget(attrs.get('__widgetType') || attrs.get('type'));
            attrs.delete('__widgetType');
            return [requiredWidget, 'wm-form'];
        }
    };
};
const ɵ5 = registerFormField;
register('wm-form-field', registerFormField.bind(this, true));
register('wm-filter-field', registerFormField.bind(this, false));
var formField_build = () => { };

var formField_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': formField_build,
    ɵ0: ɵ0$2,
    ɵ1: ɵ1$1,
    ɵ2: ɵ2$1,
    ɵ3: ɵ3,
    ɵ4: ɵ4,
    ɵ5: ɵ5
});

const tagName$I = 'div';
const registerAction = (tmpl) => {
    return {
        pre: attrs => `<${tagName$I} wmFormAction name="${attrs.get('name') || attrs.get('key')}" ${getAttrMarkup(attrs)} ${tmpl}>`,
        post: () => `</${tagName$I}>`
    };
};
const ɵ0$3 = registerAction;
register('wm-form-action', registerAction.bind(this, ''));
register('wm-filter-action', registerAction.bind(this, ` update-mode="true" `));
var formAction_build = () => { };

var formAction_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': formAction_build,
    ɵ0: ɵ0$3
});

const tagName$J = 'form';
const idGen$3 = new IDGenerator('form_');
const formWidgets = new Set([
    'wm-text',
    'wm-textarea',
    'wm-checkbox',
    'wm-slider',
    'wm-richtexteditor',
    'wm-currency',
    'wm-switch',
    'wm-select',
    'wm-checkboxset',
    'wm-radioset',
    'wm-date',
    'wm-time',
    'wm-timestamp',
    'wm-upload',
    'wm-rating',
    'wm-datetime',
    'wm-search',
    'wm-chips',
    'wm-colorpicker',
    'wm-table'
]);
let numberOfFields;
const addFormControlName = (children = []) => {
    children.forEach(childNode => {
        if (childNode.name === 'wm-form-field') {
            numberOfFields++;
        }
        if (formWidgets.has(childNode.name)) {
            let key = childNode.attrs.find((attr) => attr.name === 'key' || attr.name === 'name');
            key = key && key.value;
            if (!key) {
                // for checkbox inside table inside form, key and name attrs are not available. Using datavalue to construct formControlName instead.
                let dataValue = childNode.attrs.find((attr) => attr.name === 'datavalue');
                if (dataValue && dataValue.value) {
                    const regex = /\((.*)\)/;
                    dataValue = dataValue.value.match(regex);
                    if (dataValue && dataValue.length > 1) {
                        key = dataValue[1].replace(/["']/g, '');
                    }
                }
            }
            childNode.attrs.push(new Attribute('formControlName', key, 1, 1));
            childNode.attrs.push(new Attribute('wmFormWidget', '', 1, 1));
        }
        addFormControlName(childNode.children);
    });
};
const ɵ0$4 = addFormControlName;
const updateFormDataSource = (attrMap) => {
    if (attrMap.get('formdata.bind')) {
        const formDataSource = getDataSource(attrMap.get('formdata.bind'));
        if (formDataSource) {
            attrMap.set('formdatasource.bind', formDataSource);
        }
    }
};
const ɵ1$2 = updateFormDataSource;
const buildTask = (directiveAttr = '') => {
    return {
        requires: ['wm-livetable', 'wm-login'],
        template: (node) => {
            numberOfFields = 0;
            addFormControlName(node.children);
        },
        pre: (attrs, shared, parentLiveTable, parentLoginWidget) => {
            let tmpl;
            let dialogId;
            const role = parentLoginWidget && parentLoginWidget.get('isLogin') ? 'app-login' : '';
            const counter = idGen$3.nextUid();
            const dependsOn = attrs.get('dependson') ? `dependson="${attrs.get('dependson')}"` : '';
            const classProp = attrs.get('formlayout') === 'page' ? 'app-device-liveform panel liveform-inline' : '';
            const dialogAttributes = ['title', 'title.bind', 'iconclass', 'iconclass.bind', 'width'];
            attrs.delete('dependson');
            const liveFormTmpl = `<${tagName$J} wmForm role="${role}" ${directiveAttr} #${counter} ngNativeValidate [formGroup]="${counter}.ngform" [noValidate]="${counter}.validationtype !== 'html'"
                    class="${classProp}" [class]="${counter}.captionAlignClass" [autocomplete]="${counter}.autocomplete ? 'on' : 'off'" captionposition=${attrs.get('captionposition')}`;
            attrs.set('numberOfFields', `${numberOfFields}`);
            shared.set('counter', counter);
            updateFormDataSource(attrs);
            if (attrs.get('formlayout') === 'dialog') {
                dialogId = parentLiveTable ? parentLiveTable.get('liveform_dialog_id') : `liveform_dialog_id_${counter}`;
                attrs.set('dialogId', dialogId);
                const dialogAttrsMap = new Map();
                dialogAttributes.forEach((attr) => {
                    if (attrs.get(attr)) {
                        dialogAttrsMap.set(attr, attrs.get(attr));
                    }
                });
                attrs.set('width', '100%');
                tmpl = getAttrMarkup(attrs);
                return `<div data-identifier="liveform" init-widget class="app-liveform liveform-dialog" ${dependsOn} dialogid="${dialogId}">
                            <div wmDialog class="app-liveform-dialog" name="${dialogId}" role="form" ${getAttrMarkup(dialogAttrsMap)} modal="true">
                            <ng-template #dialogBody>
                            ${liveFormTmpl} ${tmpl}>`;
            }
            let mobileFormContentTmpl = '';
            let buttonTemplate = '';
            // Include mobile-navbar above the form when formlayout is set to page
            if (attrs.get('formlayout') === 'page') {
                const name = `device_liveform_header_${counter}`;
                const navbarAttrsMap = new Map();
                navbarAttrsMap.set('title', attrs.get('title'));
                navbarAttrsMap.set('backbtnclick.event', attrs.get('backbtnclick.event'));
                buttonTemplate = `<ng-template #buttonRef let-btn="btn">
                                            <button  wmButton name="{{btn.key}}" class="navbar-btn btn-primary btn-transparent" iconclass.bind="btn.iconclass" show.bind="btn.show"
                                                     (click)="${counter}.invokeActionEvent($event, btn.action)" type.bind="btn.type" hint.bind="btn.title" shortcutkey.bind="btn.shortcutkey" disabled.bind="btn.disabled"
                                                     tabindex.bind="btn.tabindex" [class.hidden]="btn.updateMode ? !${counter}.isUpdateMode : ${counter}.isUpdateMode"></button>
                                        </ng-template>`;
                mobileFormContentTmpl = `<header wmMobileNavbar name="${name}" ${getAttrMarkup(navbarAttrsMap)}>
                                            <ng-container *ngFor="let btn of ${counter}.buttonArray" [ngTemplateOutlet]="buttonRef" [ngTemplateOutletContext]="{btn:btn}">
                                            </ng-container>
                                        </header>
                                        <div class="form-elements panel-body" >`;
            }
            tmpl = getAttrMarkup(attrs);
            return `${liveFormTmpl} ${tmpl} ${dependsOn}>
                       ${buttonTemplate} ${mobileFormContentTmpl}`;
        },
        post: (attrs) => {
            if (attrs.get('formlayout') === 'dialog') {
                return '</form></ng-template></div></div>';
            }
            if (attrs.get('formlayout') === 'page') {
                return `</div></${tagName$J}>`;
            }
            return `</${tagName$J}>`;
        },
        provide: (attrs, shared) => {
            const provider = new Map();
            provider.set('form_reference', shared.get('counter'));
            return provider;
        }
    };
};
const ɵ2$2 = buildTask;
register('wm-form', buildTask);
register('wm-liveform', () => buildTask('wmLiveForm'));
register('wm-livefilter', () => buildTask('wmLiveFilter'));
var form_build = () => { };

var form_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': form_build,
    ɵ0: ɵ0$4,
    ɵ1: ɵ1$2,
    ɵ2: ɵ2$2
});

const tagName$K = 'div';
register('wm-calendar', () => {
    return {
        pre: attrs => `<${tagName$K} wmCalendar redrawable style="width:100%" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$K}>`
    };
});
var calendar_build = () => { };

var calendar_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': calendar_build
});

const tagName$L = 'ul';
register('wm-chips', () => {
    return {
        pre: attrs => `<${tagName$L} wmChips role="listbox" ${getAttrMarkup(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$L}>`
    };
});
var chips_build = () => { };

var chips_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': chips_build
});

const tagName$M = 'div';
register('wm-colorpicker', () => {
    return {
        pre: attrs => `<${tagName$M} wmColorPicker ${getAttrMarkup(attrs)} role="input" ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$M}>`
    };
});
var colorPicker_build = () => { };

var colorPicker_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': colorPicker_build
});

const tagName$N = 'div';
register('wm-currency', () => {
    return {
        pre: attrs => `<${tagName$N} wmCurrency ${getAttrMarkup(attrs)} role="input" ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$N}>`
    };
});
var currency_build = () => { };

var currency_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': currency_build
});

const tagName$O = 'div';
register('wm-buttongroup', () => {
    return {
        pre: attrs => `<${tagName$O} wmButtonGroup role="group" aria-labelledby="button group" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$O}>`
    };
});
var buttonGroup_build = () => { };

var buttonGroup_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': buttonGroup_build
});

const tagName$P = 'button';
register('wm-button', () => {
    return {
        pre: attrs => `<${tagName$P} wmButton ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$P}>`
    };
});
var button_build = () => { };

var button_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': button_build
});

const tagName$Q = 'div';
register('wm-checkbox', () => {
    return {
        pre: attrs => `<${tagName$Q} wmCheckbox role="input" ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$Q}>`
    };
});
var checkbox_build = () => { };

var checkbox_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': checkbox_build
});

const tagName$R = 'ul';
register('wm-checkboxset', () => {
    return {
        pre: attrs => `<${tagName$R} wmCheckboxset ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$R}>`
    };
});
var checkboxset_build = () => { };

var checkboxset_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': checkboxset_build
});

const tagName$S = 'div';
register('wm-composite', () => {
    return {
        pre: attrs => `<${tagName$S} wmComposite role="group" ${setChildAttrs(attrs)} ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$S}${clearChildAttrs()}>`
    };
});
var composite_build = () => { };

var composite_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': composite_build
});

const tagName$T = 'div';
register('wm-number', () => {
    return {
        pre: attrs => `<${tagName$T} wmNumber ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$T}>`
    };
});
var number_build = () => { };

var number_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': number_build
});

const tagName$U = 'ul';
register('wm-radioset', () => {
    return {
        pre: attrs => `<${tagName$U} wmRadioset role="radiogroup" ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$U}>`
    };
});
var radioset_build = () => { };

var radioset_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': radioset_build
});

const tagName$V = 'wm-select';
register('wm-select', () => {
    return {
        pre: attrs => `<${tagName$V} ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)}  ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$V}>`
    };
});
var select_build = () => { };

var select_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': select_build
});

const tagName$W = 'div';
register('wm-switch', () => {
    return {
        pre: attrs => `<${tagName$W} wmSwitch ${getFormMarkupAttr(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$W}>`
    };
});
var switch_build = () => { };

var switch_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': switch_build
});

const tagName$X = 'wm-input';
register('wm-text', () => {
    return {
        pre: attrs => `<${tagName$X} ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$X}>`
    };
});
var text_build = () => { };

var text_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': text_build
});

const tagName$Y = 'wm-textarea';
register('wm-textarea', () => {
    return {
        pre: attrs => `<${tagName$Y} ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$Y}>`
    };
});
var textarea_build = () => { };

var textarea_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': textarea_build
});

const tagName$Z = 'div';
register('wm-datetime', () => {
    return {
        pre: attrs => `<${tagName$Z} wmDateTime ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$Z}>`
    };
});
var dateTime_build = () => { };

var dateTime_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': dateTime_build
});

const tagName$_ = 'div';
register('wm-date', () => {
    return {
        pre: attrs => `<${tagName$_} wmDate ${getFormMarkupAttr(attrs)} ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$_}>`
    };
});
var date_build = () => { };

var date_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': date_build
});

const tagName$$ = 'div';
register('wm-time', () => {
    return {
        pre: attrs => `<${tagName$$} wmTime ${getFormMarkupAttr(attrs)} role="input" ${getChildAttrs(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$$}>`
    };
});
var time_build = () => { };

var time_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': time_build
});

const tagName$10 = 'div';
register('wm-fileupload', () => {
    return {
        pre: attrs => {
            if (attrs.get('select.event')) {
                const onSelectBinding = getDataSource(attrs.get('select.event'));
                attrs.set('datasource.bind', onSelectBinding);
            }
            return `<${tagName$10} wmFileUpload ${getAttrMarkup(attrs)} role="input">`;
        },
        post: () => `</${tagName$10}>`
    };
});
var fileUpload_build = () => { };

var fileUpload_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': fileUpload_build
});

const tagName$11 = 'div';
register('wm-rating', () => {
    return {
        pre: attrs => `<${tagName$11} wmRating ${getFormMarkupAttr(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$11}>`
    };
});
var rating_build = () => { };

var rating_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': rating_build
});

const tagName$12 = 'div';
register('wm-slider', () => {
    return {
        pre: attrs => `<${tagName$12} wmSlider ${getAttrMarkup(attrs)} ${getNgModelAttr(attrs)}>`,
        post: () => `</${tagName$12}>`
    };
});
var slider_build = () => { };

var slider_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': slider_build
});

const wmlistTag = 'wm-list';
const tagName$13 = 'div';
const dataSetKey$1 = 'dataset';
function copyAttribute(from, fromAttrName, to, toAttrName) {
    const fromAttr = from.attrs.find(a => a.name === fromAttrName);
    if (fromAttr) {
        to.attrs.push(new Attribute(toAttrName, fromAttr.value, fromAttr.sourceSpan, fromAttr.valueSpan));
    }
}
register('wm-media-list', () => {
    return {
        template: (node) => {
            let bindDataset;
            const attrObj = node.attrs.find(attr => attr.name === dataSetKey$1), 
            /**
             *  Replacing binded property value with item
             * @param children
             */
            replaceBind = (children = []) => {
                children.forEach(childNode => {
                    if (childNode.name) {
                        // return if the child Element is of wm-list .
                        if (childNode.name !== wmlistTag) {
                            childNode.attrs.forEach((attr) => {
                                if (attr.value.startsWith(`bind:${bindDataset}.data[$i]`)) {
                                    attr.value = attr.value.replace(`${bindDataset}.data[$i]`, 'item');
                                }
                                else if (attr.value.startsWith(`bind:${bindDataset}`)) {
                                    attr.value = attr.value.replace(bindDataset, 'item');
                                }
                            });
                            replaceBind(childNode.children);
                        }
                    }
                });
            };
            if (attrObj && attrObj.value.startsWith('bind:')) {
                bindDataset = attrObj.value.replace('bind:', '');
            }
            if (bindDataset) {
                replaceBind(node.children);
            }
            const template = node.children
                .find(e => e instanceof Element && e.name === 'wm-media-template');
            if (template != null) {
                copyAttribute(template, 'width', node, 'thumbnailwidth');
                copyAttribute(template, 'height', node, 'thumbnailheight');
            }
        },
        pre: attrs => `<${tagName$13} wmMediaList ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$13}>`
    };
});
var mediaList_build = () => { };

var mediaList_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': mediaList_build
});

const tagName$14 = 'ng-template';
register('wm-media-template', () => {
    return {
        pre: () => `<${tagName$14} #mediaListTemplate let-item="item" let-index="index">`,
        post: () => `</${tagName$14}>`
    };
});
var mediaListItem_build = () => { };

var mediaListItem_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': mediaListItem_build
});

const wmListTag = 'wm-list';
const listTagName = 'div';
const dataSetKey$2 = 'dataset';
register(wmListTag, () => {
    return {
        requires: ['wm-form', 'wm-liveform'],
        template: (node) => {
            const datasetAttr = node.attrs.find(attr => attr.name === dataSetKey$2);
            const widgetNameAttr = node.attrs.find(attr => attr.name === 'name');
            if (!datasetAttr) {
                return;
            }
            const boundExpr = getBoundToExpr(datasetAttr.value);
            if (!boundExpr) {
                return;
            }
            updateTemplateAttrs(node, boundExpr, widgetNameAttr.value, 'itemRef.');
        },
        pre: (attrs, shared, parentForm, parentLiveForm) => {
            const parent = parentForm || parentLiveForm;
            shared.set('form_reference', parent && parent.get('form_reference'));
            return `<${listTagName} wmList wmLiveActions ${getAttrMarkup(attrs)}>`;
        },
        post: () => `</${listTagName}>`,
        provide: (attrs, shared) => {
            const provider = new Map();
            provider.set('parent_form_reference', shared.get('form_reference'));
            return provider;
        }
    };
});
register('wm-listtemplate', () => {
    return {
        pre: () => `<ng-template #listTemplate let-item="item" let-$index="$index" let-itemRef="itemRef" let-$first="$first" let-$last="$last"  let-currentItemWidgets="currentItemWidgets" >`,
        post: () => `</ng-template>`
    };
});
function copyAttribute$1(from, fromAttrName, to, toAttrName) {
    const fromAttr = from.attrs.find(a => a.name === fromAttrName);
    if (fromAttr) {
        to.attrs.push(new Attribute(toAttrName, fromAttr.value, fromAttr.sourceSpan, fromAttr.valueSpan));
    }
}
register('wm-list-action-template', () => {
    return {
        template: (node) => {
            const position = node.attrs.find(attr => attr.name === 'position').value;
            const btns = node.children
                .filter(e => e instanceof Element && e.name === 'wm-button');
            // add swipe-position on button nodes to identify whether buttons are from left or right action templates
            btns.forEach((btnNode) => {
                copyAttribute$1(node, 'position', btnNode, 'swipe-position');
            });
        },
        pre: (attrs, el) => {
            if (attrs.get('position') === 'left') {
                return `<ng-template #listLeftActionTemplate>
                            <li class="app-list-item-action-panel app-list-item-left-action-panel actionMenu" ${getAttrMarkup(attrs)}>`;
            }
            if (attrs.get('position') === 'right') {
                return `<ng-template #listRightActionTemplate>
                            <li class="app-list-item-action-panel app-list-item-right-action-panel actionMenu" ${getAttrMarkup(attrs)}>`;
            }
        },
        post: () => `</li></ng-template>`
    };
});
var list_build = () => { };

var list_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': list_build
});

const tagName$15 = 'div';
const idGen$4 = new IDGenerator('liveform_dialog_id_');
register('wm-livetable', () => {
    return {
        pre: (attrs, shared) => {
            const counter = idGen$4.nextUid();
            shared.set('counter', counter);
            return `<${tagName$15} wmLiveTable role="table" ${getAttrMarkup(attrs)} dialogid="${counter}">`;
        },
        post: () => `</${tagName$15}>`,
        provide: (attrs, shared) => {
            const provider = new Map();
            provider.set('liveform_dialog_id', shared.get('counter'));
            return provider;
        }
    };
});
var liveTable_build = () => { };

var liveTable_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': liveTable_build
});

const tagName$16 = 'p';
register('wm-message', () => {
    return {
        pre: attrs => `<${tagName$16} wmMessage aria-label="Notification Alerts" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$16}>`
    };
});
var message_build = () => { };

var message_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': message_build
});

const tagName$17 = 'ol';
register('wm-breadcrumb', () => {
    return {
        pre: attrs => `<${tagName$17} wmBreadcrumb ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$17}>`
    };
});
var breadcrumb_build = () => { };

var breadcrumb_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': breadcrumb_build
});

const tagName$18 = 'div';
register('wm-menu', () => {
    return {
        pre: attrs => `<${tagName$18} wmMenu dropdown ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$18}>`
    };
});
var menu_build = () => { };

var menu_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': menu_build
});

const tagName$19 = 'li';
register('wm-nav-item', () => {
    return {
        pre: attrs => `<${tagName$19} wmNavItem role="presentation" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$19}>`
    };
});
var navItem_build = () => { };

var navItem_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': navItem_build
});

const tagName$1a = 'ul';
register('wm-nav', () => {
    return {
        pre: attrs => `<${tagName$1a} wmNav data-element-type="wmNav" data-role="page-header" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1a}>`
    };
});
var nav_build = () => { };

var nav_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': nav_build
});

const tagName$1b = 'nav';
register('wm-navbar', () => {
    return {
        pre: attrs => `<${tagName$1b} wmNavbar data-element-type="wmNavbar" role="navigation" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1b}>`
    };
});
var navbar_build = () => { };

var navbar_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': navbar_build
});

const tagName$1c = 'wm-popover';
register('wm-popover', () => {
    return {
        requires: ['wm-table'],
        pre: (attrs, shared, table) => {
            const contentSource = attrs.get('contentsource');
            let popoverTemplate;
            if (contentSource !== 'inline') {
                const content = attrs.get('content');
                const bindContent = attrs.get('content.bind');
                let contentMarkup = '';
                if (content) {
                    contentMarkup = `content="${content}"`;
                }
                else if (bindContent) {
                    contentMarkup = `content.bind="${bindContent}"`;
                }
                popoverTemplate = `<div wmContainer #partial partialContainer ${contentMarkup}>`;
                shared.set('hasPopoverContent', true);
            }
            let markup = `<${tagName$1c} wmPopover ${getAttrMarkup(attrs)}>`;
            const contextAttrs = table ? `let-row="row"` : ``;
            markup += `<ng-template ${contextAttrs}><button class="popover-start"></button>`;
            // todo keyboard navigation - tab
            if (popoverTemplate) {
                markup += `${popoverTemplate ? popoverTemplate : ''}`;
            }
            return markup;
        },
        post: (attrs, shared) => {
            let markup = '';
            if (shared.get('hasPopoverContent')) {
                markup += `</div>`;
            }
            return `${markup}<button class="popover-end"></button></ng-template></${tagName$1c}>`;
        }
    };
});
var popover_build = () => { };

var popover_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': popover_build
});

const tagName$1d = 'div';
const findChild = (node, childName) => {
    const child = node && node.children.find(e => (e instanceof Element && e.name === childName));
    return child;
};
const ɵ0$5 = findChild;
const createElement = name => {
    return new Element(name, [], [], noSpan, noSpan, noSpan);
};
const ɵ1$3 = createElement;
const addAtrribute = (node, name, value) => {
    const attr = new Attribute(name, value, noSpan, noSpan);
    node.attrs.push(attr);
};
const ɵ2$3 = addAtrribute;
const getElementNode = (name, node) => {
    let elementNode;
    if (!node) {
        return;
    }
    _.forEach(node.children, (child) => {
        if (child instanceof Element) {
            if (child.name === name) {
                elementNode = child;
            }
            else if (child.children) {
                elementNode = getElementNode(name, child);
            }
        }
        return !elementNode;
    });
    return elementNode;
};
const ɵ3$1 = getElementNode;
const noSpan = {};
register('wm-page', () => {
    return {
        template: (node) => {
            let pageContentNode = findChild(findChild(node, 'wm-content'), 'wm-page-content');
            if (!pageContentNode) {
                pageContentNode = getElementNode('wm-page-content', getElementNode('wm-content', node));
            }
            if (pageContentNode) {
                const conditionalNode = createElement('ng-container');
                addAtrribute(conditionalNode, '*ngIf', 'compilePageContent');
                conditionalNode.children = conditionalNode.children.concat(pageContentNode.children);
                conditionalNode.children.push(new Text('{{onPageContentReady()}}', null));
                pageContentNode.children = [conditionalNode];
                if (isMobileApp()) {
                    const loader = createElement('div');
                    addAtrribute(loader, 'wmPageContentLoader', '');
                    addAtrribute(loader, '*ngIf', '!showPageContent');
                    pageContentNode.children.push(loader);
                }
            }
        },
        pre: attrs => `<${tagName$1d} wmPage data-role="pageContainer" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1d}>`
    };
});
var page_build = () => { };

var page_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': page_build,
    ɵ0: ɵ0$5,
    ɵ1: ɵ1$3,
    ɵ2: ɵ2$3,
    ɵ3: ɵ3$1
});

const tagName$1e = 'nav';
register('wm-pagination', () => {
    return {
        pre: attrs => `<${tagName$1e} wmPagination data-identifier="pagination" aria-label="Page navigation" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1e}>`
    };
});
var pagination_build = () => { };

var pagination_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': pagination_build
});

const tagName$1f = 'main';
register('wm-content', () => {
    return {
        pre: attrs => `<${tagName$1f} wmContent data-role="page-content" role="main" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1f}>`
    };
});
var content_build = () => { };

var content_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': content_build
});

const tagName$1g = 'footer';
register('wm-footer', () => {
    return {
        pre: attrs => `<${tagName$1g} wmFooter partialContainer data-role="page-footer" role="contentinfo" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1g}>`
    };
});
var footer_build = () => { };

var footer_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': footer_build
});

const tagName$1h = 'header';
register('wm-header', () => {
    return {
        pre: attrs => `<${tagName$1h} wmHeader partialContainer data-role="page-header" role="banner" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1h}>`
    };
});
var header_build = () => { };

var header_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': header_build
});

const tagName$1i = 'aside';
register('wm-left-panel', () => {
    return {
        pre: attrs => `<${tagName$1i} wmLeftPanel partialContainer data-role="page-left-panel" role="complementary" wmSmoothscroll="${attrs.get('smoothscroll') || 'true'}" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1i}>`
    };
});
var leftPanel_build = () => { };

var leftPanel_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': leftPanel_build
});

const tagName$1j = 'header';
register('wm-mobile-navbar', () => {
    return {
        pre: attrs => `<${tagName$1j} wmMobileNavbar ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1j}>`
    };
});
var mobileNavbar_build = () => { };

var mobileNavbar_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': mobileNavbar_build
});

const tagName$1k = 'aside';
register('wm-right-panel', () => {
    return {
        pre: attrs => `<${tagName$1k} wmRightPanel partialContainer data-role="page-right-panel" role="complementary" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1k}>`
    };
});
var rightPanel_build = () => { };

var rightPanel_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': rightPanel_build
});

const tagName$1l = 'div';
register('wm-page-content', () => {
    return {
        pre: attrs => `<${tagName$1l} wmPageContent wmSmoothscroll="${attrs.get('smoothscroll') || 'true'}" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1l}>`
    };
});
var pageContent_build = () => { };

var pageContent_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': pageContent_build
});

const tagName$1m = 'div';
register('wm-mobile-tabbar', () => {
    return {
        pre: attrs => `<${tagName$1m} wmMobileTabbar ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1m}>`
    };
});
var tabBar_build = () => { };

var tabBar_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tabBar_build
});

const tagName$1n = 'section';
register('wm-top-nav', () => {
    return {
        pre: attrs => `<${tagName$1n} wmTopNav partialContainer data-role="page-topnav" role="region" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1n}>`
    };
});
var topNav_build = () => { };

var topNav_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': topNav_build
});

const tagName$1o = 'section';
const noSpan$1 = {};
const createElement$1 = name => {
    return new Element(name, [], [], noSpan$1, noSpan$1, noSpan$1);
};
const ɵ0$6 = createElement$1;
const addAtrribute$1 = (node, name, value) => {
    const attr = new Attribute(name, value, noSpan$1, noSpan$1);
    node.attrs.push(attr);
};
const ɵ1$4 = addAtrribute$1;
register('wm-partial', () => {
    return {
        template: (node) => {
            const conditionalNode = createElement$1('ng-container');
            addAtrribute$1(conditionalNode, '*ngIf', 'compileContent');
            conditionalNode.children = conditionalNode.children.concat(node.children);
            node.children.length = 0;
            node.children.push(conditionalNode);
        },
        pre: attrs => `<${tagName$1o} wmPartial data-role="partial" role="region" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1o}>`
    };
});
var partial_build = () => { };

var partial_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': partial_build,
    ɵ0: ɵ0$6,
    ɵ1: ɵ1$4
});

const tagName$1p = 'div';
register('wm-param', () => {
    return {
        pre: attrs => `<${tagName$1p} wmParam hidden ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1p}>`
    };
});
var partialParam_build = () => { };

var partialParam_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': partialParam_build
});

const tagName$1q = 'section';
register('wm-prefab', () => {
    return {
        pre: attrs => `<${tagName$1q} wmPrefab data-role="perfab" ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1q}>`
    };
});
var prefab_build = () => { };

var prefab_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': prefab_build
});

const noSpan$2 = {};
const createElement$2 = name => {
    return new Element(name, [], [], noSpan$2, noSpan$2, noSpan$2);
};
const ɵ0$7 = createElement$2;
const addAtrribute$2 = (node, name, value) => {
    const attr = new Attribute(name, value, noSpan$2, noSpan$2);
    node.attrs.push(attr);
};
const ɵ1$5 = addAtrribute$2;
const tagName$1r = 'div';
register('wm-prefab-container', () => {
    return {
        template: (node) => {
            const conditionalNode = createElement$2('ng-container');
            addAtrribute$2(conditionalNode, '*ngIf', 'compileContent');
            conditionalNode.children = conditionalNode.children.concat(node.children);
            node.children.length = 0;
            node.children.push(conditionalNode);
        },
        pre: attrs => `<${tagName$1r} wmPrefabContainer ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1r}>`
    };
});
var prefabContainer_build = () => { };

var prefabContainer_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': prefabContainer_build,
    ɵ0: ɵ0$7,
    ɵ1: ɵ1$5
});

const tagName$1s = 'div';
register('wm-table-action', () => {
    return {
        pre: attrs => `<${tagName$1s} name="${attrs.get('name') || attrs.get('key')}" wmTableAction ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1s}>`
    };
});
var tableAction_build = () => { };

var tableAction_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tableAction_build
});

const tagName$1t = 'div';
register('wm-table-column-group', () => {
    return {
        pre: attrs => `<${tagName$1t} wmTableColumnGroup ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1t}>`
    };
});
var tableColumnGroup_build = () => { };

var tableColumnGroup_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tableColumnGroup_build
});

const tagName$1u = 'div';
const idGen$5 = new IDGenerator('data_table_form_');
const formWidgets$1 = new Set([
    'wm-text',
    'wm-textarea',
    'wm-checkbox',
    'wm-slider',
    'wm-currency',
    'wm-switch',
    'wm-select',
    'wm-checkboxset',
    'wm-radioset',
    'wm-date',
    'wm-time',
    'wm-timestamp',
    'wm-rating',
    'wm-datetime',
    'wm-search',
    'wm-chips',
    'wm-colorpicker'
]);
// Add ngModelOptions standalone true as inner custom form widgets will be not part of table ngform
const addNgModelStandalone = (children = []) => {
    let formControlAttr = [];
    children.forEach(childNode => {
        if (formWidgets$1.has(childNode.name)) {
            // angular build is failing when formControlName is undefined for checkbox and mgModelOptions attr is present
            if (childNode.name === 'wm-checkbox') {
                formControlAttr = childNode.attrs.filter(function (attr) {
                    return attr.name === 'formControlName';
                });
                if (formControlAttr.length) {
                    return;
                }
            }
            childNode.attrs.push(new Attribute('[ngModelOptions]', '{standalone: true}', 1, 1));
        }
        addNgModelStandalone(childNode.children);
    });
};
const ɵ0$8 = addNgModelStandalone;
// get the filter template (widget and filter menu) to be displayed in filter row
const getFilterTemplate = (attrs, pCounter) => {
    const widget = attrs.get('filterwidget') || getDataTableFilterWidget(attrs.get('type') || DataType.STRING);
    const fieldName = attrs.get('binding');
    const type = attrs.get('type') || 'string';
    let datasourceBinding, submitEventBinding;
    const datasetAttr = attrs.get('filterdataset.bind');
    // when multicolumn is selected and filterwidget as autocomplete is assigned to dataset.
    if (attrs.get('filterwidget') === 'autocomplete') {
        if (datasetAttr) {
            const binddatasource = getDataSource(datasetAttr);
            datasourceBinding = `dataset.bind="${datasetAttr}" datasource.bind="${binddatasource}"`;
        }
        submitEventBinding = `submit.event="changeFn('${fieldName}')"`;
    }
    const innerTmpl = `#filterWidget formControlName="${fieldName + '_filter'}" ${datasourceBinding} ${submitEventBinding} change.event="changeFn('${fieldName}')"
                        disabled.bind="isDisabled('${fieldName}')"`;
    const options = { inputType: 'filterinputtype' };
    const widgetTmpl = `${getFormWidgetTemplate(widget, innerTmpl, attrs, options)}`;
    return `<ng-template #filterTmpl let-changeFn="changeFn" let-isDisabled="isDisabled">
    <div class="input-group ${widget}" data-col-identifier="${fieldName}">
        ${widgetTmpl}
        <span class="input-group-addon filter-clear-icon" *ngIf="${pCounter}.showClearIcon('${fieldName}')">
            <button class="btn-transparent btn app-button" aria-label="Clear button" type="button" (click)="${pCounter}.clearRowFilter('${fieldName}')">
                <i class="app-icon wi wi-clear" aria-hidden="true"></i>
            </button>
         </span>
        <span class="input-group-addon" dropdown container="body" (onShown)="${pCounter}.adjustContainer('${fieldName}');">
            <button class="btn-transparent btn app-button" type="button" dropdownToggle><i class="app-icon wi wi-filter-list"></i></button>
            <ul class="matchmode-dropdown dropdown-menu" *dropdownMenu>
                   <li *ngFor="let matchMode of ${pCounter}.matchModeTypesMap['${type}']"
                        [ngClass]="{active: matchMode === (${pCounter}.rowFilter['${fieldName}'].matchMode || ${pCounter}.matchModeTypesMap['${type}'][0])}">
                        <a href="javascript:void(0);" (click)="${pCounter}.onFilterConditionSelect('${fieldName}', matchMode)" [innerText]="${pCounter}.matchModeMsgs[matchMode]"></a>
                    </li>
             </ul>
        </span>
    </div></ng-template>`;
};
const ɵ1$6 = getFilterTemplate;
const getEventsTmpl = attrs => {
    let tmpl = '';
    attrs.forEach((val, key) => {
        if (key.endsWith('.event')) {
            tmpl += `${key}="${val}" `;
        }
    });
    return tmpl;
};
const ɵ2$4 = getEventsTmpl;
// Generate inline edit properties template. Properties requiring row instance are generated here.
const getInlineEditRowPropsTmpl = attrs => {
    const propAttrs = new Map();
    const props = ['disabled', 'disabled.bind'];
    props.forEach(prop => {
        if (attrs.get(prop)) {
            propAttrs.set(prop, attrs.get(prop));
            attrs.delete(prop);
        }
    });
    return getAttrMarkup(propAttrs);
};
const ɵ3$2 = getInlineEditRowPropsTmpl;
// get the inline widget template
const getInlineEditWidgetTmpl = (attrs, isNewRow, errorstyle, pCounter) => {
    const options = {};
    const fieldName = attrs.get('binding');
    const widget = attrs.get('edit-widget-type') || getEditModeWidget({
        'type': attrs.get('type'),
        'related-entity-name': attrs.get('related-entity-name'),
        'primary-key': attrs.get('primary-key')
    });
    let widgetRef = '';
    let formControl = '';
    let wmFormWidget = '';
    if (widget === FormWidgetType.UPLOAD) {
        options.uploadProps = {
            formName: idGen$5.nextUid(),
            name: fieldName
        };
        options.counter = pCounter;
    }
    else {
        widgetRef = isNewRow ? '#inlineWidgetNew' : '#inlineWidget';
        formControl = isNewRow ? `formControlName="${fieldName}_new"` : `formControlName="${fieldName}"`;
        wmFormWidget = 'wmFormWidget';
    }
    options.inputType = 'editinputtype';
    const tmplRef = isNewRow ? '#inlineWidgetTmplNew' : '#inlineWidgetTmpl';
    const eventsTmpl = widget === FormWidgetType.UPLOAD ? '' : getEventsTmpl(attrs);
    const rowPropsTl = getInlineEditRowPropsTmpl(attrs);
    const innerTmpl = `${widgetRef} ${wmFormWidget} key="${fieldName}" data-field-name="${fieldName}" ${formControl} ${eventsTmpl} ${rowPropsTl}`;
    const widgetTmpl = getFormWidgetTemplate(widget, innerTmpl, attrs, options);
    const pendingSpinnerStatus = isNewRow ? 'getPendingSpinnerStatusNew' : 'getPendingSpinnerStatus';
    const validationErrorTmpl = getValiationErrorTemplate(errorstyle);
    /* This element with error message is used to run Automation checks if any error message is present */
    const errorMsgElementForAutomation = '<span class="sr-only" *ngIf="getValidationMessage()">{{getValidationMessage()}}</span>';
    return `<ng-template ${tmplRef} let-row="row" let-getControl="getControl" let-getValidationMessage="getValidationMessage" let-${pendingSpinnerStatus}="${pendingSpinnerStatus}">
                <div data-col-identifier="${fieldName}" >
                     ${widgetTmpl}
                     ${validationErrorTmpl}
                     <div class="overlay" *ngIf="${pendingSpinnerStatus}()"><span aria-hidden="true" class="form-field-spinner fa fa-circle-o-notch fa-spin form-control-feedback"></span></div>
                     ${errorMsgElementForAutomation}
                 </div>
            </ng-template>`;
};
const ɵ4$1 = getInlineEditWidgetTmpl;
const getValiationErrorTemplate = (errorTmplType) => {
    let validationErrorTmpl = '';
    switch (errorTmplType) {
        case 'bottom':
            validationErrorTmpl = `<span class="help-block text-danger" *ngIf="getValidationMessage() && getControl() && getControl().invalid && getControl().touched">{{getValidationMessage()}}</span>`;
            break;
        case 'hover':
        default:
            validationErrorTmpl = `<span placement="top" container="body" tooltip="{{getValidationMessage()}}" class="text-danger wi wi-error"
                                        *ngIf="getValidationMessage() && getControl() && getControl().invalid && getControl().touched">
                                    </span>`;
    }
    return validationErrorTmpl;
};
const ɵ5$1 = getValiationErrorTemplate;
const getFormatExpression = (attrs) => {
    const columnValue = `row.getProperty('${attrs.get('binding')}')`;
    let formatPattern = attrs.get('custompipeformat') || attrs.get('formatpattern');
    let colExpression = '';
    // For date time data types, if format pattern is not applied, Apply default toDate format
    if (isDateTimeType(attrs.get('type')) && (!formatPattern || formatPattern === 'None')) {
        attrs.set('formatpattern', 'toDate');
        attrs.delete('datepattern');
        formatPattern = 'toDate';
    }
    switch (formatPattern) {
        case 'toDate':
            colExpression = `{{${columnValue} | toDate: colDef.datepattern}}`;
            break;
        case 'toCurrency':
            if (attrs.get('currencypattern')) {
                colExpression = `{{${columnValue} | toCurrency: '${attrs.get('currencypattern')}`;
                if (attrs.get('fractionsize')) {
                    colExpression += `': ${attrs.get('fractionsize')}}}`;
                }
                else {
                    colExpression += `'}}`;
                }
            }
            break;
        case 'numberToString':
            if (attrs.get('fractionsize')) {
                colExpression = `{{${columnValue} | numberToString: '${attrs.get('fractionsize')}'}}`;
            }
            break;
        case 'stringToNumber':
            colExpression = `{{${columnValue} | stringToNumber}}`;
            break;
        case 'timeFromNow':
            colExpression = `{{${columnValue} | timeFromNow}}`;
            break;
        case 'prefix':
            if (attrs.get('prefix')) {
                colExpression = `{{${columnValue} | prefix: '${attrs.get('prefix')}'}}`;
            }
            break;
        case 'suffix':
            if (attrs.get('suffix')) {
                colExpression = `{{${columnValue} | suffix: '${attrs.get('suffix')}'}}`;
            }
            break;
        default:
            if (checkIsCustomPipeExpression(formatPattern)) {
                colExpression = `{{${columnValue} |  ${formatPattern} : ${`row`}}}`;
            }
            break;
    }
    return colExpression;
};
const ɵ6 = getFormatExpression;
register('wm-table-column', () => {
    return {
        requires: ['wm-table'],
        template: (node, shared) => {
            if (node.children.length) {
                // If node has children, but an empty text node dont generate custom expression
                if (node.children.length === 1 && node.children[0] instanceof Text && node.children[0].value.trim().length === 0) {
                    return;
                }
                shared.set('customExpression', true);
                addNgModelStandalone(node.children);
            }
        },
        pre: (attrs, shared, parentTable) => {
            let rowFilterTmpl = '';
            let inlineEditTmpl = '';
            let inlineNewEditTmpl = '';
            let parentForm = '';
            if (parentTable) {
                const pCounter = parentTable.get('table_reference');
                rowFilterTmpl = (parentTable.get('filtermode') === 'multicolumn' && attrs.get('searchable') !== 'false') ? getFilterTemplate(attrs, pCounter) : '';
                const editMode = parentTable.get('editmode');
                const isInlineEdit = (editMode !== EDIT_MODE.DIALOG && editMode !== EDIT_MODE.FORM && attrs.get('readonly') !== 'true');
                inlineEditTmpl = isInlineEdit ? getInlineEditWidgetTmpl(attrs, false, parentTable.get('errorstyle'), pCounter) : '';
                inlineNewEditTmpl = isInlineEdit && editMode === EDIT_MODE.QUICK_EDIT && parentTable.get('shownewrow') !== 'false' ? getInlineEditWidgetTmpl(attrs, true, attrs.get('errorstyle')) : '';
                parentForm = ` [formGroup]="${pCounter}.ngform" `;
            }
            const formatPattern = attrs.get('formatpattern');
            const customExpr = `<ng-template #customExprTmpl let-row="row" let-colDef="colDef" let-editRow="editRow" let-deleteRow="deleteRow" let-addNewRow="addNewRow">`;
            let customExprTmpl = '';
            let formatExprTmpl = '';
            if (shared.get('customExpression')) {
                attrs.set('customExpression', 'true');
                customExprTmpl = `${customExpr}<div data-col-identifier="${attrs.get('binding')}">`;
            }
            else if ((formatPattern && formatPattern !== 'None') || isDateTimeType(attrs.get('type'))) {
                formatExprTmpl = getFormatExpression(attrs);
                if (formatExprTmpl) {
                    shared.set('customExpression', true);
                    attrs.set('customExpression', 'true');
                    customExprTmpl = `${customExpr}<div data-col-identifier="${attrs.get('binding')}" title="${formatExprTmpl}">${formatExprTmpl}`;
                }
            }
            return `<${tagName$1u} wmTableColumn ${getAttrMarkup(attrs)} ${parentForm}>
                    ${rowFilterTmpl}
                    ${inlineEditTmpl}
                    ${inlineNewEditTmpl}
                    ${customExprTmpl}`;
        },
        post: (attrs, shared) => {
            let customExprTmpl = '';
            if (shared.get('customExpression')) {
                customExprTmpl = `</div></ng-template>`;
            }
            return `${customExprTmpl}</${tagName$1u}>`;
        },
        imports: (attrs) => {
            const editWidgetType = attrs.get('edit-widget-type');
            const filterWidgetType = attrs.get('filterwidget') || getDataTableFilterWidget(attrs.get('type') || DataType.STRING);
            return [getRequiredFormWidget(editWidgetType), getRequiredFormWidget(filterWidgetType), 'wm-table'];
        }
    };
});
var tableColumn_build = () => { };

var tableColumn_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tableColumn_build,
    ɵ0: ɵ0$8,
    ɵ1: ɵ1$6,
    ɵ2: ɵ2$4,
    ɵ3: ɵ3$2,
    ɵ4: ɵ4$1,
    ɵ5: ɵ5$1,
    ɵ6: ɵ6
});

const tagName$1v = 'div';
const getRowExpansionActionTmpl = (attrs) => {
    const tag = attrs.get('widget-type') === 'anchor' ? 'a' : 'button';
    const directive = attrs.get('widget-type') === 'anchor' ? 'wmAnchor' : 'wmButton';
    return `<ng-template #rowExpansionActionTmpl let-row="row">
               <${tag} ${directive}
                    ${getRowActionAttrs(attrs)}
                    class="${attrs.get('class')} row-expansion-button"
                    iconclass="${attrs.get('collapseicon')}"
                    type="button"></${tag}>
            </ng-template>`;
};
const ɵ0$9 = getRowExpansionActionTmpl;
register('wm-table-row', () => {
    return {
        pre: (attrs) => {
            return `<${tagName$1v} wmTableRow ${getAttrMarkup(attrs)}>
                    ${getRowExpansionActionTmpl(attrs)}
                    <ng-template #rowExpansionTmpl let-row="row" let-rowDef="rowDef" let-containerLoad="containerLoad">
                        <div wmContainer partialContainer content.bind="rowDef.content" load.event="containerLoad(widget)"
                            [ngStyle]="{'height': rowDef.height, 'overflow-y': 'auto'}">
                         <div *ngFor="let param of rowDef.partialParams | keyvalue" wmParam hidden
                            [name]="param.key" [value]="param.value"></div>`;
        },
        post: () => `</div></ng-template></${tagName$1v}>`
    };
});
var tableRow_build = () => { };

var tableRow_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tableRow_build,
    ɵ0: ɵ0$9
});

const tagName$1w = 'div';
const getSaveCancelTemplate = () => {
    return `<button type="button" aria-label="Save edit icon" class="save row-action-button btn app-button btn-transparent save-edit-row-button hidden" title="Save">
                <i class="wi wi-done" aria-hidden="true"></i>
            </button>
            <button type="button" aria-label="Cancel edit icon" class="cancel row-action-button btn app-button btn-transparent cancel-edit-row-button hidden" title="Cancel">
                <i class="wi wi-cancel" aria-hidden="true"></i>
            </button>`;
};
const ɵ0$a = getSaveCancelTemplate;
// get the inline widget template
const getRowActionTmpl = (attrs) => {
    const action = attrs.get('action');
    const actionTmpl = action ? ` click.event.delayed="${action}" ` : '';
    const saveCancelTmpl = action && action.includes('editRow(') ? getSaveCancelTemplate() : '';
    const btnClass = action ? (action.includes('editRow(') ? 'edit edit-row-button' :
        (action.includes('deleteRow(') ? 'delete delete-row-button' : '')) : '';
    const tabIndex = attrs.get('tabindex') ? `tabindex="${attrs.get('tabindex')}"` : '';
    const tag = attrs.get('widget-type') === 'anchor' ? 'a' : 'button';
    const directive = attrs.get('widget-type') === 'anchor' ? 'wmAnchor' : 'wmButton';
    return `<ng-template #rowActionTmpl let-row="row">
               <${tag} ${directive} data-action-key="${attrs.get('key')}"
                    ${getRowActionAttrs(attrs)}
                    class="row-action row-action-button ${attrs.get('class')} ${btnClass}"
                    iconclass="${attrs.get('iconclass')}"
                    ${actionTmpl}
                    ${tabIndex}
                    type="button"></${tag}>
                ${saveCancelTmpl}
            </ng-template>`;
};
const ɵ1$7 = getRowActionTmpl;
register('wm-table-row-action', () => {
    return {
        pre: attrs => `<${tagName$1w} wmTableRowAction ${getAttrMarkup(attrs)}>
                        ${getRowActionTmpl(attrs)}`,
        post: () => `</${tagName$1w}>`
    };
});
var tableRowAction_build = () => { };

var tableRowAction_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': tableRowAction_build,
    ɵ0: ɵ0$a,
    ɵ1: ɵ1$7
});

const tagName$1x = 'div';
const dataSetKey$3 = 'dataset';
const idGen$6 = new IDGenerator('table_');
let columnIndex = 0;
/**
 * This method assigns index to the table-column and column-groups in order to the maintain the columns in the same order
 * as specified in the configuration.
 * Scenario: ordering the columns when accessrole is specified on column.
 *
 * headerIndex:    ID  |      Name        |   Street  |  ==> top level header (ID has index 0, Name 1, Street 2)
 * index:         ---- |  fname | lname    |   ---    |  ==> sub level header ( ID 0, fname 1, lname 2, Street 3)
 * fname will have headerIndex as 1 and colIndex as 1. lname will have headerIndex as 1 and columnIndex as 2.
 * If suppose we have a group in the sub-level-header, then col index will be treated as the header index.
 *
 */
function assignColumnIndex(node, parentIndex) {
    let headerIndex = 0;
    node.forEach((childNode) => {
        const nodeName = childNode.name;
        const newheaderIndex = parentIndex !== undefined ? parentIndex : headerIndex;
        if (nodeName === 'wm-table-column' || nodeName === 'wm-table-column-group') {
            childNode.attrs.push(new Attribute('index', '' + columnIndex, 1, 1));
            childNode.attrs.push(new Attribute('headerIndex', '' + newheaderIndex, 1, 1));
            if (nodeName === 'wm-table-column-group') {
                assignColumnIndex(childNode.children, headerIndex);
            }
            columnIndex++;
            headerIndex++;
        }
    });
}
register('wm-table', () => {
    return {
        template: (node, shared) => {
            // If table does not have child columns, set isdynamictable to true
            if (node.children.length) {
                const isColumnsPresent = node.children.some(childNode => {
                    return childNode.name === 'wm-table-column' || childNode.name === 'wm-table-column-group';
                });
                assignColumnIndex(node.children);
                shared.set('isdynamictable', isColumnsPresent ? 'false' : 'true');
            }
            else {
                shared.set('isdynamictable', 'true');
            }
            const datasetAttr = node.attrs.find(attr => attr.name === dataSetKey$3);
            const widgetNameAttr = node.attrs.find(attr => attr.name === 'name');
            if (!datasetAttr) {
                return;
            }
            const boundExpr = getBoundToExpr(datasetAttr.value);
            if (!boundExpr) {
                return;
            }
            updateTemplateAttrs(node, boundExpr, widgetNameAttr.value, '', 'row');
        },
        pre: (attrs, shared) => {
            const counter = idGen$6.nextUid();
            shared.set('counter', counter);
            attrs.set('isdynamictable', shared.get('isdynamictable'));
            return `<${tagName$1x} wmTable="${counter}" wmTableFilterSort wmTableCUD #${counter} data-identifier="table" role="table" ${getAttrMarkup(attrs)}>`;
        },
        post: () => `</${tagName$1x}>`,
        provide: (attrs, shared) => {
            const provider = new Map();
            provider.set('table_reference', shared.get('counter'));
            provider.set('filtermode', attrs.get('filtermode'));
            provider.set('errorstyle', attrs.get('errorstyle'));
            provider.set('editmode', attrs.get('editmode'));
            provider.set('shownewrow', attrs.get('shownewrow'));
            return provider;
        }
    };
});
var table_build = () => { };

var table_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': table_build
});

const tagName$1y = 'div';
register('wm-video', () => {
    return {
        pre: attrs => `<${tagName$1y} wmVideo ${getAttrMarkup(attrs)}>`,
        post: () => `</${tagName$1y}>`
    };
});
var video_build = () => { };

var video_build$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': video_build
});

const initComponentsBuildTask = () => { };

/**
 * Generated bundle index. Do not edit.
 */

export { accordion_build$1 as accordionBuild, accordionPane_build$1 as accordionPaneBuild, alertDialog_build$1 as alertDlgBuild, anchor_build$1 as anchorBuild, audio_build$1 as audioBuild, barcodeScanner_build$1 as barcodeScannerBuild, breadcrumb_build$1 as breadcrumbBuild, button_build$1 as buttonBuild, buttonGroup_build$1 as buttonGroupBuild, calendar_build$1 as calendarBuild, camera_build$1 as cameraBuild, cardActions_build$1 as cardActionsBuild, card_build$1 as cardBuild, cardContent_build$1 as cardContentBuild, cardFooter_build$1 as cardFooterBuild, carousel_build$1 as carouselBuild, carouselTemplate_build$1 as carouselTemplateBuild, chart_build$1 as chartBuild, checkbox_build$1 as checkboxBuild, checkboxset_build$1 as checkboxSetBuild, chips_build$1 as chipsBuild, colorPicker_build$1 as colorPickerBuild, composite_build$1 as compositeBuild, confirmDialog_build$1 as confirmDlgBuild, container_build$1 as containerBuild, content_build$1 as contentBuild, currency_build$1 as currencyBuild, date_build$1 as dateBuild, dateTime_build$1 as dateTimeBuild, dialog_build$1 as dlgBuild, dialogFooter_build$1 as dlgFooterBuild, fileUpload_build$1 as fileUploadBuild, footer_build$1 as footerBuild, formAction_build$1 as formActionBuild, form_build$1 as formBuild, formField_build$1 as formFieldBuild, header_build$1 as headerBuild, html_build$1 as htmlBuild, icon_build$1 as iconBuild, iframe_build$1 as iframeBuild, iframeDialog_build$1 as iframeDlgBuild, initComponentsBuildTask, label_build$1 as labelBuild, leftPanel_build$1 as leftPanelBuild, layoutGrid_build$1 as lgBuild, layoutGridColumn_build$1 as lgcolBuild, layoutGridRow_build$1 as lgrowBuild, list_build$1 as listBuild, liveTable_build$1 as liveTableBuild, login_build$1 as loginBuild, loginDialog_build$1 as loginDlgBuild, marquee_build$1 as marqueeBuild, mediaList_build$1 as mediaListBuild, mediaListItem_build$1 as mediaListItemBuild, menu_build$1 as menuBuild, message_build$1 as messageBuild, mobileNavbar_build$1 as mobileNavbarBuild, nav_build$1 as navBuild, navItem_build$1 as navItemBuild, navbar_build$1 as navbarBuild, number_build$1 as numberBuild, page_build$1 as pageBuild, pageContent_build$1 as pageContentBuild, pagination_build$1 as paginationBuild, panel_build$1 as panelBuild, partial_build$1 as partialBuild, partialDialog_build$1 as partialDlgBuild, partialParam_build$1 as partialParamBuild, picture_build$1 as pictureBuild, popover_build$1 as popoverBuild, prefab_build$1 as prefabBuild, prefabContainer_build$1 as prefabContainerBuild, progressBar_build$1 as progressBarBuild, progressCircle_build$1 as progressCircleBuild, radioset_build$1 as radiosetBuild, rating_build$1 as ratingBuild, rightPanel_build$1 as rightPanelBuild, richTextEditor_build$1 as rteBuild, search_build$1 as searchBuild, segmentContent_build$1 as segContentBuild, segmentedControl_build$1 as segControlBuild, select_build$1 as selectBuild, slider_build$1 as sliderBuild, spinner_build$1 as spinnerBuild, switch_build$1 as switchBuild, tabBar_build$1 as tabBarBuild, tabPane_build$1 as tabPaneBuild, tableAction_build$1 as tableActionBuild, table_build$1 as tableBuild, tableColumn_build$1 as tableColumnBuild, tableColumnGroup_build$1 as tableColumnGroupBuild, tableRowAction_build$1 as tableRowActionBuild, tableRow_build$1 as tableRowBuild, tabs_build$1 as tabsBuild, textarea_build$1 as textAreaBuild, text_build$1 as textBuild, tile_build$1 as tileBuild, time_build$1 as timeBuild, topNav_build$1 as topNavBuild, tree_build$1 as treeBuild, video_build$1 as videoBuild, wizard_build$1 as wizardBuild, wizardStep_build$1 as wizardStepBuild };
//# sourceMappingURL=index.js.map
