import { __decorate, __metadata } from 'tslib';
import { Injector, NgZone, ContentChildren, QueryList, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { CarouselComponent, SlideComponent, CarouselModule as CarouselModule$1 } from 'ngx-bootstrap/carousel';
import { register, PROP_STRING, PROP_NUMBER, PROP_ANY, PROP_BOOLEAN, StylableComponent, styler, createArrayFrom, WmComponentsModule } from '@wm/components/base';
import { SwipeAnimation } from '@swipey';

const registerProps = () => {
    register('wm-carousel', new Map([
        ['animation', Object.assign({ value: 'auto' }, PROP_STRING)],
        ['animationinterval', Object.assign({ value: 3 }, PROP_NUMBER)],
        ['controls', Object.assign({ value: 'both' }, PROP_STRING)],
        ['dataset', PROP_ANY],
        ['name', PROP_STRING],
        ['nodatamessage', Object.assign({ value: 'No Data Found' }, PROP_STRING)],
        ['show', PROP_BOOLEAN]
    ]));
};

class CarouselAnimator extends SwipeAnimation {
    constructor(carousel, interval, ngZone) {
        super();
        this.carousel = carousel;
        this.interval = interval;
        this.ngZone = ngZone;
        this._activeIndex = 0;
        this._animationPaused = false;
        this._pauseCaroselTill = 0;
        this._oldIndex = 0;
        const self = this;
        this._$el = $(this.carousel.getNativeElement()).find('>.carousel');
        this.init(this._$el.find('>.carousel-inner'));
        this._items = this._$el.find('>.carousel-inner >.carousel-item');
        this._indicators = this._$el.find('>.carousel-indicators');
        this._indicators.find('li').each(function (i) {
            $(this).on('click', () => {
                self._activeIndex = i;
                self.setActiveItem();
                // WMS-17583: Triggering onChange event
                self.carousel.onChangeCB(self._activeIndex, self._oldIndex);
                self._oldIndex = self._activeIndex;
            });
        });
        this._$el.find('>.left.carousel-control').on('click', () => {
            this._pauseCaroselTill = Date.now() + this.interval;
            this.goToUpper();
        });
        this._$el.find('>.right.carousel-control').on('click', () => {
            this._pauseCaroselTill = Date.now() + this.interval;
            this.goToLower();
        });
        this.setActiveItem();
        if (this.interval) {
            this.start();
        }
    }
    bounds() {
        this._width = this._items.width();
        this._swiping = true;
        return {
            'lower': -this._width,
            'center': 0,
            'upper': this._width
        };
    }
    context() {
        return {
            'w': this._width
        };
    }
    animation() {
        return [{
                'target': this.getTarget.bind(this),
                'css': {
                    'transform': 'translate3d(${{ (($D + $d) / w) * 100 + \'%\'}}, 0, 0)',
                    '-webkit-transform': 'translate3d(${{ (($D + $d) / w) * 100 + \'%\'}}, 0, 0)'
                }
            }];
    }
    onUpper() {
        this.resetTransition();
        this._activeIndex -= 1;
        this.setActiveItem();
        this._swiping = false;
    }
    onLower() {
        this.resetTransition();
        this._activeIndex += 1;
        this.setActiveItem();
        this._swiping = false;
    }
    onAnimation() {
        const newIndex = (this._items.length + this._activeIndex) % this._items.length;
        this.carousel.onChangeCB(newIndex, this._oldIndex);
        this._oldIndex = newIndex;
    }
    start() {
        this.ngZone.runOutsideAngular(() => {
            this._intervalId = setInterval(() => {
                if (!this._swiping && this._pauseCaroselTill < Date.now()) {
                    this.goToLower(600);
                }
            }, this.interval);
        });
    }
    pause() {
        this._animationPaused = true;
        if (!this._intervalId) {
            this.start();
        }
    }
    resume() {
        this._animationPaused = false;
    }
    stop() {
        if (this._intervalId) {
            clearInterval(this._intervalId);
            this._intervalId = 0;
        }
    }
    threshold() {
        return 5;
    }
    // Returns multiple elements as single jQuery element.
    getTarget() {
        const noOfItems = this._items.length, activeItem = this._items.eq((noOfItems + this._activeIndex) % noOfItems), leftItem = this._items.eq((noOfItems + this._activeIndex - 1) % noOfItems), rightItem = this._items.eq((noOfItems + this._activeIndex + 1) % noOfItems);
        return activeItem.add(rightItem).add(leftItem);
    }
    resetTransition() {
        this.getTarget().css({
            '-webkit-transition': 'none',
            '-webkit-transform': '',
            'transition': 'none',
            'transform': ''
        });
    }
    // Sets active item, leftItem, rightItem by removing / adding respective classes.
    setActiveItem() {
        const items = this._items, left = this._activeIndex - 1, right = this._activeIndex + 1;
        // if there is only one carousel-item then there won't be any right or left-item.
        if (items.length === 1) {
            items.eq(0).removeClass('left-item right-item');
            return;
        }
        this._indicators.find('>.active').removeClass('active');
        this._indicators.find('> li').eq((items.length + this._activeIndex) % items.length).addClass('active');
        items.filter('.active').removeClass('active');
        items.addClass('left-item');
        items.eq((items.length + left) % items.length).addClass('left-item').removeClass('right-item');
        items.eq((items.length + this._activeIndex) % items.length).removeClass('left-item right-item').addClass('active');
        items.eq((items.length + right) % items.length).addClass('right-item').removeClass('left-item');
    }
}

const WIDGET_CONFIG = {
    widgetType: 'wm-carousel'
};
const navigationClassMap = {
    indicators: 'hide-navs',
    navs: 'hide-indicators',
    none: 'hide-both'
};
let CarouselDirective = class CarouselDirective extends StylableComponent {
    constructor(component, inj, ngZone) {
        super(inj, WIDGET_CONFIG);
        this.component = component;
        this.ngZone = ngZone;
        styler(this.nativeElement, this);
    }
    onDataChange(newVal) {
        this.fieldDefs = createArrayFrom(newVal);
    }
    stopAnimation() {
        if (this.animator) {
            this.animator.stop();
        }
    }
    startAnimation() {
        if (this.animator) {
            this.animator.interval = this.animationinterval * 1000;
            this.animator.start();
        }
    }
    onSlidesRender(slides) {
        // if dynamic carousel, initialize the 'currentslide' property as the first object
        if (this.fieldDefs && this.fieldDefs.length) {
            this.currentslide = this.fieldDefs[0];
        }
        setTimeout(() => {
            if (slides.length) {
                this.animator = new CarouselAnimator(this, this.interval, this.ngZone);
            }
        }, 50);
    }
    setupHandlers() {
        this.slides.changes.subscribe(slides => {
            this.stopAnimation();
            this.onSlidesRender(slides);
        });
        this.slides.setDirty();
    }
    onChangeCB(newIndex, oldIndex) {
        // assign current and previous slides on widget. In case of static carousel, fieldDefs will be undefined, hence the check
        this.currentslide = this.fieldDefs && this.fieldDefs[newIndex];
        this.previousslide = this.fieldDefs && this.fieldDefs[oldIndex];
        this.invokeEventCallback('change', { newIndex: newIndex, oldIndex: oldIndex });
    }
    ngAfterContentInit() {
        this.setupHandlers();
    }
    ngOnDestroy() {
        this.stopAnimation();
        super.ngOnDestroy();
    }
    // Calculating animation interval if animation is enabled
    setanimationinterval() {
        this.interval = this.animation === 'auto' ? this.animationinterval * 1000 : 0;
    }
    ngOnInit() {
        super.ngOnInit();
        this.setanimationinterval();
        // TODO transition is pending
    }
    // on property change handler
    onPropertyChange(key, nv, ov) {
        if (key === 'dataset') {
            this.onDataChange(nv);
        }
        else if (key === 'controls') {
            // For showing controls
            this.navigationClass = navigationClassMap[this.controls];
        }
        else if (key === 'animation' || key === 'animationinterval') {
            if (key === 'animationinterval') {
                this.setanimationinterval();
            }
            this.animation === 'none' ? this.stopAnimation() : this.startAnimation();
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
};
CarouselDirective.initializeProps = registerProps();
CarouselDirective.ctorParameters = () => [
    { type: CarouselComponent },
    { type: Injector },
    { type: NgZone }
];
__decorate([
    ContentChildren(SlideComponent),
    __metadata("design:type", QueryList)
], CarouselDirective.prototype, "slides", void 0);
CarouselDirective = __decorate([
    Directive({
        selector: '[wmCarousel]',
        exportAs: 'wmCarousel'
    }),
    __metadata("design:paramtypes", [CarouselComponent, Injector, NgZone])
], CarouselDirective);

const registerProps$1 = () => {
    register('wm-carousel-template', new Map([
        ['show', PROP_BOOLEAN]
    ]));
};

const DEFAULT_CLS = 'app-carousel-item';
const WIDGET_CONFIG$1 = {
    widgetType: 'wm-carousel-template',
    hostClass: DEFAULT_CLS
};
let CarouselTemplateDirective = class CarouselTemplateDirective extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$1);
        styler(this.nativeElement, this);
    }
};
CarouselTemplateDirective.initializeProps = registerProps$1();
CarouselTemplateDirective.ctorParameters = () => [
    { type: Injector }
];
CarouselTemplateDirective = __decorate([
    Directive({
        selector: '[wmCarouselTemplate]'
    }),
    __metadata("design:paramtypes", [Injector])
], CarouselTemplateDirective);

const components = [
    CarouselDirective,
    CarouselTemplateDirective
];
let CarouselModule = class CarouselModule {
};
CarouselModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            CarouselModule$1,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], CarouselModule);

/**
 * Generated bundle index. Do not edit.
 */

export { CarouselModule, CarouselDirective as ɵa, registerProps as ɵb, CarouselTemplateDirective as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
