import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, NgZone, ContentChildren, QueryList, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { CarouselComponent, SlideComponent, CarouselModule as CarouselModule$1 } from 'ngx-bootstrap/carousel';
import { register, PROP_STRING, PROP_NUMBER, PROP_ANY, PROP_BOOLEAN, styler, createArrayFrom, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { SwipeAnimation } from '@swipey';

var registerProps = function () {
    register('wm-carousel', new Map([
        ['animation', __assign({ value: 'auto' }, PROP_STRING)],
        ['animationinterval', __assign({ value: 3 }, PROP_NUMBER)],
        ['controls', __assign({ value: 'both' }, PROP_STRING)],
        ['dataset', PROP_ANY],
        ['name', PROP_STRING],
        ['nodatamessage', __assign({ value: 'No Data Found' }, PROP_STRING)],
        ['show', PROP_BOOLEAN]
    ]));
};

var CarouselAnimator = /** @class */ (function (_super) {
    __extends(CarouselAnimator, _super);
    function CarouselAnimator(carousel, interval, ngZone) {
        var _this = _super.call(this) || this;
        _this.carousel = carousel;
        _this.interval = interval;
        _this.ngZone = ngZone;
        _this._activeIndex = 0;
        _this._animationPaused = false;
        _this._pauseCaroselTill = 0;
        _this._oldIndex = 0;
        var self = _this;
        _this._$el = $(_this.carousel.getNativeElement()).find('>.carousel');
        _this.init(_this._$el.find('>.carousel-inner'));
        _this._items = _this._$el.find('>.carousel-inner >.carousel-item');
        _this._indicators = _this._$el.find('>.carousel-indicators');
        _this._indicators.find('li').each(function (i) {
            $(this).on('click', function () {
                self._activeIndex = i;
                self.setActiveItem();
                // WMS-17583: Triggering onChange event
                self.carousel.onChangeCB(self._activeIndex, self._oldIndex);
                self._oldIndex = self._activeIndex;
            });
        });
        _this._$el.find('>.left.carousel-control').on('click', function () {
            _this._pauseCaroselTill = Date.now() + _this.interval;
            _this.goToUpper();
        });
        _this._$el.find('>.right.carousel-control').on('click', function () {
            _this._pauseCaroselTill = Date.now() + _this.interval;
            _this.goToLower();
        });
        _this.setActiveItem();
        if (_this.interval) {
            _this.start();
        }
        return _this;
    }
    CarouselAnimator.prototype.bounds = function () {
        this._width = this._items.width();
        this._swiping = true;
        return {
            'lower': -this._width,
            'center': 0,
            'upper': this._width
        };
    };
    CarouselAnimator.prototype.context = function () {
        return {
            'w': this._width
        };
    };
    CarouselAnimator.prototype.animation = function () {
        return [{
                'target': this.getTarget.bind(this),
                'css': {
                    'transform': 'translate3d(${{ (($D + $d) / w) * 100 + \'%\'}}, 0, 0)',
                    '-webkit-transform': 'translate3d(${{ (($D + $d) / w) * 100 + \'%\'}}, 0, 0)'
                }
            }];
    };
    CarouselAnimator.prototype.onUpper = function () {
        this.resetTransition();
        this._activeIndex -= 1;
        this.setActiveItem();
        this._swiping = false;
    };
    CarouselAnimator.prototype.onLower = function () {
        this.resetTransition();
        this._activeIndex += 1;
        this.setActiveItem();
        this._swiping = false;
    };
    CarouselAnimator.prototype.onAnimation = function () {
        var newIndex = (this._items.length + this._activeIndex) % this._items.length;
        this.carousel.onChangeCB(newIndex, this._oldIndex);
        this._oldIndex = newIndex;
    };
    CarouselAnimator.prototype.start = function () {
        var _this = this;
        this.ngZone.runOutsideAngular(function () {
            _this._intervalId = setInterval(function () {
                if (!_this._swiping && _this._pauseCaroselTill < Date.now()) {
                    _this.goToLower(600);
                }
            }, _this.interval);
        });
    };
    CarouselAnimator.prototype.pause = function () {
        this._animationPaused = true;
        if (!this._intervalId) {
            this.start();
        }
    };
    CarouselAnimator.prototype.resume = function () {
        this._animationPaused = false;
    };
    CarouselAnimator.prototype.stop = function () {
        if (this._intervalId) {
            clearInterval(this._intervalId);
            this._intervalId = 0;
        }
    };
    CarouselAnimator.prototype.threshold = function () {
        return 5;
    };
    // Returns multiple elements as single jQuery element.
    CarouselAnimator.prototype.getTarget = function () {
        var noOfItems = this._items.length, activeItem = this._items.eq((noOfItems + this._activeIndex) % noOfItems), leftItem = this._items.eq((noOfItems + this._activeIndex - 1) % noOfItems), rightItem = this._items.eq((noOfItems + this._activeIndex + 1) % noOfItems);
        return activeItem.add(rightItem).add(leftItem);
    };
    CarouselAnimator.prototype.resetTransition = function () {
        this.getTarget().css({
            '-webkit-transition': 'none',
            '-webkit-transform': '',
            'transition': 'none',
            'transform': ''
        });
    };
    // Sets active item, leftItem, rightItem by removing / adding respective classes.
    CarouselAnimator.prototype.setActiveItem = function () {
        var items = this._items, left = this._activeIndex - 1, right = this._activeIndex + 1;
        // if there is only one carousel-item then there won't be any right or left-item.
        if (items.length === 1) {
            items.eq(0).removeClass('left-item right-item');
            return;
        }
        this._indicators.find('>.active').removeClass('active');
        this._indicators.find('> li').eq((items.length + this._activeIndex) % items.length).addClass('active');
        items.filter('.active').removeClass('active');
        items.addClass('left-item');
        items.eq((items.length + left) % items.length).addClass('left-item').removeClass('right-item');
        items.eq((items.length + this._activeIndex) % items.length).removeClass('left-item right-item').addClass('active');
        items.eq((items.length + right) % items.length).addClass('right-item').removeClass('left-item');
    };
    return CarouselAnimator;
}(SwipeAnimation));

var WIDGET_CONFIG = {
    widgetType: 'wm-carousel'
};
var navigationClassMap = {
    indicators: 'hide-navs',
    navs: 'hide-indicators',
    none: 'hide-both'
};
var CarouselDirective = /** @class */ (function (_super) {
    __extends(CarouselDirective, _super);
    function CarouselDirective(component, inj, ngZone) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.component = component;
        _this.ngZone = ngZone;
        styler(_this.nativeElement, _this);
        return _this;
    }
    CarouselDirective.prototype.onDataChange = function (newVal) {
        this.fieldDefs = createArrayFrom(newVal);
    };
    CarouselDirective.prototype.stopAnimation = function () {
        if (this.animator) {
            this.animator.stop();
        }
    };
    CarouselDirective.prototype.startAnimation = function () {
        if (this.animator) {
            this.animator.interval = this.animationinterval * 1000;
            this.animator.start();
        }
    };
    CarouselDirective.prototype.onSlidesRender = function (slides) {
        var _this = this;
        // if dynamic carousel, initialize the 'currentslide' property as the first object
        if (this.fieldDefs && this.fieldDefs.length) {
            this.currentslide = this.fieldDefs[0];
        }
        setTimeout(function () {
            if (slides.length) {
                _this.animator = new CarouselAnimator(_this, _this.interval, _this.ngZone);
            }
        }, 50);
    };
    CarouselDirective.prototype.setupHandlers = function () {
        var _this = this;
        this.slides.changes.subscribe(function (slides) {
            _this.stopAnimation();
            _this.onSlidesRender(slides);
        });
        this.slides.setDirty();
    };
    CarouselDirective.prototype.onChangeCB = function (newIndex, oldIndex) {
        // assign current and previous slides on widget. In case of static carousel, fieldDefs will be undefined, hence the check
        this.currentslide = this.fieldDefs && this.fieldDefs[newIndex];
        this.previousslide = this.fieldDefs && this.fieldDefs[oldIndex];
        this.invokeEventCallback('change', { newIndex: newIndex, oldIndex: oldIndex });
    };
    CarouselDirective.prototype.ngAfterContentInit = function () {
        this.setupHandlers();
    };
    CarouselDirective.prototype.ngOnDestroy = function () {
        this.stopAnimation();
        _super.prototype.ngOnDestroy.call(this);
    };
    // Calculating animation interval if animation is enabled
    CarouselDirective.prototype.setanimationinterval = function () {
        this.interval = this.animation === 'auto' ? this.animationinterval * 1000 : 0;
    };
    CarouselDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.setanimationinterval();
        // TODO transition is pending
    };
    // on property change handler
    CarouselDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'dataset') {
            this.onDataChange(nv);
        }
        else if (key === 'controls') {
            // For showing controls
            this.navigationClass = navigationClassMap[this.controls];
        }
        else if (key === 'animation' || key === 'animationinterval') {
            if (key === 'animationinterval') {
                this.setanimationinterval();
            }
            this.animation === 'none' ? this.stopAnimation() : this.startAnimation();
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    CarouselDirective.initializeProps = registerProps();
    CarouselDirective.ctorParameters = function () { return [
        { type: CarouselComponent },
        { type: Injector },
        { type: NgZone }
    ]; };
    __decorate([
        ContentChildren(SlideComponent),
        __metadata("design:type", QueryList)
    ], CarouselDirective.prototype, "slides", void 0);
    CarouselDirective = __decorate([
        Directive({
            selector: '[wmCarousel]',
            exportAs: 'wmCarousel'
        }),
        __metadata("design:paramtypes", [CarouselComponent, Injector, NgZone])
    ], CarouselDirective);
    return CarouselDirective;
}(StylableComponent));

var registerProps$1 = function () {
    register('wm-carousel-template', new Map([
        ['show', PROP_BOOLEAN]
    ]));
};

var DEFAULT_CLS = 'app-carousel-item';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-carousel-template',
    hostClass: DEFAULT_CLS
};
var CarouselTemplateDirective = /** @class */ (function (_super) {
    __extends(CarouselTemplateDirective, _super);
    function CarouselTemplateDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        styler(_this.nativeElement, _this);
        return _this;
    }
    CarouselTemplateDirective.initializeProps = registerProps$1();
    CarouselTemplateDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    CarouselTemplateDirective = __decorate([
        Directive({
            selector: '[wmCarouselTemplate]'
        }),
        __metadata("design:paramtypes", [Injector])
    ], CarouselTemplateDirective);
    return CarouselTemplateDirective;
}(StylableComponent));

var components = [
    CarouselDirective,
    CarouselTemplateDirective
];
var CarouselModule = /** @class */ (function () {
    function CarouselModule() {
    }
    CarouselModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                CarouselModule$1,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], CarouselModule);
    return CarouselModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { CarouselModule, CarouselDirective as ɵa, registerProps as ɵb, CarouselTemplateDirective as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
