import { __decorate, __metadata } from 'tslib';
import { Injector, ContentChild, ContentChildren, QueryList, ViewChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_ANY, StylableComponent, styler, APPLY_STYLES_TYPE, MessageComponent, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { FormComponent } from '@wm/components/data/form';
import { ButtonComponent } from '@wm/components/input';

const registerProps = () => {
    register('wm-login', new Map([
        ['errormessage', PROP_STRING],
        ['eventsource', PROP_ANY]
    ]));
};

var LoginComponent_1;
const WIDGET_INFO = { widgetType: 'wm-login', hostClass: 'app-login' };
let LoginComponent = LoginComponent_1 = class LoginComponent extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_INFO);
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
    onSuccessCB() {
        this.invokeEventCallback('success');
    }
    onErrorCB(error) {
        this.loginMessage = {
            type: 'error',
            caption: this.errormessage || error || this.appLocale.LABEL_INVALID_USERNAME_OR_PASSWORD,
            show: true
        };
        this.messageCmp.showMessage(this.loginMessage.caption, this.loginMessage.type);
        this.invokeEventCallback('error');
    }
    getLoginDetails() {
        return this.formCmp.dataoutput;
    }
    doLogin() {
        if (this.eventsource) {
            this.formCmp.ngform.valid && this.eventsource.invoke({ loginInfo: this.getLoginDetails() }, this.onSuccessCB.bind(this), this.onErrorCB.bind(this));
        }
        else {
            console.warn('Default action "loginAction" does not exist. Either create the Action or assign an event to onSubmit of the login widget');
        }
    }
    initLoginButtonActions() {
        this.loginBtnCmp.getNativeElement().addEventListener('click', event => {
            // if no event is attached to the onSubmit of login widget or loginButton inside it, invoke default login action
            if (!this.nativeElement.hasAttribute('submit.event') && !this.loginBtnCmp.getNativeElement().hasAttribute('click.event') && !this.loginBtnCmp.getNativeElement().hasAttribute('tap.event')) {
                this.doLogin();
            }
        });
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        // suppresses the default form submission (in browsers like Firefox)
        this.formCmp.getNativeElement().addEventListener('submit', e => e.preventDefault());
        // get login button component
        this.buttonComponents.forEach(cmp => {
            if (cmp.getNativeElement().getAttribute('name') === 'loginButton' || _.includes(cmp.getNativeElement().classList, 'app-login-button')) {
                if (this.loginBtnCmp) {
                    return;
                }
                this.loginBtnCmp = cmp;
                this.initLoginButtonActions();
            }
        });
    }
};
LoginComponent.initializeProps = registerProps();
LoginComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ContentChild(FormComponent),
    __metadata("design:type", FormComponent)
], LoginComponent.prototype, "formCmp", void 0);
__decorate([
    ContentChildren(ButtonComponent, { descendants: true }),
    __metadata("design:type", QueryList)
], LoginComponent.prototype, "buttonComponents", void 0);
__decorate([
    ViewChild(MessageComponent, { static: true }),
    __metadata("design:type", MessageComponent)
], LoginComponent.prototype, "messageCmp", void 0);
LoginComponent = LoginComponent_1 = __decorate([
    Component({
        selector: 'div[wmLogin]',
        template: "<p wmMessage class=\"app-login-message\" #message show.bind=\"loginMessage.show\" caption.bind=\"loginMessage.caption\" type.bind=\"loginMessage.type\"></p>\n<ng-content></ng-content>\n",
        providers: [
            provideAsWidgetRef(LoginComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], LoginComponent);

const components = [
    LoginComponent
];
let LoginModule = class LoginModule {
};
LoginModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], LoginModule);

/**
 * Generated bundle index. Do not edit.
 */

export { LoginModule, LoginComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
