import { __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ContentChild, ContentChildren, QueryList, ViewChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_ANY, styler, APPLY_STYLES_TYPE, MessageComponent, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { FormComponent } from '@wm/components/data/form';
import { ButtonComponent } from '@wm/components/input';

var registerProps = function () {
    register('wm-login', new Map([
        ['errormessage', PROP_STRING],
        ['eventsource', PROP_ANY]
    ]));
};

var WIDGET_INFO = { widgetType: 'wm-login', hostClass: 'app-login' };
var LoginComponent = /** @class */ (function (_super) {
    __extends(LoginComponent, _super);
    function LoginComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_INFO) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    LoginComponent_1 = LoginComponent;
    LoginComponent.prototype.onSuccessCB = function () {
        this.invokeEventCallback('success');
    };
    LoginComponent.prototype.onErrorCB = function (error) {
        this.loginMessage = {
            type: 'error',
            caption: this.errormessage || error || this.appLocale.LABEL_INVALID_USERNAME_OR_PASSWORD,
            show: true
        };
        this.messageCmp.showMessage(this.loginMessage.caption, this.loginMessage.type);
        this.invokeEventCallback('error');
    };
    LoginComponent.prototype.getLoginDetails = function () {
        return this.formCmp.dataoutput;
    };
    LoginComponent.prototype.doLogin = function () {
        if (this.eventsource) {
            this.formCmp.ngform.valid && this.eventsource.invoke({ loginInfo: this.getLoginDetails() }, this.onSuccessCB.bind(this), this.onErrorCB.bind(this));
        }
        else {
            console.warn('Default action "loginAction" does not exist. Either create the Action or assign an event to onSubmit of the login widget');
        }
    };
    LoginComponent.prototype.initLoginButtonActions = function () {
        var _this = this;
        this.loginBtnCmp.getNativeElement().addEventListener('click', function (event) {
            // if no event is attached to the onSubmit of login widget or loginButton inside it, invoke default login action
            if (!_this.nativeElement.hasAttribute('submit.event') && !_this.loginBtnCmp.getNativeElement().hasAttribute('click.event') && !_this.loginBtnCmp.getNativeElement().hasAttribute('tap.event')) {
                _this.doLogin();
            }
        });
    };
    LoginComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        _super.prototype.ngAfterViewInit.call(this);
        // suppresses the default form submission (in browsers like Firefox)
        this.formCmp.getNativeElement().addEventListener('submit', function (e) { return e.preventDefault(); });
        // get login button component
        this.buttonComponents.forEach(function (cmp) {
            if (cmp.getNativeElement().getAttribute('name') === 'loginButton' || _.includes(cmp.getNativeElement().classList, 'app-login-button')) {
                if (_this.loginBtnCmp) {
                    return;
                }
                _this.loginBtnCmp = cmp;
                _this.initLoginButtonActions();
            }
        });
    };
    var LoginComponent_1;
    LoginComponent.initializeProps = registerProps();
    LoginComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ContentChild(FormComponent),
        __metadata("design:type", FormComponent)
    ], LoginComponent.prototype, "formCmp", void 0);
    __decorate([
        ContentChildren(ButtonComponent, { descendants: true }),
        __metadata("design:type", QueryList)
    ], LoginComponent.prototype, "buttonComponents", void 0);
    __decorate([
        ViewChild(MessageComponent, { static: true }),
        __metadata("design:type", MessageComponent)
    ], LoginComponent.prototype, "messageCmp", void 0);
    LoginComponent = LoginComponent_1 = __decorate([
        Component({
            selector: 'div[wmLogin]',
            template: "<p wmMessage class=\"app-login-message\" #message show.bind=\"loginMessage.show\" caption.bind=\"loginMessage.caption\" type.bind=\"loginMessage.type\"></p>\n<ng-content></ng-content>\n",
            providers: [
                provideAsWidgetRef(LoginComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], LoginComponent);
    return LoginComponent;
}(StylableComponent));

var components = [
    LoginComponent
];
var LoginModule = /** @class */ (function () {
    function LoginModule() {
    }
    LoginModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], LoginModule);
    return LoginModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { LoginModule, LoginComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
