import { __spread, __assign, __values, __read, __extends, __decorate, __param, __metadata, __awaiter, __generator } from 'tslib';
import { InjectionToken, forwardRef, ElementRef, Self, Inject, Directive, ContentChildren, Injector, Pipe, Input, TemplateRef, ViewContainerRef, Component, Attribute, ComponentFactoryResolver, SecurityContext, Injectable, NgModule } from '@angular/core';
import { DecimalPipe, DatePipe, Location, CommonModule } from '@angular/common';
import { FormWidgetType, checkIsCustomPipeExpression, $parseExpr, isValidWebURL, encodeUrl, stringStartsWith, prettifyLabel, deHyphenate, initCaps, getClonedObject, setCSS, isDefined, IDGenerator, toBoolean, isChangeFromWatch, $unwatch, resetChangeFromWatch, isObject, toDimension, $appDigest, removeClass, addClass, setCSSFromObj, App, setAttr, switchClass, removeAttr, $parseEvent, $invokeWatchers, $watch, transformFileURI, Viewport, noop as noop$1, ComponentType, PartialRefProvider, isMobileApp, isKitkatDevice, debounce, CURRENCY_INFO, CustomPipeManager, AbstractDialogService, DataSource, processFilterExpBindNode, MatchMode, isEqualWithFields, getFormattedDate, isAndroid, isNumberType, DataType, findValueOf, validateAccessRoles, VALIDATOR, isIos, isMobile } from '@wm/core';
import { SecurityModule, SecurityService } from '@wm/security';
import { EventManager, DomSanitizer } from '@angular/platform-browser';
import { ReplaySubject, Subject } from 'rxjs';
import { filter, debounceTime } from 'rxjs/operators';
import { Validators } from '@angular/forms';
import { SwipeAnimation } from '@swipey';

var WidgetRef = /** @class */ (function () {
    function WidgetRef() {
    }
    return WidgetRef;
}());
var DialogRef = /** @class */ (function () {
    function DialogRef() {
    }
    return DialogRef;
}());
var Context = new InjectionToken('Context Provider Reference');

var DATASET_WIDGETS = new Set([FormWidgetType.SELECT, FormWidgetType.CHECKBOXSET, FormWidgetType.RADIOSET,
    FormWidgetType.SWITCH, FormWidgetType.AUTOCOMPLETE, FormWidgetType.CHIPS, FormWidgetType.TYPEAHEAD, FormWidgetType.RATING]);
/**
 * Returns the parsed, updated bound expression
 * if the expression is $[data[$i][firstName]] + '--' + $[lastName] + '--' + $['@ID@']
 * returns __1.firstName + '--' + lastName + '--' + __1['@ID@']
 */
var getUpdatedExpr = function (expr) {
    var updated = '', ch, next, i, j, matchCh, matchCount, isQuotedStr, subStr, isQuotedStrEvaluated;
    expr = expr.replace(/\$\[data\[\$i\]/g, '$[__1');
    for (i = 0; i < expr.length; i++) {
        ch = expr[i];
        next = expr[i + 1];
        /**
         * if the expression starts with $[, check the next(ch) character,
         *    if ch is a quote(', ") change the expr to __[
         *    if ch is a whiteSpace, remove it
         *    else remove $[
         */
        if (ch === '$' && next === '[') {
            matchCount = 1;
            isQuotedStrEvaluated = false;
            isQuotedStr = false;
            for (j = i + 2; j < expr.length; j++) {
                matchCh = expr[j];
                if (matchCh === ' ') {
                    continue;
                }
                if (!isQuotedStrEvaluated) {
                    isQuotedStr = expr[j] === '"' || expr[j] === '\'';
                    isQuotedStrEvaluated = true;
                }
                if (matchCh === '[') {
                    matchCount++;
                }
                else if (matchCh === ']') {
                    matchCount--;
                }
                if (!matchCount) {
                    subStr = expr.substring(i + 2, j);
                    if (isQuotedStr) {
                        updated += '__1[' + subStr + ']';
                    }
                    else {
                        updated += subStr;
                    }
                    break;
                }
            }
            i = j;
        }
        else {
            updated += ch;
        }
    }
    return updated;
};
var ɵ0 = getUpdatedExpr;
/**
 * Returns the value for the provided key in the object
 */
var getObjValueByKey = function (obj, strKey) {
    /* check for the key-string */
    if (strKey) {
        var val_1;
        /* convert indexes to properties, so as to work for even 'key1[0].child1'*/
        strKey.replace(/\[(\w+)\]/g, '.$1').split('.').forEach(function (key) {
            // If obj is null, then assign val to null.
            val_1 = (val_1 && val_1[key]) || (_.isNull(obj) ? obj : obj[key]);
        });
        return val_1;
    }
    return obj;
};
/**
 * returns the display field data for any dataset widgets
 * Based on the bind display expression or display expression or display name,
 * data is extracted and formatted from the passed option object
 * If there is field is specified, field value is obtained from the dataObj.
 * If expression is given, evaluates the expression value.
 * else check for bindExpression, extract the value from the dataObj
 */
var getEvaluatedData = function (dataObj, options, context) {
    var expressionValue;
    var field = options.field, expr = options.expression, bindExpr = options.bindExpression;
    // if key is bound expression
    if (bindExpr) {
        // remove 'bind:' prefix from the boundExpressionName
        expressionValue = bindExpr.replace('bind:', '');
        // For bind expression passing the last argument that the object of dataset item.
        // Note: For parse expression we are sending dataObj with the propery name "__1", we used the same propery to send the last argument as dataset item
        if (checkIsCustomPipeExpression(expressionValue)) {
            expressionValue = expressionValue + ': __1';
        }
        // parse the expressionValue for replacing all the expressions with values in the object
        expressionValue = getUpdatedExpr(expressionValue);
    }
    else {
        expressionValue = expr ? expr : field;
    }
    // Handling field name with special charecters
    // Ex: field = "f name"
    if (!bindExpr && !expr) {
        return _.get(dataObj, field);
    }
    return $parseExpr(expressionValue)(context, Object.assign({}, dataObj, { __1: dataObj }));
};
var isActiveNavItem = function (link, routeName) {
    if (!link || !routeName) {
        return false;
    }
    routeName = routeName.indexOf('?') === -1 ? routeName : routeName.substring(0, routeName.indexOf('?'));
    link = link.indexOf('?') === -1 ? link : link.substring(0, link.indexOf('?'));
    var routeRegex = new RegExp('^(#\/|#)' + routeName + '$');
    return routeRegex.test(link);
};
/**
 * Returns the orderBy Expression based on the 'sort 'option in pageable object
 * returned by backend
 * @param pageableObj
 * @returns {string}
 */
var getOrderByExpr = function (pageableObj) {
    pageableObj = pageableObj || [];
    var expressions = [], KEY_VAL_SEPARATOR = ' ', FIELD_SEPARATOR = ',';
    _.forEach(pageableObj, function (obj) {
        if (obj.direction) {
            expressions.push(obj.property + KEY_VAL_SEPARATOR + obj.direction.toLowerCase());
        }
    });
    return _.join(expressions, FIELD_SEPARATOR);
};
var isDataSetWidget = function (widget) {
    return DATASET_WIDGETS.has(widget);
};
/*This function returns the url to the image after checking the validity of url*/
var getImageUrl = function (urlString, shouldEncode, defaultUrl) {
    /*In studio mode before setting picturesource, check if the studioController is loaded and new picturesource is in 'styles/images/' path or not.
     * When page is refreshed, loader.gif will be loaded first and it will be in 'style/images/'.
     * Prepend 'services/projects/' + $rootScope.project.id + '/web/resources/images/imagelists/'  if the image url is just image name in the project root,
     * and if the url pointing to resources/images/ then 'services/projects/' + $rootScope.project.id + '/web/'*/
    if (isValidWebURL(urlString)) {
        return urlString;
    }
    // If no value is provided for picturesource assign pictureplaceholder or default-image
    if (!urlString) {
        urlString = defaultUrl || 'resources/images/imagelists/default-image.png';
    }
    urlString = shouldEncode ? encodeUrl(urlString) : urlString;
    // if the resource to be loaded is inside a prefab
    if (stringStartsWith(urlString, 'services/prefabs')) {
        return urlString;
    }
    return urlString;
};
/*This method returns the url to the backgroundImage*/
var getBackGroundImageUrl = function (urlString) {
    if (urlString === '' || urlString === 'none') {
        return urlString;
    }
    return 'url(' + getImageUrl(urlString) + ')';
};
function provideAs(reference, key, multi) {
    return {
        provide: key,
        useExisting: forwardRef(function () { return reference; }),
        multi: multi
    };
}
function provideAsWidgetRef(reference) {
    return provideAs(reference, WidgetRef);
}
function provideAsDialogRef(reference) {
    return provideAs(reference, DialogRef);
}
var NAVIGATION_TYPE = {
    ADVANCED: 'Advanced',
    BASIC: 'Basic',
    CLASSIC: 'Classic',
    INLINE: 'Inline',
    NONE: 'None',
    ONDEMAND: 'On-Demand',
    PAGER: 'Pager',
    SCROLL: 'Scroll'
};
var AUTOCLOSE_TYPE = {
    OUTSIDECLICK: 'outsideClick',
    ALWAYS: 'always',
    DISABLED: 'disabled'
};
var getWatchIdentifier = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return args.join('_');
};
var typesMap = {
    number: ['number', 'integer', 'big_integer', 'short', 'float', 'big_decimal', 'double', 'long', 'byte'],
    string: ['string', 'text'],
    character: ['character'],
    date: ['date', 'time', 'timestamp', 'datetime']
};
var modes = {
    number: ['exact', 'notequals', 'lessthan', 'lessthanequal', 'greaterthan', 'greaterthanequal', 'null', 'isnotnull'],
    string: ['anywhereignorecase', 'anywhere', 'startignorecase', 'start', 'endignorecase', 'end', 'exactignorecase', 'exact', 'notequalsignorecase', 'notequals', 'null', 'isnotnull', 'empty', 'isnotempty', 'nullorempty'],
    character: ['exactignorecase', 'exact', 'notequalsignorecase', 'notequals', 'null', 'isnotnull', 'empty', 'isnotempty', 'nullorempty'],
    date: ['exact', 'lessthan', 'lessthanequal', 'greaterthan', 'greaterthanequal', 'null', 'notequals', 'isnotnull']
};
var matchModeTypesMap = {
    boolean: ['exact', 'null', 'isnotnull'],
    clob: [],
    blob: []
};
var getMatchModeTypesMap = function (multiMode) {
    if (multiMode) {
        modes.number.push('in', 'notin', 'between');
        modes.date.push('between');
        modes.string.push('in', 'notin');
        modes.character.push('in', 'notin');
    }
    _.forEach(typesMap, function (types, primType) {
        _.forEach(types, function (type) {
            matchModeTypesMap[type] = modes[primType];
        });
    });
    // this is used in filter criteria when the user types the column name manually and where we dont know the type of the column
    matchModeTypesMap['default'] = _.union(modes['number'], modes['string'], modes['character'], modes['date'], modes['date']);
    return matchModeTypesMap;
};
var getMatchModeMsgs = function (appLocale) {
    return {
        start: appLocale.LABEL_STARTS_WITH,
        startignorecase: appLocale.LABEL_STARTS_WITH_IGNORECASE,
        end: appLocale.LABEL_ENDS_WITH,
        endignorecase: appLocale.LABEL_ENDS_WITH_IGNORECASE,
        anywhere: appLocale.LABEL_CONTAINS,
        anywhereignorecase: appLocale.LABEL_CONTAINS_IGNORECASE,
        exact: appLocale.LABEL_IS_EQUAL_TO,
        exactignorecase: appLocale.LABEL_IS_EQUAL_TO_IGNORECASE,
        notequals: appLocale.LABEL_IS_NOT_EQUAL_TO,
        notequalsignorecase: appLocale.LABEL_IS_NOT_EQUAL_TO_IGNORECASE,
        lessthan: appLocale.LABEL_LESS_THAN,
        lessthanequal: appLocale.LABEL_LESS_THAN_OR_EQUALS_TO,
        greaterthan: appLocale.LABEL_GREATER_THAN,
        greaterthanequal: appLocale.LABEL_GREATER_THAN_OR_EQUALS_TO,
        null: appLocale.LABEL_IS_NULL,
        isnotnull: appLocale.LABEL_IS_NOT_NULL,
        empty: appLocale.LABEL_IS_EMPTY,
        isnotempty: appLocale.LABEL_IS_NOT_EMPTY,
        nullorempty: appLocale.LABEL_IS_NULL_OR_EMPTY,
        in: appLocale.LABEL_IN,
        notin: appLocale.LABEL_NOT_IN,
        between: appLocale.LABEL_BETWEEN
    };
};
// Returns array of classes that are evaluated true for given object or array
var getClassesArray = function (classVal) {
    var classes = [];
    if (_.isArray(classVal)) {
        classVal.forEach(function (v) {
            classes = classes.concat(getClassesArray(v));
        });
        return classes;
    }
    if (_.isObject(classVal)) {
        _.forEach(classVal, function (val, key) {
            if (val) {
                classes = classes.concat(key.split(' '));
            }
        });
        return classes;
    }
};
var ɵ1 = getClassesArray;
var getConditionalClasses = function (nv, ov) {
    var toAdd;
    var toRemove;
    // if the conditional class property has already toAdd and toRemove arrays then take that otherwise build those arrays
    var classToAdd = nv.toAdd || nv;
    var classToRemove = nv.toRemove || ov;
    if (_.isObject(nv)) {
        toAdd = _.isArray(classToAdd) ? classToAdd : getClassesArray(classToAdd || []);
        toRemove = classToRemove ? (_.isArray(classToRemove) ? classToRemove : getClassesArray(classToRemove)) : [];
    }
    else {
        toAdd = classToAdd ? [classToAdd] : [];
        toRemove = classToRemove ? [classToRemove] : [];
    }
    return { toAdd: toAdd, toRemove: toRemove };
};
/*helper function for prepareFieldDefs*/
var pushFieldDef = function (dataObject, columnDefObj, namePrefix, options) {
    /*loop over the fields in the dataObject to process them*/
    var modifiedTitle, relatedTable, relatedField, relatedInfo, fieldName, isRelated;
    if (!options) {
        options = {};
    }
    _.forEach(dataObject, function (value, title) {
        if (_.includes(title, '.')) {
            relatedInfo = _.split(title, '.');
            relatedTable = relatedInfo[0];
            relatedField = relatedInfo[1];
            isRelated = true;
        }
        if (options.noModifyTitle) {
            modifiedTitle = title;
        }
        else {
            if (_.isString(title)) {
                modifiedTitle = prettifyLabel(title);
                modifiedTitle = deHyphenate(modifiedTitle);
                modifiedTitle = namePrefix ? initCaps(namePrefix) + ' ' + modifiedTitle : modifiedTitle;
            }
            else {
                modifiedTitle = title;
            }
        }
        title = namePrefix ? namePrefix + '.' + title : title;
        if (isRelated) {
            // For related columns, shorten the title to last two words
            fieldName = _.split(modifiedTitle, ' ');
            fieldName = fieldName.length > 1 ? fieldName[fieldName.length - 2] + ' ' + fieldName[fieldName.length - 1] : fieldName[0];
        }
        else {
            fieldName = modifiedTitle;
        }
        var defObj = options.setBindingField ? { 'displayName': fieldName, 'field': title, 'relatedTable': relatedTable, 'relatedField': relatedField || modifiedTitle }
            : { 'displayName': fieldName, 'relatedTable': relatedTable, 'relatedField': relatedField || modifiedTitle };
        /*if field is a leaf node, push it in the columnDefs*/
        if (!_.isObject(value) || (_.isArray(value) && !value[0])) {
            /*if the column counter has reached upperBound return*/
            if (options.upperBound && options.columnCount === options.upperBound) {
                return;
            }
            columnDefObj.terminals.push(defObj);
            /*increment the column counter*/
            options.columnCount += 1;
        }
        else {
            /*else field is an object, process it recursively*/
            /* if parent node to be included, include it */
            if (options.columnCount !== options.upperBound) {
                columnDefObj.objects.push(defObj);
            }
            /* if field is an array node, process its first child */
            if (_.isArray(value) && value[0]) {
                pushFieldDef(value[0], columnDefObj, title + '[0]', options);
            }
            else {
                pushFieldDef(value, columnDefObj, title, options);
            }
        }
    });
};
var ɵ2 = pushFieldDef;
var getMetaDataFromData = function (data) {
    var dataObject;
    if (_.isArray(data)) {
        if (_.isObject(data[0])) {
            dataObject = getClonedObject(data[0]);
            /*Loop over the object to find out any null values. If any null values are present in the first row, check and assign the values from other row.
             * As column generation is dependent on data, for related fields if first row value is null, columns are not generated.
             * To prevent this, check the data in other rows and generate the columns. New keys from others rows are also added*/
            _.forEach(data, function (row, index) {
                if ((index + 1) >= 10) { // Limit the data search to first 10 records
                    return false;
                }
                _.assignWith(dataObject, row, function (objValue, srcValue) {
                    return (objValue === null || objValue === undefined) ? srcValue : objValue;
                });
            });
        }
        else {
            dataObject = data[0];
        }
    }
    else {
        dataObject = data;
    }
    return dataObject;
};
var ɵ3 = getMetaDataFromData;
var prepareFieldDefs = function (data, options) {
    var dataObject;
    var columnDef = {
        'objects': [],
        'terminals': []
    };
    /*if no data provided, initialize default column definitions*/
    if (!data) {
        data = [];
    }
    if (!options) {
        options = {};
    }
    options.setBindingField = true;
    options.columnCount = 0;
    dataObject = getMetaDataFromData(data);
    /*first of the many data objects from grid data*/
    pushFieldDef(dataObject, columnDef, '', options);
    if (!options || (options && !options.filter)) {
        return columnDef.terminals;
    }
    switch (options.filter) {
        case 'all':
            return columnDef;
        case 'objects':
            return columnDef.objects;
        case 'terminals':
            return columnDef.terminals;
    }
    return columnDef;
};
/**
 * To get container target element to append the list for search and datepicker panel when field present in dialog
 */
var getContainerTargetClass = function (element) {
    return 'body';
    // Below code is not working when we have nested dialogs.
    // Nested dialogs case we will have  multiple '.modal-content' classes on DOM due to that container positions is not working as expected.
    // Below code to fix the issue ==> WMS-18751
    // As regression we got ==> WMS-19835
    // if (element.closest('.modal-dialog')) {
    //     return '.modal-content';
    // } else {
    //     return 'body';
    // }
};
/**
 * To extract the Variable Name from a Widget's binddatasource attribute
 */
var extractDataSourceName = function (bindDataSource) {
    if (!bindDataSource) {
        return;
    }
    var dataSourceName;
    var parts = bindDataSource.split('.');
    if (parts.length > 1 && parts[0] === 'Variables') {
        dataSourceName = parts[1];
    }
    return dataSourceName;
};

var APPLY_STYLES_TYPE;
(function (APPLY_STYLES_TYPE) {
    APPLY_STYLES_TYPE[APPLY_STYLES_TYPE["CONTAINER"] = 0] = "CONTAINER";
    APPLY_STYLES_TYPE[APPLY_STYLES_TYPE["SCROLLABLE_CONTAINER"] = 1] = "SCROLLABLE_CONTAINER";
    APPLY_STYLES_TYPE[APPLY_STYLES_TYPE["INNER_SHELL"] = 2] = "INNER_SHELL";
    APPLY_STYLES_TYPE[APPLY_STYLES_TYPE["SHELL"] = 3] = "SHELL";
})(APPLY_STYLES_TYPE || (APPLY_STYLES_TYPE = {}));
var propNameCSSKeyMap = {
    'backgroundattachment': 'backgroundAttachment',
    'backgroundcolor': 'backgroundColor',
    'backgroundgradient': 'backgroundGradient',
    'backgroundimage': 'backgroundImage',
    'backgroundposition': 'backgroundPosition',
    'backgroundrepeat': 'backgroundRepeat',
    'backgroundsize': 'backgroundSize',
    'bordercolor': 'borderColor',
    'borderradius': 'borderRadius',
    'borderstyle': 'borderStyle',
    'borderwidth': 'borderWidth',
    'borderbottomwidth': 'borderBottomWidth',
    'borderleftwidth': 'borderLeftWidth',
    'borderrightwidth': 'borderRightWidth',
    'bordertopwidth': 'borderTopWidth',
    'color': 'color',
    'cursor': 'cursor',
    'display': 'display',
    'fontsize': 'fontSize',
    'fontfamily': 'fontFamily',
    'fontstyle': 'fontStyle',
    'fontunit': 'fontunit',
    'fontvariant': 'fontVariant',
    'fontweight': 'fontWeight',
    'height': 'height',
    'horizontalalign': 'textAlign',
    'lineheight': 'lineHeight',
    'margin': 'margin',
    'marginbottom': 'marginBottom',
    'marginleft': 'marginLeft',
    'marginright': 'marginRight',
    'margintop': 'marginTop',
    'opacity': 'opacity',
    'overflow': 'overflow',
    'padding': 'padding',
    'paddingbottom': 'paddingBottom',
    'paddingleft': 'paddingLeft',
    'paddingright': 'paddingRight',
    'paddingtop': 'paddingTop',
    'picturesource': 'backgroundImage',
    'textalign': 'textAlign',
    'textdecoration': 'textDecoration',
    'verticalalign': 'verticalAlign',
    'visibility': 'visibility',
    'whitespace': 'whiteSpace',
    'width': 'width',
    'wordbreak': 'wordbreak',
    'zindex': 'zIndex'
};
var isStyle = function (key) { return !!propNameCSSKeyMap[key]; };
var MAP_SHELL_TYPE_IGNORE_LIST = {
    height: true,
    overflow: true,
    padding: true,
    paddingbottom: true,
    paddingleft: true,
    paddingright: true,
    paddingtop: true
};
var MAP_CONTAINER_TYPE_IGNORE_LIST = {
    textalign: true
};
var MAP_SCROLLABLE_CONTAINER_TYPE_IGNORE_LIST = {
    textalign: true,
    width: true
};
function styler($node, component, type, skipList) {
    // apply init styles;
    var skipListMap = Object.create(null);
    if (skipList) {
        skipList.forEach(function (k) { return skipListMap[k] = true; });
    }
    component.registerStyleChangeListener(function (key, nv) {
        if (skipListMap[key]) {
            return;
        }
        // if the type is `shell` and the key is in the SHELL_TYPE_IGNORE_LIST, return
        if (type === APPLY_STYLES_TYPE.SHELL && MAP_SHELL_TYPE_IGNORE_LIST[key]) {
            return;
        }
        // if the type is `inner-shell` and the key is NOT in the SHELL_TYPE_IGNORE_LIST, return
        if (type === APPLY_STYLES_TYPE.INNER_SHELL) {
            if (!MAP_SHELL_TYPE_IGNORE_LIST[key]) {
                return;
            }
            if (key === 'height') {
                setCSS($node, 'overflow', nv ? 'auto' : '');
            }
        }
        // if the type is `container` and the key is in the CONTAINER_TYPE_IGNORE_LIST, return
        if (type === APPLY_STYLES_TYPE.CONTAINER && MAP_CONTAINER_TYPE_IGNORE_LIST[key]) {
            return;
        }
        if (type === APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER) {
            if (MAP_SCROLLABLE_CONTAINER_TYPE_IGNORE_LIST[key]) {
                return;
            }
            if (key === 'height') {
                setCSS($node, 'overflow', nv ? 'auto' : '');
            }
        }
        if (key === 'fontsize' || key === 'fontunit') {
            setCSS($node, 'fontSize', component.fontsize === '' ? '' : component.fontsize + (component.fontunit || 'px'));
        }
        else if (key === 'backgroundimage') {
            setCSS($node, 'backgroundImage', component.picturesource || getBackGroundImageUrl(nv));
        }
        else if (propNameCSSKeyMap[key]) {
            setCSS($node, propNameCSSKeyMap[key], nv);
        }
    });
}

var formFieldPrefix = 'wm-form-field-';
var ɵ0$1 = function () {
    var props = new Map();
    for (var key in FormWidgetType) {
        props.set(formFieldPrefix + FormWidgetType[key], new Map());
    }
    return props;
};
// map of widgetSubType vs WidgetProps
var widgetProps = (ɵ0$1)();
// returns the widgetPropsMap associated with the given identifier
var getWidgetPropsByType = function (identifier) { return widgetProps.get(identifier); };
// register the widgetProps with the given identifier
var register = function (identifier, props) {
    widgetProps.set(identifier, props);
    window.WM_LOADED_COMPONENTS = __spread(window.WM_LOADED_COMPONENTS || [], [identifier]);
};
var registerFormWidget = function (identifier, props) {
    var widgetName = formFieldPrefix + identifier;
    var existingProps = widgetProps.get(widgetName);
    if (existingProps) {
        props.forEach(function (v, k) { return existingProps.set(k, v); });
    }
};
var PROP_TYPE;
(function (PROP_TYPE) {
    PROP_TYPE[PROP_TYPE["BOOLEAN"] = 1] = "BOOLEAN";
    PROP_TYPE[PROP_TYPE["NUMBER"] = 2] = "NUMBER";
    PROP_TYPE[PROP_TYPE["STRING"] = 3] = "STRING";
})(PROP_TYPE || (PROP_TYPE = {}));
var PROP_STRING = { type: PROP_TYPE.STRING };
var PROP_NUMBER = { type: PROP_TYPE.NUMBER };
var PROP_BOOLEAN = { type: PROP_TYPE.BOOLEAN };
var PROP_ANY = { arrayType: true };

var registerProps = function () {
    register('wm-container', new Map([
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['content', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

/**
 * Widget Registry
 */
var registryById = new Map();
/**
 * returns the widget by the given widgetId
 */
var getById = function (widgetId) { return registryById.get(widgetId); };
/**
 * Registers the Widget with the given id and name
 * Makes the Widget available with the viewParent
 * returns unRegister method
 */
var register$1 = function (widget, viewParent, widgetId, name) {
    var registered = false;
    registryById.set(widgetId, widget);
    if (name) {
        if (isDefined(viewParent.Widgets)) {
            registered = true;
            viewParent.Widgets[name] = widget;
        }
    }
    // Unregister method.
    return function () {
        registryById.delete(widgetId);
        if (registered) {
            viewParent.Widgets[name] = undefined;
        }
    };
};
/**
 * Deregisters the oldname in widgets registry and sets new name
 */
var renameWidget = function (viewParent, widget, nv, ov) {
    if (!isDefined(viewParent.Widgets)) {
        return;
    }
    if (ov) {
        viewParent.Widgets[ov] = undefined;
    }
    if (nv) {
        viewParent.Widgets[nv] = widget;
    }
};

var widgetIdGenerator = new IDGenerator('widget-id');

var EVENTS_MAP = new Map([
    // ['enterkeypress', 'keypress.enter']
    ['tap', 'click']
]);
// TODO: Implement touch events for the mobile
var DISPLAY_TYPE = {
    BLOCK: 'block',
    INLINE_BLOCK: 'inline-block',
    INLINE: 'inline'
};
// set of boolean attrs
var BOOLEAN_ATTRS = new Set([
    'readonly', 'autofocus', 'disabled', 'startchecked', 'multiple',
    'selected', 'required', 'controls', 'autoplay', 'loop', 'muted', 'show'
]);
/**
 * Returns true if the provided key is a boolean attribute
 * @param {string} key
 * @returns {boolean}
 */
var isBooleanAttr = function (key) { return BOOLEAN_ATTRS.has(key); };
var DIMENSION_PROPS = new Set([
    'width',
    'height',
    'iconheight',
    'iconwidth',
    'popoverwidth',
    'popoverheight',
    'imagewidth',
    'imageheight'
]);
var isDimensionProp = function (key) { return DIMENSION_PROPS.has(key); };
var DEBOUNCE_TIMES = {
    PAGINATION_DEBOUNCE_TIME: 250
};

/**
 * Returns the parsed value based on the provided type
 * if the type is PROP_TYPE.NUMBER returns a number/NaN
 * if the type is PROP_BOOLEAN returns true/false
 * else returns the same value without any type conversion
 *
 * @param {string} key
 * @param value
 * @param {PROP_TYPE} type
 * @returns {any}
 */
var parseValue = function (key, value, type) {
    if (type === PROP_TYPE.BOOLEAN) {
        return toBoolean(value, isBooleanAttr(key) && key);
    }
    if (type === PROP_TYPE.NUMBER) {
        return +value;
    }
    return value;
};
var ɵ0$2 = parseValue;
/**
 * Whenever a property on a component changes through a proxy this method will be triggered
 * If the new value is not from a watch, the existing watch on that particular property will be removed
 * This method invokes the defaultPropertyChange handler where the common widget properties like name, class are handled
 * Notifies the component about the style/property change
 */
var globalPropertyChangeHandler = function (component, key, nv) {
    var widgetId = component.widgetId;
    var ov = component[key];
    // if the change is not from the bound watch, remove the existing watch
    if (!isChangeFromWatch()) {
        $unwatch(getWatchIdentifier(widgetId, key));
    }
    resetChangeFromWatch();
    var widgetProps = getWidgetPropsByType(component.getWidgetSubType());
    var propInfo = widgetProps.get(key);
    if (propInfo) {
        var type = propInfo.type;
        if (type) {
            nv = parseValue(key, nv, type);
        }
    }
    // Set the value in the component and trigger appDigest when there is a change in the value
    if (nv !== ov || isObject(nv) || isObject(ov)) {
        if (isDimensionProp(key)) {
            nv = toDimension(nv);
        }
        else if (_.startsWith(nv, 'resources/')) {
            var ref = component;
            if (ref._parentPrefab_ === undefined) {
                ref._parentPrefab_ = component.$element.closest('[prefabname][prefabname!="__self__"]').attr('prefabname') || '';
            }
            if (ref._parentPrefab_) {
                nv = "./app/prefabs/" + ref._parentPrefab_ + "/" + nv;
            }
        }
        component[key] = nv;
        if (isStyle(key)) {
            component.notifyStyleChange(key, nv, ov);
        }
        else {
            if (key === 'conditionalclass') {
                nv = getConditionalClasses(nv, ov);
            }
            if (propInfo) {
                component.notifyPropertyChange(key, nv, ov);
            }
        }
        $appDigest();
    }
};

/**
 * Proxy Provider - Creates a JS proxy for the given object
 */
var ɵ0$3 = function (target, key, value) {
    globalPropertyChangeHandler(target, key, value);
    return true;
}, ɵ1$1 = function (target, key) {
    var v = target[key];
    if (_.isFunction(v)) { // bind the proper context for the methods
        return v.bind(target);
    }
    return v;
};
/**
 *  proxy handler for the components
 */
var proxyHandler = {
    set: ɵ0$3,
    get: ɵ1$1
};
var $RAF = window.requestAnimationFrame;
var $RAFQueue = [];
var invokeLater = function (fn) {
    if (!$RAFQueue.length) {
        $RAF(function () {
            $RAFQueue.forEach(function (f) { return f(); });
            $RAFQueue.length = 0;
        });
    }
    $RAFQueue.push(fn);
};
var ɵ2$1 = invokeLater;
var WidgetProxyProvider = /** @class */ (function () {
    function WidgetProxyProvider() {
    }
    WidgetProxyProvider.create = function (instance, widgetSubType, propsByWidgetSubType) {
        // If the native Proxy is supported
        if (window.Proxy) {
            return new Proxy(instance, proxyHandler);
        }
        else {
            // If the native Proxy is not supported, IE11
            var widget_1 = Object.create(instance);
            // bind proper context for the methods
            invokeLater(function () {
                for (var key in instance) {
                    if (_.isFunction(instance[key]) && key !== 'constructor' && key !== 'super' && !_.startsWith(key, 'ng')) {
                        instance[key] = instance[key].bind(instance);
                    }
                }
            });
            // define setters and getters for styles
            Object.keys(propNameCSSKeyMap)
                .forEach(function (key) {
                Object.defineProperty(widget_1, key, {
                    get: function () { return instance[key]; },
                    set: function (nv) { return globalPropertyChangeHandler(instance, key, nv); }
                });
            });
            // define the setters and getters for Props
            if (propsByWidgetSubType) {
                propsByWidgetSubType
                    .forEach(function (value, key) {
                    Object.defineProperty(widget_1, key, {
                        get: function () { return instance[key]; },
                        set: function (nv) { return globalPropertyChangeHandler(instance, key, nv); }
                    });
                });
            }
            return widget_1;
        }
    };
    return WidgetProxyProvider;
}());

// Gets list of classes to add and remove and applies on the $el
var updateClasses = function (toAdd, toRemove, el) {
    if (toRemove && toRemove.length) {
        removeClass(el, _.join(toRemove, ' '));
    }
    if (toAdd && toAdd.length) {
        addClass(el, _.join(toAdd, ' '));
    }
};
var ɵ0$4 = updateClasses;
// To add and remove styles on the $el
var updateStyles = function (nv, ov, el) {
    if (ov && _.isObject(ov)) {
        var keys = Object.keys(ov || {});
        keys.forEach(function (key) {
            setCSS(el, key, '');
        });
    }
    if (nv && _.isObject(nv)) {
        setCSSFromObj(el, nv);
    }
};
var ɵ1$2 = updateStyles;
var BaseComponent = /** @class */ (function () {
    function BaseComponent(inj, config, initPromise // Promise on which the initialization has to wait
    ) {
        var _this = this;
        this.inj = inj;
        this.isAttached = false;
        /**
         * Style change subject and observable
         */
        this.styleChange = new ReplaySubject();
        /**
         * Property change subject and observable
         */
        this.propertyChange = new Subject();
        /**
         * On Ready State change subject and observable
         */
        this.readyState = new Subject();
        /**
         * Component destroy subject and observable
         */
        this.destroy = new Subject();
        /**
         * Map of event handler callbacks
         */
        this.eventHandlers = new Map();
        /**
         * Holds the event registration functions.
         * these functions needs to be executed after onViewInit
         */
        this.toBeSetupEventsQueue = [];
        this.__cloneable__ = false;
        this.$attrs = new Map();
        this.isMuted = false;
        var elementRef = inj.get(ElementRef);
        this.nativeElement = elementRef.nativeElement;
        this.widgetType = config.widgetType;
        this.widgetSubType = config.widgetSubType || config.widgetType;
        this.viewParent = inj.view.component;
        this.displayType = config.displayType || DISPLAY_TYPE.BLOCK;
        this.context = inj.view.context;
        this.widget = this.createProxy();
        this.eventManager = inj.get(EventManager);
        this.nativeElement.widget = this.widget;
        this.appLocale = inj.get(App).appLocale || {};
        this.initContext();
        if (config.hostClass) {
            addClass(this.nativeElement, config.hostClass);
        }
        this.widgetId = this.generateWidgetId();
        setAttr(this.nativeElement, 'widget-id', this.widgetId, true);
        // register default property change handler and style change handler
        this.registerStyleChangeListener(this.onStyleChange, this);
        this.registerPropertyChangeListener(this.onPropertyChange, this);
        // if the initPromise is provided, wait till the promise is resolved to proceed with the widget initialization
        if (!initPromise) {
            this.initWidget();
        }
        else {
            this.delayedInit = true;
            initPromise.then(function () {
                _this.initWidget();
                _this.setInitProps();
            });
        }
    }
    Object.defineProperty(BaseComponent.prototype, "$element", {
        /**
         * jQuery nativeElement reference of the component root
         */
        get: function () {
            return $(this.nativeElement);
        },
        enumerable: true,
        configurable: true
    });
    BaseComponent.prototype.getNativeElement = function () {
        return this.nativeElement;
    };
    BaseComponent.prototype.getWidgetType = function () {
        return this.widgetType;
    };
    BaseComponent.prototype.getWidgetSubType = function () {
        return this.widgetSubType;
    };
    BaseComponent.prototype.getWidget = function () {
        return this.widget;
    };
    BaseComponent.prototype.setProperty = function (propName, propValue) {
        this.widget[propName] = propValue;
    };
    BaseComponent.prototype.getViewParent = function () {
        return this.viewParent;
    };
    BaseComponent.prototype.notifyStyleChange = function (key, nv, ov) {
        this.styleChange.next({ key: key, nv: nv, ov: ov });
    };
    BaseComponent.prototype.notifyPropertyChange = function (key, nv, ov) {
        this.propertyChange.next({ key: key, nv: nv, ov: ov });
    };
    BaseComponent.prototype.registerStyleChangeListener = function (fn, ctx) {
        if (ctx) {
            fn = fn.bind(ctx);
        }
        this.styleChange.subscribe(function (_a) {
            var key = _a.key, nv = _a.nv, ov = _a.ov;
            return fn(key, nv, ov);
        });
    };
    BaseComponent.prototype.registerReadyStateListener = function (fn, ctx) {
        if (ctx) {
            fn = fn.bind(ctx);
        }
        if (this.readyState.isStopped) {
            fn();
            return;
        }
        this.readyState.subscribe(function () { return fn(); });
    };
    BaseComponent.prototype.registerPropertyChangeListener = function (fn, ctx) {
        if (ctx) {
            fn = fn.bind(ctx);
        }
        this.propertyChange.subscribe(function (_a) {
            var key = _a.key, nv = _a.nv, ov = _a.ov;
            return fn(key, nv, ov);
        });
    };
    BaseComponent.prototype.registerDestroyListener = function (fn, ctx) {
        if (ctx) {
            fn = fn.bind(ctx);
        }
        this.destroy.subscribe(function () { }, function () { }, function () { return fn(); });
    };
    BaseComponent.prototype.getDisplayType = function () {
        return this.displayType;
    };
    BaseComponent.prototype.createProxy = function () {
        return WidgetProxyProvider.create(this, this.widgetSubType, getWidgetPropsByType(this.widgetSubType));
    };
    BaseComponent.prototype.initContext = function () {
        var context = this.inj.view.context;
        var parentContexts = this.inj.get(Context, {});
        // assign the context property accordingly
        if (this.viewParent !== context) {
            this.context = context;
        }
        else {
            this.context = {};
        }
        if (parentContexts) {
            this.context = Object.assign.apply(Object, __spread([{}], parentContexts, [this.context]));
        }
    };
    /**
     * set the value on the proxy object ie, widget
     * setting the property on the proxy will invoke the change listeners
     * @param {string} key
     * @param value
     */
    BaseComponent.prototype.setWidgetProperty = function (key, value) {
        this.widget[key] = value;
    };
    BaseComponent.prototype.getAttr = function (attrName) {
        return this.$attrs.get(attrName);
    };
    /**
     * returns app instance
     * @returns {App}
     */
    BaseComponent.prototype.getAppInstance = function () {
        return this.inj.get(App);
    };
    /**
     * Generates a unique id
     * Default pattern is `widget-id-${id}`
     * Components can override this method to generate a different id eg, bar-chart-1
     */
    BaseComponent.prototype.generateWidgetId = function () {
        return widgetIdGenerator.nextUid();
    };
    /**
     * Handles the common functionality across the components
     * eg,
     *  1. value of the class property will be applied on the host element
     *  2. based on the value of show property component is shown/hidden
     *
     * @param {string} key
     * @param nv
     * @param ov
     */
    BaseComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'show') {
            this.nativeElement.hidden = !nv;
        }
        else if (key === 'hint') {
            setAttr(this.nativeElement, 'title', nv);
        }
        else if (key === 'class') {
            switchClass(this.nativeElement, nv, ov);
        }
        else if (key === 'name' || key === 'tabindex') {
            setAttr(this.nativeElement, key, nv);
            if (key === 'name' && nv) {
                renameWidget(this.viewParent, this.widget, nv, ov);
            }
        }
        else if (key === 'conditionalclass') {
            // update classes if old and nv value are different
            updateClasses(nv.toAdd, nv.toRemove, this.nativeElement);
        }
        else if (key === 'autoplay') {
            var tagName = this.widgetType === 'wm-audio' ? 'audio' : 'video';
            // Trigger media(audio/video) element load method after changing autoplay property
            this.nativeElement.querySelector(tagName).load();
        }
        else if (key === 'conditionalstyle') {
            // update styles if old and nv value are different
            updateStyles(nv, ov, this.nativeElement);
        }
        else if (key === 'animation') {
            // update styles if old and nv value are different
            addClass(this.nativeElement, "animated " + nv);
        }
        else if (key === 'disabled') {
            // WMS-19321: In IE, widget is in disabled state when disabled="false" attribute is present
            (nv === true) ? setAttr(this.nativeElement, 'disabled', 'true', true) : removeAttr(this.nativeElement, 'disabled', true);
        }
    };
    /**
     * Default style change handler
     */
    BaseComponent.prototype.onStyleChange = function (k, nv, ov) { };
    /**
     * Register the widget with the widgetRegistry
     */
    BaseComponent.prototype.registerWidget = function (widgetName) {
        this.registerDestroyListener(register$1(this.widget, this.viewParent, this.widgetId, widgetName));
    };
    /**
     * override the
     */
    BaseComponent.prototype.getMappedEventName = function (eventName) {
        return EVENTS_MAP.get(eventName) || eventName;
    };
    /**
     * invoke the event handler
     * Components can override this method to execute custom logic before invoking the user callback
     */
    BaseComponent.prototype.handleEvent = function (node, eventName, eventCallback, locals, meta) {
        this.eventManager.addEventListener(node, eventName, function (e) {
            locals.$event = e;
            if (meta === 'delayed') {
                setTimeout(function () { return eventCallback(); }, 150);
            }
            else {
                return eventCallback();
            }
        });
    };
    /**
     * parse the event expression and save reference to the function inside eventHandlers map
     * If the component provides a override for an event through @Event decorator invoke that
     * else invoke the resolved function
     *
     * @param {string} eventName
     * @param {string} expr
     */
    BaseComponent.prototype.processEventAttr = function (eventName, expr, meta) {
        var _this = this;
        var fn = $parseEvent(expr);
        var locals = this.context;
        locals.widget = this.widget;
        var boundFn = fn.bind(undefined, this.viewParent, locals);
        var eventCallback = function () {
            var boundFnVal;
            $invokeWatchers(true);
            try {
                locals.$event = locals.$event || new window.CustomEvent(eventName);
                // If the event is bound directly to the variable then we need to internally handle
                // the promise returned by the variable call.
                boundFnVal = boundFn();
                if (boundFnVal instanceof Promise) {
                    boundFnVal.then(function (response) { return response; }, function (err) { return err; });
                }
                else {
                    return boundFnVal;
                }
            }
            catch (e) {
                console.error(e);
            }
        };
        this.eventHandlers.set(this.getMappedEventName(eventName), { callback: eventCallback, locals: locals });
        // prepend eventName with on and convert it to camelcase.
        // eg, "click" ---> onClick
        var onEventName = _.camelCase("on-" + eventName);
        // save the eventCallback in widgetScope.
        this[onEventName] = eventCallback;
        // events needs to be setup after viewInit
        this.toBeSetupEventsQueue.push(function () {
            _this.handleEvent(_this.nativeElement, _this.getMappedEventName(eventName), eventCallback, locals, meta);
        });
    };
    /**
     * Process the bound property
     * Register a watch on the bound expression
     */
    BaseComponent.prototype.processBindAttr = function (propName, expr) {
        var _this = this;
        this.initState.delete(propName);
        this.registerDestroyListener($watch(expr, this.viewParent, this.context, function (nv) { return _this.widget[propName] = nv; }, getWatchIdentifier(this.widgetId, propName), propName === 'datasource', this.widgetProps.get(propName), function () { return _this.isMuted; }));
    };
    /**
     * Remove watch on the bound property
     */
    BaseComponent.prototype.removePropertyBinding = function (propName) {
        $unwatch(getWatchIdentifier(this.widgetId, propName));
    };
    /**
     * invoke the event callback method
     * @param {string} eventName
     * @param extraLocals
     */
    BaseComponent.prototype.invokeEventCallback = function (eventName, extraLocals) {
        var callbackInfo = this.eventHandlers.get(eventName);
        if (callbackInfo) {
            var fn = callbackInfo.callback;
            var locals = callbackInfo.locals || {};
            if (fn) {
                return fn(Object.assign(locals, extraLocals));
            }
        }
    };
    /**
     * Process the attribute
     * If the attribute is an event expression, generate a functional representation of the expression
     *      and keep in eventHandlers
     * If the attribute is a bound expression, register a watch on the expression
     */
    BaseComponent.prototype.processAttr = function (attrName, attrValue) {
        var _a = attrName.split('.'), propName = _a[0], type = _a[1], meta = _a[2], length = _a.length;
        if (type === 'bind') {
            // if the show property is bound, set the initial value to false
            if (propName === 'show') {
                this.nativeElement.hidden = true;
            }
            this.processBindAttr(propName, attrValue);
        }
        else if (type === 'event') {
            this.processEventAttr(propName, attrValue, meta);
        }
        else if (length === 1) {
            // remove class and name attributes. Component will set them on the proper node
            if (attrName === 'class') {
                removeClass(this.nativeElement, attrValue);
            }
            else if (attrName === 'tabindex' || attrName === 'name') {
                removeAttr(this.nativeElement, attrName);
            }
            this.initState.set(propName, attrValue);
        }
        else {
            // custom attributes provided on elDef;
        }
    };
    /**
     * Process the attributes
     */
    BaseComponent.prototype.processAttrs = function () {
        var e_1, _a;
        var elDef = this.inj.elDef;
        try {
            for (var _b = __values(elDef.element.attrs), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 3), attrName = _d[1], attrValue = _d[2];
                this.$attrs.set(attrName, attrValue);
                this.processAttr(attrName, attrValue);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    /**
     * Update the initState with the default property values and the values provided in the markup
     * Process the attributes
     * Register the widget
     */
    BaseComponent.prototype.initWidget = function () {
        var _this = this;
        this.initState = new Map();
        // get the widget properties
        var widgetProps = getWidgetPropsByType(this.widgetSubType);
        widgetProps.forEach(function (v, k) {
            if (isDefined(v.value)) {
                _this.initState.set(k, v.value);
            }
        });
        this.widgetProps = widgetProps;
        this.processAttrs();
        this.registerWidget(this.initState.get('name'));
    };
    /**
     * Update the default properties and the properties provided in the markup in component
     * Invoking this method will result in invocation of propertyChange handlers on the component for the first time
     */
    BaseComponent.prototype.setInitProps = function () {
        var _this = this;
        if (this.initState.get('name')) {
            this.widget.name = this.initState.get('name');
        }
        this.initState.forEach(function (v, k) {
            // name is already set, ignore name
            // if the key is part of to be ignored attributes list do not set it on the component instance
            if ((_this.widgetProps.get(k) || isStyle(k)) && k !== 'name') {
                _this.widget[k] = v;
            }
        });
        this.initState.clear();
        this.initState = undefined;
        this.readyState.next();
        this.readyState.complete();
    };
    /**
     * Returns true, if a listener registered for the given event on this widget markup.
     * @param eventName
     * @returns {boolean}
     */
    BaseComponent.prototype.hasEventCallback = function (eventName) {
        return this.eventHandlers.has(eventName);
    };
    /**
     * Sets the focus on the widget
     */
    BaseComponent.prototype.focus = function () {
        /**
         * Check for the nodes having focus-target attribute inside the element
         * If found, focus the first node (eg, date widget)
         * else, focus the element (eg, text widget)
         */
        var $target = this.$element[0].querySelector('[focus-target]');
        if (!$target) {
            $target = this.$element[0];
        }
        $target.focus();
    };
    // Defining the execute method on BaseComponent. If dataset is binded to widgets ouptut then datasource.execute will be defined
    BaseComponent.prototype.execute = function (operation, options) {
    };
    BaseComponent.prototype.mute = function () {
        this.isMuted = true;
    };
    BaseComponent.prototype.unmute = function () {
        this.isMuted = false;
    };
    /**
     * After the host page is attached from cache, this function is called.
     */
    BaseComponent.prototype.ngOnAttach = function () {
        this.isAttached = true;
    };
    /**
     * Before the host page is detached from dom and stored in cache,  this function is called.
     */
    BaseComponent.prototype.ngOnDetach = function () {
        this.isAttached = false;
    };
    /**
     * nativeElement will be available by this time
     * if the delayInit is false, properties meta will be available by this time
     * Invoke the setInitProps if the delayInit is false
     */
    BaseComponent.prototype.ngOnInit = function () {
        if (!this.delayedInit) {
            this.setInitProps();
        }
    };
    /**
     * Register the events
     */
    BaseComponent.prototype.ngAfterViewInit = function () {
        var e_2, _a;
        if (this.toBeSetupEventsQueue.length) {
            try {
                for (var _b = __values(this.toBeSetupEventsQueue), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var fn = _c.value;
                    fn();
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
        this.toBeSetupEventsQueue.length = 0;
        this.isAttached = true;
    };
    BaseComponent.prototype.ngAfterContentInit = function () { };
    BaseComponent.prototype.ngOnDestroy = function () {
        this.isDestroyed = true;
        this.widget = Object.create(null);
        this.styleChange.complete();
        this.propertyChange.complete();
        this.destroy.complete();
        this.isAttached = false;
    };
    return BaseComponent;
}());

var StylableComponent = /** @class */ (function (_super) {
    __extends(StylableComponent, _super);
    function StylableComponent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return StylableComponent;
}(BaseComponent));

var RedrawableDirective = /** @class */ (function () {
    function RedrawableDirective(widget) {
        this.redraw = function () { return widget.redraw(); };
    }
    RedrawableDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [WidgetRef,] }] }
    ]; };
    RedrawableDirective = __decorate([
        Directive({
            selector: '[redrawable]'
        }),
        __param(0, Self()), __param(0, Inject(WidgetRef)),
        __metadata("design:paramtypes", [Object])
    ], RedrawableDirective);
    return RedrawableDirective;
}());

var BaseContainerComponent = /** @class */ (function (_super) {
    __extends(BaseContainerComponent, _super);
    function BaseContainerComponent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BaseContainerComponent.prototype.updateRedrawableComponents = function (widgets) {
        var _this = this;
        _.pickBy(widgets, function (widget) {
            // check for redrawable widget and whether it is already exist in reDrawableComponents query list or not
            if (widget.nativeElement && widget.nativeElement.hasAttribute('redrawable') && !_.includes(_this.reDrawableComponents._results, widget)) {
                _this.reDrawableComponents._results.push(widget);
            }
            if (widget.Widgets) {
                _this.updateRedrawableComponents(widget.Widgets);
            }
        });
    };
    BaseContainerComponent.prototype.redrawChildren = function () {
        var _this = this;
        // If container is bound to partial content, manually updating reDrawableComponents query list.
        if (this.content && this.Widgets) {
            this.updateRedrawableComponents(this.Widgets);
        }
        setTimeout(function () {
            if (_this.reDrawableComponents) {
                _this.reDrawableComponents.forEach(function (c) { return c.redraw(); });
            }
        }, 100);
    };
    BaseContainerComponent.prototype.onPropertyChange = function (key, nv, ov) {
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
        if (key === 'show') {
            if (nv) {
                this.redrawChildren();
            }
        }
    };
    __decorate([
        ContentChildren(RedrawableDirective, { descendants: true }),
        __metadata("design:type", Object)
    ], BaseContainerComponent.prototype, "reDrawableComponents", void 0);
    BaseContainerComponent = __decorate([
        Directive()
    ], BaseContainerComponent);
    return BaseContainerComponent;
}(StylableComponent));

var DEFAULT_CLS = 'app-container';
var WIDGET_CONFIG = {
    widgetType: 'wm-container',
    hostClass: DEFAULT_CLS
};
var ContainerDirective = /** @class */ (function (_super) {
    __extends(ContainerDirective, _super);
    function ContainerDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        addClass(_this.nativeElement, DEFAULT_CLS);
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    ContainerDirective_1 = ContainerDirective;
    var ContainerDirective_1;
    ContainerDirective.initializeProps = registerProps();
    ContainerDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    ContainerDirective = ContainerDirective_1 = __decorate([
        Directive({
            selector: '[wmContainer]',
            providers: [
                provideAsWidgetRef(ContainerDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], ContainerDirective);
    return ContainerDirective;
}(BaseContainerComponent));

var ImagePipe = /** @class */ (function () {
    function ImagePipe() {
    }
    ImagePipe.prototype.transform = function (url, encode, defaultImageUrl) {
        return transformFileURI(getImageUrl(url, encode, defaultImageUrl));
    };
    ImagePipe = __decorate([
        Pipe({
            name: 'image'
        })
    ], ImagePipe);
    return ImagePipe;
}());

var registerProps$1 = function () {
    register('wm-item-template', new Map([
        ['content', PROP_STRING]
    ]));
};

/**
 * This directive is used to set custom template partial for an item and maintains separate scope for each item.
 * This takes two inputs
 *      1) template partial name as wmItemTemplate
 *      2) item object as userComponentParams
 * Example: Providing a custom template for the search/autocomplete widget (customized UI in the dropdown for each item)
 * Add this directive to the li tag and provide item object and partial name like below. And also provide partialContainer and partial-container-target to render the partial.
 * <li #liElements *ngFor="let match of matches" [wmItemTemplate]="content" [userComponentParams]="match.item" partialContainer>
 *      <a partial-container-target></a>
 * </li>
 */
var WIDGET_CONFIG$1 = { widgetType: 'wm-item-template' };
var ItemTemplateDirective = /** @class */ (function (_super) {
    __extends(ItemTemplateDirective, _super);
    function ItemTemplateDirective(inj, elRef, app) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        _this.app = app;
        _this.nativeElement = elRef.nativeElement;
        _this.context = inj.view.context;
        return _this;
    }
    ItemTemplateDirective_1 = ItemTemplateDirective;
    Object.defineProperty(ItemTemplateDirective.prototype, "$index", {
        get: function () {
            return this.context.index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ItemTemplateDirective.prototype, "wmItemTemplate", {
        set: function (value) {
            this.widget.content = value;
        },
        enumerable: true,
        configurable: true
    });
    ItemTemplateDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.partialParams.item = this.userComponentParams.dataObject;
    };
    var ItemTemplateDirective_1;
    ItemTemplateDirective.initializeProps = registerProps$1();
    ItemTemplateDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: ElementRef },
        { type: App }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ItemTemplateDirective.prototype, "userComponentParams", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], ItemTemplateDirective.prototype, "wmItemTemplate", null);
    ItemTemplateDirective = ItemTemplateDirective_1 = __decorate([
        Directive({
            selector: '[wmItemTemplate]',
            exportAs: 'itemTemplateRef',
            providers: [
                provideAsWidgetRef(ItemTemplateDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, ElementRef, App])
    ], ItemTemplateDirective);
    return ItemTemplateDirective;
}(StylableComponent));

var LazyLoadDirective = /** @class */ (function () {
    function LazyLoadDirective(inj, templateRef, viewContainer) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.viewParent = inj.view.component;
        this.context = inj.view.context;
    }
    Object.defineProperty(LazyLoadDirective.prototype, "lazyLoad", {
        set: function (expr) {
            var _this = this;
            this.unSubscribeFn = $watch(expr, this.viewParent, this.context, function (val) {
                if (!_this.embeddedView && val) {
                    _this.embeddedView = _this.viewContainer.createEmbeddedView(_this.templateRef, _this.context);
                    _this.unSubscribeFn();
                }
            });
        },
        enumerable: true,
        configurable: true
    });
    LazyLoadDirective.prototype.ngOnDestroy = function () {
        this.unSubscribeFn();
    };
    LazyLoadDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: TemplateRef },
        { type: ViewContainerRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], LazyLoadDirective.prototype, "lazyLoad", null);
    LazyLoadDirective = __decorate([
        Directive({
            selector: '[lazyLoad]'
        }),
        __metadata("design:paramtypes", [Injector,
            TemplateRef,
            ViewContainerRef])
    ], LazyLoadDirective);
    return LazyLoadDirective;
}());

var registerProps$2 = function () {
    register('wm-message', new Map([
        ['animation', PROP_STRING],
        ['caption', __assign({ value: 'Message' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['hideclose', __assign({ value: false }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['type', __assign({ value: 'success' }, PROP_STRING)],
    ]));
};

var DEFAULT_CLS$1 = 'alert app-message';
var WIDGET_CONFIG$2 = { widgetType: 'wm-message', hostClass: DEFAULT_CLS$1 };
var MessageComponent = /** @class */ (function (_super) {
    __extends(MessageComponent, _super);
    function MessageComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$2) || this;
        _this.messageClass = '';
        _this.messageIconClass = '';
        _this.type = '';
        styler(_this.nativeElement, _this);
        return _this;
    }
    MessageComponent_1 = MessageComponent;
    MessageComponent.prototype.showMessage = function (caption, type) {
        if (caption) {
            this.caption = caption;
        }
        if (type) {
            this.setWidgetProperty('type', type);
        }
        this.setWidgetProperty('show', true);
    };
    MessageComponent.prototype.hideMessage = function () {
        this.setWidgetProperty('show', false);
    };
    MessageComponent.prototype.dismiss = function ($event) {
        this.hideMessage();
        this.invokeEventCallback('close', { $event: $event });
    };
    MessageComponent.prototype.onMessageTypeChange = function (nv) {
        var msgCls, msgIconCls;
        switch (nv) {
            case 'success':
                msgCls = 'alert-success';
                msgIconCls = 'wi wi-done';
                break;
            case 'error':
                msgCls = 'alert-danger';
                msgIconCls = 'wi wi-cancel';
                break;
            case 'warn': /*To support old projects with type as "warn"*/
            case 'warning':
                msgCls = 'alert-warning';
                msgIconCls = 'wi wi-bell';
                break;
            case 'info':
                msgCls = 'alert-info';
                msgIconCls = 'wi wi-info';
                break;
            case 'loading':
                msgCls = 'alert-info alert-loading';
                msgIconCls = 'fa fa-spinner fa-spin';
                break;
        }
        switchClass(this.nativeElement, msgCls, this.messageClass);
        this.messageClass = msgCls;
        this.messageIconClass = msgIconCls;
    };
    MessageComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'type') {
            this.onMessageTypeChange(nv);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var MessageComponent_1;
    MessageComponent.initializeProps = registerProps$2();
    MessageComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    MessageComponent = MessageComponent_1 = __decorate([
        Component({
            selector: '[wmMessage]',
            template: "<i [title]=\"type + 'Alert'\" class=\"icon {{type}} {{messageIconClass}}\"></i>\n<span [innerHtml]=\"caption | trustAs: 'html'\"></span>\n<button title=\"Close\" type=\"button\" class=\"btn-transparent close\" [hidden]=\"hideclose\" (click)=\"dismiss($event)\" aria-label=\"Close\">\n    <span aria-hidden=\"true\">&times;</span>\n</button>",
            providers: [
                provideAsWidgetRef(MessageComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], MessageComponent);
    return MessageComponent;
}(StylableComponent));

var registerProps$3 = function () {
    register('wm-partial', new Map([
        ['name', PROP_STRING]
    ]));
};

var DEFAULT_CLS$2 = 'app-partial clearfix';
var WIDGET_CONFIG$3 = { widgetType: 'wm-partial', hostClass: DEFAULT_CLS$2 };
var PartialDirective = /** @class */ (function (_super) {
    __extends(PartialDirective, _super);
    function PartialDirective(inj, viewport) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$3) || this;
        _this.viewport = viewport;
        _this.registerDestroyListener(_this.viewport.subscribe("resize" /* RESIZE */, function (data) { return _this.callback('resize', data); }));
        _this.registerDestroyListener(_this.viewport.subscribe("orientationchange" /* ORIENTATION_CHANGE */, function (data) { return _this.callback('orientationchange', data); }));
        return _this;
    }
    PartialDirective_1 = PartialDirective;
    PartialDirective.prototype.callback = function (eventName, locals) {
        locals = _.assign({ widget: this }, locals);
        this.invokeEventCallback(eventName, locals);
    };
    PartialDirective.prototype.ngOnAttach = function () {
        this.callback('attach');
    };
    PartialDirective.prototype.ngOnDetach = function () {
        this.callback('detach');
    };
    PartialDirective.prototype.ngOnDestroy = function () {
        this.callback('destroy');
    };
    var PartialDirective_1;
    PartialDirective.initializeProps = registerProps$3();
    PartialDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: Viewport }
    ]; };
    PartialDirective = PartialDirective_1 = __decorate([
        Directive({
            selector: '[wmPartial]',
            providers: [
                provideAsWidgetRef(PartialDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, Viewport])
    ], PartialDirective);
    return PartialDirective;
}(StylableComponent));

var PartialParamHandlerDirective = /** @class */ (function () {
    function PartialParamHandlerDirective(widgetRef) {
        this.widgetRef = widgetRef;
        this.widgetRef.partialParams = {};
        this.widgetRef.pageParams = this.widgetRef.partialParams;
        this.widgetRef.params$ = new Subject();
    }
    PartialParamHandlerDirective.prototype.registerParams = function (name, value, bindExpr, type) {
        var _this = this;
        this.widgetRef.partialParams[name] = value;
        if (!value && bindExpr) {
            this.widgetRef.registerDestroyListener($watch(bindExpr, this.widgetRef.getViewParent(), _.get(this.widgetRef, 'context'), function (nv) {
                _this.widgetRef.partialParams[name] = nv;
                // notify the partial container of the param changes
                _this.widgetRef.params$.next();
            }));
        }
        else {
            this.widgetRef.params$.next();
        }
    };
    PartialParamHandlerDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [WidgetRef,] }] }
    ]; };
    PartialParamHandlerDirective = __decorate([
        Directive({
            selector: '[partialContainer]',
        }),
        __param(0, Self()), __param(0, Inject(WidgetRef)),
        __metadata("design:paramtypes", [Object])
    ], PartialParamHandlerDirective);
    return PartialParamHandlerDirective;
}());
var PartialParamDirective = /** @class */ (function () {
    function PartialParamDirective(bindValue, type, partialParamsProvider) {
        this.bindValue = bindValue;
        this.type = type;
        this.partialParamsProvider = partialParamsProvider;
    }
    PartialParamDirective.prototype.ngOnInit = function () {
        this.partialParamsProvider.registerParams(this.name, this.value, this.bindValue, this.type);
    };
    PartialParamDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Attribute, args: ['value.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['type',] }] },
        { type: PartialParamHandlerDirective }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], PartialParamDirective.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], PartialParamDirective.prototype, "value", void 0);
    PartialParamDirective = __decorate([
        Directive({
            selector: '[wmParam]',
        }),
        __param(0, Attribute('value.bind')),
        __param(1, Attribute('type')),
        __metadata("design:paramtypes", [Object, Object, PartialParamHandlerDirective])
    ], PartialParamDirective);
    return PartialParamDirective;
}());

var PartialContainerDirective = /** @class */ (function () {
    function PartialContainerDirective(componentInstance, vcRef, elRef, inj, app, _content, resolver, partialRefProvider) {
        var _this = this;
        this.componentInstance = componentInstance;
        this.vcRef = vcRef;
        this.elRef = elRef;
        this.inj = inj;
        this.app = app;
        this.resolver = resolver;
        this.partialRefProvider = partialRefProvider;
        this.contentInitialized = false;
        this.renderPartial = _.debounce(this._renderPartial, 200);
        componentInstance.registerPropertyChangeListener(function (key, nv, ov) {
            if (key === 'content') {
                if (componentInstance.$lazyLoad) {
                    componentInstance.$lazyLoad = function () {
                        _this.renderPartial(nv);
                        componentInstance.$lazyLoad = noop$1;
                    };
                }
                else {
                    _this.renderPartial(nv);
                }
            }
        });
        var subscription = componentInstance.params$
            .pipe(filter(function () { return _this.contentInitialized; }), debounceTime(200))
            .subscribe(function () { return _this.renderPartial(componentInstance.content); });
        // reload the partial content on partial param change
        componentInstance.registerDestroyListener(function () { return subscription.unsubscribe(); });
    }
    Object.defineProperty(PartialContainerDirective.prototype, "name", {
        get: function () {
            return this.componentInstance.name;
        },
        enumerable: true,
        configurable: true
    });
    PartialContainerDirective.prototype._renderPartial = function (nv) {
        return __awaiter(this, void 0, void 0, function () {
            var componentFactory, instanceRef;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // destroy the existing partial
                        this.vcRef.clear();
                        // when the container-target is inside the component template, it can be queried after viewInit of the component.
                        $invokeWatchers(true);
                        return [4 /*yield*/, this.partialRefProvider.getComponentFactoryRef(nv, ComponentType.PARTIAL)];
                    case 1:
                        componentFactory = _a.sent();
                        if (componentFactory) {
                            instanceRef = this.vcRef.createComponent(componentFactory, 0, this.inj);
                            if (!this.$target) {
                                this.$target = this.elRef.nativeElement.querySelector('[partial-container-target]') || this.elRef.nativeElement;
                            }
                            this.$target.innerHTML = '';
                            this.$target.appendChild(instanceRef.location.nativeElement);
                            this.contentInitialized = true;
                            setTimeout(function () { return _this.onLoadSuccess(); }, 200);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    PartialContainerDirective.prototype.onLoadSuccess = function () {
        this.componentInstance.invokeEventCallback('load');
        this.app.notify('partialLoaded');
    };
    PartialContainerDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [WidgetRef,] }] },
        { type: ViewContainerRef },
        { type: ElementRef },
        { type: Injector },
        { type: App },
        { type: String, decorators: [{ type: Attribute, args: ['content',] }] },
        { type: ComponentFactoryResolver },
        { type: PartialRefProvider }
    ]; };
    PartialContainerDirective = __decorate([
        Directive({
            selector: '[partialContainer]'
        }),
        __param(0, Self()), __param(0, Inject(WidgetRef)),
        __param(5, Attribute('content')),
        __metadata("design:paramtypes", [Object, ViewContainerRef,
            ElementRef,
            Injector,
            App, String, ComponentFactoryResolver,
            PartialRefProvider])
    ], PartialContainerDirective);
    return PartialContainerDirective;
}());

var ShowInDeviceDirective = /** @class */ (function () {
    function ShowInDeviceDirective(elRef, showInDevice, widget) {
        var displayType = widget.getDisplayType();
        if (showInDevice) {
            showInDevice.split(',').forEach(function (deviceType) {
                addClass(elRef.nativeElement, "visible-" + deviceType + "-" + displayType);
            });
        }
    }
    ShowInDeviceDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: String, decorators: [{ type: Attribute, args: ['showInDevice',] }] },
        { type: BaseComponent, decorators: [{ type: Inject, args: [WidgetRef,] }] }
    ]; };
    ShowInDeviceDirective = __decorate([
        Directive({
            selector: '[showInDevice]'
        }),
        __param(1, Attribute('showInDevice')),
        __param(2, Inject(WidgetRef)),
        __metadata("design:paramtypes", [ElementRef, String, BaseComponent])
    ], ShowInDeviceDirective);
    return ShowInDeviceDirective;
}());

var SmoothScrollDirective = /** @class */ (function () {
    function SmoothScrollDirective(inj, elRef, app) {
        this._isEnabled = false;
        this._lastScrollY = -1;
        this._waitRefreshTill = -1;
        this.pendingIscrolls = [];
        this._$el = $(elRef.nativeElement);
        this.app = app;
    }
    SmoothScrollDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.cancelSubscription = this.app.subscribe('no-iscroll', function (el) {
            _this.pendingIscrolls.push(el);
        });
    };
    SmoothScrollDirective.prototype.ngDoCheck = function () {
        if (this._isEnabled) {
            if (!this._smoothScrollInstance) {
                this._smoothScrollInstance = this.applySmoothScroll();
            }
            else {
                this.refreshIScroll();
            }
        }
        else if (this._smoothScrollInstance && this._smoothScrollInstance.destroy) {
            this._smoothScrollInstance.destroy();
        }
    };
    SmoothScrollDirective.prototype.ngOnDestroy = function () {
        if (this._smoothScrollInstance && this._smoothScrollInstance.destroy) {
            this._smoothScrollInstance.destroy();
        }
        if (this.cancelSubscription) {
            this.cancelSubscription();
        }
    };
    Object.defineProperty(SmoothScrollDirective.prototype, "wmSmoothscroll", {
        set: function (val) {
            this._isEnabled = (val === true || val === 'true');
            if (this._isEnabled) {
                if (!this._smoothScrollInstance) {
                    this._smoothScrollInstance = this.applySmoothScroll();
                }
            }
            else {
                this.ngOnDestroy();
            }
        },
        enumerable: true,
        configurable: true
    });
    SmoothScrollDirective.prototype.applySmoothScroll = function ($events, activeEl) {
        var _this = this;
        if (!isMobileApp() || isKitkatDevice()) {
            return null;
        }
        // Set the fadeScrollbars to true only when content is scrollable inside the smoothscroll-container
        var scrollOptions = {
            scrollbars: true,
            preventDefault: false,
            momentum: true,
            bounce: false,
            mouseWheel: true,
            disablePointer: true,
            disableTouch: false,
            disableMouse: false // false to be usable with a mouse (desktop)
        }, el = this._$el[0];
        if (!el.children.length) {
            return null;
        }
        this._$el.addClass('smoothscroll-wrapper');
        /*  WMS-17904
        document.scrollIntoView overrides default iScroll scroll,
        so the fix is done to ensure, the scroll action from 'scrollIntoView'
        is captured & executed through iScroll APIs
        */
        el.onscroll = function (e) {
            if (e.target.iscroll && e.target.scrollTop) {
                e.target.iscroll.scrollTo(e.target.iscroll.x, e.target.iscroll.y - e.target.scrollTop);
                e.target.scrollTop = 0;
            }
        };
        if (activeEl && activeEl.tagName === 'INPUT') {
            activeEl.focus();
        }
        // Add fadeScrollbars options only when smoothscroll container is included, which means content is scrollable.
        if ($events) {
            scrollOptions['fadeScrollbars'] = true;
        }
        var iScroll = new IScroll(el, scrollOptions);
        if ($events) {
            // map all events on previous iscroll to the newly created iscroll.
            _.forEach($events, function (listeners, key) {
                _.forEach(listeners, function (l) {
                    iScroll.on(key, l);
                });
            });
            var scope_1 = this;
            iScroll.on('scrollStart', function () {
                this._scrolling = true;
                //will notify the components which want to subscribe upon scrolling.
                if (isMobileApp()) {
                    scope_1.app.notify('iscroll-start', {});
                }
            });
            iScroll.on('scrollEnd', function () {
                this._scrolling = false;
            });
            iScroll.refresh();
        }
        this.app.notify('iscroll-start', {});
        // refresh the indicators.
        iScroll.indicatorRefresh = function () {
            var indicators = _this._$el[0].iscroll.indicators;
            var i;
            if (indicators.length) {
                for (i = 0; i < indicators.length; i++) {
                    indicators[i].refresh();
                }
            }
        };
        this._$el[0].iscroll = iScroll;
        _.forEach(this.pendingIscrolls, function (_el, index) {
            if (_el.isSameNode(_this._$el[0])) {
                _this.app.notify('iscroll-update', { el: _el });
                _this.pendingIscrolls.splice(index, 1);
                return;
            }
        });
        this.app.notify('iscroll-update', {});
        return {
            iScroll: iScroll,
            destroy: function () {
                iScroll.destroy();
                $(iScroll.scroller).css({
                    'transition-timing-function': '',
                    'transition-duration': '',
                    'transform': ''
                });
                iScroll = null;
                delete el.iscroll;
            }
        };
    };
    /*
     * When element has scroll (i.e. scrollHeight > clientHeight), a div with smoothscroll-container class will be added.
     * new iScroll will be initialised on the element after the div addition, by removing the existing iscroll on the element.
     * This div will have no height, so the elements inside this div will inherit this height, i.e. no height,
     * Scenario: tabs with 100% height, as it covers the pageContent with no scroll, this div will not be added.
     * TODO: Scenario: tabs with 100% height and add others widgets after/before, as it has scroll, this div will be added.
     *          But tabs having 100% height will not be honoured as div is having no height.
     */
    SmoothScrollDirective.prototype.refreshIScroll = function () {
        var iScroll = this._smoothScrollInstance.iScroll;
        var waitTime = 500;
        if (iScroll._scrolling || this._waitRefreshTill > Date.now()) {
            return;
        }
        // Check for scrollable content and if smoothscroll-container div is already added.
        if (iScroll.wrapper
            && !_.includes(iScroll.wrapper.children[0].classList, 'smoothscroll-container')
            && iScroll.wrapper.scrollHeight > iScroll.wrapper.clientHeight) {
            var cloneEvents = iScroll._events;
            var prevActiveEl = document.activeElement;
            // Adds the smoothscroll container div wrapper only when element has scrollable content.
            $(iScroll.wrapper.children).wrapAll('<div class="smoothscroll-container"></div>');
            this._smoothScrollInstance.destroy();
            // create new iscroll instance on the element
            this._smoothScrollInstance = this.applySmoothScroll(cloneEvents, prevActiveEl);
        }
        if (this._lastScrollY !== this._$el[0].iscroll.maxScrollY) {
            refreshIscrolls(this._smoothScrollInstance.iScroll);
            this._lastScrollY = this._$el[0].iscroll.maxScrollY;
        }
        else {
            this._smoothScrollInstance.iScroll.refresh();
        }
        this._waitRefreshTill = Date.now() + waitTime;
    };
    SmoothScrollDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: ElementRef },
        { type: App }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SmoothScrollDirective.prototype, "wmSmoothscroll", null);
    SmoothScrollDirective = __decorate([
        Directive({
            selector: '[wmSmoothscroll]'
        }),
        __metadata("design:paramtypes", [Injector, ElementRef, App])
    ], SmoothScrollDirective);
    return SmoothScrollDirective;
}());
/**
 * Refreshes the given iScorll or all iScrolls in the page.
 * @param iScroll
 */
var refreshIscrolls = function (iScroll) {
    var scrollContainer = !iScroll && $('.smoothscroll-container');
    // Fix for issue: keyboard hides the input on focus.
    // On input focus or window resize, keypad in device has to adjust.
    var bScrollIntoView = iScroll && iScroll.wrapper && iScroll.wrapper.contains(document.activeElement);
    if (bScrollIntoView && document.activeElement.tagName === 'INPUT' && ($(document.activeElement).offset().top + document.activeElement.clientHeight) > window.innerHeight * 0.9) {
        document.activeElement.scrollIntoView({ behavior: 'auto', block: 'end', inline: 'end' });
    }
    if (iScroll) {
        // refresh specify iscroll on change.
        if (iScroll.indicatorRefresh) {
            iScroll.indicatorRefresh();
        }
        if (iScroll.refresh) {
            iScroll.refresh();
        }
    }
    else if (scrollContainer.length) {
        // refresh all the iscrolls in pagecontent.
        scrollContainer.parent().each(function (i, el) {
            el.iscroll.indicatorRefresh();
            el.iscroll.refresh();
        });
    }
};
var ɵ0$5 = refreshIscrolls;
// on window resize, recalculate the iscroll position and refresh scrollers.
window.addEventListener('resize', debounce(refreshIscrolls, 200));

var TextContentDirective = /** @class */ (function () {
    function TextContentDirective(elRef) {
        this.elRef = elRef;
    }
    Object.defineProperty(TextContentDirective.prototype, "textContent", {
        set: function (nv) {
            var v = nv;
            if (nv === undefined || nv === null) {
                v = '';
            }
            this.elRef.nativeElement.textContent = v;
        },
        enumerable: true,
        configurable: true
    });
    TextContentDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], TextContentDirective.prototype, "textContent", null);
    TextContentDirective = __decorate([
        Directive({
            selector: '[textContent]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], TextContentDirective);
    return TextContentDirective;
}());

var getEpochValue = function (data) {
    var epoch;
    // For data in form of string number ('123'), convert to number (123). And don't parse date objects.
    if (!_.isDate(data) && !isNaN(data)) {
        data = parseInt(data, 10);
    }
    // get the timestamp value. If data is time string, append date string to the time value
    epoch = moment(data).valueOf() || moment(new Date().toDateString() + ' ' + data).valueOf();
    return epoch;
};
var ɵ0$6 = getEpochValue;
var TrailingZeroDecimalPipe = /** @class */ (function () {
    function TrailingZeroDecimalPipe(decimalPipe) {
        this.decimalPipe = decimalPipe;
    }
    TrailingZeroDecimalPipe.prototype.transform = function (value, selectedLocale, numberfilter, localefilter, trailingzero, decimalValue) {
        numberfilter = trailingzero ? "1." + decimalValue.length + "-16" : numberfilter;
        return this.decimalPipe.transform(value, numberfilter, localefilter || selectedLocale);
    };
    TrailingZeroDecimalPipe.ctorParameters = function () { return [
        { type: DecimalPipe }
    ]; };
    TrailingZeroDecimalPipe = __decorate([
        Pipe({
            name: 'trailingZeroDecimalPipe'
        }),
        __metadata("design:paramtypes", [DecimalPipe])
    ], TrailingZeroDecimalPipe);
    return TrailingZeroDecimalPipe;
}());
var ToDatePipe = /** @class */ (function () {
    function ToDatePipe(datePipe) {
        this.datePipe = datePipe;
    }
    ToDatePipe.prototype.transform = function (data, format) {
        var timestamp;
        // 'null' is to be treated as a special case, If user wants to enter null value, empty string will be passed to the backend
        if (data === 'null' || data === '') {
            return '';
        }
        if (!isDefined(data)) {
            return '';
        }
        timestamp = getEpochValue(data);
        if (timestamp) {
            if (format === 'timestamp') {
                return timestamp;
            }
            return this.datePipe.transform(timestamp, format);
        }
        return '';
    };
    ToDatePipe.ctorParameters = function () { return [
        { type: DatePipe }
    ]; };
    ToDatePipe = __decorate([
        Pipe({
            name: 'toDate'
        }),
        __metadata("design:paramtypes", [DatePipe])
    ], ToDatePipe);
    return ToDatePipe;
}());
var ToNumberPipe = /** @class */ (function () {
    function ToNumberPipe(decimalPipe) {
        this.decimalPipe = decimalPipe;
    }
    ToNumberPipe.prototype.transform = function (data, fracSize) {
        if (fracSize && !String(fracSize).match(/^(\d+)?\.((\d+)(-(\d+))?)?$/)) {
            fracSize = '1.' + fracSize + '-' + fracSize;
        }
        if (!_.isNaN(+data)) {
            return this.decimalPipe.transform(data, fracSize);
        }
    };
    ToNumberPipe.ctorParameters = function () { return [
        { type: DecimalPipe }
    ]; };
    ToNumberPipe = __decorate([
        Pipe({
            name: 'toNumber'
        }),
        __metadata("design:paramtypes", [DecimalPipe])
    ], ToNumberPipe);
    return ToNumberPipe;
}());
var ToCurrencyPipe = /** @class */ (function () {
    function ToCurrencyPipe(decimalPipe) {
        this.decimalPipe = decimalPipe;
    }
    ToCurrencyPipe.prototype.transform = function (data, currencySymbol, fracSize) {
        var _currencySymbol = (CURRENCY_INFO[currencySymbol] || {}).symbol || currencySymbol || '';
        var _val = new ToNumberPipe(this.decimalPipe).transform(data, fracSize);
        var isNegativeNumber = _.startsWith(_val, '-');
        if (isNegativeNumber) {
            _val = _val.replace('-', '');
        }
        return _val ? isNegativeNumber ? '-' + _currencySymbol + _val : _currencySymbol + _val : '';
    };
    ToCurrencyPipe.ctorParameters = function () { return [
        { type: DecimalPipe }
    ]; };
    ToCurrencyPipe = __decorate([
        Pipe({
            name: 'toCurrency'
        }),
        __metadata("design:paramtypes", [DecimalPipe])
    ], ToCurrencyPipe);
    return ToCurrencyPipe;
}());
var PrefixPipe = /** @class */ (function () {
    function PrefixPipe() {
    }
    PrefixPipe.prototype.transform = function (data, padding) {
        return (_.isUndefined(data) || data === null || data === '') ? data : ((padding || '') + data);
    };
    PrefixPipe = __decorate([
        Pipe({
            name: 'prefix'
        })
    ], PrefixPipe);
    return PrefixPipe;
}());
var SuffixPipe = /** @class */ (function () {
    function SuffixPipe() {
    }
    SuffixPipe.prototype.transform = function (data, padding) {
        return (_.isUndefined(data) || data === null || data === '') ? data : (data + (padding || ''));
    };
    SuffixPipe = __decorate([
        Pipe({
            name: 'suffix'
        })
    ], SuffixPipe);
    return SuffixPipe;
}());
/**
 * Custom pipe: It is work as interceptor between the user custom pipe function and angular pipe
 */
var CustomPipe = /** @class */ (function () {
    function CustomPipe(custmeUserPipe) {
        this.custmeUserPipe = custmeUserPipe;
    }
    CustomPipe.prototype.transform = function (data, pipename) {
        var argumentArr = [];
        for (var i = 2; i < arguments.length; i++) {
            argumentArr.push(arguments[i]);
        }
        var pipeRef = this.custmeUserPipe.getCustomPipe(pipename);
        if (!pipeRef || !_.isFunction(pipeRef.formatter)) {
            console.warn('formatter is not defined, please check the custom pipes documentation');
            return data;
        }
        try {
            return pipeRef.formatter.apply(pipeRef, __spread([data], argumentArr));
        }
        catch (error) {
            console.error('Pipe name: ' + pipename, error);
            return data;
        }
    };
    CustomPipe.ctorParameters = function () { return [
        { type: CustomPipeManager }
    ]; };
    CustomPipe = __decorate([
        Pipe({
            name: 'custom'
        }),
        __metadata("design:paramtypes", [CustomPipeManager])
    ], CustomPipe);
    return CustomPipe;
}());
var TimeFromNowPipe = /** @class */ (function () {
    function TimeFromNowPipe() {
    }
    TimeFromNowPipe.prototype.transform = function (data) {
        var timestamp;
        if (!isDefined(data)) {
            return undefined;
        }
        timestamp = getEpochValue(data);
        return timestamp ? moment(timestamp).fromNow() : undefined;
    };
    TimeFromNowPipe = __decorate([
        Pipe({
            name: 'timeFromNow'
        })
    ], TimeFromNowPipe);
    return TimeFromNowPipe;
}());
var NumberToStringPipe = /** @class */ (function (_super) {
    __extends(NumberToStringPipe, _super);
    function NumberToStringPipe() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NumberToStringPipe = __decorate([
        Pipe({
            name: 'numberToString'
        })
    ], NumberToStringPipe);
    return NumberToStringPipe;
}(ToNumberPipe));
var StringToNumberPipe = /** @class */ (function () {
    function StringToNumberPipe() {
    }
    StringToNumberPipe.prototype.transform = function (data) {
        return Number(data) || undefined;
    };
    StringToNumberPipe = __decorate([
        Pipe({
            name: 'stringToNumber'
        })
    ], StringToNumberPipe);
    return StringToNumberPipe;
}());
var FilterPipe = /** @class */ (function () {
    function FilterPipe() {
    }
    FilterPipe.prototype.transform = function (data, field, value) {
        if (!data) {
            return [];
        }
        // If object is passed as first paramter
        if (_.isObject(field)) {
            return _.filter(data, field);
        }
        // If key value pair is provided
        return _.filter(data, function (item) {
            return _.includes(item[field], value);
        });
    };
    FilterPipe = __decorate([
        Pipe({
            name: 'filter'
        })
    ], FilterPipe);
    return FilterPipe;
}());
var FileSizePipe = /** @class */ (function () {
    function FileSizePipe() {
    }
    FileSizePipe.prototype.transform = function (bytes, precision) {
        var units = [
            'bytes',
            'KB',
            'MB',
            'GB',
            'TB',
            'PB'
        ];
        /*Todo[shubham]
        if (isNaN(parseFloat(bytes)) || !isFinite(bytes)) {
            return isMobile() ? '' : '?';
        }*/
        var unit = 0;
        while (bytes >= 1024) {
            bytes /= 1024;
            unit++;
        }
        return bytes.toFixed(+precision) + ' ' + units[unit];
    };
    FileSizePipe = __decorate([
        Pipe({
            name: 'filesize'
        })
    ], FileSizePipe);
    return FileSizePipe;
}());
var FileIconClassPipe = /** @class */ (function () {
    function FileIconClassPipe() {
    }
    FileIconClassPipe.prototype.transform = function (fileExtension) {
        var fileClassMapping = {
            'zip': 'fa-file-zip-o',
            'pdf': 'fa-file-pdf-o',
            'rar': 'fa-file-archive-o',
            'txt': 'fa-file-text-o',
            'ppt': 'fa-file-powerpoint-o',
            'pot': 'fa-file-powerpoint-o',
            'pps': 'fa-file-powerpoint-o',
            'pptx': 'fa-file-powerpoint-o',
            'potx': 'fa-file-powerpoint-o',
            'ppsx': 'fa-file-powerpoint-o',
            'mpg': 'fa-file-movie-o',
            'mp4': 'fa-file-movie-o',
            'mov': 'fa-file-movie-o',
            'avi': 'fa-file-movie-o',
            'mp3': 'fa-file-audio-o',
            'docx': 'fa-file-word-o',
            'js': 'fa-file-code-o',
            'md': 'fa-file-code-o',
            'html': 'fa-file-code-o',
            'css': 'fa-file-code-o',
            'xlsx': 'fa-file-excel-o',
            'png': 'fa-file-image-o',
            'jpg': 'fa-file-image-o',
            'jpeg': 'fa-file-image-o',
            'file': 'fa-file-o',
            'default': 'fa-file-o'
        };
        return 'fa ' + (fileClassMapping[fileExtension] || 'fa-file-o');
    };
    FileIconClassPipe = __decorate([
        Pipe({
            name: 'fileIconClass'
        })
    ], FileIconClassPipe);
    return FileIconClassPipe;
}());
var StateClassPipe = /** @class */ (function () {
    function StateClassPipe() {
    }
    StateClassPipe.prototype.transform = function (state) {
        var stateClassMap = {
            'success': 'wi wi-done text-success',
            'error': 'wi wi-error text-danger'
        };
        return stateClassMap[state.toLowerCase()];
    };
    StateClassPipe = __decorate([
        Pipe({
            name: 'stateClass'
        })
    ], StateClassPipe);
    return StateClassPipe;
}());
var FileExtensionFromMimePipe = /** @class */ (function () {
    function FileExtensionFromMimePipe() {
    }
    FileExtensionFromMimePipe.prototype.transform = function (mimeType) {
        var typeMapping = {
            'audio/aac': '.aac',
            'application/x-abiword': '.abw',
            'application/vnd.android.package-archive': '.apk',
            'video/x-msvideo': '.avi',
            'application/vnd.amazon.ebook': '.azw',
            'application/octet-stream': '.bin',
            'image/bmp': '.bmp',
            'application/x-bzip': '.bz',
            'application/x-bzip2': '.bz2',
            'application/x-csh': '.csh',
            'text/css': '.css',
            'text/csv': '.csv',
            'application/msword': '.doc',
            'application/vnd.openxmlformats-officedocument.wordprocessingml.document': '.docx',
            'application/vnd.ms-fontobject': '.eot',
            'application/epub+zip': '.epub',
            'application/ecmascript': '.es',
            'image/gif': '.gif',
            'text/html': '.html',
            'image/x-icon': '.ico',
            'text/calendar': '.ics',
            'application/java-archive': '.jar',
            'image/jpeg': ['.jpeg', '.jpg'],
            'application/javascript': '.js',
            'application/json': '.json',
            'audio/midi': '.mid',
            'audio/x-midi': '.midi',
            'video/mpeg': '.mpeg',
            'application/vnd.apple.installer+xml': 'mpkg',
            'application/vnd.oasis.opendocument.presentation': '.odp',
            'application/vnd.oasis.opendocument.spreadsheet': '.ods',
            'application/vnd.oasis.opendocument.text': '.odt',
            'audio/ogg': '.oga',
            'video/ogg': '.ogv',
            'application/ogg': '.ogx',
            'font/otf': '.otf',
            'image/png': '.png',
            'application/pdf': '.pdf',
            'application/vnd.ms-powerpoint': '.ppt',
            'application/vnd.openxmlformats-officedocument.presentationml.presentation': '.pptx',
            'application/x-rar-compressed': '.rar',
            'application/rtf': '.rtf',
            'application/x-sh': '.sh',
            'image/svg+xml': '.svg',
            'application/x-shockwave-flash': '.swf',
            'application/x-tar': '.tar',
            'image/tiff': '.tiff',
            'application/typescript': '.ts',
            'font/ttf': '.ttf',
            'text/plain': '.txt',
            'application/vnd.visio': '.vsd',
            'audio/wav': '.wav',
            'audio/webm': '.weba',
            'video/webm': '.webm',
            'image/webp': '.webp',
            'font/woff': '.woff',
            'font/woff2': '.woff2',
            'application/xhtml+xml': '.xhtml',
            'application/vnd.ms-excel': '.xls',
            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': '.xlsx',
            'application/xml': '.xml',
            'application/vnd.mozilla.xul+xml': '.xul',
            'application/zip': '.zip',
            'video/3gpp': '.3gp',
            'audio/3gpp': '.3gp',
            'video/3gpp2': '.3g2',
            'audio/3gpp2': '.3g2',
            'application/x-7z-compressed': '.7z'
        };
        return typeMapping[mimeType];
    };
    FileExtensionFromMimePipe = __decorate([
        Pipe({
            name: 'fileExtensionFromMime'
        })
    ], FileExtensionFromMimePipe);
    return FileExtensionFromMimePipe;
}());

var TrustAsPipe = /** @class */ (function () {
    function TrustAsPipe(domSanitizer) {
        this.domSanitizer = domSanitizer;
    }
    TrustAsPipe.prototype.transform = function (content, as) {
        if (as === 'resource' || as === SecurityContext.RESOURCE_URL) {
            if (!content) {
                return '';
            }
            return this.domSanitizer.bypassSecurityTrustResourceUrl(content);
        }
        if (as === 'html' || as === SecurityContext.HTML) {
            if (content === null || content === undefined) {
                return '';
            }
            return this.domSanitizer.sanitize(SecurityContext.HTML, content);
        }
    };
    TrustAsPipe.ctorParameters = function () { return [
        { type: DomSanitizer }
    ]; };
    TrustAsPipe = __decorate([
        Pipe({
            name: 'trustAs'
        }),
        __metadata("design:paramtypes", [DomSanitizer])
    ], TrustAsPipe);
    return TrustAsPipe;
}());

var openedDialogs = [];
/*We need closedDialogs array because onHidden event is asynchronous,
and if the user uses script and calls dialog1.close() and then dialog2.close() then
we cannot be sure if both the dialogs onClose callback will be called or not.*/
var closeDialogsArray = [];
var DialogServiceImpl = /** @class */ (function () {
    function DialogServiceImpl() {
        /**
         * map which contains the references to all dialogs by name
         * @type {Map<any, any>}
         * Ex Map[[dialogName, [[dialogScope, dialogRef]]]]
         */
        this.dialogRefsCollection = new Map();
        this.appConfirmDialog = '_app-confirm-dialog';
    }
    /**
     * Register dialog by name and scope
     * @param {string} name
     * @param {BaseDialog} dialogRef
     * @param {scope}
     */
    DialogServiceImpl.prototype.register = function (name, dialogRef, scope) {
        if (!name) {
            return;
        }
        if (this.dialogRefsCollection.get(name)) {
            this.dialogRefsCollection.get(name).set(scope, dialogRef);
        }
        else {
            this.dialogRefsCollection.set(name, new Map([[scope, dialogRef]]));
        }
    };
    DialogServiceImpl.prototype.getDialogRefsCollection = function () {
        return this.dialogRefsCollection;
    };
    /**
     * De Register dialog by name and scope
     * @param name
     * @param dialogRef
     * @param scope
     */
    DialogServiceImpl.prototype.deRegister = function (name, scope) {
        if (!name) {
            return;
        }
        if (this.dialogRefsCollection.get(name)) {
            this.dialogRefsCollection.get(name).delete(scope);
        }
    };
    DialogServiceImpl.prototype.getDialogRef = function (name, scope) {
        var dialogRefMap = this.dialogRefsCollection.get(name);
        var dialogRef;
        if (scope) {
            dialogRef = dialogRefMap.get(scope);
            if (!dialogRef) {
                // Check if the scope is App level scope
                // else throw a console error
                if (!scope.pageName && !scope.partialName && !scope.prefabName) {
                    dialogRefMap.forEach(function (dRef, dialogScope) {
                        // Check if the collection of dialogs have a "common" partial scope
                        // If yes use that else through a console error
                        if (dialogScope && dialogScope.partialName === 'Common') {
                            dialogRef = dRef;
                        }
                        else {
                            console.error('No dialog with the name "' + name + '" found in the App scope.');
                        }
                    });
                }
                else {
                    console.error('No dialog with the name "' + name + '" found in the given scope.');
                }
            }
        }
        else {
            if (dialogRefMap.size === 1) {
                dialogRef = dialogRefMap.entries().next().value[1];
            }
            else {
                console.error('There are multiple instances of this dialog name. Please provide the Page/Partial/App instance in which the dialog exists.');
            }
        }
        return dialogRef;
    };
    /**
     * Opens the dialog with the given name
     * @param {string} name
     */
    DialogServiceImpl.prototype.open = function (name, scope, initState) {
        var dialogRef = this.getDialogRef(name, scope);
        if (!dialogRef) {
            return;
        }
        dialogRef.open(initState);
    };
    /**
     * closes the dialog with the given name
     * @param {string} name
     */
    DialogServiceImpl.prototype.close = function (name, scope, callBackFn) {
        var dialogRef = this.getDialogRef(name, scope);
        if (!dialogRef) {
            return;
        }
        // For notification action variable dialogs onok callback event should be called after closing the dialog.
        // So passing it as callback function
        dialogRef.closeCallBackFn = callBackFn;
        dialogRef.close();
    };
    /**
     * closes all the opened dialogs
     */
    DialogServiceImpl.prototype.closeAllDialogs = function () {
        _.forEach(openedDialogs.reverse(), function (dialog) {
            dialog.close();
        });
    };
    DialogServiceImpl.prototype.showAppConfirmDialog = function (initState) {
        this.open(this.getAppConfirmDialog(), undefined, initState);
    };
    DialogServiceImpl.prototype.closeAppConfirmDialog = function () {
        this.close(this.getAppConfirmDialog());
    };
    DialogServiceImpl.prototype.getAppConfirmDialog = function () {
        return this.appConfirmDialog;
    };
    DialogServiceImpl.prototype.setAppConfirmDialog = function (dialogName) {
        this.appConfirmDialog = dialogName;
    };
    DialogServiceImpl.prototype.addToOpenedDialogs = function (ref) {
        openedDialogs.push(ref);
    };
    DialogServiceImpl.prototype.getLastOpenedDialog = function () {
        return openedDialogs[openedDialogs.length - 1];
    };
    DialogServiceImpl.prototype.removeFromOpenedDialogs = function (ref) {
        if (openedDialogs.indexOf(ref) !== -1) {
            openedDialogs.splice(openedDialogs.indexOf(ref), 1);
        }
    };
    DialogServiceImpl.prototype.getOpenedDialogs = function () {
        return openedDialogs;
    };
    DialogServiceImpl.prototype.addToClosedDialogs = function (ref) {
        closeDialogsArray.push(ref);
    };
    DialogServiceImpl.prototype.removeFromClosedDialogs = function (ref) {
        if (closeDialogsArray.indexOf(ref) !== -1) {
            closeDialogsArray.splice(closeDialogsArray.indexOf(ref), 1);
        }
    };
    DialogServiceImpl.prototype.getDialogRefFromClosedDialogs = function () {
        return closeDialogsArray[closeDialogsArray.length - 1];
    };
    DialogServiceImpl = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], DialogServiceImpl);
    return DialogServiceImpl;
}());

var wmComponents = [
    ContainerDirective,
    ItemTemplateDirective,
    LazyLoadDirective,
    MessageComponent,
    PartialDirective,
    PartialParamHandlerDirective,
    PartialParamDirective,
    PartialContainerDirective,
    RedrawableDirective,
    ShowInDeviceDirective,
    SmoothScrollDirective,
    TextContentDirective
];
var PIPES = [
    ToDatePipe,
    FileIconClassPipe,
    FileExtensionFromMimePipe,
    FilterPipe,
    FileSizePipe,
    ToNumberPipe,
    ToCurrencyPipe,
    PrefixPipe,
    SuffixPipe,
    TimeFromNowPipe,
    NumberToStringPipe,
    StateClassPipe,
    StringToNumberPipe,
    TrailingZeroDecimalPipe,
    TrustAsPipe,
    ImagePipe,
    CustomPipe
];
var WmComponentsModule = /** @class */ (function () {
    function WmComponentsModule() {
    }
    WmComponentsModule_1 = WmComponentsModule;
    WmComponentsModule.forRoot = function () {
        return {
            ngModule: WmComponentsModule_1,
            providers: [
                ToDatePipe,
                FilterPipe,
                TrailingZeroDecimalPipe,
                TrustAsPipe,
                ImagePipe,
                CustomPipe,
                Location,
                { provide: AbstractDialogService, useClass: DialogServiceImpl }
            ]
        };
    };
    var WmComponentsModule_1;
    WmComponentsModule = WmComponentsModule_1 = __decorate([
        NgModule({
            imports: [
                CommonModule,
                SecurityModule
            ],
            declarations: __spread(wmComponents, PIPES),
            exports: __spread(wmComponents, PIPES)
        })
    ], WmComponentsModule);
    return WmComponentsModule;
}());

var noop = function () { };
var ɵ0$7 = noop;
var Live_Operations;
(function (Live_Operations) {
    Live_Operations["INSERT"] = "insert";
    Live_Operations["UPDATE"] = "update";
    Live_Operations["DELETE"] = "delete";
    Live_Operations["READ"] = "read";
})(Live_Operations || (Live_Operations = {}));
var ALLFIELDS = 'All Fields';
var LIVE_CONSTANTS = {
    'EMPTY_KEY': 'EMPTY_NULL_FILTER',
    'EMPTY_VALUE': 'No Value',
    'LABEL_KEY': 'key',
    'LABEL_VALUE': 'value',
    'NULL_EMPTY': ['null', 'empty'],
    'NULL': 'null',
    'EMPTY': 'empty'
};
// Returns true if widget is autocomplete or chips
function isSearchWidgetType(widget) {
    return _.includes([FormWidgetType.AUTOCOMPLETE, FormWidgetType.TYPEAHEAD, FormWidgetType.CHIPS], widget);
}
function onSuccess(response, res, rej) {
    if (response.error) {
        rej(response);
    }
    else {
        res(response);
    }
}
function performDataOperation(dataSource, requestData, options) {
    return new Promise(function (res, rej) {
        if (dataSource.execute(DataSource.Operation.SUPPORTS_CRUD)) {
            var fn = void 0;
            var operationType = options.operationType;
            switch (operationType) {
                case Live_Operations.UPDATE:
                    fn = DataSource.Operation.UPDATE_RECORD;
                    break;
                case Live_Operations.INSERT:
                    fn = DataSource.Operation.INSERT_RECORD;
                    break;
                case Live_Operations.DELETE:
                    fn = DataSource.Operation.DELETE_RECORD;
                    break;
            }
            dataSource.execute(fn, requestData).then(function (response) { return onSuccess(response, res, rej); }, rej);
        }
        else if (dataSource.execute(DataSource.Operation.IS_API_AWARE)) {
            if (dataSource.category === "wm.CrudVariable") {
                var operation = options.operationType === 'insert' ? 'create' : (options.operationType === 'delete' ? 'delete' : 'update');
                dataSource.execute(DataSource.Operation.INVOKE, {
                    'skipNotification': true,
                    'operation': operation,
                    'inputFields': requestData
                }).then(res, rej);
            }
            else {
                dataSource.execute(DataSource.Operation.SET_INPUT, requestData);
                dataSource.execute(DataSource.Operation.INVOKE, {
                    'skipNotification': true
                }).then(res, rej);
            }
        }
        else {
            res(requestData);
        }
    });
}
function refreshDataSource(dataSource, options) {
    return new Promise(function (res, rej) {
        if (!dataSource) {
            rej();
            return;
        }
        dataSource.execute(DataSource.Operation.LIST_RECORDS, {
            'filterFields': options.filterFields || {},
            'orderBy': options.orderBy,
            'page': options.page || 1
        }).then(res, rej);
    });
}
/**
 * @ngdoc function
 * @name wm.widgets.live.fetchRelatedFieldData
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * This function fetches the data for the related field in live form/ grid
 *
 * @param {object} columnDef field definition
 * @param {string} relatedField related field name
 * @param {string} datafield Datafield to be set on widget
 * @param {string} widget Type of the widget
 * @param {object} elScope element scope
 * @param {object} parentScope live form// grid scope
 */
function fetchRelatedFieldData(dataSource, formField, options) {
    var primaryKeys;
    var displayField;
    var relatedField = options.relatedField;
    var datafield = options.datafield;
    if (!dataSource) {
        return;
    }
    primaryKeys = dataSource.execute(DataSource.Operation.GET_RELATED_PRIMARY_KEYS, relatedField);
    formField.datafield = datafield;
    formField._primaryKey = _.isEmpty(primaryKeys) ? undefined : primaryKeys[0];
    formField.compareby = primaryKeys && primaryKeys.join(',');
    displayField = datafield === ALLFIELDS ? undefined : datafield;
    formField.displayfield = displayField = (formField.displayfield || displayField || formField._primaryKey);
    if (isSearchWidgetType(formField[options.widget])) {
        formField.dataoptions = { 'relatedField': relatedField, 'filterExpr': formField.filterexpressions ? formField.filterexpressions : {} };
        formField.datasource = dataSource;
        formField.searchkey = formField.searchkey || displayField;
        formField.displaylabel = formField.displayfield = (formField.displaylabel || displayField);
        if (formField.showPendingSpinner) {
            formField.showPendingSpinner = false;
        }
    }
    else {
        interpolateBindExpressions(formField.viewParent, formField.filterexpressions, function (filterexpressions) {
            formField.filterexpressions = filterexpressions;
            dataSource.execute(DataSource.Operation.GET_RELATED_TABLE_DATA, {
                relatedField: relatedField,
                pagesize: formField.limit,
                orderBy: formField.orderby ? _.replace(formField.orderby, /:/g, ' ') : '',
                filterFields: {},
                filterExpr: formField.filterexpressions ? formField.filterexpressions : {}
            }).then(function (response) {
                formField.dataset = response.data;
                formField.displayfield = formField.displayfield || _.head(_.keys(_.get(response, '[0]')));
                if (formField.showPendingSpinner) {
                    formField.showPendingSpinner = false;
                }
            }, noop);
        });
    }
}
/**
 * used to interpolate the bind expression for keys in the query builder
 * @param context where we find the variable obj
 * @param filterexpressions - obj containing all the rule objs
 * @param callbackFn - function to be called with the new replaced values if any in the filterexpressions object
 */
var interpolateBindExpressions = function (context, filterexpressions, callbackFn) {
    var debouncedFn = debounce(function () {
        if (_.isFunction(callbackFn)) {
            callbackFn(filterexpressions);
        }
    }, 300);
    /**
     * calling the debounced function first for the case where if there is any filterexpression without the bindedvariables.
     * without this it will never be called. processFilterExpBindNode will be called only for the binded variable expressions.
     */
    debouncedFn();
    var filterExpressions = filterexpressions ? (_.isObject(filterexpressions) ? filterexpressions : JSON.parse(filterexpressions)) : {};
    var destroyFn = context.registerDestroyListener ? context.registerDestroyListener.bind(context) : _.noop;
    var filterSubscription = processFilterExpBindNode(context, filterExpressions).subscribe(function (response) {
        filterexpressions = JSON.stringify(response.filterExpressions);
        debouncedFn();
    });
    destroyFn(function () { return filterSubscription.unsubscribe(); });
};
/**
 * @ngdoc function
 * @name wm.widgets.live.LiveWidgetUtils#getDistinctFieldProperties
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Returns the properties required for dataset widgets
 *
 * @param {object} dataSource variable source for the widget
 * @param {object} formField definition of the column/ field
 *
 */
var getDistinctFieldProperties = function (dataSource, formField) {
    var props = {};
    var fieldColumn;
    if (formField['is-related']) {
        props.tableName = formField['lookup-type'];
        fieldColumn = formField['lookup-field'];
        props.distinctField = fieldColumn;
        props.aliasColumn = fieldColumn.replace('.', '$'); // For related fields, In response . is replaced by $
        props.filterExpr = formField.filterexpressions ? (_.isObject(formField.filterexpressions) ? formField.filterexpressions : JSON.parse(formField.filterexpressions)) : {};
    }
    else {
        props.tableName = dataSource.execute(DataSource.Operation.GET_ENTITY_NAME);
        fieldColumn = formField.field || formField.key;
        props.distinctField = fieldColumn;
        props.aliasColumn = fieldColumn;
    }
    return props;
};
/**
 * @ngdoc function
 * @name wm.widgets.live.LiveWidgetUtils#getDistinctValues
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Returns the distinct values for a field
 *
 * @param {object} formField definition of the column/ field
 * @param {string} widget widget property on the field
 * @param {object} variable variable for the widget
 * @param {function} callBack Function to be executed after fetching results
 *
 */
function getDistinctValues(dataSource, formField, widget) {
    var props;
    return new Promise(function (res, rej) {
        if (isDataSetWidget(formField[widget]) && (!formField.isDataSetBound || widget === 'filterwidget')) {
            props = getDistinctFieldProperties(dataSource, formField);
            dataSource.execute(DataSource.Operation.GET_DISTINCT_DATA_BY_FIELDS, {
                fields: props.distinctField,
                entityName: props.tableName,
                pagesize: formField.limit,
                filterExpr: formField.filterexpressions ? JSON.parse(formField.filterexpressions) : {}
            }).then(function (response) {
                res({ 'field': formField, 'data': response.data, 'aliasColumn': props.aliasColumn });
            }, rej);
        }
    });
}
// Set the data field properties on dataset widgets
function setDataFields(formField, options) {
    // For search widget, set search key and display label
    if (isSearchWidgetType(formField[options.widget])) {
        formField.datafield = options.aliasColumn || LIVE_CONSTANTS.LABEL_KEY;
        formField.searchkey = options.distinctField || LIVE_CONSTANTS.LABEL_KEY;
        formField.displaylabel = formField.displayfield = (options.aliasColumn || LIVE_CONSTANTS.LABEL_VALUE);
        return;
    }
    formField.datafield = LIVE_CONSTANTS.LABEL_KEY;
    formField.displayfield = LIVE_CONSTANTS.LABEL_VALUE;
}
/**
 * @ngdoc function
 * @name wm.widgets.live.LiveWidgetUtils#setFieldDataSet
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Function to set the dataSet on the fields
 *
 * @param {object} formField definition of the column/ field
 * @param {object} data data returned from the server
 * @param {string} aliasColumn column field name
 * @param {string} widget widget property on the field
 * @param {boolean} isEnableEmptyFilter is null or empty values allowed on filter
 *
 */
function setFieldDataSet(formField, data, options) {
    var emptySupportWidgets = [FormWidgetType.SELECT, FormWidgetType.RADIOSET];
    var emptyOption = {};
    var dataSet = [];
    if (options.isEnableEmptyFilter && _.includes(emptySupportWidgets, formField[options.widget]) &&
        !formField['is-range'] && !formField.multiple) {
        // If empty option is selected, push an empty object in to dataSet
        emptyOption[LIVE_CONSTANTS.LABEL_KEY] = LIVE_CONSTANTS.EMPTY_KEY;
        emptyOption[LIVE_CONSTANTS.LABEL_VALUE] = options.EMPTY_VALUE || LIVE_CONSTANTS.EMPTY_VALUE;
        dataSet.push(emptyOption);
    }
    _.each(data, function (key) {
        var value = key[options.aliasColumn];
        var option = {};
        if (value !== null && value !== '') {
            option[LIVE_CONSTANTS.LABEL_KEY] = value;
            option[LIVE_CONSTANTS.LABEL_VALUE] = value;
            dataSet.push(option);
        }
    });
    setDataFields(formField, options);
    formField.dataset = dataSet;
}
/**
 * @ngdoc function
 * @name wm.widgets.live.LiveWidgetUtils#fetchDistinctValues
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Function to fetch the distinct values for a field
 *
 * @param {object} scope scope of the widget
 * @param {object} formFields definitions of the column/ field
 * @param {string} widget widget property on the field
 * @param {boolean} isEnableEmptyFilter is null or empty values allowed on filter
 *
 */
function fetchDistinctValues(dataSource, formFields, options) {
    if (_.isEmpty(formFields)) {
        return;
    }
    formFields.forEach(function (formField) {
        getDistinctValuesForField(dataSource, formField, options);
    });
}
/**
 * @ngdoc function
 * @name wm.widgets.live.LiveWidgetUtils#getDistinctValuesForField
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Function to fetch the distinct values for a field
 *
 * @param {object} scope scope of the widget
 * @param {object} formFields definitions of the column/ field
 * @param {string} widget widget property on the field
 * @param {boolean} isEnableEmptyFilter is null or empty values allowed on filter
 *
 */
function getDistinctValuesForField(dataSource, formField, options) {
    if (!dataSource || !formField || formField.isDataSetBound) {
        return;
    }
    if (isSearchWidgetType(formField[options.widget])) {
        var dataoptions = getDistinctFieldProperties(dataSource, formField);
        formField.dataoptions = dataoptions;
        setDataFields(formField, Object.assign(options || {}, dataoptions));
        formField.datasource = dataSource;
    }
    else {
        interpolateBindExpressions(formField.viewParent, formField.filterexpressions, function (filterexpressions) {
            formField.filterexpressions = filterexpressions;
            getDistinctValues(dataSource, formField, options.widget).then(function (res) {
                setFieldDataSet(res.field, res.data, {
                    aliasColumn: res.aliasColumn,
                    widget: options.widget,
                    isEnableEmptyFilter: getEnableEmptyFilter(options.enableemptyfilter),
                    EMPTY_VALUE: options.EMPTY_VALUE
                });
            });
        });
    }
}
function isDefinedAndNotEmpty(val) {
    return isDefined(val) && val !== '' && val !== null;
}
/**
 * @ngdoc function
 * @name wm.widgets.live.getRangeFieldValue
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Function to get the field value for range
 *
 * @param {string} minValue min value selected
 * @param {string} maxValue max value selected
 */
function getRangeFieldValue(minValue, maxValue) {
    var fieldValue;
    if (isDefinedAndNotEmpty(minValue) && isDefinedAndNotEmpty(maxValue)) {
        fieldValue = [minValue, maxValue];
    }
    else if (isDefinedAndNotEmpty(minValue)) {
        fieldValue = minValue;
    }
    else if (isDefinedAndNotEmpty(maxValue)) {
        fieldValue = maxValue;
    }
    return fieldValue;
}
/**
 * @ngdoc function
 * @name wm.widgets.live.getRangeMatchMode
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Function to get the match mode for range
 *
 * @param {string} minValue min value selected
 * @param {string} maxValue max value selected
 */
function getRangeMatchMode(minValue, maxValue) {
    var matchMode;
    // If two values exists, then it is between. Otherwise, greater or lesser
    if (isDefinedAndNotEmpty(minValue) && isDefinedAndNotEmpty(maxValue)) {
        matchMode = MatchMode.BETWEEN;
    }
    else if (isDefinedAndNotEmpty(minValue)) {
        matchMode = MatchMode.GREATER;
    }
    else if (isDefinedAndNotEmpty(maxValue)) {
        matchMode = MatchMode.LESSER;
    }
    return matchMode;
}
/**
 * @ngdoc function
 * @name wm.widgets.live.getEnableEmptyFilter
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * This function checks if enable filter options is set on live filter
 *
 * @param {object} enableemptyfilter empty filter options
 */
function getEnableEmptyFilter(enableemptyfilter) {
    return enableemptyfilter && _.intersection(enableemptyfilter.split(','), LIVE_CONSTANTS.NULL_EMPTY).length > 0;
}
/**
 * @ngdoc function
 * @name wm.widgets.live.getEmptyMatchMode
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Function to get the match mode based on the filter selected
 *
 * @param {object} enableemptyfilter empty filter options
 */
function getEmptyMatchMode(enableemptyfilter) {
    var matchMode;
    var emptyFilterOptions = _.split(enableemptyfilter, ',');
    if (_.intersection(emptyFilterOptions, LIVE_CONSTANTS.NULL_EMPTY).length === 2) {
        matchMode = MatchMode.NULLOREMPTY;
    }
    else if (_.includes(emptyFilterOptions, LIVE_CONSTANTS.NULL)) {
        matchMode = MatchMode.NULL;
    }
    else if (_.includes(emptyFilterOptions, LIVE_CONSTANTS.EMPTY)) {
        matchMode = MatchMode.EMPTY;
    }
    return matchMode;
}
/**
 * converts the data passed to array.
 *  -> Array: [1,2,3] - [1,2,3]
 *  -> String: a,b,c - ['a','b','c']
 *  -> object: {a:1} - [{a:1}]
 *  -> null - []
 *  -> undefined - []
 * @param data
 * @returns {Array<any>}
 */
var createArrayFrom = function (data) {
    if (_.isUndefined(data) || _.isNull(data)) {
        return [];
    }
    if (_.isString(data)) {
        data = data.split(',').map(Function.prototype.call, String.prototype.trim);
    }
    if (!_.isArray(data)) {
        data = [data];
    }
    return data;
};
/**
 * @ngdoc function
 * @name wm.widgets.live.applyFilterOnField
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * Function to get the updated values when filter on field is changed
 *
 * @param {object} $scope scope of the filter field/form field
 * @param {object} filterDef filter/form definition of the field
 * @param {boolean} isFirst boolean value to check if this method is called on load
 */
function applyFilterOnField(dataSource, filterDef, formFields, newVal, options) {
    if (options === void 0) { options = {}; }
    var fieldName = filterDef.field || filterDef.key;
    var filterOnFields = _.filter(formFields, { 'filter-on': fieldName });
    newVal = filterDef['is-range'] ? getRangeFieldValue(filterDef.minValue, filterDef.maxValue) : (isDefined(newVal) ? newVal : filterDef.value);
    if (!dataSource || (options.isFirst && (_.isUndefined(newVal) || newVal === ''))) {
        return;
    }
    // Loop over the fields for which the current field is filter on field
    _.forEach(filterOnFields, function (filterField) {
        var filterKey = filterField.field || filterField.key;
        var lookUpField = filterDef['lookup-field'] || filterDef._primaryKey;
        var filterWidget = filterField['edit-widget-type'] || filterField.widgettype;
        var filterFields = {};
        var filterOn = filterField['filter-on'];
        var filterVal;
        var fieldColumn;
        var matchMode;
        if (!isDataSetWidget(filterWidget) || filterField.isDataSetBound || filterOn === filterKey) {
            return;
        }
        // For related fields, add lookupfield for query generation
        if (filterDef && filterDef['is-related']) {
            filterOn += '.' + lookUpField;
        }
        if (isDefined(newVal)) {
            if (filterDef['is-range']) {
                matchMode = getRangeMatchMode(filterDef.minValue, filterDef.maxValue);
            }
            else if (getEnableEmptyFilter(options.enableemptyfilter) && newVal === LIVE_CONSTANTS.EMPTY_KEY) {
                matchMode = getEmptyMatchMode(options.enableemptyfilter);
            }
            else {
                matchMode = MatchMode.EQUALS;
            }
            filterVal = (_.isObject(newVal) && !_.isArray(newVal)) ? newVal[lookUpField] : newVal;
            filterFields[filterOn] = {
                'value': filterVal,
                'matchMode': matchMode
            };
        }
        else {
            filterFields = {};
        }
        fieldColumn = filterKey;
        if (isSearchWidgetType(filterWidget) && filterField.dataoptions) {
            filterField.dataoptions.filterFields = filterFields;
        }
        else {
            dataSource.execute(DataSource.Operation.GET_DISTINCT_DATA_BY_FIELDS, {
                'fields': fieldColumn,
                'filterFields': filterFields,
                'pagesize': filterField.limit
            }).then(function (response) {
                setFieldDataSet(filterField, response.data, {
                    aliasColumn: fieldColumn,
                    widget: options.widget || 'widgettype',
                    isEnableEmptyFilter: getEnableEmptyFilter(options.enableemptyfilter),
                    EMPTY_VALUE: options.EMPTY_VALUE
                });
            }, noop);
        }
    });
}
// Transform data as required by data table
function transformData(dataObject, variableName) {
    var newObj, tempArr, keys, oldKeys, numKeys, newObject, tempObj;
    // data sanity testing
    dataObject = dataObject || [];
    // if the dataObject is not an array make it an array
    if (!_.isArray(dataObject)) {
        // if the data returned is of type string, make it an object inside an array
        if (_.isString(dataObject)) {
            keys = variableName.substring(variableName.indexOf('.') + 1, variableName.length).split('.');
            oldKeys = [];
            numKeys = keys.length;
            newObject = {};
            tempObj = newObject;
            // loop over the keys to form appropriate data object required for grid
            keys.forEach(function (key, index) {
                // loop over old keys to create new object at the iterative level
                oldKeys.forEach(function (oldKey) {
                    tempObj = newObject[oldKey];
                });
                tempObj[key] = index === numKeys - 1 ? dataObject : {};
                oldKeys.push(key);
            });
            // change the string data to the new dataObject formed
            dataObject = newObject;
        }
        dataObject = [dataObject];
    }
    else {
        /*if the dataObject is an array and each value is a string, then lite-transform the string to an object
         * lite-transform: just checking if the first value is string and then transforming the object, instead of traversing through the whole array
         * */
        if (_.isString(dataObject[0])) {
            tempArr = [];
            _.forEach(dataObject, function (str) {
                newObj = {};
                newObj[variableName.split('.').join('-')] = str;
                tempArr.push(newObj);
            });
            dataObject = tempArr;
        }
    }
    return dataObject;
}

var momentLocale = moment.localeData();
var momentCalendarOptions = getClonedObject(momentLocale._calendar);
var momentCalendarDayOptions = momentLocale._calendarDay || {
    lastDay: '[Yesterday]',
    lastWeek: '[Last] dddd',
    nextDay: '[Tomorrow]',
    nextWeek: 'dddd',
    sameDay: '[Today]',
    sameElse: 'L'
};
var GROUP_BY_OPTIONS = {
    ALPHABET: 'alphabet',
    WORD: 'word',
    OTHERS: 'Others'
};
var TIME_ROLLUP_OPTIONS = {
    HOUR: 'hour',
    DAY: 'day',
    WEEK: 'week',
    MONTH: 'month',
    YEAR: 'year'
};
var ROLLUP_PATTERNS = {
    DAY: 'yyyy-MM-dd',
    WEEK: 'w \'Week\',  yyyy',
    MONTH: 'MMM, yyyy',
    YEAR: 'YYYY',
    HOUR: 'hh:mm a'
};
/**
 * function to get the ordered dataset based on the given orderby
 */
var getOrderedDataset = function (dataSet, orderBy, innerItem) {
    if (!orderBy) {
        return _.cloneDeep(dataSet);
    }
    // The order by only works when the dataset contains list of objects.
    var items = orderBy.split(','), fields = [], directions = [];
    items.forEach(function (obj) {
        var item = obj.split(':');
        fields.push(innerItem ? innerItem + '.' + item[0] : item[0]);
        directions.push(item[1]);
    });
    return _.orderBy(dataSet, fields, directions);
};
/**
 * Returns an array of object, each object contain the DataSetItem whose key, value, label are extracted from object keys.
 */
var transformDataWithKeys = function (dataSet) {
    var data = [];
    // if the dataset is instance of object (not an array) or the first item in the dataset array is an object,
    // then we extract the keys from the object and prepare the dataset items.
    if (_.isObject(dataSet[0]) || (_.isObject(dataSet) && !(dataSet instanceof Array))) {
        // getting keys of the object
        var objectKeys = Object.keys(dataSet[0] || dataSet);
        _.forEach(objectKeys, function (objKey, index) {
            data.push({
                key: objKey,
                label: objKey,
                value: objKey,
                index: index + 1
            });
        });
    }
    return data;
};
// Converts any type of data to array.
var extractDataAsArray = function (data) {
    if (_.isUndefined(data) || _.isNull(data) || _.trim(data) === '') {
        return [];
    }
    if (_.isString(data)) {
        data = _.split(data, ',').map(function (str) { return str.trim(); });
    }
    if (!_.isArray(data)) {
        data = [data];
    }
    return data;
};
// This function return always an object containing dataset details.
var convertDataToObject = function (dataResult) {
    if (_.isString(dataResult)) {
        dataResult = _.split(dataResult, ',').map(function (str) { return str.trim(); });
    }
    return dataResult;
};
/**
 * The first step in datasetItems creation is data transformation:
 *
 * The dataset can contain one of the following formats and each of them to be converted to the given format;
 *
 * 1) The comma separated string..eg: A,B,C => [{ key: 'A', value: 'A'}, { key: 'B', value: 'B'}, { key: 'C', value: 'C'}]
 * 2) The array of values eg: [1,2,3] => [{ key: 1, value: 1}, { key: 2, value: 2}, { key: 3, value: 3}]
 * 3) an object eg: {name: 'A', age: 20} => [ {key: 'name', value: 'A'}, {key: 'age', value: 20}]
 * 4) an array of objects...eg: [ {name: 'A', age: 20}, {name: 'B', age: 20}] ==> returns [{key: _DATAFIELD_, value: _DISPLAYFIELD, label: _DISPLAYVALUE}]
 */
var transformFormData = function (context, dataSet, myDataField, displayOptions, startIndex) {
    var data = [];
    if (!dataSet) {
        return;
    }
    dataSet = convertDataToObject(dataSet);
    // startIndex is the index of the next new item.
    if (_.isUndefined(startIndex)) {
        startIndex = 1;
    }
    if (_.isString(dataSet)) {
        dataSet = dataSet.split(',').map(function (str) { return str.trim(); });
        dataSet.forEach(function (option, index) {
            data.push({ key: option, value: option, label: (isDefined(option) && option !== null) ? option.toString() : '', index: startIndex + index });
        });
    }
    else if (_.isArray(dataSet) && !_.isObject(dataSet[0])) { // array of primitive values only
        dataSet.forEach(function (option, index) {
            data.push({ key: option, value: option, label: (isDefined(option) && option !== null) ? option.toString() : '', index: startIndex + index });
        });
    }
    else if (!(dataSet instanceof Array) && _.isObject(dataSet)) {
        var i = 0;
        _.forEach(dataSet, function (value, key) {
            data.push({ key: _.trim(key), value: key, label: (isDefined(value) && value !== null) ? value.toString() : '', index: startIndex, dataObject: dataSet });
        });
    }
    else {
        if (!myDataField) { // consider the datafield as 'ALLFIELDS' when datafield is not given.
            myDataField = ALLFIELDS;
        }
        dataSet.forEach(function (option, index) {
            var key = myDataField === ALLFIELDS ? startIndex + index : getObjValueByKey(option, myDataField);
            // Omit all the items whose datafield (key) is null or undefined.
            if (!_.isUndefined(key) && !_.isNull(key)) {
                var label = getEvaluatedData(option, {
                    field: displayOptions.displayField,
                    expression: displayOptions.displayExpr,
                    bindExpression: displayOptions.bindDisplayExpr
                }, context);
                var dataSetItem = {
                    key: key,
                    label: (isDefined(label) && label !== null) ? label.toString() : '',
                    value: myDataField === ALLFIELDS ? option : key,
                    dataObject: option,
                    index: startIndex + index
                };
                if (displayOptions.displayImgSrc || displayOptions.bindDisplayImgSrc) {
                    dataSetItem.imgSrc = getEvaluatedData(option, {
                        expression: displayOptions.displayImgSrc,
                        bindExpression: displayOptions.bindDisplayImgSrc
                    }, context);
                }
                data.push(dataSetItem);
            }
        });
    }
    return data;
};
/**
 * Private method to get the unique objects by the data field
 */
var getUniqObjsByDataField = function (data, dataField, displayField, allowEmptyFields) {
    var uniqData;
    var isAllFields = dataField === ALLFIELDS;
    uniqData = isAllFields ? _.uniqWith(data, _.isEqual) : _.uniqBy(data, 'key');
    if (!displayField || allowEmptyFields) {
        return uniqData;
    }
    // return objects having non empty datafield and display field values.
    return _.filter(uniqData, function (obj) {
        if (isAllFields) {
            return _.trim(obj.label);
        }
        return _.trim(obj.key) && _.trim(obj.label);
    });
};
/**
 * This function sets the selectedItem by comparing the field values, where fields are passed by "compareby" property.
 * works only for datafield with ALL_FIELDS
 * @param datasetItems list of dataset items.
 * @param compareWithDataObj represents the datavalue (object) whose properties are to be checked against each property of datasetItem.
 * @param compareByField specifies the property names on which datasetItem has to be compared against datavalue object.
 */
var setItemByCompare = function (datasetItems, compareWithDataObj, compareByField) {
    // compare the fields based on fields given to compareby property.
    _.forEach(datasetItems, function (opt) {
        if (isEqualWithFields(opt.value, compareWithDataObj, compareByField)) {
            opt.selected = true;
            return false;
        }
    });
};
/**
 * This method returns sorted data based to groupkey.
 * Returns a array of objects, each object containing key which is groupKey and data is the sorted data which is sorted by groupby field in the data.
 *
 * @param groupedLiData, grouped data object with key as the groupKey and its value as the array of objects grouped under the groupKey.
 * @param groupBy, string groupby property
 * @returns {any[]}
 */
var getSortedGroupedData = function (groupedLiData, groupBy, orderby) {
    var _groupedData = [];
    _.forEach(_.keys(groupedLiData), function (groupkey, index) {
        var liData = getOrderedDataset(groupedLiData[groupkey], orderby, 'dataObject');
        _groupedData.push({
            key: groupkey,
            data: _.sortBy(liData, function (data) {
                data._groupIndex = index + 1;
                return _.get(data, groupBy) || _.get(data.dataObject, groupBy);
            })
        });
    });
    return _groupedData;
};
var ɵ0$8 = getSortedGroupedData;
/**
 * This method gets the groupedData using groupby property and match and returns the sorted array of objects.
 *
 * @param compRef represents the component's reference i.e. "this" value.
 * @param data represents the dataset i.e array of objects.
 * @param groupby, string groupby property
 * @param match, string match property
 * @param orderby, string orderby property
 * @param dateformat, string dateFormat property
 * @param innerItem, represents the innerItem on which groupby has to be applied. Incase of datasetItems, 'dataObject' contains the full object. Here innerItem is dataObject.
 * @returns {any[]} groupedData, array of objects, each object having key and data.
 */
var groupData = function (compRef, data, groupby, match, orderby, dateformat, datePipe, innerItem, AppDefaults) {
    var groupedLiData = {};
    if (_.includes(groupby, '(')) {
        var groupDataByUserDefinedFn_1 = $parseEvent(groupby);
        groupedLiData = _.groupBy(data, function (val) {
            return groupDataByUserDefinedFn_1(compRef.viewParent, { 'row': val.dataObject || val });
        });
    }
    else {
        groupedLiData = getGroupedData(data, groupby, match, orderby, dateformat, datePipe, innerItem, AppDefaults);
    }
    return getSortedGroupedData(groupedLiData, groupby, orderby);
};
/**
 * This method prepares the grouped data.
 *
 * @param fieldDefs array of objects i.e. dataset
 * @param groupby string groupby
 * @param match string match
 * @param orderby string orderby
 * @param dateFormat string date format
 * @param innerItem, item to look for in the passed data
 */
var getGroupedData = function (fieldDefs, groupby, match, orderby, dateFormat, datePipe, innerItem, AppDefaults) {
    // For day, set the relevant moment calendar options
    if (match === TIME_ROLLUP_OPTIONS.DAY) {
        momentLocale._calendar = momentCalendarDayOptions;
    }
    // handling case-in-sensitive scenario
    // ordering the data based on groupby field. If there is innerItem then apply orderby using the innerItem's containing the groupby field.
    fieldDefs = _.orderBy(fieldDefs, function (fieldDef) {
        var groupKey = _.get(innerItem ? fieldDef[innerItem] : fieldDef, groupby);
        if (groupKey) {
            return _.toLower(groupKey);
        }
    });
    // extract the grouped data based on the field obtained from 'groupDataByField'.
    var groupedLiData = _.groupBy(fieldDefs, groupDataByField.bind(undefined, groupby, match, innerItem, dateFormat, datePipe, AppDefaults));
    momentLocale._calendar = momentCalendarOptions; // Reset to default moment calendar options
    return groupedLiData;
};
var ɵ1$3 = getGroupedData;
// Format the date with given date format
var filterDate = function (value, format, defaultFormat, datePipe) {
    if (format === 'timestamp') { // For timestamp format, return the epoch value
        return value;
    }
    return getFormattedDate(datePipe, value, format || defaultFormat);
};
/**
 * This method returns the groupkey based on the rollup (match) passed
 *
 * @param concatStr, string containing groupby field value
 * @param rollUp string containing the match property.
 * @param dateformat string containing the date format to display the date.
 */
var getTimeRolledUpString = function (concatStr, rollUp, dateformat, datePipe, AppDefaults) {
    var groupByKey, strMoment = moment(concatStr), dateFormat = dateformat;
    var currMoment = moment(), getSameElseFormat = function () {
        return '[' + filterDate(this.valueOf(), dateFormat, ROLLUP_PATTERNS.DAY, datePipe) + ']';
    };
    switch (rollUp) {
        case TIME_ROLLUP_OPTIONS.HOUR:
            dateFormat = dateFormat || AppDefaults.timeFormat;
            // If date is invalid, check if data is in form of hh:mm a
            if (!strMoment.isValid()) {
                strMoment = moment(new Date().toDateString() + ' ' + concatStr);
                if (strMoment.isValid()) {
                    // As only time is present, roll up at the hour level with given time format
                    momentLocale._calendar.sameDay = function () {
                        return '[' + filterDate(this.valueOf(), dateFormat, ROLLUP_PATTERNS.HOUR, datePipe) + ']';
                    };
                }
            }
            // round off to nearest last hour
            strMoment = strMoment.startOf('hour');
            momentLocale._calendar.sameElse = getSameElseFormat;
            groupByKey = strMoment.calendar(currMoment);
            break;
        case TIME_ROLLUP_OPTIONS.WEEK:
            groupByKey = filterDate(strMoment.valueOf(), dateFormat, ROLLUP_PATTERNS.WEEK, datePipe);
            break;
        case TIME_ROLLUP_OPTIONS.MONTH:
            groupByKey = filterDate(strMoment.valueOf(), dateFormat, ROLLUP_PATTERNS.MONTH, datePipe);
            break;
        case TIME_ROLLUP_OPTIONS.YEAR:
            groupByKey = strMoment.format(ROLLUP_PATTERNS.YEAR);
            break;
        case TIME_ROLLUP_OPTIONS.DAY:
            dateFormat = dateFormat || AppDefaults.dateFormat;
            strMoment = strMoment.startOf('day'); // round off to current day
            momentLocale._calendar.sameElse = getSameElseFormat;
            groupByKey = strMoment.calendar(currMoment);
            break;
    }
    // If invalid date is returned, Categorize it as Others.
    if (groupByKey === 'Invalid date') {
        return GROUP_BY_OPTIONS.OTHERS;
    }
    return groupByKey;
};
var ɵ2$2 = getTimeRolledUpString;
// groups the fields based on the groupby value.
var groupDataByField = function (groupby, match, innerItem, dateFormat, datePipe, AppDefaults, liData) {
    // get the groupby field value from the liData or innerItem in the liData.
    var concatStr = _.get(innerItem ? liData[innerItem] : liData, groupby);
    // by default set the undefined groupKey as 'others'
    if (_.isUndefined(concatStr) || _.isNull(concatStr) || concatStr.toString().trim() === '') {
        return GROUP_BY_OPTIONS.OTHERS;
    }
    // if match prop is alphabetic ,get the starting alphabet of the word as key.
    if (match === GROUP_BY_OPTIONS.ALPHABET) {
        concatStr = concatStr.substr(0, 1);
    }
    // if match contains the time options then get the concatStr using 'getTimeRolledUpString'
    if (_.includes(_.values(TIME_ROLLUP_OPTIONS), match)) {
        concatStr = getTimeRolledUpString(concatStr, match, dateFormat, datePipe, AppDefaults);
    }
    return concatStr;
};
var ɵ3$1 = groupDataByField;
/**
 * This method toggles all the list items inside the each list group.
 * @param el, component reference on which groupby is applied.
 */
var toggleAllHeaders = function (el) {
    var groups = $(el.nativeElement).find('.item-group');
    groups.find('.group-list-item').toggle();
    // toggle the collapse icon on list header.
    var groupIcons = groups.find('li.list-group-header .app-icon');
    if (groupIcons) {
        _.forEach(groupIcons, function (icon) {
            icon = $(icon);
            if (icon.hasClass('wi-chevron-down')) {
                icon.removeClass('wi-chevron-down').addClass('wi-chevron-up');
            }
            else {
                icon.removeClass('wi-chevron-up').addClass('wi-chevron-down');
            }
        });
    }
};
/**
 * On list header click, toggle the list items in this group.
 * and also toggle the header icon.
 * @param $event
 */
var handleHeaderClick = function ($event) {
    var selectedGroup = $($event.target).closest('.item-group'), selectedAppIcon = selectedGroup.find('li.list-group-header .app-icon');
    if (selectedAppIcon.hasClass('wi-chevron-down')) {
        selectedAppIcon.removeClass('wi-chevron-down').addClass('wi-chevron-up');
    }
    else {
        selectedAppIcon.removeClass('wi-chevron-up').addClass('wi-chevron-down');
    }
    selectedGroup.find('.group-list-item').toggle();
};
/**
 * configures reordering the items.
 * @param $el element to be sortable
 * @param options object containing the sortable options.
 * @param startCb callback on drag start on the element.
 * @param updateCb callback triggerred when sorting is stopped and the DOM position has changed.
 */
var configureDnD = function ($el, options, startCb, updateCb) {
    var sortOptions = Object.assign({
        containment: $el,
        delay: 100,
        opacity: 0.8,
        helper: 'clone',
        zIndex: 1050,
        tolerance: 'pointer',
        start: startCb,
        update: updateCb
    }, options);
    $el.sortable(sortOptions);
};
/**
 * key represents the datafield value
 * label represents display value or expression value
 * value displayValue for primitives and data object for allFields
 * dataObject represent the object from the dataset when datafield is ALLFIELDS. This is used as innerItem while grouping the datasetItems.
 * imgSrc picture source
 * selected represents boolean to notify selected item.
 */
var DataSetItem = /** @class */ (function () {
    function DataSetItem() {
    }
    return DataSetItem;
}());

var VIEW_MODE_OPTIONS = {
    DEFAULT: 'default',
    LABEL: 'label'
};
var EDIT_MODE = {
    QUICK_EDIT: 'quickedit',
    INLINE: 'inline',
    FORM: 'form',
    DIALOG: 'dialog'
};
// Method to set the header config of the data table
var setHeaderConfig = function (headerConfig, config, field, index) {
    _.forEach(headerConfig, function (cols) {
        if (_.get(cols, 'isGroup')) {
            if (cols.field === field) {
                cols.columns[index] = config;
            }
            else {
                setHeaderConfig(cols.columns, config, field, index);
            }
        }
    });
};
var setHeaderConfigForTable = function (headerConfig, config, fieldName, index) {
    if (fieldName) {
        setHeaderConfig(headerConfig, config, fieldName, index);
    }
    else {
        headerConfig[index] = config;
    }
};
var getRowOperationsColumn = function () {
    return {
        'field': 'rowOperations',
        'type': 'custom',
        'displayName': 'Actions',
        'width': '120px',
        'readonly': true,
        'sortable': false,
        'searchable': false,
        'resizable': false,
        'selectable': false,
        'show': true,
        'operations': [],
        'opConfig': {},
        'pcDisplay': true,
        'mobileDisplay': true,
        'tabletDisplay': true,
        'include': true,
        'isRowOperation': true
    };
};
/**
 * Returns caption and widget bootstrap classes for the field
 */
var getFieldLayoutConfig = function (captionWidth, captionPosition, os) {
    var captionCls = '', widgetCls = '';
    captionPosition = captionPosition || 'top';
    if (captionPosition === 'top') {
        if ((os && os === 'android') || !isMobileApp() || isAndroid()) { // Is android or not a mobile application
            captionCls = widgetCls = 'col-xs-12';
        }
        else if (isMobileApp()) { // Is a mobile application and not android
            captionCls = 'col-xs-4';
            widgetCls = 'col-xs-8';
        }
    }
    else if (captionWidth) {
        // handling itemsperrow containing string of classes
        _.forEach(_.split(captionWidth, ' '), function (cls) {
            var keys = _.split(cls, '-'), tier = keys[0];
            var _captionWidth, widgetWidth;
            _captionWidth = parseInt(keys[1], 10);
            widgetWidth = 12 - _captionWidth;
            widgetWidth = widgetWidth <= 0 ? 12 : widgetWidth;
            captionCls += ' ' + 'col-' + tier + '-' + _captionWidth;
            widgetCls += ' ' + 'col-' + tier + '-' + widgetWidth;
        });
    }
    return {
        'captionCls': captionCls,
        'widgetCls': widgetCls
    };
};
var getDefaultViewModeWidget = function (widget) {
    if (_.includes(['checkbox', 'toggle', 'rating'], widget)) {
        return VIEW_MODE_OPTIONS.DEFAULT;
    }
    return VIEW_MODE_OPTIONS.LABEL;
};
var parseBooleanValue = function (value) {
    if (value === 'true') {
        return true;
    }
    if (value === 'false') {
        return false;
    }
    if (/^\d+$/.test(value)) { // Check if the value is a string of number type like '123'
        return +value;
    }
    return value;
};
var ɵ0$9 = parseBooleanValue;
var parseValueByType = function (value, type, widget, trailingzero) {
    if (widget) {
        if (widget === FormWidgetType.NUMBER || widget === FormWidgetType.SLIDER || widget === FormWidgetType.CURRENCY) {
            return isNaN(Number(value)) ? null : trailingzero ? value : Number(value);
        }
        if (widget === FormWidgetType.CHECKBOX || widget === FormWidgetType.TOGGLE) {
            return parseBooleanValue(value);
        }
        return value;
    }
    if (isNumberType(type)) {
        return isNaN(Number(value)) ? null : Number(value);
    }
    if (type === DataType.BOOLEAN) {
        return parseBooleanValue(value);
    }
    return value;
};
var fieldTypeWidgetTypeMap = {
    'integer': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
    'big_integer': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
    'short': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'float': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'big_decimal': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'number': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'double': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'long': ['number', 'text', 'select', 'checkboxset', 'radioset', 'rating', 'slider', 'currency', 'autocomplete', 'chips'],
    'byte': ['number', 'text', 'select', 'checkboxset', 'radioset', 'slider', 'currency', 'autocomplete', 'chips'],
    'string': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'switch', 'currency', 'autocomplete', 'chips', 'colorpicker'],
    'character': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'switch', 'currency', 'autocomplete', 'chips'],
    'text': ['text', 'number', 'textarea', 'password', 'richtext', 'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'switch', 'currency', 'autocomplete', 'chips', 'colorpicker'],
    'date': ['date', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'time': ['time', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'timestamp': ['timestamp', 'text', 'number', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'datetime': ['datetime', 'text', 'select', 'checkboxset', 'radioset', 'autocomplete', 'chips'],
    'boolean': ['checkbox', 'radioset', 'toggle', 'select'],
    'list': ['select', 'radioset', 'checkboxset', 'switch', 'autocomplete', 'chips'],
    'clob': ['text', 'textarea', 'richtext'],
    'blob': ['upload'],
    'file': ['upload'],
    'custom': ['text', 'number', 'textarea', 'password', 'checkbox', 'toggle', 'slider', 'richtext', 'currency', 'switch',
        'select', 'checkboxset', 'radioset', 'date', 'time', 'timestamp', 'rating', 'datetime', 'autocomplete', 'chips', 'colorpicker']
};
// Get widgets applicable to the given type
var getFieldTypeWidgetTypesMap = function () {
    return fieldTypeWidgetTypeMap;
};
// Get filter widget applicable to the given type
var getDataTableFilterWidget = function (type) {
    var widget = fieldTypeWidgetTypeMap[type] && fieldTypeWidgetTypeMap[type][0];
    if (type === DataType.BOOLEAN) {
        widget = FormWidgetType.SELECT;
    }
    if (_.includes([FormWidgetType.TEXT, FormWidgetType.NUMBER, FormWidgetType.SELECT, FormWidgetType.AUTOCOMPLETE,
        FormWidgetType.DATE, FormWidgetType.TIME, FormWidgetType.DATETIME], widget)) {
        return widget;
    }
    return FormWidgetType.TEXT;
};
/**
 * @ngdoc function
 * @name wm.widgets.live.getEditModeWidget
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * This function returns the default widget for grid
 *
 * @param {object} colDef field definition
 */
var getEditModeWidget = function (colDef) {
    if (colDef['related-entity-name'] && colDef['primary-key']) {
        return FormWidgetType.SELECT;
    }
    return (fieldTypeWidgetTypeMap[colDef.type] && fieldTypeWidgetTypeMap[colDef.type][0]) || FormWidgetType.TEXT;
};
/**
 * @ngdoc function
 * @name wm.widgets.live.LiveWidgetUtils#getDefaultValue
 * @methodOf wm.widgets.live.LiveWidgetUtils
 * @function
 *
 * @description
 * return the formatted default value
 *
 * @param {string} value value to be formatted
 * @param {string} type column type of the value
 */
var getDefaultValue = function (value, type, widget) {
    if (widget) {
        if (widget === FormWidgetType.NUMBER || widget === FormWidgetType.SLIDER || widget === FormWidgetType.CURRENCY) {
            return isNaN(Number(value)) ? null : Number(value);
        }
        if (widget === FormWidgetType.CHECKBOX || widget === FormWidgetType.TOGGLE) {
            return parseBooleanValue(value);
        }
        return value;
    }
    if (isNumberType(type)) {
        return isNaN(Number(value)) ? null : Number(value);
    }
    if (type === DataType.BOOLEAN) {
        return parseBooleanValue(value);
    }
    return value;
};

var getValidLink = function (link) {
    var routRegex = /^(\/|#\/|#)(?!\W).*/;
    if (link) {
        if (routRegex.test(link)) {
            link = _.first(link.match(/[\w]+.*/g)) || '';
            return "#/" + link;
        }
        if (_.startsWith(link, 'www.')) {
            return "//" + link;
        }
        return link;
    }
};
var ɵ0$a = getValidLink;
var DatasetAwareNavComponent = /** @class */ (function (_super) {
    __extends(DatasetAwareNavComponent, _super);
    function DatasetAwareNavComponent(inj, WIDGET_CONFIG) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.nodes = [];
        _this.nodes$ = new Subject();
        // debounce function for reset nodes functions.
        _this._resetNodes = _.debounce(_this.resetNodes, 50);
        _this.securityService = _this.inj.get(SecurityService);
        _this.binditemlabel = _this.nativeElement.getAttribute('itemlabel.bind');
        _this.binditemicon = _this.nativeElement.getAttribute('itemicon.bind');
        _this.binditemaction = _this.nativeElement.getAttribute('itemaction.bind');
        _this.binditembadge = _this.nativeElement.getAttribute('itembadge.bind');
        _this.binditemchildren = _this.nativeElement.getAttribute('itemchildren.bind');
        _this.binditemid = _this.nativeElement.getAttribute('itemid.bind');
        _this.binditemlink = _this.nativeElement.getAttribute('itemlink.bind');
        _this.binditemtarget = _this.nativeElement.getAttribute('itemtarget.bind');
        _this.binduserrole = _this.nativeElement.getAttribute('userrole.bind');
        _this.bindisactive = _this.nativeElement.getAttribute('isactive.bind');
        return _this;
    }
    /**
     * constructs individual node for the widget model.
     * @param fields
     * @param node
     */
    DatasetAwareNavComponent.prototype.getNode = function (fields, node) {
        var context = this.viewParent.pageScope || this.viewParent;
        var children = getEvaluatedData(node, { expression: 'itemchildren', bindExpression: this.binditemchildren }, context) || _.get(node, fields.childrenField);
        var navNode = {
            action: getEvaluatedData(node, { expression: 'itemaction', bindExpression: this.binditemaction }, context) || _.get(node, fields.actionField),
            badge: getEvaluatedData(node, { expression: 'itembadge', bindExpression: this.binditembadge }, context) || _.get(node, fields.badgeField),
            children: Array.isArray(children) ? this.getNodes(children) : [],
            class: _.get(node, fields.classField),
            disabled: node.disabled,
            icon: getEvaluatedData(node, { expression: 'itemicon', bindExpression: this.binditemicon }, context) || _.get(node, fields.iconField),
            id: getEvaluatedData(node, { expression: 'itemid', bindExpression: this.binditemid }, context) || _.get(node, fields.idField),
            label: getEvaluatedData(node, { expression: 'itemlabel', bindExpression: this.binditemlabel }, context) || _.get(node, fields.labelField),
            link: getValidLink(getEvaluatedData(node, { expression: 'itemlink', bindExpression: this.binditemlink }, context) || _.get(node, fields.linkField)),
            target: getValidLink(getEvaluatedData(node, { expression: 'itemtarget', bindExpression: this.binditemtarget }, context) || _.get(node, fields.targetField)),
            role: getEvaluatedData(node, { expression: 'userrole', bindExpression: this.binduserrole }, context),
            isactive: this.bindisactive ? getEvaluatedData(node, { expression: 'isactive', bindExpression: this.bindisactive }, context) : _.get(node, fields.isactiveField),
            // older projects have display field & data field property for menu.
            value: this.datafield ? (this.datafield === 'All Fields' ? node : findValueOf(node, this.datafield)) : node
        };
        return _.omitBy(navNode, _.isUndefined);
    };
    DatasetAwareNavComponent.prototype.resetItemFieldMap = function () {
        this._itemFieldMap = null;
    };
    DatasetAwareNavComponent.prototype.getItemFieldsMap = function () {
        if (!this._itemFieldMap) {
            this._itemFieldMap = {
                idField: this.itemid || 'itemid',
                iconField: this.itemicon || 'icon',
                labelField: this.itemlabel || 'label',
                linkField: this.itemlink || 'link',
                targetField: this.itemtarget || 'target',
                badgeField: this.itembadge || 'badge',
                childrenField: this.itemchildren || 'children',
                classField: this.itemclass || 'class',
                actionField: this.itemaction || 'action',
                isactiveField: this.isactive || 'isactive'
            };
        }
        return this._itemFieldMap;
    };
    /**
     * returns array for the value passed as nv.
     * nv: 'a,b' => [{label:a, value:a}, {label:b, value:b}]
     * nv: [1,2] => [{label:1, value:1}, {label:2, value:2}]
     * nv: [{obj}, {obj}] => [{obj}, {obj}]
     * @param nv
     */
    DatasetAwareNavComponent.prototype.prepareNodeDataSet = function (nv) {
        nv = createArrayFrom(nv);
        return nv.map(function (val) {
            if (!isObject(val)) {
                return {
                    label: val,
                    value: val
                };
            }
            return val;
        });
    };
    /**
     * constructs dataset form the nav elements.
     */
    DatasetAwareNavComponent.prototype.getNodes = function (nv) {
        var _this = this;
        if (nv === void 0) { nv = this.dataset || {}; }
        var nodes = getOrderedDataset(this.prepareNodeDataSet(nv), this.orderby) || [];
        if (nodes.length) {
            var userRole_1 = this.userrole;
            var nodeFields_1 = this.getItemFieldsMap();
            nodes = nodes.reduce(function (result, node) {
                if (validateAccessRoles(node[userRole_1], _this.securityService.loggedInUser)) {
                    result.push(_this.getNode(nodeFields_1, node));
                }
                return result;
            }, []);
        }
        return nodes;
    };
    // enable the inherited class to extend this method.
    DatasetAwareNavComponent.prototype.resetNodes = function () {
        this.resetItemFieldMap();
        this.nodes = this.getNodes();
        $appDigest();
        // notify the node listeners
        this.nodes$.next();
    };
    DatasetAwareNavComponent.prototype.onPropertyChange = function (key, nv, ov) {
        switch (key) {
            case 'dataset':
            case 'itemicon':
            case 'itemlabel':
            case 'itemlink':
            case 'itemtarget':
            case 'itemclass':
            case 'itemchildren':
            case 'isactive':
            case 'orderby':
                // calls resetnodes method after 50ms. any calls within 50ms will be ignored.
                this._resetNodes();
                break;
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    return DatasetAwareNavComponent;
}(StylableComponent));

var DEFAULT_VALIDATOR = {
    pattern: 'regexp',
    max: 'maxvalue',
    min: 'minvalue',
    required: 'required',
    maxlength: 'maxchars',
    mindate: 'mindate',
    maxdate: 'maxdate',
    excludedates: 'excludedates',
    excludedays: 'excludedays'
};
var BaseFieldValidations = /** @class */ (function () {
    function BaseFieldValidations(instance, formwidget, widgettype, widgetControl, widgetContext, tableFieldType) {
        this.instance = instance;
        this.formwidget = formwidget;
        this.widgettype = widgettype;
        this.widgetControl = widgetControl;
        this.widgetContext = widgetContext;
        this.defaultValidatorMessages = [];
        if (tableFieldType) {
            this.tableFieldType = tableFieldType;
        }
    }
    // this method returns the collection of supported default validators
    BaseFieldValidations.prototype.getDefaultValidators = function () {
        var _validator = [];
        if (this.instance.required && this.instance.show !== false) {
            // For checkbox/toggle widget, required validation should consider true value only
            if (this.widgettype === FormWidgetType.CHECKBOX || this.widgettype === FormWidgetType.TOGGLE) {
                _validator.push(Validators.requiredTrue);
            }
            else {
                _validator.push(Validators.required);
            }
        }
        if (this.instance.maxchars) {
            _validator.push(Validators.maxLength(this.instance.maxchars));
        }
        if (this.instance.minvalue) {
            _validator.push(Validators.min(this.instance.minvalue));
        }
        if (this.instance.maxvalue && this.widgettype !== FormWidgetType.RATING) {
            _validator.push(Validators.max(this.instance.maxvalue));
        }
        if (this.instance.regexp) {
            _validator.push(Validators.pattern(this.instance.regexp));
        }
        if (this.formwidget && _.isFunction(this.formwidget.validate)) {
            _validator.push(this.formwidget.validate.bind(this.formwidget));
        }
        return _validator;
    };
    // On change of any validation property, set the angular form validators
    BaseFieldValidations.prototype.setUpValidators = function (customValidator) {
        if (this.hasValidators) {
            return;
        }
        this.instance._validators = this.getDefaultValidators();
        if (customValidator) {
            this.instance._validators.push(customValidator);
        }
        if (this.widgetContext.ngform) {
            this.widgetControl.setValidators(this.instance._validators);
            var opt = {};
            // updating the value only when prevData is not equal to current value.
            // emitEvent flag will prevent from emitting the valueChanges when value is equal to the prevDatavalue.
            if (this.formwidget && this.instance.value === this.formwidget.prevDatavalue) {
                opt['emitEvent'] = false;
            }
            this.widgetControl.updateValueAndValidity(opt);
        }
    };
    BaseFieldValidations.prototype.getPromiseList = function (validators) {
        var _this = this;
        var arr = [];
        _.forEach(validators, function (fn, index) {
            var promise = fn;
            if (fn instanceof Function && fn.bind) {
                promise = fn(_this.widgetControl, _this.widgetContext);
            }
            if (promise instanceof Promise) {
                arr.push(promise);
            }
        });
        return arr;
    };
    // this method sets the asyncValidation on the form field. Assigns validationmessages from the returned response
    BaseFieldValidations.prototype.setAsyncValidators = function (validators) {
        var _this = this;
        this._asyncValidatorFn = function () {
            return function () {
                return Promise.all(_this.getPromiseList(validators)).then(function () {
                    _this.instance.validationmessage = '';
                    return null;
                }, function (err) {
                    // if err obj has validationMessage key, then set validationMessage using this value
                    // else return the value of the first key in the err object as validation message.
                    if (err.hasOwnProperty('errorMessage')) {
                        _this.instance.validationmessage = _.get(err, 'errorMessage');
                    }
                    else {
                        var keys = _.keys(err);
                        _this.instance.validationmessage = (err[keys[0]]).toString();
                    }
                    return err;
                }).then(function (response) {
                    // form control status is not changed from pending. This is an angular issue refer https://github.com/angular/angular/issues/13200
                    var checkForStatusChange = function () {
                        setTimeout(function () {
                            if (_this.widgetControl.status === 'PENDING') {
                                checkForStatusChange();
                            }
                            else {
                                _this.instance.onStatusChange(_this.widgetControl.status, _this.tableFieldType);
                            }
                        }, 100);
                    };
                    checkForStatusChange();
                    return response;
                });
            };
        };
        this.widgetControl.setAsyncValidators([this._asyncValidatorFn()]);
        if (this.widgetContext.ngform.touched) {
            this.widgetControl.updateValueAndValidity();
        }
    };
    BaseFieldValidations.prototype.isDefaultValidator = function (type) {
        return _.get(VALIDATOR, _.toUpper(type));
    };
    // default validator is bound to a function then watch for value changes
    // otherwise set the value of default validator directly
    BaseFieldValidations.prototype.setDefaultValidator = function (key, value) {
        if (value && value instanceof Function) {
            // passing formfield and form as arguments to the default validator function
            this.watchDefaultValidatorExpr(value.bind(undefined, this.widgetControl, this.widgetContext), key);
        }
        else {
            this.instance.widget[key] = value;
            this.instance[key] = value;
        }
    };
    // sets the default validation on the form field
    BaseFieldValidations.prototype.setValidators = function (validators) {
        var _this = this;
        var _cloneValidators = _.cloneDeep(validators);
        this.hasValidators = true;
        this._syncValidators = [];
        _.forEach(_cloneValidators, function (obj, index) {
            // custom validation is bound to function.
            if (obj && obj instanceof Function) {
                // passing formfield and form as arguments to the obj (i.e. validator function)
                _cloneValidators[index] = obj.bind(undefined, _this.widgetControl, _this.widgetContext);
                _this._syncValidators.push(_cloneValidators[index]);
            }
            else {
                // checks for default validator like required, maxchars etc.
                var key = _.get(obj, 'type');
                _this.defaultValidatorMessages[key] = _.get(obj, 'errorMessage');
                if (_this.isDefaultValidator(key)) {
                    var value = _.get(obj, 'validator');
                    _this.setDefaultValidator(key, value);
                    _cloneValidators[index] = '';
                }
            }
        });
        // _syncValidators contains all the custom validations on the form field. will not include default validators.
        this._syncValidators = _.filter(_cloneValidators, function (val) {
            if (val) {
                return val;
            }
        });
        this.applyDefaultValidators();
    };
    BaseFieldValidations.prototype.observeOn = function (fields, context) {
        var _this = this;
        _.forEach(fields, function (field) {
            var formfield = _.find(_this.widgetContext[context], { 'key': field });
            if (formfield) {
                if (!formfield.notifyForFields) {
                    formfield.notifyForFields = [];
                }
                formfield.notifyForFields.push(_this.instance);
            }
        });
    };
    BaseFieldValidations.prototype.validate = function () {
        this.applyDefaultValidators();
        if (this._asyncValidatorFn) {
            this.widgetControl.setAsyncValidators([this._asyncValidatorFn()]);
            this.widgetControl.updateValueAndValidity();
        }
        // show the validation erros show when form is touched and not on load. This just highlights the field that is subscribed for changes.
        if (this.widgetContext.ngform.touched) {
            this.widgetControl.markAsTouched();
        }
    };
    // watches for changes in the bound function for default validators.
    BaseFieldValidations.prototype.watchDefaultValidatorExpr = function (fn, key) {
        var _this = this;
        var watchName = this.instance.widgetId + "_" + key + '_formField';
        $unwatch(watchName);
        this.instance.registerDestroyListener($watch('boundFn(fn)', _.extend(this.instance, this.instance.viewParent), { fn: fn }, function (nv, ov) {
            _this.instance.widget[key] = nv;
            _this.applyDefaultValidators();
        }, watchName));
    };
    // invokes both custom sync validations and default validations.
    BaseFieldValidations.prototype.applyDefaultValidators = function () {
        var validators = this.getDefaultValidators();
        this.widgetControl.setValidators(_.concat(this._syncValidators || [], validators));
        this.widgetControl.updateValueAndValidity();
        this.setCustomValidationMessage();
    };
    BaseFieldValidations.prototype.setCustomValidationMessage = function () {
        var fieldErrors = this.widgetControl && this.widgetControl.errors;
        if (!fieldErrors) {
            return;
        }
        if (fieldErrors.hasOwnProperty('errorMessage')) {
            this.instance.validationmessage = _.get(fieldErrors, 'errorMessage');
        }
        else {
            var keys = _.keys(fieldErrors);
            var key = keys[0];
            var validationMsgKey = _.get(DEFAULT_VALIDATOR, key) || this.formwidget.validateType;
            if (validationMsgKey) {
                var msg = _.get(this.defaultValidatorMessages, validationMsgKey) || this.instance.validationmessage;
                if (msg && msg instanceof Function) {
                    // passing formfield and form as arguments to the errorMessage function.
                    this.instance.validationmessage = msg(this.widgetControl, this.widgetContext);
                }
                else {
                    this.instance.validationmessage = msg;
                }
            }
            else {
                // fallback when there is no validationmessage for fields other than default validators.
                this.instance.validationmessage = '';
            }
        }
    };
    return BaseFieldValidations;
}());

var PullToRefresh = /** @class */ (function (_super) {
    __extends(PullToRefresh, _super);
    function PullToRefresh($el, app, onPullToRefresh) {
        var _this = _super.call(this) || this;
        _this.$el = $el;
        _this.app = app;
        _this.onPullToRefresh = onPullToRefresh;
        _this.count = 0;
        var template = '<div class="refresh-container"></div>';
        _this.$el.prepend(template);
        _this.infoContainer = _this.$el.find('.refresh-container');
        if (isIos()) {
            _this.infoContainer.addClass('ios-refresh-container');
        }
        _this.init(_this.$el);
        return _this;
    }
    PullToRefresh.prototype.threshold = function () {
        return 10;
    };
    PullToRefresh.prototype.direction = function () {
        return $.fn.swipey.DIRECTIONS.VERTICAL;
    };
    PullToRefresh.prototype.subscribe = function () {
        var _this = this;
        // Subscribing for variable updates, wait till the response and stop the animation.
        this.cancelSubscription = this.app.subscribe('toggle-variable-state', function (data) {
            // data.active is true means the variable update has just started whereas false means update has ended.
            if (data.active && _this.animationInProgress) {
                _this.count++;
                _this.wait();
            }
            else if (_this.count > 0) {
                _this.count--;
            }
            // call stop animation only when animation has started.
            if (!_this.count && _this.animationInProgress) {
                _this.stopAnimation();
            }
        });
    };
    PullToRefresh.prototype.bounds = function ($event, $d) {
        var pageIscroll = this.$el[0].iscroll;
        // If scroll position is not at the top of the page then do not trigger the animation.
        if ($d < 0 || (pageIscroll && !isNaN(pageIscroll.y) && pageIscroll.y !== 0)) {
            return {
                lower: undefined,
                upper: undefined
            };
        }
        if (!this.spinner) {
            this.spinner = isIos() ? new IOSSpinner(this.infoContainer) : new AndroidSpinner(this.infoContainer);
            this.subscribe();
        }
        return {
            lower: 0,
            center: 0,
            upper: isIos() ? 0 : 150,
            strict: !isIos()
        };
    };
    PullToRefresh.prototype.context = function () {
        var _this = this;
        this.infoContainer.show();
        if (isIos()) {
            this.infoContainer.addClass('entry');
        }
        return {
            spin: function ($d) {
                _this.spinner.setRotation($d * 2);
            }
        };
    };
    PullToRefresh.prototype.animation = function () {
        if (isIos()) {
            return {
                css: {
                    transform: 'translate3d(0, ${{$D + $d}}px, 0)',
                    spin: '${{spin($d)}}'
                }
            };
        }
        return {
            target: this.infoContainer,
            css: {
                transform: 'translate3d(0, ${{$D + $d}}px, 0)',
                spin: '${{spin($d)}}',
                opacity: '${{min(($D + $d) / 100, 1)}}'
            }
        };
    };
    // Start the spinner animation and invokes the pulltorefresh event. Stops the animation after the wait time.
    PullToRefresh.prototype.onAnimation = function () {
        var _this = this;
        this.animationInProgress = true;
        this.spinner.start();
        if (this.onPullToRefresh) {
            this.onPullToRefresh();
        }
        setTimeout(function () {
            // if listenToAnimation is set, then wait for stopAnimation to be invoked. Otherwise call stopAnimation manually.
            if (_this.runAnimation) {
                return;
            }
            _this.stopAnimation();
        });
    };
    PullToRefresh.prototype.stopAnimation = function () {
        var _this = this;
        setTimeout(function () {
            _this.runAnimation = false;
            _this.animationInProgress = false;
            _this.spinner.stop();
            _this.infoContainer.hide();
            setCSS(_this.infoContainer[0], 'transform', 'none');
            if (!isIos()) {
                setCSS(_this.infoContainer[0], 'opacity', 0);
            }
            _this.infoContainer.removeClass('entry');
        }, 800);
    };
    PullToRefresh.prototype.wait = function () {
        this.runAnimation = true;
    };
    return PullToRefresh;
}(SwipeAnimation));
// Android Spinner implementation
var AndroidSpinner = /** @class */ (function () {
    function AndroidSpinner($el, options) {
        this.$el = $el;
        this.options = options || {};
    }
    AndroidSpinner.prototype.polarToCartesian = function (centerX, centerY, radius, angleInDegrees) {
        var angleInRadians = (angleInDegrees - 90) * Math.PI / 180.0;
        return {
            x: centerX + (radius * Math.cos(angleInRadians)),
            y: centerY + (radius * Math.sin(angleInRadians))
        };
    };
    AndroidSpinner.prototype.describeArc = function (x, y, radius, startAngle, endAngle) {
        var start = this.polarToCartesian(x, y, radius, endAngle);
        var end = this.polarToCartesian(x, y, radius, startAngle);
        var arcSweep = endAngle - startAngle <= 180 ? '0' : '1';
        return [
            'M', start.x, start.y,
            'A', radius, radius, 0, arcSweep, 0, end.x, end.y,
            'L', end.x, end.y
        ].join(' ');
    };
    AndroidSpinner.prototype.init = function () {
        if (this.initialized) {
            return;
        }
        // append the svg to the element on which spinner has to be shown.
        var androidTemplate = $('<svg><path id="arc1"/></svg>');
        var container = $('<div class="android-spinner"></div>');
        container.append(androidTemplate);
        this.$el.append(container);
        this.cx = container.outerWidth() / 2;
        this.cy = container.outerHeight() / 2;
        this.r = this.cx * 0.6;
        this.path = container.find('path');
        this.initialized = true;
        this.container = container;
    };
    // sets rotation to the path
    AndroidSpinner.prototype.setRotation = function (deg) {
        this.init();
        this.path.attr('d', this.describeArc(this.cx, this.cy, this.r, 0, deg));
    };
    // animate the path to rotate continuously with some interval
    AndroidSpinner.prototype.start = function () {
        var _this = this;
        this.init();
        var self = this, totalTime = this.options.time || 1800, degreesToTraverse = 10;
        var i = 0;
        this.container.addClass('spin');
        this.intervalId = setInterval(function () {
            var deg;
            i = i + degreesToTraverse;
            if (i > 720) {
                deg = i = 0;
            }
            else if (i > 360) {
                deg = 720 - i;
            }
            else {
                deg = i;
            }
            self.path.attr('d', _this.describeArc(self.cx, self.cy, self.r, 0, deg));
        }, (totalTime * degreesToTraverse / 360));
    };
    // Removes the animation by clearing the intervals
    AndroidSpinner.prototype.stop = function () {
        if (this.intervalId) {
            clearInterval(this.intervalId);
            this.intervalId = 0;
        }
        this.container.removeClass('spin');
    };
    return AndroidSpinner;
}());
// iOS spinner implementation
var IOSSpinner = /** @class */ (function () {
    function IOSSpinner($el) {
        this.$el = $el;
    }
    // create the iOS spinner using svg
    IOSSpinner.prototype.init = function () {
        if (this.svg) {
            return;
        }
        var container = $('<div class="ios-spinner"><svg></svg></div>');
        var svg = container.find('svg');
        this.$el.append(container);
        var d = container.width();
        for (var i = 0; i < 12; i++) {
            svg[0].innerHTML += "<line y1=\"" + 0.15 * d + "\" y2=\"" + 0.3 * d + "\"\n                                    transform=\"translate(" + 0.5 * d + ", " + 0.5 * d + ") rotate(" + i * 30 + ")\">\n                                </line>";
        }
        container.append(svg);
        this.svg = svg;
    };
    // Sets the svg to rotate depending on "d" value
    IOSSpinner.prototype.setRotation = function (d) {
        this.init();
        this.svg.css('transform', "rotateZ( " + Math.round(d / 30) * 30 + "deg)");
    };
    // Triggers the rotation with some interval
    IOSSpinner.prototype.start = function () {
        var _this = this;
        this.init();
        var i = 0;
        this.intervalId = setInterval(function () {
            i += 10;
            _this.setRotation(i);
        }, 20);
    };
    // Removes the animation by clearing the intervals
    IOSSpinner.prototype.stop = function () {
        if (this.intervalId) {
            clearInterval(this.intervalId);
            this.intervalId = 0;
        }
    };
    return IOSSpinner;
}());

var _this = this;
var CLASS_NAME;
(function (CLASS_NAME) {
    CLASS_NAME["LEFT_PANEL"] = "page-left-panel";
    CLASS_NAME["RIGHT_PANEL"] = "page-right-panel";
    CLASS_NAME["SWIPE_ELEM"] = "page-left-panel-icon";
    CLASS_NAME["CONTENT"] = "app-content-column";
    CLASS_NAME["HEADER"] = "page-header";
    CLASS_NAME["SEARCH"] = "app-search";
})(CLASS_NAME || (CLASS_NAME = {}));
/**
 * method returns jquery class selector for given roleName
 * @param roleName
 * @returns {string}
 */
var roleSelector = function (roleName) { return "[data-role='" + roleName + "']"; };
var ɵ0$b = roleSelector;
var pageSelector = roleSelector("pageContainer");
/*setup touch event handler*/
var bindTapEvtHandler = function (selector, handler) {
    /*
     * In Iphone safari browser, tap event of HammerJs is breaking
     * functionalities of other controls like input[type="range"].
     * So, replaced the hammer Js handler with click event handler.
     */
    if (typeof (selector) === 'string') {
        selector = document.querySelector(selector);
    }
    if (!selector) {
        return;
    }
    // Add js event listeners as the event handlers has to be invoked in angular zone.
    selector.removeEventListener('click', handler);
    selector.addEventListener('click', handler);
};
var ɵ1$4 = bindTapEvtHandler;
/**
 * hide the mobile toolbar actions
 */
var hidePageContainers = function (leftNavEle, searchEle) {
    if (isMobile()) {
        if (leftNavEle) {
            try {
                leftNavEle.widget.collapse();
            }
            catch (e) {
                //
            }
        }
        if (searchEle) {
            setCSS(searchEle, 'display', 'none');
        }
    }
};
var ɵ2$3 = hidePageContainers;
/**
 * binds the touch event for content
 */
var bindContentEvents = function (leftNavEle, pageContainer, searchContainer, isTablet) {
    if (isTablet === void 0) { isTablet = false; }
    if (!isTablet) {
        // touch content to hide nav pane and left panel
        bindTapEvtHandler(pageContainer, hidePageContainers.bind(_this, leftNavEle, searchContainer));
    }
};
var ɵ3$2 = bindContentEvents;
var bindLeftPanelEvents = function (leftNavEle, searchEle) {
    // tap left to show/hide left panel
    bindTapEvtHandler(roleSelector(CLASS_NAME.SWIPE_ELEM), function () {
        if (leftNavEle) {
            leftNavEle.widget.toggle();
            // Hide search container when left panel is open
            if (leftNavEle.classList.contains('visible')) {
                if (searchEle) {
                    setCSS(searchEle, 'display', 'none');
                }
            }
        }
    });
};
var ɵ4 = bindLeftPanelEvents;
/**
 * binds the touch event for content
 */
var bindRightPanelEvents = function (rightNavEle) {
    bindTapEvtHandler(rightNavEle, hidePageContainers);
};
var ɵ5 = bindRightPanelEvents;
/**
 * toggles the search container
 */
var toggleSearchContainer = function (searchEle, leftNavEle) {
    if ($(searchEle).css('display') === 'none') {
        hidePageContainers(leftNavEle);
        setCSS(searchEle, 'display', 'inline-table');
    }
    else {
        hidePageContainers(leftNavEle, searchEle);
    }
};
var ɵ6 = toggleSearchContainer;
/**
 * Bind event with Search icon in header
 */
var bindSearchIconEvent = function (searchElements, leftNavEle) {
    $(searchElements).each(function (index, ele) {
        var searchEle = $('<a class="app-header-action"><i class="wi wi-search"></i></a>');
        $(ele).before(searchEle);
        // Tap icon to show/hide search box
        bindTapEvtHandler(searchEle, function () { return toggleSearchContainer(ele, leftNavEle); });
    });
};
var ɵ7 = bindSearchIconEvent;
var updateDeviceView = function (element, isTablet) {
    if (isTablet === void 0) { isTablet = false; }
    var leftNavEle = element.querySelector(roleSelector(CLASS_NAME.LEFT_PANEL));
    var rightNavEle = element.querySelector(roleSelector(CLASS_NAME.RIGHT_PANEL));
    var headerEle = element.querySelector(roleSelector(CLASS_NAME.HEADER));
    var searchEle = headerEle && headerEle.querySelector("." + CLASS_NAME.SEARCH);
    var pageEle = element.querySelector("." + CLASS_NAME.CONTENT);
    bindContentEvents(leftNavEle, pageEle, searchEle, isTablet);
    if (leftNavEle) {
        bindLeftPanelEvents(leftNavEle, searchEle);
    }
    else {
        // remove the icon
        $(headerEle).find(roleSelector(CLASS_NAME.SWIPE_ELEM)).remove();
    }
    if (rightNavEle) {
        bindRightPanelEvents(rightNavEle);
    }
    if (searchEle) {
        bindSearchIconEvent(searchEle, leftNavEle);
    }
};

/**
 * Generated bundle index. Do not edit.
 */

export { ALLFIELDS, APPLY_STYLES_TYPE, AUTOCLOSE_TYPE, BaseComponent, BaseContainerComponent, BaseFieldValidations, Context, CustomPipe, DEBOUNCE_TIMES, DISPLAY_TYPE, DataSetItem, DatasetAwareNavComponent, DialogRef, DialogServiceImpl, EDIT_MODE, EVENTS_MAP, FileExtensionFromMimePipe, FileIconClassPipe, FileSizePipe, FilterPipe, ImagePipe, ItemTemplateDirective, LIVE_CONSTANTS, Live_Operations, MessageComponent, NAVIGATION_TYPE, NumberToStringPipe, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, PROP_STRING, PROP_TYPE, PartialContainerDirective, PartialDirective, PrefixPipe, PullToRefresh, RedrawableDirective, StateClassPipe, StringToNumberPipe, StylableComponent, SuffixPipe, TimeFromNowPipe, ToCurrencyPipe, ToDatePipe, ToNumberPipe, TrailingZeroDecimalPipe, TrustAsPipe, WidgetRef, WmComponentsModule, applyFilterOnField, configureDnD, convertDataToObject, createArrayFrom, extractDataAsArray, extractDataSourceName, fetchDistinctValues, fetchRelatedFieldData, filterDate, getBackGroundImageUrl, getConditionalClasses, getContainerTargetClass, getDataTableFilterWidget, getDefaultValue, getDefaultViewModeWidget, getDistinctFieldProperties, getDistinctValues, getDistinctValuesForField, getEditModeWidget, getEmptyMatchMode, getEnableEmptyFilter, getEvaluatedData, getFieldLayoutConfig, getFieldTypeWidgetTypesMap, getImageUrl, getMatchModeMsgs, getMatchModeTypesMap, getObjValueByKey, getOrderByExpr, getOrderedDataset, getRangeFieldValue, getRangeMatchMode, getRowOperationsColumn, getUniqObjsByDataField, getWatchIdentifier, getWidgetPropsByType, groupData, handleHeaderClick, interpolateBindExpressions, isActiveNavItem, isBooleanAttr, isDataSetWidget, isDimensionProp, isStyle, parseValueByType, performDataOperation, prepareFieldDefs, propNameCSSKeyMap, provideAs, provideAsDialogRef, provideAsWidgetRef, refreshDataSource, register, registerFormWidget, setHeaderConfig, setHeaderConfigForTable, setItemByCompare, styler, toggleAllHeaders, transformData, transformDataWithKeys, transformFormData, updateDeviceView, ɵ0$7 as ɵ0, ContainerDirective as ɵa, registerProps as ɵb, registerProps$1 as ɵc, LazyLoadDirective as ɵd, registerProps$2 as ɵe, registerProps$3 as ɵf, PartialParamHandlerDirective as ɵg, PartialParamDirective as ɵh, ShowInDeviceDirective as ɵi, SmoothScrollDirective as ɵj, TextContentDirective as ɵk };
//# sourceMappingURL=index.js.map
