import { __decorate, __metadata, __param } from 'tslib';
import { Injector, HostBinding, Component, SecurityContext, Attribute, Directive, NgModule } from '@angular/core';
import { EventNotifier, App, setAttr, encodeUrl, addClass, getRouteNameFromLink, setCSS, setProperty, isInsecureContentRequest, toggleClass, isMobileApp, switchClass, DataSource, validateDataSourceCtx, removeNode, createElement, appendNode, transformFileURI } from '@wm/core';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, DISPLAY_TYPE, StylableComponent, styler, provideAsWidgetRef, TrustAsPipe, ImagePipe, WmComponentsModule } from '@wm/components/base';
import { CommonModule } from '@angular/common';

const registerProps = () => {
    register('wm-anchor', new Map([
        ['animation', PROP_STRING],
        ['badgevalue', PROP_STRING],
        ['caption', Object.assign({ value: 'Link' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['encodeurl', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['hyperlink', Object.assign({ value: '' }, PROP_STRING)],
        ['iconclass', PROP_STRING],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconposition', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['name', PROP_STRING],
        ['shortcutkey', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['target', Object.assign({ value: '_self' }, PROP_STRING)]
    ]));
};

var AnchorComponent_1;
const DEFAULT_CLS = 'app-anchor';
const WIDGET_CONFIG = {
    widgetType: 'wm-anchor',
    hostClass: DEFAULT_CLS,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
const regex = /Actions.goToPage_(\w+)\.invoke\(\)/g;
const disableContextMenu = ($event) => {
    $event.preventDefault();
};
let AnchorComponent = AnchorComponent_1 = class AnchorComponent extends StylableComponent {
    constructor(inj, app) {
        super(inj, WIDGET_CONFIG);
        this.app = app;
        this._eventNotifier = new EventNotifier();
        styler(this.nativeElement, this);
    }
    processEventAttr(eventName, expr, meta) {
        super.processEventAttr(eventName, expr, meta);
        if (!this.hasNavigationToCurrentPageExpr) {
            const app = this.inj.get(App);
            const fns = expr.split(';').map(Function.prototype.call, String.prototype.trim);
            if (fns.some(fn => {
                regex.lastIndex = 0;
                const matches = regex.exec(fn);
                this.hasGoToPageExpr = matches && (matches.length > 0);
                return this.hasGoToPageExpr && matches[1] === app.activePageName;
            })) {
                this.hasNavigationToCurrentPageExpr = true;
            }
        }
    }
    onActive(callback) {
        this._eventNotifier.subscribe('on-active', callback);
    }
    handleEvent(node, eventName, eventCallback, locals, meta) {
        super.handleEvent(node, eventName, e => {
            if (this.hasGoToPageExpr && locals.$event) {
                locals.$event.preventDefault();
            }
            eventCallback();
        }, locals, meta);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'hyperlink') {
            if (!nv) {
                setAttr(this.nativeElement, 'href', 'javascript:void(0)');
                this.nativeElement.addEventListener('contextmenu', disableContextMenu);
                return;
            }
            if (this.encodeurl) {
                nv = encodeUrl(nv);
            }
            // if hyperlink starts with 'www.' append '//' in the beginning
            if (nv.startsWith('www.')) {
                nv = `//${nv}`;
            }
            setAttr(this.nativeElement, 'href', nv);
            this.nativeElement.removeEventListener('contextmenu', disableContextMenu);
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    init() {
        if (this.hasNavigationToCurrentPageExpr) {
            addClass(this.nativeElement, 'active');
        }
        if (this.hyperlink && getRouteNameFromLink(this.hyperlink) === `/${this.app.activePageName}`
            || this.hasNavigationToCurrentPageExpr) {
            this._eventNotifier.notify("on-active", {});
        }
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        this.init();
    }
    ngOnAttach() {
        this.init();
        super.ngOnAttach();
    }
    ngOnDestroy() {
        this._eventNotifier.destroy();
        super.ngOnDestroy();
    }
};
AnchorComponent.initializeProps = registerProps();
AnchorComponent.ctorParameters = () => [
    { type: Injector },
    { type: App }
];
__decorate([
    HostBinding('target'),
    __metadata("design:type", String)
], AnchorComponent.prototype, "target", void 0);
__decorate([
    HostBinding('attr.accesskey'),
    __metadata("design:type", String)
], AnchorComponent.prototype, "shortcutkey", void 0);
__decorate([
    HostBinding('attr.icon-position'),
    __metadata("design:type", String)
], AnchorComponent.prototype, "iconposition", void 0);
AnchorComponent = AnchorComponent_1 = __decorate([
    Component({
        selector: 'a[wmAnchor]',
        template: "<img data-identifier=\"img\" alt=\"Image\" class=\"anchor-image-icon\" [src]=\"iconurl | image\" *ngIf=\"iconurl\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin}\"/>\n<i class=\"app-icon {{iconclass}}\" aria-hidden=\"true\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin}\" *ngIf=\"iconclass\"></i>\n<span class=\"sr-only\" *ngIf=\"iconclass\">{{caption | trustAs:'html'}} {{appLocale.LABEL_ICON}}</span>\n<span class=\"anchor-caption\" [innerHTML]=\"caption | trustAs:'html'\"></span>\n<ng-content select=\".caret\"></ng-content>\n<span *ngIf=\"badgevalue\" class=\"badge pull-right\" [textContent]=\"badgevalue\"></span>",
        providers: [
            provideAsWidgetRef(AnchorComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector,
        App])
], AnchorComponent);

const registerProps$1 = () => {
    register('wm-audio', new Map([
        ['audiopreload', Object.assign({ value: 'none' }, PROP_STRING)],
        ['audiosupportmessage', Object.assign({ value: 'Your browser does not support the audio tag.' }, PROP_STRING)],
        ['autoplay', PROP_BOOLEAN],
        ['class', PROP_STRING],
        ['controls', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['loop', PROP_BOOLEAN],
        ['mp3format', PROP_STRING],
        ['muted', PROP_BOOLEAN],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)]
    ]));
};

var AudioComponent_1;
const DEFAULT_CLS$1 = 'app-audio';
const WIDGET_CONFIG$1 = {
    widgetType: 'wm-audio',
    hostClass: DEFAULT_CLS$1,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
let AudioComponent = AudioComponent_1 = class AudioComponent extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$1);
        styler(this.nativeElement, this);
    }
};
AudioComponent.initializeProps = registerProps$1();
AudioComponent.ctorParameters = () => [
    { type: Injector }
];
AudioComponent = AudioComponent_1 = __decorate([
    Component({
        selector: '[wmAudio]',
        template: "<audio [src]=\"mp3format | trustAs: 'resource'\" [muted]=\"muted\" [autoplay]=\"autoplay\" [controls]=\"controls\" [loop]=\"loop\" [preload]=\"audiopreload\" role=\"application\" aria-describedby=\"recorded audio\">\n    <source type=\"audio/mp3\" [src]=\"mp3format | trustAs: 'resource'\">\n    {{audiosupportmessage}}\n</audio>\n",
        providers: [
            provideAsWidgetRef(AudioComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], AudioComponent);

const registerProps$2 = () => {
    register('wm-html', new Map([
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['hint', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var HtmlDirective_1;
const DEFAULT_CLS$2 = 'app-html-container';
const WIDGET_CONFIG$2 = {
    widgetType: 'wm-html',
    hostClass: DEFAULT_CLS$2
};
let HtmlDirective = HtmlDirective_1 = class HtmlDirective extends StylableComponent {
    constructor(inj, height, boundContent, trustAsPipe) {
        super(inj, WIDGET_CONFIG$2);
        this.boundContent = boundContent;
        this.trustAsPipe = trustAsPipe;
        // if the height is provided set the overflow to auto
        if (height) {
            setCSS(this.nativeElement, 'overflow', 'auto');
        }
        styler(this.nativeElement, this);
    }
    ngOnInit() {
        super.ngOnInit();
        if (this.boundContent) {
            this.nativeElement.innerHTML = '';
        }
        if (!this.content && this.nativeElement.innerHTML) {
            this.content = this.nativeElement.innerHTML;
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'content') {
            setProperty(this.nativeElement, 'innerHTML', this.trustAsPipe.transform(nv, SecurityContext.HTML));
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
};
HtmlDirective.initializeProps = registerProps$2();
HtmlDirective.ctorParameters = () => [
    { type: Injector },
    { type: String, decorators: [{ type: Attribute, args: ['height',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['content.bind',] }] },
    { type: TrustAsPipe }
];
HtmlDirective = HtmlDirective_1 = __decorate([
    Directive({
        selector: '[wmHtml]',
        providers: [
            provideAsWidgetRef(HtmlDirective_1)
        ]
    }),
    __param(1, Attribute('height')),
    __param(2, Attribute('content.bind')),
    __metadata("design:paramtypes", [Injector, String, String, TrustAsPipe])
], HtmlDirective);

const registerProps$3 = () => {
    register('wm-icon', new Map([
        ['animation', PROP_STRING],
        ['caption', PROP_STRING],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['hint', PROP_STRING],
        ['iconclass', Object.assign({ value: 'wi wi-star-border' }, PROP_STRING)],
        ['iconposition', Object.assign({ value: 'left' }, PROP_STRING)],
        ['iconsize', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var IconComponent_1;
const DEFAULT_CLS$3 = 'app-icon-wrapper';
const WIDGET_CONFIG$3 = {
    widgetType: 'wm-icon',
    hostClass: DEFAULT_CLS$3
};
let IconComponent = IconComponent_1 = class IconComponent extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$3);
        styler(this.nativeElement, this);
    }
};
IconComponent.initializeProps = registerProps$3();
IconComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    HostBinding('attr.icon-position'),
    __metadata("design:type", String)
], IconComponent.prototype, "iconposition", void 0);
__decorate([
    HostBinding('style.fontSize'),
    __metadata("design:type", String)
], IconComponent.prototype, "iconsize", void 0);
IconComponent = IconComponent_1 = __decorate([
    Component({
        selector: '[wmIcon]',
        template: "<i class=\"app-icon {{iconclass}}\"></i>\n<span class=\"sr-only\" *ngIf=\"iconclass\">{{caption}} {{appLocale.LABEL_ICON}}</span>\n<label class=\"app-label\" [textContent]=\"caption\" *ngIf=\"caption\"></label>",
        providers: [
            provideAsWidgetRef(IconComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], IconComponent);

const registerProps$4 = () => {
    register('wm-iframe', new Map([
        ['class', PROP_STRING],
        ['encodeurl', PROP_BOOLEAN],
        ['height', Object.assign({ value: '150px' }, PROP_STRING)],
        ['iframesrc', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['width', Object.assign({ value: '300px' }, PROP_STRING)]
    ]));
};

var IframeComponent_1;
const DEFAULT_CLS$4 = 'embed-responsive app-iframe';
const WIDGET_CONFIG$4 = {
    widgetType: 'wm-iframe',
    hostClass: DEFAULT_CLS$4
};
let IframeComponent = IframeComponent_1 = class IframeComponent extends StylableComponent {
    constructor(inj, trustAsPipe) {
        super(inj, WIDGET_CONFIG$4);
        this.trustAsPipe = trustAsPipe;
        /**
         * this property member is set to true when the content request url doesn't match windows protocol
         */
        this.showContentLoadError = false;
        styler(this.nativeElement, this);
    }
    computeIframeSrc() {
        this.showContentLoadError = false;
        this._iframesrc = undefined;
        if (this.iframesrc) {
            let url = this.iframesrc;
            if (this.encodeurl) {
                url = encodeUrl(this.iframesrc);
            }
            const trustedUrl = this.trustAsPipe.transform(url, SecurityContext.RESOURCE_URL);
            if (isInsecureContentRequest(url)) {
                this.showContentLoadError = true;
                this.errorMsg = `${this.appLocale.MESSAGE_ERROR_CONTENT_DISPLAY} ${this.iframesrc}`;
                this.hintMsg = this.errorMsg;
            }
            this._iframesrc = trustedUrl;
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'iframesrc' || key === 'encodeurl') {
            this.computeIframeSrc();
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
};
IframeComponent.initializeProps = registerProps$4();
IframeComponent.ctorParameters = () => [
    { type: Injector },
    { type: TrustAsPipe }
];
IframeComponent = IframeComponent_1 = __decorate([
    Component({
        selector: '[wmIframe]',
        template: "<iframe title=\"\" class=\"embed-responsive-item iframe-content\" role=\"document\"\n        scrolling=\"auto\"\n        marginheight=\"0\"\n        marginwidth=\"0\"\n        frameborder=\"0\"\n        [name]=\"name\"\n        [attr.src]=\"_iframesrc\"\n        seamless=\"seamless\">\n</iframe>\n<div class=\"wm-content-info readonly-wrapper\" *ngIf=\"showContentLoadError\">\n    <p class=\"wm-message\" [title]=\"hintMsg\" [textContent]=\"errMsg\"></p>\n</div>",
        providers: [
            provideAsWidgetRef(IframeComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, TrustAsPipe])
], IframeComponent);

const registerProps$5 = () => {
    register('wm-label', new Map([
        ['animation', PROP_STRING],
        ['caption', Object.assign({ value: 'Label' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['hint', PROP_STRING],
        ['name', PROP_STRING],
        ['required', PROP_BOOLEAN],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var LabelDirective_1;
const DEFAULT_CLS$5 = 'app-label';
const WIDGET_CONFIG$5 = {
    widgetType: 'wm-label',
    hostClass: DEFAULT_CLS$5,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
let LabelDirective = LabelDirective_1 = class LabelDirective extends StylableComponent {
    constructor(inj, trustAsPipe) {
        super(inj, WIDGET_CONFIG$5);
        this.trustAsPipe = trustAsPipe;
        styler(this.nativeElement, this);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'caption') {
            if (_.isObject(nv)) {
                setProperty(this.nativeElement, 'textContent', JSON.stringify(nv));
            }
            else {
                setProperty(this.nativeElement, 'innerHTML', this.trustAsPipe.transform(nv, SecurityContext.HTML));
            }
        }
        else if (key === 'required') {
            toggleClass(this.nativeElement, 'required', nv);
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
};
LabelDirective.initializeProps = registerProps$5();
LabelDirective.ctorParameters = () => [
    { type: Injector },
    { type: TrustAsPipe }
];
LabelDirective = LabelDirective_1 = __decorate([
    Directive({
        selector: '[wmLabel]',
        providers: [
            provideAsWidgetRef(LabelDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, TrustAsPipe])
], LabelDirective);

const registerProps$6 = () => {
    const props = new Map([
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['encodeurl', PROP_STRING],
        ['hint', PROP_STRING],
        ['name', PROP_STRING],
        ['pictureaspect', Object.assign({ value: 'None' }, PROP_STRING)],
        ['pictureplaceholder', PROP_STRING],
        ['picturesource', PROP_STRING],
        ['shape', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)]
    ]);
    if (isMobileApp()) {
        props.set('offline', Object.assign({ value: true }, PROP_BOOLEAN));
    }
    register('wm-picture', props);
};

var PictureDirective_1;
const DEFAULT_CLS$6 = 'app-picture';
const WIDGET_CONFIG$6 = {
    widgetType: 'wm-picture',
    hostClass: DEFAULT_CLS$6,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
let PictureDirective = PictureDirective_1 = class PictureDirective extends StylableComponent {
    constructor(inj, imagePipe) {
        super(inj, WIDGET_CONFIG$6);
        this.imagePipe = imagePipe;
        styler(this.nativeElement, this);
    }
    setImgSource() {
        this.imgSource = this.imagePipe.transform(this.picturesource, this.encodeurl, this.pictureplaceholder);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'pictureaspect') {
            let width = '';
            let height = '';
            switch (nv) {
                case 'None':
                    width = this.width;
                    height = this.height;
                    break;
                case 'H':
                    width = '100%';
                    break;
                case 'V':
                    height = '100%';
                    break;
                case 'Both':
                    width = '100%';
                    height = '100%';
                    break;
            }
            setCSS(this.nativeElement, 'width', width, true);
            setCSS(this.nativeElement, 'height', height, true);
        }
        else if (key === 'encodeurl' || key === 'pictureplaceholder') {
            this.setImgSource();
        }
        else if (key === 'shape') {
            switchClass(this.nativeElement, `img-${nv}`, `img-${ov}`);
        }
        else if (key === 'hint') {
            setAttr(this.nativeElement, 'alt', nv);
        }
        super.onPropertyChange(key, nv, ov);
    }
    onStyleChange(key, nv, ov) {
        if (key === 'picturesource') {
            this.setImgSource();
        }
        else {
            super.onStyleChange(key, nv, ov);
        }
    }
    ngOnInit() {
        this.setImgSource();
        super.ngOnInit();
    }
};
PictureDirective.initializeProps = registerProps$6();
PictureDirective.ctorParameters = () => [
    { type: Injector },
    { type: ImagePipe }
];
__decorate([
    HostBinding('src'),
    __metadata("design:type", String)
], PictureDirective.prototype, "imgSource", void 0);
PictureDirective = PictureDirective_1 = __decorate([
    Directive({
        selector: 'img[wmPicture]',
        providers: [
            provideAsWidgetRef(PictureDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, ImagePipe])
], PictureDirective);

const registerProps$7 = () => {
    register('wm-spinner', new Map([
        ['animation', { value: 'spin', PROP_STRING }],
        ['caption', Object.assign({ value: 'Loading...' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['iconclass', Object.assign({ value: 'fa fa-circle-o-notch fa-spin' }, PROP_STRING)],
        ['iconsize', PROP_STRING],
        ['image', PROP_STRING],
        ['imageheight', PROP_STRING],
        ['imagewidth', Object.assign({ value: '20px' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['type', Object.assign({ value: 'icon' }, PROP_STRING)],
        ['servicevariabletotrack', PROP_STRING]
    ]));
};

var SpinnerComponent_1;
const DEFAULT_CLS$7 = 'app-spinner';
const WIDGET_CONFIG$7 = { widgetType: 'wm-spinner', hostClass: DEFAULT_CLS$7 };
let SpinnerComponent = SpinnerComponent_1 = class SpinnerComponent extends StylableComponent {
    constructor(inj, imagePipe) {
        super(inj, WIDGET_CONFIG$7);
        this.imagePipe = imagePipe;
        this.iconclass = '';
        this.animation = '';
        this.showCaption = true;
        styler(this.nativeElement, this);
    }
    get spinnerMessages() {
        return this._spinnerMessages;
    }
    set spinnerMessages(newVal) {
        this.showCaption = _.isEmpty(newVal);
        this._spinnerMessages = newVal;
    }
    listenOnDataSource() {
        const variables = _.split(this.servicevariabletotrack, ',');
        this.getAppInstance().subscribe('toggle-variable-state', data => {
            const name = data.variable.execute(DataSource.Operation.GET_NAME);
            if (_.includes(variables, name) && validateDataSourceCtx(data.variable, this.getViewParent())) {
                this.widget.show = data.active;
            }
        });
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'image') {
            this.picture = this.imagePipe.transform(nv);
        }
        else if (key === 'animation') {
            if (nv === 'spin') {
                this.animation = 'fa-spin';
            }
            else {
                this.animation = nv || '';
            }
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngOnInit() {
        super.ngOnInit();
        // if variables are to be listened to, hide the widget and set the listener
        if (this.servicevariabletotrack) {
            this.widget.show = false;
            this.listenOnDataSource();
        }
    }
};
SpinnerComponent.initializeProps = registerProps$7();
SpinnerComponent.ctorParameters = () => [
    { type: Injector },
    { type: ImagePipe }
];
SpinnerComponent = SpinnerComponent_1 = __decorate([
    Component({
        selector: '[wmSpinner]',
        template: "<div class=\"spinner-message\" aria-label=\"loading gif\">\n    <span class=\"spinner-image animated infinite\" [ngClass]=\"animation\" aria-hidden=\"true\"\n        [ngStyle]=\"{width: imagewidth, height: imageheight, backgroundImage: 'url(' + picture + ')', backgroundSize: imagewidth}\"\n        *ngIf=\"type === 'image'\"></span>\n    <i class=\"spinner-image animated infinite\" [ngClass]=\"[iconclass, animation]\" [style.fonSize]=\"iconsize\" *ngIf=\"type === 'icon'\"></i>\n    <span class=\"spinner-text\" [innerHTML]=\"caption | trustAs: 'html'\" *ngIf=\"showCaption\"></span>\n    <div class=\"spinner-messages\" *ngIf=\"!showCaption && spinnerMessages\">\n        <p *ngFor=\"let value of spinnerMessages\" [textContent]=\"value\"></p>\n    </div>\n</div>",
        providers: [
            provideAsWidgetRef(SpinnerComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, ImagePipe])
], SpinnerComponent);

const registerProps$8 = () => {
    register('wm-video', new Map([
        ['autoplay', PROP_BOOLEAN],
        ['class', PROP_STRING],
        ['controls', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['loop', PROP_BOOLEAN],
        ['mp4format', PROP_STRING],
        ['muted', PROP_BOOLEAN],
        ['name', PROP_STRING],
        ['oggformat', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['subtitlelang', Object.assign({ value: 'en' }, PROP_STRING)],
        ['subtitlesource', PROP_STRING],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['videoposter', Object.assign({ value: 'resources/images/imagelists/default-image.png' }, PROP_STRING)],
        ['videopreload', Object.assign({ value: 'none' }, PROP_STRING)],
        ['videosupportmessage', Object.assign({ value: 'Your browser does not support the video tag.' }, PROP_STRING)],
        ['webmformat', PROP_STRING]
    ]));
};

var VideoComponent_1;
const DEFAULT_CLS$8 = 'app-video';
const WIDGET_CONFIG$8 = {
    widgetType: 'wm-video',
    hostClass: DEFAULT_CLS$8,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
let VideoComponent = VideoComponent_1 = class VideoComponent extends StylableComponent {
    constructor(inj, trustAsPipe) {
        super(inj, WIDGET_CONFIG$8);
        this.trustAsPipe = trustAsPipe;
        /**
         * subtitle language property eg: en
         */
        this.subtitlelang = 'en';
        styler(this.nativeElement, this);
    }
    // DO NOT use ngIf binding for the track. As of v6.0.Beta7 there is an error creating a void track node
    onPropertyChange(key, nv, ov) {
        if (key === 'subtitlesource') {
            let track = this.nativeElement.querySelector('track');
            if (track) {
                removeNode(track, true);
            }
            track = createElement('track', {
                kind: 'subtitles',
                label: this.subtitlelang,
                srclang: this.subtitlelang,
                src: this.trustAsPipe.transform(nv, SecurityContext.RESOURCE_URL),
                default: ''
            }, true);
            appendNode(track, this.nativeElement.querySelector('video'));
        }
        else if (key === 'mp4format') {
            this.mp4format = transformFileURI(nv);
        }
        else if (key === 'webmformat') {
            this.webmformat = transformFileURI(nv);
        }
        else if (key === 'oggformat') {
            this.oggformat = transformFileURI(nv);
        }
        super.onPropertyChange(key, nv, ov);
    }
};
VideoComponent.initializeProps = registerProps$8();
VideoComponent.ctorParameters = () => [
    { type: Injector },
    { type: TrustAsPipe }
];
VideoComponent = VideoComponent_1 = __decorate([
    Component({
        selector: '[wmVideo]',
        template: "<video [preload]=\"videopreload\" role=\"application\"\n       [src]=\"mp4format | trustAs: 'resource'\"\n       [muted]=\"muted\"\n       [poster]=\"videoposter | image\"\n       [controls]=\"controls\"\n       [loop]=\"loop\"\n       [autoplay]=\"autoplay\">\n    <source type=\"video/mp4\" [src]=\"mp4format | trustAs: 'resource'\" *ngIf=\"mp4format\">\n    <source type=\"video/webm\" [src]=\"webmformat | trustAs: 'resource'\" *ngIf=\"webmformat\">\n    <source type=\"video/ogg\" [src]=\"oggformat | trustAs: 'resource'\" *ngIf=\"oggformat\">\n    {{videosupportmessage}}\n</video>",
        providers: [
            provideAsWidgetRef(VideoComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, TrustAsPipe])
], VideoComponent);

const components = [
    AnchorComponent,
    AudioComponent,
    HtmlDirective,
    IconComponent,
    IframeComponent,
    LabelDirective,
    PictureDirective,
    SpinnerComponent,
    VideoComponent
];
let BasicModule = class BasicModule {
};
BasicModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], BasicModule);

/**
 * Generated bundle index. Do not edit.
 */

export { AnchorComponent, BasicModule, disableContextMenu, registerProps as ɵa, AudioComponent as ɵb, registerProps$1 as ɵc, HtmlDirective as ɵd, registerProps$2 as ɵe, IconComponent as ɵf, registerProps$3 as ɵg, IframeComponent as ɵh, registerProps$4 as ɵi, LabelDirective as ɵj, registerProps$5 as ɵk, PictureDirective as ɵl, registerProps$6 as ɵm, SpinnerComponent as ɵn, registerProps$7 as ɵo, VideoComponent as ɵp, registerProps$8 as ɵq };
//# sourceMappingURL=index.js.map
