import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, HostBinding, Component, SecurityContext, Attribute, Directive, NgModule } from '@angular/core';
import { EventNotifier, App, setAttr, encodeUrl, addClass, getRouteNameFromLink, setCSS, setProperty, isInsecureContentRequest, toggleClass, isMobileApp, switchClass, DataSource, validateDataSourceCtx, removeNode, createElement, appendNode, transformFileURI } from '@wm/core';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, DISPLAY_TYPE, styler, provideAsWidgetRef, StylableComponent, TrustAsPipe, ImagePipe, WmComponentsModule } from '@wm/components/base';
import { CommonModule } from '@angular/common';

var registerProps = function () {
    register('wm-anchor', new Map([
        ['animation', PROP_STRING],
        ['badgevalue', PROP_STRING],
        ['caption', __assign({ value: 'Link' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['encodeurl', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['hyperlink', __assign({ value: '' }, PROP_STRING)],
        ['iconclass', PROP_STRING],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconposition', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['name', PROP_STRING],
        ['shortcutkey', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['target', __assign({ value: '_self' }, PROP_STRING)]
    ]));
};

var DEFAULT_CLS = 'app-anchor';
var WIDGET_CONFIG = {
    widgetType: 'wm-anchor',
    hostClass: DEFAULT_CLS,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
var regex = /Actions.goToPage_(\w+)\.invoke\(\)/g;
var disableContextMenu = function ($event) {
    $event.preventDefault();
};
var AnchorComponent = /** @class */ (function (_super) {
    __extends(AnchorComponent, _super);
    function AnchorComponent(inj, app) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.app = app;
        _this._eventNotifier = new EventNotifier();
        styler(_this.nativeElement, _this);
        return _this;
    }
    AnchorComponent_1 = AnchorComponent;
    AnchorComponent.prototype.processEventAttr = function (eventName, expr, meta) {
        var _this = this;
        _super.prototype.processEventAttr.call(this, eventName, expr, meta);
        if (!this.hasNavigationToCurrentPageExpr) {
            var app_1 = this.inj.get(App);
            var fns = expr.split(';').map(Function.prototype.call, String.prototype.trim);
            if (fns.some(function (fn) {
                regex.lastIndex = 0;
                var matches = regex.exec(fn);
                _this.hasGoToPageExpr = matches && (matches.length > 0);
                return _this.hasGoToPageExpr && matches[1] === app_1.activePageName;
            })) {
                this.hasNavigationToCurrentPageExpr = true;
            }
        }
    };
    AnchorComponent.prototype.onActive = function (callback) {
        this._eventNotifier.subscribe('on-active', callback);
    };
    AnchorComponent.prototype.handleEvent = function (node, eventName, eventCallback, locals, meta) {
        var _this = this;
        _super.prototype.handleEvent.call(this, node, eventName, function (e) {
            if (_this.hasGoToPageExpr && locals.$event) {
                locals.$event.preventDefault();
            }
            eventCallback();
        }, locals, meta);
    };
    AnchorComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'hyperlink') {
            if (!nv) {
                setAttr(this.nativeElement, 'href', 'javascript:void(0)');
                this.nativeElement.addEventListener('contextmenu', disableContextMenu);
                return;
            }
            if (this.encodeurl) {
                nv = encodeUrl(nv);
            }
            // if hyperlink starts with 'www.' append '//' in the beginning
            if (nv.startsWith('www.')) {
                nv = "//" + nv;
            }
            setAttr(this.nativeElement, 'href', nv);
            this.nativeElement.removeEventListener('contextmenu', disableContextMenu);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    AnchorComponent.prototype.init = function () {
        if (this.hasNavigationToCurrentPageExpr) {
            addClass(this.nativeElement, 'active');
        }
        if (this.hyperlink && getRouteNameFromLink(this.hyperlink) === "/" + this.app.activePageName
            || this.hasNavigationToCurrentPageExpr) {
            this._eventNotifier.notify("on-active", {});
        }
    };
    AnchorComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        this.init();
    };
    AnchorComponent.prototype.ngOnAttach = function () {
        this.init();
        _super.prototype.ngOnAttach.call(this);
    };
    AnchorComponent.prototype.ngOnDestroy = function () {
        this._eventNotifier.destroy();
        _super.prototype.ngOnDestroy.call(this);
    };
    var AnchorComponent_1;
    AnchorComponent.initializeProps = registerProps();
    AnchorComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: App }
    ]; };
    __decorate([
        HostBinding('target'),
        __metadata("design:type", String)
    ], AnchorComponent.prototype, "target", void 0);
    __decorate([
        HostBinding('attr.accesskey'),
        __metadata("design:type", String)
    ], AnchorComponent.prototype, "shortcutkey", void 0);
    __decorate([
        HostBinding('attr.icon-position'),
        __metadata("design:type", String)
    ], AnchorComponent.prototype, "iconposition", void 0);
    AnchorComponent = AnchorComponent_1 = __decorate([
        Component({
            selector: 'a[wmAnchor]',
            template: "<img data-identifier=\"img\" alt=\"Image\" class=\"anchor-image-icon\" [src]=\"iconurl | image\" *ngIf=\"iconurl\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin}\"/>\n<i class=\"app-icon {{iconclass}}\" aria-hidden=\"true\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin}\" *ngIf=\"iconclass\"></i>\n<span class=\"sr-only\" *ngIf=\"iconclass\">{{caption | trustAs:'html'}} {{appLocale.LABEL_ICON}}</span>\n<span class=\"anchor-caption\" [innerHTML]=\"caption | trustAs:'html'\"></span>\n<ng-content select=\".caret\"></ng-content>\n<span *ngIf=\"badgevalue\" class=\"badge pull-right\" [textContent]=\"badgevalue\"></span>",
            providers: [
                provideAsWidgetRef(AnchorComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector,
            App])
    ], AnchorComponent);
    return AnchorComponent;
}(StylableComponent));

var registerProps$1 = function () {
    register('wm-audio', new Map([
        ['audiopreload', __assign({ value: 'none' }, PROP_STRING)],
        ['audiosupportmessage', __assign({ value: 'Your browser does not support the audio tag.' }, PROP_STRING)],
        ['autoplay', PROP_BOOLEAN],
        ['class', PROP_STRING],
        ['controls', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['loop', PROP_BOOLEAN],
        ['mp3format', PROP_STRING],
        ['muted', PROP_BOOLEAN],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
    ]));
};

var DEFAULT_CLS$1 = 'app-audio';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-audio',
    hostClass: DEFAULT_CLS$1,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
var AudioComponent = /** @class */ (function (_super) {
    __extends(AudioComponent, _super);
    function AudioComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        styler(_this.nativeElement, _this);
        return _this;
    }
    AudioComponent_1 = AudioComponent;
    var AudioComponent_1;
    AudioComponent.initializeProps = registerProps$1();
    AudioComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    AudioComponent = AudioComponent_1 = __decorate([
        Component({
            selector: '[wmAudio]',
            template: "<audio [src]=\"mp3format | trustAs: 'resource'\" [muted]=\"muted\" [autoplay]=\"autoplay\" [controls]=\"controls\" [loop]=\"loop\" [preload]=\"audiopreload\" role=\"application\" aria-describedby=\"recorded audio\">\n    <source type=\"audio/mp3\" [src]=\"mp3format | trustAs: 'resource'\">\n    {{audiosupportmessage}}\n</audio>\n",
            providers: [
                provideAsWidgetRef(AudioComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], AudioComponent);
    return AudioComponent;
}(StylableComponent));

var registerProps$2 = function () {
    register('wm-html', new Map([
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['hint', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$2 = 'app-html-container';
var WIDGET_CONFIG$2 = {
    widgetType: 'wm-html',
    hostClass: DEFAULT_CLS$2
};
var HtmlDirective = /** @class */ (function (_super) {
    __extends(HtmlDirective, _super);
    function HtmlDirective(inj, height, boundContent, trustAsPipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$2) || this;
        _this.boundContent = boundContent;
        _this.trustAsPipe = trustAsPipe;
        // if the height is provided set the overflow to auto
        if (height) {
            setCSS(_this.nativeElement, 'overflow', 'auto');
        }
        styler(_this.nativeElement, _this);
        return _this;
    }
    HtmlDirective_1 = HtmlDirective;
    HtmlDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        if (this.boundContent) {
            this.nativeElement.innerHTML = '';
        }
        if (!this.content && this.nativeElement.innerHTML) {
            this.content = this.nativeElement.innerHTML;
        }
    };
    HtmlDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'content') {
            setProperty(this.nativeElement, 'innerHTML', this.trustAsPipe.transform(nv, SecurityContext.HTML));
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var HtmlDirective_1;
    HtmlDirective.initializeProps = registerProps$2();
    HtmlDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: String, decorators: [{ type: Attribute, args: ['height',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['content.bind',] }] },
        { type: TrustAsPipe }
    ]; };
    HtmlDirective = HtmlDirective_1 = __decorate([
        Directive({
            selector: '[wmHtml]',
            providers: [
                provideAsWidgetRef(HtmlDirective_1)
            ]
        }),
        __param(1, Attribute('height')),
        __param(2, Attribute('content.bind')),
        __metadata("design:paramtypes", [Injector, String, String, TrustAsPipe])
    ], HtmlDirective);
    return HtmlDirective;
}(StylableComponent));

var registerProps$3 = function () {
    register('wm-icon', new Map([
        ['animation', PROP_STRING],
        ['caption', PROP_STRING],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['hint', PROP_STRING],
        ['iconclass', __assign({ value: 'wi wi-star-border' }, PROP_STRING)],
        ['iconposition', __assign({ value: 'left' }, PROP_STRING)],
        ['iconsize', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$3 = 'app-icon-wrapper';
var WIDGET_CONFIG$3 = {
    widgetType: 'wm-icon',
    hostClass: DEFAULT_CLS$3
};
var IconComponent = /** @class */ (function (_super) {
    __extends(IconComponent, _super);
    function IconComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$3) || this;
        styler(_this.nativeElement, _this);
        return _this;
    }
    IconComponent_1 = IconComponent;
    var IconComponent_1;
    IconComponent.initializeProps = registerProps$3();
    IconComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        HostBinding('attr.icon-position'),
        __metadata("design:type", String)
    ], IconComponent.prototype, "iconposition", void 0);
    __decorate([
        HostBinding('style.fontSize'),
        __metadata("design:type", String)
    ], IconComponent.prototype, "iconsize", void 0);
    IconComponent = IconComponent_1 = __decorate([
        Component({
            selector: '[wmIcon]',
            template: "<i class=\"app-icon {{iconclass}}\"></i>\n<span class=\"sr-only\" *ngIf=\"iconclass\">{{caption}} {{appLocale.LABEL_ICON}}</span>\n<label class=\"app-label\" [textContent]=\"caption\" *ngIf=\"caption\"></label>",
            providers: [
                provideAsWidgetRef(IconComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], IconComponent);
    return IconComponent;
}(StylableComponent));

var registerProps$4 = function () {
    register('wm-iframe', new Map([
        ['class', PROP_STRING],
        ['encodeurl', PROP_BOOLEAN],
        ['height', __assign({ value: '150px' }, PROP_STRING)],
        ['iframesrc', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['width', __assign({ value: '300px' }, PROP_STRING)]
    ]));
};

var DEFAULT_CLS$4 = 'embed-responsive app-iframe';
var WIDGET_CONFIG$4 = {
    widgetType: 'wm-iframe',
    hostClass: DEFAULT_CLS$4
};
var IframeComponent = /** @class */ (function (_super) {
    __extends(IframeComponent, _super);
    function IframeComponent(inj, trustAsPipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$4) || this;
        _this.trustAsPipe = trustAsPipe;
        /**
         * this property member is set to true when the content request url doesn't match windows protocol
         */
        _this.showContentLoadError = false;
        styler(_this.nativeElement, _this);
        return _this;
    }
    IframeComponent_1 = IframeComponent;
    IframeComponent.prototype.computeIframeSrc = function () {
        this.showContentLoadError = false;
        this._iframesrc = undefined;
        if (this.iframesrc) {
            var url = this.iframesrc;
            if (this.encodeurl) {
                url = encodeUrl(this.iframesrc);
            }
            var trustedUrl = this.trustAsPipe.transform(url, SecurityContext.RESOURCE_URL);
            if (isInsecureContentRequest(url)) {
                this.showContentLoadError = true;
                this.errorMsg = this.appLocale.MESSAGE_ERROR_CONTENT_DISPLAY + " " + this.iframesrc;
                this.hintMsg = this.errorMsg;
            }
            this._iframesrc = trustedUrl;
        }
    };
    IframeComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'iframesrc' || key === 'encodeurl') {
            this.computeIframeSrc();
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var IframeComponent_1;
    IframeComponent.initializeProps = registerProps$4();
    IframeComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: TrustAsPipe }
    ]; };
    IframeComponent = IframeComponent_1 = __decorate([
        Component({
            selector: '[wmIframe]',
            template: "<iframe title=\"\" class=\"embed-responsive-item iframe-content\" role=\"document\"\n        scrolling=\"auto\"\n        marginheight=\"0\"\n        marginwidth=\"0\"\n        frameborder=\"0\"\n        [name]=\"name\"\n        [attr.src]=\"_iframesrc\"\n        seamless=\"seamless\">\n</iframe>\n<div class=\"wm-content-info readonly-wrapper\" *ngIf=\"showContentLoadError\">\n    <p class=\"wm-message\" [title]=\"hintMsg\" [textContent]=\"errMsg\"></p>\n</div>",
            providers: [
                provideAsWidgetRef(IframeComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, TrustAsPipe])
    ], IframeComponent);
    return IframeComponent;
}(StylableComponent));

var registerProps$5 = function () {
    register('wm-label', new Map([
        ['animation', PROP_STRING],
        ['caption', __assign({ value: 'Label' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['hint', PROP_STRING],
        ['name', PROP_STRING],
        ['required', PROP_BOOLEAN],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$5 = 'app-label';
var WIDGET_CONFIG$5 = {
    widgetType: 'wm-label',
    hostClass: DEFAULT_CLS$5,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
var LabelDirective = /** @class */ (function (_super) {
    __extends(LabelDirective, _super);
    function LabelDirective(inj, trustAsPipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$5) || this;
        _this.trustAsPipe = trustAsPipe;
        styler(_this.nativeElement, _this);
        return _this;
    }
    LabelDirective_1 = LabelDirective;
    LabelDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'caption') {
            if (_.isObject(nv)) {
                setProperty(this.nativeElement, 'textContent', JSON.stringify(nv));
            }
            else {
                setProperty(this.nativeElement, 'innerHTML', this.trustAsPipe.transform(nv, SecurityContext.HTML));
            }
        }
        else if (key === 'required') {
            toggleClass(this.nativeElement, 'required', nv);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var LabelDirective_1;
    LabelDirective.initializeProps = registerProps$5();
    LabelDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: TrustAsPipe }
    ]; };
    LabelDirective = LabelDirective_1 = __decorate([
        Directive({
            selector: '[wmLabel]',
            providers: [
                provideAsWidgetRef(LabelDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, TrustAsPipe])
    ], LabelDirective);
    return LabelDirective;
}(StylableComponent));

var registerProps$6 = function () {
    var props = new Map([
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['encodeurl', PROP_STRING],
        ['hint', PROP_STRING],
        ['name', PROP_STRING],
        ['pictureaspect', __assign({ value: 'None' }, PROP_STRING)],
        ['pictureplaceholder', PROP_STRING],
        ['picturesource', PROP_STRING],
        ['shape', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
    ]);
    if (isMobileApp()) {
        props.set('offline', __assign({ value: true }, PROP_BOOLEAN));
    }
    register('wm-picture', props);
};

var DEFAULT_CLS$6 = 'app-picture';
var WIDGET_CONFIG$6 = {
    widgetType: 'wm-picture',
    hostClass: DEFAULT_CLS$6,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
var PictureDirective = /** @class */ (function (_super) {
    __extends(PictureDirective, _super);
    function PictureDirective(inj, imagePipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$6) || this;
        _this.imagePipe = imagePipe;
        styler(_this.nativeElement, _this);
        return _this;
    }
    PictureDirective_1 = PictureDirective;
    PictureDirective.prototype.setImgSource = function () {
        this.imgSource = this.imagePipe.transform(this.picturesource, this.encodeurl, this.pictureplaceholder);
    };
    PictureDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'pictureaspect') {
            var width = '';
            var height = '';
            switch (nv) {
                case 'None':
                    width = this.width;
                    height = this.height;
                    break;
                case 'H':
                    width = '100%';
                    break;
                case 'V':
                    height = '100%';
                    break;
                case 'Both':
                    width = '100%';
                    height = '100%';
                    break;
            }
            setCSS(this.nativeElement, 'width', width, true);
            setCSS(this.nativeElement, 'height', height, true);
        }
        else if (key === 'encodeurl' || key === 'pictureplaceholder') {
            this.setImgSource();
        }
        else if (key === 'shape') {
            switchClass(this.nativeElement, "img-" + nv, "img-" + ov);
        }
        else if (key === 'hint') {
            setAttr(this.nativeElement, 'alt', nv);
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    PictureDirective.prototype.onStyleChange = function (key, nv, ov) {
        if (key === 'picturesource') {
            this.setImgSource();
        }
        else {
            _super.prototype.onStyleChange.call(this, key, nv, ov);
        }
    };
    PictureDirective.prototype.ngOnInit = function () {
        this.setImgSource();
        _super.prototype.ngOnInit.call(this);
    };
    var PictureDirective_1;
    PictureDirective.initializeProps = registerProps$6();
    PictureDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: ImagePipe }
    ]; };
    __decorate([
        HostBinding('src'),
        __metadata("design:type", String)
    ], PictureDirective.prototype, "imgSource", void 0);
    PictureDirective = PictureDirective_1 = __decorate([
        Directive({
            selector: 'img[wmPicture]',
            providers: [
                provideAsWidgetRef(PictureDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, ImagePipe])
    ], PictureDirective);
    return PictureDirective;
}(StylableComponent));

var registerProps$7 = function () {
    register('wm-spinner', new Map([
        ['animation', { value: 'spin', PROP_STRING: PROP_STRING }],
        ['caption', __assign({ value: 'Loading...' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['iconclass', __assign({ value: 'fa fa-circle-o-notch fa-spin' }, PROP_STRING)],
        ['iconsize', PROP_STRING],
        ['image', PROP_STRING],
        ['imageheight', PROP_STRING],
        ['imagewidth', __assign({ value: '20px' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['type', __assign({ value: 'icon' }, PROP_STRING)],
        ['servicevariabletotrack', PROP_STRING]
    ]));
};

var DEFAULT_CLS$7 = 'app-spinner';
var WIDGET_CONFIG$7 = { widgetType: 'wm-spinner', hostClass: DEFAULT_CLS$7 };
var SpinnerComponent = /** @class */ (function (_super) {
    __extends(SpinnerComponent, _super);
    function SpinnerComponent(inj, imagePipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$7) || this;
        _this.imagePipe = imagePipe;
        _this.iconclass = '';
        _this.animation = '';
        _this.showCaption = true;
        styler(_this.nativeElement, _this);
        return _this;
    }
    SpinnerComponent_1 = SpinnerComponent;
    Object.defineProperty(SpinnerComponent.prototype, "spinnerMessages", {
        get: function () {
            return this._spinnerMessages;
        },
        set: function (newVal) {
            this.showCaption = _.isEmpty(newVal);
            this._spinnerMessages = newVal;
        },
        enumerable: true,
        configurable: true
    });
    SpinnerComponent.prototype.listenOnDataSource = function () {
        var _this = this;
        var variables = _.split(this.servicevariabletotrack, ',');
        this.getAppInstance().subscribe('toggle-variable-state', function (data) {
            var name = data.variable.execute(DataSource.Operation.GET_NAME);
            if (_.includes(variables, name) && validateDataSourceCtx(data.variable, _this.getViewParent())) {
                _this.widget.show = data.active;
            }
        });
    };
    SpinnerComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'image') {
            this.picture = this.imagePipe.transform(nv);
        }
        else if (key === 'animation') {
            if (nv === 'spin') {
                this.animation = 'fa-spin';
            }
            else {
                this.animation = nv || '';
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    SpinnerComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        // if variables are to be listened to, hide the widget and set the listener
        if (this.servicevariabletotrack) {
            this.widget.show = false;
            this.listenOnDataSource();
        }
    };
    var SpinnerComponent_1;
    SpinnerComponent.initializeProps = registerProps$7();
    SpinnerComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: ImagePipe }
    ]; };
    SpinnerComponent = SpinnerComponent_1 = __decorate([
        Component({
            selector: '[wmSpinner]',
            template: "<div class=\"spinner-message\" aria-label=\"loading gif\">\n    <span class=\"spinner-image animated infinite\" [ngClass]=\"animation\" aria-hidden=\"true\"\n        [ngStyle]=\"{width: imagewidth, height: imageheight, backgroundImage: 'url(' + picture + ')', backgroundSize: imagewidth}\"\n        *ngIf=\"type === 'image'\"></span>\n    <i class=\"spinner-image animated infinite\" [ngClass]=\"[iconclass, animation]\" [style.fonSize]=\"iconsize\" *ngIf=\"type === 'icon'\"></i>\n    <span class=\"spinner-text\" [innerHTML]=\"caption | trustAs: 'html'\" *ngIf=\"showCaption\"></span>\n    <div class=\"spinner-messages\" *ngIf=\"!showCaption && spinnerMessages\">\n        <p *ngFor=\"let value of spinnerMessages\" [textContent]=\"value\"></p>\n    </div>\n</div>",
            providers: [
                provideAsWidgetRef(SpinnerComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, ImagePipe])
    ], SpinnerComponent);
    return SpinnerComponent;
}(StylableComponent));

var registerProps$8 = function () {
    register('wm-video', new Map([
        ['autoplay', PROP_BOOLEAN],
        ['class', PROP_STRING],
        ['controls', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['loop', PROP_BOOLEAN],
        ['mp4format', PROP_STRING],
        ['muted', PROP_BOOLEAN],
        ['name', PROP_STRING],
        ['oggformat', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['subtitlelang', __assign({ value: 'en' }, PROP_STRING)],
        ['subtitlesource', PROP_STRING],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['videoposter', __assign({ value: 'resources/images/imagelists/default-image.png' }, PROP_STRING)],
        ['videopreload', __assign({ value: 'none' }, PROP_STRING)],
        ['videosupportmessage', __assign({ value: 'Your browser does not support the video tag.' }, PROP_STRING)],
        ['webmformat', PROP_STRING]
    ]));
};

var DEFAULT_CLS$8 = 'app-video';
var WIDGET_CONFIG$8 = {
    widgetType: 'wm-video',
    hostClass: DEFAULT_CLS$8,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
var VideoComponent = /** @class */ (function (_super) {
    __extends(VideoComponent, _super);
    function VideoComponent(inj, trustAsPipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$8) || this;
        _this.trustAsPipe = trustAsPipe;
        /**
         * subtitle language property eg: en
         */
        _this.subtitlelang = 'en';
        styler(_this.nativeElement, _this);
        return _this;
    }
    VideoComponent_1 = VideoComponent;
    // DO NOT use ngIf binding for the track. As of v6.0.Beta7 there is an error creating a void track node
    VideoComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'subtitlesource') {
            var track = this.nativeElement.querySelector('track');
            if (track) {
                removeNode(track, true);
            }
            track = createElement('track', {
                kind: 'subtitles',
                label: this.subtitlelang,
                srclang: this.subtitlelang,
                src: this.trustAsPipe.transform(nv, SecurityContext.RESOURCE_URL),
                default: ''
            }, true);
            appendNode(track, this.nativeElement.querySelector('video'));
        }
        else if (key === 'mp4format') {
            this.mp4format = transformFileURI(nv);
        }
        else if (key === 'webmformat') {
            this.webmformat = transformFileURI(nv);
        }
        else if (key === 'oggformat') {
            this.oggformat = transformFileURI(nv);
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    var VideoComponent_1;
    VideoComponent.initializeProps = registerProps$8();
    VideoComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: TrustAsPipe }
    ]; };
    VideoComponent = VideoComponent_1 = __decorate([
        Component({
            selector: '[wmVideo]',
            template: "<video [preload]=\"videopreload\" role=\"application\"\n       [src]=\"mp4format | trustAs: 'resource'\"\n       [muted]=\"muted\"\n       [poster]=\"videoposter | image\"\n       [controls]=\"controls\"\n       [loop]=\"loop\"\n       [autoplay]=\"autoplay\">\n    <source type=\"video/mp4\" [src]=\"mp4format | trustAs: 'resource'\" *ngIf=\"mp4format\">\n    <source type=\"video/webm\" [src]=\"webmformat | trustAs: 'resource'\" *ngIf=\"webmformat\">\n    <source type=\"video/ogg\" [src]=\"oggformat | trustAs: 'resource'\" *ngIf=\"oggformat\">\n    {{videosupportmessage}}\n</video>",
            providers: [
                provideAsWidgetRef(VideoComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, TrustAsPipe])
    ], VideoComponent);
    return VideoComponent;
}(StylableComponent));

var components = [
    AnchorComponent,
    AudioComponent,
    HtmlDirective,
    IconComponent,
    IframeComponent,
    LabelDirective,
    PictureDirective,
    SpinnerComponent,
    VideoComponent
];
var BasicModule = /** @class */ (function () {
    function BasicModule() {
    }
    BasicModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], BasicModule);
    return BasicModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { AnchorComponent, BasicModule, disableContextMenu, registerProps as ɵa, AudioComponent as ɵb, registerProps$1 as ɵc, HtmlDirective as ɵd, registerProps$2 as ɵe, IconComponent as ɵf, registerProps$3 as ɵg, IframeComponent as ɵh, registerProps$4 as ɵi, LabelDirective as ɵj, registerProps$5 as ɵk, PictureDirective as ɵl, registerProps$6 as ɵm, SpinnerComponent as ɵn, registerProps$7 as ɵo, VideoComponent as ɵp, registerProps$8 as ɵq };
//# sourceMappingURL=index.js.map
