import { __assign, __extends, __decorate, __param, __metadata, __spread } from 'tslib';
import { Injector, Attribute, Component, ViewChild, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { CircleProgressComponent, NgCircleProgressModule } from 'ng-circle-progress';
import { register, PROP_STRING, PROP_ANY, PROP_NUMBER, PROP_BOOLEAN, styler, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { isString, findValueOf, isDefined } from '@wm/core';

var registerProps = function () {
    register('wm-progress-bar', new Map([
        ['captionplacement', __assign({ value: 'hidden' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['datavalue', PROP_STRING],
        ['displayformat', PROP_STRING],
        ['hint', PROP_STRING],
        ['maxvalue', __assign({ value: 100 }, PROP_NUMBER)],
        ['minvalue', __assign({ value: 0 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['type', __assign({ value: 'default' }, PROP_STRING)]
    ]));
};

// This function returns the maximum number of decimal digits allowed.
var getDecimalCount = function (val) {
    val = val || '9';
    val = val.replace(/\%$/, '');
    var n = val.lastIndexOf('.');
    return (n === -1) ? 0 : (val.length - n - 1);
};
// returns true if the given value contains '%'
var isPercentageValue = function (val) {
    if (isString(val)) {
        val = val.trim();
        return val.charAt(val.length - 1) === '%';
    }
    return false;
};
var calculatePercent = function (value, min, max) {
    if (min === void 0) { min = 0; }
    if (max === void 0) { max = 0; }
    var percent = ((value - min) / (max - min)) * 100;
    if (_.isNaN(percent)) {
        console.warn('Circle Progress Bar: One of the properties Min, Max or datavalue is not a valid number');
        return 0;
    }
    return Math.abs(percent);
};

var DEFAULT_CLS = 'progress app-progress';
var WIDGET_CONFIG = { widgetType: 'wm-progress-bar', hostClass: DEFAULT_CLS };
// map of progress-bar type and classes
var TYPE_CLASS_MAP = {
    'default': '',
    'default-striped': 'progress-bar-striped',
    'success': 'progress-bar-success',
    'success-striped': 'progress-bar-success progress-bar-striped',
    'info': 'progress-bar-info',
    'info-striped': 'progress-bar-info progress-bar-striped',
    'warning': 'progress-bar-warning',
    'warning-striped': 'progress-bar-warning progress-bar-striped',
    'danger': 'progress-bar-danger',
    'danger-striped': 'progress-bar-danger progress-bar-striped'
};
var ProgressBarComponent = /** @class */ (function (_super) {
    __extends(ProgressBarComponent, _super);
    function ProgressBarComponent(inj, dataset, boundDataset) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        // progress-bar data, ngFor in the template iterates on this
        _this.data = [{
                cls: TYPE_CLASS_MAP.default,
                progressBarWidth: '0%',
                displayValue: '0'
            }];
        // flag which determines whether dataset is provided or not
        _this.hasDataset = !!(dataset || boundDataset);
        styler(_this.nativeElement, _this);
        _this._prepareData = _.debounce(function () { return _this.prepareData(); }, 50);
        return _this;
    }
    ProgressBarComponent_1 = ProgressBarComponent;
    // update the proper classes when there is a change in type
    ProgressBarComponent.prototype.onTypeChange = function () {
        if (!this.hasDataset) {
            if (this.data[0]) {
                this.data[0].cls = TYPE_CLASS_MAP[this.type];
            }
        }
    };
    // returns the formatted display value based on the provided displayformat
    ProgressBarComponent.prototype.getFormattedDisplayVal = function (val) {
        var format = this.displayformat || '9';
        val = parseFloat('' + val);
        val = (val.toFixed(getDecimalCount(format)));
        if (format && format.includes('%')) {
            val = val + "%";
        }
        return val;
    };
    ProgressBarComponent.prototype.prepareData = function () {
        var _this = this;
        // when the dataset is provided, iterate over the dataset to set the proper values in the data
        if (this.dataset && _.isArray(this.dataset) && this.type && this.datavalue) {
            this.data = this.dataset.map(function (datum) {
                var val = findValueOf(datum, _this.datavalue).toString();
                var percentVal = val;
                if (val && !val.includes('%')) {
                    percentVal = val + "%";
                }
                return {
                    cls: TYPE_CLASS_MAP[findValueOf(datum, _this.type)],
                    progressBarWidth: percentVal,
                    displayValue: _this.getFormattedDisplayVal(val)
                };
            });
        }
        else {
            // if the dataset is not provided, update the values in the default data
            var width = 0;
            var displayVal = 0;
            if (this.datavalue) {
                if (isPercentageValue(this.datavalue)) {
                    var val = (this.datavalue || '0%');
                    width = displayVal = val;
                }
                else {
                    if (isDefined(this.datavalue)) {
                        var denominator = (+this.maxvalue - +this.minvalue) || 1;
                        var val = ((+this.datavalue - +this.minvalue) * 100) / denominator + '%';
                        width = displayVal = val;
                    }
                }
            }
            this.data[0].displayValue = this.getFormattedDisplayVal(displayVal);
            this.data[0].progressBarWidth = width;
            this.data[0].cls = TYPE_CLASS_MAP[this.type];
        }
    };
    ProgressBarComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'type') {
            this.onTypeChange();
        }
        else if (key === 'minvalue' || key === 'maxvalue' || key === 'datavalue' || key === 'dataset' || key === 'displayformat') {
            this._prepareData();
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var ProgressBarComponent_1;
    ProgressBarComponent.initializeProps = registerProps();
    ProgressBarComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: String, decorators: [{ type: Attribute, args: ['dataset',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['dataset.bind',] }] }
    ]; };
    ProgressBarComponent = ProgressBarComponent_1 = __decorate([
        Component({
            selector: '[wmProgressBar]',
            template: "<div class=\"multi-bar progress-bar {{bar.cls}}\" *ngFor=\"let bar of data\" [style.width]=\"bar.progressBarWidth\" role=\"progressbar\"\n     [attr.aria-valuenow]=\"datavalue\" [attr.aria-valuemin]=\"minvalue\" [attr.aria-valuemax]=\"maxvalue\">\n    <span class=\"app-progress-label\" [attr.data-caption-placement]=\"captionplacement\">{{bar.displayValue}}</span>\n</div>",
            providers: [
                provideAsWidgetRef(ProgressBarComponent_1)
            ]
        }),
        __param(1, Attribute('dataset')), __param(2, Attribute('dataset.bind')),
        __metadata("design:paramtypes", [Injector, String, String])
    ], ProgressBarComponent);
    return ProgressBarComponent;
}(StylableComponent));

var registerProps$1 = function () {
    register('wm-progress-circle', new Map([
        ['captionplacement', __assign({ value: 'inside' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['datavalue', __assign({ value: '30' }, PROP_STRING)],
        ['displayformat', __assign({ value: '9%' }, PROP_STRING)],
        ['hint', PROP_STRING],
        ['maxvalue', __assign({ value: 100 }, PROP_NUMBER)],
        ['minvalue', __assign({ value: 0 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['subtitle', PROP_STRING],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING],
        ['type', __assign({ value: 'default' }, PROP_STRING)]
    ]));
};

var DEFAULT_CLS$1 = 'progress app-progress circle';
var WIDGET_CONFIG$1 = { widgetType: 'wm-progress-circle', hostClass: DEFAULT_CLS$1 };
var DEFAULT_OPTIONS = {
    responsive: true,
    innerStrokeWidth: 10,
    outerStrokeWidth: 10,
    unitsFontSize: '15',
    space: -10,
    toFixed: 0,
    maxPercent: 100,
    showSubtitle: false,
    clockwise: true,
    startFromZero: false,
    renderOnClick: false,
    innerStrokeColor: '',
    outerStrokeColor: ''
};
// map of progress-bar type and classes
var TYPE_CLASS_MAP$1 = {
    'default': '',
    'success': 'progress-circle-success',
    'info': 'progress-circle-info',
    'warning': 'progress-circle-warning',
    'danger': 'progress-circle-danger',
};
var ProgressCircleComponent = /** @class */ (function (_super) {
    __extends(ProgressCircleComponent, _super);
    function ProgressCircleComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        styler(_this.nativeElement, _this);
        _this.options = _.clone(DEFAULT_OPTIONS);
        _this.redraw = _.debounce(_this._redraw, 100);
        return _this;
    }
    ProgressCircleComponent_1 = ProgressCircleComponent;
    ProgressCircleComponent.prototype._redraw = function () {
        this.circleRef.render();
    };
    ProgressCircleComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        this.invokeEventCallback('beforerender', { '$event': {} });
        this.options = Object.assign(this.circleRef.options, this.options);
    };
    ProgressCircleComponent.prototype.overrideDefaults = function (options) {
        _.extend(this.options, options);
    };
    ProgressCircleComponent.prototype.updateDisplayValueFormat = function () {
        // show title and subtitle only when captionplacement is 'inside'
        this.options.showTitle = this.options.showSubtitle = (this.captionplacement === 'inside');
        // show units when title value is empty and captionplacement is 'inside'
        this.options.showUnits = !this.title && this.options.showTitle && isPercentageValue(this.displayformat);
        this.options.toFixed = getDecimalCount(this.displayformat);
        if (this.options.showTitle) {
            this.options.title = this.title || 'auto';
            this.options.showSubtitle = !!this.subtitle;
            this.options.subtitle = this.subtitle || '';
        }
        this.redraw();
    };
    ProgressCircleComponent.prototype.onPropertyChange = function (key, nv, ov) {
        switch (key) {
            case 'type':
                this.$element.removeClass(TYPE_CLASS_MAP$1[ov]);
                this.$element.addClass(TYPE_CLASS_MAP$1[nv]);
                break;
            case 'minvalue':
            case 'maxvalue':
            case 'datavalue':
                if (isPercentageValue(this.datavalue)) {
                    this.percentagevalue = parseFloat(this.datavalue);
                }
                else {
                    this.percentagevalue = calculatePercent(parseFloat(this.datavalue), this.minvalue, this.maxvalue);
                }
                break;
            case 'displayformat':
            case 'captionplacement':
            case 'title':
            case 'subtitle':
                this.updateDisplayValueFormat();
                break;
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    var ProgressCircleComponent_1;
    ProgressCircleComponent.initializeProps = registerProps$1();
    ProgressCircleComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild(CircleProgressComponent, { static: true }),
        __metadata("design:type", CircleProgressComponent)
    ], ProgressCircleComponent.prototype, "circleRef", void 0);
    ProgressCircleComponent = ProgressCircleComponent_1 = __decorate([
        Component({
            selector: '[wmProgressCircle]',
            template: "<circle-progress [percent]=\"percentagevalue\" [options]=\"options\"></circle-progress>\n",
            providers: [
                provideAsWidgetRef(ProgressCircleComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], ProgressCircleComponent);
    return ProgressCircleComponent;
}(StylableComponent));

var components = [
    ProgressBarComponent,
    ProgressCircleComponent
];
var ProgressModule = /** @class */ (function () {
    function ProgressModule() {
    }
    ProgressModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                NgCircleProgressModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], ProgressModule);
    return ProgressModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ProgressModule, ProgressBarComponent as ɵa, registerProps as ɵb, ProgressCircleComponent as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
