import { __decorate, __metadata } from 'tslib';
import { SecurityContext, Injector, NgZone, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_STRING, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, register, registerFormWidget, styler, APPLY_STYLES_TYPE, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { DomSanitizer } from '@angular/platform-browser';
import { FormWidgetType, setCSS, setAttr, setHtml } from '@wm/core';
import { BaseFormCustomComponent } from '@wm/components/input';

const richTextProps = new Map([
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['height', PROP_STRING],
    ['hint', PROP_STRING],
    ['name', PROP_STRING],
    ['placeholder', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showpreview', Object.assign({ value: false }, PROP_BOOLEAN)],
    ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)]
]);
const registerProps = () => {
    register('wm-richtexteditor', richTextProps);
    registerFormWidget(FormWidgetType.RICHTEXT, new Map(richTextProps));
};

var RichTextEditorComponent_1;
const WIDGET_INFO = { widgetType: 'wm-richtexteditor', hostClass: 'app-richtexteditor clearfix' };
const getChangeEvt = () => {
    let changeEvt;
    // for IE the event constructor doesn't work so use the createEvent proto
    if (typeof (Event) === 'function') {
        changeEvt = new Event('change');
    }
    else {
        changeEvt = document.createEvent('Event');
        changeEvt.initEvent('change', true, true);
    }
    return changeEvt;
};
const ɵ0 = getChangeEvt;
const overrideSummerNote = () => {
    if (!$.summernote.__overidden) {
        // override summernote methods
        const origFn = $.summernote.ui.button.bind($.summernote);
        $.summernote.ui.button = (...args) => {
            const retVal = origFn(...args);
            const origCallback = retVal.callback;
            retVal.callback = ($node, options) => {
                // add bs3 btn class to the buttons
                $node.addClass('btn');
                return origCallback($node, options);
            };
            return retVal;
        };
        $.summernote.__overidden = true;
    }
};
const ɵ1 = overrideSummerNote;
let RichTextEditorComponent = RichTextEditorComponent_1 = class RichTextEditorComponent extends BaseFormCustomComponent {
    constructor(inj, domSanitizer, ngZone) {
        super(inj, WIDGET_INFO);
        this.domSanitizer = domSanitizer;
        this.ngZone = ngZone;
        this._operationStack = [];
        this.isEditorLoaded = false;
        this.EDITOR_DEFAULT_OPTIONS = {
            toolbar: [
                // [groupName, [list of button]]
                ['misc', ['undo', 'redo']],
                ['style', ['style']],
                ['fontname', ['fontname']],
                ['fontsize', ['fontsize']],
                ['height', ['height']],
                ['style', ['bold', 'italic', 'underline', 'clear']],
                ['font', ['strikethrough', 'superscript', 'subscript']],
                ['color', ['color']],
                ['insert', ['table', 'picture', 'link', 'video', 'hr']],
                ['para', ['ul', 'ol', 'paragraph']],
                ['misc', ['codeview', 'fullscreen', 'help']]
            ],
            callbacks: {
                onInit: () => {
                    this.isEditorLoaded = true;
                    if (this._operationStack.length) {
                        this._operationStack.forEach(operationParam => {
                            const key = Array.from(operationParam.keys())[0], val = operationParam.get(key);
                            this.performEditorOperation(key, val);
                        });
                        this._operationStack = [];
                    }
                },
                onChange: (contents, editable) => {
                    this.proxyModel = this.domSanitizer.sanitize(SecurityContext.HTML, contents.toString());
                    this.invokeOnChange(contents, getChangeEvt(), true);
                    this.invokeOnTouched();
                }
            },
            fontNames: ['Arial', 'Arial Black', 'Comic Sans MS', 'Courier New', 'Merriweather'],
            placeholder: 'write here...',
            minHeight: 100,
            disableResizeEditor: true
        };
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER, ['height']);
        overrideSummerNote();
    }
    get htmlcontent() {
        return this.performEditorOperation('code');
    }
    get datavalue() {
        return this.htmlcontent;
    }
    set datavalue(nv) {
        if (nv !== undefined && nv !== null) {
            this.$hiddenInputEle.val(nv);
            this.performEditorOperation('reset');
            this.performEditorOperation('code', nv);
        }
    }
    ngOnInit() {
        this.$richTextEditor = $(this.nativeElement.querySelector('[richTextEditor]'));
        this.$hiddenInputEle = $(this.nativeElement.querySelector('input.model-holder'));
        super.ngOnInit();
        this.initEditor();
    }
    initEditor() {
        this.ngZone.runOutsideAngular(() => {
            this.invokeEventCallback('beforerender', { '$event': {} });
            this.$richTextEditor.summernote(this.EDITOR_DEFAULT_OPTIONS);
        });
    }
    overrideDefaults(options) {
        _.extend(this.EDITOR_DEFAULT_OPTIONS, options);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'placeholder') {
            this.performEditorOperation('placeholder', nv);
        }
        else if (key === 'disabled' || key === 'readonly') {
            this.performEditorOperation(nv ? 'disable' : 'enable');
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    onStyleChange(key, nv, ov) {
        if (key === 'height') {
            this.performEditorOperation('height', nv);
        }
    }
    performEditorOperation(key, value) {
        if (this.isEditorLoaded) {
            if (key === 'height') {
                setCSS(this.nativeElement.querySelector('div.note-editable'), key, value, true);
            }
            else if (key === 'placeholder') {
                setAttr(this.nativeElement, key, value);
                setHtml(this.nativeElement.querySelector('div.note-placeholder'), value, true);
            }
            else {
                // if editor content is empty then summernote('code') is returning empty p tags like <p></br></p>. So checking for empty and returning undefined.
                if (arguments.length === 1 && key === 'code' && this.$richTextEditor.summernote('isEmpty')) {
                    const content = this.$richTextEditor.summernote('code');
                    return content === '' ? content : undefined;
                }
                return this.$richTextEditor.summernote(key, value);
            }
        }
        else {
            const op = new Map();
            op.set(key, value);
            this._operationStack.push(op);
            return;
        }
    }
    getCurrentPosition() {
        return this.performEditorOperation('createRange');
    }
    undo() {
        this.performEditorOperation('undo');
    }
    focus() {
        this.performEditorOperation('focus');
    }
    ngOnDestroy() {
        this.performEditorOperation('destroy');
        super.ngOnDestroy();
    }
};
RichTextEditorComponent.initializeProps = registerProps();
RichTextEditorComponent.ctorParameters = () => [
    { type: Injector },
    { type: DomSanitizer },
    { type: NgZone }
];
RichTextEditorComponent = RichTextEditorComponent_1 = __decorate([
    Component({
        selector: 'div[wmRichTextEditor]',
        template: "<div richTextEditor></div>\n<div [innerHTML]=\"proxyModel\" class=\"ta-preview\" *ngIf=\"showpreview\"></div>\n<input class=\"model-holder\" [disabled]=\"disabled\" hidden>\n",
        providers: [
            provideAs(RichTextEditorComponent_1, NG_VALUE_ACCESSOR, true),
            provideAsWidgetRef(RichTextEditorComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, DomSanitizer, NgZone])
], RichTextEditorComponent);

const components = [
    RichTextEditorComponent
];
let RichTextEditorModule = class RichTextEditorModule {
};
RichTextEditorModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], RichTextEditorModule);

/**
 * Generated bundle index. Do not edit.
 */

export { RichTextEditorModule, RichTextEditorComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
