import { __assign, __spread, __extends, __decorate, __metadata } from 'tslib';
import { SecurityContext, Injector, NgZone, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_STRING, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, register, registerFormWidget, styler, APPLY_STYLES_TYPE, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { DomSanitizer } from '@angular/platform-browser';
import { FormWidgetType, setCSS, setAttr, setHtml } from '@wm/core';
import { BaseFormCustomComponent } from '@wm/components/input';

var richTextProps = new Map([
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['height', PROP_STRING],
    ['hint', PROP_STRING],
    ['name', PROP_STRING],
    ['placeholder', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['showpreview', __assign({ value: false }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
]);
var registerProps = function () {
    register('wm-richtexteditor', richTextProps);
    registerFormWidget(FormWidgetType.RICHTEXT, new Map(richTextProps));
};

var WIDGET_INFO = { widgetType: 'wm-richtexteditor', hostClass: 'app-richtexteditor clearfix' };
var getChangeEvt = function () {
    var changeEvt;
    // for IE the event constructor doesn't work so use the createEvent proto
    if (typeof (Event) === 'function') {
        changeEvt = new Event('change');
    }
    else {
        changeEvt = document.createEvent('Event');
        changeEvt.initEvent('change', true, true);
    }
    return changeEvt;
};
var ɵ0 = getChangeEvt;
var overrideSummerNote = function () {
    if (!$.summernote.__overidden) {
        // override summernote methods
        var origFn_1 = $.summernote.ui.button.bind($.summernote);
        $.summernote.ui.button = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var retVal = origFn_1.apply(void 0, __spread(args));
            var origCallback = retVal.callback;
            retVal.callback = function ($node, options) {
                // add bs3 btn class to the buttons
                $node.addClass('btn');
                return origCallback($node, options);
            };
            return retVal;
        };
        $.summernote.__overidden = true;
    }
};
var ɵ1 = overrideSummerNote;
var RichTextEditorComponent = /** @class */ (function (_super) {
    __extends(RichTextEditorComponent, _super);
    function RichTextEditorComponent(inj, domSanitizer, ngZone) {
        var _this = _super.call(this, inj, WIDGET_INFO) || this;
        _this.domSanitizer = domSanitizer;
        _this.ngZone = ngZone;
        _this._operationStack = [];
        _this.isEditorLoaded = false;
        _this.EDITOR_DEFAULT_OPTIONS = {
            toolbar: [
                // [groupName, [list of button]]
                ['misc', ['undo', 'redo']],
                ['style', ['style']],
                ['fontname', ['fontname']],
                ['fontsize', ['fontsize']],
                ['height', ['height']],
                ['style', ['bold', 'italic', 'underline', 'clear']],
                ['font', ['strikethrough', 'superscript', 'subscript']],
                ['color', ['color']],
                ['insert', ['table', 'picture', 'link', 'video', 'hr']],
                ['para', ['ul', 'ol', 'paragraph']],
                ['misc', ['codeview', 'fullscreen', 'help']]
            ],
            callbacks: {
                onInit: function () {
                    _this.isEditorLoaded = true;
                    if (_this._operationStack.length) {
                        _this._operationStack.forEach(function (operationParam) {
                            var key = Array.from(operationParam.keys())[0], val = operationParam.get(key);
                            _this.performEditorOperation(key, val);
                        });
                        _this._operationStack = [];
                    }
                },
                onChange: function (contents, editable) {
                    _this.proxyModel = _this.domSanitizer.sanitize(SecurityContext.HTML, contents.toString());
                    _this.invokeOnChange(contents, getChangeEvt(), true);
                    _this.invokeOnTouched();
                }
            },
            fontNames: ['Arial', 'Arial Black', 'Comic Sans MS', 'Courier New', 'Merriweather'],
            placeholder: 'write here...',
            minHeight: 100,
            disableResizeEditor: true
        };
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER, ['height']);
        overrideSummerNote();
        return _this;
    }
    RichTextEditorComponent_1 = RichTextEditorComponent;
    Object.defineProperty(RichTextEditorComponent.prototype, "htmlcontent", {
        get: function () {
            return this.performEditorOperation('code');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RichTextEditorComponent.prototype, "datavalue", {
        get: function () {
            return this.htmlcontent;
        },
        set: function (nv) {
            if (nv !== undefined && nv !== null) {
                this.$hiddenInputEle.val(nv);
                this.performEditorOperation('reset');
                this.performEditorOperation('code', nv);
            }
        },
        enumerable: true,
        configurable: true
    });
    RichTextEditorComponent.prototype.ngOnInit = function () {
        this.$richTextEditor = $(this.nativeElement.querySelector('[richTextEditor]'));
        this.$hiddenInputEle = $(this.nativeElement.querySelector('input.model-holder'));
        _super.prototype.ngOnInit.call(this);
        this.initEditor();
    };
    RichTextEditorComponent.prototype.initEditor = function () {
        var _this = this;
        this.ngZone.runOutsideAngular(function () {
            _this.invokeEventCallback('beforerender', { '$event': {} });
            _this.$richTextEditor.summernote(_this.EDITOR_DEFAULT_OPTIONS);
        });
    };
    RichTextEditorComponent.prototype.overrideDefaults = function (options) {
        _.extend(this.EDITOR_DEFAULT_OPTIONS, options);
    };
    RichTextEditorComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'placeholder') {
            this.performEditorOperation('placeholder', nv);
        }
        else if (key === 'disabled' || key === 'readonly') {
            this.performEditorOperation(nv ? 'disable' : 'enable');
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    RichTextEditorComponent.prototype.onStyleChange = function (key, nv, ov) {
        if (key === 'height') {
            this.performEditorOperation('height', nv);
        }
    };
    RichTextEditorComponent.prototype.performEditorOperation = function (key, value) {
        if (this.isEditorLoaded) {
            if (key === 'height') {
                setCSS(this.nativeElement.querySelector('div.note-editable'), key, value, true);
            }
            else if (key === 'placeholder') {
                setAttr(this.nativeElement, key, value);
                setHtml(this.nativeElement.querySelector('div.note-placeholder'), value, true);
            }
            else {
                // if editor content is empty then summernote('code') is returning empty p tags like <p></br></p>. So checking for empty and returning undefined.
                if (arguments.length === 1 && key === 'code' && this.$richTextEditor.summernote('isEmpty')) {
                    var content = this.$richTextEditor.summernote('code');
                    return content === '' ? content : undefined;
                }
                return this.$richTextEditor.summernote(key, value);
            }
        }
        else {
            var op = new Map();
            op.set(key, value);
            this._operationStack.push(op);
            return;
        }
    };
    RichTextEditorComponent.prototype.getCurrentPosition = function () {
        return this.performEditorOperation('createRange');
    };
    RichTextEditorComponent.prototype.undo = function () {
        this.performEditorOperation('undo');
    };
    RichTextEditorComponent.prototype.focus = function () {
        this.performEditorOperation('focus');
    };
    RichTextEditorComponent.prototype.ngOnDestroy = function () {
        this.performEditorOperation('destroy');
        _super.prototype.ngOnDestroy.call(this);
    };
    var RichTextEditorComponent_1;
    RichTextEditorComponent.initializeProps = registerProps();
    RichTextEditorComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: DomSanitizer },
        { type: NgZone }
    ]; };
    RichTextEditorComponent = RichTextEditorComponent_1 = __decorate([
        Component({
            selector: 'div[wmRichTextEditor]',
            template: "<div richTextEditor></div>\n<div [innerHTML]=\"proxyModel\" class=\"ta-preview\" *ngIf=\"showpreview\"></div>\n<input class=\"model-holder\" [disabled]=\"disabled\" hidden>\n",
            providers: [
                provideAs(RichTextEditorComponent_1, NG_VALUE_ACCESSOR, true),
                provideAsWidgetRef(RichTextEditorComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, DomSanitizer, NgZone])
    ], RichTextEditorComponent);
    return RichTextEditorComponent;
}(BaseFormCustomComponent));

var components = [
    RichTextEditorComponent
];
var RichTextEditorModule = /** @class */ (function () {
    function RichTextEditorModule() {
    }
    RichTextEditorModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], RichTextEditorModule);
    return RichTextEditorModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { RichTextEditorModule, RichTextEditorComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
