import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Attribute, ViewChild, ElementRef, ViewChildren, QueryList, Component, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { TypeaheadDirective, TypeaheadModule } from 'ngx-bootstrap/typeahead';
import { PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, register, registerFormWidget, interpolateBindExpressions, ALLFIELDS, extractDataAsArray, convertDataToObject, transformFormData, getUniqObjsByDataField, styler, getContainerTargetClass, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { Observable, of, from } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { FormWidgetType, DataSource, getClonedObject, AppConstants, findValueOf, addClass, isDefined, isMobile, adjustContainerPosition, toBoolean } from '@wm/core';
import { DatasetAwareFormComponent } from '@wm/components/input';

var searchProps = new Map([
    ['casesensitive', __assign({ value: false }, PROP_BOOLEAN)],
    ['class', PROP_STRING],
    ['datacompletemsg', __assign({ value: 'No more data to load' }, PROP_STRING)],
    ['datafield', __assign({ value: 'All Fields' }, PROP_STRING)],
    ['dataoptions', PROP_ANY],
    ['dataset', PROP_ANY],
    ['datasource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['disabled', PROP_BOOLEAN],
    ['displayimagesrc', PROP_STRING],
    ['displaylabel', PROP_STRING],
    ['displayexpression', PROP_STRING],
    ['dropup', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['imagewidth', __assign({ value: '16px' }, PROP_STRING)],
    ['limit', PROP_NUMBER],
    ['loadingdatamsg', __assign({ value: 'Loading items...' }, PROP_STRING)],
    ['matchmode', PROP_STRING],
    ['minchars', PROP_NUMBER],
    ['name', PROP_STRING],
    ['navsearchbar', __assign({ value: false }, PROP_BOOLEAN)],
    ['orderby', PROP_STRING],
    ['placeholder', __assign({ value: 'Search' }, PROP_STRING)],
    ['position', PROP_STRING],
    ['query', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['searchkey', PROP_STRING],
    ['searchon', __assign({ value: 'typing' }, PROP_STRING)],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['showsearchicon', PROP_BOOLEAN],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['type', __assign({ value: 'search' }, PROP_STRING)],
    ['debouncetime', __assign({ value: 250 }, PROP_NUMBER)],
    ['width', PROP_STRING]
]);
var registerProps = function () {
    register('wm-search', searchProps);
    registerFormWidget(FormWidgetType.AUTOCOMPLETE, new Map(searchProps));
    registerFormWidget(FormWidgetType.TYPEAHEAD, new Map(searchProps));
};

var LocalDataProvider = /** @class */ (function () {
    function LocalDataProvider() {
    }
    LocalDataProvider.prototype.applyFilter = function (entry, queryText, filtername) {
        entry = _.isNumber(entry) ? entry.toString() : entry;
        if (_.includes(filtername, 'start')) {
            return _.startsWith(entry, queryText);
        }
        else if (_.includes(filtername, 'end')) {
            return _.endsWith(entry, queryText);
        }
        else if (_.includes(filtername, 'exact')) {
            return _.isEqual(entry, queryText);
        }
        return _.includes(entry, queryText);
    };
    // LocalData filtering is done based on the searchkey.
    LocalDataProvider.prototype.filter = function (config) {
        var _this = this;
        var entries = config.dataset;
        var casesensitive = config.casesensitive;
        var matchMode = config.matchMode;
        // for supporting existing projects
        if (matchMode && !_.includes(matchMode, 'ignorecase')) {
            casesensitive = true;
        }
        var queryText = config.query, filteredData;
        return new Promise(function (resolve, reject) {
            /**
             * If searchKey is defined, then check for match string against each item in the dataset with item's field name as the searchKey
             * return the filtered data containing the matching string.
             */
            if (config.searchKey) {
                var keys_1 = _.split(config.searchKey, ',');
                filteredData = _.filter(config.dataset, function (item) {
                    return keys_1.some(function (key) {
                        var a = _.get(item, key), b = queryText;
                        if (!casesensitive) {
                            a = _.toLower(_.toString(a));
                            b = _.toLower(_.toString(b));
                        }
                        return _this.applyFilter(a, b, matchMode);
                    });
                });
            }
            else {
                // local search on data with array of objects.
                // Iterate over each item and return the filtered data containing the matching string.
                if (_.isArray(entries) && _.isObject(entries[0])) {
                    filteredData = _.filter(entries, function (entry) {
                        var a = _.values(entry).join(' ');
                        if (!casesensitive) {
                            a = _.toLower(a);
                            queryText = _.toLower(queryText);
                        }
                        return _this.applyFilter(a, queryText);
                    });
                }
                else {
                    filteredData = _.filter(entries, function (entry) {
                        if (!casesensitive) {
                            entry = _.toLower(entry);
                            queryText = _.toLower(queryText);
                        }
                        return _this.applyFilter(entry, queryText, matchMode);
                    });
                }
            }
            resolve({
                data: filteredData,
                hasMoreData: false,
                isLastPage: true
            });
        });
    };
    return LocalDataProvider;
}());

var RemoteDataProvider = /** @class */ (function () {
    function RemoteDataProvider() {
    }
    RemoteDataProvider.prototype.filter = function (config) {
        var _this = this;
        return this.filterData(config).then(function (response) { return _this.onFilterSuccess(config, response); }, function () { return _this.onFilterFailure(); });
    };
    RemoteDataProvider.prototype.filterData = function (config) {
        if (config.dataoptions) {
            var dataoptions_1 = config.dataoptions;
            var requestParams_1 = config.datasource.execute(DataSource.Operation.GET_REQUEST_PARAMS, config);
            // If options are specified, make specifics calls to fetch the results
            // Fetch the related field data
            if (dataoptions_1.relatedField) {
                return new Promise(function (resolve, reject) {
                    interpolateBindExpressions(config.viewParent, dataoptions_1.filterExpr, function (filterexpressions) {
                        requestParams_1.filterExpr = dataoptions_1.filterExpr = filterexpressions;
                        dataoptions_1.filterExpr = filterexpressions;
                        config.datasource.execute(DataSource.Operation.GET_RELATED_TABLE_DATA, _.assign({ relatedField: dataoptions_1.relatedField }, requestParams_1)).then(resolve, reject);
                    });
                });
            }
            // Fetch the distinct data
            if (dataoptions_1.distinctField) {
                return config.datasource.execute(DataSource.Operation.GET_DISTINCT_DATA_BY_FIELDS, {
                    pagesize: config.limit || config.pagesize,
                    page: config.page,
                    fields: dataoptions_1.distinctField,
                    entityName: dataoptions_1.tableName,
                    filterFields: _.assign(dataoptions_1.filterFields, requestParams_1.filterFields),
                    filterExpr: getClonedObject(dataoptions_1.filterExpr || {})
                });
            }
        }
        // search records using the searchkey
        return config.datasource.execute(DataSource.Operation.SEARCH_RECORDS, config);
    };
    // Check if the page retrieved currently is the last page. If last page, don't send any more request
    RemoteDataProvider.prototype.isLast = function (page, dataSize, maxResults, currentResults) {
        // if last page info is not returned by backend and current results is less than max results, this is the last page
        if (dataSize === AppConstants.INT_MAX_VALUE) {
            return currentResults !== 0 && currentResults < maxResults;
        }
        var pageCount = ((dataSize > maxResults) ? (Math.ceil(dataSize / maxResults)) : (dataSize < 0 ? 0 : 1));
        return page === pageCount;
    };
    // this function transform the response data in case it is not an array
    RemoteDataProvider.prototype.getTransformedData = function (variable, data) {
        var operationResult = variable.operation + 'Result'; // when output is only string it is available as oprationNameResult
        var tempResponse = data[operationResult];
        // in case data received is value as string then add that string value to object and convert object into array
        if (tempResponse) {
            var tempObj = {};
            _.set(tempObj, operationResult, tempResponse);
            data = [tempObj]; // convert data into an array having tempObj
        }
        else {
            // in case data received is already an object then convert it into an array
            data = [data];
        }
        return data;
    };
    RemoteDataProvider.prototype.onFilterFailure = function () {
        return [];
    };
    RemoteDataProvider.prototype.isLastPageForDistinctApi = function (data, page, totalElements, _isLastPage) {
        return page > 1 && !_isLastPage && _.isEmpty(data) && totalElements === AppConstants.INT_MAX_VALUE;
    };
    // this function processes the response depending on pageOptions, isPageable and prepares the formattedDataset.
    RemoteDataProvider.prototype.onFilterSuccess = function (config, response) {
        var _this = this;
        var data = response.data;
        var formattedData;
        var _isLastPage;
        var page;
        var isPaginatedData;
        var expressionArray = _.split(config.binddataset, '.');
        var dataExpression = _.slice(expressionArray, _.indexOf(expressionArray, 'dataSet') + 1).join('.');
        var $I = '[$i]';
        return new Promise(function (resolve, reject) {
            var pageOptions = response.pagination;
            if (config.datasource.execute(DataSource.Operation.IS_PAGEABLE)) {
                page = pageOptions.number + 1;
                _isLastPage = _this.isLast(page, (config.limit > 0 && config.limit) || pageOptions.totalElements, pageOptions.size, pageOptions.numberOfElements);
                isPaginatedData = true;
                if (_this.isLastPageForDistinctApi(data, page, pageOptions.totalElements, _isLastPage)) {
                    _isLastPage = true;
                    resolve({
                        data: [],
                        isLastPage: _isLastPage,
                        hasMoreData: page > 1,
                        isPaginatedData: isPaginatedData,
                        page: page
                    });
                    return;
                }
            }
            // if data expression exists, extract the data from the expression path
            if (dataExpression) {
                if (_.isArray(data)) {
                    var index = dataExpression.lastIndexOf($I);
                    if (index > -1) {
                        var restExpr_1 = dataExpression.substr(index + 5);
                        formattedData = data.map(function (datum) { return findValueOf(datum, restExpr_1); });
                    }
                }
                else if (_.isObject(data)) {
                    formattedData = _.get(data, dataExpression);
                }
                data = formattedData || data;
            }
            if (!_.isArray(data)) {
                data = _this.getTransformedData(config.datasource, data);
            }
            // in case of no data received, resolve the promise with empty array
            if (!data.length) {
                resolve({ data: [], isLastPage: _isLastPage, hasMoreData: page > 1, isPaginatedData: isPaginatedData, page: page });
            }
            else {
                resolve({ data: data, isLastPage: _isLastPage, hasMoreData: page > 1, isPaginatedData: isPaginatedData, page: page });
            }
        });
    };
    return RemoteDataProvider;
}());

var DataProvider = /** @class */ (function () {
    function DataProvider() {
    }
    DataProvider.prototype.filter = function (config) {
        var _this = this;
        var promise;
        /**
         * Make call to remoteDataProvider when searchkey is available and data is not from local / model variable.
         * Otherwise use localDataProvider
         * If datasource is a serviceVariable with no input params, then perform local search.
         * when there is no dataset on the datasource when first time make a remote call to set the dataset for service variable.
         */
        var hasNoVariableDataset = config.datasource && config.datasource.execute(DataSource.Operation.IS_UPDATE_REQUIRED, config.hasData);
        if (!config.isLocalFilter && (config.dataoptions || ((config.datasource && config.datasource.execute(DataSource.Operation.IS_API_AWARE))
            && config.searchKey
            && hasNoVariableDataset))) {
            promise = DataProvider.remoteDataProvider.filter(config);
        }
        else {
            promise = DataProvider.localDataProvider.filter(config);
        }
        return promise.then(function (response) {
            _this.updateDataset = config.datasource && !config.datasource.execute(DataSource.Operation.SUPPORTS_CRUD) && hasNoVariableDataset;
            _this.hasMoreData = response.hasMoreData;
            _this.isLastPage = response.isLastPage;
            _this.page = response.page;
            _this.isPaginatedData = response.isPaginatedData;
            return response;
        });
    };
    DataProvider.remoteDataProvider = new RemoteDataProvider();
    DataProvider.localDataProvider = new LocalDataProvider();
    return DataProvider;
}());

var WIDGET_CONFIG = { widgetType: 'wm-search', hostClass: 'input-group' };
var SearchComponent = /** @class */ (function (_super) {
    __extends(SearchComponent, _super);
    function SearchComponent(inj, binddatavalue, binddataset) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.binddatavalue = binddatavalue;
        _this.binddataset = binddataset;
        _this.query = '';
        _this.page = 1;
        // this flag will not allow the empty datafield values.
        _this.allowempty = false;
        addClass(_this.nativeElement, 'app-search', true);
        /**
         * Listens for the change in the ngModel on every search and retrieves the data as observable.
         * This observable data is passed to the typeahead.
         * @type {Observable<any>}
         */
        _this.typeaheadDataSource = Observable
            .create(function (observer) {
            // Runs on every search
            if (_this.listenQuery) {
                _this._defaultQueryInvoked = false;
                _this._loadingItems = true;
                observer.next(_this.query);
            }
            // on keydown, while scrolling the dropdown items, when last item is reached next call is triggered
            // unless the call is resolved, we are able to scroll next to first item and soon
            // This shows flickering from first item to next new items appended.
            // By setting container to undefined, key events changes will be stopped while loading items
            if (_this.lastSelectedIndex) {
                _this.typeahead._container = undefined;
            }
        }).pipe(mergeMap(function (token) { return _this.getDataSourceAsObservable(token); }));
        _this.dataProvider = new DataProvider();
        /**
         * When default datavalue is not found within the dataset, a filter call is made to get the record using fetchDefaultModel.
         * after getting the response, set the queryModel and query.
         */
        var datavalueSubscription = _this.datavalue$.subscribe(function (val) {
            var query = (_.isArray(val) ? val[0] : val);
            if (query === null || query === '') {
                _this._modelByValue = '';
                // reset the query.
                _this.query = _this.queryModel = '';
                // on clear or reset filter, empty the lastResults to fetch new records.
                _this._lastResult = undefined;
                return;
            }
            if (!_this._unsubscribeDv) {
                _this._defaultQueryInvoked = false;
                // if prev datavalue is not equal to current datavalue then clear the modelByKey and queryModel
                if (!_.isObject(val) && _this.prevDatavalue !== val) {
                    _this._modelByKey = undefined;
                    _this.query = _this.queryModel = '';
                }
                // if the datafield is ALLFILEDS do not fetch the records
                // update the query model with the values we have
                _this.updateByDatavalue(val);
            }
        });
        _this.registerDestroyListener(function () { return datavalueSubscription.unsubscribe(); });
        var datasetSubscription = _this.dataset$.subscribe(function () {
            // set the next item index.
            _this.startIndex = _this.datasetItems.length;
            _this._lastResult = undefined;
            var defaultValue = _this.datavalue || _this.toBeProcessedDatavalue;
            // invoking updateByDataset only when datavalue is present, as entered value i.e. query is setting back to ''
            if (isDefined(defaultValue)) {
                _this.updateByDataset(defaultValue);
            }
        });
        _this.registerDestroyListener(function () { return datasetSubscription.unsubscribe(); });
        return _this;
    }
    SearchComponent_1 = SearchComponent;
    Object.defineProperty(SearchComponent.prototype, "datasource", {
        // getter setter is added to pass the datasource to searchcomponent.
        get: function () {
            return this._datasource;
        },
        set: function (nv) {
            this._datasource = nv;
            var data = this.datavalue || this.toBeProcessedDatavalue;
            this.updateByDatavalue(data);
        },
        enumerable: true,
        configurable: true
    });
    // on clear, trigger search with page size 1
    SearchComponent.prototype.clearSearch = function ($event, loadOnClear) {
        this.query = '';
        this.onInputChange($event);
        this.dataProvider.isLastPage = false;
        this.listenQuery = false;
        if (loadOnClear) {
            this.listenQuery = true;
            this._unsubscribeDv = false;
            this.loadMoreData();
        }
        this.invokeEventCallback('clearsearch');
    };
    // function to  clear the input value
    SearchComponent.prototype.clearText = function () {
        this.query = '';
        this.showClosebtn = false;
    };
    // Close the full screen mode in mobile view of auto complete
    SearchComponent.prototype.closeSearch = function () {
        this._loadingItems = false;
        this.page = 1;
        // after closing the search, insert the element at its previous position (elIndex)
        this.insertAtIndex(this.elIndex);
        this.elIndex = undefined;
        this.parentEl = undefined;
        this.$element.removeClass('full-screen');
        if (this._domUpdated) {
            this._domUpdated = false;
        }
        this.listenQuery = false;
        this._unsubscribeDv = true;
        this.typeahead.hide();
    };
    SearchComponent.prototype.renderMobileAutoComplete = function () {
        // Get the parent element of the search element which can be next or prev element, if both are empty then get the parent of element.
        if (!isDefined(this.elIndex)) {
            this.parentEl = this.$element.parent();
            this.elIndex = this.parentEl.children().index(this.$element);
        }
        if (!this.$element.hasClass('full-screen')) {
            // this flag is set to notify that the typeahead-container dom has changed its position
            this._domUpdated = true;
            // Add full screen class on focus of the input element.
            this.$element.addClass('full-screen');
            // Add position to set the height to auto
            if (this.position === 'inline') {
                this.$element.addClass(this.position);
            }
        }
        // focus is lost when element is changed to full-screen, keydown to select next items will not work
        // Hence explicitly focusing the input
        if (this.$element.hasClass('full-screen')) {
            this.$element.find('.app-search-input').focus();
        }
    };
    SearchComponent.prototype.getDataSourceAsObservable = function (query) {
        // show dropdown only when there is change in query. This should not apply when dataoptions with filterFields are updated.
        // when lastResult is not available i.e. still the first call is pending and second query is invoked then do not return.
        if (this._lastQuery === query && !_.get(this.dataoptions, 'filterFields') && isDefined(this._lastResult)) {
            this._loadingItems = false;
            return of(this._lastResult);
        }
        /*
             WMS-19177: On focus of input field for autocomplete type, do not trigger api call when
                 a) searchkey and displayexpression are different
                 b) binddisplaylabel contains multiple values
                 c) binddisplaylabel contains single value which does not match with searchkey
        */
        if (this.type === 'autocomplete' && this.searchkey && this.query && this.datavalue) {
            var bindDisplayLabelArray = [];
            var searchKeyArray = _.split(this.searchkey, ',');
            var displaylabel = void 0;
            if (this.binddisplaylabel) {
                bindDisplayLabelArray = _.split(this.binddisplaylabel, '+');
                if (bindDisplayLabelArray.length === 1) {
                    if (!_.includes(bindDisplayLabelArray[0], '|')) {
                        var regex = /\[(.*?)\]/;
                        displaylabel = regex.exec(bindDisplayLabelArray[0])[1];
                    }
                    else {
                        displaylabel = true;
                    }
                }
            }
            if ((bindDisplayLabelArray.length > 1 || (displaylabel && !_.includes(searchKeyArray, displaylabel))
                || (this.displayexpression && !_.includes(searchKeyArray, this.displayexpression)))) {
                this._loadingItems = false;
                return of([]);
            }
        }
        this._lastQuery = this.query;
        return from(this.getDataSource(query));
    };
    SearchComponent.prototype.handleEvent = function (node, eventName, eventCallback, locals) {
        if (!_.includes(['blur', 'focus', 'select', 'submit', 'change'], eventName)) {
            _super.prototype.handleEvent.call(this, node, eventName, eventCallback, locals);
        }
    };
    // highlight the characters in the dropdown matching the query.
    SearchComponent.prototype.highlight = function (match, query) {
        if (this.typeaheadContainer) {
            // highlight of chars will work only when label are strings.
            match.value = match.item.label.toString();
            return this.typeaheadContainer.highlight(match, query);
        }
    };
    // inserts the element at the index position
    SearchComponent.prototype.insertAtIndex = function (i) {
        if (i === 0) {
            this.parentEl.prepend(this.$element);
        }
        else {
            var $elAtIndex = this.parentEl.children().eq(i);
            if ($elAtIndex.length) {
                this.$element.insertBefore(this.parentEl.children().eq(i));
            }
            else {
                this.$element.insertAfter(this.parentEl.children().eq(i - 1));
            }
        }
    };
    // Check if the widget is of type autocomplete in mobile view/ app
    SearchComponent.prototype.isMobileAutoComplete = function () {
        return this.type === 'autocomplete' && isMobile();
    };
    SearchComponent.prototype.loadMoreData = function (incrementPage) {
        if (this.dataProvider.isLastPage) {
            return;
        }
        // Increase the page number and trigger force query update
        this.page = incrementPage ? this.page + 1 : this.page;
        this.isScrolled = true;
        this._loadingItems = true;
        // when invoking new set of results, reset the lastQuery.
        if (incrementPage) {
            this._lastQuery = undefined;
        }
        // trigger the typeahead change manually to fetch the next set of results.
        this.typeahead.onInput({
            target: {
                value: _.trim(this.query) || '0' // dummy data to notify the observables
            }
        });
    };
    // on focusout, subscribe to the datavalue changes again
    SearchComponent.prototype.onFocusOut = function () {
        var _this = this;
        this._unsubscribeDv = false;
        this._loadingItems = false;
        // reset the page value on focusout.
        this.page = 1;
        // if domUpdated is true then do not hide the dropdown in the fullscreen
        if (!this._domUpdated && this._isOpen) {
            this.listenQuery = false;
            // hide the typeahead only after the item is selected from dropdown.
            setTimeout(function () {
                if (_this.typeahead._typeahead.isShown) {
                    _this.typeahead.hide();
                }
            }, 200);
        }
        this._isOpen = false;
        // on outside click, typeahead is hidden. To avoid this, when fullscreen is set, overridding isFocused flag on the typeahead container
        if (this._domUpdated && this.typeahead && this.typeahead._container) {
            this.typeahead._container.isFocused = true;
        }
    };
    SearchComponent.prototype.onInputChange = function ($event) {
        // reset all the previous page details in order to fetch new set of result.
        this.result = [];
        this.page = 1;
        this.listenQuery = this.isUpdateOnKeyPress();
        this._modelByValue = undefined;
        // when input is cleared, reset the datavalue
        if (this.query === '') {
            this.queryModel = '';
            this._modelByValue = '';
            this.invokeOnChange(this._modelByValue, {}, true);
            // trigger onSubmit only when the search input is cleared off and do not trigger when tab is pressed.
            if ($event && $event.which !== 9) {
                this.invokeEventCallback('submit', { $event: $event });
            }
        }
        else {
            // invoking change event on every input value change.
            this.invokeEventCallback('change', {
                $event: $event,
                newVal: this._modelByValue || this.query,
                oldVal: this.prevDatavalue
            });
        }
        this.showClosebtn = (this.query !== '');
    };
    // Triggered for enter event
    SearchComponent.prototype.handleEnterEvent = function ($event) {
        // submit event triggered when there is no search results
        if (!this.typeahead._container) {
            this.onSearchSelect($event);
        }
    };
    // Triggerred when typeahead option is selected.
    SearchComponent.prototype.onSearchSelect = function ($event) {
        var item;
        if (this.typeaheadContainer && this.typeaheadContainer.active) {
            item = this.typeaheadContainer.active.item;
        }
        $event = this.eventData($event, item || {});
        // searchOn is set as onBtnClick, then invoke the search api call manually.
        if (!this.isUpdateOnKeyPress()) {
            this.listenQuery = true;
            // trigger the typeahead change manually to fetch the next set of results.
            this.typeahead.onInput({
                target: {
                    value: this.query // dummy data to notify the observables
                }
            });
            return;
        }
        // when matches are available.
        if (this.typeaheadContainer && this.liElements.length) {
            this.typeaheadContainer.selectActiveMatch();
        }
        else {
            this.queryModel = this.query;
            this.invokeEventCallback('submit', { $event: $event });
        }
    };
    SearchComponent.prototype.invokeOnBeforeServiceCall = function (inputData) {
        this.invokeEventCallback('beforeservicecall', { inputData: inputData });
    };
    SearchComponent.prototype.onDropdownOpen = function () {
        var _this = this;
        // setting the ulElements, liElement on typeaheadContainer.
        // as we are using customOption template, liElements are not available on typeaheadContainer so append them explicitly.
        var fn = _.debounce(function () {
            _this._isOpen = true;
            _this.typeaheadContainer = _this.typeahead._container || _this.typeahead._typeahead.instance;
            _this.typeaheadContainer.liElements = _this.liElements;
            _this.typeaheadContainer.ulElement = _this.ulElement;
            adjustContainerPosition($('typeahead-container'), _this.nativeElement, _this.typeahead._typeahead, $('typeahead-container .dropdown-menu'));
        });
        fn();
        // open full-screen search view
        if (this.isMobileAutoComplete()) {
            if (!this.$element.hasClass('full-screen')) {
                this.renderMobileAutoComplete();
            }
            var dropdownEl = this.dropdownEl.closest('typeahead-container');
            dropdownEl.insertAfter(this.$element.find('input').first());
            var screenHeight = this.$element.closest('.app-content').height();
            dropdownEl.css({ position: 'relative', top: 0, height: screenHeight + 'px' });
            this.showClosebtn = this.query && this.query !== '';
            if (!_.isUndefined(this.dataProvider.isLastPage) && !this.dataProvider.isLastPage) {
                this.triggerSearch();
            }
        }
    };
    SearchComponent.prototype.selectNext = function () {
        var matches = this.typeaheadContainer.matches;
        if (!matches) {
            return;
        }
        var index = matches.indexOf(this.typeaheadContainer.active);
        // on keydown, if scroll is at the bottom and next page records are available, fetch next page items.
        if (!this._loadingItems && !this.dataProvider.isLastPage && index + 1 > matches.length - 1) {
            // index is saved in order to select the lastSelected item in the dropdown after fetching next page items.
            this.lastSelectedIndex = index;
            this.loadMoreData(true);
        }
    };
    SearchComponent.prototype.setLastActiveMatchAsSelected = function () {
        if (this.lastSelectedIndex) {
            this.typeaheadContainer._active = this.typeaheadContainer.matches[this.lastSelectedIndex];
            this.typeaheadContainer.nextActiveMatch();
            this.lastSelectedIndex = undefined;
        }
    };
    SearchComponent.prototype.triggerSearch = function () {
        if (this.dataProvider.isLastPage || !this.$element.hasClass('full-screen')) {
            return;
        }
        var typeAheadDropDown = this.dropdownEl;
        var $lastItem = typeAheadDropDown.find('li').last();
        // Check if last item is not below the full screen
        if ($lastItem.length && typeAheadDropDown.length && (typeAheadDropDown.height() + typeAheadDropDown.position().top > $lastItem.height() + $lastItem.position().top)) {
            this.loadMoreData(true);
        }
    };
    SearchComponent.prototype.isUpdateOnKeyPress = function () {
        return this.searchon === 'typing';
    };
    SearchComponent.prototype.debounceDefaultQuery = function (data) {
        var _this = this;
        this._defaultQueryInvoked = true;
        this.getDataSource(data, true).then(function (response) {
            if (response.length) {
                _this.queryModel = response;
                _this._lastQuery = _this.query = _this.queryModel[0].label || '';
                _this._modelByValue = _this.queryModel[0].value;
                _this._modelByKey = _this.queryModel[0].key;
            }
            else {
                _this._modelByValue = undefined;
                _this.queryModel = undefined;
                _this.query = '';
            }
        });
    };
    SearchComponent.prototype.updateByDatavalue = function (data) {
        this.updateByDataset(data);
        this.updateByDataSource(data);
    };
    SearchComponent.prototype.updateByDataSource = function (data) {
        // value is present but the corresponding key is not found then fetch next set
        // modelByKey will be set only when datavalue is available inside the localData otherwise make a N/w call.
        if (isDefined(data) && !_.isObject(data) && this.datasource && !isDefined(this._modelByKey) && this.datafield !== ALLFIELDS) {
            // Avoid making default query if queryModel already exists.
            if (isDefined(this.queryModel) && !_.isEmpty(this.queryModel)) {
                this.updateDatavalueFromQueryModel();
                return;
            }
            // Make default query call only when datasource supports CRUD (live variable).
            if (!this._defaultQueryInvoked && this.datasource.execute(DataSource.Operation.SUPPORTS_CRUD)) {
                this.debounceDefaultQuery(data);
            }
        }
    };
    // updates the model value using queryModel
    SearchComponent.prototype.updateDatavalueFromQueryModel = function () {
        if (isDefined(this.queryModel)) {
            this._modelByValue = _.isArray(this.queryModel) ? this.queryModel[0].value : this.queryModel;
            this._modelByKey = _.isArray(this.queryModel) ? this.queryModel[0].key : this.queryModel;
            this.toBeProcessedDatavalue = undefined;
        }
    };
    SearchComponent.prototype.updateByDataset = function (data) {
        // default query is already invoked then do not make other default query call.
        // For local search i.e. searchkey is undefined, do not return, verify the datavalue against the datasetItems .
        if (this._defaultQueryInvoked && this.searchkey) {
            return;
        }
        var selectedItem = _.find(this.datasetItems, function (item) {
            return (_.isObject(item.value) ? _.isEqual(item.value, data) : (_.toString(item.value)).toLowerCase() === (_.toString(data)).toLowerCase());
        });
        // set the default only when it is available in dataset.
        if (selectedItem) {
            this.queryModel = [selectedItem];
        }
        else if (this.datafield === ALLFIELDS && _.isObject(data)) {
            this.queryModel = this.getTransformedData(extractDataAsArray(data));
        }
        else {
            // resetting the queryModel only when prevDatavalue is equal to data
            if (this.prevDatavalue !== data) {
                this.queryModel = undefined;
                this.query = '';
                return;
            }
        }
        this.updateDatavalueFromQueryModel();
        // Show the label value on input.
        this._lastQuery = this.query = isDefined(this.queryModel) && this.queryModel.length ? _.get(this.queryModel[0], 'label') : '';
    };
    // If we have last search results then open dropdown on focus
    SearchComponent.prototype.handleFocus = function ($event) {
        if (this.type === 'search' && this.query === this._lastQuery && this._lastResult) {
            this.typeahead.keyUpEventEmitter.emit(this.query);
        }
    };
    // This method returns a promise that provides the filtered data from the datasource.
    SearchComponent.prototype.getDataSource = function (query, searchOnDataField, nextItemIndex) {
        var _this = this;
        // For default query, searchOnDataField is set to true, then do not make a n/w call when datafield is ALLFIELDS
        if (searchOnDataField && this.datafield === ALLFIELDS) {
            this._loadingItems = false;
            return Promise.resolve([]);
        }
        // For default datavalue, search key as to be on datafield to get the default data from the filter call.
        var dataConfig = {
            dataset: this.dataset ? convertDataToObject(this.dataset) : undefined,
            binddataset: this.binddataset,
            datasource: this.datasource,
            datafield: this.datafield,
            hasData: this.dataset && this.dataset.length,
            query: query,
            isLocalFilter: !this.searchkey,
            searchKey: searchOnDataField ? this.datafield : this.searchkey,
            // default search call match mode should be startignorecase
            matchMode: searchOnDataField ? 'startignorecase' : this.matchmode,
            casesensitive: this.casesensitive,
            isformfield: this.isformfield,
            orderby: this.orderby,
            limit: this.limit,
            pagesize: this.pagesize,
            page: this.page,
            onBeforeservicecall: this.invokeOnBeforeServiceCall.bind(this)
        };
        if (this.dataoptions) {
            dataConfig.dataoptions = this.dataoptions;
            dataConfig.viewParent = this.viewParent;
        }
        this._loadingItems = true;
        return this.dataProvider.filter(dataConfig)
            .then(function (response) {
            // response from dataProvider returns always data object.
            response = response.data || response;
            // for service variable, updating the dataset only if it is not defined or empty
            if ((!isDefined(_this.dataset) || !_this.dataset.length) && _this.dataProvider.updateDataset) {
                _this.dataset = response;
            }
            if (_this.dataProvider.hasMoreData) {
                _this.formattedDataset = _this.formattedDataset.concat(response);
            }
            else {
                _this.formattedDataset = response;
            }
            // explicitly setting the optionslimit as the matches more than 20 will be ignored if optionslimit is not specified.
            if (_this.formattedDataset.length > 20 && !isDefined(_this.limit)) {
                _this.typeahead.typeaheadOptionsLimit = _this.formattedDataset.length;
            }
            // In mobile, trigger the search by default until the results have height upto page height. Other results can be fetched by scrolling
            if (_this._isOpen && _this.isMobileAutoComplete() && !_.isUndefined(_this.dataProvider.isLastPage) && !_this.dataProvider.isLastPage) {
                _this.triggerSearch();
            }
            var transformedData = _this.getTransformedData(_this.formattedDataset, nextItemIndex);
            // result contains the datafield values.
            _this.result = _.map(transformedData, 'value');
            return transformedData;
        }, function (error) {
            _this._loadingItems = false;
            return [];
        }).then(function (result) {
            if (_this.isScrolled) {
                (_.debounce(function () {
                    _this.setLastActiveMatchAsSelected();
                }, 30))();
                _this.isScrolled = false;
            }
            // When no result is found, set the datavalue to undefined.
            if (!result.length) {
                _this._modelByValue = undefined;
                _this.queryModel = query;
            }
            // on focusout i.e. on other widget focus, if n/w is pending loading icon is shown, when data is available then dropdown is shown again.
            // on unsubscribing do not show the results.
            if (_this._unsubscribeDv) {
                result = [];
            }
            _this._loadingItems = false;
            _this._lastResult = result;
            return result;
        });
    };
    // this method returns the parent context i.e. Page context and not parent component's context.
    SearchComponent.prototype.getContext = function (context) {
        if (context && _.get(context, 'widget')) {
            return this.getContext(context.viewParent);
        }
        return context;
    };
    SearchComponent.prototype.getTransformedData = function (data, itemIndex, iscustom) {
        if (isDefined(itemIndex)) {
            itemIndex++;
        }
        var transformedData = transformFormData(this.getContext(this.viewParent), data, this.datafield, {
            displayField: this.displaylabel || this.displayfield,
            displayExpr: iscustom ? '' : this.displayexpression,
            bindDisplayExpr: iscustom ? '' : this.binddisplaylabel,
            bindDisplayImgSrc: this.binddisplayimagesrc,
            displayImgSrc: this.displayimagesrc
        }, itemIndex);
        return getUniqObjsByDataField(transformedData, this.datafield, this.displayfield || this.displaylabel, toBoolean(this.allowempty));
    };
    // OptionsListTemplate listens to the scroll event and triggers this function.
    SearchComponent.prototype.onScroll = function ($scrollEl, evt) {
        var totalHeight = $scrollEl.scrollHeight, clientHeight = $scrollEl.clientHeight;
        // If scroll is at the bottom and no request is in progress and next page records are available, fetch next page items.
        if (!this._loadingItems && !this.dataProvider.isLastPage && ($scrollEl.scrollTop + clientHeight >= totalHeight)) {
            this.loadMoreData(true);
        }
    };
    SearchComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        if (!isDefined(this.minchars)) {
            // for autocomplete set the minchars to 0
            if (this.type === 'autocomplete') {
                this.minchars = 0;
            }
            else {
                this.minchars = 1;
            }
        }
        this.listenQuery = this.isUpdateOnKeyPress();
        // by default for autocomplete do not show the search icon
        // by default show the searchicon for type = search
        this.showsearchicon = isDefined(this.showsearchicon) ? this.showsearchicon : (this.type === 'search');
    };
    SearchComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.nativeElement, this);
        this.containerTarget = getContainerTargetClass(this.nativeElement);
    };
    SearchComponent.prototype.eventData = function ($event, item) {
        if ($event) {
            $event['data'] = {
                item: item.dataObject,
                model: item.value,
                label: item.label,
                query: this.query
            };
        }
        return $event;
    };
    // triggered on select on option from the list. Set the queryModel, query and modelByKey from the matched item.
    SearchComponent.prototype.typeaheadOnSelect = function (match, $event) {
        var item = match.item;
        this.queryModel = item;
        item.selected = true;
        this.query = item.label;
        $event = $event || this.$typeaheadEvent;
        // As item.key can vary from key in the datasetItems
        this._modelByKey = item.key;
        this._modelByValue = item.value;
        $event = this.eventData($event, item);
        this.invokeOnTouched();
        this.invokeOnChange(this.datavalue, $event || {});
        // updating the variable bound to default value as invokeOnChange is not updating the variable.
        if (this.datavalue !== this.prevDatavalue) {
            this.updateBoundVariable(this.datavalue);
        }
        if (this.$element.hasClass('full-screen')) {
            this.closeSearch();
        }
        this.invokeEventCallback('select', { $event: $event, selectedValue: this.datavalue });
        this.invokeEventCallback('submit', { $event: $event });
        this.updatePrevDatavalue(this.datavalue);
    };
    SearchComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        // Backward compatability from 10.3.1 to 10.3.2
        // 10.3.1 we had displaylabel as modal value
        // From 10.3.2 onwords we will be having displayexpresion as modal value
        if (this.type === 'autocomplete' && key === 'displaylabel' && this.displaylabel && (!this.displayexpression || (this.displayexpression && !this.datasource))) {
            this.displayexpression = this.displaylabel;
            this.displaylabel = undefined;
        }
        // when dataoptions are provided and there is no displaylabel given then displaylabel is set as the relatedfield
        if (key === 'displaylabel' && this.dataoptions && this.binddisplaylabel === null) {
            this.query = _.get(this._modelByValue, nv) || this._modelByValue;
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    var SearchComponent_1;
    SearchComponent.initializeProps = registerProps();
    SearchComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: undefined, decorators: [{ type: Attribute, args: ['datavalue.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] }
    ]; };
    __decorate([
        ViewChild(TypeaheadDirective),
        __metadata("design:type", TypeaheadDirective)
    ], SearchComponent.prototype, "typeahead", void 0);
    __decorate([
        ViewChild('ulElement'),
        __metadata("design:type", ElementRef)
    ], SearchComponent.prototype, "ulElement", void 0);
    __decorate([
        ViewChildren('liElements'),
        __metadata("design:type", QueryList)
    ], SearchComponent.prototype, "liElements", void 0);
    SearchComponent = SearchComponent_1 = __decorate([
        Component({
            selector: '[wmSearch]',
            template: "<ng-template #customItemTemplate let-model=\"item\" let-index=\"index\" let-query=\"query\" let-match=\"match\">\n    <!-- Default item template -->\n    <a *ngIf=\"!content\">\n        <img *ngIf=\"model.imgSrc\" [src]=\"model.imgSrc\" alt=\"Search\" [style.width]=\"imagewidth\">\n        <span [title]=\"model.label\" [innerHtml]=\"highlight(match, query) || model.label\"></span>\n    </a>\n    <!-- Custom partial template -->\n    <a *ngIf=\"content\" class=\"customTemplate\" partial-container-target>\n    </a>\n</ng-template>\n\n<ng-template #customListTemplate let-matches=\"matches\" let-itemTemplate=\"itemTemplate\" let-query=\"query\">\n    <ul #ulElement class=\"app-search dropdown-menu\" scrollable>\n        <li #liElements *ngFor=\"let match of matches\" [wmItemTemplate]=\"content\" [userComponentParams]=\"match.item\"  partialContainer [ngClass]=\"{active: typeaheadContainer && typeaheadContainer.isActive(match)}\"\n            (mouseenter)=\"typeaheadContainer.selectActive(match);\"\n            (click)=\"typeaheadContainer.selectMatch(match, $event); $typeaheadEvent = $event;\">\n            <!-- itemTemplate comes from the <input> -->\n            <ng-container *ngIf=\"!match.isHeader()\" [ngTemplateOutlet]=\"itemTemplate\"\n                          [ngTemplateOutletContext]=\"{item: match.item, index: i, match: match, query: query}\">\n            </ng-container>\n        </li>\n        <div class=\"status\" [hidden]=\"_loadingItems || !(datacompletemsg && dataProvider.isLastPage)\">\n            <span [textContent]=\"datacompletemsg\"></span>\n        </div>\n        <div class=\"status\" [hidden]=\"!_loadingItems\">\n            <i class=\"fa fa-circle-o-notch fa-spin\"></i>\n            <span [textContent]=\"loadingdatamsg\"></span>\n        </div>\n    </ul>\n</ng-template>\n\n<!--This template is for search inside mobile navbar.-->\n<ng-container *ngIf=\"navsearchbar; else searchTemplate\">\n    <input [title]=\"query || ''\" type=\"text\" class=\"app-textbox form-control list-of-objs app-search-input\" [placeholder]=\"placeholder || ''\"\n           focus-target\n           container=\"body\"\n           [disabled]=\"disabled\"\n           autocomplete=\"off\"\n           [(ngModel)]=\"query\"\n           [readonly]=\"readonly\"\n           [disabled]=\"disabled\"\n           [typeahead]=\"typeaheadDataSource\"\n           [typeaheadWaitMs]=\"debouncetime\"\n           [typeaheadItemTemplate]=\"customItemTemplate\"\n           [optionsListTemplate]=\"customListTemplate\"\n           (input)=\"onInputChange($event)\"\n           (keydown)=\"listenQuery = true\"\n           (keydown.enter)=\"$typeaheadEvent = $event;handleEnterEvent($event)\"\n           (keydown.arrowdown)=\"selectNext($event)\"\n           (blur)=\"invokeOnTouched($event)\"\n           (click)=\"handleFocus($event);\"\n           (focus)=\"_unsubscribeDv = false; listenQuery = true; handleFocus($event); invokeOnFocus($event);\"\n           (focusout)=\"onFocusOut()\"\n           [typeaheadMinLength]=\"minchars\"\n           [typeaheadOptionsLimit]=\"limit\"\n           (typeaheadLoading)=\"_loadingItems\"\n           (typeaheadOnSelect)=\"typeaheadOnSelect($event)\"\n           typeaheadOptionField=\"label\"\n           [typeaheadAsync]=\"true\"\n           [typeaheadScrollable]=\"true\"\n           [typeaheadOptionsInScrollableView]=\"optionslimitinscrollableview\"\n           [dropup]=\"dropup\"\n           [tabindex]=\"tabindex\"\n           [attr.name]=\"name\"/>\n    <i class=\"btn-close wi wi-cancel\" [hidden]=\"!showClosebtn\" (click)=\"clearSearch($event);\"></i>\n</ng-container>\n<!--This template is for both web and fullscreen mode in mobile.-->\n<ng-template #searchTemplate>\n    <span class=\"wi wi-arrow-left form-control-feedback back-btn\" aria-hidden=\"true\" (click)=\"closeSearch()\"></span>\n    <span class=\"sr-only\">Back button</span>\n    <input [title]=\"query || ''\" type=\"text\" class=\"app-textbox form-control list-of-objs app-search-input\" [placeholder]=\"placeholder || ''\"\n           focus-target\n           [container]=\"containerTarget || 'body'\"\n           [disabled]=\"disabled\"\n           autocomplete=\"off\"\n           [(ngModel)]=\"query\"\n           [readonly]=\"readonly\"\n           [typeahead]=\"typeaheadDataSource\"\n           [typeaheadWaitMs]=\"debouncetime\"\n           [typeaheadItemTemplate]=\"customItemTemplate\"\n           [optionsListTemplate]=\"customListTemplate\"\n           (input)=\"onInputChange($event)\"\n           (keydown)=\"listenQuery = true\"\n           (keydown.enter)=\"$typeaheadEvent = $event;handleEnterEvent($event)\"\n           (keydown.arrowdown)=\"selectNext($event)\"\n           (blur)=\"invokeOnTouched($event); onFocusOut()\"\n           (click)=\"handleFocus($event);\"\n           (focus)=\"_unsubscribeDv = false; listenQuery = true; handleFocus($event); invokeOnFocus($event)\"\n           [typeaheadMinLength]=\"minchars\"\n           [typeaheadOptionsLimit]=\"limit\"\n           (typeaheadLoading)=\"_loadingItems\"\n           (typeaheadOnSelect)=\"typeaheadOnSelect($event)\"\n           typeaheadOptionField=\"label\"\n           [typeaheadAsync]=\"true\"\n           [typeaheadScrollable]=\"true\"\n           [typeaheadOptionsInScrollableView]=\"optionslimitinscrollableview\"\n           [tabindex]=\"tabindex\"\n           [dropup]=\"dropup\"\n           [attr.name]=\"name\">\n    <input class=\"model-holder\" ng-model=\"proxyModel\" ng-required=\"required\" tabindex=\"-1\">\n    <span *ngIf=\"_loadingItems\" aria-hidden=\"true\" class=\"fa fa-circle-o-notch fa-spin form-control-feedback\"></span>\n    <span class=\"wi wi-close form-control-feedback clear-btn\" [hidden]=\"!showClosebtn\" (click)=\"clearSearch($event, true)\"></span>\n    <span class=\"sr-only\">Clear button</span>\n    <span *ngIf=\"showsearchicon\" class=\"input-group-addon\" aria-label=\"search icon\" [ngClass]=\"{'disabled': disabled}\">\n        <form>\n            <button title=\"Search\" [disabled]=\"disabled\" class=\"app-search-button wi wi-search\" type=\"submit\"\n                    (click)=\"onSearchSelect($event)\"></button>\n        </form>\n    </span>\n</ng-template>\n",
            providers: [
                provideAs(SearchComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(SearchComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(SearchComponent_1)
            ]
        }),
        __param(1, Attribute('datavalue.bind')),
        __param(2, Attribute('dataset.bind')),
        __metadata("design:paramtypes", [Injector, Object, Object])
    ], SearchComponent);
    return SearchComponent;
}(DatasetAwareFormComponent));

var ScrollableDirective = /** @class */ (function () {
    function ScrollableDirective(inj, searchRef) {
        this.searchRef = searchRef;
        this.elementRef = inj.get(ElementRef);
    }
    ScrollableDirective.prototype.ngAfterContentInit = function () {
        // add the scroll event listener on the ul element.
        this.elementRef.nativeElement.addEventListener('scroll', this.notifyParent.bind(this));
        this.searchRef.dropdownEl = $(this.elementRef.nativeElement);
        this.searchRef.onDropdownOpen();
    };
    ScrollableDirective.prototype.ngAfterViewInit = function () {
        if (!this.searchRef.isMobileAutoComplete()) {
            // assigning width for the dropdown.
            var typeAheadInput = this.searchRef.$element.find('input').first();
            this.searchRef.dropdownEl.width(typeAheadInput.outerWidth());
        }
    };
    ScrollableDirective.prototype.notifyParent = function (evt) {
        this.searchRef.onScroll(this.elementRef.nativeElement, evt);
    };
    ScrollableDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: SearchComponent }
    ]; };
    ScrollableDirective = __decorate([
        Directive({
            selector: '[scrollable]'
        }),
        __metadata("design:paramtypes", [Injector, SearchComponent])
    ], ScrollableDirective);
    return ScrollableDirective;
}());

var components = [
    SearchComponent,
    ScrollableDirective
];
var SearchModule = /** @class */ (function () {
    function SearchModule() {
    }
    SearchModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                FormsModule,
                TypeaheadModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], SearchModule);
    return SearchModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { SearchComponent, SearchModule, registerProps as ɵa, ScrollableDirective as ɵb };
//# sourceMappingURL=index.js.map
