import { __assign, __extends, __decorate, __param, __metadata, __spread } from 'tslib';
import { Injector, Attribute, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_ANY, PROP_NUMBER, PROP_BOOLEAN, styler, APPLY_STYLES_TYPE, getEvaluatedData, getOrderedDataset, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { $parseExpr, getClonedObject, $appDigest, $parseEvent } from '@wm/core';

var registerProps = function () {
    register('wm-tree', new Map([
        ['class', PROP_STRING],
        ['dataset', __assign({ value: 'node1, node2, node3' }, PROP_ANY)],
        ['datavalue', PROP_STRING],
        ['levels', __assign({ value: 0 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['nodeaction', PROP_STRING],
        ['nodechildren', PROP_STRING],
        ['nodeclick', __assign({ value: 'none' }, PROP_STRING)],
        ['nodeicon', PROP_STRING],
        ['nodeid', PROP_STRING],
        ['nodelabel', PROP_STRING],
        ['orderby', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['treeicons', PROP_STRING]
    ]));
};

var defaultTreeIconClass = 'plus-minus';
var ICON_CLASSES = {
    'folder': {
        'expanded': 'wi-folder-open',
        'collapsed': 'wi-folder'
    },
    'circle-plus-minus': {
        'expanded': 'wi-remove-circle-outline',
        'collapsed': 'wi-add-circle-outline'
    },
    'chevron': {
        'expanded': 'wi-keyboard-arrow-down',
        'collapsed': 'wi-keyboard-arrow-right'
    },
    'menu': {
        'expanded': 'wi-arrow-down',
        'collapsed': 'wi-arrow-right'
    },
    'triangle': {
        'expanded': 'wi-arrow-drop-down-circle',
        'collapsed': 'wi-play-circle-filled'
    },
    'expand-collapse': {
        'expanded': 'wi-expand-less',
        'collapsed': 'wi-expand-more'
    },
    'plus-minus': {
        'expanded': 'wi-minus',
        'collapsed': 'wi-plus'
    }
};
var WIDGET_INFO = { widgetType: 'wm-tree', hostClass: 'app-tree' };
var TreeDirective = /** @class */ (function (_super) {
    __extends(TreeDirective, _super);
    function TreeDirective(inj, binddatavalue, bindnodelabel, bindnodeicon, bindnodechildren, bindnodeid) {
        var _this = _super.call(this, inj, WIDGET_INFO) || this;
        _this.binddatavalue = binddatavalue;
        _this.bindnodelabel = bindnodelabel;
        _this.bindnodeicon = bindnodeicon;
        _this.bindnodechildren = bindnodechildren;
        _this.bindnodeid = bindnodeid;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        _this.bindEvents();
        return _this;
    }
    TreeDirective_1 = TreeDirective;
    TreeDirective.prototype.onPropertyChange = function (key, nv, ov) {
        switch (key) {
            case 'dataset':
                this.nodes = this.getNodes(nv);
                this._selectNode = undefined;
                this.renderTree();
                break;
            case 'nodeicon':
            case 'nodelabel':
            case 'nodechildren':
            case 'orderby':
                this.renderTree();
                break;
            case 'treeicons':
                this.changeTreeIcons(nv, ov);
                break;
            case 'datavalue':
                this.selectById(nv);
                break;
            default:
                _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    TreeDirective.prototype.constructNodes = function (nodes, parent, levels, deep, _evalDataValue) {
        var _this = this;
        var $ul = $('<ul></ul>');
        var _iconClses = ICON_CLASSES[this.treeicons || defaultTreeIconClass];
        var _expr = (this.binddatavalue || this.datavalue);
        var _iconCls, _cls;
        _cls = levels > 0 ? ' expanded ' : ' collapsed ';
        _iconCls = _cls + (levels > 0 ? _iconClses.expanded : _iconClses.collapsed);
        deep = deep || 0;
        parent.append($ul);
        nodes.forEach(function (node, idx) {
            var $li = $('<li></li>'), $iconNode = $('<i></i>'), nodeLabel = getEvaluatedData(node, { expression: _this.nodelabel, bindExpression: _this.bindnodelabel }, _this.viewParent) || node.label, nodeIcon = getEvaluatedData(node, { expression: _this.nodeicon, bindExpression: _this.bindnodeicon }, _this.viewParent) || node.icon, nodeChildren = getEvaluatedData(node, { expression: _this.nodechildren, bindExpression: _this.bindnodechildren }, _this.viewParent) || node.children, nodeIdValue = getEvaluatedData(node, { expression: _this.nodeid, bindExpression: _this.bindnodeid }, _this.viewParent) || node.children;
            var isNodeMatched = false, expandCollapseIcon;
            $li.data('nodedata', node)
                .append($iconNode)
                .append("<span class=\"title\">" + nodeLabel + "</span>")
                .appendTo($ul);
            // if datavalue(ie, expr) is provided select the tree node accordingly
            // if datavalue === 'FirstNode' -- select the FirstNode at level 0
            // if datavalue === 'LastNode' -- select the LastNode at level 0
            // if datavalue is a bind expression evaluate the expression for each node of the tree till the condition is satisfied.
            // if node identifier is present then verify the datavalue is bound expr or static value and compare with the node model
            if (_this.bindnodeid || _this.nodeid) {
                isNodeMatched = _this.binddatavalue ? nodeIdValue === _evalDataValue : nodeIdValue === _expr;
                if (nodeIdValue) {
                    $li.attr('id', nodeIdValue);
                }
            }
            else if (_this.binddatavalue) { // evaluate the expression only if it is bound (useExpression)
                isNodeMatched = !!$parseExpr(_expr)(_this, node);
            }
            // Perform LastNode check only at level 0.(ie, deep = 0);
            if (!_this._selectNode) {
                if ((_expr === 'FirstNode' && idx === 0)
                    || (!deep && _expr === 'LastNode' && idx === nodes.length - 1)
                    || isNodeMatched) {
                    // save a reference of the node to be selected in `_selectNode`
                    _this._selectNode = $li;
                    _this.datavalue = nodeIdValue;
                }
            }
            if (nodeIcon) {
                $iconNode.addClass(nodeIcon);
            }
            if (nodeChildren && nodeChildren.length) { // parent node
                $li.addClass("parent-node " + _cls);
                expandCollapseIcon = $("<i class=\"wi " + _iconCls + "\"></i>");
                if (nodeIcon) {
                    $iconNode.addClass(nodeIcon);
                }
                $li.prepend(expandCollapseIcon);
                _this.constructNodes(nodeChildren, $li, levels - 1, deep + 1, _evalDataValue);
            }
            else {
                if (!nodeIcon) {
                    $iconNode.addClass('leaf-node');
                }
                $li.addClass('leaf-node');
            }
        });
    };
    TreeDirective.prototype.getNodesFromString = function (value) {
        return value.split(',').map(function (item) {
            return {
                'label': item && item.trim()
            };
        });
    };
    TreeDirective.prototype.getNodes = function (newVal) {
        var nodes;
        if (_.isString(newVal) && !_.isEmpty(newVal)) {
            newVal = newVal.trim();
            if (newVal) {
                nodes = this.getNodesFromString(newVal);
            }
        }
        else if (_.isArray(newVal)) {
            newVal = getOrderedDataset(newVal, this.orderby);
            nodes = newVal;
        }
        else if (_.isObject(newVal)) {
            nodes = [newVal];
        }
        return nodes;
    };
    TreeDirective.prototype.changeTreeIcons = function (nv, ov) {
        var $el = $(this.nativeElement);
        nv = nv || defaultTreeIconClass;
        ov = ov || defaultTreeIconClass;
        $el.find('i.expanded').removeClass(ICON_CLASSES[ov].expanded).addClass(ICON_CLASSES[nv].expanded);
        $el.find('i.collapsed').removeClass(ICON_CLASSES[ov].collapsed).addClass(ICON_CLASSES[nv].collapsed);
    };
    TreeDirective.prototype.toggleExpandCollapseNode = function ($event, $i, $li) {
        var treeIcons = ICON_CLASSES[this.treeicons || defaultTreeIconClass];
        var $liPath = $li.find('> span.title').parents('.app-tree li');
        var path = '';
        // construct the path of the node
        $liPath
            .each(function (i, el) {
            var current = $(el).children('.title').text();
            path = "/" + (current + path);
        });
        var eventParams = {
            '$event': $event,
            "$item": $li.data('nodedata'),
            "$path": path
        };
        if ($i.hasClass('collapsed')) {
            $i.removeClass("collapsed " + treeIcons.collapsed).addClass("expanded " + treeIcons.expanded);
            $li.removeClass('collapsed').addClass('expanded');
            this.invokeEventCallback('expand', eventParams);
        }
        else if ($i.hasClass('expanded')) {
            $i.removeClass("expanded " + treeIcons.expanded).addClass("collapsed " + treeIcons.collapsed);
            $li.removeClass('expanded').addClass('collapsed');
            this.invokeEventCallback('collapse', eventParams);
        }
    };
    TreeDirective.prototype.renderTree = function (forceRender) {
        var _this = this;
        var docFrag, $li, $liPath, data, path = '';
        var levels = +this.nativeElement.getAttribute('levels') || 0, $el = $(this.nativeElement);
        $el.empty();
        if (forceRender) {
            this._selectNode = undefined;
        }
        if (this.nodes && this.nodes.length) {
            docFrag = document.createDocumentFragment();
            this.constructNodes(this.nodes, $(docFrag), levels, 0, this.datavalue);
            $el.append(docFrag);
        }
        if (this._selectNode) {
            $li = this._selectNode;
            $li.addClass('selected');
            data = $li.data('nodedata');
            $liPath = $li.parentsUntil($el, 'li.parent-node.collapsed');
            if (!$liPath.length) {
                $liPath = $li;
            }
            $liPath
                .each(function () {
                var $current = $(_this), $i = $current.children('i.collapsed'), $title = $current.children('.title');
                _this.toggleExpandCollapseNode(undefined, $i, $current);
                path = "/" + ($title.text() + path);
            });
            this.selecteditem = getClonedObject(data) || {};
            this.selecteditem.path = path;
            this.invokeEventCallback('select', { $event: undefined, $item: data, $path: path });
            $appDigest();
        }
    };
    TreeDirective.prototype.selectNode = function (evt, value) {
        var _this = this;
        var target = evt && $(evt.target), $el = $(this.nativeElement), $li = _.isObject(value) ? value : $el.find("li[id=\"" + value + "\"]").first();
        var data, path = '', $liPath, nodeAction;
        $el.find('.selected').removeClass('selected');
        if (!$li.length) {
            return;
        }
        $li.addClass('selected');
        data = $li.data('nodedata');
        nodeAction = data[this.nodeaction || 'action'];
        // if the selectNode is initiated by click event then use the nativeElement target from event
        $liPath = target ? target.parents('.app-tree li') : $li.find('> span.title').parents('.app-tree li');
        // construct the path of the node
        $liPath
            .each(function (i, el) {
            var current = $(el).children('.title').text();
            path = "/" + (current + path);
        });
        // expand the current node till the viewParent level which is collapsed
        $li.parentsUntil($el, 'li.parent-node.collapsed')
            .each(function (index, el) {
            var $current = $(el), $i = $current.children('i.collapsed');
            _this.toggleExpandCollapseNode(undefined, $i, $current);
        });
        this.selecteditem = getClonedObject(data) || {};
        this.selecteditem.path = path;
        if (target) {
            if (this.nodeid) {
                this.datavalue = $parseExpr(this.nodeid)(this, data);
            }
            else if (this.bindnodeid) {
                this.datavalue = getEvaluatedData(data, { expression: this.nodeid, bindExpression: this.bindnodeid }, this.viewParent);
            }
            else {
                this.datavalue = getClonedObject(data) || {};
            }
        }
        if (nodeAction) {
            $parseEvent(nodeAction)(this);
        }
        this.invokeEventCallback('select', { $event: evt, $item: data, $path: path });
        $appDigest();
    };
    // click event is added on the host nativeElement
    TreeDirective.prototype.bindEvents = function () {
        var _this = this;
        $(this.nativeElement).on('click', function (evt) {
            var target = $(evt.target), li = target.closest('li'), $i = target.is('i') ? target : target.siblings('i.collapsed,i.expanded');
            evt.stopPropagation();
            if (target.is('span.title')) { // select the node only if it is nodelabel
                _this.selecteditem = {};
                _this.selectNode(evt, li);
            }
            if (target.is('i') || (target.is('span.title') && _this.nodeclick === 'expand')) {
                _this.toggleExpandCollapseNode(evt, $i, li);
            }
        });
    };
    TreeDirective.prototype.selectById = function (value) {
        this.selectNode(undefined, value);
    };
    TreeDirective.prototype.deselectById = function (value) {
        this.selecteditem = {};
        this.selectById();
    };
    TreeDirective.prototype.redraw = function () {
        this.renderTree(true);
    };
    var TreeDirective_1;
    TreeDirective.initializeProps = registerProps();
    TreeDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: undefined, decorators: [{ type: Attribute, args: ['datavalue.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['nodelabel.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['nodeicon.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['nodechildren.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['nodeid.bind',] }] }
    ]; };
    TreeDirective = TreeDirective_1 = __decorate([
        Directive({
            selector: 'div[wmTree]',
            providers: [
                provideAsWidgetRef(TreeDirective_1)
            ]
        }),
        __param(1, Attribute('datavalue.bind')),
        __param(2, Attribute('nodelabel.bind')),
        __param(3, Attribute('nodeicon.bind')),
        __param(4, Attribute('nodechildren.bind')),
        __param(5, Attribute('nodeid.bind')),
        __metadata("design:paramtypes", [Injector, Object, Object, Object, Object, Object])
    ], TreeDirective);
    return TreeDirective;
}(StylableComponent));

var components = [
    TreeDirective
];
var TreeModule = /** @class */ (function () {
    function TreeModule() {
    }
    TreeModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], TreeModule);
    return TreeModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { TreeModule, TreeDirective as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
