import { __decorate, __metadata } from 'tslib';
import { Injector, ContentChildren, Component, QueryList, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, StylableComponent, styler, APPLY_STYLES_TYPE, RedrawableDirective, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { noop, removeAttr, isNumber, StatePersistence, DynamicComponentRefProvider } from '@wm/core';

const registerProps = () => {
    register('wm-accordion', new Map([
        ['class', PROP_STRING],
        ['closeothers', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['defaultpaneindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['statehandler', Object.assign({ value: 'none' }, PROP_STRING)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)]
    ]));
};

const registerProps$1 = () => {
    register('wm-accordionpane', new Map([
        ['active', PROP_BOOLEAN],
        ['badgetype', Object.assign({ value: 'default' }, PROP_STRING)],
        ['badgevalue', PROP_STRING],
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['dynamicPaneIndex', PROP_NUMBER],
        ['iconclass', PROP_STRING],
        ['isdefaultpane', PROP_BOOLEAN],
        ['isdynamic', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['smoothscroll', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['subheading', PROP_STRING],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['title', Object.assign({ value: 'Title' }, PROP_STRING)]
    ]));
};

var AccordionPaneComponent_1;
const DEFAULT_CLS = 'app-accordion-panel panel';
const WIDGET_CONFIG = { widgetType: 'wm-accordionpane', hostClass: DEFAULT_CLS };
let AccordionPaneComponent = AccordionPaneComponent_1 = class AccordionPaneComponent extends StylableComponent {
    constructor(inj, accordionRef) {
        super(inj, WIDGET_CONFIG);
        this.accordionRef = accordionRef;
        this.isActive = false;
        this.$lazyLoad = noop;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SHELL);
        // title property here serves the purpose of heading.
        // remove title property as attribute
        removeAttr(this.nativeElement, 'title');
    }
    /**
     * handles the pane expand
     * invoke $lazyLoad method which takes care of loading the partial when the content property is provided - lazyLoading or partial
     * invoke redraw on the re-drawable children
     * invoke expand callback
     * notify parent about the change
     * @param {Event} evt
     */
    expand(evt) {
        if (this.isActive) {
            return;
        }
        this.isActive = true;
        this.$lazyLoad();
        this.redrawChildren();
        this.invokeEventCallback('expand', { $event: evt });
        this.notifyParent(true, evt);
    }
    /**
     * handles the pane collapse
     * invoke collapse callback
     * notify parent about the change
     * @param {Event} evt
     */
    collapse(evt) {
        if (!this.isActive) {
            return;
        }
        this.isActive = false;
        this.invokeEventCallback('collapse', { $event: evt });
        this.notifyParent(false, evt);
    }
    remove() {
        const paneIndex = this.accordionRef.getPaneIndexByRef(this);
        if (this.isActive && this.accordionRef.panes.length > 1) {
            this === this.accordionRef.panes.last ? this.accordionRef.panes.toArray()[paneIndex - 1].expand() : this.accordionRef.panes.toArray()[paneIndex + 1].expand();
        }
        const availablePanes = this.accordionRef.panes.toArray();
        availablePanes.splice(this.accordionRef.getPaneIndexByRef(this), 1);
        this.accordionRef.panes.reset([...availablePanes]);
        this.nativeElement.remove();
    }
    toggle(evt) {
        if (this.isActive) {
            this.collapse(evt);
        }
        else {
            this.expand(evt);
        }
    }
    // Todo - Vinay externalize
    redrawChildren() {
        setTimeout(() => {
            if (this.reDrawableComponents) {
                this.reDrawableComponents.forEach(c => c.redraw());
            }
        }, 100);
    }
    notifyParent(isExpand, evt) {
        this.accordionRef.notifyChange(this, isExpand, evt);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'content') {
            if (this.isActive) {
                setTimeout(() => this.$lazyLoad(), 100);
            }
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        styler(this.nativeElement.querySelector('.panel-body'), this, APPLY_STYLES_TYPE.INNER_SHELL);
        if (this.isdynamic) {
            this.accordionRef.registerDynamicPane(this);
        }
    }
};
AccordionPaneComponent.initializeProps = registerProps$1();
AccordionPaneComponent.ctorParameters = () => [
    { type: Injector },
    { type: AccordionDirective }
];
__decorate([
    ContentChildren(RedrawableDirective, { descendants: true }),
    __metadata("design:type", Object)
], AccordionPaneComponent.prototype, "reDrawableComponents", void 0);
AccordionPaneComponent = AccordionPaneComponent_1 = __decorate([
    Component({
        selector: 'div[wmAccordionPane]',
        template: "<div class=\"panel-heading clearfix\" role=\"tab\" (click)=\"toggle($event)\" [ngClass]=\"{active: isActive}\">\n    <h3 class=\"panel-title\">\n        <a href=\"javascript:void(0);\" role=\"button\" class=\"accordion-toggle\" [attr.aria-expanded]=\"isActive\" [attr.aria-controls]=\"name\">\n            <div class=\"pull-left\">\n                <i class=\"app-icon panel-icon {{iconclass}}\" [hidden]=\"!iconclass\"></i>\n            </div>\n            <div class=\"pull-left\">\n                <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\"></div>\n                <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n            </div>\n        </a>\n    </h3>\n    <div class=\"panel-actions\">\n        <span class=\"label label-{{badgetype}}\">{{badgevalue}}</span>\n        <button type=\"button\" aria-label=\"Collapse/Expand\" [attr.aria-expanded]=\"isActive\" [attr.aria-controls]=\"name\"\n                class=\"app-icon panel-action wi\"\n                title=\"{{appLocale.LABEL_COLLAPSE}}/{{appLocale.LABEL_EXPAND}}\">\n            <i [ngClass]=\"isActive ? 'wi-minus' : 'wi-plus'\"></i>\n        </button>\n    </div>\n</div>\n<div class=\"panel-collapse collapse\" role=\"tabpanel\" aria-labelledby=\"panel description\" [ngClass]=\"isActive ? 'collapse in' : 'collapse'\">\n    <div class=\"panel-body\" [wmSmoothscroll]=\"smoothscroll\" partial-container-target>\n        <ng-content></ng-content>\n    </div>\n</div>\n",
        providers: [
            provideAsWidgetRef(AccordionPaneComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, AccordionDirective])
], AccordionPaneComponent);

var AccordionDirective_1;
const DEFAULT_CLS$1 = 'app-accordion panel-group';
const WIDGET_CONFIG$1 = {
    widgetType: 'wm-accordion',
    hostClass: DEFAULT_CLS$1
};
let AccordionDirective = AccordionDirective_1 = class AccordionDirective extends StylableComponent {
    constructor(inj, statePersistence, dynamicComponentProvider) {
        let resolveFn = noop;
        super(inj, WIDGET_CONFIG$1, new Promise(res => resolveFn = res));
        this.promiseResolverFn = resolveFn;
        this.statePersistence = statePersistence;
        this.dynamicComponentProvider = dynamicComponentProvider;
        this.dynamicPanes = [];
        this.dynamicPaneIndex = 0;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
    }
    /**
     * AccordionPane children components invoke this method to communicate with the parent
     * if isExpand is true and when closeothers is true, all the other panes are collapsed
     * if the evt argument is defined on-change callback will be invoked.
     * updates the activePane index property
     * @param {AccordionPaneComponent} paneRef
     * @param {boolean} isExpand
     * @param {Event} evt
     */
    notifyChange(paneRef, isExpand, evt) {
        if (isExpand) {
            this.closePanesExcept(paneRef);
            const index = this.getPaneIndexByRef(paneRef);
            this.activePane = paneRef.getWidget();
            // if the event is defined invoke the change callback.
            // programmatic invocations of expand/collapse on accordion-pane will not trigger change event
            if (evt) {
                this.invokeEventCallback('change', {
                    $event: evt,
                    newPaneIndex: index,
                    oldPaneIndex: this.activePaneIndex
                });
            }
            this.activePaneIndex = index;
        }
        const mode = this.statePersistence.computeMode(this.statehandler);
        if (evt && mode && mode.toLowerCase() !== 'none') {
            const activePanes = [];
            this.panes.forEach(function (pane) {
                if (pane.isActive) {
                    activePanes.push(pane.name);
                }
            });
            if (activePanes.length) {
                this.statePersistence.setWidgetState(this, activePanes);
            }
            else {
                this.statePersistence.removeWidgetState(this);
            }
        }
    }
    /**
     * This method is used to register the dynamic panes.
     * After all panes are initialzed, update the querylist manually based on index.
     * @param paneRef - refrence of the tabpane
     */
    registerDynamicPane(paneRef) {
        this.dynamicPanes.push(paneRef);
        const isLastPane = this.dynamicPanes.length === this.dynamicPaneIndex;
        if (isLastPane) {
            for (let i = 0; i < this.dynamicPanes.length; i++) {
                const newPaneRef = _.find(this.dynamicPanes, pane => pane.dynamicPaneIndex === i);
                const isDuplicatePane = _.find(this.panes.toArray(), newPaneRef);
                if (!isDuplicatePane) {
                    this.panes.reset([...this.panes.toArray(), newPaneRef]);
                    if (newPaneRef.active) {
                        newPaneRef.expand();
                    }
                }
            }
        }
    }
    /**
     * This method is to add the tabpane dynamically
     * @param tabpanes - list of tabpanes
     */
    addPane(tabpanes) {
        if (!_.isArray(tabpanes)) {
            tabpanes = [tabpanes];
        }
        const paneNamesList = [];
        _.forEach(tabpanes, (pane, index) => {
            const isPaneAlreadyCreated = _.find(this.panes.toArray(), { name: pane.name });
            const isPaneNameExist = _.indexOf(paneNamesList, pane.name);
            // If user tries to add tabpane with the same name which is already exists then do not create the pane
            if (isPaneAlreadyCreated || isPaneNameExist > 0) {
                console.warn(`The tab pane with name ${pane.name} already exists`);
                return;
            }
            let paramMarkup = '';
            let propsTmpl = '';
            this.dynamicPaneIndex++;
            const name = pane.name ? pane.name : `accordionpane${this.panes.toArray().length + (index + 1)}`;
            paneNamesList.push(name);
            const partialParams = _.get(pane, 'params');
            _.forEach(pane, (value, key) => {
                if (key !== 'params') {
                    propsTmpl = `${propsTmpl} ${key}="${value}"`;
                }
            });
            _.forEach(partialParams, (value, key) => {
                paramMarkup = `${paramMarkup} <wm-param name="${key}" value="${value}"></wm-param>`;
            });
            const markup = `<wm-accordionpane dynamicPaneIndex="${this.dynamicPaneIndex - 1}" isdynamic="true" name="${name}" ${propsTmpl}>
                            ${paramMarkup}
                        </wm-accordionpane>`;
            if (!this._dynamicContext) {
                this._dynamicContext = Object.create(this.viewParent);
                this._dynamicContext[this.getAttr('wmAccordian')] = this;
            }
            this.dynamicComponentProvider.addComponent(this.getNativeElement(), markup, this._dynamicContext, { inj: this.inj });
        });
        return paneNamesList;
    }
    /**
     * This method is to remove the tabpane
     * @param paneName - name of the pane
     */
    removePane(paneName) {
        const paneRef = this.getPaneRefByName(paneName);
        if (paneRef) {
            paneRef.remove();
        }
    }
    isValidPaneIndex(index) {
        return (index >= 0 && index < this.panes.length);
    }
    getPaneRefByName(name) {
        return _.find(this.panes.toArray(), { name: name });
    }
    getPaneIndexByRef(paneRef) {
        return this.panes.toArray().indexOf(paneRef);
    }
    getPaneRefByIndex(index) {
        return this.panes.toArray()[index];
    }
    // Except the pane provided close all other panes
    closePanesExcept(paneRef) {
        if (isNumber(paneRef)) {
            paneRef = this.getPaneRefByIndex(paneRef);
        }
        if (this.closeothers) {
            this.panes.forEach(pane => {
                if (pane !== paneRef) {
                    pane.collapse();
                }
            });
        }
    }
    expandPane(index) {
        this.closePanesExcept(index);
        this.panes.toArray()[index].expand();
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'defaultpaneindex') {
            this.defaultpaneindex = nv;
        }
        else if (key === 'statehandler') {
            const widgetState = this.statePersistence.getWidgetState(this);
            let paneToSelect = [];
            if (nv !== 'none' && _.isArray(widgetState)) {
                widgetState.forEach(paneName => {
                    paneToSelect = this.panes.filter(function (pane) {
                        return paneName === pane.name;
                    });
                    if (!paneToSelect.length) {
                        console.warn('Accordion pane name ' + paneName + ' in State is incorrect.');
                    }
                    else {
                        this.expandPane(this.getPaneIndexByRef(paneToSelect[0]));
                    }
                });
            }
            else {
                if (this.isValidPaneIndex(this.defaultpaneindex)) {
                    this.expandPane(this.defaultpaneindex);
                }
            }
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngAfterContentInit() {
        super.ngAfterContentInit();
        this.promiseResolverFn();
    }
};
AccordionDirective.initializeProps = registerProps();
AccordionDirective.ctorParameters = () => [
    { type: Injector },
    { type: StatePersistence },
    { type: DynamicComponentRefProvider }
];
__decorate([
    ContentChildren(AccordionPaneComponent),
    __metadata("design:type", QueryList)
], AccordionDirective.prototype, "panes", void 0);
AccordionDirective = AccordionDirective_1 = __decorate([
    Directive({
        selector: 'div[wmAccordion]',
        providers: [
            provideAsWidgetRef(AccordionDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, StatePersistence, DynamicComponentRefProvider])
], AccordionDirective);

const components = [
    AccordionPaneComponent,
    AccordionDirective
];
let AccordionModule = class AccordionModule {
};
AccordionModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], AccordionModule);

/**
 * Generated bundle index. Do not edit.
 */

export { AccordionModule, AccordionPaneComponent as ɵa, AccordionDirective as ɵb, registerProps as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
