import { __assign, __extends, __spread, __decorate, __metadata } from 'tslib';
import { Injector, ContentChildren, Component, QueryList, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, styler, APPLY_STYLES_TYPE, RedrawableDirective, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { noop, removeAttr, isNumber, StatePersistence, DynamicComponentRefProvider } from '@wm/core';

var registerProps = function () {
    register('wm-accordion', new Map([
        ['class', PROP_STRING],
        ['closeothers', __assign({ value: true }, PROP_BOOLEAN)],
        ['defaultpaneindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['statehandler', __assign({ value: 'none' }, PROP_STRING)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
    ]));
};

var registerProps$1 = function () {
    register('wm-accordionpane', new Map([
        ['active', PROP_BOOLEAN],
        ['badgetype', __assign({ value: 'default' }, PROP_STRING)],
        ['badgevalue', PROP_STRING],
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['dynamicPaneIndex', PROP_NUMBER],
        ['iconclass', PROP_STRING],
        ['isdefaultpane', PROP_BOOLEAN],
        ['isdynamic', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['smoothscroll', __assign({ value: true }, PROP_BOOLEAN)],
        ['subheading', PROP_STRING],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', __assign({ value: 'Title' }, PROP_STRING)]
    ]));
};

var DEFAULT_CLS = 'app-accordion-panel panel';
var WIDGET_CONFIG = { widgetType: 'wm-accordionpane', hostClass: DEFAULT_CLS };
var AccordionPaneComponent = /** @class */ (function (_super) {
    __extends(AccordionPaneComponent, _super);
    function AccordionPaneComponent(inj, accordionRef) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.accordionRef = accordionRef;
        _this.isActive = false;
        _this.$lazyLoad = noop;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SHELL);
        // title property here serves the purpose of heading.
        // remove title property as attribute
        removeAttr(_this.nativeElement, 'title');
        return _this;
    }
    AccordionPaneComponent_1 = AccordionPaneComponent;
    /**
     * handles the pane expand
     * invoke $lazyLoad method which takes care of loading the partial when the content property is provided - lazyLoading or partial
     * invoke redraw on the re-drawable children
     * invoke expand callback
     * notify parent about the change
     * @param {Event} evt
     */
    AccordionPaneComponent.prototype.expand = function (evt) {
        if (this.isActive) {
            return;
        }
        this.isActive = true;
        this.$lazyLoad();
        this.redrawChildren();
        this.invokeEventCallback('expand', { $event: evt });
        this.notifyParent(true, evt);
    };
    /**
     * handles the pane collapse
     * invoke collapse callback
     * notify parent about the change
     * @param {Event} evt
     */
    AccordionPaneComponent.prototype.collapse = function (evt) {
        if (!this.isActive) {
            return;
        }
        this.isActive = false;
        this.invokeEventCallback('collapse', { $event: evt });
        this.notifyParent(false, evt);
    };
    AccordionPaneComponent.prototype.remove = function () {
        var paneIndex = this.accordionRef.getPaneIndexByRef(this);
        if (this.isActive && this.accordionRef.panes.length > 1) {
            this === this.accordionRef.panes.last ? this.accordionRef.panes.toArray()[paneIndex - 1].expand() : this.accordionRef.panes.toArray()[paneIndex + 1].expand();
        }
        var availablePanes = this.accordionRef.panes.toArray();
        availablePanes.splice(this.accordionRef.getPaneIndexByRef(this), 1);
        this.accordionRef.panes.reset(__spread(availablePanes));
        this.nativeElement.remove();
    };
    AccordionPaneComponent.prototype.toggle = function (evt) {
        if (this.isActive) {
            this.collapse(evt);
        }
        else {
            this.expand(evt);
        }
    };
    // Todo - Vinay externalize
    AccordionPaneComponent.prototype.redrawChildren = function () {
        var _this = this;
        setTimeout(function () {
            if (_this.reDrawableComponents) {
                _this.reDrawableComponents.forEach(function (c) { return c.redraw(); });
            }
        }, 100);
    };
    AccordionPaneComponent.prototype.notifyParent = function (isExpand, evt) {
        this.accordionRef.notifyChange(this, isExpand, evt);
    };
    AccordionPaneComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'content') {
            if (this.isActive) {
                setTimeout(function () { return _this.$lazyLoad(); }, 100);
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    AccordionPaneComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.nativeElement.querySelector('.panel-body'), this, APPLY_STYLES_TYPE.INNER_SHELL);
        if (this.isdynamic) {
            this.accordionRef.registerDynamicPane(this);
        }
    };
    var AccordionPaneComponent_1;
    AccordionPaneComponent.initializeProps = registerProps$1();
    AccordionPaneComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: AccordionDirective }
    ]; };
    __decorate([
        ContentChildren(RedrawableDirective, { descendants: true }),
        __metadata("design:type", Object)
    ], AccordionPaneComponent.prototype, "reDrawableComponents", void 0);
    AccordionPaneComponent = AccordionPaneComponent_1 = __decorate([
        Component({
            selector: 'div[wmAccordionPane]',
            template: "<div class=\"panel-heading clearfix\" role=\"tab\" (click)=\"toggle($event)\" [ngClass]=\"{active: isActive}\">\n    <h3 class=\"panel-title\">\n        <a href=\"javascript:void(0);\" role=\"button\" class=\"accordion-toggle\" [attr.aria-expanded]=\"isActive\" [attr.aria-controls]=\"name\">\n            <div class=\"pull-left\">\n                <i class=\"app-icon panel-icon {{iconclass}}\" [hidden]=\"!iconclass\"></i>\n            </div>\n            <div class=\"pull-left\">\n                <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\"></div>\n                <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n            </div>\n        </a>\n    </h3>\n    <div class=\"panel-actions\">\n        <span class=\"label label-{{badgetype}}\">{{badgevalue}}</span>\n        <button type=\"button\" aria-label=\"Collapse/Expand\" [attr.aria-expanded]=\"isActive\" [attr.aria-controls]=\"name\"\n                class=\"app-icon panel-action wi\"\n                title=\"{{appLocale.LABEL_COLLAPSE}}/{{appLocale.LABEL_EXPAND}}\">\n            <i [ngClass]=\"isActive ? 'wi-minus' : 'wi-plus'\"></i>\n        </button>\n    </div>\n</div>\n<div class=\"panel-collapse collapse\" role=\"tabpanel\" aria-labelledby=\"panel description\" [ngClass]=\"isActive ? 'collapse in' : 'collapse'\">\n    <div class=\"panel-body\" [wmSmoothscroll]=\"smoothscroll\" partial-container-target>\n        <ng-content></ng-content>\n    </div>\n</div>\n",
            providers: [
                provideAsWidgetRef(AccordionPaneComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, AccordionDirective])
    ], AccordionPaneComponent);
    return AccordionPaneComponent;
}(StylableComponent));

var DEFAULT_CLS$1 = 'app-accordion panel-group';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-accordion',
    hostClass: DEFAULT_CLS$1
};
var AccordionDirective = /** @class */ (function (_super) {
    __extends(AccordionDirective, _super);
    function AccordionDirective(inj, statePersistence, dynamicComponentProvider) {
        var _this = this;
        var resolveFn = noop;
        _this = _super.call(this, inj, WIDGET_CONFIG$1, new Promise(function (res) { return resolveFn = res; })) || this;
        _this.promiseResolverFn = resolveFn;
        _this.statePersistence = statePersistence;
        _this.dynamicComponentProvider = dynamicComponentProvider;
        _this.dynamicPanes = [];
        _this.dynamicPaneIndex = 0;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        return _this;
    }
    AccordionDirective_1 = AccordionDirective;
    /**
     * AccordionPane children components invoke this method to communicate with the parent
     * if isExpand is true and when closeothers is true, all the other panes are collapsed
     * if the evt argument is defined on-change callback will be invoked.
     * updates the activePane index property
     * @param {AccordionPaneComponent} paneRef
     * @param {boolean} isExpand
     * @param {Event} evt
     */
    AccordionDirective.prototype.notifyChange = function (paneRef, isExpand, evt) {
        if (isExpand) {
            this.closePanesExcept(paneRef);
            var index = this.getPaneIndexByRef(paneRef);
            this.activePane = paneRef.getWidget();
            // if the event is defined invoke the change callback.
            // programmatic invocations of expand/collapse on accordion-pane will not trigger change event
            if (evt) {
                this.invokeEventCallback('change', {
                    $event: evt,
                    newPaneIndex: index,
                    oldPaneIndex: this.activePaneIndex
                });
            }
            this.activePaneIndex = index;
        }
        var mode = this.statePersistence.computeMode(this.statehandler);
        if (evt && mode && mode.toLowerCase() !== 'none') {
            var activePanes_1 = [];
            this.panes.forEach(function (pane) {
                if (pane.isActive) {
                    activePanes_1.push(pane.name);
                }
            });
            if (activePanes_1.length) {
                this.statePersistence.setWidgetState(this, activePanes_1);
            }
            else {
                this.statePersistence.removeWidgetState(this);
            }
        }
    };
    /**
     * This method is used to register the dynamic panes.
     * After all panes are initialzed, update the querylist manually based on index.
     * @param paneRef - refrence of the tabpane
     */
    AccordionDirective.prototype.registerDynamicPane = function (paneRef) {
        this.dynamicPanes.push(paneRef);
        var isLastPane = this.dynamicPanes.length === this.dynamicPaneIndex;
        if (isLastPane) {
            var _loop_1 = function (i) {
                var newPaneRef = _.find(this_1.dynamicPanes, function (pane) { return pane.dynamicPaneIndex === i; });
                var isDuplicatePane = _.find(this_1.panes.toArray(), newPaneRef);
                if (!isDuplicatePane) {
                    this_1.panes.reset(__spread(this_1.panes.toArray(), [newPaneRef]));
                    if (newPaneRef.active) {
                        newPaneRef.expand();
                    }
                }
            };
            var this_1 = this;
            for (var i = 0; i < this.dynamicPanes.length; i++) {
                _loop_1(i);
            }
        }
    };
    /**
     * This method is to add the tabpane dynamically
     * @param tabpanes - list of tabpanes
     */
    AccordionDirective.prototype.addPane = function (tabpanes) {
        var _this = this;
        if (!_.isArray(tabpanes)) {
            tabpanes = [tabpanes];
        }
        var paneNamesList = [];
        _.forEach(tabpanes, function (pane, index) {
            var isPaneAlreadyCreated = _.find(_this.panes.toArray(), { name: pane.name });
            var isPaneNameExist = _.indexOf(paneNamesList, pane.name);
            // If user tries to add tabpane with the same name which is already exists then do not create the pane
            if (isPaneAlreadyCreated || isPaneNameExist > 0) {
                console.warn("The tab pane with name " + pane.name + " already exists");
                return;
            }
            var paramMarkup = '';
            var propsTmpl = '';
            _this.dynamicPaneIndex++;
            var name = pane.name ? pane.name : "accordionpane" + (_this.panes.toArray().length + (index + 1));
            paneNamesList.push(name);
            var partialParams = _.get(pane, 'params');
            _.forEach(pane, function (value, key) {
                if (key !== 'params') {
                    propsTmpl = propsTmpl + " " + key + "=\"" + value + "\"";
                }
            });
            _.forEach(partialParams, function (value, key) {
                paramMarkup = paramMarkup + " <wm-param name=\"" + key + "\" value=\"" + value + "\"></wm-param>";
            });
            var markup = "<wm-accordionpane dynamicPaneIndex=\"" + (_this.dynamicPaneIndex - 1) + "\" isdynamic=\"true\" name=\"" + name + "\" " + propsTmpl + ">\n                            " + paramMarkup + "\n                        </wm-accordionpane>";
            if (!_this._dynamicContext) {
                _this._dynamicContext = Object.create(_this.viewParent);
                _this._dynamicContext[_this.getAttr('wmAccordian')] = _this;
            }
            _this.dynamicComponentProvider.addComponent(_this.getNativeElement(), markup, _this._dynamicContext, { inj: _this.inj });
        });
        return paneNamesList;
    };
    /**
     * This method is to remove the tabpane
     * @param paneName - name of the pane
     */
    AccordionDirective.prototype.removePane = function (paneName) {
        var paneRef = this.getPaneRefByName(paneName);
        if (paneRef) {
            paneRef.remove();
        }
    };
    AccordionDirective.prototype.isValidPaneIndex = function (index) {
        return (index >= 0 && index < this.panes.length);
    };
    AccordionDirective.prototype.getPaneRefByName = function (name) {
        return _.find(this.panes.toArray(), { name: name });
    };
    AccordionDirective.prototype.getPaneIndexByRef = function (paneRef) {
        return this.panes.toArray().indexOf(paneRef);
    };
    AccordionDirective.prototype.getPaneRefByIndex = function (index) {
        return this.panes.toArray()[index];
    };
    // Except the pane provided close all other panes
    AccordionDirective.prototype.closePanesExcept = function (paneRef) {
        if (isNumber(paneRef)) {
            paneRef = this.getPaneRefByIndex(paneRef);
        }
        if (this.closeothers) {
            this.panes.forEach(function (pane) {
                if (pane !== paneRef) {
                    pane.collapse();
                }
            });
        }
    };
    AccordionDirective.prototype.expandPane = function (index) {
        this.closePanesExcept(index);
        this.panes.toArray()[index].expand();
    };
    AccordionDirective.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'defaultpaneindex') {
            this.defaultpaneindex = nv;
        }
        else if (key === 'statehandler') {
            var widgetState = this.statePersistence.getWidgetState(this);
            var paneToSelect_1 = [];
            if (nv !== 'none' && _.isArray(widgetState)) {
                widgetState.forEach(function (paneName) {
                    paneToSelect_1 = _this.panes.filter(function (pane) {
                        return paneName === pane.name;
                    });
                    if (!paneToSelect_1.length) {
                        console.warn('Accordion pane name ' + paneName + ' in State is incorrect.');
                    }
                    else {
                        _this.expandPane(_this.getPaneIndexByRef(paneToSelect_1[0]));
                    }
                });
            }
            else {
                if (this.isValidPaneIndex(this.defaultpaneindex)) {
                    this.expandPane(this.defaultpaneindex);
                }
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    AccordionDirective.prototype.ngAfterContentInit = function () {
        _super.prototype.ngAfterContentInit.call(this);
        this.promiseResolverFn();
    };
    var AccordionDirective_1;
    AccordionDirective.initializeProps = registerProps();
    AccordionDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: StatePersistence },
        { type: DynamicComponentRefProvider }
    ]; };
    __decorate([
        ContentChildren(AccordionPaneComponent),
        __metadata("design:type", QueryList)
    ], AccordionDirective.prototype, "panes", void 0);
    AccordionDirective = AccordionDirective_1 = __decorate([
        Directive({
            selector: 'div[wmAccordion]',
            providers: [
                provideAsWidgetRef(AccordionDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, StatePersistence, DynamicComponentRefProvider])
    ], AccordionDirective);
    return AccordionDirective;
}(StylableComponent));

var components = [
    AccordionPaneComponent,
    AccordionDirective
];
var AccordionModule = /** @class */ (function () {
    function AccordionModule() {
    }
    AccordionModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], AccordionModule);
    return AccordionModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { AccordionModule, AccordionPaneComponent as ɵa, AccordionDirective as ɵb, registerProps as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
