import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Directive, Attribute, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, BaseContainerComponent, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, StylableComponent, PROP_NUMBER, WmComponentsModule } from '@wm/components/base';
import { setCSS, isMobileApp, switchClass } from '@wm/core';

const registerProps = () => {
    register('wm-layoutgrid', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var LayoutgridDirective_1;
const DEFAULT_CLS = 'app-grid-layout clearfix';
const WIDGET_CONFIG = {
    widgetType: 'wm-layoutgrid',
    hostClass: DEFAULT_CLS
};
let LayoutgridDirective = LayoutgridDirective_1 = class LayoutgridDirective extends BaseContainerComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG);
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
};
LayoutgridDirective.initializeProps = registerProps();
LayoutgridDirective.ctorParameters = () => [
    { type: Injector }
];
LayoutgridDirective = LayoutgridDirective_1 = __decorate([
    Directive({
        selector: '[wmLayoutGrid]',
        providers: [
            provideAsWidgetRef(LayoutgridDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], LayoutgridDirective);

const registerProps$1 = () => {
    register('wm-layout-grid-row', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var LayoutGridRowDirective_1;
const DEFAULT_CLS$1 = 'app-grid-row clearfix';
const WIDGET_CONFIG$1 = {
    widgetType: 'wm-layout-grid-row',
    hostClass: DEFAULT_CLS$1
};
let LayoutGridRowDirective = LayoutGridRowDirective_1 = class LayoutGridRowDirective extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$1);
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
};
LayoutGridRowDirective.initializeProps = registerProps$1();
LayoutGridRowDirective.ctorParameters = () => [
    { type: Injector }
];
LayoutGridRowDirective = LayoutGridRowDirective_1 = __decorate([
    Directive({
        selector: '[wmLayoutGridRow]',
        providers: [
            provideAsWidgetRef(LayoutGridRowDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], LayoutGridRowDirective);

const registerProps$2 = () => {
    register('wm-gridcolumn', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['columnwidth', PROP_NUMBER],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var LayoutGridColumnDirective_1;
const DEFAULT_CLS$2 = 'app-grid-column';
const WIDGET_CONFIG$2 = {
    widgetType: 'wm-gridcolumn',
    hostClass: DEFAULT_CLS$2
};
let LayoutGridColumnDirective = LayoutGridColumnDirective_1 = class LayoutGridColumnDirective extends StylableComponent {
    constructor(inj, height) {
        super(inj, WIDGET_CONFIG$2);
        // if the height is provided set the overflow to auto
        if (height) {
            setCSS(this.nativeElement, 'overflow', 'auto');
        }
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
    onPropertyChange(key, nv, ov) {
        const prefix = isMobileApp() ? 'xs' : 'sm';
        if (key === 'columnwidth') {
            switchClass(this.nativeElement, `col-${prefix}-${nv}`, ov ? `col-${prefix}-${ov}` : '');
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
};
LayoutGridColumnDirective.initializeProps = registerProps$2();
LayoutGridColumnDirective.ctorParameters = () => [
    { type: Injector },
    { type: undefined, decorators: [{ type: Attribute, args: ['height',] }] }
];
LayoutGridColumnDirective = LayoutGridColumnDirective_1 = __decorate([
    Directive({
        selector: '[wmLayoutGridColumn]',
        providers: [
            provideAsWidgetRef(LayoutGridColumnDirective_1)
        ]
    }),
    __param(1, Attribute('height')),
    __metadata("design:paramtypes", [Injector, Object])
], LayoutGridColumnDirective);

const components = [
    LayoutgridDirective,
    LayoutGridRowDirective,
    LayoutGridColumnDirective
];
let LayoutGridModule = class LayoutGridModule {
};
LayoutGridModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], LayoutGridModule);

/**
 * Generated bundle index. Do not edit.
 */

export { LayoutGridModule, LayoutgridDirective as ɵa, registerProps as ɵb, LayoutGridRowDirective as ɵc, registerProps$1 as ɵd, LayoutGridColumnDirective as ɵe, registerProps$2 as ɵf };
//# sourceMappingURL=index.js.map
