import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Directive, Attribute, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, BaseContainerComponent, StylableComponent, PROP_NUMBER, WmComponentsModule } from '@wm/components/base';
import { setCSS, isMobileApp, switchClass } from '@wm/core';

var registerProps = function () {
    register('wm-layoutgrid', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS = 'app-grid-layout clearfix';
var WIDGET_CONFIG = {
    widgetType: 'wm-layoutgrid',
    hostClass: DEFAULT_CLS
};
var LayoutgridDirective = /** @class */ (function (_super) {
    __extends(LayoutgridDirective, _super);
    function LayoutgridDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    LayoutgridDirective_1 = LayoutgridDirective;
    var LayoutgridDirective_1;
    LayoutgridDirective.initializeProps = registerProps();
    LayoutgridDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    LayoutgridDirective = LayoutgridDirective_1 = __decorate([
        Directive({
            selector: '[wmLayoutGrid]',
            providers: [
                provideAsWidgetRef(LayoutgridDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], LayoutgridDirective);
    return LayoutgridDirective;
}(BaseContainerComponent));

var registerProps$1 = function () {
    register('wm-layout-grid-row', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$1 = 'app-grid-row clearfix';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-layout-grid-row',
    hostClass: DEFAULT_CLS$1
};
var LayoutGridRowDirective = /** @class */ (function (_super) {
    __extends(LayoutGridRowDirective, _super);
    function LayoutGridRowDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    LayoutGridRowDirective_1 = LayoutGridRowDirective;
    var LayoutGridRowDirective_1;
    LayoutGridRowDirective.initializeProps = registerProps$1();
    LayoutGridRowDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    LayoutGridRowDirective = LayoutGridRowDirective_1 = __decorate([
        Directive({
            selector: '[wmLayoutGridRow]',
            providers: [
                provideAsWidgetRef(LayoutGridRowDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], LayoutGridRowDirective);
    return LayoutGridRowDirective;
}(StylableComponent));

var registerProps$2 = function () {
    register('wm-gridcolumn', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['columnwidth', PROP_NUMBER],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$2 = 'app-grid-column';
var WIDGET_CONFIG$2 = {
    widgetType: 'wm-gridcolumn',
    hostClass: DEFAULT_CLS$2
};
var LayoutGridColumnDirective = /** @class */ (function (_super) {
    __extends(LayoutGridColumnDirective, _super);
    function LayoutGridColumnDirective(inj, height) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$2) || this;
        // if the height is provided set the overflow to auto
        if (height) {
            setCSS(_this.nativeElement, 'overflow', 'auto');
        }
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    LayoutGridColumnDirective_1 = LayoutGridColumnDirective;
    LayoutGridColumnDirective.prototype.onPropertyChange = function (key, nv, ov) {
        var prefix = isMobileApp() ? 'xs' : 'sm';
        if (key === 'columnwidth') {
            switchClass(this.nativeElement, "col-" + prefix + "-" + nv, ov ? "col-" + prefix + "-" + ov : '');
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var LayoutGridColumnDirective_1;
    LayoutGridColumnDirective.initializeProps = registerProps$2();
    LayoutGridColumnDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: undefined, decorators: [{ type: Attribute, args: ['height',] }] }
    ]; };
    LayoutGridColumnDirective = LayoutGridColumnDirective_1 = __decorate([
        Directive({
            selector: '[wmLayoutGridColumn]',
            providers: [
                provideAsWidgetRef(LayoutGridColumnDirective_1)
            ]
        }),
        __param(1, Attribute('height')),
        __metadata("design:paramtypes", [Injector, Object])
    ], LayoutGridColumnDirective);
    return LayoutGridColumnDirective;
}(StylableComponent));

var components = [
    LayoutgridDirective,
    LayoutGridRowDirective,
    LayoutGridColumnDirective
];
var LayoutGridModule = /** @class */ (function () {
    function LayoutGridModule() {
    }
    LayoutGridModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], LayoutGridModule);
    return LayoutGridModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { LayoutGridModule, LayoutgridDirective as ɵa, registerProps as ɵb, LayoutGridRowDirective as ɵc, registerProps$1 as ɵd, LayoutGridColumnDirective as ɵe, registerProps$2 as ɵf };
//# sourceMappingURL=index.js.map
