import { __decorate, __metadata } from 'tslib';
import { Injector, ViewChild, ElementRef, ContentChildren, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { BsDropdownModule } from 'ngx-bootstrap/dropdown';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_ANY, styler, APPLY_STYLES_TYPE, RedrawableDirective, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { MenuAdapterComponent, MenuModule } from '@wm/components/navigation/menu';
import { noop, removeAttr, toggleClass, $appDigest, setCSS } from '@wm/core';

const registerProps = () => {
    register('wm-panel', new Map([
        ['actions', {}],
        ['animation', PROP_STRING],
        ['autoclose', PROP_STRING],
        ['badgetype', Object.assign({ value: 'default' }, PROP_STRING)],
        ['badgevalue', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', PROP_BOOLEAN],
        ['collapsible', PROP_BOOLEAN],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['content', PROP_STRING],
        ['datafield', PROP_STRING],
        ['enablefullscreen', PROP_BOOLEAN],
        ['expanded', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['fullscreen', PROP_BOOLEAN],
        ['helptext', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['itemaction', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['subheading', PROP_STRING],
        ['title', PROP_STRING],
        ['userrole', PROP_STRING]
    ]));
};

var PanelComponent_1;
const DEFAULT_CLS = 'app-panel panel';
const WIDGET_CONFIG = { widgetType: 'wm-panel', hostClass: DEFAULT_CLS };
let PanelComponent = PanelComponent_1 = class PanelComponent extends MenuAdapterComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG);
        this.$lazyLoad = noop;
        this.expanded = true;
        this.helpClass = '';
        this.helptext = '';
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SHELL);
        this.fullScreenTitle = `${this.appLocale.LABEL_FULLSCREEN}/${this.appLocale.LABEL_EXITFULLSCREEN}`;
        this.expandCollapseTitle = `${this.appLocale.LABEL_COLLAPSE}/${this.appLocale.LABEL_EXPAND}`;
        removeAttr(this.nativeElement, 'title');
    }
    // conditions to show the footer
    get hideFooter() {
        return !this.hasFooter || !this.expanded;
    }
    // conditions to show header
    get showHeader() {
        return this.iconurl || this.iconclass || this.collapsible || this.actions || this.title || this.subheading || this.enablefullscreen;
    }
    // toggle the panel state between collapsed - expanded. invoke the respective callbacks
    toggle($event) {
        if (this.collapsible) {
            this.invokeEventCallback(this.expanded ? 'collapse' : 'expand', { $event });
            this.expanded = !this.expanded;
            if (this.expanded) {
                this.$lazyLoad();
                this.reDrawChildren();
            }
        }
    }
    expand($event) {
        if (!this.expanded) {
            this.toggle($event);
        }
    }
    collapse($event) {
        if (this.expanded) {
            this.toggle($event);
        }
    }
    // toggle the fullscreen state of the panel. invoke the respective callbacks
    toggleFullScreen($event) {
        if (this.enablefullscreen) {
            this.invokeEventCallback(this.fullscreen ? 'exitfullscreen' : 'fullscreen', { $event });
            this.fullscreen = !this.fullscreen;
            toggleClass(this.nativeElement, 'fullscreen', this.fullscreen);
            // Re-plot the widgets inside panel
            this.reDrawChildren();
        }
        this.computeDimensions();
    }
    // show/hide help
    toggleHelp() {
        this.helpClass = this.helpClass ? '' : 'show-help';
        toggleClass(this.nativeElement, 'show-help', !!this.helpClass);
        $appDigest();
    }
    // On action item click
    menuActionItemClick($event, $item) {
        this.selectedAction = _.omit($item, ['children', 'value']);
        this.invokeEventCallback('actionsclick', { $event, $item: this.selectedAction });
    }
    // hide the panel and invoke the respective event handler
    close($event) {
        this.getWidget().show = false;
        this.invokeEventCallback('close', { $event });
    }
    // calculation of dimensions when the panel the panel state changes from/to fullscreen
    computeDimensions() {
        const headerHeight = this.panelHeader.nativeElement.offsetHeight;
        const footer = this.nativeElement.querySelector('.panel-footer');
        const content = this.panelContent.nativeElement;
        const vHeight = window.innerHeight;
        let inlineHeight;
        if (this.fullscreen) {
            inlineHeight = this.hideFooter ? (vHeight - headerHeight) : vHeight - (footer.offsetHeight + headerHeight);
        }
        else {
            inlineHeight = this.height || '';
        }
        setCSS(content, 'height', inlineHeight);
    }
    // notify the redrawable components(charts, grid)
    reDrawChildren() {
        setTimeout(() => {
            if (this.reDrawableComponents) {
                this.reDrawableComponents.forEach(c => c.redraw());
            }
        }, 100);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'expanded') {
            this.expanded = nv;
        }
        else if (key === 'content') {
            setTimeout(() => {
                if (this.expanded || !this.collapsible) {
                    this.$lazyLoad();
                }
            }, 20);
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngAfterContentInit() {
        super.ngAfterContentInit();
        this.hasFooter = !!this.nativeElement.querySelector('[wmPanelFooter]');
        styler(this.panelBody.nativeElement, this, APPLY_STYLES_TYPE.INNER_SHELL);
    }
};
PanelComponent.initializeProps = registerProps();
PanelComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ViewChild('panelHeading', { static: false }),
    __metadata("design:type", ElementRef)
], PanelComponent.prototype, "panelHeader", void 0);
__decorate([
    ViewChild('panelContent', { static: true }),
    __metadata("design:type", ElementRef)
], PanelComponent.prototype, "panelContent", void 0);
__decorate([
    ViewChild('panelBody', { static: true }),
    __metadata("design:type", ElementRef)
], PanelComponent.prototype, "panelBody", void 0);
__decorate([
    ContentChildren(RedrawableDirective, { descendants: true }),
    __metadata("design:type", Object)
], PanelComponent.prototype, "reDrawableComponents", void 0);
PanelComponent = PanelComponent_1 = __decorate([
    Component({
        selector: '[wmPanel]',
        template: "<div class=\"panel-heading {{helpClass}}\" *ngIf=\"showHeader\" #panelHeading>\n    <h3 class=\"panel-title\">\n        <a href=\"javascript:void(0)\" class=\"panel-toggle\" (click)=\"toggle($event)\">\n            <div class=\"pull-left\">\n                <i class=\"app-icon panel-icon {{iconclass}}\" *ngIf=\"iconclass && !iconurl\"></i>\n                <img data-identifier=\"img\" title=\"iconurl\" alt=\"Panel icon\" class=\"panel-image-icon\"\n                     [src]=\"iconurl | image\" *ngIf=\"iconurl\"\n                     [ngStyle]=\"{width: iconwidth ,height: iconheight, margin: iconmargin}\" />\n            </div>\n            <div class=\"pull-left\">\n                <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\" [title]=\"title\"></div>\n                <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n            </div>\n        </a>\n        <div class=\"panel-actions\">\n            <span *ngIf=\"badgevalue\" aria-label=\"badge\"\n                  class=\"app-badge label label-{{badgetype}}\">{{badgevalue}}</span>\n            <div wmMenu aria-label=\"dropdown\" dropdown type=\"anchor\" class=\"panel-action\" dataset.bind=\"actions\"\n                 *ngIf=\"actions\" caption=\"\" menuposition=\"down,left\" iconclass=\"wi wi-more-vert\"\n                 autoclose.bind=\"autoclose\" datafield.bind=\"datafield\" select.event=\"menuActionItemClick($event,$item)\">\n            </div>\n            <button type=\"button\" aria-label=\"Help\" class=\"app-icon panel-action wi\" [title]=\"appLocale.LABEL_HELP\"\n                    *ngIf=\"helptext\" (click)=\"toggleHelp()\"><i class=\"wi-question\"></i></button>\n            <button type=\"button\" aria-label=\"Collapse/Expand\" class=\"app-icon wi panel-action\" *ngIf=\"collapsible\"\n                    [title]=\"expandCollapseTitle\" (click)=\"toggle($event);\"><i\n                   [ngClass]=\"expanded ? 'wi-minus' : 'wi-plus'\"></i></button>\n            <button type=\"button\" aria-label=\"Fullscreen/Exit\" class=\"app-icon panel-action wi\" *ngIf=\"enablefullscreen\"\n                    [title]=\"fullScreenTitle\" (click)=\"toggleFullScreen($event);\"><i\n                   [ngClass]=\"fullscreen ? 'wi-fullscreen-exit' : 'wi-fullscreen'\"></i></button>\n            <button type=\"button\" aria-label=\"Close button\" class=\"app-icon panel-action\"\n                    title=\"{{appLocale.LABEL_CLOSE}}\" *ngIf=\"closable\" (click)=\"close($event)\"><i\n                   class=\"wi wi-close\"></i></button>\n        </div>\n    </h3>\n</div>\n<div class=\"panel-content\" [hidden]=\"!expanded\" #panelContent>\n    <!-- Todo Bandhavya - implement smooth scroll -->\n    <div partial-container-target #panelBody [ngClass]=\"['panel-body', helpClass]\">\n        <ng-content></ng-content>\n    </div>\n    <aside class=\"panel-help-message\" [ngClass]=\"helpClass\">\n        <h5 class=\"panel-help-header\" [textContent]=\"appLocale.LABEL_HELP\"></h5>\n        <div class=\"panel-help-content\" [innerHTML]=\"helptext | trustAs : 'html'\"></div>\n    </aside>\n</div>\n<div class=\"app-panel-footer panel-footer clearfix\" [hidden]=\"hideFooter\">\n    <ng-content select=\"[wmPanelFooter]\"></ng-content>\n</div>",
        providers: [
            provideAsWidgetRef(PanelComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], PanelComponent);

const components = [
    PanelComponent
];
let PanelModule = class PanelModule {
};
PanelModule = __decorate([
    NgModule({
        imports: [
            BsDropdownModule,
            CommonModule,
            MenuModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], PanelModule);

/**
 * Generated bundle index. Do not edit.
 */

export { PanelModule, PanelComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
