import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ViewChild, ElementRef, ContentChildren, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { BsDropdownModule } from 'ngx-bootstrap/dropdown';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_ANY, styler, APPLY_STYLES_TYPE, RedrawableDirective, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { MenuAdapterComponent, MenuModule } from '@wm/components/navigation/menu';
import { noop, removeAttr, toggleClass, $appDigest, setCSS } from '@wm/core';

var registerProps = function () {
    register('wm-panel', new Map([
        ['actions', {}],
        ['animation', PROP_STRING],
        ['autoclose', PROP_STRING],
        ['badgetype', __assign({ value: 'default' }, PROP_STRING)],
        ['badgevalue', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', PROP_BOOLEAN],
        ['collapsible', PROP_BOOLEAN],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['content', PROP_STRING],
        ['datafield', PROP_STRING],
        ['enablefullscreen', PROP_BOOLEAN],
        ['expanded', __assign({ value: true }, PROP_BOOLEAN)],
        ['fullscreen', PROP_BOOLEAN],
        ['helptext', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['itemaction', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['subheading', PROP_STRING],
        ['title', PROP_STRING],
        ['userrole', PROP_STRING]
    ]));
};

var DEFAULT_CLS = 'app-panel panel';
var WIDGET_CONFIG = { widgetType: 'wm-panel', hostClass: DEFAULT_CLS };
var PanelComponent = /** @class */ (function (_super) {
    __extends(PanelComponent, _super);
    function PanelComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.$lazyLoad = noop;
        _this.expanded = true;
        _this.helpClass = '';
        _this.helptext = '';
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SHELL);
        _this.fullScreenTitle = _this.appLocale.LABEL_FULLSCREEN + "/" + _this.appLocale.LABEL_EXITFULLSCREEN;
        _this.expandCollapseTitle = _this.appLocale.LABEL_COLLAPSE + "/" + _this.appLocale.LABEL_EXPAND;
        removeAttr(_this.nativeElement, 'title');
        return _this;
    }
    PanelComponent_1 = PanelComponent;
    Object.defineProperty(PanelComponent.prototype, "hideFooter", {
        // conditions to show the footer
        get: function () {
            return !this.hasFooter || !this.expanded;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PanelComponent.prototype, "showHeader", {
        // conditions to show header
        get: function () {
            return this.iconurl || this.iconclass || this.collapsible || this.actions || this.title || this.subheading || this.enablefullscreen;
        },
        enumerable: true,
        configurable: true
    });
    // toggle the panel state between collapsed - expanded. invoke the respective callbacks
    PanelComponent.prototype.toggle = function ($event) {
        if (this.collapsible) {
            this.invokeEventCallback(this.expanded ? 'collapse' : 'expand', { $event: $event });
            this.expanded = !this.expanded;
            if (this.expanded) {
                this.$lazyLoad();
                this.reDrawChildren();
            }
        }
    };
    PanelComponent.prototype.expand = function ($event) {
        if (!this.expanded) {
            this.toggle($event);
        }
    };
    PanelComponent.prototype.collapse = function ($event) {
        if (this.expanded) {
            this.toggle($event);
        }
    };
    // toggle the fullscreen state of the panel. invoke the respective callbacks
    PanelComponent.prototype.toggleFullScreen = function ($event) {
        if (this.enablefullscreen) {
            this.invokeEventCallback(this.fullscreen ? 'exitfullscreen' : 'fullscreen', { $event: $event });
            this.fullscreen = !this.fullscreen;
            toggleClass(this.nativeElement, 'fullscreen', this.fullscreen);
            // Re-plot the widgets inside panel
            this.reDrawChildren();
        }
        this.computeDimensions();
    };
    // show/hide help
    PanelComponent.prototype.toggleHelp = function () {
        this.helpClass = this.helpClass ? '' : 'show-help';
        toggleClass(this.nativeElement, 'show-help', !!this.helpClass);
        $appDigest();
    };
    // On action item click
    PanelComponent.prototype.menuActionItemClick = function ($event, $item) {
        this.selectedAction = _.omit($item, ['children', 'value']);
        this.invokeEventCallback('actionsclick', { $event: $event, $item: this.selectedAction });
    };
    // hide the panel and invoke the respective event handler
    PanelComponent.prototype.close = function ($event) {
        this.getWidget().show = false;
        this.invokeEventCallback('close', { $event: $event });
    };
    // calculation of dimensions when the panel the panel state changes from/to fullscreen
    PanelComponent.prototype.computeDimensions = function () {
        var headerHeight = this.panelHeader.nativeElement.offsetHeight;
        var footer = this.nativeElement.querySelector('.panel-footer');
        var content = this.panelContent.nativeElement;
        var vHeight = window.innerHeight;
        var inlineHeight;
        if (this.fullscreen) {
            inlineHeight = this.hideFooter ? (vHeight - headerHeight) : vHeight - (footer.offsetHeight + headerHeight);
        }
        else {
            inlineHeight = this.height || '';
        }
        setCSS(content, 'height', inlineHeight);
    };
    // notify the redrawable components(charts, grid)
    PanelComponent.prototype.reDrawChildren = function () {
        var _this = this;
        setTimeout(function () {
            if (_this.reDrawableComponents) {
                _this.reDrawableComponents.forEach(function (c) { return c.redraw(); });
            }
        }, 100);
    };
    PanelComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'expanded') {
            this.expanded = nv;
        }
        else if (key === 'content') {
            setTimeout(function () {
                if (_this.expanded || !_this.collapsible) {
                    _this.$lazyLoad();
                }
            }, 20);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    PanelComponent.prototype.ngAfterContentInit = function () {
        _super.prototype.ngAfterContentInit.call(this);
        this.hasFooter = !!this.nativeElement.querySelector('[wmPanelFooter]');
        styler(this.panelBody.nativeElement, this, APPLY_STYLES_TYPE.INNER_SHELL);
    };
    var PanelComponent_1;
    PanelComponent.initializeProps = registerProps();
    PanelComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('panelHeading', { static: false }),
        __metadata("design:type", ElementRef)
    ], PanelComponent.prototype, "panelHeader", void 0);
    __decorate([
        ViewChild('panelContent', { static: true }),
        __metadata("design:type", ElementRef)
    ], PanelComponent.prototype, "panelContent", void 0);
    __decorate([
        ViewChild('panelBody', { static: true }),
        __metadata("design:type", ElementRef)
    ], PanelComponent.prototype, "panelBody", void 0);
    __decorate([
        ContentChildren(RedrawableDirective, { descendants: true }),
        __metadata("design:type", Object)
    ], PanelComponent.prototype, "reDrawableComponents", void 0);
    PanelComponent = PanelComponent_1 = __decorate([
        Component({
            selector: '[wmPanel]',
            template: "<div class=\"panel-heading {{helpClass}}\" *ngIf=\"showHeader\" #panelHeading>\n    <h3 class=\"panel-title\">\n        <a href=\"javascript:void(0)\" class=\"panel-toggle\" (click)=\"toggle($event)\">\n            <div class=\"pull-left\">\n                <i class=\"app-icon panel-icon {{iconclass}}\" *ngIf=\"iconclass && !iconurl\"></i>\n                <img data-identifier=\"img\" title=\"iconurl\" alt=\"Panel icon\" class=\"panel-image-icon\"\n                     [src]=\"iconurl | image\" *ngIf=\"iconurl\"\n                     [ngStyle]=\"{width: iconwidth ,height: iconheight, margin: iconmargin}\" />\n            </div>\n            <div class=\"pull-left\">\n                <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\" [title]=\"title\"></div>\n                <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n            </div>\n        </a>\n        <div class=\"panel-actions\">\n            <span *ngIf=\"badgevalue\" aria-label=\"badge\"\n                  class=\"app-badge label label-{{badgetype}}\">{{badgevalue}}</span>\n            <div wmMenu aria-label=\"dropdown\" dropdown type=\"anchor\" class=\"panel-action\" dataset.bind=\"actions\"\n                 *ngIf=\"actions\" caption=\"\" menuposition=\"down,left\" iconclass=\"wi wi-more-vert\"\n                 autoclose.bind=\"autoclose\" datafield.bind=\"datafield\" select.event=\"menuActionItemClick($event,$item)\">\n            </div>\n            <button type=\"button\" aria-label=\"Help\" class=\"app-icon panel-action wi\" [title]=\"appLocale.LABEL_HELP\"\n                    *ngIf=\"helptext\" (click)=\"toggleHelp()\"><i class=\"wi-question\"></i></button>\n            <button type=\"button\" aria-label=\"Collapse/Expand\" class=\"app-icon wi panel-action\" *ngIf=\"collapsible\"\n                    [title]=\"expandCollapseTitle\" (click)=\"toggle($event);\"><i\n                   [ngClass]=\"expanded ? 'wi-minus' : 'wi-plus'\"></i></button>\n            <button type=\"button\" aria-label=\"Fullscreen/Exit\" class=\"app-icon panel-action wi\" *ngIf=\"enablefullscreen\"\n                    [title]=\"fullScreenTitle\" (click)=\"toggleFullScreen($event);\"><i\n                   [ngClass]=\"fullscreen ? 'wi-fullscreen-exit' : 'wi-fullscreen'\"></i></button>\n            <button type=\"button\" aria-label=\"Close button\" class=\"app-icon panel-action\"\n                    title=\"{{appLocale.LABEL_CLOSE}}\" *ngIf=\"closable\" (click)=\"close($event)\"><i\n                   class=\"wi wi-close\"></i></button>\n        </div>\n    </h3>\n</div>\n<div class=\"panel-content\" [hidden]=\"!expanded\" #panelContent>\n    <!-- Todo Bandhavya - implement smooth scroll -->\n    <div partial-container-target #panelBody [ngClass]=\"['panel-body', helpClass]\">\n        <ng-content></ng-content>\n    </div>\n    <aside class=\"panel-help-message\" [ngClass]=\"helpClass\">\n        <h5 class=\"panel-help-header\" [textContent]=\"appLocale.LABEL_HELP\"></h5>\n        <div class=\"panel-help-content\" [innerHTML]=\"helptext | trustAs : 'html'\"></div>\n    </aside>\n</div>\n<div class=\"app-panel-footer panel-footer clearfix\" [hidden]=\"hideFooter\">\n    <ng-content select=\"[wmPanelFooter]\"></ng-content>\n</div>",
            providers: [
                provideAsWidgetRef(PanelComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], PanelComponent);
    return PanelComponent;
}(MenuAdapterComponent));

var components = [
    PanelComponent
];
var PanelModule = /** @class */ (function () {
    function PanelModule() {
    }
    PanelModule = __decorate([
        NgModule({
            imports: [
                BsDropdownModule,
                CommonModule,
                MenuModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], PanelModule);
    return PanelModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { PanelModule, PanelComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
