import { __extends, __values, __assign, __spread, __decorate, __metadata, __param } from 'tslib';
import { Injector, Attribute, HostBinding, ContentChildren, Component, QueryList, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_STRING, PROP_NUMBER, PROP_BOOLEAN, register, styler, APPLY_STYLES_TYPE, RedrawableDirective, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { addClass, setCSSFromObj, setCSS, $appDigest, isMobileApp, noop, removeAttr, removeClass, appendNode, DynamicComponentRefProvider, StatePersistence } from '@wm/core';
import { SwipeAnimation } from '@swipey';

var TabsAnimator = /** @class */ (function (_super) {
    __extends(TabsAnimator, _super);
    function TabsAnimator(tabs) {
        var e_1, _a;
        var _this = _super.call(this) || this;
        _this.tabs = tabs;
        _this._$el = $(_this.tabs.getNativeElement()).find('>.tab-content');
        var childEls = _this._$el.find('>[wmTabPane]');
        _this._noOfTabs = childEls.length;
        var maxWidth = _this._noOfTabs * 100 + "%";
        addClass(_this.tabs.getNativeElement(), 'has-transition');
        setCSSFromObj(_this._$el[0], { maxWidth: maxWidth, width: maxWidth });
        var width = 100 / _this._noOfTabs + "%";
        try {
            for (var _b = __values(Array.from(childEls)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var child = _c.value;
                setCSS(child, 'width', width);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        _this.init(_this._$el);
        return _this;
    }
    TabsAnimator.prototype.bounds = function () {
        var activeTabIndex = this.tabs.getActiveTabIndex(), w = this._$el.find('>.tab-pane').first().width(), noOfTabs = this._$el.find('>.tab-pane:visible').length, centerVal = -1 * activeTabIndex * w;
        this.clearContentFocus();
        return {
            strict: false,
            lower: activeTabIndex === noOfTabs - 1 ? 0 : -w,
            center: centerVal,
            upper: activeTabIndex === 0 ? centerVal : w
        };
    };
    TabsAnimator.prototype.context = function () {
        return {
            'w': this._$el.width()
        };
    };
    TabsAnimator.prototype.animation = function () {
        return {
            'transform': 'translate3d(${{ ($D + $d)/w * 100 + \'%\'}}, 0, 0)',
            '-webkit-transform': 'translate3d(${{ ($D + $d)/w * 100 + \'%\'}}, 0, 0)'
        };
    };
    /* WMS-18031 | On Tab Switch | Remove the Focus on the Existing Tab Contents */
    TabsAnimator.prototype.clearContentFocus = function () {
        var activeTab = this._$el.find('>.tab-pane.active');
        if (activeTab.length && activeTab.find(document.activeElement).length) {
            document.activeElement.blur();
        }
    };
    TabsAnimator.prototype.transitionTabIntoView = function () {
        var activeTabIndex = this.tabs.getActiveTabIndex();
        setCSS(this._$el[0], 'transform', "translate3d(" + -1 * activeTabIndex / this._noOfTabs * 100 + "%, 0, 0)");
    };
    TabsAnimator.prototype.onUpper = function () {
        this.tabs.prev();
        $appDigest();
    };
    TabsAnimator.prototype.onLower = function () {
        this.tabs.next();
        $appDigest();
    };
    TabsAnimator.prototype.threshold = function () {
        return 5;
    };
    return TabsAnimator;
}(SwipeAnimation));

var registerProps = function () {
    var props = new Map([
        ['class', PROP_STRING],
        ['defaultpaneindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['justified', PROP_BOOLEAN],
        ['tabsposition', __assign({ value: 'top' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['statehandler', __assign({ value: 'none' }, PROP_STRING)],
        ['transition', PROP_STRING]
    ]);
    if (isMobileApp()) {
        props.set('transition', __assign({ value: 'slide' }, PROP_STRING));
    }
    register('wm-tabs', props);
};

var registerProps$1 = function () {
    register('wm-tabpane', new Map([
        ['active', PROP_BOOLEAN],
        ['badgevalue', PROP_STRING],
        ['badgetype', __assign({ value: 'default' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['disabled', PROP_BOOLEAN],
        ['dynamicPaneIndex', PROP_NUMBER],
        ['heading', PROP_STRING],
        ['isdefaulttab', PROP_STRING],
        ['isdynamic', PROP_BOOLEAN],
        ['name', PROP_STRING],
        ['paneicon', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['smoothscroll', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', __assign({ value: 'Tab Title' }, PROP_STRING)]
    ]));
};

var DEFAULT_CLS = 'tab-pane';
var WIDGET_CONFIG = {
    widgetType: 'wm-tabpane',
    hostClass: DEFAULT_CLS
};
var TabPaneComponent = /** @class */ (function (_super) {
    __extends(TabPaneComponent, _super);
    function TabPaneComponent(inj, tabsRef, heading, title) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.tabsRef = tabsRef;
        _this.heading = heading;
        _this.title = title;
        _this._isFirstLoad = true;
        _this.$lazyLoad = noop;
        _this.isActive = false;
        _this.disabled = false;
        // title property here serves the purpose of heading.
        // TODO: make it common for all the widget.
        removeAttr(_this.nativeElement, 'title');
        return _this;
    }
    TabPaneComponent_1 = TabPaneComponent;
    // parent tabs component will call this method for the order of callbacks to be proper
    // order of callbacks - deselect, select, change
    TabPaneComponent.prototype.invokeOnSelectCallback = function ($event) {
        this.invokeEventCallback('select', { $event: $event });
    };
    TabPaneComponent.prototype.select = function ($event) {
        // When called programatically $event won't be available
        if (this.isActive || this.disabled) {
            return;
        }
        this.isActive = true;
        this.$lazyLoad();
        this.redrawChildren();
        this.notifyParent($event);
    };
    TabPaneComponent.prototype.deselect = function () {
        if (this.isActive) {
            this.isActive = false;
            this.invokeEventCallback('deselect');
        }
    };
    TabPaneComponent.prototype.remove = function () {
        if (this.isActive) {
            this === this.tabsRef.panes.last ? this.tabsRef.prev() : this.tabsRef.next();
        }
        var availablePanes = this.tabsRef.panes.toArray();
        availablePanes.splice(this.tabsRef.getPaneIndexByRef(this), 1);
        this.tabsRef.panes.reset(__spread(availablePanes));
        this.nativeElement.remove();
        this.tabsRef.$element.find(".tab-header[data-paneid='" + this.$element.attr('widget-id') + "'] a").remove();
    };
    TabPaneComponent.prototype.redrawChildren = function () {
        var _this = this;
        setTimeout(function () {
            if (_this.reDrawableComponents) {
                _this.reDrawableComponents.forEach(function (c) { return c.redraw(); });
            }
        }, 100);
    };
    TabPaneComponent.prototype.notifyParent = function (evt) {
        this.tabsRef.notifyChange(this, evt);
    };
    // select next valid tab
    TabPaneComponent.prototype.handleSwipeLeft = function () {
        this.tabsRef.next();
    };
    // select prev valid tab
    TabPaneComponent.prototype.handleSwipeRight = function () {
        this.tabsRef.prev();
    };
    // select event is called manually
    TabPaneComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName !== 'select') {
            _super.prototype.handleEvent.call(this, this.nativeElement, eventName, callback, locals);
        }
    };
    TabPaneComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'content') {
            if (this.isActive) {
                setTimeout(function () { return _this.$lazyLoad(); }, 100);
            }
        }
        else if (key === 'show') {
            // overridding the base component show propertyChangeHandler
            this.nativeElement.hidden = !nv;
            // setting default tab pane on pageload whenever show property is bindable
            if (this._isFirstLoad) {
                this.tabsRef.selectDefaultPaneByIndex(this.tabsRef.defaultpaneindex);
            }
            this._isFirstLoad = false;
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    TabPaneComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.title = this.title || this.heading;
    };
    TabPaneComponent.prototype.ngAfterViewInit = function () {
        styler(this.nativeElement.querySelector('.tab-body'), this, APPLY_STYLES_TYPE.CONTAINER);
        _super.prototype.ngAfterViewInit.call(this);
        if (this.isdynamic) {
            this.tabsRef.registerDynamicTab(this);
        }
    };
    var TabPaneComponent_1;
    TabPaneComponent.initializeProps = registerProps$1();
    TabPaneComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: TabsComponent },
        { type: undefined, decorators: [{ type: Attribute, args: ['heading',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['title',] }] }
    ]; };
    __decorate([
        HostBinding('class.active'),
        __metadata("design:type", Object)
    ], TabPaneComponent.prototype, "isActive", void 0);
    __decorate([
        HostBinding('class.disabled'),
        __metadata("design:type", Object)
    ], TabPaneComponent.prototype, "disabled", void 0);
    __decorate([
        ContentChildren(RedrawableDirective, { descendants: true }),
        __metadata("design:type", Object)
    ], TabPaneComponent.prototype, "reDrawableComponents", void 0);
    TabPaneComponent = TabPaneComponent_1 = __decorate([
        Component({
            selector: 'div[wmTabPane]',
            template: "<div class=\"tab-body\" partial-container-target [wmSmoothscroll]=\"smoothscroll\">\n    <ng-content></ng-content>\n</div>",
            providers: [
                provideAsWidgetRef(TabPaneComponent_1)
            ]
        }),
        __param(2, Attribute('heading')),
        __param(3, Attribute('title')),
        __metadata("design:paramtypes", [Injector,
            TabsComponent, Object, Object])
    ], TabPaneComponent);
    return TabPaneComponent;
}(StylableComponent));

var DEFAULT_CLS$1 = 'app-tabs clearfix';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-tabs',
    hostClass: DEFAULT_CLS$1
};
var TabsComponent = /** @class */ (function (_super) {
    __extends(TabsComponent, _super);
    function TabsComponent(inj, dynamicComponentProvider, _transition, _tabsPosition, statePersistence) {
        var _this = this;
        // handle to the promise resolver
        var resolveFn = noop;
        _this = _super.call(this, inj, WIDGET_CONFIG$1, new Promise(function (res) { return resolveFn = res; })) || this;
        _this.transition = _transition;
        _this.tabsposition = _tabsPosition;
        _this.statePersistence = statePersistence;
        _this.dynamicComponentProvider = dynamicComponentProvider;
        _this.dynamicTabs = [];
        _this.dynamicPaneIndex = 0;
        _this.promiseResolverFn = resolveFn;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    TabsComponent_1 = TabsComponent;
    TabsComponent.prototype.animateIn = function (element) {
        var tabHeader = $(element);
        // when the animation is not present toggle the active class.
        tabHeader.siblings('.active').removeClass('active');
        tabHeader.addClass('active');
        var ul = this.nativeElement.querySelector('ul.nav.nav-tabs');
        // move the tabheader into the viewport
        var $prevHeaderEle = tabHeader.prev();
        if ($prevHeaderEle.length) {
            ul.scrollLeft = $prevHeaderEle[0].offsetLeft;
        }
        else {
            ul.scrollLeft = 0;
        }
    };
    /**
     * This method is used to register the dynamic panes.
     * After all panes are initialzed, update the querylist manually based on index.
     * @param paneRef - refrence of the tabpane
     */
    TabsComponent.prototype.registerDynamicTab = function (paneRef) {
        this.dynamicTabs.push(paneRef);
        var isLastPane = this.dynamicTabs.length === this.dynamicPaneIndex;
        if (isLastPane) {
            var _loop_1 = function (i) {
                var newPaneRef = _.find(this_1.dynamicTabs, function (pane) { return pane.dynamicPaneIndex === i; });
                var isPaneAlreadyExist = _.find(this_1.panes.toArray(), newPaneRef);
                if (!isPaneAlreadyExist) {
                    this_1.panes.reset(__spread(this_1.panes.toArray(), [newPaneRef]));
                    if (newPaneRef.active) {
                        setTimeout(function () {
                            newPaneRef.select();
                        }, 20);
                    }
                }
            };
            var this_1 = this;
            for (var i = 0; i < this.dynamicTabs.length; i++) {
                _loop_1(i);
            }
        }
    };
    /**
     * This method is to add the tabpane dynamically
     * @param tabpanes - list of tabpanes
     */
    TabsComponent.prototype.addPane = function (tabpanes) {
        var _this = this;
        if (!_.isArray(tabpanes)) {
            tabpanes = [tabpanes];
        }
        var paneNamesList = [];
        _.forEach(tabpanes, function (pane, index) {
            var isPaneAlreadyCreated = _.find(_this.panes.toArray(), { name: pane.name });
            var isPaneNameExist = _.indexOf(paneNamesList, pane.name);
            // If user tries to add tabpane with the same name which is already exists then do not create the pane
            if (isPaneAlreadyCreated || isPaneNameExist > 0) {
                console.warn("The tab pane with name " + pane.name + " already exists");
                return;
            }
            var paramMarkup = '';
            var propsTmpl = '';
            _this.dynamicPaneIndex++;
            var name = pane.name ? pane.name : "tabpane" + (_this.panes.toArray().length + (index + 1));
            paneNamesList.push(name);
            var partialParams = _.get(pane, 'params');
            _.forEach(pane, function (value, key) {
                if (key !== 'params') {
                    propsTmpl = propsTmpl + " " + key + "=\"" + value + "\"";
                }
            });
            _.forEach(partialParams, function (value, key) {
                paramMarkup = paramMarkup + " <wm-param name=\"" + key + "\" value=\"" + value + "\"></wm-param>";
            });
            var markup = "<wm-tabpane dynamicPaneIndex=\"" + (_this.dynamicPaneIndex - 1) + "\" isdynamic=\"true\" name=\"" + name + "\" " + propsTmpl + ">\n                            " + paramMarkup + "\n                        </wm-tabpane>";
            if (!_this._dynamicContext) {
                _this._dynamicContext = Object.create(_this.viewParent);
                _this._dynamicContext[_this.getAttr('wmTab')] = _this;
            }
            _this.dynamicComponentProvider.addComponent(_this.getNativeElement().querySelector('.tab-content'), markup, _this._dynamicContext, { inj: _this.inj });
        });
        return paneNamesList;
    };
    /**
     * This method is to remove the tabpane
     * @param paneIndex - index of the pane
     */
    TabsComponent.prototype.removePane = function (paneName) {
        var paneRef = this.getPaneRefByName(paneName);
        if (paneRef) {
            paneRef.remove();
        }
    };
    /**
     * TabPane children components invoke this method to communicate with the parent
     * if the evt argument is defined on-change callback will be invoked.
     */
    TabsComponent.prototype.notifyChange = function (paneRef, evt) {
        if (!this.isSelectableTab(paneRef)) {
            return;
        }
        var headerElement;
        // invoke deselect event callback on the preset active tab
        if (this.activeTab) {
            this.activeTab.deselect();
        }
        // invoke select callback on the selected tab
        paneRef.invokeOnSelectCallback(evt);
        this.activeTab = paneRef.getWidget();
        var newPaneIndex = this.getPaneIndexByRef(paneRef);
        var mode = this.statePersistence.computeMode(this.statehandler);
        if (!this.isPageLoadCall && mode && mode.toLowerCase() !== 'none') {
            this.statePersistence.setWidgetState(this, this.activeTab.name);
        }
        else {
            this.defaultpaneindex = this.getPaneIndexByRef(paneRef);
            this.isPageLoadCall = false;
        }
        // invoke change callback if the evt is present, select a tab programmatically will not have the event
        if (evt) {
            this.invokeEventCallback('change', {
                $event: evt,
                newPaneIndex: newPaneIndex,
                oldPaneIndex: this._oldPaneIndex || 0
            });
        }
        this._oldPaneIndex = newPaneIndex;
        if (evt) {
            headerElement = $(evt.target).closest('li.tab-header');
        }
        else {
            headerElement = this.nativeElement.querySelector("li[data-paneid=" + paneRef.widgetId + "]");
        }
        this.animateIn(headerElement);
        // this.setTabsLeftPosition(this.getPaneIndexByRef(this.activeTab), this.panes.length);
        if (this.canSlide()) {
            if (!this.tabsAnimator) {
                this.tabsAnimator = new TabsAnimator(this);
                this.tabsAnimator.setGesturesEnabled(this.canSlide());
            }
            this.tabsAnimator.transitionTabIntoView();
        }
    };
    TabsComponent.prototype.goToTab = function (tabIndex) {
        if (this.isValidPaneIndex(tabIndex - 1)) {
            var tab = this.getPaneRefByIndex(tabIndex - 1);
            tab.select();
        }
    };
    TabsComponent.prototype.getPaneIndexByRef = function (paneRef) {
        return this.panes.toArray().indexOf(paneRef);
    };
    // Returns the active tab index from tabs.
    TabsComponent.prototype.getActiveTabIndex = function () {
        return _.findIndex(this.panes.toArray(), { isActive: true });
    };
    TabsComponent.prototype.isValidPaneIndex = function (index) {
        return (index >= 0 && index < this.panes.length);
    };
    TabsComponent.prototype.getPaneRefByIndex = function (index) {
        return this.panes.toArray()[index];
    };
    TabsComponent.prototype.getPaneRefByName = function (name) {
        return _.find(this.panes.toArray(), { name: name });
    };
    // returns false if the pane is hidden or disabled
    TabsComponent.prototype.isSelectableTab = function (paneRef) {
        return paneRef.show && !paneRef.disabled;
    };
    TabsComponent.prototype.canSlide = function () {
        return this.transition === 'slide' && !this.vertical;
    };
    TabsComponent.prototype.getSelectableTabAfterIndex = function (index) {
        for (var i = index + 1; i < this.panes.length; i++) {
            var pane = this.getPaneRefByIndex(i);
            if (this.isSelectableTab(pane)) {
                return pane;
            }
        }
    };
    TabsComponent.prototype.getSelectableTabBeforeIndex = function (index) {
        for (var i = index - 1; i >= 0; i--) {
            var pane = this.getPaneRefByIndex(i);
            if (this.isSelectableTab(pane)) {
                return pane;
            }
        }
    };
    // select next tab relative to the current active tab
    TabsComponent.prototype.next = function () {
        var pane = this.getSelectableTabAfterIndex(this.getActiveTabIndex());
        if (pane) {
            pane.select();
        }
    };
    // select prev tab relative to the current active tab
    TabsComponent.prototype.prev = function () {
        var pane = this.getSelectableTabBeforeIndex(this.getActiveTabIndex());
        if (pane) {
            pane.select();
        }
    };
    /**
     * this method will be invoked during the initialization of the component and on defaultpaneindex property change,
     * if the provided defaultpaneindex is not valid, find the first pane index which can be shown and select it
     */
    TabsComponent.prototype.selectDefaultPaneByIndex = function (index) {
        if (!this.isValidPaneIndex(index)) {
            return;
        }
        var paneRef = this.getPaneRefByIndex(index);
        if (!this.isSelectableTab(paneRef)) {
            paneRef = this.getSelectableTabAfterIndex(0);
        }
        if (paneRef) {
            paneRef.select();
        }
    };
    // update the postion of tab header
    TabsComponent.prototype.setTabsPosition = function () {
        var ul = this.nativeElement.children[0];
        this.vertical = (this.tabsposition === 'left' || this.tabsposition === 'right');
        removeClass(this.nativeElement, 'inverted');
        if (this.tabsposition === 'bottom' || this.tabsposition === 'right') {
            appendNode(ul, this.nativeElement);
            addClass(this.nativeElement, 'inverted');
        }
    };
    TabsComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'defaultpaneindex') {
            this.defaultpaneindex = nv;
        }
        else if (key === 'statehandler') {
            this.isPageLoadCall = true;
            var widgetState_1 = this.statePersistence.getWidgetState(this);
            if (nv !== 'none' && widgetState_1) {
                var paneToSelect = this.panes.filter(function (pane) {
                    return widgetState_1 === pane.name;
                });
                if (!paneToSelect.length) {
                    console.warn('Tab pane name ' + widgetState_1 + ' in State is incorrect. Falling back to the default pane');
                    setTimeout(function () { return _this.selectDefaultPaneByIndex(_this.defaultpaneindex || 0); }, 20);
                }
                else {
                    var index_1 = this.getPaneIndexByRef(paneToSelect[0]);
                    setTimeout(function () { return _this.selectDefaultPaneByIndex(index_1); }, 20);
                }
            }
            else {
                setTimeout(function () { return _this.selectDefaultPaneByIndex(_this.defaultpaneindex || 0); }, 20);
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    TabsComponent.prototype.registerTabsScroll = function () {
        var _this = this;
        setTimeout(function () {
            var $ul = _this.$element.find('> ul');
            var $liPosition;
            var $li = $ul.children();
            $liPosition = $li.last().position();
            if ($liPosition && ($liPosition.left > $ul.width())) {
                $ul.on('mousewheel', function (e) {
                    var left = $ul[0].scrollLeft, _delta = -1 * e.originalEvent.wheelDelta;
                    e.stopPropagation();
                    e.preventDefault();
                    $ul.animate({ scrollLeft: left + _delta }, { 'duration': 10 });
                });
            }
        });
    };
    TabsComponent.prototype.ngAfterContentInit = function () {
        this.promiseResolverFn();
        _super.prototype.ngAfterContentInit.call(this);
        this.setTabsPosition();
    };
    TabsComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        this.registerTabsScroll();
    };
    var TabsComponent_1;
    TabsComponent.initializeProps = registerProps();
    TabsComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: DynamicComponentRefProvider },
        { type: String, decorators: [{ type: Attribute, args: ['transition',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['tabsposition',] }] },
        { type: StatePersistence }
    ]; };
    __decorate([
        ContentChildren(TabPaneComponent),
        __metadata("design:type", QueryList)
    ], TabsComponent.prototype, "panes", void 0);
    TabsComponent = TabsComponent_1 = __decorate([
        Component({
            selector: 'div[wmTabs]',
            template: "<ul class=\"nav nav-tabs\" [ngClass]=\"{'nav-stacked': vertical, 'nav-justified': justified}\" role=\"tablist\">\n    <li class=\"tab-header\" *ngFor=\"let pane of panes;\" [attr.data-paneid]=\"pane.widgetId\" [ngClass]=\"{'active': pane.isActive, 'disabled': pane.disabled}\"\n        [hidden]=\"!pane.show\" (click)=\"pane.select($event)\" role=\"tab\">\n        <a href=\"javascript:void(0);\" role=\"button\" [attr.aria-label]=\"pane.title\" [attr.title]=\"pane.title\" [tabindex]=\"pane.tabindex\">\n            <div class=\"tab-heading\">\n                <i [ngClass]=\"['app-icon', pane.paneicon]\" *ngIf=\"pane.paneicon\"></i>\n                <span [textContent]=\"pane.title\"></span>\n                <span *ngIf=\"pane.badgevalue\" class=\"label label-{{pane.badgetype}}\" [textContent]=\"pane.badgevalue\"></span>\n            </div>\n        </a>\n    </li>\n</ul>\n<div class=\"tab-content\" [ngClass]=\"{'tab-stacked': vertical, 'tab-justified': justified}\">\n    <ng-content select=\"div[wmTabPane]\"></ng-content>\n</div>\n",
            providers: [
                provideAsWidgetRef(TabsComponent_1)
            ]
        }),
        __param(2, Attribute('transition')),
        __param(3, Attribute('tabsposition')),
        __metadata("design:paramtypes", [Injector,
            DynamicComponentRefProvider, String, String, StatePersistence])
    ], TabsComponent);
    return TabsComponent;
}(StylableComponent));

var components = [
    TabPaneComponent,
    TabsComponent
];
var TabsModule = /** @class */ (function () {
    function TabsModule() {
    }
    TabsModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], TabsModule);
    return TabsModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { TabsModule, TabPaneComponent as ɵa, TabsComponent as ɵb, registerProps as ɵc, registerProps$1 as ɵd };
//# sourceMappingURL=index.js.map
