import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Self, ContentChildren, HostBinding, Directive, QueryList, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, BaseComponent, RedrawableDirective, provideAsWidgetRef, StylableComponent, styler, APPLY_STYLES_TYPE, WmComponentsModule } from '@wm/components/base';
import { noop } from '@wm/core';
import { NgForm } from '@angular/forms';

const registerProps = () => {
    register('wm-wizard', new Map([
        ['actionsalignment', Object.assign({ value: 'right' }, PROP_STRING)],
        ['cancelable', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['cancelbtnlabel', Object.assign({ value: 'Cancel' }, PROP_STRING)],
        ['class', Object.assign({ value: 'classic' }, PROP_STRING)],
        ['defaultstep', Object.assign({ value: 'none' }, PROP_STRING)],
        ['donebtnlabel', Object.assign({ value: 'Done' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['nextbtnlabel', Object.assign({ value: 'Next' }, PROP_STRING)],
        ['previousbtnlabel', Object.assign({ value: 'Previous' }, PROP_STRING)],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['stepstyle', Object.assign({ value: 'auto' }, PROP_STRING)]
    ]));
};

const registerProps$1 = () => {
    register('wm-wizardstep', new Map([
        ['class', PROP_STRING],
        ['enableskip', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['iconclass', Object.assign({ value: 'wi wi-done' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['title', Object.assign({ value: 'Step Title' }, PROP_STRING)]
    ]));
};

var WizardStepDirective_1;
const DEFAULT_CLS = 'app-wizard-step-content';
const WIDGET_CONFIG = {
    widgetType: 'wm-wizardstep',
    hostClass: DEFAULT_CLS,
};
let WizardStepDirective = WizardStepDirective_1 = class WizardStepDirective extends BaseComponent {
    constructor(inj, ngForm) {
        super(inj, WIDGET_CONFIG);
        this.ngForm = ngForm;
        this.isDone = false;
        this.status = 2 /* DISABLED */;
        // If we inject directly as a param in the constructor, getting circular dependency error in unit testcases
        this.wizardComponent = this.inj.get(WizardComponent);
    }
    get isCurrent() {
        return this.active;
    }
    /**
     * along with the wizard form , also validate the forms
     * if there are any inside the wizard
     * @returns {boolean}
     */
    get isValid() {
        return this.ngForm.valid && this.areEmbeddedFormsValid();
    }
    /**
     * get all the forms inside the wizard if any and validate
     * @returns {boolean}
     */
    areEmbeddedFormsValid() {
        let embeddedForms = this.getAllEmbeddedForms();
        for (let form of embeddedForms) {
            if (form.widget && !form.widget.ngform.valid) {
                return false;
            }
        }
        return true;
    }
    getAllEmbeddedForms() {
        return this.$element.find('form');
    }
    get enableNext() {
        return !this.disablenext;
    }
    get enableDone() {
        return !this.disabledone;
    }
    get enablePrev() {
        return !this.disableprevious;
    }
    set active(nv) {
        const isActive = this.active;
        this.status = 1 /* CURRENT */;
        if (nv && !isActive) {
            this.redrawChildren();
        }
    }
    get active() {
        return this.status === 1 /* CURRENT */;
    }
    set done(nv) {
        if (nv) {
            this.status = 3 /* COMPLETED */;
        }
    }
    get done() {
        return this.status === 3 /* COMPLETED */;
    }
    set disabled(nv) {
        if (nv) {
            this.status = 2 /* DISABLED */;
        }
    }
    get disabled() {
        return this.status === 2 /* DISABLED */;
    }
    invokeNextCB(index) {
        return this.invokeEventCallback('next', { currentStep: this, stepIndex: index });
    }
    invokePrevCB(index) {
        return this.invokeEventCallback('prev', { currentStep: this, stepIndex: index });
    }
    invokeSkipCB(index) {
        return this.invokeEventCallback('skip', { currentStep: this, stepIndex: index });
    }
    // redraw all the projected components which are projected.
    redrawChildren() {
        setTimeout(() => {
            if (this.reDrawableComponents) {
                this.reDrawableComponents.forEach(c => c.redraw());
            }
            const stepIndex = this.wizardComponent.getCurrentStepIndex();
            this.invokeEventCallback('load', { stepIndex });
        }, 100);
    }
};
WizardStepDirective.initializeProps = registerProps$1();
WizardStepDirective.ctorParameters = () => [
    { type: Injector },
    { type: NgForm, decorators: [{ type: Self }] }
];
__decorate([
    ContentChildren(RedrawableDirective, { descendants: true }),
    __metadata("design:type", Object)
], WizardStepDirective.prototype, "reDrawableComponents", void 0);
__decorate([
    HostBinding('class.current'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], WizardStepDirective.prototype, "isCurrent", null);
WizardStepDirective = WizardStepDirective_1 = __decorate([
    Directive({
        selector: 'form[wmWizardStep]',
        providers: [
            provideAsWidgetRef(WizardStepDirective_1)
        ],
        exportAs: 'wmWizardStep'
    }),
    __param(1, Self()),
    __metadata("design:paramtypes", [Injector, NgForm])
], WizardStepDirective);

var WizardComponent_1;
const DEFAULT_CLS$1 = 'app-wizard panel clearfix';
const WIDGET_CONFIG$1 = {
    widgetType: 'wm-wizard',
    hostClass: DEFAULT_CLS$1
};
let WizardComponent = WizardComponent_1 = class WizardComponent extends StylableComponent {
    constructor(inj) {
        let resolveFn = noop;
        super(inj, WIDGET_CONFIG$1, new Promise(res => resolveFn = res));
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SHELL);
        this.promiseResolverFn = resolveFn;
        // initialize the message object with default values
        this.message = {
            caption: '',
            type: ''
        };
    }
    get hasPrevStep() {
        return !this.isFirstStep(this.currentStep);
    }
    get hasNextStep() {
        return !this.isLastStep(this.currentStep);
    }
    get showDoneBtn() {
        if (!this.currentStep) {
            return;
        }
        return !this.hasNextStep && this.currentStep.enableDone;
    }
    get enablePrev() {
        if (!this.currentStep) {
            return;
        }
        return this.currentStep.enablePrev;
    }
    get enableNext() {
        if (!this.currentStep) {
            return;
        }
        return this.currentStep.enableNext && this.currentStep.isValid;
    }
    get enableDone() {
        if (!this.currentStep) {
            return;
        }
        return this.currentStep.enableDone && this.currentStep.isValid;
    }
    /**
     * returns next valid step. the index passed is also checked if its valid step
     * @param index
     * @returns {WizardStepDirective}
     */
    getNextValidStepFormIndex(index) {
        for (let i = index; i < this.steps.length; i++) {
            const step = this.getStepRefByIndex(i);
            if (step.show) {
                return step;
            }
        }
    }
    /**
     * returns previous valid step. the index passed is also checked if its valid step
     * @param index
     * @returns {WizardStepDirective}
     */
    getPreviousValidStepFormIndex(index) {
        for (let i = index; i >= 0; i--) {
            const step = this.getStepRefByIndex(i);
            if (step.show) {
                return step;
            }
        }
    }
    /**
     * returns current step index value.
     * @returns {number}
     */
    getCurrentStepIndex() {
        return this.getStepIndexByRef(this.currentStep);
    }
    /**
     * returns stepRef when index is passed.
     * @param {number} index
     * @returns {WizardStepDirective}
     */
    getStepRefByIndex(index) {
        return this.steps.toArray()[index];
    }
    /**
     * returns the index value of the step.
     * @param {WizardStepDirective} wizardStep
     * @returns {number}
     */
    getStepIndexByRef(wizardStep) {
        return this.steps.toArray().indexOf(wizardStep);
    }
    /**
     * gets stepRef by searching on the name property.
     * @param {string} name
     * @returns {WizardStepDirective}
     */
    getStepRefByName(name) {
        return this.steps.find(step => step.name === name);
    }
    /**
     * sets default step as current step if configured
     * or finds first valid step and set it as current step.
     * @param {WizardStepDirective} step
     */
    setDefaultStep(step) {
        // If the default step has show true then only update the currentStep
        if (step && step.show) {
            this.currentStep = step;
            step.active = true;
            step.isInitialized = true;
            // Mark all previous step status COMPLETED
            let index = this.getStepIndexByRef(step) - 1;
            while (index >= 0) {
                const prevStep = this.getStepRefByIndex(index);
                prevStep.done = true;
                prevStep.isInitialized = true;
                index--;
            }
        }
        else {
            // set next valid step as current step
            step = this.getNextValidStepFormIndex(0);
            if (step) {
                this.setDefaultStep(step);
            }
        }
    }
    /**
     * Selects the associated step when the wizard header is clicked.
     * @param $event
     * @param {WizardStepDirective} currentStep
     */
    onWizardHeaderClick($event, currentStep) {
        // select the step if it's status is done
        if (currentStep.done) {
            // set all the next steps status as disabled and previous steps as done
            this.steps.forEach((step, index) => {
                if (index < this.getStepIndexByRef(currentStep)) {
                    step.done = true;
                    step.isDone = true;
                }
                else {
                    step.disabled = true;
                    step.isDone = false;
                }
            });
            // set the selected step as current step and make it active
            this.currentStep = currentStep;
            this.currentStep.active = true;
        }
    }
    // Method to navigate to next step
    next(eventName = 'next') {
        const currentStep = this.currentStep;
        const currentStepIndex = this.getCurrentStepIndex();
        let nextStep;
        // abort if onSkip method returns false
        if (eventName === 'skip') {
            if (currentStep.invokeSkipCB(currentStepIndex) === false) {
                return;
            }
        }
        else if (eventName === 'next') {
            if (currentStep.invokeNextCB(currentStepIndex) === false) {
                return;
            }
        }
        nextStep = this.getNextValidStepFormIndex(currentStepIndex + 1);
        nextStep.isInitialized = true;
        // If there are any steps which has show then only change state of current step else remain same
        if (nextStep) {
            currentStep.isDone = true;
            currentStep.done = true;
            nextStep.active = true;
            this.currentStep = nextStep;
        }
    }
    // Method to navigate to previous step
    prev() {
        const currentStep = this.currentStep;
        const currentStepIndex = this.getCurrentStepIndex();
        let prevStep;
        // abort if onPrev method returns false.
        if (currentStep.invokePrevCB(currentStepIndex) === false) {
            return;
        }
        prevStep = this.getPreviousValidStepFormIndex(currentStepIndex - 1);
        // If there are any steps which has show then only change state of current step else remain same
        if (prevStep) {
            currentStep.isDone = false;
            currentStep.disabled = true;
            prevStep.active = true;
            this.currentStep = prevStep;
        }
    }
    skip() {
        this.next('skip');
    }
    // Method to invoke on-Done event on wizard
    done() {
        this.currentStep.isDone = true;
        this.invokeEventCallback('done', { steps: this.steps.toArray() });
    }
    // Method to invoke on-Cancel event on wizard
    cancel() {
        this.invokeEventCallback('cancel', { steps: this.steps.toArray() });
    }
    isFirstStep(stepRef) {
        return this.steps.first === stepRef;
    }
    isLastStep(stepRef) {
        return this.steps.last === stepRef;
    }
    // Define the property change handler. This Method will be triggered when there is a change in the widget property
    onPropertyChange(key, nv, ov) {
        // Monitoring changes for properties and accordingly handling respective changes
        if (key === 'stepstyle') {
            this.stepClass = nv === 'justified' ? 'nav-justified' : '';
        }
        else if (key === 'defaultstep') {
            this.setDefaultStep(this.getStepRefByName(nv));
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngAfterContentInit() {
        super.ngAfterContentInit();
        this.promiseResolverFn();
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        styler(this.nativeElement.querySelector('.panel-body'), this, APPLY_STYLES_TYPE.INNER_SHELL);
    }
};
WizardComponent.initializeProps = registerProps();
WizardComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ContentChildren(WizardStepDirective),
    __metadata("design:type", QueryList)
], WizardComponent.prototype, "steps", void 0);
WizardComponent = WizardComponent_1 = __decorate([
    Component({
        selector: 'div[wmWizard]',
        template: "<div class=\"app-wizard-heading\" role=\"tab\">\n    <ul class=\"app-wizard-steps nav nav-pills {{stepClass}}\">\n        <li class=\"app-wizard-step\" *ngFor=\"let step of steps; let i = index\" [hidden]=\"!step.show\"\n            [ngClass]=\"{active: step.done, current: step.active, disabled: step.disabled}\"\n            (click)=\"onWizardHeaderClick($event, step)\">\n            <a href=\"javascript:void(0)\" [attr.aria-label]=\"step.title\" [attr.title]=\"step.title\">\n                <span class=\"arrow\" *ngIf=\"class && class.includes('classic')\"></span>\n                <span class=\"wizard-step-number\">\n                    <span class=\"count\" *ngIf=\"class && class.includes('number') && !step.isDone\">{{i+1}}</span>\n                    <i class=\"dottedstepper\" *ngIf=\"class && (class.includes('dottedstepper') || class.includes('text-inline')) && !step.isDone\"></i>\n                    <i class=\"app-icon {{step.iconclass}}\" *ngIf=\"step.iconclass && step.isDone\"></i>\n                </span>\n                <span class=\"title-wrapper\">\n                    <span *ngIf=\"class && class.includes('text-inline')\">{{'0' + (i+1)}}</span>\n                    <span class=\"step-title\" [textContent]=\"step.title\"></span>\n                </span>\n            </a>\n        </li>\n    </ul>\n</div>\n<div class=\"app-wizard-body panel-body\" role=\"tabpanel\">\n    <p wmMessage *ngIf=\"message.caption\" caption.bind=\"message.caption\" type.bind=\"message.type\"></p>\n    <ng-content select=\"form[wmWizardStep]\"></ng-content>\n</div>\n<div class=\"app-wizard-actions panel-footer {{actionsalignment}}\">\n    <a class=\"app-wizard-skip\" name=\"skipStep_{{name}}\" *ngIf=\"currentStep?.enableskip\" title=\"Skip step\" (click)=\"skip()\">Skip &raquo;</a>\n    <div class=\"app-wizard-actions-right\">\n        <button type=\"button\" name=\"cancelBtn_{{name}}\" class=\"btn app-button btn-default\" *ngIf=\"cancelable\"\n                (click)=\"cancel()\" [title]=\"cancelbtnlabel\" [textContent]=\"cancelbtnlabel\"></button>\n        <button type=\"button\" name=\"previousBtn_{{name}}\" class=\"btn app-button btn-default\"\n                *ngIf=\"hasPrevStep\" (click)=\"prev()\" [disabled]=\"!enablePrev\">\n            <i class=\"app-icon wi wi-chevron-left\"></i>\n            <span class=\"btn-caption\" [textContent]=\"previousbtnlabel\"></span>\n        </button>\n        <button type=\"button\" name=\"nextBtn_{{name}}\" class=\"btn app-button btn-primary\"\n                *ngIf=\"hasNextStep\" (click)=\"next()\" [disabled]=\"!enableNext\">\n            <span class=\"btn-caption\" [textContent]=\"nextbtnlabel\"></span>\n            <i class=\"app-icon wi wi-chevron-right\"></i>\n        </button>\n        <button type=\"button\" name=\"doneBtn_{{name}}\" class=\"btn app-button btn-success\"\n                *ngIf=\"showDoneBtn\" (click)=\"done()\"\n                [disabled]=\"!enableDone\">\n            <i class=\"app-icon wi wi-done\"></i>\n            <span class=\"btn-caption\" [textContent]=\"donebtnlabel\"></span>\n        </button>\n    </div>\n</div>\n",
        providers: [
            provideAsWidgetRef(WizardComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], WizardComponent);

const components = [
    WizardStepDirective,
    WizardComponent
];
let WizardModule = class WizardModule {
};
WizardModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], WizardModule);

/**
 * Generated bundle index. Do not edit.
 */

export { WizardModule, WizardStepDirective as ɵa, registerProps$1 as ɵb, WizardComponent as ɵc, registerProps as ɵd };
//# sourceMappingURL=index.js.map
