import { __assign, __extends, __values, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Self, ContentChildren, HostBinding, Directive, QueryList, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, RedrawableDirective, provideAsWidgetRef, BaseComponent, styler, APPLY_STYLES_TYPE, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { noop } from '@wm/core';
import { NgForm } from '@angular/forms';

var registerProps = function () {
    register('wm-wizard', new Map([
        ['actionsalignment', __assign({ value: 'right' }, PROP_STRING)],
        ['cancelable', __assign({ value: true }, PROP_BOOLEAN)],
        ['cancelbtnlabel', __assign({ value: 'Cancel' }, PROP_STRING)],
        ['class', __assign({ value: 'classic' }, PROP_STRING)],
        ['defaultstep', __assign({ value: 'none' }, PROP_STRING)],
        ['donebtnlabel', __assign({ value: 'Done' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['nextbtnlabel', __assign({ value: 'Next' }, PROP_STRING)],
        ['previousbtnlabel', __assign({ value: 'Previous' }, PROP_STRING)],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['stepstyle', __assign({ value: 'auto' }, PROP_STRING)]
    ]));
};

var registerProps$1 = function () {
    register('wm-wizardstep', new Map([
        ['class', PROP_STRING],
        ['enableskip', __assign({ value: false }, PROP_BOOLEAN)],
        ['iconclass', __assign({ value: 'wi wi-done' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['title', __assign({ value: 'Step Title' }, PROP_STRING)]
    ]));
};

var DEFAULT_CLS = 'app-wizard-step-content';
var WIDGET_CONFIG = {
    widgetType: 'wm-wizardstep',
    hostClass: DEFAULT_CLS,
};
var WizardStepDirective = /** @class */ (function (_super) {
    __extends(WizardStepDirective, _super);
    function WizardStepDirective(inj, ngForm) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.ngForm = ngForm;
        _this.isDone = false;
        _this.status = 2 /* DISABLED */;
        // If we inject directly as a param in the constructor, getting circular dependency error in unit testcases
        _this.wizardComponent = _this.inj.get(WizardComponent);
        return _this;
    }
    WizardStepDirective_1 = WizardStepDirective;
    Object.defineProperty(WizardStepDirective.prototype, "isCurrent", {
        get: function () {
            return this.active;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardStepDirective.prototype, "isValid", {
        /**
         * along with the wizard form , also validate the forms
         * if there are any inside the wizard
         * @returns {boolean}
         */
        get: function () {
            return this.ngForm.valid && this.areEmbeddedFormsValid();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * get all the forms inside the wizard if any and validate
     * @returns {boolean}
     */
    WizardStepDirective.prototype.areEmbeddedFormsValid = function () {
        var e_1, _a;
        var embeddedForms = this.getAllEmbeddedForms();
        try {
            for (var embeddedForms_1 = __values(embeddedForms), embeddedForms_1_1 = embeddedForms_1.next(); !embeddedForms_1_1.done; embeddedForms_1_1 = embeddedForms_1.next()) {
                var form = embeddedForms_1_1.value;
                if (form.widget && !form.widget.ngform.valid) {
                    return false;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (embeddedForms_1_1 && !embeddedForms_1_1.done && (_a = embeddedForms_1.return)) _a.call(embeddedForms_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return true;
    };
    WizardStepDirective.prototype.getAllEmbeddedForms = function () {
        return this.$element.find('form');
    };
    Object.defineProperty(WizardStepDirective.prototype, "enableNext", {
        get: function () {
            return !this.disablenext;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardStepDirective.prototype, "enableDone", {
        get: function () {
            return !this.disabledone;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardStepDirective.prototype, "enablePrev", {
        get: function () {
            return !this.disableprevious;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardStepDirective.prototype, "active", {
        get: function () {
            return this.status === 1 /* CURRENT */;
        },
        set: function (nv) {
            var isActive = this.active;
            this.status = 1 /* CURRENT */;
            if (nv && !isActive) {
                this.redrawChildren();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardStepDirective.prototype, "done", {
        get: function () {
            return this.status === 3 /* COMPLETED */;
        },
        set: function (nv) {
            if (nv) {
                this.status = 3 /* COMPLETED */;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardStepDirective.prototype, "disabled", {
        get: function () {
            return this.status === 2 /* DISABLED */;
        },
        set: function (nv) {
            if (nv) {
                this.status = 2 /* DISABLED */;
            }
        },
        enumerable: true,
        configurable: true
    });
    WizardStepDirective.prototype.invokeNextCB = function (index) {
        return this.invokeEventCallback('next', { currentStep: this, stepIndex: index });
    };
    WizardStepDirective.prototype.invokePrevCB = function (index) {
        return this.invokeEventCallback('prev', { currentStep: this, stepIndex: index });
    };
    WizardStepDirective.prototype.invokeSkipCB = function (index) {
        return this.invokeEventCallback('skip', { currentStep: this, stepIndex: index });
    };
    // redraw all the projected components which are projected.
    WizardStepDirective.prototype.redrawChildren = function () {
        var _this = this;
        setTimeout(function () {
            if (_this.reDrawableComponents) {
                _this.reDrawableComponents.forEach(function (c) { return c.redraw(); });
            }
            var stepIndex = _this.wizardComponent.getCurrentStepIndex();
            _this.invokeEventCallback('load', { stepIndex: stepIndex });
        }, 100);
    };
    var WizardStepDirective_1;
    WizardStepDirective.initializeProps = registerProps$1();
    WizardStepDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: NgForm, decorators: [{ type: Self }] }
    ]; };
    __decorate([
        ContentChildren(RedrawableDirective, { descendants: true }),
        __metadata("design:type", Object)
    ], WizardStepDirective.prototype, "reDrawableComponents", void 0);
    __decorate([
        HostBinding('class.current'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], WizardStepDirective.prototype, "isCurrent", null);
    WizardStepDirective = WizardStepDirective_1 = __decorate([
        Directive({
            selector: 'form[wmWizardStep]',
            providers: [
                provideAsWidgetRef(WizardStepDirective_1)
            ],
            exportAs: 'wmWizardStep'
        }),
        __param(1, Self()),
        __metadata("design:paramtypes", [Injector, NgForm])
    ], WizardStepDirective);
    return WizardStepDirective;
}(BaseComponent));

var DEFAULT_CLS$1 = 'app-wizard panel clearfix';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-wizard',
    hostClass: DEFAULT_CLS$1
};
var WizardComponent = /** @class */ (function (_super) {
    __extends(WizardComponent, _super);
    function WizardComponent(inj) {
        var _this = this;
        var resolveFn = noop;
        _this = _super.call(this, inj, WIDGET_CONFIG$1, new Promise(function (res) { return resolveFn = res; })) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SHELL);
        _this.promiseResolverFn = resolveFn;
        // initialize the message object with default values
        _this.message = {
            caption: '',
            type: ''
        };
        return _this;
    }
    WizardComponent_1 = WizardComponent;
    Object.defineProperty(WizardComponent.prototype, "hasPrevStep", {
        get: function () {
            return !this.isFirstStep(this.currentStep);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardComponent.prototype, "hasNextStep", {
        get: function () {
            return !this.isLastStep(this.currentStep);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardComponent.prototype, "showDoneBtn", {
        get: function () {
            if (!this.currentStep) {
                return;
            }
            return !this.hasNextStep && this.currentStep.enableDone;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardComponent.prototype, "enablePrev", {
        get: function () {
            if (!this.currentStep) {
                return;
            }
            return this.currentStep.enablePrev;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardComponent.prototype, "enableNext", {
        get: function () {
            if (!this.currentStep) {
                return;
            }
            return this.currentStep.enableNext && this.currentStep.isValid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WizardComponent.prototype, "enableDone", {
        get: function () {
            if (!this.currentStep) {
                return;
            }
            return this.currentStep.enableDone && this.currentStep.isValid;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * returns next valid step. the index passed is also checked if its valid step
     * @param index
     * @returns {WizardStepDirective}
     */
    WizardComponent.prototype.getNextValidStepFormIndex = function (index) {
        for (var i = index; i < this.steps.length; i++) {
            var step = this.getStepRefByIndex(i);
            if (step.show) {
                return step;
            }
        }
    };
    /**
     * returns previous valid step. the index passed is also checked if its valid step
     * @param index
     * @returns {WizardStepDirective}
     */
    WizardComponent.prototype.getPreviousValidStepFormIndex = function (index) {
        for (var i = index; i >= 0; i--) {
            var step = this.getStepRefByIndex(i);
            if (step.show) {
                return step;
            }
        }
    };
    /**
     * returns current step index value.
     * @returns {number}
     */
    WizardComponent.prototype.getCurrentStepIndex = function () {
        return this.getStepIndexByRef(this.currentStep);
    };
    /**
     * returns stepRef when index is passed.
     * @param {number} index
     * @returns {WizardStepDirective}
     */
    WizardComponent.prototype.getStepRefByIndex = function (index) {
        return this.steps.toArray()[index];
    };
    /**
     * returns the index value of the step.
     * @param {WizardStepDirective} wizardStep
     * @returns {number}
     */
    WizardComponent.prototype.getStepIndexByRef = function (wizardStep) {
        return this.steps.toArray().indexOf(wizardStep);
    };
    /**
     * gets stepRef by searching on the name property.
     * @param {string} name
     * @returns {WizardStepDirective}
     */
    WizardComponent.prototype.getStepRefByName = function (name) {
        return this.steps.find(function (step) { return step.name === name; });
    };
    /**
     * sets default step as current step if configured
     * or finds first valid step and set it as current step.
     * @param {WizardStepDirective} step
     */
    WizardComponent.prototype.setDefaultStep = function (step) {
        // If the default step has show true then only update the currentStep
        if (step && step.show) {
            this.currentStep = step;
            step.active = true;
            step.isInitialized = true;
            // Mark all previous step status COMPLETED
            var index = this.getStepIndexByRef(step) - 1;
            while (index >= 0) {
                var prevStep = this.getStepRefByIndex(index);
                prevStep.done = true;
                prevStep.isInitialized = true;
                index--;
            }
        }
        else {
            // set next valid step as current step
            step = this.getNextValidStepFormIndex(0);
            if (step) {
                this.setDefaultStep(step);
            }
        }
    };
    /**
     * Selects the associated step when the wizard header is clicked.
     * @param $event
     * @param {WizardStepDirective} currentStep
     */
    WizardComponent.prototype.onWizardHeaderClick = function ($event, currentStep) {
        var _this = this;
        // select the step if it's status is done
        if (currentStep.done) {
            // set all the next steps status as disabled and previous steps as done
            this.steps.forEach(function (step, index) {
                if (index < _this.getStepIndexByRef(currentStep)) {
                    step.done = true;
                    step.isDone = true;
                }
                else {
                    step.disabled = true;
                    step.isDone = false;
                }
            });
            // set the selected step as current step and make it active
            this.currentStep = currentStep;
            this.currentStep.active = true;
        }
    };
    // Method to navigate to next step
    WizardComponent.prototype.next = function (eventName) {
        if (eventName === void 0) { eventName = 'next'; }
        var currentStep = this.currentStep;
        var currentStepIndex = this.getCurrentStepIndex();
        var nextStep;
        // abort if onSkip method returns false
        if (eventName === 'skip') {
            if (currentStep.invokeSkipCB(currentStepIndex) === false) {
                return;
            }
        }
        else if (eventName === 'next') {
            if (currentStep.invokeNextCB(currentStepIndex) === false) {
                return;
            }
        }
        nextStep = this.getNextValidStepFormIndex(currentStepIndex + 1);
        nextStep.isInitialized = true;
        // If there are any steps which has show then only change state of current step else remain same
        if (nextStep) {
            currentStep.isDone = true;
            currentStep.done = true;
            nextStep.active = true;
            this.currentStep = nextStep;
        }
    };
    // Method to navigate to previous step
    WizardComponent.prototype.prev = function () {
        var currentStep = this.currentStep;
        var currentStepIndex = this.getCurrentStepIndex();
        var prevStep;
        // abort if onPrev method returns false.
        if (currentStep.invokePrevCB(currentStepIndex) === false) {
            return;
        }
        prevStep = this.getPreviousValidStepFormIndex(currentStepIndex - 1);
        // If there are any steps which has show then only change state of current step else remain same
        if (prevStep) {
            currentStep.isDone = false;
            currentStep.disabled = true;
            prevStep.active = true;
            this.currentStep = prevStep;
        }
    };
    WizardComponent.prototype.skip = function () {
        this.next('skip');
    };
    // Method to invoke on-Done event on wizard
    WizardComponent.prototype.done = function () {
        this.currentStep.isDone = true;
        this.invokeEventCallback('done', { steps: this.steps.toArray() });
    };
    // Method to invoke on-Cancel event on wizard
    WizardComponent.prototype.cancel = function () {
        this.invokeEventCallback('cancel', { steps: this.steps.toArray() });
    };
    WizardComponent.prototype.isFirstStep = function (stepRef) {
        return this.steps.first === stepRef;
    };
    WizardComponent.prototype.isLastStep = function (stepRef) {
        return this.steps.last === stepRef;
    };
    // Define the property change handler. This Method will be triggered when there is a change in the widget property
    WizardComponent.prototype.onPropertyChange = function (key, nv, ov) {
        // Monitoring changes for properties and accordingly handling respective changes
        if (key === 'stepstyle') {
            this.stepClass = nv === 'justified' ? 'nav-justified' : '';
        }
        else if (key === 'defaultstep') {
            this.setDefaultStep(this.getStepRefByName(nv));
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    WizardComponent.prototype.ngAfterContentInit = function () {
        _super.prototype.ngAfterContentInit.call(this);
        this.promiseResolverFn();
    };
    WizardComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.nativeElement.querySelector('.panel-body'), this, APPLY_STYLES_TYPE.INNER_SHELL);
    };
    var WizardComponent_1;
    WizardComponent.initializeProps = registerProps();
    WizardComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ContentChildren(WizardStepDirective),
        __metadata("design:type", QueryList)
    ], WizardComponent.prototype, "steps", void 0);
    WizardComponent = WizardComponent_1 = __decorate([
        Component({
            selector: 'div[wmWizard]',
            template: "<div class=\"app-wizard-heading\" role=\"tab\">\n    <ul class=\"app-wizard-steps nav nav-pills {{stepClass}}\">\n        <li class=\"app-wizard-step\" *ngFor=\"let step of steps; let i = index\" [hidden]=\"!step.show\"\n            [ngClass]=\"{active: step.done, current: step.active, disabled: step.disabled}\"\n            (click)=\"onWizardHeaderClick($event, step)\">\n            <a href=\"javascript:void(0)\" [attr.aria-label]=\"step.title\" [attr.title]=\"step.title\">\n                <span class=\"arrow\" *ngIf=\"class && class.includes('classic')\"></span>\n                <span class=\"wizard-step-number\">\n                    <span class=\"count\" *ngIf=\"class && class.includes('number') && !step.isDone\">{{i+1}}</span>\n                    <i class=\"dottedstepper\" *ngIf=\"class && (class.includes('dottedstepper') || class.includes('text-inline')) && !step.isDone\"></i>\n                    <i class=\"app-icon {{step.iconclass}}\" *ngIf=\"step.iconclass && step.isDone\"></i>\n                </span>\n                <span class=\"title-wrapper\">\n                    <span *ngIf=\"class && class.includes('text-inline')\">{{'0' + (i+1)}}</span>\n                    <span class=\"step-title\" [textContent]=\"step.title\"></span>\n                </span>\n            </a>\n        </li>\n    </ul>\n</div>\n<div class=\"app-wizard-body panel-body\" role=\"tabpanel\">\n    <p wmMessage *ngIf=\"message.caption\" caption.bind=\"message.caption\" type.bind=\"message.type\"></p>\n    <ng-content select=\"form[wmWizardStep]\"></ng-content>\n</div>\n<div class=\"app-wizard-actions panel-footer {{actionsalignment}}\">\n    <a class=\"app-wizard-skip\" name=\"skipStep_{{name}}\" *ngIf=\"currentStep?.enableskip\" title=\"Skip step\" (click)=\"skip()\">Skip &raquo;</a>\n    <div class=\"app-wizard-actions-right\">\n        <button type=\"button\" name=\"cancelBtn_{{name}}\" class=\"btn app-button btn-default\" *ngIf=\"cancelable\"\n                (click)=\"cancel()\" [title]=\"cancelbtnlabel\" [textContent]=\"cancelbtnlabel\"></button>\n        <button type=\"button\" name=\"previousBtn_{{name}}\" class=\"btn app-button btn-default\"\n                *ngIf=\"hasPrevStep\" (click)=\"prev()\" [disabled]=\"!enablePrev\">\n            <i class=\"app-icon wi wi-chevron-left\"></i>\n            <span class=\"btn-caption\" [textContent]=\"previousbtnlabel\"></span>\n        </button>\n        <button type=\"button\" name=\"nextBtn_{{name}}\" class=\"btn app-button btn-primary\"\n                *ngIf=\"hasNextStep\" (click)=\"next()\" [disabled]=\"!enableNext\">\n            <span class=\"btn-caption\" [textContent]=\"nextbtnlabel\"></span>\n            <i class=\"app-icon wi wi-chevron-right\"></i>\n        </button>\n        <button type=\"button\" name=\"doneBtn_{{name}}\" class=\"btn app-button btn-success\"\n                *ngIf=\"showDoneBtn\" (click)=\"done()\"\n                [disabled]=\"!enableDone\">\n            <i class=\"app-icon wi wi-done\"></i>\n            <span class=\"btn-caption\" [textContent]=\"donebtnlabel\"></span>\n        </button>\n    </div>\n</div>\n",
            providers: [
                provideAsWidgetRef(WizardComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], WizardComponent);
    return WizardComponent;
}(StylableComponent));

var components = [
    WizardStepDirective,
    WizardComponent
];
var WizardModule = /** @class */ (function () {
    function WizardModule() {
    }
    WizardModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], WizardModule);
    return WizardModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { WizardModule, WizardStepDirective as ɵa, registerProps$1 as ɵb, WizardComponent as ɵc, registerProps as ɵd };
//# sourceMappingURL=index.js.map
