import { __decorate, __metadata } from 'tslib';
import { Injector, ViewChild, ElementRef, Component, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { BsDropdownModule } from 'ngx-bootstrap/dropdown';
import { register, PROP_STRING, PROP_BOOLEAN, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { MenuAdapterComponent, MenuModule } from '@wm/components/navigation/menu';

const registerProps = () => {
    register('wm-card', new Map([
        ['actions', PROP_STRING],
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['imageheight', Object.assign({ value: '200px' }, PROP_STRING)],
        ['itemaction', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['name', PROP_STRING],
        ['picturesource', PROP_STRING],
        ['picturetitle', Object.assign({ value: '' }, PROP_STRING)],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['subheading', PROP_STRING],
        ['title', PROP_STRING],
        ['userrole', PROP_STRING]
    ]));
};

var CardComponent_1;
const DEFAULT_CLS = 'app-card card app-panel';
const WIDGET_CONFIG = {
    widgetType: 'wm-card',
    hostClass: DEFAULT_CLS
};
let CardComponent = CardComponent_1 = class CardComponent extends MenuAdapterComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG);
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SHELL);
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        styler(this.cardContainerElRef.nativeElement, this, APPLY_STYLES_TYPE.INNER_SHELL);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'title' || key === 'subheading' || key === 'iconclass' || key === 'iconurl' || key === 'actions') {
            this.showHeader = !!(this.title || this.subheading || this.iconclass || this.iconurl || this.actions);
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
};
CardComponent.initializeProps = registerProps();
CardComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ViewChild('cardContainerWrapper', { static: true }),
    __metadata("design:type", ElementRef)
], CardComponent.prototype, "cardContainerElRef", void 0);
CardComponent = CardComponent_1 = __decorate([
    Component({
        selector: '[wmCard]',
        template: "<div class=\"app-card-header panel-heading\" *ngIf=\"showHeader\">\n    <div class=\"app-card-avatar\" *ngIf=\"iconclass || iconurl\">\n        <i class=\"app-icon\" [ngClass]=\"iconclass\" *ngIf=\"iconclass && !iconurl\"></i>\n        <img class=\"img-circle\" [src]=\"iconurl\" *ngIf=\"iconurl\" />\n    </div>\n    <div class=\"app-card-header-text\">\n        <h4 class=\"card-heading\" [textContent]=\"title\"></h4>\n        <h5 class=\"card-subheading text-muted\" [textContent]=\"subheading\"></h5>\n    </div>\n    <div class=\"panel-actions\" *ngIf=\"actions\">\n        <!-- TODO(punith) need to bind autoclose-->\n        <div wmMenu dropdown\n             [autoClose]=\"autoclose !== 'disabled'\"\n             class=\"panel-action\"\n             type=\"anchor\"\n             iconclass=\"wi wi-more-vert\"\n             menuposition=\"down,left\"\n             hint=\"Actions\"\n             caption=\"\"\n             dataset.bind=\"actions\">\n        </div>\n    </div>\n</div>\n<div class=\"app-card-image\" *ngIf=\"picturesource\"  [ngStyle]=\"{'max-height':imageheight}\">\n   <img wmPicture class=\"card-image\" picturesource.bind=\"picturesource\" hint.bind=\"picturetitle\" />\n</div>\n<div #cardContainerWrapper>\n    <ng-content select=\"[wmCardContent]\"></ng-content>\n</div>\n<div>\n    <ng-content select=\"[wmCardActions]\"></ng-content>\n</div>\n<div>\n    <ng-content select=\"[wmCardFooter]\"></ng-content>\n</div>\n",
        providers: [
            provideAsWidgetRef(CardComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], CardComponent);
// Todo(swathi) - menu

const registerProps$1 = () => {
    register('wm-card-actions', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ]));
};

const DEFAULT_CLS$1 = 'app-card-actions';
const WIDGET_CONFIG$1 = {
    widgetType: 'wm-card-actions',
    hostClass: DEFAULT_CLS$1
};
let CardActionsDirective = class CardActionsDirective extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$1);
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
};
CardActionsDirective.initializeProps = registerProps$1();
CardActionsDirective.ctorParameters = () => [
    { type: Injector }
];
CardActionsDirective = __decorate([
    Directive({
        selector: '[wmCardActions]'
    }),
    __metadata("design:paramtypes", [Injector])
], CardActionsDirective);

const registerProps$2 = () => {
    register('wm-card-content', new Map([
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var CardContentComponent_1;
const DEFAULT_CLS$2 = 'app-card-content card-body card-block';
const WIDGET_CONFIG$2 = {
    widgetType: 'wm-card-content',
    hostClass: DEFAULT_CLS$2
};
let CardContentComponent = CardContentComponent_1 = class CardContentComponent extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$2);
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        styler(this.cardContentContainerElRef.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
};
CardContentComponent.initializeProps = registerProps$2();
CardContentComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ViewChild('cardContentContainer'),
    __metadata("design:type", ElementRef)
], CardContentComponent.prototype, "cardContentContainerElRef", void 0);
CardContentComponent = CardContentComponent_1 = __decorate([
    Component({
        selector: 'div[wmCardContent]',
        template: "<div partial-container-target #cardContentContainer>\n    <ng-content></ng-content>\n</div>",
        providers: [
            provideAsWidgetRef(CardContentComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], CardContentComponent);

const registerProps$3 = () => {
    register('wm-card-footer', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ]));
};

const DEFAULT_CLS$3 = 'app-card-footer text-muted card-footer';
const WIDGET_CONFIG$3 = {
    widgetType: 'wm-card-footer',
    hostClass: DEFAULT_CLS$3
};
let CardFooterDirective = class CardFooterDirective extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$3);
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
};
CardFooterDirective.initializeProps = registerProps$3();
CardFooterDirective.ctorParameters = () => [
    { type: Injector }
];
CardFooterDirective = __decorate([
    Directive({
        selector: '[wmCardFooter]'
    }),
    __metadata("design:paramtypes", [Injector])
], CardFooterDirective);

const components = [
    CardComponent,
    CardActionsDirective,
    CardContentComponent,
    CardFooterDirective
];
let CardModule = class CardModule {
};
CardModule = __decorate([
    NgModule({
        imports: [
            BasicModule,
            BsDropdownModule,
            CommonModule,
            MenuModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], CardModule);

/**
 * Generated bundle index. Do not edit.
 */

export { CardModule, CardComponent as ɵa, registerProps as ɵb, CardActionsDirective as ɵc, registerProps$1 as ɵd, CardContentComponent as ɵe, registerProps$2 as ɵf, CardFooterDirective as ɵg, registerProps$3 as ɵh };
//# sourceMappingURL=index.js.map
