import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ViewChild, ElementRef, Component, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { BsDropdownModule } from 'ngx-bootstrap/dropdown';
import { register, PROP_STRING, PROP_BOOLEAN, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { MenuAdapterComponent, MenuModule } from '@wm/components/navigation/menu';

var registerProps = function () {
    register('wm-card', new Map([
        ['actions', PROP_STRING],
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['imageheight', __assign({ value: '200px' }, PROP_STRING)],
        ['itemaction', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['name', PROP_STRING],
        ['picturesource', PROP_STRING],
        ['picturetitle', __assign({ value: '' }, PROP_STRING)],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['subheading', PROP_STRING],
        ['title', PROP_STRING],
        ['userrole', PROP_STRING]
    ]));
};

var DEFAULT_CLS = 'app-card card app-panel';
var WIDGET_CONFIG = {
    widgetType: 'wm-card',
    hostClass: DEFAULT_CLS
};
var CardComponent = /** @class */ (function (_super) {
    __extends(CardComponent, _super);
    function CardComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SHELL);
        return _this;
    }
    CardComponent_1 = CardComponent;
    CardComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.cardContainerElRef.nativeElement, this, APPLY_STYLES_TYPE.INNER_SHELL);
    };
    CardComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'title' || key === 'subheading' || key === 'iconclass' || key === 'iconurl' || key === 'actions') {
            this.showHeader = !!(this.title || this.subheading || this.iconclass || this.iconurl || this.actions);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var CardComponent_1;
    CardComponent.initializeProps = registerProps();
    CardComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('cardContainerWrapper', { static: true }),
        __metadata("design:type", ElementRef)
    ], CardComponent.prototype, "cardContainerElRef", void 0);
    CardComponent = CardComponent_1 = __decorate([
        Component({
            selector: '[wmCard]',
            template: "<div class=\"app-card-header panel-heading\" *ngIf=\"showHeader\">\n    <div class=\"app-card-avatar\" *ngIf=\"iconclass || iconurl\">\n        <i class=\"app-icon\" [ngClass]=\"iconclass\" *ngIf=\"iconclass && !iconurl\"></i>\n        <img class=\"img-circle\" [src]=\"iconurl\" *ngIf=\"iconurl\" />\n    </div>\n    <div class=\"app-card-header-text\">\n        <h4 class=\"card-heading\" [textContent]=\"title\"></h4>\n        <h5 class=\"card-subheading text-muted\" [textContent]=\"subheading\"></h5>\n    </div>\n    <div class=\"panel-actions\" *ngIf=\"actions\">\n        <!-- TODO(punith) need to bind autoclose-->\n        <div wmMenu dropdown\n             [autoClose]=\"autoclose !== 'disabled'\"\n             class=\"panel-action\"\n             type=\"anchor\"\n             iconclass=\"wi wi-more-vert\"\n             menuposition=\"down,left\"\n             hint=\"Actions\"\n             caption=\"\"\n             dataset.bind=\"actions\">\n        </div>\n    </div>\n</div>\n<div class=\"app-card-image\" *ngIf=\"picturesource\"  [ngStyle]=\"{'max-height':imageheight}\">\n   <img wmPicture class=\"card-image\" picturesource.bind=\"picturesource\" hint.bind=\"picturetitle\" />\n</div>\n<div #cardContainerWrapper>\n    <ng-content select=\"[wmCardContent]\"></ng-content>\n</div>\n<div>\n    <ng-content select=\"[wmCardActions]\"></ng-content>\n</div>\n<div>\n    <ng-content select=\"[wmCardFooter]\"></ng-content>\n</div>\n",
            providers: [
                provideAsWidgetRef(CardComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], CardComponent);
    return CardComponent;
}(MenuAdapterComponent));
// Todo(swathi) - menu

var registerProps$1 = function () {
    register('wm-card-actions', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ]));
};

var DEFAULT_CLS$1 = 'app-card-actions';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-card-actions',
    hostClass: DEFAULT_CLS$1
};
var CardActionsDirective = /** @class */ (function (_super) {
    __extends(CardActionsDirective, _super);
    function CardActionsDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    CardActionsDirective.initializeProps = registerProps$1();
    CardActionsDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    CardActionsDirective = __decorate([
        Directive({
            selector: '[wmCardActions]'
        }),
        __metadata("design:paramtypes", [Injector])
    ], CardActionsDirective);
    return CardActionsDirective;
}(StylableComponent));

var registerProps$2 = function () {
    register('wm-card-content', new Map([
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$2 = 'app-card-content card-body card-block';
var WIDGET_CONFIG$2 = {
    widgetType: 'wm-card-content',
    hostClass: DEFAULT_CLS$2
};
var CardContentComponent = /** @class */ (function (_super) {
    __extends(CardContentComponent, _super);
    function CardContentComponent(inj) {
        return _super.call(this, inj, WIDGET_CONFIG$2) || this;
    }
    CardContentComponent_1 = CardContentComponent;
    CardContentComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.cardContentContainerElRef.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    };
    var CardContentComponent_1;
    CardContentComponent.initializeProps = registerProps$2();
    CardContentComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('cardContentContainer'),
        __metadata("design:type", ElementRef)
    ], CardContentComponent.prototype, "cardContentContainerElRef", void 0);
    CardContentComponent = CardContentComponent_1 = __decorate([
        Component({
            selector: 'div[wmCardContent]',
            template: "<div partial-container-target #cardContentContainer>\n    <ng-content></ng-content>\n</div>",
            providers: [
                provideAsWidgetRef(CardContentComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], CardContentComponent);
    return CardContentComponent;
}(StylableComponent));

var registerProps$3 = function () {
    register('wm-card-footer', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ]));
};

var DEFAULT_CLS$3 = 'app-card-footer text-muted card-footer';
var WIDGET_CONFIG$3 = {
    widgetType: 'wm-card-footer',
    hostClass: DEFAULT_CLS$3
};
var CardFooterDirective = /** @class */ (function (_super) {
    __extends(CardFooterDirective, _super);
    function CardFooterDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$3) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    CardFooterDirective.initializeProps = registerProps$3();
    CardFooterDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    CardFooterDirective = __decorate([
        Directive({
            selector: '[wmCardFooter]'
        }),
        __metadata("design:paramtypes", [Injector])
    ], CardFooterDirective);
    return CardFooterDirective;
}(StylableComponent));

var components = [
    CardComponent,
    CardActionsDirective,
    CardContentComponent,
    CardFooterDirective
];
var CardModule = /** @class */ (function () {
    function CardModule() {
    }
    CardModule = __decorate([
        NgModule({
            imports: [
                BasicModule,
                BsDropdownModule,
                CommonModule,
                MenuModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], CardModule);
    return CardModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { CardModule, CardComponent as ɵa, registerProps as ɵb, CardActionsDirective as ɵc, registerProps$1 as ɵd, CardContentComponent as ɵe, registerProps$2 as ɵf, CardFooterDirective as ɵg, registerProps$3 as ɵh };
//# sourceMappingURL=index.js.map
