import { __awaiter, __decorate, __metadata, __param } from 'tslib';
import { Injector, NgZone, Optional, SkipSelf, Attribute, ViewChild, ViewContainerRef, ContentChildren, HostBinding, HostListener, Component, Inject, Self, Directive, ContentChild, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, register, StylableComponent, styler, getFieldLayoutConfig, Live_Operations, performDataOperation, parseValueByType, PartialDirective, MessageComponent, WidgetRef, provideAsWidgetRef, BaseComponent, getWidgetPropsByType, getEvaluatedData, BaseFieldValidations, getDefaultViewModeWidget, Context, provideAs, fetchDistinctValues, applyFilterOnField, isDataSetWidget, getDistinctValuesForField, getRangeFieldValue, getRangeMatchMode, getEnableEmptyFilter, getEmptyMatchMode, fetchRelatedFieldData, ALLFIELDS, ToDatePipe, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { InputModule } from '@wm/components/input';
import { DesignDialogModule } from '@wm/components/dialogs/design-dialog';
import { FormArray, FormBuilder, NG_VALUE_ACCESSOR } from '@angular/forms';
import { isMobileApp, debounce, removeClass, isDefined, DataSource, $appDigest, getFiles, DataType, getClonedObject, $invokeWatchers, getValidJSON, extendProto, $parseEvent, App, AbstractDialogService, DynamicComponentRefProvider, FormWidgetType, isMobile, addForIdAttributes, Viewport, triggerFn, isEmptyObject, isDateTimeType, getValidDateObject } from '@wm/core';
import { PrefabDirective } from '@wm/components/prefab';
import { ListComponent } from '@wm/components/data/list';
import { TableComponent } from '@wm/components/data/table';
import { debounceTime } from 'rxjs/operators';
import { LiveTableComponent } from '@wm/components/data/live-table';

const propsMap = new Map([
    ['autocomplete', Object.assign({ value: false }, PROP_BOOLEAN)],
    ['captionalign', Object.assign({ value: 'left' }, PROP_STRING)],
    ['captionposition', Object.assign({ value: 'left' }, PROP_STRING)],
    ['captionsize', PROP_STRING],
    ['captionwidth', Object.assign({ value: 'xs-12 sm-3 md-3 lg-3' }, PROP_STRING)],
    ['class', PROP_STRING],
    ['collapsible', PROP_BOOLEAN],
    ['dataset', PROP_ANY],
    ['datasource', PROP_STRING],
    ['expanded', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['formdatasource', PROP_STRING],
    ['name', PROP_STRING],
    ['numberOfFields', PROP_NUMBER],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showindevice', Object.assign({ displayType: 'inline-block', value: 'all' }, PROP_STRING)],
    ['subheading', PROP_STRING],
    ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
    ['title', PROP_STRING]
]);
const registerFormProps = () => {
    if (isMobileApp()) {
        propsMap.set('captionwidth', Object.assign({ value: 'xs-4 sm-4 md-4 lg-4' }, PROP_STRING));
    }
    const formMap = new Map(propsMap);
    formMap.set('action', PROP_STRING);
    formMap.set('defaultmode', Object.assign({ value: 'Edit' }, PROP_STRING));
    formMap.set('metadata', PROP_STRING);
    formMap.set('method', PROP_STRING);
    formMap.set('postmessage', Object.assign({ value: 'Data posted successfully' }, PROP_STRING));
    formMap.set('target', PROP_STRING);
    formMap.set('enctype', PROP_STRING);
    formMap.set('errormessage', Object.assign({ value: 'An error occured. Please try again!' }, PROP_STRING));
    formMap.set('messagelayout', Object.assign({ value: 'Inline' }, PROP_STRING));
    formMap.set('formdata', PROP_STRING);
    formMap.set('novalidate', PROP_BOOLEAN);
    formMap.set('validationtype', Object.assign({ value: 'default' }, PROP_STRING));
    formMap.set('iconclass', PROP_STRING);
    register('wm-form', formMap);
};
const registerLiveFormProps = () => {
    if (isMobileApp()) {
        propsMap.set('captionwidth', Object.assign({ value: 'xs-4 sm-4 md-4 lg-4' }, PROP_STRING));
    }
    const liveFormMap = new Map(propsMap);
    liveFormMap.set('defaultmode', Object.assign({ value: 'View' }, PROP_STRING));
    liveFormMap.set('formlayout', Object.assign({ value: 'inline' }, PROP_STRING));
    liveFormMap.set('insertmessage', Object.assign({ value: 'Record added successfully' }, PROP_STRING));
    liveFormMap.set('updatemessage', Object.assign({ value: 'Record updated successfully' }, PROP_STRING));
    liveFormMap.set('deletemessage', Object.assign({ value: 'Record deleted successfully' }, PROP_STRING));
    liveFormMap.set('errormessage', Object.assign({ value: 'An error occured. Please try again!' }, PROP_STRING));
    liveFormMap.set('messagelayout', Object.assign({ value: 'Toaster' }, PROP_STRING));
    liveFormMap.set('formdata', PROP_STRING);
    liveFormMap.set('novalidate', PROP_BOOLEAN);
    liveFormMap.set('validationtype', Object.assign({ value: 'default' }, PROP_STRING));
    liveFormMap.set('iconclass', PROP_STRING);
    register('wm-liveform', liveFormMap);
};
const registerLiveFilterProps = () => {
    if (isMobileApp()) {
        propsMap.set('captionwidth', Object.assign({ value: 'xs-4 sm-4 md-4 lg-4' }, PROP_STRING));
    }
    const liveFilterMap = new Map(propsMap);
    liveFilterMap.set('autoupdate', PROP_BOOLEAN);
    liveFilterMap.set('enableemptyfilter', Object.assign({ value: ' ' }, PROP_STRING));
    liveFilterMap.set('pagesize', Object.assign({ value: 20 }, PROP_NUMBER));
    liveFilterMap.set('iconclass', Object.assign({ value: 'wi wi-filter-list' }, PROP_STRING));
    register('wm-livefilter', liveFilterMap);
};

var FormComponent_1;
const WIDGET_CONFIG = { widgetType: 'wm-form', hostClass: 'panel app-panel app-form' };
const LOGIN_FORM_CONFIG = { widgetType: 'wm-form', hostClass: 'app-form app-login-form' };
const LIVE_FORM_CONFIG = { widgetType: 'wm-liveform', hostClass: 'panel app-panel app-liveform liveform-inline' };
const LIVE_FILTER_CONFIG = { widgetType: 'wm-livefilter', hostClass: 'panel app-panel app-livefilter clearfix liveform-inline' };
const getWidgetConfig = (isLiveForm, isLiveFilter, role) => {
    let config = WIDGET_CONFIG;
    if (isLiveForm !== null) {
        config = LIVE_FORM_CONFIG;
    }
    else if (isLiveFilter !== null) {
        config = LIVE_FILTER_CONFIG;
    }
    else if (role === 'app-login') {
        config = LOGIN_FORM_CONFIG;
    }
    return config;
};
const ɵ0 = getWidgetConfig;
// Generate the form field with given field definition. Add a grid column wrapper around the form field.
const setMarkupForFormField = (field, columnWidth) => {
    let propsTmpl = '';
    _.forEach(field, (value, key) => {
        propsTmpl = `${propsTmpl} ${key}="${value}"`;
    });
    return `<wm-gridcolumn columnwidth="${columnWidth}">
                  <wm-form-field ${propsTmpl}></wm-form-field>
            </wm-gridcolumn>`;
};
const ɵ1 = setMarkupForFormField;
// Function to find out the first invalid element in form
const findInvalidElement = ($formEle, ngForm) => {
    const $ele = $formEle.find('form.ng-invalid:visible, [formControlName].ng-invalid:visible').first();
    let formObj = ngForm;
    // If element is form, find out the first invalid element in this form
    if ($ele.is('form')) {
        formObj = ngForm && ngForm.controls[$ele.attr('formControlName') || $ele.attr('name')];
        if (formObj) {
            return findInvalidElement($ele, formObj);
        }
    }
    return {
        ngForm: formObj,
        $ele: $ele
    };
};
const ɵ2 = findInvalidElement;
const setTouchedState = ngForm => {
    if (ngForm.valid) {
        return;
    }
    if (ngForm.controls) {
        _.forEach(ngForm.controls, ctrl => {
            setTouchedState(ctrl);
        });
    }
    else {
        ngForm.markAsTouched();
    }
};
const ɵ3 = setTouchedState;
let FormComponent = FormComponent_1 = class FormComponent extends StylableComponent {
    constructor(inj, fb, app, dialogService, dynamicComponentProvider, ngZone, parentList, parentForm, parentPrefab, parentPartial, onBeforeSubmitEvt, onSubmitEvt, onBeforeRenderEvt, binddataset, bindformdata, isLiveForm, isLiveFilter, role, key, name) {
        super(inj, getWidgetConfig(isLiveForm, isLiveFilter, role));
        this.fb = fb;
        this.app = app;
        this.dialogService = dialogService;
        this.dynamicComponentProvider = dynamicComponentProvider;
        this.ngZone = ngZone;
        this.parentList = parentList;
        this.parentForm = parentForm;
        this.parentPrefab = parentPrefab;
        this.parentPartial = parentPartial;
        this.onBeforeSubmitEvt = onBeforeSubmitEvt;
        this.onSubmitEvt = onSubmitEvt;
        this.onBeforeRenderEvt = onBeforeRenderEvt;
        this.binddataset = binddataset;
        this.bindformdata = bindformdata;
        this._widgetClass = '';
        this._captionClass = '';
        this.formFields = [];
        this.formfields = {};
        this.formWidgets = {};
        this.filterWidgets = {};
        this.buttonArray = [];
        this.dataoutput = {};
        this.formdata = {};
        this.statusMessage = {
            caption: '',
            type: ''
        };
        this.primaryKey = [];
        this._debouncedUpdateFieldSource = _.debounce(this.updateFieldSource, 350);
        this.validationMessages = [];
        this._debouncedSubmitForm = debounce(($event) => {
            // calling submit event in ngZone as change detection is not triggered post the submit callback and actions like notification are not shown
            this.ngZone.run(() => {
                this.submitForm($event);
            });
        }, 250);
        styler(this.nativeElement, this);
        this.isUpdateMode = true;
        this.dialogId = this.nativeElement.getAttribute('dialogId');
        this.ngform = fb.group({});
        this.bindingValue = key || name;
        // On value change in form, update the dataoutput
        const onValueChangeSubscription = this.ngform.valueChanges
            .subscribe(this.updateDataOutput.bind(this));
        this.registerDestroyListener(() => onValueChangeSubscription.unsubscribe());
        this.elScope = this;
        this.addEventsToContext(this.context);
    }
    set isLayoutDialog(nv) {
        if (nv) {
            removeClass(this.nativeElement, 'panel app-panel liveform-inline');
        }
        this._isLayoutDialog = nv;
    }
    get isLayoutDialog() {
        return this._isLayoutDialog;
    }
    set isUpdateMode(nv) {
        this._isUpdateMode = nv;
        this.formFields.forEach(field => {
            field.setReadOnlyState(nv);
        });
    }
    get isUpdateMode() {
        return this._isUpdateMode;
    }
    submit($event) {
        this._debouncedSubmitForm($event);
    }
    onReset() {
        this.reset();
    }
    // returns the formArray control on the parentForm.
    get parentFormArray() {
        return this.parentForm && this.isParentList && this.parentForm.ngform.get(this.parentList.name);
    }
    // returns true only when closest livelist is the immediate parent of the form.
    // checks for livelist having any form elements with parentForm name.
    // If livelist is not containing any parentForm elements then it returns true.
    get isParentList() {
        if (this.parentList && this.parentForm && !isDefined(this._formIsInList)) {
            const formEle = this.$element;
            const listEle = formEle.closest('.app-livelist[name="' + this.parentList.name + '"]');
            if (listEle.length) {
                this._formIsInList = !(listEle.find('form[widget-id="' + this.parentForm.widgetId + '"]')).length;
            }
        }
        return isDefined(this._formIsInList) ? this._formIsInList : this.parentList;
    }
    ngAfterContentInit() {
        setTimeout(() => {
            this.componentRefs.forEach(componentRef => {
                if (componentRef.name) {
                    // Register widgets inside form with formWidgets
                    this.formWidgets[componentRef.name] = componentRef;
                }
            });
        }, 250);
    }
    ngAfterViewInit() {
        if (this.parentForm && this.parentForm.ngform && this.isParentList) {
            // setting formArray control on ngform.
            if (!_.get(this.parentForm.ngform.controls, this.parentList.name)) {
                this.parentForm.ngform.setControl(this.parentList.name, new FormArray([]));
            }
            // pushing the ngform to formArray.
            this.parentFormArray.push(this.ngform);
            this.formArrayIndex = this.parentFormArray.length - 1;
        }
        this.addInnerNgFormToForm(this.bindingValue);
    }
    findOperationType() { }
    addInnerNgFormToForm(binding) {
        if (this.parentForm && this.parentForm.ngform) {
            // handling as formArray when parent is list. Hence returning here from adding to parentForm again.
            if (this.isParentList) {
                return;
            }
            // assigning the control name of the form inside the prefab with the prefab name.
            // This will happen only when there is only one form inside prefab.
            // We do not support changing this when multiple forms are inside prefab.
            // This scenario has to be handled by end user by changing form names in prefab according to the datamodel binding.
            const parentContainer = this.parentPrefab || this.parentPartial;
            if (parentContainer) {
                let parentContentEl;
                if (this.parentPrefab) {
                    parentContentEl = parentContainer.$element.find('[wmprefabcontainer]');
                }
                else {
                    parentContentEl = parentContainer.$element;
                }
                /**
                 * assigning the form name to the parentContainer name i.e. partial name or perfab name by following
                 * 1. retrieving all the forms in the parentContainer
                 * 2. consider the first form
                 * 3. if this form is inside list i.e. also find if this list is also inside the parentContainer (prefab / partial)
                 * 4. then do not change the form name when form is inside list and also when there are siblings to the form
                 * 5. this means form name change applies only when there is single form (immediate child) inside the parentContainer.
                 */
                const prefabInnerForm = parentContentEl.find('form').first();
                if (_.get(prefabInnerForm[0], 'widget.widgetId') === this.widgetId) {
                    // check whether form inside prefab container is inside the list. If true, do not change the name.
                    if (!prefabInnerForm.isParentList && !prefabInnerForm.siblings('form').length) {
                        binding = _.get(parentContainer, 'name');
                    }
                }
            }
            let counter = 1;
            let innerBinding = binding;
            // Inner forms may have same names. If same name is present, append unqiue identifier
            while (this.parentForm.ngform.controls.hasOwnProperty(innerBinding)) {
                innerBinding = `${binding}_${counter}`;
                counter++;
            }
            this.formGroupName = innerBinding;
            // If parent form is present, add the current form as as formGroup for parent form
            this.parentForm.ngform.addControl(innerBinding, this.ngform);
        }
    }
    // Expose the events on context so that they can be accessed by form actions
    addEventsToContext(context) {
        context.cancel = () => this.cancel();
        context.reset = () => this.reset();
        context.save = evt => this.save(evt);
        context.saveAndNew = () => this.saveAndNew();
        context.saveAndView = () => this.saveAndView();
        context.delete = () => this.delete();
        context.new = () => this.new();
        context.edit = () => this.edit();
        context.highlightInvalidFields = () => this.highlightInvalidFields();
        context.filter = () => this.filter();
        context.clearFilter = () => this.clearFilter();
        context.submit = evt => this.submit(evt);
    }
    // This method gets all the inner forms and validates each form.
    setValidationOnInnerForms(validateTouch) {
        const formEle = this.getNativeElement();
        const formObjs = formEle.querySelectorAll('.app-form');
        const validationMsgs = [];
        _.forEach(formObjs, e => {
            const formInstance = e.widget;
            // differentiating the validationMessages prefix based on the formGroupName
            // as the formName's are same when forms are in list
            let formName = _.get(formInstance, 'formGroupName') || formInstance.name;
            if (isDefined(formInstance.formArrayIndex)) {
                formName = formName + '[' + formInstance.formArrayIndex + ']';
            }
            let current = formInstance;
            while (_.get(current, 'parentForm')) {
                let parentName = current.parentForm.formGroupName || current.parentForm.name;
                if (current.parentForm.parentFormArray) {
                    parentName = parentName + '[' + current.parentForm.formArrayIndex + ']';
                }
                formName = parentName + '.' + formName;
                current = current.parentForm;
            }
            this.setValidationOnFields(formInstance, formName, validateTouch);
        });
    }
    /**
     * This method sets validation on formFields.
     * Applies to innerform and also sets innerform validation on parent form.
     * @param prefix contains the form name, which also includes its parents form name
     * @param {boolean} validateTouch
     */
    setValidationOnFields(form, prefix, validateTouch) {
        const controls = form.ngform.controls;
        const formFields = form.formFields;
        if (!formFields) {
            return;
        }
        _.forEach(controls, (v, k) => {
            const field = formFields.find(e => e.key === k);
            if (!field || (validateTouch && !v.touched)) {
                return;
            }
            // invoking the prepareValidation on both parent form and current form.
            this.prepareValidationObj(v, k, field, prefix);
            this.prepareValidationObj.call(form, v, k, field, prefix);
        });
    }
    // Assigns / updates validationMessages based on angular errors on field
    prepareValidationObj(v, k, field, prefix) {
        const index = this.validationMessages.findIndex(e => (e.field === k && e.fullyQualifiedFormName === prefix));
        if (v.invalid) {
            if (index === -1) {
                /**
                 * field contains the fieldName
                 * value contains the field value
                 * errorType contains the list of errors
                 * message contains the validation message
                 * getElement returns the element having focus-target
                 * formName returns the name of the form
                 */
                this.validationMessages.push({
                    field: k,
                    value: field.value,
                    errorType: _.keys(v.errors),
                    message: field.validationmessage || '',
                    getElement: () => {
                        return field.$element.find('[focus-target]');
                    },
                    formName: _.last(prefix.split('.')),
                    fullyQualifiedFormName: prefix
                });
            }
            else {
                this.validationMessages[index].value = field.value;
                this.validationMessages[index].errorType = _.keys(v.errors);
            }
        }
        else if (v.valid && index > -1) {
            this.validationMessages.splice(index, 1);
        }
    }
    // This will return a object containing the error details from the list of formFields that are invalid
    setValidationMsgs(validateTouch) {
        if (!this.formFields.length && _.isEmpty(this.ngform.controls)) {
            return;
        }
        this.setValidationOnFields(this, this.name, validateTouch);
        this.setValidationOnInnerForms(validateTouch);
    }
    // change and blur events are added from the template
    handleEvent(node, eventName, callback, locals) {
        if (eventName !== 'submit') {
            super.handleEvent(this.nativeElement, eventName, callback, locals);
        }
    }
    updateFieldSource() {
        if (this.formdatasource && this.formdatasource.execute(DataSource.Operation.IS_API_AWARE)) {
            return;
        }
        else if (this.formdatasource && !this.formdatasource.twoWayBinding) {
            return;
        }
        this.formFields.forEach(formField => {
            formField.setFormWidget('datavaluesource', this.formdatasource);
            formField.setFormWidget('binddatavalue', `${this.bindformdata}.${formField.key}`);
        });
    }
    // This method loops through the form fields and highlights the invalid fields by setting state to touched
    highlightInvalidFields() {
        setTouchedState(this.ngform);
    }
    // Disable the form submit if form is in invalid state. Highlight all the invalid fields if validation type is default
    validateFieldsOnSubmit() {
        this.setValidationMsgs();
        // Disable the form submit if form is in invalid state. For delete operation, do not check the validation.
        if (this.operationType !== 'delete' && (this.validationtype === 'html' || this.validationtype === 'default')
            && this.ngform && this.ngform.invalid) {
            if (this.ngform.invalid) {
                if (this.validationtype === 'default') {
                    this.highlightInvalidFields();
                }
                // Find the first invalid untoched element and set it to touched.
                // Safari does not form validations. this will ensure that error is shown for user
                const eleForm = findInvalidElement(this.$element, this.ngform);
                const $invalidForm = eleForm.ngForm;
                let $invalidEle = eleForm.$ele;
                $invalidEle = $invalidEle.parent().find('[focus-target]');
                if ($invalidEle.length) {
                    // on save click in page layout liveform, focus of autocomplete widget opens full-screen search.
                    if (!$invalidEle.hasClass('app-search-input')) {
                        $invalidEle.focus();
                    }
                    const ngEle = $invalidForm && $invalidForm.controls[$invalidEle.attr('formControlName') || $invalidEle.attr('name')];
                    if (ngEle && ngEle.markAsTouched) {
                        ngEle.markAsTouched();
                    }
                    $appDigest();
                    return true;
                }
                return true;
            }
            return false;
        }
        return false;
    }
    onPropertyChange(key, nv, ov) {
        switch (key) {
            case 'captionalign':
                this.captionAlignClass = 'align-' + nv;
                break;
            case 'captionposition':
                this.setLayoutConfig();
                break;
            case 'captionwidth':
                this.setLayoutConfig();
                break;
            case 'captionsize':
                this.captionsize = nv;
                break;
            case 'novalidate':
                //  Set validation type based on the novalidate property
                this.widget.validationtype = (nv === true || nv === 'true') ? 'none' : 'default';
                break;
            case 'formdata':
                // For livelist when multiselect is enabled, formdata will be array of objects. In this case consider the last object as formdata.
                _.isArray(nv) ? this.setFormData(_.last(nv)) : this.setFormData(nv);
                // if dataset on the formFields are not set as the datasourceChange is triggered before the formFields are registered.
                if (!this.isDataSourceUpdated && this.datasource) {
                    this.onDataSourceChange();
                }
                break;
            case 'defaultmode':
                if (!this.isLayoutDialog) {
                    if (nv && nv === 'Edit') {
                        this.updateMode = true;
                    }
                    else {
                        this.updateMode = false;
                    }
                    this.isUpdateMode = this.updateMode;
                }
                break;
            case 'datasource':
                this.onDataSourceChange();
                break;
            case 'formdatasource':
                this.onFormDataSourceChange();
                break;
            case 'metadata':
                this.generateFormFields();
                break;
            case 'dataset':
                const formFields = this.getFormFields();
                formFields.forEach(field => {
                    // notifying the dataset change to the form-field widget.
                    if (!field.isDataSetBound && _.get(field.formWidget, 'dataset$')) {
                        field.formWidget.dataset$.next();
                    }
                });
                break;
            default:
                super.onPropertyChange(key, nv, ov);
        }
    }
    // Event callbacks on success/error
    onResultCb(data, status, event) {
        const params = { $event: event, $data: data, $operation: this.operationType };
        // whether service call success or failure call this method
        this.invokeEventCallback('result', params);
        if (status) {
            // if service call is success call this method
            this.invokeEventCallback('success', params);
        }
        else {
            // if service call fails call this method
            this.invokeEventCallback('error', params);
        }
    }
    // Display or hide the inline message/ toaster
    toggleMessage(show, msg, type, header) {
        let template;
        if (show && msg) {
            template = (type === 'error' && this.errormessage) ? this.errormessage : msg;
            if (this.messagelayout === 'Inline') {
                template = this.checkAppServiceErrorMsg(type) || template;
                this.statusMessage = { 'caption': template || '', type: type };
                if (this.messageRef) {
                    this.messageRef.showMessage(this.statusMessage.caption, this.statusMessage.type);
                }
            }
            else {
                this.app.notifyApp(template, type, header);
            }
        }
        else {
            this.statusMessage.caption = '';
        }
    }
    // if there is an App.onServiceError handler for the app, that message should be used instead of server returned error message
    checkAppServiceErrorMsg(type) {
        const notificationAction = _.get(this.app, 'Actions.appNotification');
        if (notificationAction && type === 'error') {
            return notificationAction.getMessage();
        }
        return;
    }
    // Hide the inline message/ toaster
    clearMessage() {
        this.toggleMessage(false);
    }
    // Set the classes on the form based on the captionposition and captionwidth properties
    setLayoutConfig() {
        let layoutConfig;
        layoutConfig = getFieldLayoutConfig(this.captionwidth, this.captionposition, _.get(this.app.selectedViewPort, 'os'));
        this._widgetClass = layoutConfig.widgetCls;
        this._captionClass = layoutConfig.captionCls;
        $appDigest();
    }
    registerFormWidget(widget) {
        const name = widget.name || widget.key;
        this.formWidgets[name] = widget;
    }
    registerFormFields(formField) {
        const fieldCount = this.formFields.push(formField);
        this.formfields[formField.key] = formField;
        this.registerFormWidget(formField);
        this._debouncedUpdateFieldSource();
        if (this.parentForm) {
            this.parentForm.formFields.push(formField);
            this.parentForm.formfields[formField.key] = formField;
            // inner formfields are pushed to parentForm, passing current innerForm's formdata to set innerFormdata to these innerFormFields
            this.parentForm.setFormData(this.parentForm.formdata, this.formFields, this.formdata || {});
        }
        // WMS-18967: Applying formadata after formfields are ready
        if (fieldCount === this.numberOfFields && !_.isEmpty(this.formdata)) {
            this.onDataSourceChange();
            _.isArray(this.formdata) ? this.setFormData(_.last(this.formdata)) : this.setFormData(this.formdata);
        }
    }
    registerActions(formAction) {
        this.buttonArray.push(formAction);
    }
    // Update the dataoutput whenever there is a change in inside form widget value
    updateFormDataOutput(dataObject) {
        // Set the values of the widgets inside the live form (other than form fields) in form data
        _.forEach(this.ngform.value, (val, key) => {
            if (!_.find(this.formFields, { key })) {
                dataObject[key] = val;
            }
        });
        this.dataoutput = dataObject;
    }
    // Construct the data object merging the form fields and custom widgets data
    constructDataObject() {
        const formData = {};
        const formFields = this.getFormFields();
        // Get all form fields and prepare form data as key value pairs
        formFields.forEach(field => {
            let fieldName, fieldValue;
            fieldValue = field.datavalue || field._control.value;
            fieldValue = (fieldValue === null || fieldValue === '') ? undefined : fieldValue;
            if (field.type === 'file') {
                fieldValue = getFiles(this.name, field.key + '_formWidget', field.multiple);
            }
            fieldName = field.key || field.target || field.name;
            // on current Form, when field is innerForm field then we group these fields under new object using formGroupName as key
            // For suppose, applicantName, applicantID are formfields in currentForm
            // This form is having innerform i.e. address, these fields are listed as
            // {"applicantName": "", "applicantID": "", "address": {"street": ""}}
            if (field.form.widgetId !== this.widgetId && field.form.formGroupName) {
                const fd = _.get(formData, field.form.formGroupName, {});
                fd[fieldName] = fieldValue;
                _.set(formData, field.form.formGroupName, fd);
            }
            else if (field.form.isParentList) {
                // setting formdata based on formArrayIndex
                const fd = _.get(formData, field.form.parentList.name, []);
                fd[field.form.formArrayIndex] = fd[field.form.formArrayIndex] || {};
                fd[field.form.formArrayIndex][fieldName] = fieldValue;
                _.set(formData, field.form.parentList.name, fd);
            }
            else {
                // In case of update the field will be already present in form data
                _.set(formData, fieldName, fieldValue);
            }
        });
        this.updateFormDataOutput(formData);
        return this.dataoutput;
    }
    updateDataOutput() {
        this.constructDataObject();
        if (this.ngform.touched) {
            this.setValidationMsgs(true);
        }
    }
    // FormFields will contain all the fields in parent and inner form also.
    // This returns the formFields in the form based on the form name.
    getFormFields() {
        return _.filter(this.formFields, formField => {
            return formField.form.name === this.name;
        });
    }
    setFieldValue(field, data, innerFormdata) {
        const key = field.key || field.name;
        // if customfield param value is not in the formdata then do not assign field value
        // as it can contain default value which will again be overridden by undefined.
        const fd = innerFormdata ? innerFormdata : data;
        if (fd) {
            if (fd.hasOwnProperty(key)) {
                field.value = _.get(fd, key);
            }
            else if (_.includes(key, '.')) {
                // key contains '.' when mapping the fields to child reference i.e. childCol is having key as "parent.childCol"
                if (fd.hasOwnProperty(_.split(key, '.')[0])) {
                    field.value = _.get(fd, key);
                }
            }
        }
        const formGroupName = field.form.formGroupName;
        /**
         * if formGroupName is defined which means field is inside the inner form
         * then set the formdata on the field's form using formGroupName
         */
        if (formGroupName && _.get(data, formGroupName)) {
            this.setFormData.call(field.form, data[formGroupName]);
        }
        // if formdata is assigned later then on propertyChangeHandler, even inner forms data also needs to be updated.
        if (_.get(field.form, 'parentFormArray')) {
            this.setFormDataFromParentFormData.call(field.form, data);
        }
    }
    setFormData(data, innerFormFields, innerFormdata) {
        // whereas formFields explicitly passed can contain innerform fields also.
        const formFields = innerFormFields || this.formFields;
        formFields.forEach(field => {
            this.setFieldValue(field, data, innerFormdata);
        });
        this.constructDataObject();
    }
    resetFormState() {
        // clearing the validationMessages on reset.
        if (this.validationMessages.length) {
            this.validationMessages = [];
        }
        if (!this.ngform) {
            return;
        }
        this.ngform.markAsUntouched();
        this.ngform.markAsPristine();
    }
    reset() {
        this.resetFormState();
        this.formFields.forEach(field => {
            field.value = '';
            if (field.widgettype === 'upload' || field.type === DataType.BLOB) {
                this.resetFileUploadWidget(field, true);
            }
        });
        this.constructDataObject();
        this.clearMessage();
    }
    savePrevformFields() {
        this.prevformFields = getClonedObject(this.formFields.map(field => {
            return {
                'key': field.key,
                'type': field.type,
                'widgettype': field.widgettype,
                'outputformat': field.outputformat,
                'value': field.value
            };
        }));
    }
    savePrevDataValues() {
        this.prevDataValues = this.formFields.map((obj) => {
            return { 'key': obj.key, 'value': obj.value };
        });
    }
    setReadonlyFields() {
        this.formFields.forEach(field => {
            field.setReadOnlyState();
        });
    }
    resetFileUploadWidget(field, skipValueSet) {
        const $formEle = this.$element;
        $formEle.find('[name="' + field.key + '_formWidget"]').val('');
        field._control.reset();
        if (!skipValueSet) {
            field.href = '';
            field.value = null;
        }
    }
    emptyDataModel() {
        this.formFields.forEach(field => {
            if (isDefined(field)) {
                if (field.type === DataType.BLOB) {
                    this.resetFileUploadWidget(field);
                }
                else {
                    field.datavalue = '';
                }
            }
        });
    }
    setDefaultValues() {
        this.formFields.forEach(field => {
            this.onFieldDefaultValueChange(field, field.defaultvalue);
        });
    }
    setOperationType(mode) {
        this.operationType = mode || 'insert';
    }
    new() {
        this.resetFormState();
        this.operationType = Live_Operations.INSERT;
        this.clearMessage();
        if (this.isSelected) {
            this.savePrevformFields();
        }
        this.emptyDataModel();
        setTimeout(() => {
            this.setDefaultValues();
            this.savePrevDataValues();
            this.constructDataObject();
        });
        this.isUpdateMode = true;
    }
    edit() {
        this.resetFormState();
        this.clearMessage();
        this.operationType = Live_Operations.UPDATE;
        if (this.isSelected) {
            this.savePrevformFields();
            this.savePrevDataValues();
        }
        this.prevDataObject = getClonedObject(this.formdata || {});
        this.setReadonlyFields();
        this.isUpdateMode = true;
        $appDigest();
    }
    triggerWMEvent(eventName) {
        const tableId = this.nativeElement.getAttribute('dependson');
        if (!tableId) {
            return;
        }
        $invokeWatchers(true);
        this.app.notify('wm-event', { eventName, widgetName: tableId });
    }
    submitForm($event) {
        let template, formData, result;
        const dataSource = this.datasource;
        // Disable the form submit if form is in invalid state.
        if (this.validateFieldsOnSubmit()) {
            return;
        }
        formData = getClonedObject(this.constructDataObject());
        const getParams = () => {
            return { $event, $formData: formData, $data: formData };
        };
        if (this.onBeforeSubmitEvt) {
            if (this.invokeEventCallback('beforesubmit', getParams()) === false) {
                return;
            }
            else {
                this.resetFormState();
            }
        }
        else {
            this.resetFormState();
        }
        if (this.onSubmitEvt || dataSource) {
            // If on submit is there execute it and if it returns true do service variable invoke else return
            // If its a service variable call setInput and assign form data and invoke the service
            if (dataSource) {
                const currentPageNum = dataSource.pagination && dataSource.pagination.number + 1;
                const operationType = this.operationType ? this.operationType : 'insert';
                performDataOperation(dataSource, formData, { operationType: operationType })
                    .then((data) => {
                    if (dataSource.category === 'wm.CrudVariable') {
                        this.triggerWMEvent('resetEditMode');
                        this.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                            'skipToggleState': true,
                            'operation': 'list',
                            'page': currentPageNum,
                            'preventMissingOpMsg': true
                        });
                        if (this.dialogId) {
                            this.closeDialog();
                        }
                    }
                    if (isDefined(data.body)) {
                        result = getValidJSON(data.body);
                        result = isDefined(result) ? result : data.body;
                    }
                    else {
                        result = data;
                    }
                    return {
                        'result': result,
                        'status': true,
                        'message': this.postmessage,
                        'type': 'success'
                    };
                }, (error) => {
                    template = this.errormessage || error.error || error;
                    $appDigest();
                    return {
                        'result': _.get(error, 'error') || error,
                        'status': false,
                        'message': template,
                        'type': 'error'
                    };
                }).then(response => {
                    if ((response.type === 'error' && !this.onError) || (response.type === 'success' && !this.onSuccess)) {
                        this.toggleMessage(true, response.message, response.type);
                    }
                    this.invokeEventCallback('submit', getParams());
                    this.onResultCb(response.result, response.status, $event);
                });
            }
            else {
                this.invokeEventCallback('submit', getParams());
                this.onResultCb({}, true, $event);
            }
        }
        else {
            this.onResultCb({}, true, $event);
        }
    }
    closeDialog() {
        this.dialogService.close(this.dialogId);
    }
    // Method to show/hide the panel header or footer based on the buttons
    showButtons(position) {
        return _.some(this.buttonArray, btn => {
            return _.includes(btn.position, position) && btn.updateMode === this.isUpdateMode;
        });
    }
    // Expand or collapse the panel
    expandCollapsePanel() {
        if (this.collapsible) {
            // flip the active flag
            this.expanded = !this.expanded;
        }
    }
    onDataSourceUpdate(response, newForm, updateMode) {
        if (newForm) {
            this.new();
        }
        else {
            this.setFormData(response);
            this.closeDialog();
        }
        this.isUpdateMode = isDefined(updateMode) ? updateMode : true;
    }
    /**
     * This method tranverses through the parent containing formdata and returns the parent formdata.
     * @param form, represents the current form instance
     * @returns {any}
     */
    getNearestParentFormData(form) {
        if (!form.parentForm) {
            return {};
        }
        if (!form.bindformdata && _.get(form.parentForm, 'formdata') && !_.isEmpty(form.parentForm.formdata)) {
            return form.parentForm.formdata;
        }
        return this.getNearestParentFormData(form.parentForm);
    }
    setFormDataFromParentFormData(formdata) {
        if (_.isEmpty(this.formdata) && this.parentForm && formdata) {
            /**
             * If form is inside list,
             * 1. directly applying the formdata to the formArray to update the formdata on list of forms using patchValue.
             * 2. and also setting individual formdata to the form inside each item based on formArrayIndex, when form is not bound to formdata.
             * 3. explicitly invoking formdata as formFields will contain all the formfields including innerForm's fields
             * (if we do not invoke explicitly then formFields will just contain the fields inside current form and not inner form fields)
             * 4. if childFormArrayData is not available we will check for inner form data using "formGroupName" and will set the innerform data
             * If not inside list, just set the formdata on all the formfields including innerForm fields
             */
            if (this.parentFormArray) {
                let childFormArrayData = _.get(formdata, this.parentList.name);
                if (childFormArrayData) {
                    if (!_.isArray(childFormArrayData)) {
                        childFormArrayData = [childFormArrayData];
                    }
                    this.parentFormArray.patchValue(childFormArrayData);
                    if (!this.bindformdata) {
                        this.formdata = childFormArrayData[this.formArrayIndex];
                    }
                    this.setFormData(this.formdata);
                }
                else if (this.isParentList && this.parentForm.formGroupName) {
                    const parentFormData = _.get(formdata, this.parentForm.formGroupName);
                    this.setFormDataFromParentFormData(parentFormData);
                }
            }
            else {
                this.setFormData(formdata);
            }
        }
    }
    // On form data source change. This method is overridden by live form and live filter
    onDataSourceChange() {
        if (_.get(this.formFields, 'length') && !this.bindformdata) {
            this.isDataSourceUpdated = true;
            /**
             * formdata on the parent form will be set before the inner forms are rendered.
             * Hence handling formdata on the innerForms, which might be rendered slowly (suppose prefab with form)
             * setting inner form's formdata obtained from parent formdata
             * this applies only when formdata is not given on the inner form.
             */
            const formdata = this.getNearestParentFormData(this);
            this.setFormDataFromParentFormData(formdata);
        }
    }
    // On form data source change. This method is overridden by live form and live filter
    onFormDataSourceChange() {
        this.updateFieldSource();
    }
    // On form field default value change. This method is overridden by live form and live filter
    onFieldDefaultValueChange(field, nv) {
        field.value = parseValueByType(nv, undefined, field.widgettype, field.trailingzero);
    }
    // On form field value change. This method is overridden by live form and live filter
    onFieldValueChange() {
    }
    // Function to generate and compile the form fields from the metadata
    generateFormFields() {
        return __awaiter(this, void 0, void 0, function* () {
            let noOfColumns;
            let $gridLayout;
            // Check if grid layout is present or not for first time
            if (_.isUndefined(this._isGridLayoutPresent)) {
                this._isGridLayoutPresent = this.$element.find('.panel-body [wmlayoutgrid]').length > 0;
            }
            if (this._isGridLayoutPresent) {
                $gridLayout = this.$element.find('.form-elements [wmlayoutgrid]').first();
                noOfColumns = Number($gridLayout.attr('columns')) || 1;
            }
            else {
                $gridLayout = this.$element.find('.form-elements .dynamic-form-container');
                if (!$gridLayout.length) {
                    this.$element.find('.form-elements').prepend('<div class="dynamic-form-container"></div>');
                    $gridLayout = this.$element.find('.form-elements .dynamic-form-container');
                }
                noOfColumns = 1;
            }
            const columnWidth = 12 / noOfColumns;
            let fieldTemplate = '';
            let colCount = 0;
            let index;
            let userFields;
            let fields = this.metadata ? this.metadata.data || this.metadata : [];
            this.formFields = []; // empty the form fields
            if (_.isEmpty(fields)) {
                return;
            }
            if (this.onBeforeRenderEvt) {
                userFields = this.invokeEventCallback('beforerender', { $metadata: fields });
                if (userFields) {
                    fields = userFields;
                }
            }
            if (!_.isArray(fields)) {
                return;
            }
            while (fields[colCount]) {
                let colTmpl = '';
                for (index = 0; index < noOfColumns; index++) {
                    if (fields[colCount]) {
                        colTmpl += setMarkupForFormField(fields[colCount], columnWidth);
                    }
                    colCount++;
                }
                fieldTemplate += `<wm-gridrow>${colTmpl}</wm-gridrow>`;
            }
            if (!this._isGridLayoutPresent) {
                fieldTemplate = `<wm-layoutgrid>${fieldTemplate}</wm-layoutgrid>`;
            }
            this.dynamicFormRef.clear();
            if (!this._dynamicContext) {
                this._dynamicContext = Object.create(this.viewParent);
                this._dynamicContext.form = this;
            }
            const componentFactoryRef = yield this.dynamicComponentProvider.getComponentFactoryRef('app-form-dynamic-' + this.widgetId, fieldTemplate, {
                noCache: true,
                transpile: true
            });
            const component = this.dynamicFormRef.createComponent(componentFactoryRef, 0, this.inj);
            extendProto(component.instance, this._dynamicContext);
            $gridLayout[0].appendChild(component.location.nativeElement);
            this.setFormData(this.formdata);
        });
    }
    get mode() {
        return this.operationType || this.findOperationType();
    }
    get dirty() {
        return this.ngform && this.ngform.dirty;
    }
    /**
     * This method sets the form state to pristine by internally calling angular markAsPristine method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    markAsPristine(value = false) {
        this.ngform.markAsPristine({ onlySelf: value });
    }
    /**
     * This method sets the form state to dirty by internally calling angular markAsDirty method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    markAsDirty(value = false) {
        this.ngform.markAsDirty({ onlySelf: value });
    }
    get invalid() {
        return this.ngform && this.ngform.invalid;
    }
    get touched() {
        return this.ngform && this.ngform.touched;
    }
    get valid() {
        return this.ngform && this.ngform.valid;
    }
    get pristine() {
        return this.ngform && this.ngform.pristine;
    }
    /**
     * This method sets the form state to touched by internally calling angular markAsTouched method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    markAsTouched(value = false) {
        this.ngform.markAsTouched({ onlySelf: value });
    }
    /**
     * This method sets the form state to untouched by internally calling angular markAsUntouched method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    markAsUntouched(value = false) {
        this.ngform.markAsUntouched({ onlySelf: value });
    }
    invokeActionEvent($event, expression) {
        const fn = $parseEvent(expression);
        fn(this.viewParent, Object.assign(this.context, { $event }));
    }
    ngOnDestroy() {
        // on form destroy, removing this form from the parentForm too.
        const controls = this.parentForm && _.get(this.parentForm, 'ngform.controls');
        if (controls) {
            // when current form is inside the list (i.e. incase of formArray).
            if (_.get(this.parentList, 'name') && controls.hasOwnProperty(this.parentList.name)) {
                this.parentForm.ngform.removeControl(this.parentList.name);
            }
            // when we have formGroupName set i.e. multiple formInstance with counter appended to formName
            if (this.formGroupName && controls.hasOwnProperty(this.formGroupName)) {
                this.parentForm.ngform.removeControl(this.formGroupName);
            }
            else {
                this.parentForm.ngform.removeControl(this.name);
            }
        }
        super.ngOnDestroy();
    }
};
FormComponent.initializeProps = registerFormProps();
FormComponent.ctorParameters = () => [
    { type: Injector },
    { type: FormBuilder },
    { type: App },
    { type: AbstractDialogService },
    { type: DynamicComponentRefProvider },
    { type: NgZone },
    { type: ListComponent, decorators: [{ type: Optional }] },
    { type: FormComponent, decorators: [{ type: SkipSelf }, { type: Optional }] },
    { type: PrefabDirective, decorators: [{ type: SkipSelf }, { type: Optional }] },
    { type: PartialDirective, decorators: [{ type: SkipSelf }, { type: Optional }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['beforesubmit.event',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['submit.event',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['beforerender.event',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['formdata.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['wmLiveForm',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['wmLiveFilter',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['role',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['key',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['name',] }] }
];
__decorate([
    ViewChild('dynamicForm', { static: true, read: ViewContainerRef }),
    __metadata("design:type", ViewContainerRef)
], FormComponent.prototype, "dynamicFormRef", void 0);
__decorate([
    ViewChild(MessageComponent),
    __metadata("design:type", Object)
], FormComponent.prototype, "messageRef", void 0);
__decorate([
    ContentChildren(WidgetRef, { descendants: true }),
    __metadata("design:type", Object)
], FormComponent.prototype, "componentRefs", void 0);
__decorate([
    HostBinding('action'),
    __metadata("design:type", String)
], FormComponent.prototype, "action", void 0);
__decorate([
    HostListener('submit', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], FormComponent.prototype, "submit", null);
__decorate([
    HostListener('reset'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FormComponent.prototype, "onReset", null);
FormComponent = FormComponent_1 = __decorate([
    Component({
        selector: 'form[wmForm]',
        template: "<ng-template #content><ng-content></ng-content></ng-template>\n\n<ng-container *ngIf=\"formlayout === 'page'; then pageLayoutTemplate else defaultTemplate\">\n</ng-container>\n\n<ng-template #defaultTemplate>\n    <div class=\"panel-heading\" *ngIf=\"!isLayoutDialog && (title || subheading || iconclass || showButtons('header'))\">\n        <h3 class=\"panel-title\">\n            <div class=\"pull-left\">\n                <i class=\"app-icon panel-icon\" aria-hidden=\"true\" [ngClass]=\"iconclass\" *ngIf=\"iconclass\"></i>\n            </div>\n            <div class=\"pull-left\">\n                <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\"></div>\n                <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n            </div>\n            <div class=\"form-action panel-actions basic-btn-grp\">\n                <ng-container *ngFor=\"let btn of buttonArray | filter : 'position' : 'header'\"\n                              [ngTemplateOutlet]=\"btn.widgetType === 'button' ? buttonRef : anchorRef\" [ngTemplateOutletContext]=\"{btn:btn}\">\n                </ng-container>\n                <button type=\"button\" class=\"app-icon wi panel-action\" *ngIf=\"collapsible\" title=\"Collapse/Expand\"\n                        [ngClass]=\"expanded ? 'wi-minus': 'wi-plus'\" (click)=\"expandCollapsePanel();\"></button>\n            </div>\n        </h3>\n    </div>\n    <div class=\"panel-body\" [ngClass]=\"{'form-elements': isLayoutDialog, 'form-body': !isLayoutDialog}\" [class.hidden]=\"!expanded\">\n        <p wmMessage [hidden]=\"!statusMessage.caption.toString()\" caption.bind=\"statusMessage.caption\" type.bind=\"statusMessage.type\"></p>\n        <div [ngClass]=\"{'form-elements': !isLayoutDialog, 'form-content': isLayoutDialog}\">\n            <ng-container *ngTemplateOutlet=\"content\"></ng-container>\n        </div>\n        <div class=\"basic-btn-grp form-action panel-footer clearfix\" [ngClass]=\"{'modal-footer': isLayoutDialog}\" [class.hidden]=\"!expanded || !showButtons('footer')\">\n            <ng-container *ngFor=\"let btn of buttonArray | filter : 'position' : 'footer'\"\n                          [ngTemplateOutlet]=\"btn.widgetType === 'button' ? buttonRef : anchorRef\" [ngTemplateOutletContext]=\"{btn:btn}\">\n            </ng-container>\n        </div>\n    </div>\n</ng-template>\n\n<ng-template #pageLayoutTemplate>\n    <ng-container *ngTemplateOutlet=\"content\"></ng-container>\n</ng-template>\n\n<ng-template #buttonRef let-btn=\"btn\">\n    <button  wmButton name=\"{{btn.key}}\" caption.bind=\"btn.displayName\" class.bind=\"btn.class\" iconclass.bind=\"btn.iconclass\" show.bind=\"btn.show\"\n             (click)=\"invokeActionEvent($event, btn.action)\" type.bind=\"btn.type\" hint.bind=\"btn.title\" shortcutkey.bind=\"btn.shortcutkey\" disabled.bind=\"ngform.status === 'PENDING' || btn.disabled\"\n             tabindex.bind=\"btn.tabindex\" [class.hidden]=\"btn.updateMode ? !isUpdateMode : isUpdateMode\"></button>\n</ng-template>\n\n<ng-template #anchorRef let-btn=\"btn\">\n    <a wmAnchor name=\"{{btn.key}}\" caption.bind=\"btn.displayName\" class.bind=\"btn.class\" iconclass.bind=\"btn.iconclass\" show.bind=\"btn.show\"\n             (click)=\"invokeActionEvent($event, btn.action)\" hint.bind=\"btn.title\" shortcutkey.bind=\"btn.shortcutkey\"\n             hyperlink.bind=\"btn.hyperlink\" target.bind=\"btn.target\"\n             tabindex.bind=\"btn.tabindex\" [class.hidden]=\"btn.updateMode ? !isUpdateMode : isUpdateMode\"></a>\n</ng-template>\n\n<ng-container #dynamicForm></ng-container>\n",
        providers: [
            provideAsWidgetRef(FormComponent_1)
        ]
    }),
    __param(6, Optional()),
    __param(7, SkipSelf()), __param(7, Optional()),
    __param(8, SkipSelf()), __param(8, Optional()),
    __param(9, SkipSelf()), __param(9, Optional()),
    __param(10, Attribute('beforesubmit.event')),
    __param(11, Attribute('submit.event')),
    __param(12, Attribute('beforerender.event')),
    __param(13, Attribute('dataset.bind')),
    __param(14, Attribute('formdata.bind')),
    __param(15, Attribute('wmLiveForm')),
    __param(16, Attribute('wmLiveFilter')),
    __param(17, Attribute('role')),
    __param(18, Attribute('key')),
    __param(19, Attribute('name')),
    __metadata("design:paramtypes", [Injector,
        FormBuilder,
        App,
        AbstractDialogService,
        DynamicComponentRefProvider,
        NgZone,
        ListComponent,
        FormComponent,
        PrefabDirective,
        PartialDirective, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object])
], FormComponent);

let FormWidgetDirective = class FormWidgetDirective {
    constructor(form, table, componentInstance, fb, name, key) {
        this.componentInstance = componentInstance;
        this.name = name;
        this.key = key;
        this.fb = fb;
        this.parent = form || table;
        this.ngform = this.parent.ngform;
        this.componentInstance.registerPropertyChangeListener((k, nv) => {
            if (k === 'datavalue' && this._control) {
                this._control.setValue(nv);
            }
            else if (k === 'name' && !this._control) {
                this.addControl(this.key || nv);
            }
        });
    }
    get _control() {
        const fieldName = this.key || this.name;
        if (!fieldName) {
            return undefined;
        }
        return this.ngform && this.ngform.controls[fieldName];
    }
    addControl(fieldName) {
        this.ngform.addControl(fieldName, this.createControl());
    }
    createControl() {
        return this.fb.control(this.componentInstance.datavalue);
    }
    ngOnInit() {
        const fieldName = this.key || this.name;
        if (fieldName && !this._control) {
            this.addControl(fieldName);
            this.parent.registerFormWidget(this.componentInstance);
        }
    }
};
FormWidgetDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [FormComponent,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [TableComponent,] }] },
    { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [WidgetRef,] }] },
    { type: FormBuilder },
    { type: undefined, decorators: [{ type: Attribute, args: ['name',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['key',] }] }
];
FormWidgetDirective = __decorate([
    Directive({
        selector: '[wmFormWidget]'
    }),
    __param(0, Optional()), __param(0, Inject(FormComponent)),
    __param(1, Optional()), __param(1, Inject(TableComponent)),
    __param(2, Self()), __param(2, Inject(WidgetRef)),
    __param(4, Attribute('name')),
    __param(5, Attribute('key')),
    __metadata("design:paramtypes", [Object, Object, Object, FormBuilder, Object, Object])
], FormWidgetDirective);

const registerProps = () => {
    register('wm-form-action', new Map([
        ['accessroles', PROP_STRING],
        ['action', PROP_STRING],
        ['class', PROP_STRING],
        ['binding', PROP_STRING],
        ['disabled', PROP_BOOLEAN],
        ['display-name', PROP_STRING],
        ['hyperlink', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconname', PROP_STRING],
        ['key', PROP_STRING],
        ['position', Object.assign({ value: 'footer' }, PROP_STRING)],
        ['shortcutkey', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['target', PROP_STRING],
        ['title', PROP_STRING],
        ['type', Object.assign({ value: 'button' }, PROP_STRING)],
        ['update-mode', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['widget-type', Object.assign({ value: 'button' }, PROP_STRING)]
    ]));
};

var FormActionDirective_1;
const WIDGET_CONFIG$1 = { widgetType: 'wm-form-action', hostClass: '' };
let FormActionDirective = FormActionDirective_1 = class FormActionDirective extends BaseComponent {
    constructor(inj, form) {
        super(inj, WIDGET_CONFIG$1);
        this.form = form;
    }
    populateAction() {
        this.buttonDef = {
            key: this.key || this.binding,
            displayName: this['display-name'],
            show: this.show,
            class: this.class ? this.class : (this['widget-type'] === 'button' ? 'btn-secondary' : ''),
            iconclass: this.iconclass || '',
            title: _.isUndefined(this.title) ? (this['display-name'] || '') : this.title,
            action: this.action,
            accessroles: this.accessroles,
            shortcutkey: this.shortcutkey,
            disabled: this.disabled,
            tabindex: this.tabindex,
            iconname: this.iconname,
            type: this.type,
            updateMode: this['update-mode'],
            position: this.position,
            widgetType: this['widget-type'],
            hyperlink: this.hyperlink,
            target: this.target
        };
        this._propsInitialized = true;
    }
    ngOnInit() {
        super.ngOnInit();
        this.populateAction();
        this.form.registerActions(this.buttonDef);
    }
    onPropertyChange(key, nv, ov) {
        if (!this._propsInitialized) {
            return;
        }
        switch (key) {
            case 'display-name':
                this.buttonDef.displayName = nv;
            default:
                this.buttonDef[key] = nv;
                break;
        }
    }
};
FormActionDirective.initializeProps = registerProps();
FormActionDirective.ctorParameters = () => [
    { type: Injector },
    { type: FormComponent, decorators: [{ type: Optional }] }
];
FormActionDirective = FormActionDirective_1 = __decorate([
    Directive({
        selector: '[wmFormAction]',
        providers: [
            provideAsWidgetRef(FormActionDirective_1)
        ]
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [Injector, FormComponent])
], FormActionDirective);

const registerProps$1 = () => {
    const formFieldMap = new Map([
        ['dataentrymode', Object.assign({ value: 'default' }, PROP_STRING)],
        ['debouncetime', Object.assign({ value: 250 }, PROP_NUMBER)],
        ['defaultvalue', PROP_STRING],
        ['displayname', PROP_STRING],
        ['display-name', PROP_STRING],
        ['field', PROP_STRING],
        ['filterexpressions', PROP_STRING],
        ['filter-on', PROP_STRING],
        ['generator', PROP_STRING],
        ['hint', PROP_STRING],
        ['inputtype', PROP_STRING],
        ['is-primary-key', PROP_BOOLEAN],
        ['is-range', PROP_BOOLEAN],
        ['is-related', PROP_BOOLEAN],
        ['isformfield', { value: true }],
        ['key', PROP_STRING],
        ['limit', PROP_NUMBER],
        ['lookup-type', PROP_STRING],
        ['lookup-field', PROP_STRING],
        ['name', PROP_STRING],
        ['matchmode', PROP_STRING],
        ['maxdefaultvalue', PROP_STRING],
        ['maxplaceholder', PROP_STRING],
        ['mobile-display', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['period', PROP_BOOLEAN],
        ['pc-display', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['placeholder', PROP_STRING],
        ['primary-key', PROP_BOOLEAN],
        ['related-entity-name', PROP_STRING],
        ['required', PROP_BOOLEAN],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tablet-display', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['type', PROP_STRING],
        ['validationmessage', PROP_STRING],
        ['viewmodewidget', PROP_STRING],
        ['widgettype', PROP_STRING]
    ]);
    for (const key in FormWidgetType) {
        const widgetName = 'wm-form-field-' + FormWidgetType[key];
        const widgetProps = getWidgetPropsByType(widgetName);
        formFieldMap.forEach((v, k) => {
            if (widgetProps.get(k) == undefined) {
                widgetProps.set(k, v);
            }
        });
    }
};

var FormFieldDirective_1;
// Custom validator to show validation error, if setValidationMessage method is used
const customValidatorFn = () => {
    return { custom: true };
};
const ɵ0$1 = customValidatorFn;
const FILE_TYPES = {
    'image': 'image/*',
    'video': 'video/*',
    'audio': 'audio/*'
};
const ɵ1$1 = {};
let FormFieldDirective = FormFieldDirective_1 = class FormFieldDirective extends StylableComponent {
    constructor(inj, form, fb, viewport, parentList, bindChipclass, binddataset, binddisplayexpression, binddisplaylabel, _widgetType, name, binddisplayimagesrc, key, isRange, pcDisplay, mobileDisplay, tabletDisplay, contexts) {
        const WIDGET_CONFIG = {
            widgetType: 'wm-form-field',
            hostClass: '',
            widgetSubType: 'wm-form-field-' + (_widgetType || FormWidgetType.TEXT).trim()
        };
        super(inj, WIDGET_CONFIG, new Promise(res => this._initPropsRes = res));
        this.fieldDefConfig = {};
        this.class = '';
        this.binddataset = binddataset;
        this.binddisplayimagesrc = binddisplayimagesrc;
        this.binddisplayexpression = binddisplayexpression;
        this.bindChipclass = bindChipclass;
        this.binddisplaylabel = binddisplaylabel;
        this.form = form;
        this.fb = fb;
        this.viewport = viewport;
        this._fieldName = key || name;
        this.isRange = isRange;
        this.excludeProps = new Set(['type', 'name']);
        this.widgettype = _widgetType;
        this.parentList = parentList;
        this.notifyForFields = [];
        if (this.binddataset || this.$element.attr('dataset')) {
            this.isDataSetBound = true;
        }
        contexts[0]._onFocusField = this._onFocusField.bind(this);
        contexts[0]._onBlurField = this._onBlurField.bind(this);
        this._debounceSetUpValidators = debounce(() => this.setUpValidators(), 500);
    }
    _onFocusField($evt) {
        this._activeField = true;
        $($evt.target).closest('.live-field').addClass('active');
    }
    _onBlurField($evt) {
        $($evt.target).closest('.live-field').removeClass('active');
        this._activeField = false;
    }
    // Expression to be evaluated in view mode of form field
    evaluateExpr(object, displayExpr) {
        if (!displayExpr) {
            displayExpr = Object.keys(object)[0];
            // If dataset is not ready, display expression will not be defined
            if (!displayExpr) {
                return;
            }
        }
        return getEvaluatedData(object, {
            expression: displayExpr
        }, this.viewParent);
    }
    // Expression to be evaluated in view mode of form field
    getDisplayExpr() {
        const caption = [];
        const value = this.value;
        const displayExpr = this.displayexpression || this.displayfield || this.displaylabel;
        if (_.isObject(value)) {
            if (_.isArray(value)) {
                _.forEach(value, obj => {
                    if (_.isObject(obj)) {
                        caption.push(this.evaluateExpr(obj, displayExpr));
                    }
                });
            }
            else {
                caption.push(this.evaluateExpr(value, displayExpr));
            }
            return _.join(caption, ',');
        }
        return (value === undefined || value === null) ? '' : this.value;
    }
    getCaption() {
        return (this.value === undefined || this.value === null) ? (_.get(this.form.dataoutput, this._fieldName) || '') : this.value;
    }
    // Notifies changes to observing validation fields
    notifyChanges() {
        _.forEach(this.notifyForFields, field => {
            field.fieldValidations.validate();
        });
    }
    // Registers observer of validation fields
    observeOn(fields) {
        this.fieldValidations.observeOn(fields, 'formfields');
    }
    // Method to setup validators for reactive form control
    setUpValidators(customValidator) {
        this.fieldValidations.setUpValidators(customValidator);
    }
    // sets the custom async validation on the form field
    setAsyncValidators(validators) {
        this.fieldValidations.setAsyncValidators(validators);
    }
    // sets the default and custom sync validation on the form field
    setValidators(validators) {
        this.fieldValidations.setValidators(validators);
    }
    // Method to set the properties on inner form widget
    setFormWidget(key, val) {
        if (this.formWidget && this.formWidget.widget) {
            this.formWidget.widget[key] = val;
        }
    }
    boundFn(fn) {
        return fn();
    }
    // Method to set the properties on inner max form widget (when range is selected)
    setMaxFormWidget(key, val) {
        if (this.formWidgetMax) {
            this.formWidgetMax.widget[key] = val;
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key !== 'tabindex') {
            super.onPropertyChange(key, nv, ov);
        }
        if (this.excludeProps.has(key)) {
            return;
        }
        // As upload widget is an HTML widget, only required property is setup
        if (this.widgettype === FormWidgetType.UPLOAD) {
            if (key === 'required') {
                this._debounceSetUpValidators();
            }
            super.onPropertyChange(key, nv, ov);
            return;
        }
        this.setFormWidget(key, nv);
        // Placeholder should not be setup on max widget
        if (key !== 'placeholder') {
            this.setMaxFormWidget(key, nv);
        }
        switch (key) {
            case 'defaultvalue':
                this.form.onFieldDefaultValueChange(this, nv);
                break;
            case 'maxdefaultvalue':
                this.maxValue = nv;
                this.setMaxFormWidget('datavalue', nv);
                this.form.onMaxDefaultValueChange();
                break;
            case 'maxplaceholder':
                this.setMaxFormWidget('placeholder', nv);
                break;
            case 'required':
            case 'maxchars':
            case 'minvalue':
            case 'maxvalue':
            case 'regexp':
            case 'show':
                this._debounceSetUpValidators();
                break;
            case 'primary-key':
                if (nv) {
                    this.form.setPrimaryKey(this._fieldName);
                }
                break;
            case 'display-name':
                this.displayname = nv;
                break;
            case 'readonly':
                this.setReadOnlyState();
                break;
        }
    }
    onStyleChange(key, nv, ov) {
        this.setFormWidget(key, nv);
        this.setMaxFormWidget(key, nv);
        super.onStyleChange(key, nv, ov);
    }
    get datavalue() {
        return this.formWidget && this.formWidget.datavalue;
    }
    set datavalue(val) {
        if (this._control && this.widgettype !== FormWidgetType.UPLOAD) {
            this._control.setValue(val);
        }
    }
    get value() {
        return this.datavalue;
    }
    set value(val) {
        this.datavalue = val;
    }
    get maxValue() {
        return this.formWidgetMax && this.formWidgetMax.datavalue;
    }
    set maxValue(val) {
        if (this._maxControl) {
            this._maxControl.setValue(val);
        }
    }
    get minValue() {
        return this.value;
    }
    set minValue(val) {
        this.value = val;
    }
    // Get the reactive form control
    get _control() {
        return this.ngform && this.ngform.controls[this._fieldName];
    }
    // Get the reactive max form control
    get _maxControl() {
        return this.ngform && this.ngform.controls[this._fieldName + '_max'];
    }
    // Get the displayValue
    get displayValue() {
        return this.formWidget && this.formWidget.displayValue;
    }
    //Get the timestamp
    get timestamp() {
        return this.formWidget && this.formWidget.timestamp;
    }
    // Create the reactive form control
    createControl() {
        return this.fb.control(undefined, {
            validators: this._validators
        });
    }
    // On field value change, propagate event to parent form
    onValueChange(val) {
        if (!this.isDestroyed) {
            this.form.onFieldValueChange(this, val);
            this.notifyChanges();
            if (this.$element.find('.active').length > 0 || this.form.touched) {
                this.ngform.controls[this._fieldName].markAsTouched();
                this.fieldValidations.setCustomValidationMessage();
            }
        }
    }
    onStatusChange(status) {
        if (!this.isDestroyed) {
            this.showPendingSpinner = (status === 'PENDING');
            // while running validation, widget is disabled and spinner is shown
            // otherwise formWidget disabled state is reset to the state of the formField.
            if (status === 'PENDING') {
                this.formWidget.disabled = true;
            }
            else if (this.formWidget.disabled !== this.disabled) {
                this.formWidget.disabled = this.disabled;
            }
        }
    }
    // Method to expose validation message and set control to invalid
    setValidationMessage(val) {
        setTimeout(() => {
            this.validationmessage = val;
            this.setUpValidators(customValidatorFn);
        });
    }
    setReadOnlyState() {
        let readOnly;
        if (this.form.isUpdateMode) {
            if (this['primary-key'] && !this['is-related'] && !this.period) {
                /*If the field is primary but is assigned set readonly false.
                   Assigned is where the user inputs the value while a new entry.
                   This is not editable(in update mode) once entry is successful*/
                readOnly = !(this.generator === 'assigned' && this.form.operationType !== 'update');
            }
            else {
                readOnly = this.readonly;
            }
        }
        else {
            // In view mode, set widget state to readonly always
            readOnly = true;
        }
        this.setFormWidget('readonly', readOnly);
    }
    resetDisplayInput() {
        if ((!isDefined(this.value) || this.value === '')) {
            this.formWidget && this.formWidget.resetDisplayInput && this.formWidget.resetDisplayInput();
        }
    }
    triggerUploadEvent($event, eventName) {
        const params = { $event };
        if (eventName === 'change') {
            params.newVal = $event.target.files;
            params.oldVal = this._oldUploadVal;
            this._oldUploadVal = params.newVal;
        }
        this.invokeEventCallback(eventName, params);
    }
    registerFormField() {
        const fieldName = this._fieldName;
        if (this.parentList && !(this.form.parentList === this.parentList)) {
            let counter = 1;
            let _fieldName = fieldName;
            while (this.ngform.controls.hasOwnProperty(_fieldName)) {
                _fieldName = `${fieldName}_${counter}`;
                counter++;
            }
            this.ngform.addControl(_fieldName, this.createControl());
            this._fieldName = _fieldName;
        }
        else {
            this.ngform.addControl(fieldName, this.createControl());
        }
        const onValueChangeSubscription = this._control.valueChanges
            .pipe(debounceTime(200))
            .subscribe(this.onValueChange.bind(this));
        this.registerDestroyListener(() => onValueChangeSubscription.unsubscribe());
        const onStatusChangeSubscription = this._control.statusChanges
            .pipe(debounceTime(100))
            .subscribe(this.onStatusChange.bind(this));
        this.registerDestroyListener(() => onStatusChangeSubscription.unsubscribe());
        if (this.isRange === 'true') {
            this.ngform.addControl(fieldName + '_max', this.createControl());
            // registering for valueChanges on MaxformWidget
            const onMaxValueChangeSubscription = this._maxControl.valueChanges
                .pipe(debounceTime(200))
                .subscribe(this.onValueChange.bind(this));
            this.registerDestroyListener(() => onMaxValueChangeSubscription.unsubscribe());
        }
        this.value = _.get(this.form.formdata, this._fieldName);
    }
    ngOnInit() {
        this.ngform = this.form.ngform;
        this.registerFormField();
        // Instantiate custom validators class for form field
        this.fieldValidations = new BaseFieldValidations(this, this.formWidget, this.widgettype, this._control, this.form);
        super.ngOnInit();
    }
    ngAfterContentInit() {
        super.ngAfterContentInit();
        if (this.formWidget) {
            this._initPropsRes();
            // setting displayExpressions on the formwidget explicitly as expr was evaluated to "".
            this.setFormWidget('binddisplaylabel', this.binddisplaylabel);
            this.setFormWidget('binddisplayexpression', this.binddisplayexpression);
            this.setFormWidget('binddisplayimagesrc', this.binddisplayimagesrc);
            this.setFormWidget('bindChipclass', this.bindChipclass);
        }
        this.registerReadyStateListener(() => {
            this.key = this._fieldName || this.target || this.binding;
            this.viewmodewidget = this.viewmodewidget || getDefaultViewModeWidget(this.widgettype);
            // For upload widget, generate the permitted field
            if (this.widgettype === FormWidgetType.UPLOAD) {
                let fileType;
                // Create the accepts string from file type and extensions
                fileType = this.filetype ? FILE_TYPES[this.filetype] : '';
                this.permitted = fileType + (this.extensions ? (fileType ? ',' : '') + this.extensions : '');
            }
            if (isMobile() && this.viewport.isMobileType) {
                if (!this['mobile-display']) {
                    this.widget.show = false;
                }
            }
            else if (this.viewport.isTabletType) {
                if (!this['tablet-display']) {
                    this.widget.show = false;
                }
            }
            else {
                if (!this['pc-display']) {
                    this.widget.show = false;
                }
            }
            this.fieldDefConfig.displaname = this.displayname;
            this.fieldDefConfig.show = this.show;
            this.fieldDefConfig.isRelated = this['is-related'];
            this.fieldDefConfig.inputtype = this.inputtype;
            this.fieldDefConfig.generator = this.generator;
            this.fieldDefConfig.placeholder = this.placeholder;
            this.fieldDefConfig.primaryKey = this['primary-key'];
            this.fieldDefConfig.required = this.required;
            this.fieldDefConfig._readonly = this.readonly;
            this.fieldDefConfig.regexp = this.regexp;
            this.fieldDefConfig.type = this.type;
            this.fieldDefConfig.key = this.key;
            this.fieldDefConfig.mobileDisplay = this['mobile-display'];
            this.fieldDefConfig.name = this.name;
            this.fieldDefConfig.pcDisplay = this['pc-display'];
            this.fieldDefConfig.tabletDisplay = this['tablet-display'];
            this.fieldDefConfig.validationmessage = this.validationmessage;
            this.fieldDefConfig.viewmodewidget = this.viewmodewidget;
            this.fieldDefConfig.widget = this.widgettype;
            // Register the form field with parent form
            this.form.registerFormFields(this.widget);
            addForIdAttributes(this.nativeElement);
            this.setReadOnlyState();
        });
    }
};
FormFieldDirective.initializeProps = registerProps$1();
FormFieldDirective.ctorParameters = () => [
    { type: Injector },
    { type: FormComponent },
    { type: FormBuilder },
    { type: Viewport },
    { type: ListComponent, decorators: [{ type: Optional }] },
    { type: String, decorators: [{ type: Attribute, args: ['chipclass.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['displayexpression.bind',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['displaylabel.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['widgettype',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['name',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['displayimagesrc.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['key',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['is-range',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['pc-display',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['mobile-display',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['tablet-display',] }] },
    { type: Array, decorators: [{ type: Self }, { type: Inject, args: [Context,] }] }
];
__decorate([
    ContentChild('formWidget', { static: true }),
    __metadata("design:type", Object)
], FormFieldDirective.prototype, "formWidget", void 0);
__decorate([
    ContentChild('formWidgetMax'),
    __metadata("design:type", Object)
], FormFieldDirective.prototype, "formWidgetMax", void 0);
FormFieldDirective = FormFieldDirective_1 = __decorate([
    Directive({
        selector: '[wmFormField]',
        exportAs: 'wmFormField',
        providers: [
            provideAsWidgetRef(FormFieldDirective_1),
            provideAs(FormFieldDirective_1, NG_VALUE_ACCESSOR, true),
            { provide: Context, useValue: ɵ1$1, multi: true }
        ]
    }),
    __param(4, Optional()),
    __param(5, Attribute('chipclass.bind')),
    __param(6, Attribute('dataset.bind')),
    __param(7, Attribute('displayexpression.bind')),
    __param(8, Attribute('displaylabel.bind')),
    __param(9, Attribute('widgettype')),
    __param(10, Attribute('name')),
    __param(11, Attribute('displayimagesrc.bind')),
    __param(12, Attribute('key')),
    __param(13, Attribute('is-range')),
    __param(14, Attribute('pc-display')),
    __param(15, Attribute('mobile-display')),
    __param(16, Attribute('tablet-display')),
    __param(17, Self()), __param(17, Inject(Context)),
    __metadata("design:paramtypes", [Injector,
        FormComponent,
        FormBuilder,
        Viewport,
        ListComponent, String, Object, String, String, Object, Object, String, Object, Object, Object, Object, Object, Array])
], FormFieldDirective);

let LiveActionsDirective = class LiveActionsDirective {
    constructor(subscribedWidget, app, dialogService) {
        this.subscribedWidget = subscribedWidget;
        this.app = app;
        this.dialogService = dialogService;
        subscribedWidget.addRow = this.addRow.bind(this);
        subscribedWidget.updateRow = this.updateRow.bind(this);
        subscribedWidget.deleteRow = this.deleteRow.bind(this);
        subscribedWidget.call = this.call.bind(this);
    }
    addRow() {
        this.app.notify('wm-event', { eventName: Live_Operations.INSERT, widgetName: this.subscribedWidget.name, row: this.subscribedWidget.selecteditem });
    }
    updateRow() {
        this.app.notify('wm-event', { eventName: Live_Operations.UPDATE, widgetName: this.subscribedWidget.name, row: this.subscribedWidget.selecteditem });
    }
    deleteRow() {
        this.app.notify('wm-event', { eventName: Live_Operations.DELETE, widgetName: this.subscribedWidget.name, row: this.subscribedWidget.selecteditem });
    }
    successHandler(options, response) {
        triggerFn(options.success, response);
    }
    errorHandler(options, error) {
        this.app.notifyApp(error, 'error', 'ERROR');
        triggerFn(options.error, error);
    }
    getRecords(options, operation) {
        let index;
        let dataNavigator;
        if (this.subscribedWidget.navigation !== 'None' && this.subscribedWidget.dataNavigator) {
            dataNavigator = this.subscribedWidget.dataNavigator;
            // If operation is delete, decrease the data size and check if navigation to previous page is required
            if (operation === Live_Operations.DELETE) {
                dataNavigator.dataSize -= 1;
                dataNavigator.calculatePagingValues();
                index = dataNavigator.pageCount < dataNavigator.dn.currentPage ? 'prev' : undefined;
            }
            else {
                // If operation is insert, go to last page. If update operation, stay on current page
                index = operation === Live_Operations.INSERT ? 'last' : 'current';
                if (index === 'last') {
                    dataNavigator.dataSize += 1;
                }
                dataNavigator.calculatePagingValues();
            }
            dataNavigator.navigatePage(index, null, true, response => {
                this.successHandler(options, response);
            });
        }
        else {
            this.subscribedWidget.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                'skipToggleState': true
            }).then(response => {
                this.successHandler(options, response);
            }, err => {
                this.errorHandler(options, err);
            });
        }
    }
    performCUDOperation(requestData, operation, options) {
        performDataOperation(this.subscribedWidget.datasource, requestData, {
            operationType: operation
        }).then(response => {
            if (response.error) {
                this.errorHandler(options, response.error);
                return;
            }
            if (this.subscribedWidget.datasource.category === 'wm.CrudVariable' && operation === 'delete') {
                this.subscribedWidget.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                    'skipToggleState': true
                }).then(response => {
                    this.successHandler(options, response);
                }, err => {
                    this.errorHandler(options, err);
                });
            }
            else {
                this.getRecords(options, operation);
            }
            // show delete success toaster
            if (operation === 'delete') {
                this.app.notifyApp(this.app.appLocale.MESSAGE_DELETE_RECORD_SUCCESS, 'success');
            }
        }, (error) => {
            this.errorHandler(options, error);
        });
    }
    insertRecord(requestData, operation, options) {
        this.performCUDOperation(requestData, operation, options);
    }
    updateRecord(requestData, operation, options) {
        this.performCUDOperation(requestData, operation, options);
    }
    deleteRecord(requestData, operation, options) {
        // Show the delete confirmation dialog. On Ok, delete the record.
        this.dialogService.showAppConfirmDialog({
            title: this.app.appLocale.MESSAGE_DELETE_RECORD || 'Delete Record',
            iconclass: 'wi wi-delete fa-lg',
            message: this.subscribedWidget.confirmdelete || 'Are you sure you want to delete this?',
            oktext: this.subscribedWidget.deleteoktext || 'Ok',
            canceltext: this.subscribedWidget.deletecanceltext || 'Cancel',
            onOk: () => {
                this.performCUDOperation(requestData, operation, options);
                this.dialogService.closeAppConfirmDialog();
            },
            onCancel: () => {
                triggerFn(options.cancelDeleteCallback);
                this.dialogService.closeAppConfirmDialog();
            }
        });
    }
    performOperation(operation, options) {
        const requestData = {
            row: options.row,
            prevData: {},
            rowData: {},
            transform: true,
            skipNotification: true
        };
        if (operation === Live_Operations.UPDATE) {
            requestData.rowData = options.rowData;
            requestData.prevData = options.prevData;
        }
        /* decide routine based on CRUD operation to be performed */
        switch (operation) {
            case Live_Operations.INSERT:
                this.insertRecord(requestData, operation, options);
                break;
            case Live_Operations.UPDATE:
                this.updateRecord(requestData, operation, options);
                break;
            case Live_Operations.DELETE:
                this.deleteRecord(requestData, operation, options);
                break;
            case Live_Operations.READ:
                this.getRecords(options, operation);
                break;
        }
    }
    // API exposed to make CRUD operations
    call(operation, options, success, error) {
        options.success = success;
        options.error = error;
        this.performOperation(operation, options);
    }
};
LiveActionsDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [ListComponent,] }] },
    { type: App },
    { type: AbstractDialogService }
];
LiveActionsDirective = __decorate([
    Directive({
        selector: '[wmLiveActions]'
    }),
    __param(0, Self()), __param(0, Inject(ListComponent)),
    __metadata("design:paramtypes", [Object, App,
        AbstractDialogService])
], LiveActionsDirective);

let DependsonDirective = class DependsonDirective {
    constructor(dialogId, dependson, dialogService, app) {
        this.dialogService = dialogService;
        this.app = app;
        // If dialogId is present, form is in dialog mode
        if (dialogId) {
            this.isLayoutDialog = true;
            this.dialogId = dialogId;
        }
        this.dependson = dependson;
        // Listen to the wm-event called from subscribed widgets
        this.eventSubscription = this.app.subscribe('wm-event', this.handleEvent.bind(this));
    }
    openFormDialog() {
        this.dialogService.open(this.dialogId);
    }
    onUpdate(newValue, table, editClicked) {
        this.editMode = editClicked || this.editMode;
        this.form.operationType = Live_Operations.UPDATE;
        this.form.isSelected = true;
        let rowData = this.currentFormData;
        if (newValue && table) {
            if (table.multiselect) {
                rowData = newValue[0];
            }
            else {
                rowData = newValue[newValue.length - 1];
            }
        }
        this.form.setFormData(rowData);
        this.form.edit();
        if (!this.editMode) {
            this.form.isUpdateMode = false;
        }
    }
    onInsert() {
        this.form.operationType = Live_Operations.INSERT;
        this.form.isSelected = true;
        this.form.setFormData({});
        this.form.new();
    }
    handleEvent(options) {
        if (this.dependson !== options.widgetName) {
            return;
        }
        this.currentOp = options.eventName;
        switch (options.eventName) {
            case Live_Operations.UPDATE:
                this.currentFormData = options.row;
                if (this.isLayoutDialog) {
                    this.openFormDialog();
                }
                else {
                    this.onUpdate(undefined, undefined, true);
                }
                break;
            case Live_Operations.INSERT:
                if (this.isLayoutDialog) {
                    this.openFormDialog();
                }
                else {
                    this.onInsert();
                }
                break;
            case Live_Operations.DELETE:
                this.app.Widgets[this.dependson].call('delete', { row: options.row });
                break;
            case Live_Operations.READ:
                if (!this.isLayoutDialog) {
                    this.form.isUpdateMode = false;
                }
                break;
            case 'rerender':
                if (options.dataSource) {
                    options.dataSource.execute(DataSource.Operation.LIST_RECORDS, {
                        'skipToggleState': true,
                        'operation': 'list'
                    });
                }
                break;
            case 'selectedItemChange':
                if (options.table) {
                    this.onUpdate(options.row, options.table);
                }
                break;
            case 'resetEditMode':
                this.editMode = false;
        }
    }
    onFormRender() {
        // On opening the form in dialog mode, complete the pending operations
        if (this.form && this.isLayoutDialog) {
            this.form.isLayoutDialog = true;
            setTimeout(() => {
                if (this.currentOp === Live_Operations.UPDATE) {
                    this.onUpdate(undefined, undefined, true);
                }
                else if (this.currentOp === Live_Operations.INSERT) {
                    this.onInsert();
                }
            }, 250);
        }
    }
    ngAfterContentInit() {
        // If form instance is present, form is in inline mode. Else, it is in dialog mode and listen to form instance changes
        if (this.formChildren.first) {
            this.form = this.formChildren.first;
        }
        else {
            this.formSubscription = this.formChildren.changes.subscribe((val) => {
                this.form = val.first;
                this.onFormRender();
            });
        }
        // find the dependent widget (table/list)
        this.app.notify('setup-cud-listener', this.dependson);
    }
    ngOnDestroy() {
        if (this.formSubscription) {
            this.formSubscription.unsubscribe();
        }
        if (this.eventSubscription) {
            this.eventSubscription();
        }
    }
};
DependsonDirective.ctorParameters = () => [
    { type: String, decorators: [{ type: Attribute, args: ['dialogid',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['dependson',] }] },
    { type: AbstractDialogService },
    { type: App }
];
__decorate([
    ContentChildren(FormComponent, { descendants: true }),
    __metadata("design:type", Object)
], DependsonDirective.prototype, "formChildren", void 0);
DependsonDirective = __decorate([
    Directive({
        selector: '[dependson]'
    }),
    __param(0, Attribute('dialogid')),
    __param(1, Attribute('dependson')),
    __metadata("design:paramtypes", [String, String, AbstractDialogService,
        App])
], DependsonDirective);

const FILTER_CONSTANTS = {
    'EMPTY_KEY': 'EMPTY_NULL_FILTER'
};
const noop = () => { };
const ɵ0$2 = noop;
let LiveFilterDirective = class LiveFilterDirective {
    constructor(form) {
        this.form = form;
        // debounce the filter function. If multiple filter calls are made at same time, calls will be delayed and last call is fired
        this._filter = debounce(options => {
            this.filter(options);
        }, 250);
        form.clearFilter = this.clearFilter.bind(this);
        form.applyFilter = this.applyFilter.bind(this);
        form.filter = this.filter.bind(this);
        form.filterOnDefault = this.filterOnDefault.bind(this);
        form.execute = this.execute.bind(this);
        form.onFieldDefaultValueChange = this.onFieldDefaultValueChange.bind(this);
        form.onMaxDefaultValueChange = this.onMaxDefaultValueChange.bind(this);
        form.onDataSourceChange = this.onDataSourceChange.bind(this);
        form.onFieldValueChange = this.onFieldValueChange.bind(this);
        form.submitForm = this.submitForm.bind(this);
        form.registerFormWidget = this.registerFormWidget.bind(this);
    }
    execute(operation, options) {
        if (operation === DataSource.Operation.LIST_RECORDS || operation === DataSource.Operation.DOWNLOAD) {
            return this.applyFilter(options);
        }
        if (operation === DataSource.Operation.GET_OPTIONS) {
            return this._options || {};
        }
        if (operation === DataSource.Operation.GET_PAGING_OPTIONS) {
            return this.form.pagination;
        }
        if (!this.form.datasource) {
            return {};
        }
        if (operation === DataSource.Operation.FETCH_DISTINCT_VALUES) {
            return fetchDistinctValues(this.form.datasource, this.form.formFields, {
                widget: 'widgettype',
                enableemptyfilter: this.form.enableemptyfilter,
                EMPTY_VALUE: this.form.appLocale.LABEL_NO_VALUE
            });
        }
        return this.form.datasource.execute(operation, options);
    }
    onFieldDefaultValueChange(field, nv) {
        field.minValue = nv;
        field.value = nv;
        this.filterOnDefault();
    }
    onFieldValueChange(field, nv) {
        applyFilterOnField(this.form.datasource, field.widget, this.form.formFields, nv, {
            EMPTY_VALUE: this.form.appLocale.LABEL_NO_VALUE
        });
        if (this.form.autoupdate && isDefined(nv)) {
            this._filter();
        }
    }
    onMaxDefaultValueChange() {
        setTimeout(() => {
            this.filterOnDefault();
        });
    }
    onDataSourceChange() {
        const dataSource = this.form.datasource;
        if (!dataSource) {
            return;
        }
        this.form.formFields.forEach(field => {
            if (isDataSetWidget(field.widgettype)) {
                getDistinctValuesForField(dataSource, field.widget, {
                    widget: 'widgettype',
                    enableemptyfilter: this.form.enableemptyfilter,
                    EMPTY_VALUE: this.form.appLocale.LABEL_NO_VALUE
                });
                applyFilterOnField(dataSource, field.widget, this.form.formFields, field.value, {
                    isFirst: true,
                    EMPTY_VALUE: this.form.appLocale.LABEL_NO_VALUE
                });
            }
        });
        // On load check if default value exists and apply filter, Call the filter with the result options
        this._filter(this._options);
    }
    clearFilter() {
        this.form.resetFormState();
        this.form.formFields.forEach(filterField => {
            // Added check for range field
            if (!filterField.readonly && filterField.show) {
                if (filterField.widgettype === FormWidgetType.AUTOCOMPLETE || filterField.widgettype === FormWidgetType.TYPEAHEAD) {
                    this.form.$element.find('div[name=' + filterField.name + '] input').val('');
                }
                if (filterField['is-range']) {
                    filterField.minValue = '';
                    filterField.maxValue = '';
                }
                else {
                    filterField.value = '';
                    filterField.resetDisplayInput();
                }
            }
        });
        // If variable has any bindings, wait for the bindings to be updated
        setTimeout(() => {
            // Setting result to the default data
            this._filter();
        });
    }
    submitForm() {
        this.filter();
    }
    applyFilter(options) {
        options = options ? (options.data || options) : {};
        options.page = options.page || 1;
        options.orderBy = isDefined(options.orderBy) ? options.orderBy : this.orderBy;
        return this.filter(options);
    }
    filter(options) {
        if (!this.form.datasource) {
            return;
        }
        const filterFields = {};
        const dataModel = {};
        let page = 1, orderBy, isValid;
        options = options || {};
        page = options.page || page;
        orderBy = isDefined(options.orderBy) ? options.orderBy : (this.orderBy || '');
        this.orderBy = orderBy; // Store the order by in scope. This can be used to retain the sort after filtering
        // Copy the values to be sent to the user as '$data' before servicecall
        this.form.formFields.forEach(field => {
            const fieldSelector = 'div[name=' + field.name + '] input';
            const $el = this.form.$element;
            let fieldEle;
            if ((field.widgettype === FormWidgetType.AUTOCOMPLETE || field.widgettype === FormWidgetType.TYPEAHEAD) && $el) {
                fieldEle = $el.find(fieldSelector);
                if (!field['is-range']) {
                    dataModel[field.field] = {
                        'value': isDefined(field.value) ? field.value : fieldEle.val() // For autocomplete, set the datavalue. If not present, set query value
                    };
                }
                else {
                    dataModel[field.field] = {
                        'minValue': isDefined(field.minValue) ? field.minValue : fieldEle.first().val(),
                        'maxValue': isDefined(field.maxValue) ? field.maxValue : fieldEle.last().val()
                    };
                }
                return;
            }
            if (!field['is-range']) {
                dataModel[field.field] = {
                    'value': field.value
                };
            }
            else {
                dataModel[field.field] = {
                    'minValue': field.minValue,
                    'maxValue': field.maxValue
                };
            }
        });
        /*Perform this function for the event onBeforeservicecall*/
        try {
            isValid = this.form.invokeEventCallback('beforeservicecall', { $data: dataModel });
            if (isValid === false) {
                return;
            }
            if (isValid && isValid.error) {
                this.form.toggleMessage(true, isValid.error, 'error', 'ERROR');
                return;
            }
            /*Update these values in the formFields with new reference, inorder to maintain the UI values*/
            this.form.formFields.forEach(filterField => {
                if (!filterField['is-range']) {
                    filterField._value = dataModel[filterField.field].value;
                }
                else {
                    filterField._minValue = dataModel[filterField.field].minValue;
                    filterField._maxValue = dataModel[filterField.field].maxValue;
                }
            });
        }
        catch (err) {
            if (err.message === 'Abort') {
                return;
            }
        }
        /* Construct the formFields Variable to send it to the queryBuilder */
        this.form.formFields.forEach(filterField => {
            let fieldValue;
            let matchMode;
            let colName = filterField.field;
            const minValue = filterField._minValue;
            const maxvalue = filterField._maxValue;
            /* if field is part of a related entity, column name will be 'entity.fieldName' */
            if (filterField['is-related']) {
                colName += '.' + filterField['lookup-field'];
            }
            if (filterField['is-range']) {
                /*Based on the min and max values, decide the matchmode condition*/
                fieldValue = getRangeFieldValue(minValue, maxvalue);
                matchMode = getRangeMatchMode(minValue, maxvalue);
                if (isDefined(fieldValue)) {
                    filterFields[colName] = {
                        'value': fieldValue,
                        'matchMode': matchMode,
                        'logicalOp': 'AND'
                    };
                }
            }
            else {
                switch (filterField.widgettype) {
                    case FormWidgetType.SELECT:
                    case FormWidgetType.RADIOSET:
                        if (getEnableEmptyFilter(this.form.enableemptyfilter) && filterField._value === FILTER_CONSTANTS.EMPTY_KEY) {
                            matchMode = getEmptyMatchMode(this.form.enableemptyfilter);
                            fieldValue = filterField._value;
                        }
                        else {
                            if (filterField.type === DataType.BOOLEAN) {
                                if (isDefined(filterField._value) && filterField._value !== '') {
                                    fieldValue = JSON.parse(filterField._value);
                                }
                            }
                            else {
                                fieldValue = filterField._value;
                            }
                        }
                        break;
                    case FormWidgetType.CHECKBOXSET:
                    case FormWidgetType.CHIPS:
                        if (filterField._value && filterField._value.length) {
                            fieldValue = filterField._value;
                        }
                        break;
                    case FormWidgetType.CHECKBOX:
                    case FormWidgetType.TOGGLE:
                        if (isDefined(filterField._value) && filterField._value !== '') {
                            fieldValue = filterField.type === DataType.BOOLEAN ? JSON.parse(filterField._value) : filterField._value;
                        }
                        break;
                    default:
                        fieldValue = filterField._value;
                        break;
                }
                if (isDefined(fieldValue) && fieldValue !== '' && fieldValue !== null) {
                    filterFields[colName] = {};
                    if (matchMode) {
                        filterFields[colName].matchMode = matchMode;
                        fieldValue = undefined;
                    }
                    else if (filterField.type === DataType.STRING || filterField.isRelated) { // Only for string types and related fields, custom match modes are enabled.
                        filterFields[colName].matchMode = matchMode || filterField.matchmode ||
                            this.form.datasource.execute(DataSource.Operation.GET_MATCH_MODE);
                    }
                    filterFields[colName].value = fieldValue;
                    filterFields[colName].logicalOp = 'AND';
                    filterFields[colName].type = filterField.type;
                }
            }
        });
        if (options.exportType) {
            return this.form.datasource.execute(DataSource.Operation.DOWNLOAD, {
                data: {
                    matchMode: 'anywhereignorecase',
                    filterFields: filterFields,
                    orderBy: orderBy,
                    exportType: options.exportType,
                    logicalOp: 'AND',
                    exportSize: options.exportSize,
                    fields: options.fields,
                    fileName: options.fileName
                }
            });
        }
        return this.form.datasource.execute(DataSource.Operation.LIST_RECORDS, {
            filterFields: filterFields,
            orderBy: orderBy,
            page: page,
            pagesize: this.form.pagesize || 20,
            skipDataSetUpdate: true,
            inFlightBehavior: 'executeAll'
        }).then(response => {
            const data = response.data;
            this.form.pagination = response.pagination;
            if (data.error) {
                // disable readonly and show the appropriate error
                this.form.toggleMessage(true, data.error, 'error', 'ERROR');
                this.form.onResultCb(data, false);
            }
            else {
                this._options = {
                    'page': page,
                    'orderBy': orderBy
                };
                this.form.result = getClonedObject(data);
                this.form.onResultCb(data, true);
            }
            $appDigest();
            return this.form.result;
        }, error => {
            this.form.toggleMessage(true, error, 'error', 'ERROR');
            this.form.onResultCb(error, false);
            return error;
        });
    }
    // Calls the filter function if default values are present
    filterOnDefault() {
        /*Check if default value is present for any filter field*/
        const defaultObj = _.find(this.form.formFields, obj => {
            return isDefined(obj.value) || isDefined(obj.minValue) || isDefined(obj.maxValue);
        });
        /*If default value exists and data is loaded, apply the filter*/
        if (defaultObj) {
            this._filter(this._options);
        }
    }
    registerFormWidget(widget) {
        const name = widget.key || widget.name;
        this.form.filterWidgets[name] = widget;
    }
};
LiveFilterDirective.initializeProps = registerLiveFilterProps();
LiveFilterDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [FormComponent,] }] }
];
LiveFilterDirective = __decorate([
    Directive({
        selector: '[wmLiveFilter]'
    }),
    __param(0, Self()), __param(0, Inject(FormComponent)),
    __metadata("design:paramtypes", [Object])
], LiveFilterDirective);

const isTimeType = field => field.widgettype === DataType.TIME || (field.type === DataType.TIME && !field.widgettype);
const ɵ0$3 = isTimeType;
const getValidTime = val => {
    if (val) {
        const date = (new Date()).toDateString();
        return (new Date(`${date} ${val}`)).getTime();
    }
    return undefined;
};
const ɵ1$2 = getValidTime;
let LiveFormDirective = class LiveFormDirective {
    constructor(form, liveTable, datePipe, dialogService, formlayout) {
        this.form = form;
        this.datePipe = datePipe;
        this.dialogService = dialogService;
        this._debouncedSavePrevDataValues = debounce(() => {
            this.savePrevDataValues();
        }, 250);
        // If parent live table is present and this form is first child of live table, set this form instance on livetable
        if (liveTable && !this.form.parentForm) {
            this.form._liveTableParent = liveTable;
            this.form.isLayoutDialog = liveTable.isLayoutDialog;
            liveTable.onFormReady(this.form);
        }
        else {
            this.form.isLayoutDialog = formlayout === 'dialog';
        }
        // CUD operations
        form.cancel = this.cancel.bind(this);
        form.reset = this.reset.bind(this);
        form.delete = this.delete.bind(this);
        form.save = this.save.bind(this);
        form.saveAndNew = this.saveAndNew.bind(this);
        form.saveAndView = this.saveAndView.bind(this);
        form.setPrimaryKey = this.setPrimaryKey.bind(this);
        form.constructDataObject = this.constructDataObject.bind(this);
        form.changeDataObject = this.setFormData.bind(this);
        form.setFormData = this.setFormData.bind(this);
        form.findOperationType = this.findOperationType.bind(this);
        form.clearData = this.clearData.bind(this);
        form.onFieldDefaultValueChange = this.onFieldDefaultValueChange.bind(this);
        form.onDataSourceChange = this.onDataSourceChange.bind(this);
        form.onFieldValueChange = this.onFieldValueChange.bind(this);
        form.submitForm = this.submitForm.bind(this);
        form.form_registerFormFields = form.registerFormFields;
        form.registerFormFields = this.registerFormFields.bind(this);
    }
    /*
    WMS:18583
    Overriding 'registerFormFields' for liveForms,
    On Each field added, check if the dataSource is updated or not
    if the field is added after the dataSource is updated at LiveForm, retrigger
    */
    registerFormFields(formField) {
        const formFields = this.form.getFormFields();
        this.form.form_registerFormFields(formField);
        if (this.form.isDataSourceUpdated) {
            this._updateFieldOnDataSourceChange(formField, formFields);
        }
    }
    _updateFieldOnDataSourceChange(field, formFields) {
        if (!field.isDataSetBound && isDataSetWidget(field.widgettype)) {
            if (field['is-related']) {
                field.isDataSetBound = true;
                fetchRelatedFieldData(this.form.datasource, field.widget, {
                    relatedField: field.key,
                    datafield: ALLFIELDS,
                    widget: 'widgettype',
                });
            }
            else {
                getDistinctValuesForField(this.form.datasource, field.widget, {
                    widget: 'widgettype',
                    enableemptyfilter: this.form.enableemptyfilter
                });
                applyFilterOnField(this.form.datasource, field.widget, formFields, field.value, { isFirst: true });
            }
        }
    }
    onDataSourceChange() {
        if (_.get(this.form.formFields, 'length')) {
            this.form.isDataSourceUpdated = true;
        }
        const formFields = this.form.getFormFields();
        formFields.forEach(field => {
            this._updateFieldOnDataSourceChange(field, formFields);
        });
    }
    onFieldDefaultValueChange(field, nv) {
        // In Edit, do  not set default values
        if (this.form.operationType === 'update') {
            return;
        }
        // Set the default value only if it exists.
        if (isDefined(nv) && nv !== null && nv !== '' && nv !== 'null') {
            field.value = parseValueByType(nv, field.type, field.widgettype, field.trailingzero);
        }
        else {
            field.value = undefined;
        }
        this._debouncedSavePrevDataValues();
    }
    onFieldValueChange(field, nv) {
        applyFilterOnField(this.form.datasource, field.widget, this.form.formFields, nv);
    }
    getBlobURL(dataObj, key, value) {
        let href = '';
        let primaryKeys;
        let primaryKey;
        if (value === null || value === undefined || !this.form.datasource) {
            return href;
        }
        primaryKeys = this.form.datasource.execute(DataSource.Operation.GET_PRIMARY_KEY) || [];
        primaryKey = dataObj[primaryKeys[0]];
        // TODO: Handle mobile case
        // if (CONSTANTS.hasCordova && CONSTANTS.isRunMode) {
        //     href += $rootScope.project.deployedUrl;
        // }
        href += this.form.datasource.execute(DataSource.Operation.GET_BLOB_URL, {
            primaryValue: primaryKey,
            columnName: key
        });
        href += '?' + Math.random();
        return href;
    }
    setFormData(dataObj, formFields) {
        if (!dataObj) {
            return;
        }
        formFields = formFields || this.form.formFields;
        formFields.forEach(field => {
            const value = _.get(dataObj, field.key || field.name);
            if (isTimeType(field)) {
                field.value = getValidTime(value);
            }
            else if (field.type === DataType.BLOB) {
                this.form.resetFileUploadWidget(field, true);
                field.href = this.getBlobURL(dataObj, field.key, value);
                field.value = _.isString(value) ? '' : value;
            }
            else {
                this.form.setFieldValue(field, dataObj);
            }
        });
        this.savePrevDataValues();
        this.form.constructDataObject();
    }
    onDataSourceUpdate(response, newForm, updateMode) {
        if (newForm) {
            this.form.new();
        }
        else {
            this.form.setFormData(response);
            this.closeDialog();
        }
        this.form.isUpdateMode = isDefined(updateMode) ? updateMode : true;
    }
    savePrevformFields() {
        this.form.prevformFields = getClonedObject(this.form.formFields.map(field => {
            return {
                'key': field.key,
                'type': field.type,
                'widgettype': field.widgettype,
                'outputformat': field.outputformat,
                'value': field.value
            };
        }));
    }
    getPrevformFields() {
        this.form.formFields.map(field => {
            const prevField = this.form.prevformFields.find(pField => pField.key === field.key);
            field.value = prevField.value;
        });
    }
    getDataObject() {
        if (this.form.operationType === Live_Operations.INSERT) {
            return {};
        }
        if (isDefined(this.form.prevDataObject) && !isEmptyObject(this.form.prevDataObject)) {
            return getClonedObject(this.form.prevDataObject);
        }
        return getClonedObject(this.form.formdata || {});
    }
    constructDataObject(isPreviousData) {
        const dataObject = this.getDataObject();
        const formName = this.form.name;
        let formFields;
        formFields = isPreviousData ? this.form.prevformFields : this.form.getFormFields();
        formFields.forEach(field => {
            let dateTime, fieldValue;
            const fieldTarget = _.split(field.key, '.');
            const fieldName = fieldTarget[0] || field.key;
            /*collect the values from the fields and construct the object*/
            /*Format the output of date time widgets to the given output format*/
            if ((field.widgettype && isDateTimeType(field.widgettype)) || isDateTimeType(field.type)) {
                if (field.value) {
                    dateTime = getValidDateObject(field.value);
                    if (field.outputformat === DataType.TIMESTAMP || field.type === DataType.TIMESTAMP) {
                        fieldValue = field.value ? dateTime : null;
                    }
                    else if (field.outputformat) {
                        fieldValue = this.datePipe.transform(dateTime, field.outputformat);
                    }
                    else {
                        fieldValue = field.value;
                    }
                }
                else {
                    fieldValue = undefined;
                }
            }
            else if (field.type === DataType.BLOB) {
                fieldValue = getFiles(formName, fieldName + '_formWidget', field.multiple);
            }
            else if (field.type === DataType.LIST) {
                fieldValue = field.value || undefined;
            }
            else {
                fieldValue = (field.value === null || field.value === '') ? undefined : field.value;
            }
            if (fieldTarget.length === 1) {
                dataObject[fieldName] = fieldValue;
            }
            else {
                dataObject[fieldName] = dataObject[fieldName] || {};
                dataObject[fieldName][fieldTarget[1]] = fieldValue;
            }
        });
        if (!isPreviousData) {
            this.form.updateFormDataOutput(dataObject);
            return this.form.dataoutput;
        }
        return dataObject;
    }
    setPrimaryKey(fieldName) {
        /*Store the primary key of data*/
        this.form.primaryKey = this.form.primaryKey || [];
        if (this.form.primaryKey.indexOf(fieldName) === -1) {
            this.form.primaryKey.push(fieldName);
        }
    }
    findOperationType() {
        let operation;
        let isPrimary = false;
        const sourceOperation = this.form.datasource && this.form.datasource.execute(DataSource.Operation.GET_OPERATION_TYPE);
        if (sourceOperation && sourceOperation !== 'read') {
            return sourceOperation;
        }
        /*If OperationType is not set then based on the formdata object return the operation type,
            this case occurs only if the form is outside a livegrid*/
        /*If the formdata object has primary key value then return update else insert*/
        if (this.form.primaryKey && !_.isEmpty(this.form.formdata)) {
            /*If only one column is primary key*/
            if (this.form.primaryKey.length === 1) {
                if (this.form.formdata[this.form.primaryKey[0]]) {
                    operation = Live_Operations.UPDATE;
                }
                /*If only no column is primary key*/
            }
            else if (this.form.primaryKey.length === 0) {
                _.forEach(this.form.formdata, (value) => {
                    if (value) {
                        isPrimary = true;
                    }
                });
                if (isPrimary) {
                    operation = Live_Operations.UPDATE;
                }
                /*If multiple columns are primary key*/
            }
            else {
                isPrimary = _.some(this.form.primaryKey, (primaryKey) => {
                    if (this.form.formdata[primaryKey]) {
                        return true;
                    }
                });
                if (isPrimary) {
                    operation = Live_Operations.UPDATE;
                }
            }
        }
        return operation || Live_Operations.INSERT;
    }
    getPrevDataValues() {
        const prevDataValues = _.fromPairs(_.map(this.form.prevDataValues, (item) => {
            return [item.key, item.value];
        })); // Convert of array of values to an object
        this.form.formFields.forEach(field => {
            field.value = prevDataValues[field.key] || '';
            field.resetDisplayInput();
        });
        return prevDataValues;
    }
    savePrevDataValues() {
        this.form.prevDataValues = this.form.formFields.map((obj) => {
            return { 'key': obj.key, 'value': obj.value };
        });
    }
    clearData() {
        this.form.toggleMessage(false);
        this.form.emptyDataModel();
    }
    setReadonlyFields() {
        this.form.formFields.forEach(field => {
            field.setReadOnlyState();
        });
    }
    reset() {
        let prevDataValues;
        this.form.resetFormState();
        prevDataValues = this.getPrevDataValues();
        this.form.formFields.forEach(field => {
            if (field.type === DataType.BLOB || field.widgettype === 'upload') {
                this.form.resetFileUploadWidget(field, true);
                field.href = this.getBlobURL(prevDataValues, field.key, field.value);
            }
        });
        this.form.constructDataObject();
    }
    closeDialog() {
        if (this.form.isLayoutDialog) {
            this.dialogService.close(this.form.dialogId);
        }
    }
    cancel() {
        this.form.clearMessage();
        this.form.isUpdateMode = false;
        this.form.reset();
        /*Show the previous selected data*/
        if (this.form.isSelected) {
            this.getPrevformFields();
        }
        this.form.isUpdateMode = false;
        if (this.form._liveTableParent) {
            this.form._liveTableParent.onCancel();
        }
        else {
            /*
              WMS-18759: dialog close is calling twice and dialog onHidden subscription is calling once from the library
              due to this if we have another dialog inside liveform and if we click on cancel of second dialog the library
              subscription is calling for both the dialogs.
             */
            this.closeDialog();
        }
        $appDigest();
    }
    delete(callBackFn) {
        this.form.resetFormState();
        this.form.operationType = Live_Operations.DELETE;
        this.form.prevDataObject = getClonedObject(this.form.formdata || {});
        this.form.save(undefined, undefined, undefined, callBackFn);
    }
    // Function use to save the form and open new form after save
    saveAndNew() {
        this.save(undefined, true, true);
    }
    // Function use to save the form and open new form after save
    saveAndView() {
        this.save(undefined, false);
    }
    submitForm($event) {
        this.save($event);
    }
    save(event, updateMode, newForm) {
        if (!this.form.datasource) {
            return;
        }
        let data, prevData, operationType, isValid;
        const requestData = {};
        operationType = this.form.operationType = this.form.operationType || this.findOperationType();
        // Disable the form submit if form is in invalid state.
        if (this.form.validateFieldsOnSubmit()) {
            return;
        }
        data = getClonedObject(this.form.constructDataObject());
        prevData = this.form.prevformFields ? this.form.constructDataObject(true) : data;
        try {
            isValid = this.form.invokeEventCallback('beforeservicecall', { $event: event, $operation: this.form.operationType, $data: data, options: requestData });
            if (isValid === false) {
                return;
            }
            if (isValid && isValid.error) {
                this.form.toggleMessage(true, isValid.error, 'error');
                return;
            }
        }
        catch (err) {
            if (err.message === 'Abort') {
                return;
            }
        }
        // If operation is update, form is not touched and current data and previous data is same, Show no changes detected message
        if (this.form.operationType === Live_Operations.UPDATE && this.form.ngform && this.form.ngform.pristine &&
            (this.form.isSelected && _.isEqual(data, prevData))) {
            this.form.toggleMessage(true, this.form.appLocale.MESSAGE_NO_CHANGES, 'info', '');
            $appDigest();
            return;
        }
        requestData.row = data;
        requestData.transform = true;
        requestData.skipNotification = true;
        if (operationType === Live_Operations.UPDATE) {
            requestData.rowData = this.form.formdata;
            requestData.prevData = prevData;
        }
        performDataOperation(this.form.datasource, requestData, {
            operationType: operationType
        }).then((response) => {
            const msg = operationType === Live_Operations.INSERT ? this.form.insertmessage : (operationType === Live_Operations.UPDATE ?
                this.form.updatemessage : this.form.deletemessage);
            let result;
            if (operationType === Live_Operations.DELETE) {
                result = requestData.row;
                this.form.emptyDataModel();
                this.form.prevDataValues = [];
                this.form.isSelected = false;
            }
            else {
                result = response;
            }
            if (!this.form.onSuccess) {
                this.form.toggleMessage(true, msg, 'success');
            }
            if (this.form._liveTableParent) {
                // highlight the current updated row
                this.form._liveTableParent.onResult(operationType, response, newForm, updateMode);
            }
            else {
                /*get updated data without refreshing page*/
                this.form.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                    'skipToggleState': true
                });
                this.onDataSourceUpdate(response, newForm, updateMode);
            }
            return {
                'result': result,
                'status': true
            };
        }, (error) => {
            if (!this.form.onError) {
                this.form.toggleMessage(true, error, 'error');
            }
            $appDigest();
            return {
                'result': error,
                'status': false
            };
        }).then(response => {
            // reset the form to pristine state
            this.form.resetFormState();
            this.form.onResultCb(response.result, response.status, event);
        });
    }
};
LiveFormDirective.initializeProps = registerLiveFormProps();
LiveFormDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [FormComponent,] }] },
    { type: LiveTableComponent, decorators: [{ type: Optional }] },
    { type: ToDatePipe },
    { type: AbstractDialogService },
    { type: String, decorators: [{ type: Attribute, args: ['formlayout',] }] }
];
LiveFormDirective = __decorate([
    Directive({
        selector: '[wmLiveForm]'
    }),
    __param(0, Self()), __param(0, Inject(FormComponent)),
    __param(1, Optional()),
    __param(4, Attribute('formlayout')),
    __metadata("design:paramtypes", [Object, LiveTableComponent,
        ToDatePipe,
        AbstractDialogService, String])
], LiveFormDirective);

const components = [
    FormComponent,
    FormWidgetDirective,
    FormActionDirective,
    FormFieldDirective,
    LiveActionsDirective,
    DependsonDirective,
    LiveFilterDirective,
    LiveFormDirective
];
let FormModule = class FormModule {
};
FormModule = __decorate([
    NgModule({
        imports: [
            BasicModule,
            CommonModule,
            DesignDialogModule,
            InputModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], FormModule);

/**
 * Generated bundle index. Do not edit.
 */

export { FormComponent, FormModule, ɵ0, ɵ1, ɵ2, ɵ3, registerFormProps as ɵa, registerLiveFormProps as ɵb, registerLiveFilterProps as ɵc, FormWidgetDirective as ɵd, FormActionDirective as ɵe, registerProps as ɵf, FormFieldDirective as ɵg, registerProps$1 as ɵh, LiveActionsDirective as ɵi, DependsonDirective as ɵj, LiveFilterDirective as ɵk, LiveFormDirective as ɵl };
//# sourceMappingURL=index.js.map
