import { __assign, __extends, __awaiter, __generator, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, NgZone, Optional, SkipSelf, Attribute, ViewChild, ViewContainerRef, ContentChildren, HostBinding, HostListener, Component, Inject, Self, Directive, ContentChild, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, register, styler, getFieldLayoutConfig, Live_Operations, performDataOperation, parseValueByType, PartialDirective, MessageComponent, WidgetRef, provideAsWidgetRef, StylableComponent, BaseComponent, getWidgetPropsByType, getEvaluatedData, BaseFieldValidations, getDefaultViewModeWidget, Context, provideAs, fetchDistinctValues, applyFilterOnField, isDataSetWidget, getDistinctValuesForField, getRangeFieldValue, getRangeMatchMode, getEnableEmptyFilter, getEmptyMatchMode, fetchRelatedFieldData, ALLFIELDS, ToDatePipe, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { InputModule } from '@wm/components/input';
import { DesignDialogModule } from '@wm/components/dialogs/design-dialog';
import { FormArray, FormBuilder, NG_VALUE_ACCESSOR } from '@angular/forms';
import { isMobileApp, debounce, removeClass, isDefined, DataSource, $appDigest, getFiles, DataType, getClonedObject, $invokeWatchers, getValidJSON, extendProto, $parseEvent, App, AbstractDialogService, DynamicComponentRefProvider, FormWidgetType, isMobile, addForIdAttributes, Viewport, triggerFn, isEmptyObject, isDateTimeType, getValidDateObject } from '@wm/core';
import { PrefabDirective } from '@wm/components/prefab';
import { ListComponent } from '@wm/components/data/list';
import { TableComponent } from '@wm/components/data/table';
import { debounceTime } from 'rxjs/operators';
import { LiveTableComponent } from '@wm/components/data/live-table';

var propsMap = new Map([
    ['autocomplete', __assign({ value: false }, PROP_BOOLEAN)],
    ['captionalign', __assign({ value: 'left' }, PROP_STRING)],
    ['captionposition', __assign({ value: 'left' }, PROP_STRING)],
    ['captionsize', PROP_STRING],
    ['captionwidth', __assign({ value: 'xs-12 sm-3 md-3 lg-3' }, PROP_STRING)],
    ['class', PROP_STRING],
    ['collapsible', PROP_BOOLEAN],
    ['dataset', PROP_ANY],
    ['datasource', PROP_STRING],
    ['expanded', __assign({ value: true }, PROP_BOOLEAN)],
    ['formdatasource', PROP_STRING],
    ['name', PROP_STRING],
    ['numberOfFields', PROP_NUMBER],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['showindevice', __assign({ displayType: 'inline-block', value: 'all' }, PROP_STRING)],
    ['subheading', PROP_STRING],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['title', PROP_STRING]
]);
var registerFormProps = function () {
    if (isMobileApp()) {
        propsMap.set('captionwidth', __assign({ value: 'xs-4 sm-4 md-4 lg-4' }, PROP_STRING));
    }
    var formMap = new Map(propsMap);
    formMap.set('action', PROP_STRING);
    formMap.set('defaultmode', __assign({ value: 'Edit' }, PROP_STRING));
    formMap.set('metadata', PROP_STRING);
    formMap.set('method', PROP_STRING);
    formMap.set('postmessage', __assign({ value: 'Data posted successfully' }, PROP_STRING));
    formMap.set('target', PROP_STRING);
    formMap.set('enctype', PROP_STRING);
    formMap.set('errormessage', __assign({ value: 'An error occured. Please try again!' }, PROP_STRING));
    formMap.set('messagelayout', __assign({ value: 'Inline' }, PROP_STRING));
    formMap.set('formdata', PROP_STRING);
    formMap.set('novalidate', PROP_BOOLEAN);
    formMap.set('validationtype', __assign({ value: 'default' }, PROP_STRING));
    formMap.set('iconclass', PROP_STRING);
    register('wm-form', formMap);
};
var registerLiveFormProps = function () {
    if (isMobileApp()) {
        propsMap.set('captionwidth', __assign({ value: 'xs-4 sm-4 md-4 lg-4' }, PROP_STRING));
    }
    var liveFormMap = new Map(propsMap);
    liveFormMap.set('defaultmode', __assign({ value: 'View' }, PROP_STRING));
    liveFormMap.set('formlayout', __assign({ value: 'inline' }, PROP_STRING));
    liveFormMap.set('insertmessage', __assign({ value: 'Record added successfully' }, PROP_STRING));
    liveFormMap.set('updatemessage', __assign({ value: 'Record updated successfully' }, PROP_STRING));
    liveFormMap.set('deletemessage', __assign({ value: 'Record deleted successfully' }, PROP_STRING));
    liveFormMap.set('errormessage', __assign({ value: 'An error occured. Please try again!' }, PROP_STRING));
    liveFormMap.set('messagelayout', __assign({ value: 'Toaster' }, PROP_STRING));
    liveFormMap.set('formdata', PROP_STRING);
    liveFormMap.set('novalidate', PROP_BOOLEAN);
    liveFormMap.set('validationtype', __assign({ value: 'default' }, PROP_STRING));
    liveFormMap.set('iconclass', PROP_STRING);
    register('wm-liveform', liveFormMap);
};
var registerLiveFilterProps = function () {
    if (isMobileApp()) {
        propsMap.set('captionwidth', __assign({ value: 'xs-4 sm-4 md-4 lg-4' }, PROP_STRING));
    }
    var liveFilterMap = new Map(propsMap);
    liveFilterMap.set('autoupdate', PROP_BOOLEAN);
    liveFilterMap.set('enableemptyfilter', __assign({ value: ' ' }, PROP_STRING));
    liveFilterMap.set('pagesize', __assign({ value: 20 }, PROP_NUMBER));
    liveFilterMap.set('iconclass', __assign({ value: 'wi wi-filter-list' }, PROP_STRING));
    register('wm-livefilter', liveFilterMap);
};

var WIDGET_CONFIG = { widgetType: 'wm-form', hostClass: 'panel app-panel app-form' };
var LOGIN_FORM_CONFIG = { widgetType: 'wm-form', hostClass: 'app-form app-login-form' };
var LIVE_FORM_CONFIG = { widgetType: 'wm-liveform', hostClass: 'panel app-panel app-liveform liveform-inline' };
var LIVE_FILTER_CONFIG = { widgetType: 'wm-livefilter', hostClass: 'panel app-panel app-livefilter clearfix liveform-inline' };
var getWidgetConfig = function (isLiveForm, isLiveFilter, role) {
    var config = WIDGET_CONFIG;
    if (isLiveForm !== null) {
        config = LIVE_FORM_CONFIG;
    }
    else if (isLiveFilter !== null) {
        config = LIVE_FILTER_CONFIG;
    }
    else if (role === 'app-login') {
        config = LOGIN_FORM_CONFIG;
    }
    return config;
};
var ɵ0 = getWidgetConfig;
// Generate the form field with given field definition. Add a grid column wrapper around the form field.
var setMarkupForFormField = function (field, columnWidth) {
    var propsTmpl = '';
    _.forEach(field, function (value, key) {
        propsTmpl = propsTmpl + " " + key + "=\"" + value + "\"";
    });
    return "<wm-gridcolumn columnwidth=\"" + columnWidth + "\">\n                  <wm-form-field " + propsTmpl + "></wm-form-field>\n            </wm-gridcolumn>";
};
var ɵ1 = setMarkupForFormField;
// Function to find out the first invalid element in form
var findInvalidElement = function ($formEle, ngForm) {
    var $ele = $formEle.find('form.ng-invalid:visible, [formControlName].ng-invalid:visible').first();
    var formObj = ngForm;
    // If element is form, find out the first invalid element in this form
    if ($ele.is('form')) {
        formObj = ngForm && ngForm.controls[$ele.attr('formControlName') || $ele.attr('name')];
        if (formObj) {
            return findInvalidElement($ele, formObj);
        }
    }
    return {
        ngForm: formObj,
        $ele: $ele
    };
};
var ɵ2 = findInvalidElement;
var setTouchedState = function (ngForm) {
    if (ngForm.valid) {
        return;
    }
    if (ngForm.controls) {
        _.forEach(ngForm.controls, function (ctrl) {
            setTouchedState(ctrl);
        });
    }
    else {
        ngForm.markAsTouched();
    }
};
var ɵ3 = setTouchedState;
var FormComponent = /** @class */ (function (_super) {
    __extends(FormComponent, _super);
    function FormComponent(inj, fb, app, dialogService, dynamicComponentProvider, ngZone, parentList, parentForm, parentPrefab, parentPartial, onBeforeSubmitEvt, onSubmitEvt, onBeforeRenderEvt, binddataset, bindformdata, isLiveForm, isLiveFilter, role, key, name) {
        var _this = _super.call(this, inj, getWidgetConfig(isLiveForm, isLiveFilter, role)) || this;
        _this.fb = fb;
        _this.app = app;
        _this.dialogService = dialogService;
        _this.dynamicComponentProvider = dynamicComponentProvider;
        _this.ngZone = ngZone;
        _this.parentList = parentList;
        _this.parentForm = parentForm;
        _this.parentPrefab = parentPrefab;
        _this.parentPartial = parentPartial;
        _this.onBeforeSubmitEvt = onBeforeSubmitEvt;
        _this.onSubmitEvt = onSubmitEvt;
        _this.onBeforeRenderEvt = onBeforeRenderEvt;
        _this.binddataset = binddataset;
        _this.bindformdata = bindformdata;
        _this._widgetClass = '';
        _this._captionClass = '';
        _this.formFields = [];
        _this.formfields = {};
        _this.formWidgets = {};
        _this.filterWidgets = {};
        _this.buttonArray = [];
        _this.dataoutput = {};
        _this.formdata = {};
        _this.statusMessage = {
            caption: '',
            type: ''
        };
        _this.primaryKey = [];
        _this._debouncedUpdateFieldSource = _.debounce(_this.updateFieldSource, 350);
        _this.validationMessages = [];
        _this._debouncedSubmitForm = debounce(function ($event) {
            // calling submit event in ngZone as change detection is not triggered post the submit callback and actions like notification are not shown
            _this.ngZone.run(function () {
                _this.submitForm($event);
            });
        }, 250);
        styler(_this.nativeElement, _this);
        _this.isUpdateMode = true;
        _this.dialogId = _this.nativeElement.getAttribute('dialogId');
        _this.ngform = fb.group({});
        _this.bindingValue = key || name;
        // On value change in form, update the dataoutput
        var onValueChangeSubscription = _this.ngform.valueChanges
            .subscribe(_this.updateDataOutput.bind(_this));
        _this.registerDestroyListener(function () { return onValueChangeSubscription.unsubscribe(); });
        _this.elScope = _this;
        _this.addEventsToContext(_this.context);
        return _this;
    }
    FormComponent_1 = FormComponent;
    Object.defineProperty(FormComponent.prototype, "isLayoutDialog", {
        get: function () {
            return this._isLayoutDialog;
        },
        set: function (nv) {
            if (nv) {
                removeClass(this.nativeElement, 'panel app-panel liveform-inline');
            }
            this._isLayoutDialog = nv;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormComponent.prototype, "isUpdateMode", {
        get: function () {
            return this._isUpdateMode;
        },
        set: function (nv) {
            this._isUpdateMode = nv;
            this.formFields.forEach(function (field) {
                field.setReadOnlyState(nv);
            });
        },
        enumerable: true,
        configurable: true
    });
    FormComponent.prototype.submit = function ($event) {
        this._debouncedSubmitForm($event);
    };
    FormComponent.prototype.onReset = function () {
        this.reset();
    };
    Object.defineProperty(FormComponent.prototype, "parentFormArray", {
        // returns the formArray control on the parentForm.
        get: function () {
            return this.parentForm && this.isParentList && this.parentForm.ngform.get(this.parentList.name);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormComponent.prototype, "isParentList", {
        // returns true only when closest livelist is the immediate parent of the form.
        // checks for livelist having any form elements with parentForm name.
        // If livelist is not containing any parentForm elements then it returns true.
        get: function () {
            if (this.parentList && this.parentForm && !isDefined(this._formIsInList)) {
                var formEle = this.$element;
                var listEle = formEle.closest('.app-livelist[name="' + this.parentList.name + '"]');
                if (listEle.length) {
                    this._formIsInList = !(listEle.find('form[widget-id="' + this.parentForm.widgetId + '"]')).length;
                }
            }
            return isDefined(this._formIsInList) ? this._formIsInList : this.parentList;
        },
        enumerable: true,
        configurable: true
    });
    FormComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        setTimeout(function () {
            _this.componentRefs.forEach(function (componentRef) {
                if (componentRef.name) {
                    // Register widgets inside form with formWidgets
                    _this.formWidgets[componentRef.name] = componentRef;
                }
            });
        }, 250);
    };
    FormComponent.prototype.ngAfterViewInit = function () {
        if (this.parentForm && this.parentForm.ngform && this.isParentList) {
            // setting formArray control on ngform.
            if (!_.get(this.parentForm.ngform.controls, this.parentList.name)) {
                this.parentForm.ngform.setControl(this.parentList.name, new FormArray([]));
            }
            // pushing the ngform to formArray.
            this.parentFormArray.push(this.ngform);
            this.formArrayIndex = this.parentFormArray.length - 1;
        }
        this.addInnerNgFormToForm(this.bindingValue);
    };
    FormComponent.prototype.findOperationType = function () { };
    FormComponent.prototype.addInnerNgFormToForm = function (binding) {
        if (this.parentForm && this.parentForm.ngform) {
            // handling as formArray when parent is list. Hence returning here from adding to parentForm again.
            if (this.isParentList) {
                return;
            }
            // assigning the control name of the form inside the prefab with the prefab name.
            // This will happen only when there is only one form inside prefab.
            // We do not support changing this when multiple forms are inside prefab.
            // This scenario has to be handled by end user by changing form names in prefab according to the datamodel binding.
            var parentContainer = this.parentPrefab || this.parentPartial;
            if (parentContainer) {
                var parentContentEl = void 0;
                if (this.parentPrefab) {
                    parentContentEl = parentContainer.$element.find('[wmprefabcontainer]');
                }
                else {
                    parentContentEl = parentContainer.$element;
                }
                /**
                 * assigning the form name to the parentContainer name i.e. partial name or perfab name by following
                 * 1. retrieving all the forms in the parentContainer
                 * 2. consider the first form
                 * 3. if this form is inside list i.e. also find if this list is also inside the parentContainer (prefab / partial)
                 * 4. then do not change the form name when form is inside list and also when there are siblings to the form
                 * 5. this means form name change applies only when there is single form (immediate child) inside the parentContainer.
                 */
                var prefabInnerForm = parentContentEl.find('form').first();
                if (_.get(prefabInnerForm[0], 'widget.widgetId') === this.widgetId) {
                    // check whether form inside prefab container is inside the list. If true, do not change the name.
                    if (!prefabInnerForm.isParentList && !prefabInnerForm.siblings('form').length) {
                        binding = _.get(parentContainer, 'name');
                    }
                }
            }
            var counter = 1;
            var innerBinding = binding;
            // Inner forms may have same names. If same name is present, append unqiue identifier
            while (this.parentForm.ngform.controls.hasOwnProperty(innerBinding)) {
                innerBinding = binding + "_" + counter;
                counter++;
            }
            this.formGroupName = innerBinding;
            // If parent form is present, add the current form as as formGroup for parent form
            this.parentForm.ngform.addControl(innerBinding, this.ngform);
        }
    };
    // Expose the events on context so that they can be accessed by form actions
    FormComponent.prototype.addEventsToContext = function (context) {
        var _this = this;
        context.cancel = function () { return _this.cancel(); };
        context.reset = function () { return _this.reset(); };
        context.save = function (evt) { return _this.save(evt); };
        context.saveAndNew = function () { return _this.saveAndNew(); };
        context.saveAndView = function () { return _this.saveAndView(); };
        context.delete = function () { return _this.delete(); };
        context.new = function () { return _this.new(); };
        context.edit = function () { return _this.edit(); };
        context.highlightInvalidFields = function () { return _this.highlightInvalidFields(); };
        context.filter = function () { return _this.filter(); };
        context.clearFilter = function () { return _this.clearFilter(); };
        context.submit = function (evt) { return _this.submit(evt); };
    };
    // This method gets all the inner forms and validates each form.
    FormComponent.prototype.setValidationOnInnerForms = function (validateTouch) {
        var _this = this;
        var formEle = this.getNativeElement();
        var formObjs = formEle.querySelectorAll('.app-form');
        var validationMsgs = [];
        _.forEach(formObjs, function (e) {
            var formInstance = e.widget;
            // differentiating the validationMessages prefix based on the formGroupName
            // as the formName's are same when forms are in list
            var formName = _.get(formInstance, 'formGroupName') || formInstance.name;
            if (isDefined(formInstance.formArrayIndex)) {
                formName = formName + '[' + formInstance.formArrayIndex + ']';
            }
            var current = formInstance;
            while (_.get(current, 'parentForm')) {
                var parentName = current.parentForm.formGroupName || current.parentForm.name;
                if (current.parentForm.parentFormArray) {
                    parentName = parentName + '[' + current.parentForm.formArrayIndex + ']';
                }
                formName = parentName + '.' + formName;
                current = current.parentForm;
            }
            _this.setValidationOnFields(formInstance, formName, validateTouch);
        });
    };
    /**
     * This method sets validation on formFields.
     * Applies to innerform and also sets innerform validation on parent form.
     * @param prefix contains the form name, which also includes its parents form name
     * @param {boolean} validateTouch
     */
    FormComponent.prototype.setValidationOnFields = function (form, prefix, validateTouch) {
        var _this = this;
        var controls = form.ngform.controls;
        var formFields = form.formFields;
        if (!formFields) {
            return;
        }
        _.forEach(controls, function (v, k) {
            var field = formFields.find(function (e) { return e.key === k; });
            if (!field || (validateTouch && !v.touched)) {
                return;
            }
            // invoking the prepareValidation on both parent form and current form.
            _this.prepareValidationObj(v, k, field, prefix);
            _this.prepareValidationObj.call(form, v, k, field, prefix);
        });
    };
    // Assigns / updates validationMessages based on angular errors on field
    FormComponent.prototype.prepareValidationObj = function (v, k, field, prefix) {
        var index = this.validationMessages.findIndex(function (e) { return (e.field === k && e.fullyQualifiedFormName === prefix); });
        if (v.invalid) {
            if (index === -1) {
                /**
                 * field contains the fieldName
                 * value contains the field value
                 * errorType contains the list of errors
                 * message contains the validation message
                 * getElement returns the element having focus-target
                 * formName returns the name of the form
                 */
                this.validationMessages.push({
                    field: k,
                    value: field.value,
                    errorType: _.keys(v.errors),
                    message: field.validationmessage || '',
                    getElement: function () {
                        return field.$element.find('[focus-target]');
                    },
                    formName: _.last(prefix.split('.')),
                    fullyQualifiedFormName: prefix
                });
            }
            else {
                this.validationMessages[index].value = field.value;
                this.validationMessages[index].errorType = _.keys(v.errors);
            }
        }
        else if (v.valid && index > -1) {
            this.validationMessages.splice(index, 1);
        }
    };
    // This will return a object containing the error details from the list of formFields that are invalid
    FormComponent.prototype.setValidationMsgs = function (validateTouch) {
        if (!this.formFields.length && _.isEmpty(this.ngform.controls)) {
            return;
        }
        this.setValidationOnFields(this, this.name, validateTouch);
        this.setValidationOnInnerForms(validateTouch);
    };
    // change and blur events are added from the template
    FormComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName !== 'submit') {
            _super.prototype.handleEvent.call(this, this.nativeElement, eventName, callback, locals);
        }
    };
    FormComponent.prototype.updateFieldSource = function () {
        var _this = this;
        if (this.formdatasource && this.formdatasource.execute(DataSource.Operation.IS_API_AWARE)) {
            return;
        }
        else if (this.formdatasource && !this.formdatasource.twoWayBinding) {
            return;
        }
        this.formFields.forEach(function (formField) {
            formField.setFormWidget('datavaluesource', _this.formdatasource);
            formField.setFormWidget('binddatavalue', _this.bindformdata + "." + formField.key);
        });
    };
    // This method loops through the form fields and highlights the invalid fields by setting state to touched
    FormComponent.prototype.highlightInvalidFields = function () {
        setTouchedState(this.ngform);
    };
    // Disable the form submit if form is in invalid state. Highlight all the invalid fields if validation type is default
    FormComponent.prototype.validateFieldsOnSubmit = function () {
        this.setValidationMsgs();
        // Disable the form submit if form is in invalid state. For delete operation, do not check the validation.
        if (this.operationType !== 'delete' && (this.validationtype === 'html' || this.validationtype === 'default')
            && this.ngform && this.ngform.invalid) {
            if (this.ngform.invalid) {
                if (this.validationtype === 'default') {
                    this.highlightInvalidFields();
                }
                // Find the first invalid untoched element and set it to touched.
                // Safari does not form validations. this will ensure that error is shown for user
                var eleForm = findInvalidElement(this.$element, this.ngform);
                var $invalidForm = eleForm.ngForm;
                var $invalidEle = eleForm.$ele;
                $invalidEle = $invalidEle.parent().find('[focus-target]');
                if ($invalidEle.length) {
                    // on save click in page layout liveform, focus of autocomplete widget opens full-screen search.
                    if (!$invalidEle.hasClass('app-search-input')) {
                        $invalidEle.focus();
                    }
                    var ngEle = $invalidForm && $invalidForm.controls[$invalidEle.attr('formControlName') || $invalidEle.attr('name')];
                    if (ngEle && ngEle.markAsTouched) {
                        ngEle.markAsTouched();
                    }
                    $appDigest();
                    return true;
                }
                return true;
            }
            return false;
        }
        return false;
    };
    FormComponent.prototype.onPropertyChange = function (key, nv, ov) {
        switch (key) {
            case 'captionalign':
                this.captionAlignClass = 'align-' + nv;
                break;
            case 'captionposition':
                this.setLayoutConfig();
                break;
            case 'captionwidth':
                this.setLayoutConfig();
                break;
            case 'captionsize':
                this.captionsize = nv;
                break;
            case 'novalidate':
                //  Set validation type based on the novalidate property
                this.widget.validationtype = (nv === true || nv === 'true') ? 'none' : 'default';
                break;
            case 'formdata':
                // For livelist when multiselect is enabled, formdata will be array of objects. In this case consider the last object as formdata.
                _.isArray(nv) ? this.setFormData(_.last(nv)) : this.setFormData(nv);
                // if dataset on the formFields are not set as the datasourceChange is triggered before the formFields are registered.
                if (!this.isDataSourceUpdated && this.datasource) {
                    this.onDataSourceChange();
                }
                break;
            case 'defaultmode':
                if (!this.isLayoutDialog) {
                    if (nv && nv === 'Edit') {
                        this.updateMode = true;
                    }
                    else {
                        this.updateMode = false;
                    }
                    this.isUpdateMode = this.updateMode;
                }
                break;
            case 'datasource':
                this.onDataSourceChange();
                break;
            case 'formdatasource':
                this.onFormDataSourceChange();
                break;
            case 'metadata':
                this.generateFormFields();
                break;
            case 'dataset':
                var formFields = this.getFormFields();
                formFields.forEach(function (field) {
                    // notifying the dataset change to the form-field widget.
                    if (!field.isDataSetBound && _.get(field.formWidget, 'dataset$')) {
                        field.formWidget.dataset$.next();
                    }
                });
                break;
            default:
                _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    // Event callbacks on success/error
    FormComponent.prototype.onResultCb = function (data, status, event) {
        var params = { $event: event, $data: data, $operation: this.operationType };
        // whether service call success or failure call this method
        this.invokeEventCallback('result', params);
        if (status) {
            // if service call is success call this method
            this.invokeEventCallback('success', params);
        }
        else {
            // if service call fails call this method
            this.invokeEventCallback('error', params);
        }
    };
    // Display or hide the inline message/ toaster
    FormComponent.prototype.toggleMessage = function (show, msg, type, header) {
        var template;
        if (show && msg) {
            template = (type === 'error' && this.errormessage) ? this.errormessage : msg;
            if (this.messagelayout === 'Inline') {
                template = this.checkAppServiceErrorMsg(type) || template;
                this.statusMessage = { 'caption': template || '', type: type };
                if (this.messageRef) {
                    this.messageRef.showMessage(this.statusMessage.caption, this.statusMessage.type);
                }
            }
            else {
                this.app.notifyApp(template, type, header);
            }
        }
        else {
            this.statusMessage.caption = '';
        }
    };
    // if there is an App.onServiceError handler for the app, that message should be used instead of server returned error message
    FormComponent.prototype.checkAppServiceErrorMsg = function (type) {
        var notificationAction = _.get(this.app, 'Actions.appNotification');
        if (notificationAction && type === 'error') {
            return notificationAction.getMessage();
        }
        return;
    };
    // Hide the inline message/ toaster
    FormComponent.prototype.clearMessage = function () {
        this.toggleMessage(false);
    };
    // Set the classes on the form based on the captionposition and captionwidth properties
    FormComponent.prototype.setLayoutConfig = function () {
        var layoutConfig;
        layoutConfig = getFieldLayoutConfig(this.captionwidth, this.captionposition, _.get(this.app.selectedViewPort, 'os'));
        this._widgetClass = layoutConfig.widgetCls;
        this._captionClass = layoutConfig.captionCls;
        $appDigest();
    };
    FormComponent.prototype.registerFormWidget = function (widget) {
        var name = widget.name || widget.key;
        this.formWidgets[name] = widget;
    };
    FormComponent.prototype.registerFormFields = function (formField) {
        var fieldCount = this.formFields.push(formField);
        this.formfields[formField.key] = formField;
        this.registerFormWidget(formField);
        this._debouncedUpdateFieldSource();
        if (this.parentForm) {
            this.parentForm.formFields.push(formField);
            this.parentForm.formfields[formField.key] = formField;
            // inner formfields are pushed to parentForm, passing current innerForm's formdata to set innerFormdata to these innerFormFields
            this.parentForm.setFormData(this.parentForm.formdata, this.formFields, this.formdata || {});
        }
        // WMS-18967: Applying formadata after formfields are ready
        if (fieldCount === this.numberOfFields && !_.isEmpty(this.formdata)) {
            this.onDataSourceChange();
            _.isArray(this.formdata) ? this.setFormData(_.last(this.formdata)) : this.setFormData(this.formdata);
        }
    };
    FormComponent.prototype.registerActions = function (formAction) {
        this.buttonArray.push(formAction);
    };
    // Update the dataoutput whenever there is a change in inside form widget value
    FormComponent.prototype.updateFormDataOutput = function (dataObject) {
        var _this = this;
        // Set the values of the widgets inside the live form (other than form fields) in form data
        _.forEach(this.ngform.value, function (val, key) {
            if (!_.find(_this.formFields, { key: key })) {
                dataObject[key] = val;
            }
        });
        this.dataoutput = dataObject;
    };
    // Construct the data object merging the form fields and custom widgets data
    FormComponent.prototype.constructDataObject = function () {
        var _this = this;
        var formData = {};
        var formFields = this.getFormFields();
        // Get all form fields and prepare form data as key value pairs
        formFields.forEach(function (field) {
            var fieldName, fieldValue;
            fieldValue = field.datavalue || field._control.value;
            fieldValue = (fieldValue === null || fieldValue === '') ? undefined : fieldValue;
            if (field.type === 'file') {
                fieldValue = getFiles(_this.name, field.key + '_formWidget', field.multiple);
            }
            fieldName = field.key || field.target || field.name;
            // on current Form, when field is innerForm field then we group these fields under new object using formGroupName as key
            // For suppose, applicantName, applicantID are formfields in currentForm
            // This form is having innerform i.e. address, these fields are listed as
            // {"applicantName": "", "applicantID": "", "address": {"street": ""}}
            if (field.form.widgetId !== _this.widgetId && field.form.formGroupName) {
                var fd = _.get(formData, field.form.formGroupName, {});
                fd[fieldName] = fieldValue;
                _.set(formData, field.form.formGroupName, fd);
            }
            else if (field.form.isParentList) {
                // setting formdata based on formArrayIndex
                var fd = _.get(formData, field.form.parentList.name, []);
                fd[field.form.formArrayIndex] = fd[field.form.formArrayIndex] || {};
                fd[field.form.formArrayIndex][fieldName] = fieldValue;
                _.set(formData, field.form.parentList.name, fd);
            }
            else {
                // In case of update the field will be already present in form data
                _.set(formData, fieldName, fieldValue);
            }
        });
        this.updateFormDataOutput(formData);
        return this.dataoutput;
    };
    FormComponent.prototype.updateDataOutput = function () {
        this.constructDataObject();
        if (this.ngform.touched) {
            this.setValidationMsgs(true);
        }
    };
    // FormFields will contain all the fields in parent and inner form also.
    // This returns the formFields in the form based on the form name.
    FormComponent.prototype.getFormFields = function () {
        var _this = this;
        return _.filter(this.formFields, function (formField) {
            return formField.form.name === _this.name;
        });
    };
    FormComponent.prototype.setFieldValue = function (field, data, innerFormdata) {
        var key = field.key || field.name;
        // if customfield param value is not in the formdata then do not assign field value
        // as it can contain default value which will again be overridden by undefined.
        var fd = innerFormdata ? innerFormdata : data;
        if (fd) {
            if (fd.hasOwnProperty(key)) {
                field.value = _.get(fd, key);
            }
            else if (_.includes(key, '.')) {
                // key contains '.' when mapping the fields to child reference i.e. childCol is having key as "parent.childCol"
                if (fd.hasOwnProperty(_.split(key, '.')[0])) {
                    field.value = _.get(fd, key);
                }
            }
        }
        var formGroupName = field.form.formGroupName;
        /**
         * if formGroupName is defined which means field is inside the inner form
         * then set the formdata on the field's form using formGroupName
         */
        if (formGroupName && _.get(data, formGroupName)) {
            this.setFormData.call(field.form, data[formGroupName]);
        }
        // if formdata is assigned later then on propertyChangeHandler, even inner forms data also needs to be updated.
        if (_.get(field.form, 'parentFormArray')) {
            this.setFormDataFromParentFormData.call(field.form, data);
        }
    };
    FormComponent.prototype.setFormData = function (data, innerFormFields, innerFormdata) {
        var _this = this;
        // whereas formFields explicitly passed can contain innerform fields also.
        var formFields = innerFormFields || this.formFields;
        formFields.forEach(function (field) {
            _this.setFieldValue(field, data, innerFormdata);
        });
        this.constructDataObject();
    };
    FormComponent.prototype.resetFormState = function () {
        // clearing the validationMessages on reset.
        if (this.validationMessages.length) {
            this.validationMessages = [];
        }
        if (!this.ngform) {
            return;
        }
        this.ngform.markAsUntouched();
        this.ngform.markAsPristine();
    };
    FormComponent.prototype.reset = function () {
        var _this = this;
        this.resetFormState();
        this.formFields.forEach(function (field) {
            field.value = '';
            if (field.widgettype === 'upload' || field.type === DataType.BLOB) {
                _this.resetFileUploadWidget(field, true);
            }
        });
        this.constructDataObject();
        this.clearMessage();
    };
    FormComponent.prototype.savePrevformFields = function () {
        this.prevformFields = getClonedObject(this.formFields.map(function (field) {
            return {
                'key': field.key,
                'type': field.type,
                'widgettype': field.widgettype,
                'outputformat': field.outputformat,
                'value': field.value
            };
        }));
    };
    FormComponent.prototype.savePrevDataValues = function () {
        this.prevDataValues = this.formFields.map(function (obj) {
            return { 'key': obj.key, 'value': obj.value };
        });
    };
    FormComponent.prototype.setReadonlyFields = function () {
        this.formFields.forEach(function (field) {
            field.setReadOnlyState();
        });
    };
    FormComponent.prototype.resetFileUploadWidget = function (field, skipValueSet) {
        var $formEle = this.$element;
        $formEle.find('[name="' + field.key + '_formWidget"]').val('');
        field._control.reset();
        if (!skipValueSet) {
            field.href = '';
            field.value = null;
        }
    };
    FormComponent.prototype.emptyDataModel = function () {
        var _this = this;
        this.formFields.forEach(function (field) {
            if (isDefined(field)) {
                if (field.type === DataType.BLOB) {
                    _this.resetFileUploadWidget(field);
                }
                else {
                    field.datavalue = '';
                }
            }
        });
    };
    FormComponent.prototype.setDefaultValues = function () {
        var _this = this;
        this.formFields.forEach(function (field) {
            _this.onFieldDefaultValueChange(field, field.defaultvalue);
        });
    };
    FormComponent.prototype.setOperationType = function (mode) {
        this.operationType = mode || 'insert';
    };
    FormComponent.prototype.new = function () {
        var _this = this;
        this.resetFormState();
        this.operationType = Live_Operations.INSERT;
        this.clearMessage();
        if (this.isSelected) {
            this.savePrevformFields();
        }
        this.emptyDataModel();
        setTimeout(function () {
            _this.setDefaultValues();
            _this.savePrevDataValues();
            _this.constructDataObject();
        });
        this.isUpdateMode = true;
    };
    FormComponent.prototype.edit = function () {
        this.resetFormState();
        this.clearMessage();
        this.operationType = Live_Operations.UPDATE;
        if (this.isSelected) {
            this.savePrevformFields();
            this.savePrevDataValues();
        }
        this.prevDataObject = getClonedObject(this.formdata || {});
        this.setReadonlyFields();
        this.isUpdateMode = true;
        $appDigest();
    };
    FormComponent.prototype.triggerWMEvent = function (eventName) {
        var tableId = this.nativeElement.getAttribute('dependson');
        if (!tableId) {
            return;
        }
        $invokeWatchers(true);
        this.app.notify('wm-event', { eventName: eventName, widgetName: tableId });
    };
    FormComponent.prototype.submitForm = function ($event) {
        var _this = this;
        var template, formData, result;
        var dataSource = this.datasource;
        // Disable the form submit if form is in invalid state.
        if (this.validateFieldsOnSubmit()) {
            return;
        }
        formData = getClonedObject(this.constructDataObject());
        var getParams = function () {
            return { $event: $event, $formData: formData, $data: formData };
        };
        if (this.onBeforeSubmitEvt) {
            if (this.invokeEventCallback('beforesubmit', getParams()) === false) {
                return;
            }
            else {
                this.resetFormState();
            }
        }
        else {
            this.resetFormState();
        }
        if (this.onSubmitEvt || dataSource) {
            // If on submit is there execute it and if it returns true do service variable invoke else return
            // If its a service variable call setInput and assign form data and invoke the service
            if (dataSource) {
                var currentPageNum_1 = dataSource.pagination && dataSource.pagination.number + 1;
                var operationType = this.operationType ? this.operationType : 'insert';
                performDataOperation(dataSource, formData, { operationType: operationType })
                    .then(function (data) {
                    if (dataSource.category === 'wm.CrudVariable') {
                        _this.triggerWMEvent('resetEditMode');
                        _this.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                            'skipToggleState': true,
                            'operation': 'list',
                            'page': currentPageNum_1,
                            'preventMissingOpMsg': true
                        });
                        if (_this.dialogId) {
                            _this.closeDialog();
                        }
                    }
                    if (isDefined(data.body)) {
                        result = getValidJSON(data.body);
                        result = isDefined(result) ? result : data.body;
                    }
                    else {
                        result = data;
                    }
                    return {
                        'result': result,
                        'status': true,
                        'message': _this.postmessage,
                        'type': 'success'
                    };
                }, function (error) {
                    template = _this.errormessage || error.error || error;
                    $appDigest();
                    return {
                        'result': _.get(error, 'error') || error,
                        'status': false,
                        'message': template,
                        'type': 'error'
                    };
                }).then(function (response) {
                    if ((response.type === 'error' && !_this.onError) || (response.type === 'success' && !_this.onSuccess)) {
                        _this.toggleMessage(true, response.message, response.type);
                    }
                    _this.invokeEventCallback('submit', getParams());
                    _this.onResultCb(response.result, response.status, $event);
                });
            }
            else {
                this.invokeEventCallback('submit', getParams());
                this.onResultCb({}, true, $event);
            }
        }
        else {
            this.onResultCb({}, true, $event);
        }
    };
    FormComponent.prototype.closeDialog = function () {
        this.dialogService.close(this.dialogId);
    };
    // Method to show/hide the panel header or footer based on the buttons
    FormComponent.prototype.showButtons = function (position) {
        var _this = this;
        return _.some(this.buttonArray, function (btn) {
            return _.includes(btn.position, position) && btn.updateMode === _this.isUpdateMode;
        });
    };
    // Expand or collapse the panel
    FormComponent.prototype.expandCollapsePanel = function () {
        if (this.collapsible) {
            // flip the active flag
            this.expanded = !this.expanded;
        }
    };
    FormComponent.prototype.onDataSourceUpdate = function (response, newForm, updateMode) {
        if (newForm) {
            this.new();
        }
        else {
            this.setFormData(response);
            this.closeDialog();
        }
        this.isUpdateMode = isDefined(updateMode) ? updateMode : true;
    };
    /**
     * This method tranverses through the parent containing formdata and returns the parent formdata.
     * @param form, represents the current form instance
     * @returns {any}
     */
    FormComponent.prototype.getNearestParentFormData = function (form) {
        if (!form.parentForm) {
            return {};
        }
        if (!form.bindformdata && _.get(form.parentForm, 'formdata') && !_.isEmpty(form.parentForm.formdata)) {
            return form.parentForm.formdata;
        }
        return this.getNearestParentFormData(form.parentForm);
    };
    FormComponent.prototype.setFormDataFromParentFormData = function (formdata) {
        if (_.isEmpty(this.formdata) && this.parentForm && formdata) {
            /**
             * If form is inside list,
             * 1. directly applying the formdata to the formArray to update the formdata on list of forms using patchValue.
             * 2. and also setting individual formdata to the form inside each item based on formArrayIndex, when form is not bound to formdata.
             * 3. explicitly invoking formdata as formFields will contain all the formfields including innerForm's fields
             * (if we do not invoke explicitly then formFields will just contain the fields inside current form and not inner form fields)
             * 4. if childFormArrayData is not available we will check for inner form data using "formGroupName" and will set the innerform data
             * If not inside list, just set the formdata on all the formfields including innerForm fields
             */
            if (this.parentFormArray) {
                var childFormArrayData = _.get(formdata, this.parentList.name);
                if (childFormArrayData) {
                    if (!_.isArray(childFormArrayData)) {
                        childFormArrayData = [childFormArrayData];
                    }
                    this.parentFormArray.patchValue(childFormArrayData);
                    if (!this.bindformdata) {
                        this.formdata = childFormArrayData[this.formArrayIndex];
                    }
                    this.setFormData(this.formdata);
                }
                else if (this.isParentList && this.parentForm.formGroupName) {
                    var parentFormData = _.get(formdata, this.parentForm.formGroupName);
                    this.setFormDataFromParentFormData(parentFormData);
                }
            }
            else {
                this.setFormData(formdata);
            }
        }
    };
    // On form data source change. This method is overridden by live form and live filter
    FormComponent.prototype.onDataSourceChange = function () {
        if (_.get(this.formFields, 'length') && !this.bindformdata) {
            this.isDataSourceUpdated = true;
            /**
             * formdata on the parent form will be set before the inner forms are rendered.
             * Hence handling formdata on the innerForms, which might be rendered slowly (suppose prefab with form)
             * setting inner form's formdata obtained from parent formdata
             * this applies only when formdata is not given on the inner form.
             */
            var formdata = this.getNearestParentFormData(this);
            this.setFormDataFromParentFormData(formdata);
        }
    };
    // On form data source change. This method is overridden by live form and live filter
    FormComponent.prototype.onFormDataSourceChange = function () {
        this.updateFieldSource();
    };
    // On form field default value change. This method is overridden by live form and live filter
    FormComponent.prototype.onFieldDefaultValueChange = function (field, nv) {
        field.value = parseValueByType(nv, undefined, field.widgettype, field.trailingzero);
    };
    // On form field value change. This method is overridden by live form and live filter
    FormComponent.prototype.onFieldValueChange = function () {
    };
    // Function to generate and compile the form fields from the metadata
    FormComponent.prototype.generateFormFields = function () {
        return __awaiter(this, void 0, void 0, function () {
            var noOfColumns, $gridLayout, columnWidth, fieldTemplate, colCount, index, userFields, fields, colTmpl, componentFactoryRef, component;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // Check if grid layout is present or not for first time
                        if (_.isUndefined(this._isGridLayoutPresent)) {
                            this._isGridLayoutPresent = this.$element.find('.panel-body [wmlayoutgrid]').length > 0;
                        }
                        if (this._isGridLayoutPresent) {
                            $gridLayout = this.$element.find('.form-elements [wmlayoutgrid]').first();
                            noOfColumns = Number($gridLayout.attr('columns')) || 1;
                        }
                        else {
                            $gridLayout = this.$element.find('.form-elements .dynamic-form-container');
                            if (!$gridLayout.length) {
                                this.$element.find('.form-elements').prepend('<div class="dynamic-form-container"></div>');
                                $gridLayout = this.$element.find('.form-elements .dynamic-form-container');
                            }
                            noOfColumns = 1;
                        }
                        columnWidth = 12 / noOfColumns;
                        fieldTemplate = '';
                        colCount = 0;
                        fields = this.metadata ? this.metadata.data || this.metadata : [];
                        this.formFields = []; // empty the form fields
                        if (_.isEmpty(fields)) {
                            return [2 /*return*/];
                        }
                        if (this.onBeforeRenderEvt) {
                            userFields = this.invokeEventCallback('beforerender', { $metadata: fields });
                            if (userFields) {
                                fields = userFields;
                            }
                        }
                        if (!_.isArray(fields)) {
                            return [2 /*return*/];
                        }
                        while (fields[colCount]) {
                            colTmpl = '';
                            for (index = 0; index < noOfColumns; index++) {
                                if (fields[colCount]) {
                                    colTmpl += setMarkupForFormField(fields[colCount], columnWidth);
                                }
                                colCount++;
                            }
                            fieldTemplate += "<wm-gridrow>" + colTmpl + "</wm-gridrow>";
                        }
                        if (!this._isGridLayoutPresent) {
                            fieldTemplate = "<wm-layoutgrid>" + fieldTemplate + "</wm-layoutgrid>";
                        }
                        this.dynamicFormRef.clear();
                        if (!this._dynamicContext) {
                            this._dynamicContext = Object.create(this.viewParent);
                            this._dynamicContext.form = this;
                        }
                        return [4 /*yield*/, this.dynamicComponentProvider.getComponentFactoryRef('app-form-dynamic-' + this.widgetId, fieldTemplate, {
                                noCache: true,
                                transpile: true
                            })];
                    case 1:
                        componentFactoryRef = _a.sent();
                        component = this.dynamicFormRef.createComponent(componentFactoryRef, 0, this.inj);
                        extendProto(component.instance, this._dynamicContext);
                        $gridLayout[0].appendChild(component.location.nativeElement);
                        this.setFormData(this.formdata);
                        return [2 /*return*/];
                }
            });
        });
    };
    Object.defineProperty(FormComponent.prototype, "mode", {
        get: function () {
            return this.operationType || this.findOperationType();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormComponent.prototype, "dirty", {
        get: function () {
            return this.ngform && this.ngform.dirty;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * This method sets the form state to pristine by internally calling angular markAsPristine method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    FormComponent.prototype.markAsPristine = function (value) {
        if (value === void 0) { value = false; }
        this.ngform.markAsPristine({ onlySelf: value });
    };
    /**
     * This method sets the form state to dirty by internally calling angular markAsDirty method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    FormComponent.prototype.markAsDirty = function (value) {
        if (value === void 0) { value = false; }
        this.ngform.markAsDirty({ onlySelf: value });
    };
    Object.defineProperty(FormComponent.prototype, "invalid", {
        get: function () {
            return this.ngform && this.ngform.invalid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormComponent.prototype, "touched", {
        get: function () {
            return this.ngform && this.ngform.touched;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormComponent.prototype, "valid", {
        get: function () {
            return this.ngform && this.ngform.valid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormComponent.prototype, "pristine", {
        get: function () {
            return this.ngform && this.ngform.pristine;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * This method sets the form state to touched by internally calling angular markAsTouched method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    FormComponent.prototype.markAsTouched = function (value) {
        if (value === void 0) { value = false; }
        this.ngform.markAsTouched({ onlySelf: value });
    };
    /**
     * This method sets the form state to untouched by internally calling angular markAsUntouched method on the form
     * @param value, When true, mark only this control. When false or not supplied, marks all direct ancestors. Default is false
     * @returns {void}
     */
    FormComponent.prototype.markAsUntouched = function (value) {
        if (value === void 0) { value = false; }
        this.ngform.markAsUntouched({ onlySelf: value });
    };
    FormComponent.prototype.invokeActionEvent = function ($event, expression) {
        var fn = $parseEvent(expression);
        fn(this.viewParent, Object.assign(this.context, { $event: $event }));
    };
    FormComponent.prototype.ngOnDestroy = function () {
        // on form destroy, removing this form from the parentForm too.
        var controls = this.parentForm && _.get(this.parentForm, 'ngform.controls');
        if (controls) {
            // when current form is inside the list (i.e. incase of formArray).
            if (_.get(this.parentList, 'name') && controls.hasOwnProperty(this.parentList.name)) {
                this.parentForm.ngform.removeControl(this.parentList.name);
            }
            // when we have formGroupName set i.e. multiple formInstance with counter appended to formName
            if (this.formGroupName && controls.hasOwnProperty(this.formGroupName)) {
                this.parentForm.ngform.removeControl(this.formGroupName);
            }
            else {
                this.parentForm.ngform.removeControl(this.name);
            }
        }
        _super.prototype.ngOnDestroy.call(this);
    };
    var FormComponent_1;
    FormComponent.initializeProps = registerFormProps();
    FormComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: FormBuilder },
        { type: App },
        { type: AbstractDialogService },
        { type: DynamicComponentRefProvider },
        { type: NgZone },
        { type: ListComponent, decorators: [{ type: Optional }] },
        { type: FormComponent, decorators: [{ type: SkipSelf }, { type: Optional }] },
        { type: PrefabDirective, decorators: [{ type: SkipSelf }, { type: Optional }] },
        { type: PartialDirective, decorators: [{ type: SkipSelf }, { type: Optional }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['beforesubmit.event',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['submit.event',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['beforerender.event',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['formdata.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['wmLiveForm',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['wmLiveFilter',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['role',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['key',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['name',] }] }
    ]; };
    __decorate([
        ViewChild('dynamicForm', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], FormComponent.prototype, "dynamicFormRef", void 0);
    __decorate([
        ViewChild(MessageComponent),
        __metadata("design:type", Object)
    ], FormComponent.prototype, "messageRef", void 0);
    __decorate([
        ContentChildren(WidgetRef, { descendants: true }),
        __metadata("design:type", Object)
    ], FormComponent.prototype, "componentRefs", void 0);
    __decorate([
        HostBinding('action'),
        __metadata("design:type", String)
    ], FormComponent.prototype, "action", void 0);
    __decorate([
        HostListener('submit', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], FormComponent.prototype, "submit", null);
    __decorate([
        HostListener('reset'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], FormComponent.prototype, "onReset", null);
    FormComponent = FormComponent_1 = __decorate([
        Component({
            selector: 'form[wmForm]',
            template: "<ng-template #content><ng-content></ng-content></ng-template>\n\n<ng-container *ngIf=\"formlayout === 'page'; then pageLayoutTemplate else defaultTemplate\">\n</ng-container>\n\n<ng-template #defaultTemplate>\n    <div class=\"panel-heading\" *ngIf=\"!isLayoutDialog && (title || subheading || iconclass || showButtons('header'))\">\n        <h3 class=\"panel-title\">\n            <div class=\"pull-left\">\n                <i class=\"app-icon panel-icon\" aria-hidden=\"true\" [ngClass]=\"iconclass\" *ngIf=\"iconclass\"></i>\n            </div>\n            <div class=\"pull-left\">\n                <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\"></div>\n                <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n            </div>\n            <div class=\"form-action panel-actions basic-btn-grp\">\n                <ng-container *ngFor=\"let btn of buttonArray | filter : 'position' : 'header'\"\n                              [ngTemplateOutlet]=\"btn.widgetType === 'button' ? buttonRef : anchorRef\" [ngTemplateOutletContext]=\"{btn:btn}\">\n                </ng-container>\n                <button type=\"button\" class=\"app-icon wi panel-action\" *ngIf=\"collapsible\" title=\"Collapse/Expand\"\n                        [ngClass]=\"expanded ? 'wi-minus': 'wi-plus'\" (click)=\"expandCollapsePanel();\"></button>\n            </div>\n        </h3>\n    </div>\n    <div class=\"panel-body\" [ngClass]=\"{'form-elements': isLayoutDialog, 'form-body': !isLayoutDialog}\" [class.hidden]=\"!expanded\">\n        <p wmMessage [hidden]=\"!statusMessage.caption.toString()\" caption.bind=\"statusMessage.caption\" type.bind=\"statusMessage.type\"></p>\n        <div [ngClass]=\"{'form-elements': !isLayoutDialog, 'form-content': isLayoutDialog}\">\n            <ng-container *ngTemplateOutlet=\"content\"></ng-container>\n        </div>\n        <div class=\"basic-btn-grp form-action panel-footer clearfix\" [ngClass]=\"{'modal-footer': isLayoutDialog}\" [class.hidden]=\"!expanded || !showButtons('footer')\">\n            <ng-container *ngFor=\"let btn of buttonArray | filter : 'position' : 'footer'\"\n                          [ngTemplateOutlet]=\"btn.widgetType === 'button' ? buttonRef : anchorRef\" [ngTemplateOutletContext]=\"{btn:btn}\">\n            </ng-container>\n        </div>\n    </div>\n</ng-template>\n\n<ng-template #pageLayoutTemplate>\n    <ng-container *ngTemplateOutlet=\"content\"></ng-container>\n</ng-template>\n\n<ng-template #buttonRef let-btn=\"btn\">\n    <button  wmButton name=\"{{btn.key}}\" caption.bind=\"btn.displayName\" class.bind=\"btn.class\" iconclass.bind=\"btn.iconclass\" show.bind=\"btn.show\"\n             (click)=\"invokeActionEvent($event, btn.action)\" type.bind=\"btn.type\" hint.bind=\"btn.title\" shortcutkey.bind=\"btn.shortcutkey\" disabled.bind=\"ngform.status === 'PENDING' || btn.disabled\"\n             tabindex.bind=\"btn.tabindex\" [class.hidden]=\"btn.updateMode ? !isUpdateMode : isUpdateMode\"></button>\n</ng-template>\n\n<ng-template #anchorRef let-btn=\"btn\">\n    <a wmAnchor name=\"{{btn.key}}\" caption.bind=\"btn.displayName\" class.bind=\"btn.class\" iconclass.bind=\"btn.iconclass\" show.bind=\"btn.show\"\n             (click)=\"invokeActionEvent($event, btn.action)\" hint.bind=\"btn.title\" shortcutkey.bind=\"btn.shortcutkey\"\n             hyperlink.bind=\"btn.hyperlink\" target.bind=\"btn.target\"\n             tabindex.bind=\"btn.tabindex\" [class.hidden]=\"btn.updateMode ? !isUpdateMode : isUpdateMode\"></a>\n</ng-template>\n\n<ng-container #dynamicForm></ng-container>\n",
            providers: [
                provideAsWidgetRef(FormComponent_1)
            ]
        }),
        __param(6, Optional()),
        __param(7, SkipSelf()), __param(7, Optional()),
        __param(8, SkipSelf()), __param(8, Optional()),
        __param(9, SkipSelf()), __param(9, Optional()),
        __param(10, Attribute('beforesubmit.event')),
        __param(11, Attribute('submit.event')),
        __param(12, Attribute('beforerender.event')),
        __param(13, Attribute('dataset.bind')),
        __param(14, Attribute('formdata.bind')),
        __param(15, Attribute('wmLiveForm')),
        __param(16, Attribute('wmLiveFilter')),
        __param(17, Attribute('role')),
        __param(18, Attribute('key')),
        __param(19, Attribute('name')),
        __metadata("design:paramtypes", [Injector,
            FormBuilder,
            App,
            AbstractDialogService,
            DynamicComponentRefProvider,
            NgZone,
            ListComponent,
            FormComponent,
            PrefabDirective,
            PartialDirective, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object])
    ], FormComponent);
    return FormComponent;
}(StylableComponent));

var FormWidgetDirective = /** @class */ (function () {
    function FormWidgetDirective(form, table, componentInstance, fb, name, key) {
        var _this = this;
        this.componentInstance = componentInstance;
        this.name = name;
        this.key = key;
        this.fb = fb;
        this.parent = form || table;
        this.ngform = this.parent.ngform;
        this.componentInstance.registerPropertyChangeListener(function (k, nv) {
            if (k === 'datavalue' && _this._control) {
                _this._control.setValue(nv);
            }
            else if (k === 'name' && !_this._control) {
                _this.addControl(_this.key || nv);
            }
        });
    }
    Object.defineProperty(FormWidgetDirective.prototype, "_control", {
        get: function () {
            var fieldName = this.key || this.name;
            if (!fieldName) {
                return undefined;
            }
            return this.ngform && this.ngform.controls[fieldName];
        },
        enumerable: true,
        configurable: true
    });
    FormWidgetDirective.prototype.addControl = function (fieldName) {
        this.ngform.addControl(fieldName, this.createControl());
    };
    FormWidgetDirective.prototype.createControl = function () {
        return this.fb.control(this.componentInstance.datavalue);
    };
    FormWidgetDirective.prototype.ngOnInit = function () {
        var fieldName = this.key || this.name;
        if (fieldName && !this._control) {
            this.addControl(fieldName);
            this.parent.registerFormWidget(this.componentInstance);
        }
    };
    FormWidgetDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [FormComponent,] }] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [TableComponent,] }] },
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [WidgetRef,] }] },
        { type: FormBuilder },
        { type: undefined, decorators: [{ type: Attribute, args: ['name',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['key',] }] }
    ]; };
    FormWidgetDirective = __decorate([
        Directive({
            selector: '[wmFormWidget]'
        }),
        __param(0, Optional()), __param(0, Inject(FormComponent)),
        __param(1, Optional()), __param(1, Inject(TableComponent)),
        __param(2, Self()), __param(2, Inject(WidgetRef)),
        __param(4, Attribute('name')),
        __param(5, Attribute('key')),
        __metadata("design:paramtypes", [Object, Object, Object, FormBuilder, Object, Object])
    ], FormWidgetDirective);
    return FormWidgetDirective;
}());

var registerProps = function () {
    register('wm-form-action', new Map([
        ['accessroles', PROP_STRING],
        ['action', PROP_STRING],
        ['class', PROP_STRING],
        ['binding', PROP_STRING],
        ['disabled', PROP_BOOLEAN],
        ['display-name', PROP_STRING],
        ['hyperlink', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconname', PROP_STRING],
        ['key', PROP_STRING],
        ['position', __assign({ value: 'footer' }, PROP_STRING)],
        ['shortcutkey', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['target', PROP_STRING],
        ['title', PROP_STRING],
        ['type', __assign({ value: 'button' }, PROP_STRING)],
        ['update-mode', __assign({ value: true }, PROP_BOOLEAN)],
        ['widget-type', __assign({ value: 'button' }, PROP_STRING)]
    ]));
};

var WIDGET_CONFIG$1 = { widgetType: 'wm-form-action', hostClass: '' };
var FormActionDirective = /** @class */ (function (_super) {
    __extends(FormActionDirective, _super);
    function FormActionDirective(inj, form) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        _this.form = form;
        return _this;
    }
    FormActionDirective_1 = FormActionDirective;
    FormActionDirective.prototype.populateAction = function () {
        this.buttonDef = {
            key: this.key || this.binding,
            displayName: this['display-name'],
            show: this.show,
            class: this.class ? this.class : (this['widget-type'] === 'button' ? 'btn-secondary' : ''),
            iconclass: this.iconclass || '',
            title: _.isUndefined(this.title) ? (this['display-name'] || '') : this.title,
            action: this.action,
            accessroles: this.accessroles,
            shortcutkey: this.shortcutkey,
            disabled: this.disabled,
            tabindex: this.tabindex,
            iconname: this.iconname,
            type: this.type,
            updateMode: this['update-mode'],
            position: this.position,
            widgetType: this['widget-type'],
            hyperlink: this.hyperlink,
            target: this.target
        };
        this._propsInitialized = true;
    };
    FormActionDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.populateAction();
        this.form.registerActions(this.buttonDef);
    };
    FormActionDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (!this._propsInitialized) {
            return;
        }
        switch (key) {
            case 'display-name':
                this.buttonDef.displayName = nv;
            default:
                this.buttonDef[key] = nv;
                break;
        }
    };
    var FormActionDirective_1;
    FormActionDirective.initializeProps = registerProps();
    FormActionDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: FormComponent, decorators: [{ type: Optional }] }
    ]; };
    FormActionDirective = FormActionDirective_1 = __decorate([
        Directive({
            selector: '[wmFormAction]',
            providers: [
                provideAsWidgetRef(FormActionDirective_1)
            ]
        }),
        __param(1, Optional()),
        __metadata("design:paramtypes", [Injector, FormComponent])
    ], FormActionDirective);
    return FormActionDirective;
}(BaseComponent));

var registerProps$1 = function () {
    var formFieldMap = new Map([
        ['dataentrymode', __assign({ value: 'default' }, PROP_STRING)],
        ['debouncetime', __assign({ value: 250 }, PROP_NUMBER)],
        ['defaultvalue', PROP_STRING],
        ['displayname', PROP_STRING],
        ['display-name', PROP_STRING],
        ['field', PROP_STRING],
        ['filterexpressions', PROP_STRING],
        ['filter-on', PROP_STRING],
        ['generator', PROP_STRING],
        ['hint', PROP_STRING],
        ['inputtype', PROP_STRING],
        ['is-primary-key', PROP_BOOLEAN],
        ['is-range', PROP_BOOLEAN],
        ['is-related', PROP_BOOLEAN],
        ['isformfield', { value: true }],
        ['key', PROP_STRING],
        ['limit', PROP_NUMBER],
        ['lookup-type', PROP_STRING],
        ['lookup-field', PROP_STRING],
        ['name', PROP_STRING],
        ['matchmode', PROP_STRING],
        ['maxdefaultvalue', PROP_STRING],
        ['maxplaceholder', PROP_STRING],
        ['mobile-display', __assign({ value: true }, PROP_BOOLEAN)],
        ['period', PROP_BOOLEAN],
        ['pc-display', __assign({ value: true }, PROP_BOOLEAN)],
        ['placeholder', PROP_STRING],
        ['primary-key', PROP_BOOLEAN],
        ['related-entity-name', PROP_STRING],
        ['required', PROP_BOOLEAN],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tablet-display', __assign({ value: true }, PROP_BOOLEAN)],
        ['type', PROP_STRING],
        ['validationmessage', PROP_STRING],
        ['viewmodewidget', PROP_STRING],
        ['widgettype', PROP_STRING]
    ]);
    var _loop_1 = function (key) {
        var widgetName = 'wm-form-field-' + FormWidgetType[key];
        var widgetProps = getWidgetPropsByType(widgetName);
        formFieldMap.forEach(function (v, k) {
            if (widgetProps.get(k) == undefined) {
                widgetProps.set(k, v);
            }
        });
    };
    for (var key in FormWidgetType) {
        _loop_1(key);
    }
};

// Custom validator to show validation error, if setValidationMessage method is used
var customValidatorFn = function () {
    return { custom: true };
};
var ɵ0$1 = customValidatorFn;
var FILE_TYPES = {
    'image': 'image/*',
    'video': 'video/*',
    'audio': 'audio/*'
};
var ɵ1$1 = {};
var FormFieldDirective = /** @class */ (function (_super) {
    __extends(FormFieldDirective, _super);
    function FormFieldDirective(inj, form, fb, viewport, parentList, bindChipclass, binddataset, binddisplayexpression, binddisplaylabel, _widgetType, name, binddisplayimagesrc, key, isRange, pcDisplay, mobileDisplay, tabletDisplay, contexts) {
        var _this = this;
        var WIDGET_CONFIG = {
            widgetType: 'wm-form-field',
            hostClass: '',
            widgetSubType: 'wm-form-field-' + (_widgetType || FormWidgetType.TEXT).trim()
        };
        _this = _super.call(this, inj, WIDGET_CONFIG, new Promise(function (res) { return _this._initPropsRes = res; })) || this;
        _this.fieldDefConfig = {};
        _this.class = '';
        _this.binddataset = binddataset;
        _this.binddisplayimagesrc = binddisplayimagesrc;
        _this.binddisplayexpression = binddisplayexpression;
        _this.bindChipclass = bindChipclass;
        _this.binddisplaylabel = binddisplaylabel;
        _this.form = form;
        _this.fb = fb;
        _this.viewport = viewport;
        _this._fieldName = key || name;
        _this.isRange = isRange;
        _this.excludeProps = new Set(['type', 'name']);
        _this.widgettype = _widgetType;
        _this.parentList = parentList;
        _this.notifyForFields = [];
        if (_this.binddataset || _this.$element.attr('dataset')) {
            _this.isDataSetBound = true;
        }
        contexts[0]._onFocusField = _this._onFocusField.bind(_this);
        contexts[0]._onBlurField = _this._onBlurField.bind(_this);
        _this._debounceSetUpValidators = debounce(function () { return _this.setUpValidators(); }, 500);
        return _this;
    }
    FormFieldDirective_1 = FormFieldDirective;
    FormFieldDirective.prototype._onFocusField = function ($evt) {
        this._activeField = true;
        $($evt.target).closest('.live-field').addClass('active');
    };
    FormFieldDirective.prototype._onBlurField = function ($evt) {
        $($evt.target).closest('.live-field').removeClass('active');
        this._activeField = false;
    };
    // Expression to be evaluated in view mode of form field
    FormFieldDirective.prototype.evaluateExpr = function (object, displayExpr) {
        if (!displayExpr) {
            displayExpr = Object.keys(object)[0];
            // If dataset is not ready, display expression will not be defined
            if (!displayExpr) {
                return;
            }
        }
        return getEvaluatedData(object, {
            expression: displayExpr
        }, this.viewParent);
    };
    // Expression to be evaluated in view mode of form field
    FormFieldDirective.prototype.getDisplayExpr = function () {
        var _this = this;
        var caption = [];
        var value = this.value;
        var displayExpr = this.displayexpression || this.displayfield || this.displaylabel;
        if (_.isObject(value)) {
            if (_.isArray(value)) {
                _.forEach(value, function (obj) {
                    if (_.isObject(obj)) {
                        caption.push(_this.evaluateExpr(obj, displayExpr));
                    }
                });
            }
            else {
                caption.push(this.evaluateExpr(value, displayExpr));
            }
            return _.join(caption, ',');
        }
        return (value === undefined || value === null) ? '' : this.value;
    };
    FormFieldDirective.prototype.getCaption = function () {
        return (this.value === undefined || this.value === null) ? (_.get(this.form.dataoutput, this._fieldName) || '') : this.value;
    };
    // Notifies changes to observing validation fields
    FormFieldDirective.prototype.notifyChanges = function () {
        _.forEach(this.notifyForFields, function (field) {
            field.fieldValidations.validate();
        });
    };
    // Registers observer of validation fields
    FormFieldDirective.prototype.observeOn = function (fields) {
        this.fieldValidations.observeOn(fields, 'formfields');
    };
    // Method to setup validators for reactive form control
    FormFieldDirective.prototype.setUpValidators = function (customValidator) {
        this.fieldValidations.setUpValidators(customValidator);
    };
    // sets the custom async validation on the form field
    FormFieldDirective.prototype.setAsyncValidators = function (validators) {
        this.fieldValidations.setAsyncValidators(validators);
    };
    // sets the default and custom sync validation on the form field
    FormFieldDirective.prototype.setValidators = function (validators) {
        this.fieldValidations.setValidators(validators);
    };
    // Method to set the properties on inner form widget
    FormFieldDirective.prototype.setFormWidget = function (key, val) {
        if (this.formWidget && this.formWidget.widget) {
            this.formWidget.widget[key] = val;
        }
    };
    FormFieldDirective.prototype.boundFn = function (fn) {
        return fn();
    };
    // Method to set the properties on inner max form widget (when range is selected)
    FormFieldDirective.prototype.setMaxFormWidget = function (key, val) {
        if (this.formWidgetMax) {
            this.formWidgetMax.widget[key] = val;
        }
    };
    FormFieldDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key !== 'tabindex') {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
        if (this.excludeProps.has(key)) {
            return;
        }
        // As upload widget is an HTML widget, only required property is setup
        if (this.widgettype === FormWidgetType.UPLOAD) {
            if (key === 'required') {
                this._debounceSetUpValidators();
            }
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
            return;
        }
        this.setFormWidget(key, nv);
        // Placeholder should not be setup on max widget
        if (key !== 'placeholder') {
            this.setMaxFormWidget(key, nv);
        }
        switch (key) {
            case 'defaultvalue':
                this.form.onFieldDefaultValueChange(this, nv);
                break;
            case 'maxdefaultvalue':
                this.maxValue = nv;
                this.setMaxFormWidget('datavalue', nv);
                this.form.onMaxDefaultValueChange();
                break;
            case 'maxplaceholder':
                this.setMaxFormWidget('placeholder', nv);
                break;
            case 'required':
            case 'maxchars':
            case 'minvalue':
            case 'maxvalue':
            case 'regexp':
            case 'show':
                this._debounceSetUpValidators();
                break;
            case 'primary-key':
                if (nv) {
                    this.form.setPrimaryKey(this._fieldName);
                }
                break;
            case 'display-name':
                this.displayname = nv;
                break;
            case 'readonly':
                this.setReadOnlyState();
                break;
        }
    };
    FormFieldDirective.prototype.onStyleChange = function (key, nv, ov) {
        this.setFormWidget(key, nv);
        this.setMaxFormWidget(key, nv);
        _super.prototype.onStyleChange.call(this, key, nv, ov);
    };
    Object.defineProperty(FormFieldDirective.prototype, "datavalue", {
        get: function () {
            return this.formWidget && this.formWidget.datavalue;
        },
        set: function (val) {
            if (this._control && this.widgettype !== FormWidgetType.UPLOAD) {
                this._control.setValue(val);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormFieldDirective.prototype, "value", {
        get: function () {
            return this.datavalue;
        },
        set: function (val) {
            this.datavalue = val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormFieldDirective.prototype, "maxValue", {
        get: function () {
            return this.formWidgetMax && this.formWidgetMax.datavalue;
        },
        set: function (val) {
            if (this._maxControl) {
                this._maxControl.setValue(val);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormFieldDirective.prototype, "minValue", {
        get: function () {
            return this.value;
        },
        set: function (val) {
            this.value = val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormFieldDirective.prototype, "_control", {
        // Get the reactive form control
        get: function () {
            return this.ngform && this.ngform.controls[this._fieldName];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormFieldDirective.prototype, "_maxControl", {
        // Get the reactive max form control
        get: function () {
            return this.ngform && this.ngform.controls[this._fieldName + '_max'];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormFieldDirective.prototype, "displayValue", {
        // Get the displayValue
        get: function () {
            return this.formWidget && this.formWidget.displayValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormFieldDirective.prototype, "timestamp", {
        //Get the timestamp
        get: function () {
            return this.formWidget && this.formWidget.timestamp;
        },
        enumerable: true,
        configurable: true
    });
    // Create the reactive form control
    FormFieldDirective.prototype.createControl = function () {
        return this.fb.control(undefined, {
            validators: this._validators
        });
    };
    // On field value change, propagate event to parent form
    FormFieldDirective.prototype.onValueChange = function (val) {
        if (!this.isDestroyed) {
            this.form.onFieldValueChange(this, val);
            this.notifyChanges();
            if (this.$element.find('.active').length > 0 || this.form.touched) {
                this.ngform.controls[this._fieldName].markAsTouched();
                this.fieldValidations.setCustomValidationMessage();
            }
        }
    };
    FormFieldDirective.prototype.onStatusChange = function (status) {
        if (!this.isDestroyed) {
            this.showPendingSpinner = (status === 'PENDING');
            // while running validation, widget is disabled and spinner is shown
            // otherwise formWidget disabled state is reset to the state of the formField.
            if (status === 'PENDING') {
                this.formWidget.disabled = true;
            }
            else if (this.formWidget.disabled !== this.disabled) {
                this.formWidget.disabled = this.disabled;
            }
        }
    };
    // Method to expose validation message and set control to invalid
    FormFieldDirective.prototype.setValidationMessage = function (val) {
        var _this = this;
        setTimeout(function () {
            _this.validationmessage = val;
            _this.setUpValidators(customValidatorFn);
        });
    };
    FormFieldDirective.prototype.setReadOnlyState = function () {
        var readOnly;
        if (this.form.isUpdateMode) {
            if (this['primary-key'] && !this['is-related'] && !this.period) {
                /*If the field is primary but is assigned set readonly false.
                   Assigned is where the user inputs the value while a new entry.
                   This is not editable(in update mode) once entry is successful*/
                readOnly = !(this.generator === 'assigned' && this.form.operationType !== 'update');
            }
            else {
                readOnly = this.readonly;
            }
        }
        else {
            // In view mode, set widget state to readonly always
            readOnly = true;
        }
        this.setFormWidget('readonly', readOnly);
    };
    FormFieldDirective.prototype.resetDisplayInput = function () {
        if ((!isDefined(this.value) || this.value === '')) {
            this.formWidget && this.formWidget.resetDisplayInput && this.formWidget.resetDisplayInput();
        }
    };
    FormFieldDirective.prototype.triggerUploadEvent = function ($event, eventName) {
        var params = { $event: $event };
        if (eventName === 'change') {
            params.newVal = $event.target.files;
            params.oldVal = this._oldUploadVal;
            this._oldUploadVal = params.newVal;
        }
        this.invokeEventCallback(eventName, params);
    };
    FormFieldDirective.prototype.registerFormField = function () {
        var fieldName = this._fieldName;
        if (this.parentList && !(this.form.parentList === this.parentList)) {
            var counter = 1;
            var _fieldName = fieldName;
            while (this.ngform.controls.hasOwnProperty(_fieldName)) {
                _fieldName = fieldName + "_" + counter;
                counter++;
            }
            this.ngform.addControl(_fieldName, this.createControl());
            this._fieldName = _fieldName;
        }
        else {
            this.ngform.addControl(fieldName, this.createControl());
        }
        var onValueChangeSubscription = this._control.valueChanges
            .pipe(debounceTime(200))
            .subscribe(this.onValueChange.bind(this));
        this.registerDestroyListener(function () { return onValueChangeSubscription.unsubscribe(); });
        var onStatusChangeSubscription = this._control.statusChanges
            .pipe(debounceTime(100))
            .subscribe(this.onStatusChange.bind(this));
        this.registerDestroyListener(function () { return onStatusChangeSubscription.unsubscribe(); });
        if (this.isRange === 'true') {
            this.ngform.addControl(fieldName + '_max', this.createControl());
            // registering for valueChanges on MaxformWidget
            var onMaxValueChangeSubscription_1 = this._maxControl.valueChanges
                .pipe(debounceTime(200))
                .subscribe(this.onValueChange.bind(this));
            this.registerDestroyListener(function () { return onMaxValueChangeSubscription_1.unsubscribe(); });
        }
        this.value = _.get(this.form.formdata, this._fieldName);
    };
    FormFieldDirective.prototype.ngOnInit = function () {
        this.ngform = this.form.ngform;
        this.registerFormField();
        // Instantiate custom validators class for form field
        this.fieldValidations = new BaseFieldValidations(this, this.formWidget, this.widgettype, this._control, this.form);
        _super.prototype.ngOnInit.call(this);
    };
    FormFieldDirective.prototype.ngAfterContentInit = function () {
        var _this = this;
        _super.prototype.ngAfterContentInit.call(this);
        if (this.formWidget) {
            this._initPropsRes();
            // setting displayExpressions on the formwidget explicitly as expr was evaluated to "".
            this.setFormWidget('binddisplaylabel', this.binddisplaylabel);
            this.setFormWidget('binddisplayexpression', this.binddisplayexpression);
            this.setFormWidget('binddisplayimagesrc', this.binddisplayimagesrc);
            this.setFormWidget('bindChipclass', this.bindChipclass);
        }
        this.registerReadyStateListener(function () {
            _this.key = _this._fieldName || _this.target || _this.binding;
            _this.viewmodewidget = _this.viewmodewidget || getDefaultViewModeWidget(_this.widgettype);
            // For upload widget, generate the permitted field
            if (_this.widgettype === FormWidgetType.UPLOAD) {
                var fileType = void 0;
                // Create the accepts string from file type and extensions
                fileType = _this.filetype ? FILE_TYPES[_this.filetype] : '';
                _this.permitted = fileType + (_this.extensions ? (fileType ? ',' : '') + _this.extensions : '');
            }
            if (isMobile() && _this.viewport.isMobileType) {
                if (!_this['mobile-display']) {
                    _this.widget.show = false;
                }
            }
            else if (_this.viewport.isTabletType) {
                if (!_this['tablet-display']) {
                    _this.widget.show = false;
                }
            }
            else {
                if (!_this['pc-display']) {
                    _this.widget.show = false;
                }
            }
            _this.fieldDefConfig.displaname = _this.displayname;
            _this.fieldDefConfig.show = _this.show;
            _this.fieldDefConfig.isRelated = _this['is-related'];
            _this.fieldDefConfig.inputtype = _this.inputtype;
            _this.fieldDefConfig.generator = _this.generator;
            _this.fieldDefConfig.placeholder = _this.placeholder;
            _this.fieldDefConfig.primaryKey = _this['primary-key'];
            _this.fieldDefConfig.required = _this.required;
            _this.fieldDefConfig._readonly = _this.readonly;
            _this.fieldDefConfig.regexp = _this.regexp;
            _this.fieldDefConfig.type = _this.type;
            _this.fieldDefConfig.key = _this.key;
            _this.fieldDefConfig.mobileDisplay = _this['mobile-display'];
            _this.fieldDefConfig.name = _this.name;
            _this.fieldDefConfig.pcDisplay = _this['pc-display'];
            _this.fieldDefConfig.tabletDisplay = _this['tablet-display'];
            _this.fieldDefConfig.validationmessage = _this.validationmessage;
            _this.fieldDefConfig.viewmodewidget = _this.viewmodewidget;
            _this.fieldDefConfig.widget = _this.widgettype;
            // Register the form field with parent form
            _this.form.registerFormFields(_this.widget);
            addForIdAttributes(_this.nativeElement);
            _this.setReadOnlyState();
        });
    };
    var FormFieldDirective_1;
    FormFieldDirective.initializeProps = registerProps$1();
    FormFieldDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: FormComponent },
        { type: FormBuilder },
        { type: Viewport },
        { type: ListComponent, decorators: [{ type: Optional }] },
        { type: String, decorators: [{ type: Attribute, args: ['chipclass.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['displayexpression.bind',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['displaylabel.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['widgettype',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['name',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['displayimagesrc.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['key',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['is-range',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['pc-display',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['mobile-display',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['tablet-display',] }] },
        { type: Array, decorators: [{ type: Self }, { type: Inject, args: [Context,] }] }
    ]; };
    __decorate([
        ContentChild('formWidget', { static: true }),
        __metadata("design:type", Object)
    ], FormFieldDirective.prototype, "formWidget", void 0);
    __decorate([
        ContentChild('formWidgetMax'),
        __metadata("design:type", Object)
    ], FormFieldDirective.prototype, "formWidgetMax", void 0);
    FormFieldDirective = FormFieldDirective_1 = __decorate([
        Directive({
            selector: '[wmFormField]',
            exportAs: 'wmFormField',
            providers: [
                provideAsWidgetRef(FormFieldDirective_1),
                provideAs(FormFieldDirective_1, NG_VALUE_ACCESSOR, true),
                { provide: Context, useValue: ɵ1$1, multi: true }
            ]
        }),
        __param(4, Optional()),
        __param(5, Attribute('chipclass.bind')),
        __param(6, Attribute('dataset.bind')),
        __param(7, Attribute('displayexpression.bind')),
        __param(8, Attribute('displaylabel.bind')),
        __param(9, Attribute('widgettype')),
        __param(10, Attribute('name')),
        __param(11, Attribute('displayimagesrc.bind')),
        __param(12, Attribute('key')),
        __param(13, Attribute('is-range')),
        __param(14, Attribute('pc-display')),
        __param(15, Attribute('mobile-display')),
        __param(16, Attribute('tablet-display')),
        __param(17, Self()), __param(17, Inject(Context)),
        __metadata("design:paramtypes", [Injector,
            FormComponent,
            FormBuilder,
            Viewport,
            ListComponent, String, Object, String, String, Object, Object, String, Object, Object, Object, Object, Object, Array])
    ], FormFieldDirective);
    return FormFieldDirective;
}(StylableComponent));

var LiveActionsDirective = /** @class */ (function () {
    function LiveActionsDirective(subscribedWidget, app, dialogService) {
        this.subscribedWidget = subscribedWidget;
        this.app = app;
        this.dialogService = dialogService;
        subscribedWidget.addRow = this.addRow.bind(this);
        subscribedWidget.updateRow = this.updateRow.bind(this);
        subscribedWidget.deleteRow = this.deleteRow.bind(this);
        subscribedWidget.call = this.call.bind(this);
    }
    LiveActionsDirective.prototype.addRow = function () {
        this.app.notify('wm-event', { eventName: Live_Operations.INSERT, widgetName: this.subscribedWidget.name, row: this.subscribedWidget.selecteditem });
    };
    LiveActionsDirective.prototype.updateRow = function () {
        this.app.notify('wm-event', { eventName: Live_Operations.UPDATE, widgetName: this.subscribedWidget.name, row: this.subscribedWidget.selecteditem });
    };
    LiveActionsDirective.prototype.deleteRow = function () {
        this.app.notify('wm-event', { eventName: Live_Operations.DELETE, widgetName: this.subscribedWidget.name, row: this.subscribedWidget.selecteditem });
    };
    LiveActionsDirective.prototype.successHandler = function (options, response) {
        triggerFn(options.success, response);
    };
    LiveActionsDirective.prototype.errorHandler = function (options, error) {
        this.app.notifyApp(error, 'error', 'ERROR');
        triggerFn(options.error, error);
    };
    LiveActionsDirective.prototype.getRecords = function (options, operation) {
        var _this = this;
        var index;
        var dataNavigator;
        if (this.subscribedWidget.navigation !== 'None' && this.subscribedWidget.dataNavigator) {
            dataNavigator = this.subscribedWidget.dataNavigator;
            // If operation is delete, decrease the data size and check if navigation to previous page is required
            if (operation === Live_Operations.DELETE) {
                dataNavigator.dataSize -= 1;
                dataNavigator.calculatePagingValues();
                index = dataNavigator.pageCount < dataNavigator.dn.currentPage ? 'prev' : undefined;
            }
            else {
                // If operation is insert, go to last page. If update operation, stay on current page
                index = operation === Live_Operations.INSERT ? 'last' : 'current';
                if (index === 'last') {
                    dataNavigator.dataSize += 1;
                }
                dataNavigator.calculatePagingValues();
            }
            dataNavigator.navigatePage(index, null, true, function (response) {
                _this.successHandler(options, response);
            });
        }
        else {
            this.subscribedWidget.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                'skipToggleState': true
            }).then(function (response) {
                _this.successHandler(options, response);
            }, function (err) {
                _this.errorHandler(options, err);
            });
        }
    };
    LiveActionsDirective.prototype.performCUDOperation = function (requestData, operation, options) {
        var _this = this;
        performDataOperation(this.subscribedWidget.datasource, requestData, {
            operationType: operation
        }).then(function (response) {
            if (response.error) {
                _this.errorHandler(options, response.error);
                return;
            }
            if (_this.subscribedWidget.datasource.category === 'wm.CrudVariable' && operation === 'delete') {
                _this.subscribedWidget.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                    'skipToggleState': true
                }).then(function (response) {
                    _this.successHandler(options, response);
                }, function (err) {
                    _this.errorHandler(options, err);
                });
            }
            else {
                _this.getRecords(options, operation);
            }
            // show delete success toaster
            if (operation === 'delete') {
                _this.app.notifyApp(_this.app.appLocale.MESSAGE_DELETE_RECORD_SUCCESS, 'success');
            }
        }, function (error) {
            _this.errorHandler(options, error);
        });
    };
    LiveActionsDirective.prototype.insertRecord = function (requestData, operation, options) {
        this.performCUDOperation(requestData, operation, options);
    };
    LiveActionsDirective.prototype.updateRecord = function (requestData, operation, options) {
        this.performCUDOperation(requestData, operation, options);
    };
    LiveActionsDirective.prototype.deleteRecord = function (requestData, operation, options) {
        var _this = this;
        // Show the delete confirmation dialog. On Ok, delete the record.
        this.dialogService.showAppConfirmDialog({
            title: this.app.appLocale.MESSAGE_DELETE_RECORD || 'Delete Record',
            iconclass: 'wi wi-delete fa-lg',
            message: this.subscribedWidget.confirmdelete || 'Are you sure you want to delete this?',
            oktext: this.subscribedWidget.deleteoktext || 'Ok',
            canceltext: this.subscribedWidget.deletecanceltext || 'Cancel',
            onOk: function () {
                _this.performCUDOperation(requestData, operation, options);
                _this.dialogService.closeAppConfirmDialog();
            },
            onCancel: function () {
                triggerFn(options.cancelDeleteCallback);
                _this.dialogService.closeAppConfirmDialog();
            }
        });
    };
    LiveActionsDirective.prototype.performOperation = function (operation, options) {
        var requestData = {
            row: options.row,
            prevData: {},
            rowData: {},
            transform: true,
            skipNotification: true
        };
        if (operation === Live_Operations.UPDATE) {
            requestData.rowData = options.rowData;
            requestData.prevData = options.prevData;
        }
        /* decide routine based on CRUD operation to be performed */
        switch (operation) {
            case Live_Operations.INSERT:
                this.insertRecord(requestData, operation, options);
                break;
            case Live_Operations.UPDATE:
                this.updateRecord(requestData, operation, options);
                break;
            case Live_Operations.DELETE:
                this.deleteRecord(requestData, operation, options);
                break;
            case Live_Operations.READ:
                this.getRecords(options, operation);
                break;
        }
    };
    // API exposed to make CRUD operations
    LiveActionsDirective.prototype.call = function (operation, options, success, error) {
        options.success = success;
        options.error = error;
        this.performOperation(operation, options);
    };
    LiveActionsDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [ListComponent,] }] },
        { type: App },
        { type: AbstractDialogService }
    ]; };
    LiveActionsDirective = __decorate([
        Directive({
            selector: '[wmLiveActions]'
        }),
        __param(0, Self()), __param(0, Inject(ListComponent)),
        __metadata("design:paramtypes", [Object, App,
            AbstractDialogService])
    ], LiveActionsDirective);
    return LiveActionsDirective;
}());

var DependsonDirective = /** @class */ (function () {
    function DependsonDirective(dialogId, dependson, dialogService, app) {
        this.dialogService = dialogService;
        this.app = app;
        // If dialogId is present, form is in dialog mode
        if (dialogId) {
            this.isLayoutDialog = true;
            this.dialogId = dialogId;
        }
        this.dependson = dependson;
        // Listen to the wm-event called from subscribed widgets
        this.eventSubscription = this.app.subscribe('wm-event', this.handleEvent.bind(this));
    }
    DependsonDirective.prototype.openFormDialog = function () {
        this.dialogService.open(this.dialogId);
    };
    DependsonDirective.prototype.onUpdate = function (newValue, table, editClicked) {
        this.editMode = editClicked || this.editMode;
        this.form.operationType = Live_Operations.UPDATE;
        this.form.isSelected = true;
        var rowData = this.currentFormData;
        if (newValue && table) {
            if (table.multiselect) {
                rowData = newValue[0];
            }
            else {
                rowData = newValue[newValue.length - 1];
            }
        }
        this.form.setFormData(rowData);
        this.form.edit();
        if (!this.editMode) {
            this.form.isUpdateMode = false;
        }
    };
    DependsonDirective.prototype.onInsert = function () {
        this.form.operationType = Live_Operations.INSERT;
        this.form.isSelected = true;
        this.form.setFormData({});
        this.form.new();
    };
    DependsonDirective.prototype.handleEvent = function (options) {
        if (this.dependson !== options.widgetName) {
            return;
        }
        this.currentOp = options.eventName;
        switch (options.eventName) {
            case Live_Operations.UPDATE:
                this.currentFormData = options.row;
                if (this.isLayoutDialog) {
                    this.openFormDialog();
                }
                else {
                    this.onUpdate(undefined, undefined, true);
                }
                break;
            case Live_Operations.INSERT:
                if (this.isLayoutDialog) {
                    this.openFormDialog();
                }
                else {
                    this.onInsert();
                }
                break;
            case Live_Operations.DELETE:
                this.app.Widgets[this.dependson].call('delete', { row: options.row });
                break;
            case Live_Operations.READ:
                if (!this.isLayoutDialog) {
                    this.form.isUpdateMode = false;
                }
                break;
            case 'rerender':
                if (options.dataSource) {
                    options.dataSource.execute(DataSource.Operation.LIST_RECORDS, {
                        'skipToggleState': true,
                        'operation': 'list'
                    });
                }
                break;
            case 'selectedItemChange':
                if (options.table) {
                    this.onUpdate(options.row, options.table);
                }
                break;
            case 'resetEditMode':
                this.editMode = false;
        }
    };
    DependsonDirective.prototype.onFormRender = function () {
        var _this = this;
        // On opening the form in dialog mode, complete the pending operations
        if (this.form && this.isLayoutDialog) {
            this.form.isLayoutDialog = true;
            setTimeout(function () {
                if (_this.currentOp === Live_Operations.UPDATE) {
                    _this.onUpdate(undefined, undefined, true);
                }
                else if (_this.currentOp === Live_Operations.INSERT) {
                    _this.onInsert();
                }
            }, 250);
        }
    };
    DependsonDirective.prototype.ngAfterContentInit = function () {
        var _this = this;
        // If form instance is present, form is in inline mode. Else, it is in dialog mode and listen to form instance changes
        if (this.formChildren.first) {
            this.form = this.formChildren.first;
        }
        else {
            this.formSubscription = this.formChildren.changes.subscribe(function (val) {
                _this.form = val.first;
                _this.onFormRender();
            });
        }
        // find the dependent widget (table/list)
        this.app.notify('setup-cud-listener', this.dependson);
    };
    DependsonDirective.prototype.ngOnDestroy = function () {
        if (this.formSubscription) {
            this.formSubscription.unsubscribe();
        }
        if (this.eventSubscription) {
            this.eventSubscription();
        }
    };
    DependsonDirective.ctorParameters = function () { return [
        { type: String, decorators: [{ type: Attribute, args: ['dialogid',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['dependson',] }] },
        { type: AbstractDialogService },
        { type: App }
    ]; };
    __decorate([
        ContentChildren(FormComponent, { descendants: true }),
        __metadata("design:type", Object)
    ], DependsonDirective.prototype, "formChildren", void 0);
    DependsonDirective = __decorate([
        Directive({
            selector: '[dependson]'
        }),
        __param(0, Attribute('dialogid')),
        __param(1, Attribute('dependson')),
        __metadata("design:paramtypes", [String, String, AbstractDialogService,
            App])
    ], DependsonDirective);
    return DependsonDirective;
}());

var FILTER_CONSTANTS = {
    'EMPTY_KEY': 'EMPTY_NULL_FILTER'
};
var noop = function () { };
var ɵ0$2 = noop;
var LiveFilterDirective = /** @class */ (function () {
    function LiveFilterDirective(form) {
        var _this = this;
        this.form = form;
        // debounce the filter function. If multiple filter calls are made at same time, calls will be delayed and last call is fired
        this._filter = debounce(function (options) {
            _this.filter(options);
        }, 250);
        form.clearFilter = this.clearFilter.bind(this);
        form.applyFilter = this.applyFilter.bind(this);
        form.filter = this.filter.bind(this);
        form.filterOnDefault = this.filterOnDefault.bind(this);
        form.execute = this.execute.bind(this);
        form.onFieldDefaultValueChange = this.onFieldDefaultValueChange.bind(this);
        form.onMaxDefaultValueChange = this.onMaxDefaultValueChange.bind(this);
        form.onDataSourceChange = this.onDataSourceChange.bind(this);
        form.onFieldValueChange = this.onFieldValueChange.bind(this);
        form.submitForm = this.submitForm.bind(this);
        form.registerFormWidget = this.registerFormWidget.bind(this);
    }
    LiveFilterDirective.prototype.execute = function (operation, options) {
        if (operation === DataSource.Operation.LIST_RECORDS || operation === DataSource.Operation.DOWNLOAD) {
            return this.applyFilter(options);
        }
        if (operation === DataSource.Operation.GET_OPTIONS) {
            return this._options || {};
        }
        if (operation === DataSource.Operation.GET_PAGING_OPTIONS) {
            return this.form.pagination;
        }
        if (!this.form.datasource) {
            return {};
        }
        if (operation === DataSource.Operation.FETCH_DISTINCT_VALUES) {
            return fetchDistinctValues(this.form.datasource, this.form.formFields, {
                widget: 'widgettype',
                enableemptyfilter: this.form.enableemptyfilter,
                EMPTY_VALUE: this.form.appLocale.LABEL_NO_VALUE
            });
        }
        return this.form.datasource.execute(operation, options);
    };
    LiveFilterDirective.prototype.onFieldDefaultValueChange = function (field, nv) {
        field.minValue = nv;
        field.value = nv;
        this.filterOnDefault();
    };
    LiveFilterDirective.prototype.onFieldValueChange = function (field, nv) {
        applyFilterOnField(this.form.datasource, field.widget, this.form.formFields, nv, {
            EMPTY_VALUE: this.form.appLocale.LABEL_NO_VALUE
        });
        if (this.form.autoupdate && isDefined(nv)) {
            this._filter();
        }
    };
    LiveFilterDirective.prototype.onMaxDefaultValueChange = function () {
        var _this = this;
        setTimeout(function () {
            _this.filterOnDefault();
        });
    };
    LiveFilterDirective.prototype.onDataSourceChange = function () {
        var _this = this;
        var dataSource = this.form.datasource;
        if (!dataSource) {
            return;
        }
        this.form.formFields.forEach(function (field) {
            if (isDataSetWidget(field.widgettype)) {
                getDistinctValuesForField(dataSource, field.widget, {
                    widget: 'widgettype',
                    enableemptyfilter: _this.form.enableemptyfilter,
                    EMPTY_VALUE: _this.form.appLocale.LABEL_NO_VALUE
                });
                applyFilterOnField(dataSource, field.widget, _this.form.formFields, field.value, {
                    isFirst: true,
                    EMPTY_VALUE: _this.form.appLocale.LABEL_NO_VALUE
                });
            }
        });
        // On load check if default value exists and apply filter, Call the filter with the result options
        this._filter(this._options);
    };
    LiveFilterDirective.prototype.clearFilter = function () {
        var _this = this;
        this.form.resetFormState();
        this.form.formFields.forEach(function (filterField) {
            // Added check for range field
            if (!filterField.readonly && filterField.show) {
                if (filterField.widgettype === FormWidgetType.AUTOCOMPLETE || filterField.widgettype === FormWidgetType.TYPEAHEAD) {
                    _this.form.$element.find('div[name=' + filterField.name + '] input').val('');
                }
                if (filterField['is-range']) {
                    filterField.minValue = '';
                    filterField.maxValue = '';
                }
                else {
                    filterField.value = '';
                    filterField.resetDisplayInput();
                }
            }
        });
        // If variable has any bindings, wait for the bindings to be updated
        setTimeout(function () {
            // Setting result to the default data
            _this._filter();
        });
    };
    LiveFilterDirective.prototype.submitForm = function () {
        this.filter();
    };
    LiveFilterDirective.prototype.applyFilter = function (options) {
        options = options ? (options.data || options) : {};
        options.page = options.page || 1;
        options.orderBy = isDefined(options.orderBy) ? options.orderBy : this.orderBy;
        return this.filter(options);
    };
    LiveFilterDirective.prototype.filter = function (options) {
        var _this = this;
        if (!this.form.datasource) {
            return;
        }
        var filterFields = {};
        var dataModel = {};
        var page = 1, orderBy, isValid;
        options = options || {};
        page = options.page || page;
        orderBy = isDefined(options.orderBy) ? options.orderBy : (this.orderBy || '');
        this.orderBy = orderBy; // Store the order by in scope. This can be used to retain the sort after filtering
        // Copy the values to be sent to the user as '$data' before servicecall
        this.form.formFields.forEach(function (field) {
            var fieldSelector = 'div[name=' + field.name + '] input';
            var $el = _this.form.$element;
            var fieldEle;
            if ((field.widgettype === FormWidgetType.AUTOCOMPLETE || field.widgettype === FormWidgetType.TYPEAHEAD) && $el) {
                fieldEle = $el.find(fieldSelector);
                if (!field['is-range']) {
                    dataModel[field.field] = {
                        'value': isDefined(field.value) ? field.value : fieldEle.val() // For autocomplete, set the datavalue. If not present, set query value
                    };
                }
                else {
                    dataModel[field.field] = {
                        'minValue': isDefined(field.minValue) ? field.minValue : fieldEle.first().val(),
                        'maxValue': isDefined(field.maxValue) ? field.maxValue : fieldEle.last().val()
                    };
                }
                return;
            }
            if (!field['is-range']) {
                dataModel[field.field] = {
                    'value': field.value
                };
            }
            else {
                dataModel[field.field] = {
                    'minValue': field.minValue,
                    'maxValue': field.maxValue
                };
            }
        });
        /*Perform this function for the event onBeforeservicecall*/
        try {
            isValid = this.form.invokeEventCallback('beforeservicecall', { $data: dataModel });
            if (isValid === false) {
                return;
            }
            if (isValid && isValid.error) {
                this.form.toggleMessage(true, isValid.error, 'error', 'ERROR');
                return;
            }
            /*Update these values in the formFields with new reference, inorder to maintain the UI values*/
            this.form.formFields.forEach(function (filterField) {
                if (!filterField['is-range']) {
                    filterField._value = dataModel[filterField.field].value;
                }
                else {
                    filterField._minValue = dataModel[filterField.field].minValue;
                    filterField._maxValue = dataModel[filterField.field].maxValue;
                }
            });
        }
        catch (err) {
            if (err.message === 'Abort') {
                return;
            }
        }
        /* Construct the formFields Variable to send it to the queryBuilder */
        this.form.formFields.forEach(function (filterField) {
            var fieldValue;
            var matchMode;
            var colName = filterField.field;
            var minValue = filterField._minValue;
            var maxvalue = filterField._maxValue;
            /* if field is part of a related entity, column name will be 'entity.fieldName' */
            if (filterField['is-related']) {
                colName += '.' + filterField['lookup-field'];
            }
            if (filterField['is-range']) {
                /*Based on the min and max values, decide the matchmode condition*/
                fieldValue = getRangeFieldValue(minValue, maxvalue);
                matchMode = getRangeMatchMode(minValue, maxvalue);
                if (isDefined(fieldValue)) {
                    filterFields[colName] = {
                        'value': fieldValue,
                        'matchMode': matchMode,
                        'logicalOp': 'AND'
                    };
                }
            }
            else {
                switch (filterField.widgettype) {
                    case FormWidgetType.SELECT:
                    case FormWidgetType.RADIOSET:
                        if (getEnableEmptyFilter(_this.form.enableemptyfilter) && filterField._value === FILTER_CONSTANTS.EMPTY_KEY) {
                            matchMode = getEmptyMatchMode(_this.form.enableemptyfilter);
                            fieldValue = filterField._value;
                        }
                        else {
                            if (filterField.type === DataType.BOOLEAN) {
                                if (isDefined(filterField._value) && filterField._value !== '') {
                                    fieldValue = JSON.parse(filterField._value);
                                }
                            }
                            else {
                                fieldValue = filterField._value;
                            }
                        }
                        break;
                    case FormWidgetType.CHECKBOXSET:
                    case FormWidgetType.CHIPS:
                        if (filterField._value && filterField._value.length) {
                            fieldValue = filterField._value;
                        }
                        break;
                    case FormWidgetType.CHECKBOX:
                    case FormWidgetType.TOGGLE:
                        if (isDefined(filterField._value) && filterField._value !== '') {
                            fieldValue = filterField.type === DataType.BOOLEAN ? JSON.parse(filterField._value) : filterField._value;
                        }
                        break;
                    default:
                        fieldValue = filterField._value;
                        break;
                }
                if (isDefined(fieldValue) && fieldValue !== '' && fieldValue !== null) {
                    filterFields[colName] = {};
                    if (matchMode) {
                        filterFields[colName].matchMode = matchMode;
                        fieldValue = undefined;
                    }
                    else if (filterField.type === DataType.STRING || filterField.isRelated) { // Only for string types and related fields, custom match modes are enabled.
                        filterFields[colName].matchMode = matchMode || filterField.matchmode ||
                            _this.form.datasource.execute(DataSource.Operation.GET_MATCH_MODE);
                    }
                    filterFields[colName].value = fieldValue;
                    filterFields[colName].logicalOp = 'AND';
                    filterFields[colName].type = filterField.type;
                }
            }
        });
        if (options.exportType) {
            return this.form.datasource.execute(DataSource.Operation.DOWNLOAD, {
                data: {
                    matchMode: 'anywhereignorecase',
                    filterFields: filterFields,
                    orderBy: orderBy,
                    exportType: options.exportType,
                    logicalOp: 'AND',
                    exportSize: options.exportSize,
                    fields: options.fields,
                    fileName: options.fileName
                }
            });
        }
        return this.form.datasource.execute(DataSource.Operation.LIST_RECORDS, {
            filterFields: filterFields,
            orderBy: orderBy,
            page: page,
            pagesize: this.form.pagesize || 20,
            skipDataSetUpdate: true,
            inFlightBehavior: 'executeAll'
        }).then(function (response) {
            var data = response.data;
            _this.form.pagination = response.pagination;
            if (data.error) {
                // disable readonly and show the appropriate error
                _this.form.toggleMessage(true, data.error, 'error', 'ERROR');
                _this.form.onResultCb(data, false);
            }
            else {
                _this._options = {
                    'page': page,
                    'orderBy': orderBy
                };
                _this.form.result = getClonedObject(data);
                _this.form.onResultCb(data, true);
            }
            $appDigest();
            return _this.form.result;
        }, function (error) {
            _this.form.toggleMessage(true, error, 'error', 'ERROR');
            _this.form.onResultCb(error, false);
            return error;
        });
    };
    // Calls the filter function if default values are present
    LiveFilterDirective.prototype.filterOnDefault = function () {
        /*Check if default value is present for any filter field*/
        var defaultObj = _.find(this.form.formFields, function (obj) {
            return isDefined(obj.value) || isDefined(obj.minValue) || isDefined(obj.maxValue);
        });
        /*If default value exists and data is loaded, apply the filter*/
        if (defaultObj) {
            this._filter(this._options);
        }
    };
    LiveFilterDirective.prototype.registerFormWidget = function (widget) {
        var name = widget.key || widget.name;
        this.form.filterWidgets[name] = widget;
    };
    LiveFilterDirective.initializeProps = registerLiveFilterProps();
    LiveFilterDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [FormComponent,] }] }
    ]; };
    LiveFilterDirective = __decorate([
        Directive({
            selector: '[wmLiveFilter]'
        }),
        __param(0, Self()), __param(0, Inject(FormComponent)),
        __metadata("design:paramtypes", [Object])
    ], LiveFilterDirective);
    return LiveFilterDirective;
}());

var isTimeType = function (field) { return field.widgettype === DataType.TIME || (field.type === DataType.TIME && !field.widgettype); };
var ɵ0$3 = isTimeType;
var getValidTime = function (val) {
    if (val) {
        var date = (new Date()).toDateString();
        return (new Date(date + " " + val)).getTime();
    }
    return undefined;
};
var ɵ1$2 = getValidTime;
var LiveFormDirective = /** @class */ (function () {
    function LiveFormDirective(form, liveTable, datePipe, dialogService, formlayout) {
        var _this = this;
        this.form = form;
        this.datePipe = datePipe;
        this.dialogService = dialogService;
        this._debouncedSavePrevDataValues = debounce(function () {
            _this.savePrevDataValues();
        }, 250);
        // If parent live table is present and this form is first child of live table, set this form instance on livetable
        if (liveTable && !this.form.parentForm) {
            this.form._liveTableParent = liveTable;
            this.form.isLayoutDialog = liveTable.isLayoutDialog;
            liveTable.onFormReady(this.form);
        }
        else {
            this.form.isLayoutDialog = formlayout === 'dialog';
        }
        // CUD operations
        form.cancel = this.cancel.bind(this);
        form.reset = this.reset.bind(this);
        form.delete = this.delete.bind(this);
        form.save = this.save.bind(this);
        form.saveAndNew = this.saveAndNew.bind(this);
        form.saveAndView = this.saveAndView.bind(this);
        form.setPrimaryKey = this.setPrimaryKey.bind(this);
        form.constructDataObject = this.constructDataObject.bind(this);
        form.changeDataObject = this.setFormData.bind(this);
        form.setFormData = this.setFormData.bind(this);
        form.findOperationType = this.findOperationType.bind(this);
        form.clearData = this.clearData.bind(this);
        form.onFieldDefaultValueChange = this.onFieldDefaultValueChange.bind(this);
        form.onDataSourceChange = this.onDataSourceChange.bind(this);
        form.onFieldValueChange = this.onFieldValueChange.bind(this);
        form.submitForm = this.submitForm.bind(this);
        form.form_registerFormFields = form.registerFormFields;
        form.registerFormFields = this.registerFormFields.bind(this);
    }
    /*
    WMS:18583
    Overriding 'registerFormFields' for liveForms,
    On Each field added, check if the dataSource is updated or not
    if the field is added after the dataSource is updated at LiveForm, retrigger
    */
    LiveFormDirective.prototype.registerFormFields = function (formField) {
        var formFields = this.form.getFormFields();
        this.form.form_registerFormFields(formField);
        if (this.form.isDataSourceUpdated) {
            this._updateFieldOnDataSourceChange(formField, formFields);
        }
    };
    LiveFormDirective.prototype._updateFieldOnDataSourceChange = function (field, formFields) {
        if (!field.isDataSetBound && isDataSetWidget(field.widgettype)) {
            if (field['is-related']) {
                field.isDataSetBound = true;
                fetchRelatedFieldData(this.form.datasource, field.widget, {
                    relatedField: field.key,
                    datafield: ALLFIELDS,
                    widget: 'widgettype',
                });
            }
            else {
                getDistinctValuesForField(this.form.datasource, field.widget, {
                    widget: 'widgettype',
                    enableemptyfilter: this.form.enableemptyfilter
                });
                applyFilterOnField(this.form.datasource, field.widget, formFields, field.value, { isFirst: true });
            }
        }
    };
    LiveFormDirective.prototype.onDataSourceChange = function () {
        var _this = this;
        if (_.get(this.form.formFields, 'length')) {
            this.form.isDataSourceUpdated = true;
        }
        var formFields = this.form.getFormFields();
        formFields.forEach(function (field) {
            _this._updateFieldOnDataSourceChange(field, formFields);
        });
    };
    LiveFormDirective.prototype.onFieldDefaultValueChange = function (field, nv) {
        // In Edit, do  not set default values
        if (this.form.operationType === 'update') {
            return;
        }
        // Set the default value only if it exists.
        if (isDefined(nv) && nv !== null && nv !== '' && nv !== 'null') {
            field.value = parseValueByType(nv, field.type, field.widgettype, field.trailingzero);
        }
        else {
            field.value = undefined;
        }
        this._debouncedSavePrevDataValues();
    };
    LiveFormDirective.prototype.onFieldValueChange = function (field, nv) {
        applyFilterOnField(this.form.datasource, field.widget, this.form.formFields, nv);
    };
    LiveFormDirective.prototype.getBlobURL = function (dataObj, key, value) {
        var href = '';
        var primaryKeys;
        var primaryKey;
        if (value === null || value === undefined || !this.form.datasource) {
            return href;
        }
        primaryKeys = this.form.datasource.execute(DataSource.Operation.GET_PRIMARY_KEY) || [];
        primaryKey = dataObj[primaryKeys[0]];
        // TODO: Handle mobile case
        // if (CONSTANTS.hasCordova && CONSTANTS.isRunMode) {
        //     href += $rootScope.project.deployedUrl;
        // }
        href += this.form.datasource.execute(DataSource.Operation.GET_BLOB_URL, {
            primaryValue: primaryKey,
            columnName: key
        });
        href += '?' + Math.random();
        return href;
    };
    LiveFormDirective.prototype.setFormData = function (dataObj, formFields) {
        var _this = this;
        if (!dataObj) {
            return;
        }
        formFields = formFields || this.form.formFields;
        formFields.forEach(function (field) {
            var value = _.get(dataObj, field.key || field.name);
            if (isTimeType(field)) {
                field.value = getValidTime(value);
            }
            else if (field.type === DataType.BLOB) {
                _this.form.resetFileUploadWidget(field, true);
                field.href = _this.getBlobURL(dataObj, field.key, value);
                field.value = _.isString(value) ? '' : value;
            }
            else {
                _this.form.setFieldValue(field, dataObj);
            }
        });
        this.savePrevDataValues();
        this.form.constructDataObject();
    };
    LiveFormDirective.prototype.onDataSourceUpdate = function (response, newForm, updateMode) {
        if (newForm) {
            this.form.new();
        }
        else {
            this.form.setFormData(response);
            this.closeDialog();
        }
        this.form.isUpdateMode = isDefined(updateMode) ? updateMode : true;
    };
    LiveFormDirective.prototype.savePrevformFields = function () {
        this.form.prevformFields = getClonedObject(this.form.formFields.map(function (field) {
            return {
                'key': field.key,
                'type': field.type,
                'widgettype': field.widgettype,
                'outputformat': field.outputformat,
                'value': field.value
            };
        }));
    };
    LiveFormDirective.prototype.getPrevformFields = function () {
        var _this = this;
        this.form.formFields.map(function (field) {
            var prevField = _this.form.prevformFields.find(function (pField) { return pField.key === field.key; });
            field.value = prevField.value;
        });
    };
    LiveFormDirective.prototype.getDataObject = function () {
        if (this.form.operationType === Live_Operations.INSERT) {
            return {};
        }
        if (isDefined(this.form.prevDataObject) && !isEmptyObject(this.form.prevDataObject)) {
            return getClonedObject(this.form.prevDataObject);
        }
        return getClonedObject(this.form.formdata || {});
    };
    LiveFormDirective.prototype.constructDataObject = function (isPreviousData) {
        var _this = this;
        var dataObject = this.getDataObject();
        var formName = this.form.name;
        var formFields;
        formFields = isPreviousData ? this.form.prevformFields : this.form.getFormFields();
        formFields.forEach(function (field) {
            var dateTime, fieldValue;
            var fieldTarget = _.split(field.key, '.');
            var fieldName = fieldTarget[0] || field.key;
            /*collect the values from the fields and construct the object*/
            /*Format the output of date time widgets to the given output format*/
            if ((field.widgettype && isDateTimeType(field.widgettype)) || isDateTimeType(field.type)) {
                if (field.value) {
                    dateTime = getValidDateObject(field.value);
                    if (field.outputformat === DataType.TIMESTAMP || field.type === DataType.TIMESTAMP) {
                        fieldValue = field.value ? dateTime : null;
                    }
                    else if (field.outputformat) {
                        fieldValue = _this.datePipe.transform(dateTime, field.outputformat);
                    }
                    else {
                        fieldValue = field.value;
                    }
                }
                else {
                    fieldValue = undefined;
                }
            }
            else if (field.type === DataType.BLOB) {
                fieldValue = getFiles(formName, fieldName + '_formWidget', field.multiple);
            }
            else if (field.type === DataType.LIST) {
                fieldValue = field.value || undefined;
            }
            else {
                fieldValue = (field.value === null || field.value === '') ? undefined : field.value;
            }
            if (fieldTarget.length === 1) {
                dataObject[fieldName] = fieldValue;
            }
            else {
                dataObject[fieldName] = dataObject[fieldName] || {};
                dataObject[fieldName][fieldTarget[1]] = fieldValue;
            }
        });
        if (!isPreviousData) {
            this.form.updateFormDataOutput(dataObject);
            return this.form.dataoutput;
        }
        return dataObject;
    };
    LiveFormDirective.prototype.setPrimaryKey = function (fieldName) {
        /*Store the primary key of data*/
        this.form.primaryKey = this.form.primaryKey || [];
        if (this.form.primaryKey.indexOf(fieldName) === -1) {
            this.form.primaryKey.push(fieldName);
        }
    };
    LiveFormDirective.prototype.findOperationType = function () {
        var _this = this;
        var operation;
        var isPrimary = false;
        var sourceOperation = this.form.datasource && this.form.datasource.execute(DataSource.Operation.GET_OPERATION_TYPE);
        if (sourceOperation && sourceOperation !== 'read') {
            return sourceOperation;
        }
        /*If OperationType is not set then based on the formdata object return the operation type,
            this case occurs only if the form is outside a livegrid*/
        /*If the formdata object has primary key value then return update else insert*/
        if (this.form.primaryKey && !_.isEmpty(this.form.formdata)) {
            /*If only one column is primary key*/
            if (this.form.primaryKey.length === 1) {
                if (this.form.formdata[this.form.primaryKey[0]]) {
                    operation = Live_Operations.UPDATE;
                }
                /*If only no column is primary key*/
            }
            else if (this.form.primaryKey.length === 0) {
                _.forEach(this.form.formdata, function (value) {
                    if (value) {
                        isPrimary = true;
                    }
                });
                if (isPrimary) {
                    operation = Live_Operations.UPDATE;
                }
                /*If multiple columns are primary key*/
            }
            else {
                isPrimary = _.some(this.form.primaryKey, function (primaryKey) {
                    if (_this.form.formdata[primaryKey]) {
                        return true;
                    }
                });
                if (isPrimary) {
                    operation = Live_Operations.UPDATE;
                }
            }
        }
        return operation || Live_Operations.INSERT;
    };
    LiveFormDirective.prototype.getPrevDataValues = function () {
        var prevDataValues = _.fromPairs(_.map(this.form.prevDataValues, function (item) {
            return [item.key, item.value];
        })); // Convert of array of values to an object
        this.form.formFields.forEach(function (field) {
            field.value = prevDataValues[field.key] || '';
            field.resetDisplayInput();
        });
        return prevDataValues;
    };
    LiveFormDirective.prototype.savePrevDataValues = function () {
        this.form.prevDataValues = this.form.formFields.map(function (obj) {
            return { 'key': obj.key, 'value': obj.value };
        });
    };
    LiveFormDirective.prototype.clearData = function () {
        this.form.toggleMessage(false);
        this.form.emptyDataModel();
    };
    LiveFormDirective.prototype.setReadonlyFields = function () {
        this.form.formFields.forEach(function (field) {
            field.setReadOnlyState();
        });
    };
    LiveFormDirective.prototype.reset = function () {
        var _this = this;
        var prevDataValues;
        this.form.resetFormState();
        prevDataValues = this.getPrevDataValues();
        this.form.formFields.forEach(function (field) {
            if (field.type === DataType.BLOB || field.widgettype === 'upload') {
                _this.form.resetFileUploadWidget(field, true);
                field.href = _this.getBlobURL(prevDataValues, field.key, field.value);
            }
        });
        this.form.constructDataObject();
    };
    LiveFormDirective.prototype.closeDialog = function () {
        if (this.form.isLayoutDialog) {
            this.dialogService.close(this.form.dialogId);
        }
    };
    LiveFormDirective.prototype.cancel = function () {
        this.form.clearMessage();
        this.form.isUpdateMode = false;
        this.form.reset();
        /*Show the previous selected data*/
        if (this.form.isSelected) {
            this.getPrevformFields();
        }
        this.form.isUpdateMode = false;
        if (this.form._liveTableParent) {
            this.form._liveTableParent.onCancel();
        }
        else {
            /*
              WMS-18759: dialog close is calling twice and dialog onHidden subscription is calling once from the library
              due to this if we have another dialog inside liveform and if we click on cancel of second dialog the library
              subscription is calling for both the dialogs.
             */
            this.closeDialog();
        }
        $appDigest();
    };
    LiveFormDirective.prototype.delete = function (callBackFn) {
        this.form.resetFormState();
        this.form.operationType = Live_Operations.DELETE;
        this.form.prevDataObject = getClonedObject(this.form.formdata || {});
        this.form.save(undefined, undefined, undefined, callBackFn);
    };
    // Function use to save the form and open new form after save
    LiveFormDirective.prototype.saveAndNew = function () {
        this.save(undefined, true, true);
    };
    // Function use to save the form and open new form after save
    LiveFormDirective.prototype.saveAndView = function () {
        this.save(undefined, false);
    };
    LiveFormDirective.prototype.submitForm = function ($event) {
        this.save($event);
    };
    LiveFormDirective.prototype.save = function (event, updateMode, newForm) {
        var _this = this;
        if (!this.form.datasource) {
            return;
        }
        var data, prevData, operationType, isValid;
        var requestData = {};
        operationType = this.form.operationType = this.form.operationType || this.findOperationType();
        // Disable the form submit if form is in invalid state.
        if (this.form.validateFieldsOnSubmit()) {
            return;
        }
        data = getClonedObject(this.form.constructDataObject());
        prevData = this.form.prevformFields ? this.form.constructDataObject(true) : data;
        try {
            isValid = this.form.invokeEventCallback('beforeservicecall', { $event: event, $operation: this.form.operationType, $data: data, options: requestData });
            if (isValid === false) {
                return;
            }
            if (isValid && isValid.error) {
                this.form.toggleMessage(true, isValid.error, 'error');
                return;
            }
        }
        catch (err) {
            if (err.message === 'Abort') {
                return;
            }
        }
        // If operation is update, form is not touched and current data and previous data is same, Show no changes detected message
        if (this.form.operationType === Live_Operations.UPDATE && this.form.ngform && this.form.ngform.pristine &&
            (this.form.isSelected && _.isEqual(data, prevData))) {
            this.form.toggleMessage(true, this.form.appLocale.MESSAGE_NO_CHANGES, 'info', '');
            $appDigest();
            return;
        }
        requestData.row = data;
        requestData.transform = true;
        requestData.skipNotification = true;
        if (operationType === Live_Operations.UPDATE) {
            requestData.rowData = this.form.formdata;
            requestData.prevData = prevData;
        }
        performDataOperation(this.form.datasource, requestData, {
            operationType: operationType
        }).then(function (response) {
            var msg = operationType === Live_Operations.INSERT ? _this.form.insertmessage : (operationType === Live_Operations.UPDATE ?
                _this.form.updatemessage : _this.form.deletemessage);
            var result;
            if (operationType === Live_Operations.DELETE) {
                result = requestData.row;
                _this.form.emptyDataModel();
                _this.form.prevDataValues = [];
                _this.form.isSelected = false;
            }
            else {
                result = response;
            }
            if (!_this.form.onSuccess) {
                _this.form.toggleMessage(true, msg, 'success');
            }
            if (_this.form._liveTableParent) {
                // highlight the current updated row
                _this.form._liveTableParent.onResult(operationType, response, newForm, updateMode);
            }
            else {
                /*get updated data without refreshing page*/
                _this.form.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                    'skipToggleState': true
                });
                _this.onDataSourceUpdate(response, newForm, updateMode);
            }
            return {
                'result': result,
                'status': true
            };
        }, function (error) {
            if (!_this.form.onError) {
                _this.form.toggleMessage(true, error, 'error');
            }
            $appDigest();
            return {
                'result': error,
                'status': false
            };
        }).then(function (response) {
            // reset the form to pristine state
            _this.form.resetFormState();
            _this.form.onResultCb(response.result, response.status, event);
        });
    };
    LiveFormDirective.initializeProps = registerLiveFormProps();
    LiveFormDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [FormComponent,] }] },
        { type: LiveTableComponent, decorators: [{ type: Optional }] },
        { type: ToDatePipe },
        { type: AbstractDialogService },
        { type: String, decorators: [{ type: Attribute, args: ['formlayout',] }] }
    ]; };
    LiveFormDirective = __decorate([
        Directive({
            selector: '[wmLiveForm]'
        }),
        __param(0, Self()), __param(0, Inject(FormComponent)),
        __param(1, Optional()),
        __param(4, Attribute('formlayout')),
        __metadata("design:paramtypes", [Object, LiveTableComponent,
            ToDatePipe,
            AbstractDialogService, String])
    ], LiveFormDirective);
    return LiveFormDirective;
}());

var components = [
    FormComponent,
    FormWidgetDirective,
    FormActionDirective,
    FormFieldDirective,
    LiveActionsDirective,
    DependsonDirective,
    LiveFilterDirective,
    LiveFormDirective
];
var FormModule = /** @class */ (function () {
    function FormModule() {
    }
    FormModule = __decorate([
        NgModule({
            imports: [
                BasicModule,
                CommonModule,
                DesignDialogModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], FormModule);
    return FormModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { FormComponent, FormModule, ɵ0, ɵ1, ɵ2, ɵ3, registerFormProps as ɵa, registerLiveFormProps as ɵb, registerLiveFilterProps as ɵc, FormWidgetDirective as ɵd, FormActionDirective as ɵe, registerProps as ɵf, FormFieldDirective as ɵg, registerProps$1 as ɵh, LiveActionsDirective as ɵi, DependsonDirective as ɵj, LiveFilterDirective as ɵk, LiveFormDirective as ɵl };
//# sourceMappingURL=index.js.map
