import { __decorate, __metadata, __param } from 'tslib';
import { Injector, ElementRef, HostBinding, HostListener, Input, Directive, ChangeDetectorRef, NgZone, Attribute, ContentChild, TemplateRef, ContentChildren, ViewChild, ViewChildren, QueryList, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, StylableComponent, styler, APPLY_STYLES_TYPE, extractDataSourceName, NAVIGATION_TYPE, getOrderedDataset, groupData, configureDnD, DEBOUNCE_TIMES, handleHeaderClick, toggleAllHeaders, ToDatePipe, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { PaginationComponent, PaginationModule } from '@wm/components/data/pagination';
import { isMobileApp, $watch, App, noop, getClonedObject, isDefined, $appDigest, $invokeWatchers, DataSource, isDataSourceEqual, isObject, switchClass, setListClass, isNumber, AppDefaults, StatePersistence } from '@wm/core';
import { ButtonComponent } from '@wm/components/input';
import { Subject } from 'rxjs';
import { SwipeAnimation } from '@swipey';

const registerProps = () => {
    register('wm-list', new Map([
        ['boundarylinks', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['class', PROP_STRING],
        ['collapsible', PROP_BOOLEAN],
        ['dateformat', PROP_STRING],
        ['dataset', PROP_ANY],
        ['datasource', PROP_ANY],
        ['directionlinks', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['disableitem', PROP_BOOLEAN],
        ['enablereorder', PROP_BOOLEAN],
        ['forceellipses', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['groupby', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['itemclass', Object.assign({ value: '' }, PROP_STRING)],
        ['itemsperrow', PROP_STRING],
        ['listclass', Object.assign({ value: 'list-group' }, PROP_STRING)],
        ['multiselect', PROP_BOOLEAN],
        ['loadingdatamsg', Object.assign({ value: 'Loading...' }, PROP_STRING)],
        ['loadingicon', Object.assign({ value: 'fa fa-circle-o-notch' }, PROP_STRING)],
        ['match', PROP_STRING],
        ['maxsize', Object.assign({ value: 5 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['navigation', PROP_STRING],
        ['navigationalign', Object.assign({ value: 'left' }, PROP_STRING)],
        ['nodatamessage', Object.assign({ value: 'No data found' }, PROP_STRING)],
        ['ondemandmessage', Object.assign({ value: 'Load More' }, PROP_STRING)],
        ['orderby', PROP_STRING],
        ['paginationclass', PROP_STRING],
        ['pagesize', PROP_NUMBER],
        ['pulltorefresh', Object.assign({ value: isMobileApp() }, PROP_BOOLEAN)],
        ['selectfirstitem', PROP_BOOLEAN],
        ['selectionlimit', PROP_NUMBER],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['showcount', PROP_BOOLEAN],
        ['showrecordcount', PROP_BOOLEAN],
        ['statehandler', Object.assign({ value: 'none' }, PROP_STRING)],
        ['subheading', PROP_STRING],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING]
    ]));
};

let ListItemDirective = class ListItemDirective {
    constructor(inj, elRef, app) {
        this.inj = inj;
        this.app = app;
        this.destroy = new Subject();
        this.destroy$ = this.destroy.asObservable();
        this.itemClass = '';
        this._currentItemWidgets = {};
        this.isActive = false;
        this.disableItem = false;
        this.nativeElement = elRef.nativeElement;
        this.listComponent = inj.view.component;
        this.context = inj.view.context;
        this.itemClassWatcher(this.listComponent);
        this.disableItemWatcher(this.listComponent);
        $(this.nativeElement).data('listItemContext', this);
    }
    onFocus() {
        // maintains which element is focused/selected most recently.
        this.listComponent.lastSelectedItem = this;
    }
    get $index() {
        return this.context.index;
    }
    get $even() {
        return this.context.even;
    }
    get $odd() {
        return this.context.odd;
    }
    get $first() {
        return this.context.first;
    }
    get $last() {
        return this.context.last;
    }
    get currentItemWidgets() {
        const componentElements = Array.from(this.nativeElement.querySelectorAll('[widget-id]'));
        return Object.assign(this._currentItemWidgets, componentElements.reduce((result, comp) => {
            result[comp.widget.name] = comp.widget;
            return result;
        }, {}));
    }
    set wmListItem(val) {
        this.item = val;
    }
    registerWatch(expression, callback) {
        // Removing ngFor context as the same properties are availble on listitem scope.
        // passing viewparent context for accessing varibales and widgets.
        this.destroy$.subscribe($watch(expression, this.listComponent.viewParent, this, callback));
    }
    itemClassWatcher(listComponent) {
        if (listComponent.binditemclass) {
            this.registerWatch(listComponent.binditemclass, nv => this.itemClass = nv || '');
        }
        else {
            this.itemClass = listComponent.itemclass;
        }
    }
    disableItemWatcher($list) {
        if ($list.binddisableitem) {
            this.registerWatch($list.binddisableitem, nv => this.disableItem = nv || false);
        }
        else {
            this.disableItem = $list.disableitem || false;
        }
    }
    setUpCUDHandlers() {
        const $editItem = this.nativeElement.querySelector('.edit-list-item');
        const $deleteItem = this.nativeElement.querySelector('.delete-list-item');
        if ($editItem) {
            // Triggered on click of edit action
            $editItem.addEventListener('click', evt => {
                this.listComponent.update();
            });
        }
        if ($deleteItem) {
            // Triggered on click of delete action
            $deleteItem.addEventListener('click', evt => {
                this.listComponent.delete();
            });
        }
    }
    ngOnInit() {
        if (this.listComponent.mouseEnterCB) {
            this.nativeElement.addEventListener('mouseenter', ($event) => {
                this.listComponent.invokeEventCallback('mouseenter', { widget: this, $event });
            });
        }
        if (this.listComponent.mouseLeaveCB) {
            this.nativeElement.addEventListener('mouseleave', ($event) => {
                this.listComponent.invokeEventCallback('mouseleave', { widget: this, $event });
            });
        }
        // adding item attribute on every list item
        $(this.nativeElement).attr('listitemindex', this.$index);
    }
    ngAfterViewInit() {
        this.setUpCUDHandlers();
    }
};
ListItemDirective.ctorParameters = () => [
    { type: Injector },
    { type: ElementRef },
    { type: App }
];
__decorate([
    HostBinding('class.active'),
    __metadata("design:type", Object)
], ListItemDirective.prototype, "isActive", void 0);
__decorate([
    HostBinding('class.disable-item'),
    __metadata("design:type", Object)
], ListItemDirective.prototype, "disableItem", void 0);
__decorate([
    HostListener('focus'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ListItemDirective.prototype, "onFocus", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], ListItemDirective.prototype, "wmListItem", null);
ListItemDirective = __decorate([
    Directive({
        selector: '[wmListItem]',
        exportAs: 'listItemRef'
    }),
    __metadata("design:paramtypes", [Injector, ElementRef, App])
], ListItemDirective);

class ListAnimator extends SwipeAnimation {
    constructor(list) {
        super();
        this.list = list;
        this.$el = $(this.list.getNativeElement()).find('ul.app-livelist-container').first();
        this.leftChildrenCount = this.$el.find('>.app-list-item-left-action-panel > button:visible').length;
        this.rightChildrenCount = this.$el.find('>.app-list-item-right-action-panel > button:visible').length;
        // when there are no children in both the templates then do not apply swipeAnimation;
        if (!this.leftChildrenCount && !this.rightChildrenCount) {
            return;
        }
        // initialise swipe animation on the list component.
        this.init(this.$el);
        // retrieves all the button components which are placed outside the listTemplate.
        this.$btnSubscription = this.list.btnComponents.changes.subscribe(items => this.actionItems = items);
    }
    // This method sets the css for left or right action panels based on the template. Appends the actionTemplate before li.
    createActionPanel(li, actionPanelTemplate) {
        actionPanelTemplate.css({
            width: li.outerWidth() + 'px',
            height: li.outerHeight() + 'px',
            marginBottom: -1 * li.outerHeight() + 'px',
            float: 'left',
            padding: 0
        });
        return actionPanelTemplate.insertBefore(li);
    }
    // Returns the total width occupied by all the children inside the element
    computeTotalChildrenWidth($ele) {
        return _.reduce($ele.children(), (totalWidth, el) => {
            return totalWidth + $(el).outerWidth();
        }, 0);
    }
    // Returns amount of transition to be applied on element when swiped left or right
    computeTransitionProportions($ele) {
        const totalWidth = this.computeTotalChildrenWidth($ele);
        const reverse = this.position === 'right';
        let d = 0;
        return _.map($ele.children(), e => {
            const f = (totalWidth - d) / totalWidth;
            d += $(e).outerWidth();
            return reverse ? f : (d / totalWidth);
        });
    }
    // Resets the transform applied on the element.
    resetElement(el) {
        if (el) {
            el.css({
                transform: 'none',
                transition: 'none'
            });
        }
    }
    resetState() {
        this.resetElement(this.li);
        this.resetElement(this.actionPanel);
        if (this.actionPanel) {
            this.actionPanel = null;
        }
    }
    // Returns the target button (child element) inside the left and right actionPanels.
    getChildActionElement(actionTemplate) {
        if (actionTemplate.children().length) {
            if (this.position === 'left') {
                return actionTemplate.children().first();
            }
            return actionTemplate.children().last();
        }
    }
    // create the actionPanels and set the background-color for remaining panel as that of first child element
    // calculates the children's width and its transition proportionates.
    initActionPanel(actionTemplate) {
        this.actionPanel = this.createActionPanel(this.li, actionTemplate);
        this.actionPanel.css({
            backgroundColor: this.getChildActionElement(this.actionPanel).css('background-color')
        });
        this.limit = this.computeTotalChildrenWidth(this.actionPanel);
        this.transitionProportions = this.computeTransitionProportions(this.actionPanel);
    }
    bounds(e, $d) {
        const target = $(e.target).closest('li');
        // default bounds when action template markup is not available.
        let bounds = {
            strictUpper: true,
            strictLower: true,
            lower: 0,
            center: 0,
            upper: 0
        };
        // apply swipe animation only on list items having "app-list-item" class.
        if (!target.hasClass('app-list-item')) {
            return bounds;
        }
        if (!this.li || this.li[0] !== target[0]) {
            let selector = $d > 0 ? '.app-list-item-left-action-panel' : '.app-list-item-right-action-panel';
            let actionTemplate = this.$el.find('>' + selector);
            // when groupby is set select the action panel from the list group items.
            if (!actionTemplate.length && this.list.groupby) {
                selector = 'li > ul.list-group >' + selector;
                actionTemplate = this.$el.find('>' + selector);
            }
            // check for children visiblity. If children are visible then initiate the action panel.
            if (!actionTemplate.length || !actionTemplate.find('button:visible').length) {
                return bounds;
            }
            this.resetState();
            this.li = target;
            this.position = actionTemplate.attr('position');
            this.initActionPanel(actionTemplate);
            if ($d > 0) {
                // bounds while swiping from right to left to open left action panel. It can be moved upto limit value (Upper bound).
                bounds = {
                    strictUpper: false,
                    lower: 0,
                    center: 0,
                    upper: this.limit
                };
            }
            else {
                // bounds while swiping from left to right to open right action panel. It can be moved in reverse direction with -limit value (lower bound).
                bounds = {
                    strictLower: false,
                    lower: -this.limit,
                    center: 0,
                    upper: 0
                };
            }
        }
        else if (this.position === 'left') {
            // when left action panel is visible (i.e. center at limit value) then this can be moved by distance (limit) in reverse direction to close the view.
            bounds = {
                strictUpper: false,
                lower: -this.limit,
                center: this.limit
            };
        }
        else if (this.position === 'right') {
            // when right action panel is visible (i.e. center at -limit value) then this can be moved by distance (limit) to close the view.
            bounds = {
                center: -this.limit,
                upper: this.limit,
                strictLower: false
            };
        }
        return bounds;
    }
    context() {
        return {
            computeActionTransition: (index, $d) => {
                const sign = $d > 0 ? 1 : -1;
                if (sign * $d > this.limit) {
                    // once the distance swiped is beyond the limit, then calculate the proportionate distance moved after the limit value.
                    return ($d - sign * this.limit) + (this.transitionProportions[index] * sign * this.limit);
                }
                return this.transitionProportions[index] * $d;
            }
        };
    }
    animation() {
        return [{
                target: () => this.li,
                css: {
                    transform: 'translate3d(${{$D + $d}}px, 0, 0)'
                }
            }, {
                target: () => (this.actionPanel && this.actionPanel.children()),
                css: {
                    transform: 'translate3d(${{computeActionTransition($i, $D + $d)}}px, 0, 0)'
                }
            }];
    }
    // Triggers full swipe event on the target element.
    invokeFullSwipeEvt($event) {
        let target, actions, index;
        // Check if button are visible or not, invoke the tap event of the last button which is visible.
        if (this.position === 'left') {
            actions = this.actionItems.filter(btn => {
                return btn.getAttr('swipe-position') === 'left' && btn.$element.is(':visible');
            });
            index = 0;
        }
        else {
            actions = this.actionItems.filter(btn => {
                return btn.getAttr('swipe-position') === 'right' && btn.$element.is(':visible');
            });
            index = actions.length - 1;
        }
        target = actions[index];
        if (target && target.hasEventCallback('tap')) {
            target.invokeEventCallback('tap', { $event });
        }
        this.resetState();
        this.li = null;
    }
    // Called when swipeEnd is triggered. d contains the total distance covered by the element until touchEnd.
    onAnimation($event, d) {
        // set the selecteditem on the list component on swipe.
        this.list.triggerListItemSelection(this.li, $event);
        if (this.actionPanel && this.actionPanel.attr('enablefullswipe') === 'true') {
            const sign = d > 0 ? 1 : -1;
            const $el = this.getChildActionElement(this.actionPanel);
            if ($el) {
                const index = this.position === 'right' ? this.rightChildrenCount - 1 : 0;
                // proportionate amount of distance covered by the target element.
                const distPercentage = this.transitionProportions[index] * sign * d * 100 / (this.li.outerWidth() - this.limit + $el.width());
                // If distance travelled by the target button element is more than 50% of the list item width then invoke the fullswipe.
                if (distPercentage > 50) {
                    // invoke fullswipe event
                    this.invokeFullSwipeEvt($event);
                }
            }
        }
    }
    onLower() {
        if (this.position === 'left') {
            this.resetState();
            this.li = null;
        }
    }
    onUpper() {
        if (this.position === 'right') {
            this.resetState();
            this.li = null;
        }
    }
    threshold() {
        return 10;
    }
}

var ListComponent_1;
const DEFAULT_CLS = 'app-livelist app-panel';
const WIDGET_CONFIG = { widgetType: 'wm-list', hostClass: DEFAULT_CLS };
const unsupportedStatePersistenceTypes = ['On-Demand', 'Scroll'];
let ListComponent = ListComponent_1 = class ListComponent extends StylableComponent {
    constructor(inj, cdRef, datePipe, app, appDefaults, ngZone, binditemclass, binddisableitem, binddataset, binddatasource, mouseEnterCB, mouseLeaveCB, statePersistence) {
        let resolveFn = noop;
        const propsInitPromise = new Promise(res => resolveFn = res);
        super(inj, WIDGET_CONFIG, propsInitPromise);
        this.propsInitPromise = propsInitPromise;
        this.promiseResolverFn = resolveFn;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SHELL);
        this.cdRef = cdRef;
        this.ngZone = ngZone;
        this.datePipe = datePipe;
        this.statePersistence = statePersistence;
        this._pageLoad = true;
        this.binditemclass = binditemclass;
        this.binddisableitem = binddisableitem;
        this.binddataset = binddataset;
        this.mouseEnterCB = mouseEnterCB;
        this.mouseLeaveCB = mouseLeaveCB;
        this.binddatasource = binddatasource;
        this.app = app;
        this.appDefaults = appDefaults;
        this.variableInflight = false;
        this.noDataFound = !binddataset;
        // Updates pagination, filter, sort etc options for service and crud variables
        this._listenerDestroyers = [
            this.app.subscribe('check-state-persistence-options', options => {
                let dataSourceName = _.get(this.datasource, 'name');
                // in Prefabs, this.datasource is not resolved at the time of variable invocation, so additional check is required.
                if (!dataSourceName) {
                    dataSourceName = extractDataSourceName(this.binddatasource);
                }
                if (_.get(options, 'variable.name') !== dataSourceName) {
                    return;
                }
                this.handleStateParams(options);
            }),
            // Show loading status based on the variable life cycle
            this.app.subscribe('toggle-variable-state', this.handleLoading.bind(this)),
            this.app.subscribe('setup-cud-listener', param => {
                if (this.nativeElement.getAttribute('name') !== param) {
                    return;
                }
                this._isDependent = true;
            })
        ];
    }
    get selecteditem() {
        if (this.multiselect) {
            return getClonedObject(this._items);
        }
        if (_.isEmpty(this._items)) {
            return {};
        }
        return getClonedObject(this._items[0]);
    }
    /**
     * Returns list of widgets present on list item by considering name and index of the widget.
     * If we did'nt pass index, it returns array of all the widgets which are matching to widget name
     * @param widgteName: Name of the widget
     * @param index: Index of the widget
     */
    getWidgets(widgteName, index) {
        let $target;
        const retVal = [];
        if (!widgteName) {
            return;
        }
        if (!isDefined(index)) {
            _.forEach(this.listItems.toArray(), (el) => {
                $target = _.get(el.currentItemWidgets, widgteName);
                if ($target) {
                    retVal.push($target);
                }
            });
            return retVal;
        }
        index = +index || 0;
        $target = _.get(this.listItems.toArray(), index);
        if ($target) {
            return [_.get($target.currentItemWidgets, widgteName)];
        }
    }
    // returns listitem reference by index value. This refers to the same method getListItemByIndex.
    getItem(index) {
        return this.getListItemByIndex(index);
    }
    // return index of listItem(listItemDirective). This refers to the same method getListItemIndex.
    getIndex(item) {
        return this.getListItemIndex(item);
    }
    set selecteditem(items) {
        this._items.length = 0;
        this.deselectListItems();
        if (_.isArray(items)) {
            items.forEach(item => this.selectItem(item));
        }
        else {
            this.selectItem(items);
        }
        $appDigest();
    }
    getConfiguredState() {
        const mode = this.statePersistence.computeMode(this.statehandler);
        return mode && mode.toLowerCase();
    }
    handleStateParams(options) {
        if (this._pageLoad && this.getConfiguredState() !== 'none') {
            this._pageLoad = false;
            const widgetState = this.statePersistence.getWidgetState(this);
            if (_.get(widgetState, 'pagination')) {
                options.options = options.options || {};
                options.options.page = widgetState.pagination;
            }
            if (_.get(widgetState, 'selectedItem')) {
                this._selectedItemsExist = true;
            }
        }
    }
    triggerWMEvent(eventName, item) {
        $invokeWatchers(true);
        // If we have multiselect for the livelist(List with form template), in run mode deleting a record is getting failed. Becuase the selecteditem will be array of objects. So consider the last object.
        let row = this.multiselect ? _.last(this.selecteditem) : this.selecteditem;
        if (item) {
            row = item;
        }
        this.app.notify('wm-event', { eventName, widgetName: this.name, row: row });
    }
    create() {
        if (this._isDependent) {
            this.triggerWMEvent('insert');
        }
    }
    editRow(item) {
        if (this._isDependent) {
            let listItem;
            if (item) {
                listItem = this.getItemRefByIndexOrModel(item).item;
            }
            this.triggerWMEvent('update', listItem);
        }
    }
    update(item) {
        this.editRow(item);
    }
    deleteRow(item) {
        if (this._isDependent) {
            let listItem;
            if (item) {
                listItem = this.getItemRefByIndexOrModel(item).item;
            }
            this.triggerWMEvent('delete', listItem);
        }
    }
    delete(item) {
        this.deleteRow(item);
    }
    handleLoading(data) {
        const dataSource = this.datasource;
        if (dataSource && dataSource.execute(DataSource.Operation.IS_API_AWARE) && isDataSourceEqual(data.variable, dataSource)) {
            this.ngZone.run(() => {
                this.handleStateParams(data);
                this.variableInflight = data.active;
            });
        }
    }
    resetNavigation() {
        this.showNavigation = false;
        this.navControls = undefined;
        this.infScroll = false;
        this.onDemandLoad = false;
    }
    enableBasicNavigation() {
        this.navControls = NAVIGATION_TYPE.BASIC;
        this.showNavigation = true;
    }
    enableInlineNavigation() {
        this.navControls = NAVIGATION_TYPE.INLINE;
    }
    enableClassicNavigation() {
        this.navControls = NAVIGATION_TYPE.CLASSIC;
        this.showNavigation = true;
    }
    enablePagerNavigation() {
        this.navControls = NAVIGATION_TYPE.PAGER;
        this.showNavigation = true;
    }
    setNavigationTypeNone() {
        this.navControls = NAVIGATION_TYPE.NONE;
        this.showNavigation = false;
    }
    enableInfiniteScroll() {
        this.infScroll = true;
    }
    enableOnDemandLoad() {
        this.onDemandLoad = true;
        this.showNavigation = true;
    }
    /**
     * Sets Navigation type for the list.
     * @param type
     */
    onNavigationTypeChange(type) {
        this.resetNavigation();
        switch (type) {
            case NAVIGATION_TYPE.BASIC:
                this.enableBasicNavigation();
                break;
            case NAVIGATION_TYPE.INLINE:
                this.enableInlineNavigation();
                break;
            case NAVIGATION_TYPE.ADVANCED:
            case NAVIGATION_TYPE.CLASSIC:
                this.enableClassicNavigation();
                break;
            case NAVIGATION_TYPE.PAGER:
                this.enablePagerNavigation();
                break;
            case NAVIGATION_TYPE.NONE:
                this.setNavigationTypeNone();
                break;
            case NAVIGATION_TYPE.SCROLL:
                this.enableInfiniteScroll();
                break;
            case NAVIGATION_TYPE.ONDEMAND:
                this.enableOnDemandLoad();
                break;
        }
    }
    fetchNextDatasetOnScroll() {
        this.dataNavigator.navigatePage('next');
    }
    setIscrollHandlers(el) {
        let lastScrollTop = 0;
        const wrapper = _.get(el.iscroll, 'wrapper');
        const self = el.iscroll;
        el.iscroll.on('scrollEnd', () => {
            const clientHeight = wrapper.clientHeight, totalHeight = wrapper.scrollHeight, scrollTop = Math.abs(el.iscroll.y);
            if ((lastScrollTop < scrollTop) && (totalHeight * 0.9 < scrollTop + clientHeight)) {
                this.debouncedFetchNextDatasetOnScroll();
                if (self.indicatorRefresh) {
                    self.indicatorRefresh();
                }
            }
            lastScrollTop = scrollTop;
        });
    }
    // Applying iscroll event to invoke the next calls for infinte scroll.
    bindIScrollEvt() {
        const $scrollParent = this.$element.closest('[wmsmoothscroll="true"]');
        const iScroll = _.get($scrollParent[0], 'iscroll');
        // when iscroll is not initialised the notify the smoothscroll and subscribe to the iscroll update
        if (!iScroll) {
            const iScrollSubscription = this.app.subscribe('iscroll-update', (_el) => {
                if (!_.isEmpty(_el) && _el.isSameNode($scrollParent[0])) {
                    this.setIscrollHandlers($scrollParent[0]);
                    iScrollSubscription();
                }
            });
            this.app.notify('no-iscroll', $scrollParent[0]);
            return;
        }
        this.setIscrollHandlers($scrollParent[0]);
    }
    bindScrollEvt() {
        const $el = this.$element;
        const $ul = $el.find('> ul');
        const $firstChild = $ul.children().first();
        const self = this;
        let $scrollParent;
        let scrollNode;
        let lastScrollTop = 0;
        if (!$firstChild.length) {
            return;
        }
        $scrollParent = $firstChild.scrollParent(false);
        if ($scrollParent[0] === document) {
            scrollNode = document.body;
        }
        else {
            scrollNode = $scrollParent[0];
        }
        // has scroll
        if (scrollNode.scrollHeight > scrollNode.clientHeight) {
            $scrollParent
                .each((index, node) => {
                // scrollTop property is 0 or undefined for body in IE, safari.
                lastScrollTop = node === document ? (node.body.scrollTop || $(window).scrollTop()) : node.scrollTop;
            })
                .off('scroll.scroll_evt')
                .on('scroll.scroll_evt', function (evt) {
                let target = evt.target;
                let clientHeight;
                let totalHeight;
                let scrollTop;
                // scrollingElement is undefined for IE, safari. use body as target Element
                target = target === document ? (target.scrollingElement || document.body) : target;
                clientHeight = target.clientHeight;
                totalHeight = target.scrollHeight;
                scrollTop = target === document.body ? $(window).scrollTop() : target.scrollTop;
                if ((lastScrollTop < scrollTop) && (totalHeight * 0.9 < scrollTop + clientHeight)) {
                    $(this).off('scroll.scroll_evt');
                    self.debouncedFetchNextDatasetOnScroll();
                }
                lastScrollTop = scrollTop;
            });
            $ul.off('wheel.scroll_evt');
        }
        else {
            // if there is no scrollable element register wheel event on ul element
            $ul.on('wheel.scroll_evt', e => {
                if (e.originalEvent.deltaY > 0) {
                    $ul.off('wheel.scroll_evt');
                    this.debouncedFetchNextDatasetOnScroll();
                }
            });
        }
    }
    /**
     * Update fieldDefs property, fieldDefs is the model of the List Component.
     * fieldDefs is an Array type.
     * @param newVal
     */
    updateFieldDefs(newVal) {
        if (this.infScroll || this.onDemandLoad) {
            if (!isDefined(this.fieldDefs) || this.dataNavigator.isFirstPage()) {
                this.fieldDefs = [];
                this.currentPage = 1;
            }
            else if (this.fieldDefs.length / this.pagesize <= this.dataNavigator.pageCount) {
                let itemsLength, itemsToPush = [];
                // we push the newVal only when dn.currentPage gets incremented because that is when new items gets added to newVal
                if (this.fieldDefs.length === this.currentPage * this.pagesize && (this.currentPage + 1) === this.dataNavigator.dn.currentPage) {
                    itemsToPush = newVal;
                    this.currentPage++;
                }
                else if (this.fieldDefs.length < this.currentPage * this.pagesize) {
                    if ((this.fieldDefs.length === (this.currentPage - 1) * this.pagesize) && ((this.currentPage - 1) === this.dataNavigator.dn.currentPage)) {
                        // if dn.currentPage is not incremented still only old newVal is present hence we push empty array
                        newVal = [];
                    }
                    else if (this.dataNavigator.dataSize < this.currentPage * this.pagesize) {
                        // if number of elements added to datanavigator is less than  product of currentpage and pagesize we only add elements extra elements added
                        itemsLength = this.dataNavigator.dataSize - this.fieldDefs.length;
                    }
                    else {
                        // if number of elements added to datanavigator is greater than  product of currentpage and pagesize we add elements the extra elements in newVal
                        itemsLength = this.currentPage * this.pagesize - this.fieldDefs.length;
                        this.currentPage++;
                    }
                    const startIndex = newVal.length - itemsLength;
                    itemsToPush = newVal.slice(startIndex);
                }
                else if (this.fieldDefs.length === this.currentPage * this.pagesize && this.currentPage === this.dataNavigator.dn.currentPage) {
                    // if dn.currentPage is not incremented still only old newVal is present hence we push empty array
                    itemsToPush = [];
                }
                newVal = itemsToPush;
            }
            this.fieldDefs = [...this.fieldDefs, ...newVal];
        }
        else {
            this.fieldDefs = newVal;
        }
        if (this.orderby) {
            this.fieldDefs = getOrderedDataset(this.fieldDefs, this.orderby);
        }
        if (this.groupby) {
            this.groupedData = groupData(this, this.fieldDefs, this.groupby, this.match, this.orderby, this.dateformat, this.datePipe, undefined, this.appDefaults);
        }
        if (!this.fieldDefs.length) {
            this.noDataFound = true;
            this.selecteditem = undefined;
        }
        $appDigest();
        this.listItems.setDirty();
    }
    onDataChange(newVal) {
        // Check for newVal is not empty
        if (!_.isEmpty(newVal)) {
            this.noDataFound = false;
            this.isDataChanged = true;
            if (this.datasource && this.datasource.execute(DataSource.Operation.IS_API_AWARE)) {
                // clone the the data in case of live and service variables to prevent the two-way binding for these variables.
                newVal = _.cloneDeep(newVal);
            }
            if (isObject(newVal) && !_.isArray(newVal)) {
                newVal = _.isEmpty(newVal) ? [] : [newVal];
            }
            if (_.isString(newVal)) {
                newVal = newVal.split(',');
            }
            // if the page number is greater than 1 on initial load then we render the first page.
            if (this.datasource && this.datasource.owner === 'App' && (this.infScroll || this.onDemandLoad) && !this.currentPage && this.datasource.execute(DataSource.Operation.GET_PAGING_OPTIONS).number > 0) {
                newVal = this.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                    'page': 1
                });
            }
            if (_.isArray(newVal)) {
                if (newVal.length) {
                    this.invokeEventCallback('beforedatarender', { $data: newVal });
                }
                this.updateFieldDefs(newVal);
            }
        }
        else {
            this.updateFieldDefs([]);
        }
    }
    // Updates the dataSource when pagination is enabled for the Component.
    setupDataSource() {
        const dataNavigator = this.dataNavigator;
        dataNavigator.options = {
            maxResults: this.pagesize || 20
        };
        this.dataNavigatorWatched = true;
        if (this.navigatorResultWatch) {
            this.navigatorResultWatch.unsubscribe();
        }
        /*Register a watch on the "result" property of the "dataNavigator" so that the paginated data is displayed in the live-list.*/
        this.navigatorResultWatch = dataNavigator.resultEmitter.subscribe((newVal) => {
            this.onDataChange(newVal);
        }, true);
        /*De-register the watch if it is exists */
        if (this.navigatorMaxResultWatch) {
            this.navigatorMaxResultWatch.unsubscribe();
        }
        /*Register a watch on the "maxResults" property of the "dataNavigator" so that the "pageSize" is displayed in the live-list.*/
        this.navigatorMaxResultWatch = dataNavigator.maxResultsEmitter.subscribe((val) => {
            this.pagesize = val;
        });
        dataNavigator.maxResults = this.pagesize || 20;
        this.removePropertyBinding('dataset');
        // when list having "datasetboundexpr" attr indicates that list is bound to item context
        // when dataset is bound to "item.FIELD" then item context is passed as the datasource.
        const datasetBoundExpr = this.getAttr('datasetboundexpr');
        this.dataNavigator.setBindDataSet(this.binddataset, this.viewParent, datasetBoundExpr ? this.context : this.datasource, this.dataset, this.binddatasource, datasetBoundExpr, this.statehandler);
    }
    onDataSetChange(newVal) {
        if (_.get(this.datasource, 'category') === 'wm.Variable' && this.getConfiguredState() !== 'none' && this._pageLoad) {
            const widgetState = this.statePersistence.getWidgetState(this);
            this._pageLoad = false;
            if (_.get(widgetState, 'pagination')) {
                this.dataNavigator.pageChanged({ page: widgetState.pagination }, true);
            }
            if (_.get(widgetState, 'selectedItem')) {
                this._selectedItemsExist = true;
            }
        }
        if (!this.dataNavigatorWatched) {
            if (this.navigation && this.navigation !== NAVIGATION_TYPE.NONE) {
                this.setupDataSource();
            }
            else {
                this.onDataChange(newVal);
            }
        }
        else if (this.navigation && this.navigation !== NAVIGATION_TYPE.NONE) {
            // If navigation is already setup and datasource is changed, update the datasource on navigation
            this.dataNavigator.setDataSource(this.datasource);
        }
    }
    // All the ListItem's Active state is set to false.
    deselectListItems() {
        this.listItems.forEach(item => item.isActive = false);
    }
    // Deselect all the ListItems and clear the selecteditem(InOutBound Property model)
    clearSelectedItems() {
        this.deselectListItems();
        this._items.length = 0;
        $appDigest();
    }
    /**
     * return the ListItemDirective instance by checking the equality of the model.
     * @param listModel: model to be searched for
     * @returns ListItem if the model is matched else return null.
     */
    getListItemByModel(listModel) {
        return this.listItems.find((listItem) => {
            let itemObj = listItem.item;
            if (this.groupby && !_.has(listModel, '_groupIndex')) {
                // If groupby is enabled, item contains _groupIndex property which should be excluded while comparing model.
                itemObj = _.clone(itemObj);
                delete itemObj._groupIndex;
            }
            if (_.isEqual(itemObj, listModel)) {
                return true;
            }
        }) || null;
    }
    updateSelectedItemsWidgets(statePersistenceTriggered) {
        let obj = {}, widgetState;
        const pageNum = _.get(this.dataNavigator, 'dn.currentPage') || 1;
        if (this.getConfiguredState() !== 'none') {
            // remove previously configured selected items for current page and construct new ones later below.
            widgetState = this.statePersistence.getWidgetState(this) || {};
            if (_.get(widgetState, 'selectedItem')) {
                // when multiselect is on and an item is selected without pressing CTRL, previously selected items in state should be empty.
                if (this.multiselect && this.selecteditem.length === 1) {
                    widgetState.selectedItem = [];
                }
                else {
                    _.remove(widgetState.selectedItem, function (selectedItem) {
                        return selectedItem.page === pageNum;
                    });
                }
            }
        }
        if (this.multiselect) {
            this.selectedItemWidgets.length = 0;
        }
        this.listItems.forEach((item, index) => {
            if (item.isActive) {
                if (this.multiselect) {
                    this.selectedItemWidgets.push(item.currentItemWidgets);
                }
                else {
                    this.selectedItemWidgets = item.currentItemWidgets;
                }
                obj = { page: pageNum, index: index };
                if (_.get(widgetState, 'selectedItem') && this.multiselect) {
                    if (!_.some(widgetState.selectedItem, obj)) {
                        widgetState.selectedItem.push(obj);
                    }
                }
                else {
                    widgetState.selectedItem = [obj];
                }
            }
        });
        if (this.getConfiguredState() !== 'none' && !statePersistenceTriggered) {
            if (unsupportedStatePersistenceTypes.indexOf(this.navigation) < 0) {
                this.statePersistence.removeWidgetState(this, 'selectedItem');
                this.statePersistence.setWidgetState(this, { 'selectedItem': widgetState.selectedItem });
            }
            else {
                console.warn('Retain State handling on Widget ' + this.name + ' is not supported for current pagination type.');
            }
        }
    }
    /**
     * Selects the listItem and updates selecteditem property.
     * If the listItem is already a selected item then deselects the item.
     * @param {ListItemDirective} $listItem: Item to be selected of deselected.
     */
    toggleListItemSelection($listItem, statePersistenceTriggered) {
        // item is not allowed to get selected if it is disabled.
        if ($listItem && !$listItem.disableItem) {
            let item = $listItem.item;
            if (this.groupby && _.has(item, '_groupIndex')) {
                // If groupby is enabled, item contains _groupIndex property which should be excluded from selecteditem.
                item = _.clone(item);
                delete item._groupIndex;
            }
            if ($listItem.isActive) {
                this._items = _.pullAllWith(this._items, [item], _.isEqual);
                $listItem.isActive = false;
            }
            else {
                // if multiselect is false, clear the selectItem list before adding an item to the selectItem list.
                if (!this.multiselect) {
                    this.clearSelectedItems();
                }
                this._items.push(item);
                this.invokeEventCallback('select', { widget: $listItem, $data: item });
                $listItem.isActive = true;
            }
            this.updateSelectedItemsWidgets(statePersistenceTriggered);
        }
    }
    /**
     * Method is Invoked when the model for the List Widget is changed.
     * @param {QueryList<ListItemDirective>} listItems
     */
    onListRender(listItems) {
        // Added render callback event. This method(onListRender) is calling multiple times so checking isDatachanged flag because this falg is changed whenever new data is rendered.
        if (this.isDataChanged) {
            // Whenever dataset is changed, trigger watchers to evaluate listitem bind expressions.
            $invokeWatchers(true);
            this.invokeEventCallback('render', { $data: this.fieldDefs });
        }
        if (this.isDataChanged && this.getConfiguredState() !== 'none' && listItems.length && this._selectedItemsExist) {
            const widgetState = this.statePersistence.getWidgetState(this);
            if (_.get(widgetState, 'selectedItem')) {
                this._selectedItemsExist = false;
                const selectedItemsLength = _.isArray(this.selecteditem) ? this.selecteditem.length : _.toNumber(!_.isEmpty(this.selecteditem));
                const currentPage = _.get(this.dataNavigator, 'dn.currentPage') || 1;
                widgetState.pagination = widgetState.pagination || 1;
                // to prevent item selection from being triggered more than once
                if (selectedItemsLength !== widgetState.selectedItem.length && widgetState.pagination === currentPage) {
                    widgetState.selectedItem.forEach((item) => {
                        if (item.page === currentPage) {
                            this.selectItem(item.index, true);
                        }
                    });
                }
            }
        }
        const selectedItems = _.isArray(this.selecteditem) ? this.selecteditem : [this.selecteditem];
        this.firstSelectedItem = this.lastSelectedItem = null;
        // don't select first item if multi-select is enabled and at least item is already selected in the list.
        // don't select first item if state information has selected items
        if (listItems.length && this.selectfirstitem && !(this._items.length && this.multiselect) && this._selectedItemsExist !== false) {
            const $firstItem = listItems.first;
            if (!$firstItem.disableItem &&
                this.isDataChanged &&
                // "infinite scroll" or "load on demand" is enabled and at least one item is selected then dont alter the selected list items.
                !((this.infScroll || this.onDemandLoad) &&
                    this._items.length)) {
                this.clearSelectedItems();
                this.firstSelectedItem = this.lastSelectedItem = $firstItem;
                // selecting the first record
                this.selectItem(0);
            }
        }
        else {
            this.deselectListItems();
            selectedItems.forEach(selecteditem => {
                const listItem = this.getListItemByModel(selecteditem);
                if (listItem) {
                    listItem.isActive = true;
                    this.lastSelectedItem = listItem;
                    // focus the active element
                    listItem.nativeElement.focus();
                }
            });
        }
        if (this.fieldDefs.length && this.infScroll) {
            const smoothScrollEl = this.$element.closest('[wmsmoothscroll]');
            // if smoothscroll is set to false, then native scroll has to be applied
            // otherwise smoothscroll events will be binded.
            if (isMobileApp() && smoothScrollEl.length && smoothScrollEl.attr('wmsmoothscroll') === 'true') {
                this.bindIScrollEvt();
            }
            else {
                this.bindScrollEvt();
            }
        }
        this.isDataChanged = false;
    }
    triggerListItemSelection($el, $event) {
        if ($el && $el[0]) {
            const listItemContext = $el.data('listItemContext');
            // Trigger click event only if the list item is from the corresponding list.
            if (listItemContext.listComponent === this) {
                this.onItemClick($event, listItemContext);
            }
        }
    }
    setupHandlers() {
        this.listItems.changes.subscribe(listItems => {
            this.onListRender(listItems);
            this.cdRef.detectChanges();
        });
        // handle click event in capturing phase.
        this.nativeElement.querySelector('ul.app-livelist-container').addEventListener('click', ($event) => {
            let target = $($event.target).closest('.app-list-item');
            // Recursively find the current list item
            while (target.get(0) && (target.closest('ul.app-livelist-container').get(0) !== $event.currentTarget)) {
                target = target.parent().closest('.app-list-item');
            }
            this.triggerListItemSelection(target, $event);
        }, true);
    }
    // Triggers on drag start while reordering.
    onReorderStart(evt, ui) {
        ui.placeholder.height(ui.item.height());
        this.$ulEle.data('oldIndex', ui.item.index());
    }
    // Triggers after the sorting.
    onUpdate(evt, ui) {
        const data = this.fieldDefs;
        const newIndex = ui.item.index();
        const oldIndex = this.$ulEle.data('oldIndex');
        if (this.getConfiguredState() !== 'none') {
            this.statePersistence.removeWidgetState(this, 'selectedItem');
        }
        const minIndex = _.min([newIndex, oldIndex]);
        const maxIndex = _.max([newIndex, oldIndex]);
        const draggedItem = _.pullAt(data, oldIndex)[0];
        this.reorderProps.minIndex = _.min([minIndex, this.reorderProps.minIndex]);
        this.reorderProps.maxIndex = _.max([maxIndex, this.reorderProps.maxIndex]);
        data.splice(newIndex, 0, draggedItem);
        this.cdRef.markForCheck();
        this.cdRef.detectChanges();
        const $changedItem = {
            oldIndex: oldIndex,
            newIndex: newIndex,
            item: data[newIndex]
        };
        this.invokeEventCallback('reorder', { $event: evt, $data: data, $changedItem });
        this.$ulEle.removeData('oldIndex');
    }
    // configures reordering the list items.
    configureDnD() {
        const options = {
            appendTo: 'body',
        };
        const $el = $(this.nativeElement);
        this.$ulEle = $el.find('.app-livelist-container');
        configureDnD(this.$ulEle, options, this.onReorderStart.bind(this), this.onUpdate.bind(this));
        this.$ulEle.droppable({ 'accept': '.app-list-item' });
    }
    // returns true if the selection limit is reached.
    checkSelectionLimit(count) {
        return (!this.selectionlimit || count < this.selectionlimit);
    }
    // returns listitem reference by index value.
    getListItemByIndex(index) {
        return this.listItems.toArray()[index];
    }
    /**
     * return index of an (listItemDirective) in the listItem
     * @param {ListItemDirective} item
     * @returns {number}
     */
    getListItemIndex(item) {
        return this.listItems.toArray().indexOf(item);
    }
    // this method is called form other data widgets like table.
    execute(operation, options) {
        if ([DataSource.Operation.IS_API_AWARE, DataSource.Operation.IS_PAGEABLE, DataSource.Operation.SUPPORTS_SERVER_FILTER].includes(operation)) {
            return false;
        }
        return this.datasource.execute(operation, options);
    }
    handleKeyDown($event, action) {
        $event.stopPropagation();
        const listItems = this.listItems;
        let presentIndex = this.getListItemIndex(this.lastSelectedItem);
        if (this.multiselect) {
            const firstIndex = this.getListItemIndex(this.firstSelectedItem);
            const selectCount = _.isArray(this.selecteditem) ? this.selecteditem.length : (_.isObject(this.selecteditem) ? 1 : 0);
            if (action === 'selectPrev') {
                if (presentIndex > 0) {
                    if ((presentIndex <= firstIndex) && this.checkSelectionLimit(selectCount)) {
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex - 1);
                        this.toggleListItemSelection(this.lastSelectedItem);
                    }
                    else if (presentIndex > firstIndex) {
                        this.toggleListItemSelection(this.getListItemByIndex(presentIndex));
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex - 1);
                    }
                    else {
                        this.invokeEventCallback('selectionlimitexceed', { $event });
                    }
                }
            }
            else if (action === 'selectNext') {
                if (presentIndex < listItems.length - 1) {
                    if ((presentIndex >= firstIndex) && this.checkSelectionLimit(selectCount)) {
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex + 1);
                        this.toggleListItemSelection(this.lastSelectedItem);
                    }
                    else if (presentIndex < firstIndex) {
                        this.toggleListItemSelection(this.getListItemByIndex(presentIndex));
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex + 1);
                    }
                    else {
                        this.invokeEventCallback('selectionlimitexceed', { $event });
                    }
                }
            }
        }
        if (action === 'focusPrev') {
            presentIndex = presentIndex <= 0 ? 0 : (presentIndex - 1);
            this.lastSelectedItem = this.getListItemByIndex(presentIndex);
            this.lastSelectedItem.nativeElement.focus();
        }
        else if (action === 'focusNext') {
            presentIndex = presentIndex < (listItems.length - 1) ? (presentIndex + 1) : (listItems.length - 1);
            this.lastSelectedItem = this.getListItemByIndex(presentIndex);
            this.lastSelectedItem.nativeElement.focus();
        }
        else if (action === 'select') {
            // if the enter click is pressed on the item which is not the last selected item, the find the item from which the event is originated.
            if (presentIndex === -1 || !$($event.target).closest(this.lastSelectedItem.nativeElement)) {
                const $li = $($event.target).closest('li.app-list-item');
                const $ul = $li.closest('ul.app-livelist-container');
                presentIndex = $ul.find('li.app-list-item').index($li);
            }
            this.onItemClick($event, this.getListItemByIndex(presentIndex));
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'dataset') {
            if (!nv && this.binddatasource && !this.datasource) {
                return;
            }
            this.onDataSetChange(nv);
        }
        else if (key === 'datasource') {
            if (this.dataset) {
                this.onDataSetChange(this.dataset);
            }
        }
        else if (key === 'navigation') {
            // Support for older projects where navigation type was advanced instead of classic
            if (nv === 'Advanced') {
                this.navigation = 'Classic';
                return;
            }
            switchClass(this.nativeElement, nv, ov);
            this.onNavigationTypeChange(nv);
            if (this.dataNavigator) {
                this.dataNavigator.navigationClass = this.paginationclass;
            }
        }
        else if (key === 'itemsperrow') {
            setListClass(this);
        }
        else if (key === 'tabindex') {
            return;
        }
        else if (key === 'pulltorefresh' && nv) {
            this.app.notify('pullToRefresh:enable');
            this.subscribeToPullToRefresh();
        }
        else if (key === 'paginationclass') {
            if (this.dataNavigator) {
                // Adding setTimeout because in pagination component updateNavSize method is overriding navigationclass
                setTimeout(() => this.dataNavigator.navigationClass = nv);
            }
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    onItemClick(evt, $listItem) {
        let selectCount;
        if (!$listItem.disableItem) {
            this.firstSelectedItem = this.firstSelectedItem || $listItem;
            // Setting selectCount value based number of items selected.
            selectCount = _.isArray(this.selecteditem) ? this.selecteditem.length : (_.isObject(this.selecteditem) ? 1 : 0);
            // Handling multiselect for mobile applications
            if (this.multiselect && isMobileApp()) {
                if (this.checkSelectionLimit(selectCount) || $listItem.isActive) {
                    this.toggleListItemSelection($listItem);
                }
                else {
                    this.invokeEventCallback('selectionlimitexceed', { $event: evt });
                }
            }
            else if ((evt.ctrlKey || evt.metaKey) && this.multiselect) {
                if (this.checkSelectionLimit(selectCount) || $listItem.isActive) {
                    this.firstSelectedItem = this.lastSelectedItem = $listItem;
                    this.toggleListItemSelection($listItem);
                }
                else {
                    this.invokeEventCallback('selectionlimitexceed', { $event: evt });
                }
            }
            else if (evt.shiftKey && this.multiselect) {
                let first = $listItem.context.index;
                let last = this.firstSelectedItem.context.index;
                // if first is greater than last, then swap values
                if (first > last) {
                    last = [first, first = last][0];
                }
                if (this.checkSelectionLimit(last - first)) {
                    this.clearSelectedItems();
                    this.listItems.forEach(($liItem) => {
                        const index = $liItem.context.index;
                        if (index >= first && index <= last) {
                            this.toggleListItemSelection($liItem);
                        }
                    });
                    this.lastSelectedItem = $listItem;
                }
                else {
                    this.invokeEventCallback('selectionlimitexceed', { $event: evt });
                }
            }
            else {
                if (!$listItem.isActive || selectCount > 1) {
                    this.clearSelectedItems();
                    this.toggleListItemSelection($listItem);
                    this.firstSelectedItem = this.lastSelectedItem = $listItem;
                }
            }
            $appDigest();
        }
    }
    // Empty the list content on clear
    clear() {
        this.updateFieldDefs([]);
    }
    /**
     *  Returns ListItem Reference based on the input provided.
     * @param val: index | model of the list item.
     * @returns {ListItemDirective}
     */
    getItemRefByIndexOrModel(val) {
        let listItem;
        if (isNumber(val)) {
            listItem = this.getListItemByIndex(val);
        }
        else {
            listItem = this.getListItemByModel(val);
        }
        return listItem;
    }
    /**
     * deselects item in the list.
     * @param val: index | model of the list item.
     */
    deselectItem(val) {
        const listItem = this.getItemRefByIndexOrModel(val);
        if (listItem && listItem.isActive) {
            this.toggleListItemSelection(listItem);
        }
    }
    /**
     * selects item in the list.
     * @param val: index | model of the list item.
     */
    selectItem(val, statePersistenceTriggered) {
        const listItem = this.getItemRefByIndexOrModel(val);
        if (!listItem) {
            return;
        }
        if (!listItem.isActive) {
            this.toggleListItemSelection(listItem, statePersistenceTriggered);
        }
        // focus the element.
        listItem.nativeElement.focus();
    }
    beforePaginationChange($event, $index) {
        this.invokeEventCallback('paginationchange', { $event, $index });
    }
    handleEvent(node, eventName, eventCallback, locals) {
        // tap and doubleTap events are not getting propagated.So, using mouse events instead.
        const touchToMouse = {
            tap: 'click',
            doubletap: 'dblclick'
        };
        if (_.includes(['click', 'tap', 'dblclick', 'doubletap'], eventName)) {
            this.eventManager.addEventListener(this.nativeElement, touchToMouse[eventName] || eventName, (evt) => {
                const target = $(evt.target).closest('.app-list-item');
                if (target.length) {
                    const listItemContext = target.data('listItemContext');
                    if (!listItemContext.disableItem) {
                        this.invokeEventCallback(eventName, { widget: listItemContext, $event: evt, item: listItemContext.item });
                    }
                }
            });
        }
    }
    // Invoke the datasource variable by default when pulltorefresh event is not specified.
    subscribeToPullToRefresh() {
        this._listenerDestroyers.push(this.app.subscribe('pulltorefresh', () => {
            if (this.datasource && this.datasource.listRecords) {
                this.datasource.listRecords();
            }
        }));
    }
    ngOnInit() {
        super.ngOnInit();
        this.handleHeaderClick = noop;
        this._items = [];
        this.fieldDefs = [];
        // When pagination is infinite scroll dataset is applying after debounce time(250ms) so making next call after previous data has rendered
        this.debouncedFetchNextDatasetOnScroll = _.debounce(this.fetchNextDatasetOnScroll, DEBOUNCE_TIMES.PAGINATION_DEBOUNCE_TIME);
        this.reorderProps = {
            minIndex: null,
            maxIndex: null
        };
    }
    setUpCUDHandlers() {
        const $addItem = document.getElementsByClassName("add-list-item")[0];
        if ($addItem) {
            // Triggered on click of add action
            $addItem.addEventListener('click', evt => {
                this.create();
            });
        }
    }
    ngAfterViewInit() {
        this.promiseResolverFn();
        this.propsInitPromise.then(() => {
            super.ngAfterViewInit();
            this.setUpCUDHandlers();
            this.selectedItemWidgets = this.multiselect ? [] : {};
            if (this.enablereorder && !this.groupby) {
                this.configureDnD();
            }
            if (this.groupby && this.collapsible) {
                this.handleHeaderClick = handleHeaderClick;
                this.toggleAllHeaders = toggleAllHeaders.bind(undefined, this);
            }
            setListClass(this);
        });
        this.setupHandlers();
        const $ul = this.nativeElement.querySelector('ul.app-livelist-container');
        styler($ul, this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        if (isMobileApp() && $ul.querySelector('.app-list-item-action-panel')) {
            this._listAnimator = new ListAnimator(this);
        }
    }
    ngOnDestroy() {
        if (this._listAnimator && this._listAnimator.$btnSubscription) {
            this._listAnimator.$btnSubscription.unsubscribe();
        }
        this._listenerDestroyers.forEach(d => d && d());
        super.ngOnDestroy();
    }
    ngOnDetach() {
        super.ngOnDetach();
        this._pageLoad = true;
    }
};
ListComponent.initializeProps = registerProps();
ListComponent.ctorParameters = () => [
    { type: Injector },
    { type: ChangeDetectorRef },
    { type: ToDatePipe },
    { type: App },
    { type: AppDefaults },
    { type: NgZone },
    { type: String, decorators: [{ type: Attribute, args: ['itemclass.bind',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['disableitem.bind',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['datasource.bind',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['mouseenter.event',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['mouseleave.event',] }] },
    { type: StatePersistence }
];
__decorate([
    ContentChild('listTemplate'),
    __metadata("design:type", TemplateRef)
], ListComponent.prototype, "listTemplate", void 0);
__decorate([
    ContentChild('listLeftActionTemplate'),
    __metadata("design:type", TemplateRef)
], ListComponent.prototype, "listLeftActionTemplate", void 0);
__decorate([
    ContentChild('listRightActionTemplate'),
    __metadata("design:type", TemplateRef)
], ListComponent.prototype, "listRightActionTemplate", void 0);
__decorate([
    ContentChildren(ButtonComponent),
    __metadata("design:type", Object)
], ListComponent.prototype, "btnComponents", void 0);
__decorate([
    ViewChild(PaginationComponent),
    __metadata("design:type", Object)
], ListComponent.prototype, "dataNavigator", void 0);
__decorate([
    ViewChildren(ListItemDirective),
    __metadata("design:type", QueryList)
], ListComponent.prototype, "listItems", void 0);
ListComponent = ListComponent_1 = __decorate([
    Component({
        selector: 'div[wmList]',
        template: "<div class=\"panel-heading\" *ngIf=\"title || subheading || iconclass\">\n    <h3 class=\"panel-title\">\n        <div class=\"pull-left\">\n            <i class=\"app-icon panel-icon {{iconclass}}\" aria-hidden=\"true\" [hidden]=\"!iconclass\"></i>\n        </div>\n        <div class=\"pull-left\">\n            <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\"></div>\n            <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n        </div>\n    </h3>\n</div>\n\n<nav class=\"app-datanavigator\" aria-label=\"Page navigation\" *ngIf=\"navigation === 'Inline' && !noDataFound\">\n    <ul class=\"pager\">\n        <li class=\"previous\" [ngClass]=\"{'disabled': (dataNavigator ? dataNavigator.isDisablePrevious : true)}\">\n            <a href=\"javascript:void(0);\" (click)=\"dataNavigator.navigatePage('prev', $event)\">\n                <i class=\"wi wi-chevron-left\" aria-hidden=\"true\"></i>\n                <span class=\"sr-only\">Previous</span>\n            </a>\n        </li>\n    </ul>\n</nav>\n\n<ul tabindex=\"0\" class=\"app-livelist-container clearfix {{listclass}}\"\n    (keydown.enter)=\"handleKeyDown($event, 'select')\"\n\n    (keydown.arrowup)=\"handleKeyDown($event, 'focusPrev')\"\n    (keydown.arrowdown)=\"handleKeyDown($event, 'focusNext')\"\n    (keydown.arrowleft)=\"handleKeyDown($event, 'focusPrev')\"\n    (keydown.arrowright)=\"handleKeyDown($event, 'focusNext')\"\n\n    (keydown.shift.arrowup)=\"handleKeyDown($event, 'selectPrev')\"\n    (keydown.shift.arrowdown)=\"handleKeyDown($event, 'selectNext')\"\n    (keydown.shift.arrowleft)=\"handleKeyDown($event, 'selectPrev')\"\n    (keydown.shift.arrowright)=\"handleKeyDown($event, 'selectNext')\"\n    [hidden]=\"noDataFound && !variableInflight\"\n>\n    <ng-template [ngIf]=\"!groupby\" [ngIfElse]=\"groupedListTemplate\">\n        <li *ngFor=\"let item of fieldDefs; index as $index; first as $first; last as $last;\" class=\"app-list-item {{itemsPerRowClass}}\" [ngClass]=\"listItemRef.itemClass\"\n            [wmListItem]=\"item\"\n            #listItemRef=\"listItemRef\"\n            tabindex=\"0\">\n            <ng-container [ngTemplateOutlet]=\"listTemplate\" [ngTemplateOutletContext]=\"{item: item, $index: $index, itemRef: listItemRef, $first: $first, $last: $last, currentItemWidgets: listItemRef.currentItemWidgets}\"></ng-container>\n        </li>\n    </ng-template>\n    <ng-container [ngTemplateOutlet]=\"listLeftActionTemplate\"></ng-container>\n    <ng-container [ngTemplateOutlet]=\"listRightActionTemplate\"></ng-container>\n</ul>\n\n<div class=\"no-data-msg\" *ngIf=\"noDataFound && !variableInflight\" [textContent]=\"nodatamessage\"></div>\n\n<div class=\"loading-data-msg\" *ngIf=\"variableInflight && !pulltorefresh\">\n    <span>\n        <i class=\"app-icon panel-icon fa-spin\" aria-hidden=\"true\" [ngClass]=\"loadingicon\"></i>\n        <span class=\"sr-only\">Loading</span>\n        <span class=\"loading-text\" [textContent]=\"loadingdatamsg\"></span>\n    </span>\n</div>\n\n<nav class=\"app-datanavigator\" *ngIf=\"navigation === 'Inline' && !noDataFound\">\n    <ul class=\"pager\">\n        <li class=\"next\" [ngClass]=\"{'disabled': (dataNavigator ? dataNavigator.isDisableNext  : true)}\">\n            <a href=\"javascript:void(0);\" (click)=\"dataNavigator.navigatePage('next', $event)\">\n                <i class=\"wi wi-chevron-right\" aria-hidden=\"true\"></i>\n                <span class=\"sr-only\">Next</span>\n            </a>\n        </li>\n    </ul>\n</nav>\n\n<div class=\"panel-footer\" *ngIf=\"navigation !== 'None'\" [hidden]=\"!showNavigation || (onDemandLoad && dataNavigator.isDisableNext) || (dataNavigator.dataSize <= pagesize) || !dataNavigator.dataSize\">\n    <nav wmPagination\n         navigationalign.bind=\"navigationalign\"\n         navigation.bind=\"navControls\"\n         showrecordcount.bind=\"showrecordcount\"\n         maxsize.bind=\"maxsize\"\n         boundarylinks.bind=\"boundarylinks\"\n         forceellipses.bind=\"forceellipses\"\n         directionlinks.bind=\"directionlinks\"\n         show.bind=\"!onDemandLoad\"\n         paginationchange.event=\"beforePaginationChange($event, $index)\"\n    ></nav>\n    <a *ngIf=\"onDemandLoad\" href=\"javascript:void(0);\"\n       (click)=\"dataNavigator.navigatePage('next', $event)\"\n       class=\"app-button btn btn-block\"\n       [ngClass]=\"paginationclass\"\n       [textContent]=\"ondemandmessage\"\n    ></a>\n</div>\n\n<!--This template will be displayed when groupby is specified-->\n<ng-template #groupedListTemplate>\n    <li *ngFor=\"let groupObj of groupedData\" class=\"app-list-item-group clearfix\">\n        <ul class=\"list-group item-group\" [ngClass]=\"listclass\">\n            <li class=\"app-list-item-header list-item list-group-header\" (click)=\"handleHeaderClick($event)\" [ngClass]=\"{'collapsible-content': collapsible}\">\n            <h4>{{groupObj.key}}\n                <div class=\"header-action\">\n                    <i class=\"app-icon wi action wi-chevron-up\" *ngIf=\"collapsible\"></i>\n                    <span *ngIf=\"showcount\" class=\"label label-default\">{{groupObj.data.length}}</span>\n                    </div>\n                </h4>\n            </li>\n            <li *ngFor=\"let item of groupObj.data; index as $index; first as $first; last as $last;\" tabindex=\"0\"\n                class=\"app-list-item group-list-item {{itemsPerRowClass}}\"\n                [ngClass]=\"listItemRef.itemClass\"\n                [wmListItem]=\"item\" #listItemRef=\"listItemRef\">\n                <ng-container [ngTemplateOutlet]=\"listTemplate\" [ngTemplateOutletContext]=\"{item: item, $index: $index, itemRef: listItemRef, $first: $first, $last: $last, currentItemWidgets: listItemRef.currentItemWidgets}\"></ng-container>\n            </li>\n        </ul>\n    </li>\n</ng-template>\n",
        providers: [
            provideAsWidgetRef(ListComponent_1)
        ]
    }),
    __param(6, Attribute('itemclass.bind')),
    __param(7, Attribute('disableitem.bind')),
    __param(8, Attribute('dataset.bind')),
    __param(9, Attribute('datasource.bind')),
    __param(10, Attribute('mouseenter.event')),
    __param(11, Attribute('mouseleave.event')),
    __metadata("design:paramtypes", [Injector,
        ChangeDetectorRef,
        ToDatePipe,
        App,
        AppDefaults,
        NgZone, String, String, String, String, String, String, StatePersistence])
], ListComponent);

const components = [
    ListComponent,
    ListItemDirective
];
let ListModule = class ListModule {
};
ListModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            PaginationModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], ListModule);

/**
 * Generated bundle index. Do not edit.
 */

export { ListComponent, ListModule, ListItemDirective as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
