import { __assign, __decorate, __metadata, __extends, __spread, __param } from 'tslib';
import { Injector, ElementRef, HostBinding, HostListener, Input, Directive, ChangeDetectorRef, NgZone, Attribute, ContentChild, TemplateRef, ContentChildren, ViewChild, ViewChildren, QueryList, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, styler, APPLY_STYLES_TYPE, extractDataSourceName, NAVIGATION_TYPE, getOrderedDataset, groupData, configureDnD, DEBOUNCE_TIMES, handleHeaderClick, toggleAllHeaders, ToDatePipe, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { PaginationComponent, PaginationModule } from '@wm/components/data/pagination';
import { isMobileApp, $watch, App, noop, getClonedObject, $appDigest, isDefined, $invokeWatchers, DataSource, isDataSourceEqual, isObject, switchClass, setListClass, isNumber, AppDefaults, StatePersistence } from '@wm/core';
import { ButtonComponent } from '@wm/components/input';
import { Subject } from 'rxjs';
import { SwipeAnimation } from '@swipey';

var registerProps = function () {
    register('wm-list', new Map([
        ['boundarylinks', __assign({ value: false }, PROP_BOOLEAN)],
        ['class', PROP_STRING],
        ['collapsible', PROP_BOOLEAN],
        ['dateformat', PROP_STRING],
        ['dataset', PROP_ANY],
        ['datasource', PROP_ANY],
        ['directionlinks', __assign({ value: true }, PROP_BOOLEAN)],
        ['disableitem', PROP_BOOLEAN],
        ['enablereorder', PROP_BOOLEAN],
        ['forceellipses', __assign({ value: true }, PROP_BOOLEAN)],
        ['groupby', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['itemclass', __assign({ value: '' }, PROP_STRING)],
        ['itemsperrow', PROP_STRING],
        ['listclass', __assign({ value: 'list-group' }, PROP_STRING)],
        ['multiselect', PROP_BOOLEAN],
        ['loadingdatamsg', __assign({ value: 'Loading...' }, PROP_STRING)],
        ['loadingicon', __assign({ value: 'fa fa-circle-o-notch' }, PROP_STRING)],
        ['match', PROP_STRING],
        ['maxsize', __assign({ value: 5 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['navigation', PROP_STRING],
        ['navigationalign', __assign({ value: 'left' }, PROP_STRING)],
        ['nodatamessage', __assign({ value: 'No data found' }, PROP_STRING)],
        ['ondemandmessage', __assign({ value: 'Load More' }, PROP_STRING)],
        ['orderby', PROP_STRING],
        ['paginationclass', PROP_STRING],
        ['pagesize', PROP_NUMBER],
        ['pulltorefresh', __assign({ value: isMobileApp() }, PROP_BOOLEAN)],
        ['selectfirstitem', PROP_BOOLEAN],
        ['selectionlimit', PROP_NUMBER],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['showcount', PROP_BOOLEAN],
        ['showrecordcount', PROP_BOOLEAN],
        ['statehandler', __assign({ value: 'none' }, PROP_STRING)],
        ['subheading', PROP_STRING],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING]
    ]));
};

var ListItemDirective = /** @class */ (function () {
    function ListItemDirective(inj, elRef, app) {
        this.inj = inj;
        this.app = app;
        this.destroy = new Subject();
        this.destroy$ = this.destroy.asObservable();
        this.itemClass = '';
        this._currentItemWidgets = {};
        this.isActive = false;
        this.disableItem = false;
        this.nativeElement = elRef.nativeElement;
        this.listComponent = inj.view.component;
        this.context = inj.view.context;
        this.itemClassWatcher(this.listComponent);
        this.disableItemWatcher(this.listComponent);
        $(this.nativeElement).data('listItemContext', this);
    }
    ListItemDirective.prototype.onFocus = function () {
        // maintains which element is focused/selected most recently.
        this.listComponent.lastSelectedItem = this;
    };
    Object.defineProperty(ListItemDirective.prototype, "$index", {
        get: function () {
            return this.context.index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemDirective.prototype, "$even", {
        get: function () {
            return this.context.even;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemDirective.prototype, "$odd", {
        get: function () {
            return this.context.odd;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemDirective.prototype, "$first", {
        get: function () {
            return this.context.first;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemDirective.prototype, "$last", {
        get: function () {
            return this.context.last;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemDirective.prototype, "currentItemWidgets", {
        get: function () {
            var componentElements = Array.from(this.nativeElement.querySelectorAll('[widget-id]'));
            return Object.assign(this._currentItemWidgets, componentElements.reduce(function (result, comp) {
                result[comp.widget.name] = comp.widget;
                return result;
            }, {}));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemDirective.prototype, "wmListItem", {
        set: function (val) {
            this.item = val;
        },
        enumerable: true,
        configurable: true
    });
    ListItemDirective.prototype.registerWatch = function (expression, callback) {
        // Removing ngFor context as the same properties are availble on listitem scope.
        // passing viewparent context for accessing varibales and widgets.
        this.destroy$.subscribe($watch(expression, this.listComponent.viewParent, this, callback));
    };
    ListItemDirective.prototype.itemClassWatcher = function (listComponent) {
        var _this = this;
        if (listComponent.binditemclass) {
            this.registerWatch(listComponent.binditemclass, function (nv) { return _this.itemClass = nv || ''; });
        }
        else {
            this.itemClass = listComponent.itemclass;
        }
    };
    ListItemDirective.prototype.disableItemWatcher = function ($list) {
        var _this = this;
        if ($list.binddisableitem) {
            this.registerWatch($list.binddisableitem, function (nv) { return _this.disableItem = nv || false; });
        }
        else {
            this.disableItem = $list.disableitem || false;
        }
    };
    ListItemDirective.prototype.setUpCUDHandlers = function () {
        var _this = this;
        var $editItem = this.nativeElement.querySelector('.edit-list-item');
        var $deleteItem = this.nativeElement.querySelector('.delete-list-item');
        if ($editItem) {
            // Triggered on click of edit action
            $editItem.addEventListener('click', function (evt) {
                _this.listComponent.update();
            });
        }
        if ($deleteItem) {
            // Triggered on click of delete action
            $deleteItem.addEventListener('click', function (evt) {
                _this.listComponent.delete();
            });
        }
    };
    ListItemDirective.prototype.ngOnInit = function () {
        var _this = this;
        if (this.listComponent.mouseEnterCB) {
            this.nativeElement.addEventListener('mouseenter', function ($event) {
                _this.listComponent.invokeEventCallback('mouseenter', { widget: _this, $event: $event });
            });
        }
        if (this.listComponent.mouseLeaveCB) {
            this.nativeElement.addEventListener('mouseleave', function ($event) {
                _this.listComponent.invokeEventCallback('mouseleave', { widget: _this, $event: $event });
            });
        }
        // adding item attribute on every list item
        $(this.nativeElement).attr('listitemindex', this.$index);
    };
    ListItemDirective.prototype.ngAfterViewInit = function () {
        this.setUpCUDHandlers();
    };
    ListItemDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: ElementRef },
        { type: App }
    ]; };
    __decorate([
        HostBinding('class.active'),
        __metadata("design:type", Object)
    ], ListItemDirective.prototype, "isActive", void 0);
    __decorate([
        HostBinding('class.disable-item'),
        __metadata("design:type", Object)
    ], ListItemDirective.prototype, "disableItem", void 0);
    __decorate([
        HostListener('focus'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], ListItemDirective.prototype, "onFocus", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], ListItemDirective.prototype, "wmListItem", null);
    ListItemDirective = __decorate([
        Directive({
            selector: '[wmListItem]',
            exportAs: 'listItemRef'
        }),
        __metadata("design:paramtypes", [Injector, ElementRef, App])
    ], ListItemDirective);
    return ListItemDirective;
}());

var ListAnimator = /** @class */ (function (_super) {
    __extends(ListAnimator, _super);
    function ListAnimator(list) {
        var _this = _super.call(this) || this;
        _this.list = list;
        _this.$el = $(_this.list.getNativeElement()).find('ul.app-livelist-container').first();
        _this.leftChildrenCount = _this.$el.find('>.app-list-item-left-action-panel > button:visible').length;
        _this.rightChildrenCount = _this.$el.find('>.app-list-item-right-action-panel > button:visible').length;
        // when there are no children in both the templates then do not apply swipeAnimation;
        if (!_this.leftChildrenCount && !_this.rightChildrenCount) {
            return _this;
        }
        // initialise swipe animation on the list component.
        _this.init(_this.$el);
        // retrieves all the button components which are placed outside the listTemplate.
        _this.$btnSubscription = _this.list.btnComponents.changes.subscribe(function (items) { return _this.actionItems = items; });
        return _this;
    }
    // This method sets the css for left or right action panels based on the template. Appends the actionTemplate before li.
    ListAnimator.prototype.createActionPanel = function (li, actionPanelTemplate) {
        actionPanelTemplate.css({
            width: li.outerWidth() + 'px',
            height: li.outerHeight() + 'px',
            marginBottom: -1 * li.outerHeight() + 'px',
            float: 'left',
            padding: 0
        });
        return actionPanelTemplate.insertBefore(li);
    };
    // Returns the total width occupied by all the children inside the element
    ListAnimator.prototype.computeTotalChildrenWidth = function ($ele) {
        return _.reduce($ele.children(), function (totalWidth, el) {
            return totalWidth + $(el).outerWidth();
        }, 0);
    };
    // Returns amount of transition to be applied on element when swiped left or right
    ListAnimator.prototype.computeTransitionProportions = function ($ele) {
        var totalWidth = this.computeTotalChildrenWidth($ele);
        var reverse = this.position === 'right';
        var d = 0;
        return _.map($ele.children(), function (e) {
            var f = (totalWidth - d) / totalWidth;
            d += $(e).outerWidth();
            return reverse ? f : (d / totalWidth);
        });
    };
    // Resets the transform applied on the element.
    ListAnimator.prototype.resetElement = function (el) {
        if (el) {
            el.css({
                transform: 'none',
                transition: 'none'
            });
        }
    };
    ListAnimator.prototype.resetState = function () {
        this.resetElement(this.li);
        this.resetElement(this.actionPanel);
        if (this.actionPanel) {
            this.actionPanel = null;
        }
    };
    // Returns the target button (child element) inside the left and right actionPanels.
    ListAnimator.prototype.getChildActionElement = function (actionTemplate) {
        if (actionTemplate.children().length) {
            if (this.position === 'left') {
                return actionTemplate.children().first();
            }
            return actionTemplate.children().last();
        }
    };
    // create the actionPanels and set the background-color for remaining panel as that of first child element
    // calculates the children's width and its transition proportionates.
    ListAnimator.prototype.initActionPanel = function (actionTemplate) {
        this.actionPanel = this.createActionPanel(this.li, actionTemplate);
        this.actionPanel.css({
            backgroundColor: this.getChildActionElement(this.actionPanel).css('background-color')
        });
        this.limit = this.computeTotalChildrenWidth(this.actionPanel);
        this.transitionProportions = this.computeTransitionProportions(this.actionPanel);
    };
    ListAnimator.prototype.bounds = function (e, $d) {
        var target = $(e.target).closest('li');
        // default bounds when action template markup is not available.
        var bounds = {
            strictUpper: true,
            strictLower: true,
            lower: 0,
            center: 0,
            upper: 0
        };
        // apply swipe animation only on list items having "app-list-item" class.
        if (!target.hasClass('app-list-item')) {
            return bounds;
        }
        if (!this.li || this.li[0] !== target[0]) {
            var selector = $d > 0 ? '.app-list-item-left-action-panel' : '.app-list-item-right-action-panel';
            var actionTemplate = this.$el.find('>' + selector);
            // when groupby is set select the action panel from the list group items.
            if (!actionTemplate.length && this.list.groupby) {
                selector = 'li > ul.list-group >' + selector;
                actionTemplate = this.$el.find('>' + selector);
            }
            // check for children visiblity. If children are visible then initiate the action panel.
            if (!actionTemplate.length || !actionTemplate.find('button:visible').length) {
                return bounds;
            }
            this.resetState();
            this.li = target;
            this.position = actionTemplate.attr('position');
            this.initActionPanel(actionTemplate);
            if ($d > 0) {
                // bounds while swiping from right to left to open left action panel. It can be moved upto limit value (Upper bound).
                bounds = {
                    strictUpper: false,
                    lower: 0,
                    center: 0,
                    upper: this.limit
                };
            }
            else {
                // bounds while swiping from left to right to open right action panel. It can be moved in reverse direction with -limit value (lower bound).
                bounds = {
                    strictLower: false,
                    lower: -this.limit,
                    center: 0,
                    upper: 0
                };
            }
        }
        else if (this.position === 'left') {
            // when left action panel is visible (i.e. center at limit value) then this can be moved by distance (limit) in reverse direction to close the view.
            bounds = {
                strictUpper: false,
                lower: -this.limit,
                center: this.limit
            };
        }
        else if (this.position === 'right') {
            // when right action panel is visible (i.e. center at -limit value) then this can be moved by distance (limit) to close the view.
            bounds = {
                center: -this.limit,
                upper: this.limit,
                strictLower: false
            };
        }
        return bounds;
    };
    ListAnimator.prototype.context = function () {
        var _this = this;
        return {
            computeActionTransition: function (index, $d) {
                var sign = $d > 0 ? 1 : -1;
                if (sign * $d > _this.limit) {
                    // once the distance swiped is beyond the limit, then calculate the proportionate distance moved after the limit value.
                    return ($d - sign * _this.limit) + (_this.transitionProportions[index] * sign * _this.limit);
                }
                return _this.transitionProportions[index] * $d;
            }
        };
    };
    ListAnimator.prototype.animation = function () {
        var _this = this;
        return [{
                target: function () { return _this.li; },
                css: {
                    transform: 'translate3d(${{$D + $d}}px, 0, 0)'
                }
            }, {
                target: function () { return (_this.actionPanel && _this.actionPanel.children()); },
                css: {
                    transform: 'translate3d(${{computeActionTransition($i, $D + $d)}}px, 0, 0)'
                }
            }];
    };
    // Triggers full swipe event on the target element.
    ListAnimator.prototype.invokeFullSwipeEvt = function ($event) {
        var target, actions, index;
        // Check if button are visible or not, invoke the tap event of the last button which is visible.
        if (this.position === 'left') {
            actions = this.actionItems.filter(function (btn) {
                return btn.getAttr('swipe-position') === 'left' && btn.$element.is(':visible');
            });
            index = 0;
        }
        else {
            actions = this.actionItems.filter(function (btn) {
                return btn.getAttr('swipe-position') === 'right' && btn.$element.is(':visible');
            });
            index = actions.length - 1;
        }
        target = actions[index];
        if (target && target.hasEventCallback('tap')) {
            target.invokeEventCallback('tap', { $event: $event });
        }
        this.resetState();
        this.li = null;
    };
    // Called when swipeEnd is triggered. d contains the total distance covered by the element until touchEnd.
    ListAnimator.prototype.onAnimation = function ($event, d) {
        // set the selecteditem on the list component on swipe.
        this.list.triggerListItemSelection(this.li, $event);
        if (this.actionPanel && this.actionPanel.attr('enablefullswipe') === 'true') {
            var sign = d > 0 ? 1 : -1;
            var $el = this.getChildActionElement(this.actionPanel);
            if ($el) {
                var index = this.position === 'right' ? this.rightChildrenCount - 1 : 0;
                // proportionate amount of distance covered by the target element.
                var distPercentage = this.transitionProportions[index] * sign * d * 100 / (this.li.outerWidth() - this.limit + $el.width());
                // If distance travelled by the target button element is more than 50% of the list item width then invoke the fullswipe.
                if (distPercentage > 50) {
                    // invoke fullswipe event
                    this.invokeFullSwipeEvt($event);
                }
            }
        }
    };
    ListAnimator.prototype.onLower = function () {
        if (this.position === 'left') {
            this.resetState();
            this.li = null;
        }
    };
    ListAnimator.prototype.onUpper = function () {
        if (this.position === 'right') {
            this.resetState();
            this.li = null;
        }
    };
    ListAnimator.prototype.threshold = function () {
        return 10;
    };
    return ListAnimator;
}(SwipeAnimation));

var DEFAULT_CLS = 'app-livelist app-panel';
var WIDGET_CONFIG = { widgetType: 'wm-list', hostClass: DEFAULT_CLS };
var unsupportedStatePersistenceTypes = ['On-Demand', 'Scroll'];
var ListComponent = /** @class */ (function (_super) {
    __extends(ListComponent, _super);
    function ListComponent(inj, cdRef, datePipe, app, appDefaults, ngZone, binditemclass, binddisableitem, binddataset, binddatasource, mouseEnterCB, mouseLeaveCB, statePersistence) {
        var _this = this;
        var resolveFn = noop;
        var propsInitPromise = new Promise(function (res) { return resolveFn = res; });
        _this = _super.call(this, inj, WIDGET_CONFIG, propsInitPromise) || this;
        _this.propsInitPromise = propsInitPromise;
        _this.promiseResolverFn = resolveFn;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SHELL);
        _this.cdRef = cdRef;
        _this.ngZone = ngZone;
        _this.datePipe = datePipe;
        _this.statePersistence = statePersistence;
        _this._pageLoad = true;
        _this.binditemclass = binditemclass;
        _this.binddisableitem = binddisableitem;
        _this.binddataset = binddataset;
        _this.mouseEnterCB = mouseEnterCB;
        _this.mouseLeaveCB = mouseLeaveCB;
        _this.binddatasource = binddatasource;
        _this.app = app;
        _this.appDefaults = appDefaults;
        _this.variableInflight = false;
        _this.noDataFound = !binddataset;
        // Updates pagination, filter, sort etc options for service and crud variables
        _this._listenerDestroyers = [
            _this.app.subscribe('check-state-persistence-options', function (options) {
                var dataSourceName = _.get(_this.datasource, 'name');
                // in Prefabs, this.datasource is not resolved at the time of variable invocation, so additional check is required.
                if (!dataSourceName) {
                    dataSourceName = extractDataSourceName(_this.binddatasource);
                }
                if (_.get(options, 'variable.name') !== dataSourceName) {
                    return;
                }
                _this.handleStateParams(options);
            }),
            // Show loading status based on the variable life cycle
            _this.app.subscribe('toggle-variable-state', _this.handleLoading.bind(_this)),
            _this.app.subscribe('setup-cud-listener', function (param) {
                if (_this.nativeElement.getAttribute('name') !== param) {
                    return;
                }
                _this._isDependent = true;
            })
        ];
        return _this;
    }
    ListComponent_1 = ListComponent;
    Object.defineProperty(ListComponent.prototype, "selecteditem", {
        get: function () {
            if (this.multiselect) {
                return getClonedObject(this._items);
            }
            if (_.isEmpty(this._items)) {
                return {};
            }
            return getClonedObject(this._items[0]);
        },
        set: function (items) {
            var _this = this;
            this._items.length = 0;
            this.deselectListItems();
            if (_.isArray(items)) {
                items.forEach(function (item) { return _this.selectItem(item); });
            }
            else {
                this.selectItem(items);
            }
            $appDigest();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns list of widgets present on list item by considering name and index of the widget.
     * If we did'nt pass index, it returns array of all the widgets which are matching to widget name
     * @param widgteName: Name of the widget
     * @param index: Index of the widget
     */
    ListComponent.prototype.getWidgets = function (widgteName, index) {
        var $target;
        var retVal = [];
        if (!widgteName) {
            return;
        }
        if (!isDefined(index)) {
            _.forEach(this.listItems.toArray(), function (el) {
                $target = _.get(el.currentItemWidgets, widgteName);
                if ($target) {
                    retVal.push($target);
                }
            });
            return retVal;
        }
        index = +index || 0;
        $target = _.get(this.listItems.toArray(), index);
        if ($target) {
            return [_.get($target.currentItemWidgets, widgteName)];
        }
    };
    // returns listitem reference by index value. This refers to the same method getListItemByIndex.
    ListComponent.prototype.getItem = function (index) {
        return this.getListItemByIndex(index);
    };
    // return index of listItem(listItemDirective). This refers to the same method getListItemIndex.
    ListComponent.prototype.getIndex = function (item) {
        return this.getListItemIndex(item);
    };
    ListComponent.prototype.getConfiguredState = function () {
        var mode = this.statePersistence.computeMode(this.statehandler);
        return mode && mode.toLowerCase();
    };
    ListComponent.prototype.handleStateParams = function (options) {
        if (this._pageLoad && this.getConfiguredState() !== 'none') {
            this._pageLoad = false;
            var widgetState = this.statePersistence.getWidgetState(this);
            if (_.get(widgetState, 'pagination')) {
                options.options = options.options || {};
                options.options.page = widgetState.pagination;
            }
            if (_.get(widgetState, 'selectedItem')) {
                this._selectedItemsExist = true;
            }
        }
    };
    ListComponent.prototype.triggerWMEvent = function (eventName, item) {
        $invokeWatchers(true);
        // If we have multiselect for the livelist(List with form template), in run mode deleting a record is getting failed. Becuase the selecteditem will be array of objects. So consider the last object.
        var row = this.multiselect ? _.last(this.selecteditem) : this.selecteditem;
        if (item) {
            row = item;
        }
        this.app.notify('wm-event', { eventName: eventName, widgetName: this.name, row: row });
    };
    ListComponent.prototype.create = function () {
        if (this._isDependent) {
            this.triggerWMEvent('insert');
        }
    };
    ListComponent.prototype.editRow = function (item) {
        if (this._isDependent) {
            var listItem = void 0;
            if (item) {
                listItem = this.getItemRefByIndexOrModel(item).item;
            }
            this.triggerWMEvent('update', listItem);
        }
    };
    ListComponent.prototype.update = function (item) {
        this.editRow(item);
    };
    ListComponent.prototype.deleteRow = function (item) {
        if (this._isDependent) {
            var listItem = void 0;
            if (item) {
                listItem = this.getItemRefByIndexOrModel(item).item;
            }
            this.triggerWMEvent('delete', listItem);
        }
    };
    ListComponent.prototype.delete = function (item) {
        this.deleteRow(item);
    };
    ListComponent.prototype.handleLoading = function (data) {
        var _this = this;
        var dataSource = this.datasource;
        if (dataSource && dataSource.execute(DataSource.Operation.IS_API_AWARE) && isDataSourceEqual(data.variable, dataSource)) {
            this.ngZone.run(function () {
                _this.handleStateParams(data);
                _this.variableInflight = data.active;
            });
        }
    };
    ListComponent.prototype.resetNavigation = function () {
        this.showNavigation = false;
        this.navControls = undefined;
        this.infScroll = false;
        this.onDemandLoad = false;
    };
    ListComponent.prototype.enableBasicNavigation = function () {
        this.navControls = NAVIGATION_TYPE.BASIC;
        this.showNavigation = true;
    };
    ListComponent.prototype.enableInlineNavigation = function () {
        this.navControls = NAVIGATION_TYPE.INLINE;
    };
    ListComponent.prototype.enableClassicNavigation = function () {
        this.navControls = NAVIGATION_TYPE.CLASSIC;
        this.showNavigation = true;
    };
    ListComponent.prototype.enablePagerNavigation = function () {
        this.navControls = NAVIGATION_TYPE.PAGER;
        this.showNavigation = true;
    };
    ListComponent.prototype.setNavigationTypeNone = function () {
        this.navControls = NAVIGATION_TYPE.NONE;
        this.showNavigation = false;
    };
    ListComponent.prototype.enableInfiniteScroll = function () {
        this.infScroll = true;
    };
    ListComponent.prototype.enableOnDemandLoad = function () {
        this.onDemandLoad = true;
        this.showNavigation = true;
    };
    /**
     * Sets Navigation type for the list.
     * @param type
     */
    ListComponent.prototype.onNavigationTypeChange = function (type) {
        this.resetNavigation();
        switch (type) {
            case NAVIGATION_TYPE.BASIC:
                this.enableBasicNavigation();
                break;
            case NAVIGATION_TYPE.INLINE:
                this.enableInlineNavigation();
                break;
            case NAVIGATION_TYPE.ADVANCED:
            case NAVIGATION_TYPE.CLASSIC:
                this.enableClassicNavigation();
                break;
            case NAVIGATION_TYPE.PAGER:
                this.enablePagerNavigation();
                break;
            case NAVIGATION_TYPE.NONE:
                this.setNavigationTypeNone();
                break;
            case NAVIGATION_TYPE.SCROLL:
                this.enableInfiniteScroll();
                break;
            case NAVIGATION_TYPE.ONDEMAND:
                this.enableOnDemandLoad();
                break;
        }
    };
    ListComponent.prototype.fetchNextDatasetOnScroll = function () {
        this.dataNavigator.navigatePage('next');
    };
    ListComponent.prototype.setIscrollHandlers = function (el) {
        var _this = this;
        var lastScrollTop = 0;
        var wrapper = _.get(el.iscroll, 'wrapper');
        var self = el.iscroll;
        el.iscroll.on('scrollEnd', function () {
            var clientHeight = wrapper.clientHeight, totalHeight = wrapper.scrollHeight, scrollTop = Math.abs(el.iscroll.y);
            if ((lastScrollTop < scrollTop) && (totalHeight * 0.9 < scrollTop + clientHeight)) {
                _this.debouncedFetchNextDatasetOnScroll();
                if (self.indicatorRefresh) {
                    self.indicatorRefresh();
                }
            }
            lastScrollTop = scrollTop;
        });
    };
    // Applying iscroll event to invoke the next calls for infinte scroll.
    ListComponent.prototype.bindIScrollEvt = function () {
        var _this = this;
        var $scrollParent = this.$element.closest('[wmsmoothscroll="true"]');
        var iScroll = _.get($scrollParent[0], 'iscroll');
        // when iscroll is not initialised the notify the smoothscroll and subscribe to the iscroll update
        if (!iScroll) {
            var iScrollSubscription_1 = this.app.subscribe('iscroll-update', function (_el) {
                if (!_.isEmpty(_el) && _el.isSameNode($scrollParent[0])) {
                    _this.setIscrollHandlers($scrollParent[0]);
                    iScrollSubscription_1();
                }
            });
            this.app.notify('no-iscroll', $scrollParent[0]);
            return;
        }
        this.setIscrollHandlers($scrollParent[0]);
    };
    ListComponent.prototype.bindScrollEvt = function () {
        var _this = this;
        var $el = this.$element;
        var $ul = $el.find('> ul');
        var $firstChild = $ul.children().first();
        var self = this;
        var $scrollParent;
        var scrollNode;
        var lastScrollTop = 0;
        if (!$firstChild.length) {
            return;
        }
        $scrollParent = $firstChild.scrollParent(false);
        if ($scrollParent[0] === document) {
            scrollNode = document.body;
        }
        else {
            scrollNode = $scrollParent[0];
        }
        // has scroll
        if (scrollNode.scrollHeight > scrollNode.clientHeight) {
            $scrollParent
                .each(function (index, node) {
                // scrollTop property is 0 or undefined for body in IE, safari.
                lastScrollTop = node === document ? (node.body.scrollTop || $(window).scrollTop()) : node.scrollTop;
            })
                .off('scroll.scroll_evt')
                .on('scroll.scroll_evt', function (evt) {
                var target = evt.target;
                var clientHeight;
                var totalHeight;
                var scrollTop;
                // scrollingElement is undefined for IE, safari. use body as target Element
                target = target === document ? (target.scrollingElement || document.body) : target;
                clientHeight = target.clientHeight;
                totalHeight = target.scrollHeight;
                scrollTop = target === document.body ? $(window).scrollTop() : target.scrollTop;
                if ((lastScrollTop < scrollTop) && (totalHeight * 0.9 < scrollTop + clientHeight)) {
                    $(this).off('scroll.scroll_evt');
                    self.debouncedFetchNextDatasetOnScroll();
                }
                lastScrollTop = scrollTop;
            });
            $ul.off('wheel.scroll_evt');
        }
        else {
            // if there is no scrollable element register wheel event on ul element
            $ul.on('wheel.scroll_evt', function (e) {
                if (e.originalEvent.deltaY > 0) {
                    $ul.off('wheel.scroll_evt');
                    _this.debouncedFetchNextDatasetOnScroll();
                }
            });
        }
    };
    /**
     * Update fieldDefs property, fieldDefs is the model of the List Component.
     * fieldDefs is an Array type.
     * @param newVal
     */
    ListComponent.prototype.updateFieldDefs = function (newVal) {
        if (this.infScroll || this.onDemandLoad) {
            if (!isDefined(this.fieldDefs) || this.dataNavigator.isFirstPage()) {
                this.fieldDefs = [];
                this.currentPage = 1;
            }
            else if (this.fieldDefs.length / this.pagesize <= this.dataNavigator.pageCount) {
                var itemsLength = void 0, itemsToPush = [];
                // we push the newVal only when dn.currentPage gets incremented because that is when new items gets added to newVal
                if (this.fieldDefs.length === this.currentPage * this.pagesize && (this.currentPage + 1) === this.dataNavigator.dn.currentPage) {
                    itemsToPush = newVal;
                    this.currentPage++;
                }
                else if (this.fieldDefs.length < this.currentPage * this.pagesize) {
                    if ((this.fieldDefs.length === (this.currentPage - 1) * this.pagesize) && ((this.currentPage - 1) === this.dataNavigator.dn.currentPage)) {
                        // if dn.currentPage is not incremented still only old newVal is present hence we push empty array
                        newVal = [];
                    }
                    else if (this.dataNavigator.dataSize < this.currentPage * this.pagesize) {
                        // if number of elements added to datanavigator is less than  product of currentpage and pagesize we only add elements extra elements added
                        itemsLength = this.dataNavigator.dataSize - this.fieldDefs.length;
                    }
                    else {
                        // if number of elements added to datanavigator is greater than  product of currentpage and pagesize we add elements the extra elements in newVal
                        itemsLength = this.currentPage * this.pagesize - this.fieldDefs.length;
                        this.currentPage++;
                    }
                    var startIndex = newVal.length - itemsLength;
                    itemsToPush = newVal.slice(startIndex);
                }
                else if (this.fieldDefs.length === this.currentPage * this.pagesize && this.currentPage === this.dataNavigator.dn.currentPage) {
                    // if dn.currentPage is not incremented still only old newVal is present hence we push empty array
                    itemsToPush = [];
                }
                newVal = itemsToPush;
            }
            this.fieldDefs = __spread(this.fieldDefs, newVal);
        }
        else {
            this.fieldDefs = newVal;
        }
        if (this.orderby) {
            this.fieldDefs = getOrderedDataset(this.fieldDefs, this.orderby);
        }
        if (this.groupby) {
            this.groupedData = groupData(this, this.fieldDefs, this.groupby, this.match, this.orderby, this.dateformat, this.datePipe, undefined, this.appDefaults);
        }
        if (!this.fieldDefs.length) {
            this.noDataFound = true;
            this.selecteditem = undefined;
        }
        $appDigest();
        this.listItems.setDirty();
    };
    ListComponent.prototype.onDataChange = function (newVal) {
        // Check for newVal is not empty
        if (!_.isEmpty(newVal)) {
            this.noDataFound = false;
            this.isDataChanged = true;
            if (this.datasource && this.datasource.execute(DataSource.Operation.IS_API_AWARE)) {
                // clone the the data in case of live and service variables to prevent the two-way binding for these variables.
                newVal = _.cloneDeep(newVal);
            }
            if (isObject(newVal) && !_.isArray(newVal)) {
                newVal = _.isEmpty(newVal) ? [] : [newVal];
            }
            if (_.isString(newVal)) {
                newVal = newVal.split(',');
            }
            // if the page number is greater than 1 on initial load then we render the first page.
            if (this.datasource && this.datasource.owner === 'App' && (this.infScroll || this.onDemandLoad) && !this.currentPage && this.datasource.execute(DataSource.Operation.GET_PAGING_OPTIONS).number > 0) {
                newVal = this.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                    'page': 1
                });
            }
            if (_.isArray(newVal)) {
                if (newVal.length) {
                    this.invokeEventCallback('beforedatarender', { $data: newVal });
                }
                this.updateFieldDefs(newVal);
            }
        }
        else {
            this.updateFieldDefs([]);
        }
    };
    // Updates the dataSource when pagination is enabled for the Component.
    ListComponent.prototype.setupDataSource = function () {
        var _this = this;
        var dataNavigator = this.dataNavigator;
        dataNavigator.options = {
            maxResults: this.pagesize || 20
        };
        this.dataNavigatorWatched = true;
        if (this.navigatorResultWatch) {
            this.navigatorResultWatch.unsubscribe();
        }
        /*Register a watch on the "result" property of the "dataNavigator" so that the paginated data is displayed in the live-list.*/
        this.navigatorResultWatch = dataNavigator.resultEmitter.subscribe(function (newVal) {
            _this.onDataChange(newVal);
        }, true);
        /*De-register the watch if it is exists */
        if (this.navigatorMaxResultWatch) {
            this.navigatorMaxResultWatch.unsubscribe();
        }
        /*Register a watch on the "maxResults" property of the "dataNavigator" so that the "pageSize" is displayed in the live-list.*/
        this.navigatorMaxResultWatch = dataNavigator.maxResultsEmitter.subscribe(function (val) {
            _this.pagesize = val;
        });
        dataNavigator.maxResults = this.pagesize || 20;
        this.removePropertyBinding('dataset');
        // when list having "datasetboundexpr" attr indicates that list is bound to item context
        // when dataset is bound to "item.FIELD" then item context is passed as the datasource.
        var datasetBoundExpr = this.getAttr('datasetboundexpr');
        this.dataNavigator.setBindDataSet(this.binddataset, this.viewParent, datasetBoundExpr ? this.context : this.datasource, this.dataset, this.binddatasource, datasetBoundExpr, this.statehandler);
    };
    ListComponent.prototype.onDataSetChange = function (newVal) {
        if (_.get(this.datasource, 'category') === 'wm.Variable' && this.getConfiguredState() !== 'none' && this._pageLoad) {
            var widgetState = this.statePersistence.getWidgetState(this);
            this._pageLoad = false;
            if (_.get(widgetState, 'pagination')) {
                this.dataNavigator.pageChanged({ page: widgetState.pagination }, true);
            }
            if (_.get(widgetState, 'selectedItem')) {
                this._selectedItemsExist = true;
            }
        }
        if (!this.dataNavigatorWatched) {
            if (this.navigation && this.navigation !== NAVIGATION_TYPE.NONE) {
                this.setupDataSource();
            }
            else {
                this.onDataChange(newVal);
            }
        }
        else if (this.navigation && this.navigation !== NAVIGATION_TYPE.NONE) {
            // If navigation is already setup and datasource is changed, update the datasource on navigation
            this.dataNavigator.setDataSource(this.datasource);
        }
    };
    // All the ListItem's Active state is set to false.
    ListComponent.prototype.deselectListItems = function () {
        this.listItems.forEach(function (item) { return item.isActive = false; });
    };
    // Deselect all the ListItems and clear the selecteditem(InOutBound Property model)
    ListComponent.prototype.clearSelectedItems = function () {
        this.deselectListItems();
        this._items.length = 0;
        $appDigest();
    };
    /**
     * return the ListItemDirective instance by checking the equality of the model.
     * @param listModel: model to be searched for
     * @returns ListItem if the model is matched else return null.
     */
    ListComponent.prototype.getListItemByModel = function (listModel) {
        var _this = this;
        return this.listItems.find(function (listItem) {
            var itemObj = listItem.item;
            if (_this.groupby && !_.has(listModel, '_groupIndex')) {
                // If groupby is enabled, item contains _groupIndex property which should be excluded while comparing model.
                itemObj = _.clone(itemObj);
                delete itemObj._groupIndex;
            }
            if (_.isEqual(itemObj, listModel)) {
                return true;
            }
        }) || null;
    };
    ListComponent.prototype.updateSelectedItemsWidgets = function (statePersistenceTriggered) {
        var _this = this;
        var obj = {}, widgetState;
        var pageNum = _.get(this.dataNavigator, 'dn.currentPage') || 1;
        if (this.getConfiguredState() !== 'none') {
            // remove previously configured selected items for current page and construct new ones later below.
            widgetState = this.statePersistence.getWidgetState(this) || {};
            if (_.get(widgetState, 'selectedItem')) {
                // when multiselect is on and an item is selected without pressing CTRL, previously selected items in state should be empty.
                if (this.multiselect && this.selecteditem.length === 1) {
                    widgetState.selectedItem = [];
                }
                else {
                    _.remove(widgetState.selectedItem, function (selectedItem) {
                        return selectedItem.page === pageNum;
                    });
                }
            }
        }
        if (this.multiselect) {
            this.selectedItemWidgets.length = 0;
        }
        this.listItems.forEach(function (item, index) {
            if (item.isActive) {
                if (_this.multiselect) {
                    _this.selectedItemWidgets.push(item.currentItemWidgets);
                }
                else {
                    _this.selectedItemWidgets = item.currentItemWidgets;
                }
                obj = { page: pageNum, index: index };
                if (_.get(widgetState, 'selectedItem') && _this.multiselect) {
                    if (!_.some(widgetState.selectedItem, obj)) {
                        widgetState.selectedItem.push(obj);
                    }
                }
                else {
                    widgetState.selectedItem = [obj];
                }
            }
        });
        if (this.getConfiguredState() !== 'none' && !statePersistenceTriggered) {
            if (unsupportedStatePersistenceTypes.indexOf(this.navigation) < 0) {
                this.statePersistence.removeWidgetState(this, 'selectedItem');
                this.statePersistence.setWidgetState(this, { 'selectedItem': widgetState.selectedItem });
            }
            else {
                console.warn('Retain State handling on Widget ' + this.name + ' is not supported for current pagination type.');
            }
        }
    };
    /**
     * Selects the listItem and updates selecteditem property.
     * If the listItem is already a selected item then deselects the item.
     * @param {ListItemDirective} $listItem: Item to be selected of deselected.
     */
    ListComponent.prototype.toggleListItemSelection = function ($listItem, statePersistenceTriggered) {
        // item is not allowed to get selected if it is disabled.
        if ($listItem && !$listItem.disableItem) {
            var item = $listItem.item;
            if (this.groupby && _.has(item, '_groupIndex')) {
                // If groupby is enabled, item contains _groupIndex property which should be excluded from selecteditem.
                item = _.clone(item);
                delete item._groupIndex;
            }
            if ($listItem.isActive) {
                this._items = _.pullAllWith(this._items, [item], _.isEqual);
                $listItem.isActive = false;
            }
            else {
                // if multiselect is false, clear the selectItem list before adding an item to the selectItem list.
                if (!this.multiselect) {
                    this.clearSelectedItems();
                }
                this._items.push(item);
                this.invokeEventCallback('select', { widget: $listItem, $data: item });
                $listItem.isActive = true;
            }
            this.updateSelectedItemsWidgets(statePersistenceTriggered);
        }
    };
    /**
     * Method is Invoked when the model for the List Widget is changed.
     * @param {QueryList<ListItemDirective>} listItems
     */
    ListComponent.prototype.onListRender = function (listItems) {
        var _this = this;
        // Added render callback event. This method(onListRender) is calling multiple times so checking isDatachanged flag because this falg is changed whenever new data is rendered.
        if (this.isDataChanged) {
            // Whenever dataset is changed, trigger watchers to evaluate listitem bind expressions.
            $invokeWatchers(true);
            this.invokeEventCallback('render', { $data: this.fieldDefs });
        }
        if (this.isDataChanged && this.getConfiguredState() !== 'none' && listItems.length && this._selectedItemsExist) {
            var widgetState = this.statePersistence.getWidgetState(this);
            if (_.get(widgetState, 'selectedItem')) {
                this._selectedItemsExist = false;
                var selectedItemsLength = _.isArray(this.selecteditem) ? this.selecteditem.length : _.toNumber(!_.isEmpty(this.selecteditem));
                var currentPage_1 = _.get(this.dataNavigator, 'dn.currentPage') || 1;
                widgetState.pagination = widgetState.pagination || 1;
                // to prevent item selection from being triggered more than once
                if (selectedItemsLength !== widgetState.selectedItem.length && widgetState.pagination === currentPage_1) {
                    widgetState.selectedItem.forEach(function (item) {
                        if (item.page === currentPage_1) {
                            _this.selectItem(item.index, true);
                        }
                    });
                }
            }
        }
        var selectedItems = _.isArray(this.selecteditem) ? this.selecteditem : [this.selecteditem];
        this.firstSelectedItem = this.lastSelectedItem = null;
        // don't select first item if multi-select is enabled and at least item is already selected in the list.
        // don't select first item if state information has selected items
        if (listItems.length && this.selectfirstitem && !(this._items.length && this.multiselect) && this._selectedItemsExist !== false) {
            var $firstItem = listItems.first;
            if (!$firstItem.disableItem &&
                this.isDataChanged &&
                // "infinite scroll" or "load on demand" is enabled and at least one item is selected then dont alter the selected list items.
                !((this.infScroll || this.onDemandLoad) &&
                    this._items.length)) {
                this.clearSelectedItems();
                this.firstSelectedItem = this.lastSelectedItem = $firstItem;
                // selecting the first record
                this.selectItem(0);
            }
        }
        else {
            this.deselectListItems();
            selectedItems.forEach(function (selecteditem) {
                var listItem = _this.getListItemByModel(selecteditem);
                if (listItem) {
                    listItem.isActive = true;
                    _this.lastSelectedItem = listItem;
                    // focus the active element
                    listItem.nativeElement.focus();
                }
            });
        }
        if (this.fieldDefs.length && this.infScroll) {
            var smoothScrollEl = this.$element.closest('[wmsmoothscroll]');
            // if smoothscroll is set to false, then native scroll has to be applied
            // otherwise smoothscroll events will be binded.
            if (isMobileApp() && smoothScrollEl.length && smoothScrollEl.attr('wmsmoothscroll') === 'true') {
                this.bindIScrollEvt();
            }
            else {
                this.bindScrollEvt();
            }
        }
        this.isDataChanged = false;
    };
    ListComponent.prototype.triggerListItemSelection = function ($el, $event) {
        if ($el && $el[0]) {
            var listItemContext = $el.data('listItemContext');
            // Trigger click event only if the list item is from the corresponding list.
            if (listItemContext.listComponent === this) {
                this.onItemClick($event, listItemContext);
            }
        }
    };
    ListComponent.prototype.setupHandlers = function () {
        var _this = this;
        this.listItems.changes.subscribe(function (listItems) {
            _this.onListRender(listItems);
            _this.cdRef.detectChanges();
        });
        // handle click event in capturing phase.
        this.nativeElement.querySelector('ul.app-livelist-container').addEventListener('click', function ($event) {
            var target = $($event.target).closest('.app-list-item');
            // Recursively find the current list item
            while (target.get(0) && (target.closest('ul.app-livelist-container').get(0) !== $event.currentTarget)) {
                target = target.parent().closest('.app-list-item');
            }
            _this.triggerListItemSelection(target, $event);
        }, true);
    };
    // Triggers on drag start while reordering.
    ListComponent.prototype.onReorderStart = function (evt, ui) {
        ui.placeholder.height(ui.item.height());
        this.$ulEle.data('oldIndex', ui.item.index());
    };
    // Triggers after the sorting.
    ListComponent.prototype.onUpdate = function (evt, ui) {
        var data = this.fieldDefs;
        var newIndex = ui.item.index();
        var oldIndex = this.$ulEle.data('oldIndex');
        if (this.getConfiguredState() !== 'none') {
            this.statePersistence.removeWidgetState(this, 'selectedItem');
        }
        var minIndex = _.min([newIndex, oldIndex]);
        var maxIndex = _.max([newIndex, oldIndex]);
        var draggedItem = _.pullAt(data, oldIndex)[0];
        this.reorderProps.minIndex = _.min([minIndex, this.reorderProps.minIndex]);
        this.reorderProps.maxIndex = _.max([maxIndex, this.reorderProps.maxIndex]);
        data.splice(newIndex, 0, draggedItem);
        this.cdRef.markForCheck();
        this.cdRef.detectChanges();
        var $changedItem = {
            oldIndex: oldIndex,
            newIndex: newIndex,
            item: data[newIndex]
        };
        this.invokeEventCallback('reorder', { $event: evt, $data: data, $changedItem: $changedItem });
        this.$ulEle.removeData('oldIndex');
    };
    // configures reordering the list items.
    ListComponent.prototype.configureDnD = function () {
        var options = {
            appendTo: 'body',
        };
        var $el = $(this.nativeElement);
        this.$ulEle = $el.find('.app-livelist-container');
        configureDnD(this.$ulEle, options, this.onReorderStart.bind(this), this.onUpdate.bind(this));
        this.$ulEle.droppable({ 'accept': '.app-list-item' });
    };
    // returns true if the selection limit is reached.
    ListComponent.prototype.checkSelectionLimit = function (count) {
        return (!this.selectionlimit || count < this.selectionlimit);
    };
    // returns listitem reference by index value.
    ListComponent.prototype.getListItemByIndex = function (index) {
        return this.listItems.toArray()[index];
    };
    /**
     * return index of an (listItemDirective) in the listItem
     * @param {ListItemDirective} item
     * @returns {number}
     */
    ListComponent.prototype.getListItemIndex = function (item) {
        return this.listItems.toArray().indexOf(item);
    };
    // this method is called form other data widgets like table.
    ListComponent.prototype.execute = function (operation, options) {
        if ([DataSource.Operation.IS_API_AWARE, DataSource.Operation.IS_PAGEABLE, DataSource.Operation.SUPPORTS_SERVER_FILTER].includes(operation)) {
            return false;
        }
        return this.datasource.execute(operation, options);
    };
    ListComponent.prototype.handleKeyDown = function ($event, action) {
        $event.stopPropagation();
        var listItems = this.listItems;
        var presentIndex = this.getListItemIndex(this.lastSelectedItem);
        if (this.multiselect) {
            var firstIndex = this.getListItemIndex(this.firstSelectedItem);
            var selectCount = _.isArray(this.selecteditem) ? this.selecteditem.length : (_.isObject(this.selecteditem) ? 1 : 0);
            if (action === 'selectPrev') {
                if (presentIndex > 0) {
                    if ((presentIndex <= firstIndex) && this.checkSelectionLimit(selectCount)) {
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex - 1);
                        this.toggleListItemSelection(this.lastSelectedItem);
                    }
                    else if (presentIndex > firstIndex) {
                        this.toggleListItemSelection(this.getListItemByIndex(presentIndex));
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex - 1);
                    }
                    else {
                        this.invokeEventCallback('selectionlimitexceed', { $event: $event });
                    }
                }
            }
            else if (action === 'selectNext') {
                if (presentIndex < listItems.length - 1) {
                    if ((presentIndex >= firstIndex) && this.checkSelectionLimit(selectCount)) {
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex + 1);
                        this.toggleListItemSelection(this.lastSelectedItem);
                    }
                    else if (presentIndex < firstIndex) {
                        this.toggleListItemSelection(this.getListItemByIndex(presentIndex));
                        this.lastSelectedItem = this.getListItemByIndex(presentIndex + 1);
                    }
                    else {
                        this.invokeEventCallback('selectionlimitexceed', { $event: $event });
                    }
                }
            }
        }
        if (action === 'focusPrev') {
            presentIndex = presentIndex <= 0 ? 0 : (presentIndex - 1);
            this.lastSelectedItem = this.getListItemByIndex(presentIndex);
            this.lastSelectedItem.nativeElement.focus();
        }
        else if (action === 'focusNext') {
            presentIndex = presentIndex < (listItems.length - 1) ? (presentIndex + 1) : (listItems.length - 1);
            this.lastSelectedItem = this.getListItemByIndex(presentIndex);
            this.lastSelectedItem.nativeElement.focus();
        }
        else if (action === 'select') {
            // if the enter click is pressed on the item which is not the last selected item, the find the item from which the event is originated.
            if (presentIndex === -1 || !$($event.target).closest(this.lastSelectedItem.nativeElement)) {
                var $li = $($event.target).closest('li.app-list-item');
                var $ul = $li.closest('ul.app-livelist-container');
                presentIndex = $ul.find('li.app-list-item').index($li);
            }
            this.onItemClick($event, this.getListItemByIndex(presentIndex));
        }
    };
    ListComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'dataset') {
            if (!nv && this.binddatasource && !this.datasource) {
                return;
            }
            this.onDataSetChange(nv);
        }
        else if (key === 'datasource') {
            if (this.dataset) {
                this.onDataSetChange(this.dataset);
            }
        }
        else if (key === 'navigation') {
            // Support for older projects where navigation type was advanced instead of classic
            if (nv === 'Advanced') {
                this.navigation = 'Classic';
                return;
            }
            switchClass(this.nativeElement, nv, ov);
            this.onNavigationTypeChange(nv);
            if (this.dataNavigator) {
                this.dataNavigator.navigationClass = this.paginationclass;
            }
        }
        else if (key === 'itemsperrow') {
            setListClass(this);
        }
        else if (key === 'tabindex') {
            return;
        }
        else if (key === 'pulltorefresh' && nv) {
            this.app.notify('pullToRefresh:enable');
            this.subscribeToPullToRefresh();
        }
        else if (key === 'paginationclass') {
            if (this.dataNavigator) {
                // Adding setTimeout because in pagination component updateNavSize method is overriding navigationclass
                setTimeout(function () { return _this.dataNavigator.navigationClass = nv; });
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    ListComponent.prototype.onItemClick = function (evt, $listItem) {
        var _this = this;
        var selectCount;
        if (!$listItem.disableItem) {
            this.firstSelectedItem = this.firstSelectedItem || $listItem;
            // Setting selectCount value based number of items selected.
            selectCount = _.isArray(this.selecteditem) ? this.selecteditem.length : (_.isObject(this.selecteditem) ? 1 : 0);
            // Handling multiselect for mobile applications
            if (this.multiselect && isMobileApp()) {
                if (this.checkSelectionLimit(selectCount) || $listItem.isActive) {
                    this.toggleListItemSelection($listItem);
                }
                else {
                    this.invokeEventCallback('selectionlimitexceed', { $event: evt });
                }
            }
            else if ((evt.ctrlKey || evt.metaKey) && this.multiselect) {
                if (this.checkSelectionLimit(selectCount) || $listItem.isActive) {
                    this.firstSelectedItem = this.lastSelectedItem = $listItem;
                    this.toggleListItemSelection($listItem);
                }
                else {
                    this.invokeEventCallback('selectionlimitexceed', { $event: evt });
                }
            }
            else if (evt.shiftKey && this.multiselect) {
                var first_1 = $listItem.context.index;
                var last_1 = this.firstSelectedItem.context.index;
                // if first is greater than last, then swap values
                if (first_1 > last_1) {
                    last_1 = [first_1, first_1 = last_1][0];
                }
                if (this.checkSelectionLimit(last_1 - first_1)) {
                    this.clearSelectedItems();
                    this.listItems.forEach(function ($liItem) {
                        var index = $liItem.context.index;
                        if (index >= first_1 && index <= last_1) {
                            _this.toggleListItemSelection($liItem);
                        }
                    });
                    this.lastSelectedItem = $listItem;
                }
                else {
                    this.invokeEventCallback('selectionlimitexceed', { $event: evt });
                }
            }
            else {
                if (!$listItem.isActive || selectCount > 1) {
                    this.clearSelectedItems();
                    this.toggleListItemSelection($listItem);
                    this.firstSelectedItem = this.lastSelectedItem = $listItem;
                }
            }
            $appDigest();
        }
    };
    // Empty the list content on clear
    ListComponent.prototype.clear = function () {
        this.updateFieldDefs([]);
    };
    /**
     *  Returns ListItem Reference based on the input provided.
     * @param val: index | model of the list item.
     * @returns {ListItemDirective}
     */
    ListComponent.prototype.getItemRefByIndexOrModel = function (val) {
        var listItem;
        if (isNumber(val)) {
            listItem = this.getListItemByIndex(val);
        }
        else {
            listItem = this.getListItemByModel(val);
        }
        return listItem;
    };
    /**
     * deselects item in the list.
     * @param val: index | model of the list item.
     */
    ListComponent.prototype.deselectItem = function (val) {
        var listItem = this.getItemRefByIndexOrModel(val);
        if (listItem && listItem.isActive) {
            this.toggleListItemSelection(listItem);
        }
    };
    /**
     * selects item in the list.
     * @param val: index | model of the list item.
     */
    ListComponent.prototype.selectItem = function (val, statePersistenceTriggered) {
        var listItem = this.getItemRefByIndexOrModel(val);
        if (!listItem) {
            return;
        }
        if (!listItem.isActive) {
            this.toggleListItemSelection(listItem, statePersistenceTriggered);
        }
        // focus the element.
        listItem.nativeElement.focus();
    };
    ListComponent.prototype.beforePaginationChange = function ($event, $index) {
        this.invokeEventCallback('paginationchange', { $event: $event, $index: $index });
    };
    ListComponent.prototype.handleEvent = function (node, eventName, eventCallback, locals) {
        var _this = this;
        // tap and doubleTap events are not getting propagated.So, using mouse events instead.
        var touchToMouse = {
            tap: 'click',
            doubletap: 'dblclick'
        };
        if (_.includes(['click', 'tap', 'dblclick', 'doubletap'], eventName)) {
            this.eventManager.addEventListener(this.nativeElement, touchToMouse[eventName] || eventName, function (evt) {
                var target = $(evt.target).closest('.app-list-item');
                if (target.length) {
                    var listItemContext = target.data('listItemContext');
                    if (!listItemContext.disableItem) {
                        _this.invokeEventCallback(eventName, { widget: listItemContext, $event: evt, item: listItemContext.item });
                    }
                }
            });
        }
    };
    // Invoke the datasource variable by default when pulltorefresh event is not specified.
    ListComponent.prototype.subscribeToPullToRefresh = function () {
        var _this = this;
        this._listenerDestroyers.push(this.app.subscribe('pulltorefresh', function () {
            if (_this.datasource && _this.datasource.listRecords) {
                _this.datasource.listRecords();
            }
        }));
    };
    ListComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.handleHeaderClick = noop;
        this._items = [];
        this.fieldDefs = [];
        // When pagination is infinite scroll dataset is applying after debounce time(250ms) so making next call after previous data has rendered
        this.debouncedFetchNextDatasetOnScroll = _.debounce(this.fetchNextDatasetOnScroll, DEBOUNCE_TIMES.PAGINATION_DEBOUNCE_TIME);
        this.reorderProps = {
            minIndex: null,
            maxIndex: null
        };
    };
    ListComponent.prototype.setUpCUDHandlers = function () {
        var _this = this;
        var $addItem = document.getElementsByClassName("add-list-item")[0];
        if ($addItem) {
            // Triggered on click of add action
            $addItem.addEventListener('click', function (evt) {
                _this.create();
            });
        }
    };
    ListComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.promiseResolverFn();
        this.propsInitPromise.then(function () {
            _super.prototype.ngAfterViewInit.call(_this);
            _this.setUpCUDHandlers();
            _this.selectedItemWidgets = _this.multiselect ? [] : {};
            if (_this.enablereorder && !_this.groupby) {
                _this.configureDnD();
            }
            if (_this.groupby && _this.collapsible) {
                _this.handleHeaderClick = handleHeaderClick;
                _this.toggleAllHeaders = toggleAllHeaders.bind(undefined, _this);
            }
            setListClass(_this);
        });
        this.setupHandlers();
        var $ul = this.nativeElement.querySelector('ul.app-livelist-container');
        styler($ul, this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        if (isMobileApp() && $ul.querySelector('.app-list-item-action-panel')) {
            this._listAnimator = new ListAnimator(this);
        }
    };
    ListComponent.prototype.ngOnDestroy = function () {
        if (this._listAnimator && this._listAnimator.$btnSubscription) {
            this._listAnimator.$btnSubscription.unsubscribe();
        }
        this._listenerDestroyers.forEach(function (d) { return d && d(); });
        _super.prototype.ngOnDestroy.call(this);
    };
    ListComponent.prototype.ngOnDetach = function () {
        _super.prototype.ngOnDetach.call(this);
        this._pageLoad = true;
    };
    var ListComponent_1;
    ListComponent.initializeProps = registerProps();
    ListComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: ChangeDetectorRef },
        { type: ToDatePipe },
        { type: App },
        { type: AppDefaults },
        { type: NgZone },
        { type: String, decorators: [{ type: Attribute, args: ['itemclass.bind',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['disableitem.bind',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['datasource.bind',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['mouseenter.event',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['mouseleave.event',] }] },
        { type: StatePersistence }
    ]; };
    __decorate([
        ContentChild('listTemplate'),
        __metadata("design:type", TemplateRef)
    ], ListComponent.prototype, "listTemplate", void 0);
    __decorate([
        ContentChild('listLeftActionTemplate'),
        __metadata("design:type", TemplateRef)
    ], ListComponent.prototype, "listLeftActionTemplate", void 0);
    __decorate([
        ContentChild('listRightActionTemplate'),
        __metadata("design:type", TemplateRef)
    ], ListComponent.prototype, "listRightActionTemplate", void 0);
    __decorate([
        ContentChildren(ButtonComponent),
        __metadata("design:type", Object)
    ], ListComponent.prototype, "btnComponents", void 0);
    __decorate([
        ViewChild(PaginationComponent),
        __metadata("design:type", Object)
    ], ListComponent.prototype, "dataNavigator", void 0);
    __decorate([
        ViewChildren(ListItemDirective),
        __metadata("design:type", QueryList)
    ], ListComponent.prototype, "listItems", void 0);
    ListComponent = ListComponent_1 = __decorate([
        Component({
            selector: 'div[wmList]',
            template: "<div class=\"panel-heading\" *ngIf=\"title || subheading || iconclass\">\n    <h3 class=\"panel-title\">\n        <div class=\"pull-left\">\n            <i class=\"app-icon panel-icon {{iconclass}}\" aria-hidden=\"true\" [hidden]=\"!iconclass\"></i>\n        </div>\n        <div class=\"pull-left\">\n            <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\"></div>\n            <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n        </div>\n    </h3>\n</div>\n\n<nav class=\"app-datanavigator\" aria-label=\"Page navigation\" *ngIf=\"navigation === 'Inline' && !noDataFound\">\n    <ul class=\"pager\">\n        <li class=\"previous\" [ngClass]=\"{'disabled': (dataNavigator ? dataNavigator.isDisablePrevious : true)}\">\n            <a href=\"javascript:void(0);\" (click)=\"dataNavigator.navigatePage('prev', $event)\">\n                <i class=\"wi wi-chevron-left\" aria-hidden=\"true\"></i>\n                <span class=\"sr-only\">Previous</span>\n            </a>\n        </li>\n    </ul>\n</nav>\n\n<ul tabindex=\"0\" class=\"app-livelist-container clearfix {{listclass}}\"\n    (keydown.enter)=\"handleKeyDown($event, 'select')\"\n\n    (keydown.arrowup)=\"handleKeyDown($event, 'focusPrev')\"\n    (keydown.arrowdown)=\"handleKeyDown($event, 'focusNext')\"\n    (keydown.arrowleft)=\"handleKeyDown($event, 'focusPrev')\"\n    (keydown.arrowright)=\"handleKeyDown($event, 'focusNext')\"\n\n    (keydown.shift.arrowup)=\"handleKeyDown($event, 'selectPrev')\"\n    (keydown.shift.arrowdown)=\"handleKeyDown($event, 'selectNext')\"\n    (keydown.shift.arrowleft)=\"handleKeyDown($event, 'selectPrev')\"\n    (keydown.shift.arrowright)=\"handleKeyDown($event, 'selectNext')\"\n    [hidden]=\"noDataFound && !variableInflight\"\n>\n    <ng-template [ngIf]=\"!groupby\" [ngIfElse]=\"groupedListTemplate\">\n        <li *ngFor=\"let item of fieldDefs; index as $index; first as $first; last as $last;\" class=\"app-list-item {{itemsPerRowClass}}\" [ngClass]=\"listItemRef.itemClass\"\n            [wmListItem]=\"item\"\n            #listItemRef=\"listItemRef\"\n            tabindex=\"0\">\n            <ng-container [ngTemplateOutlet]=\"listTemplate\" [ngTemplateOutletContext]=\"{item: item, $index: $index, itemRef: listItemRef, $first: $first, $last: $last, currentItemWidgets: listItemRef.currentItemWidgets}\"></ng-container>\n        </li>\n    </ng-template>\n    <ng-container [ngTemplateOutlet]=\"listLeftActionTemplate\"></ng-container>\n    <ng-container [ngTemplateOutlet]=\"listRightActionTemplate\"></ng-container>\n</ul>\n\n<div class=\"no-data-msg\" *ngIf=\"noDataFound && !variableInflight\" [textContent]=\"nodatamessage\"></div>\n\n<div class=\"loading-data-msg\" *ngIf=\"variableInflight && !pulltorefresh\">\n    <span>\n        <i class=\"app-icon panel-icon fa-spin\" aria-hidden=\"true\" [ngClass]=\"loadingicon\"></i>\n        <span class=\"sr-only\">Loading</span>\n        <span class=\"loading-text\" [textContent]=\"loadingdatamsg\"></span>\n    </span>\n</div>\n\n<nav class=\"app-datanavigator\" *ngIf=\"navigation === 'Inline' && !noDataFound\">\n    <ul class=\"pager\">\n        <li class=\"next\" [ngClass]=\"{'disabled': (dataNavigator ? dataNavigator.isDisableNext  : true)}\">\n            <a href=\"javascript:void(0);\" (click)=\"dataNavigator.navigatePage('next', $event)\">\n                <i class=\"wi wi-chevron-right\" aria-hidden=\"true\"></i>\n                <span class=\"sr-only\">Next</span>\n            </a>\n        </li>\n    </ul>\n</nav>\n\n<div class=\"panel-footer\" *ngIf=\"navigation !== 'None'\" [hidden]=\"!showNavigation || (onDemandLoad && dataNavigator.isDisableNext) || (dataNavigator.dataSize <= pagesize) || !dataNavigator.dataSize\">\n    <nav wmPagination\n         navigationalign.bind=\"navigationalign\"\n         navigation.bind=\"navControls\"\n         showrecordcount.bind=\"showrecordcount\"\n         maxsize.bind=\"maxsize\"\n         boundarylinks.bind=\"boundarylinks\"\n         forceellipses.bind=\"forceellipses\"\n         directionlinks.bind=\"directionlinks\"\n         show.bind=\"!onDemandLoad\"\n         paginationchange.event=\"beforePaginationChange($event, $index)\"\n    ></nav>\n    <a *ngIf=\"onDemandLoad\" href=\"javascript:void(0);\"\n       (click)=\"dataNavigator.navigatePage('next', $event)\"\n       class=\"app-button btn btn-block\"\n       [ngClass]=\"paginationclass\"\n       [textContent]=\"ondemandmessage\"\n    ></a>\n</div>\n\n<!--This template will be displayed when groupby is specified-->\n<ng-template #groupedListTemplate>\n    <li *ngFor=\"let groupObj of groupedData\" class=\"app-list-item-group clearfix\">\n        <ul class=\"list-group item-group\" [ngClass]=\"listclass\">\n            <li class=\"app-list-item-header list-item list-group-header\" (click)=\"handleHeaderClick($event)\" [ngClass]=\"{'collapsible-content': collapsible}\">\n            <h4>{{groupObj.key}}\n                <div class=\"header-action\">\n                    <i class=\"app-icon wi action wi-chevron-up\" *ngIf=\"collapsible\"></i>\n                    <span *ngIf=\"showcount\" class=\"label label-default\">{{groupObj.data.length}}</span>\n                    </div>\n                </h4>\n            </li>\n            <li *ngFor=\"let item of groupObj.data; index as $index; first as $first; last as $last;\" tabindex=\"0\"\n                class=\"app-list-item group-list-item {{itemsPerRowClass}}\"\n                [ngClass]=\"listItemRef.itemClass\"\n                [wmListItem]=\"item\" #listItemRef=\"listItemRef\">\n                <ng-container [ngTemplateOutlet]=\"listTemplate\" [ngTemplateOutletContext]=\"{item: item, $index: $index, itemRef: listItemRef, $first: $first, $last: $last, currentItemWidgets: listItemRef.currentItemWidgets}\"></ng-container>\n            </li>\n        </ul>\n    </li>\n</ng-template>\n",
            providers: [
                provideAsWidgetRef(ListComponent_1)
            ]
        }),
        __param(6, Attribute('itemclass.bind')),
        __param(7, Attribute('disableitem.bind')),
        __param(8, Attribute('dataset.bind')),
        __param(9, Attribute('datasource.bind')),
        __param(10, Attribute('mouseenter.event')),
        __param(11, Attribute('mouseleave.event')),
        __metadata("design:paramtypes", [Injector,
            ChangeDetectorRef,
            ToDatePipe,
            App,
            AppDefaults,
            NgZone, String, String, String, String, String, String, StatePersistence])
    ], ListComponent);
    return ListComponent;
}(StylableComponent));

var components = [
    ListComponent,
    ListItemDirective
];
var ListModule = /** @class */ (function () {
    function ListModule() {
    }
    ListModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                PaginationModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], ListModule);
    return ListModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ListComponent, ListModule, ListItemDirective as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
