import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, ElementRef, ChangeDetectorRef, Attribute, ContentChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, styler, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { debounceTime } from 'rxjs/operators';
import { $appDigest, getClonedObject, isDefined, triggerFn, AbstractDialogService, App } from '@wm/core';
import { TableComponent } from '@wm/components/data/table';

var registerProps = function () {
    register('wm-livetable', new Map([
        ['class', PROP_STRING],
        ['formlayout', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ]));
};

var DEFAULT_CLS = 'app-livegrid';
var WIDGET_CONFIG = { widgetType: 'wm-livetable', hostClass: DEFAULT_CLS };
var LiveTableComponent = /** @class */ (function (_super) {
    __extends(LiveTableComponent, _super);
    function LiveTableComponent(inj, elRef, cdr, dialogService, app, layoutType, dialogId) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.dialogService = dialogService;
        _this.app = app;
        _this.$queue = [];
        _this.tableOptions = {
            'multiselect': false,
            'setGridEditMode': ''
        };
        styler(_this.nativeElement, _this);
        if (layoutType === 'dialog') {
            _this.isLayoutDialog = true;
            _this.dialogId = dialogId;
        }
        return _this;
    }
    LiveTableComponent_1 = LiveTableComponent;
    LiveTableComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        _super.prototype.ngAfterContentInit.call(this);
        if (this.table) {
            this.table._liveTableParent = this;
            this.table.datagridElement.datatable('option', this.tableOptions);
            this.table.selectedItemChange$
                .pipe(debounceTime(250))
                .subscribe(this.onSelectedItemChange.bind(this));
            if (!this.isLayoutDialog && !this.form) {
                this.table.datagridElement.datatable('option', {
                    'beforeRowUpdate': function () {
                        _this.showErrorMessage();
                    },
                    'beforeRowDelete': function () {
                        _this.showErrorMessage();
                    },
                    'beforeRowInsert': function () {
                        _this.showErrorMessage();
                    }
                });
            }
        }
    };
    LiveTableComponent.prototype.openDialog = function () {
        this.dialogService.open(this.dialogId);
        $appDigest();
    };
    LiveTableComponent.prototype.closeDialog = function () {
        this.dialogService.close(this.dialogId);
    };
    LiveTableComponent.prototype.focusFirstInput = function () {
        var $firstInput = $(this.form.$element).find('[role="input"]').first();
        $firstInput.focus();
        $firstInput.select();
    };
    LiveTableComponent.prototype.onDialogOpen = function () {
        this.focusFirstInput();
    };
    LiveTableComponent.prototype.setFormData = function (val) {
        if (!this.form.isDestroyed) {
            this.form.getWidget().formdata = val;
        }
    };
    LiveTableComponent.prototype._addNewRow = function () {
        this.form.isSelected = true;
        this.setFormData('');
        this.form.new();
        if (this.isLayoutDialog) {
            this.onDialogOpen();
        }
    };
    LiveTableComponent.prototype.addNewRow = function () {
        if (this.isLayoutDialog) {
            this.openDialog();
            this.$queue.push(this._addNewRow.bind(this));
            return;
        }
        this._addNewRow();
    };
    LiveTableComponent.prototype._updateRow = function (row, eventName) {
        this.setFormData(row);
        this.form.isSelected = true;
        this.form.edit();
        if (this.isLayoutDialog) {
            this.form.isUpdateMode = (eventName === 'dblclick') ? this.form.updateMode : true;
            this.onDialogOpen();
        }
    };
    LiveTableComponent.prototype.updateRow = function (row, eventName) {
        if (this.isLayoutDialog) {
            this.openDialog();
            this.$queue.push(this._updateRow.bind(this, row, eventName));
            return;
        }
        this._updateRow(row, eventName);
    };
    LiveTableComponent.prototype.onSelectedItemChange = function (newValue) {
        var rowData;
        if (!this.form || !this.table) {
            return;
        }
        if (newValue && newValue.length > 0 && !this.form.isSelected) {
            this.form.isSelected = true;
        }
        /*Update the rowdata of only that grid form that is associated with the specific grid on which row selection is being performed...
         * Since both the grid & gridform are associated with the same "parentgrid", match the same*/
        if (newValue && newValue.length > 0) {
            if (this.table.multiselect) {
                rowData = newValue[0];
            }
            else {
                rowData = newValue[newValue.length - 1];
            }
            this.setFormData(getClonedObject(rowData));
            /*If the form is already in update mode, call the form update function*/
            if (this.form.isUpdateMode) {
                this.form.edit();
            }
        }
        else {
            this.form.isSelected = false;
            this.setFormData('');
            this.form.clearData();
        }
    };
    LiveTableComponent.prototype.onCancel = function () {
        this.form.isUpdateMode = false;
        if (this.isLayoutDialog) {
            this.closeDialog();
        }
    };
    LiveTableComponent.prototype.onResult = function (operation, response, newForm, updateMode) {
        this.form.isUpdateMode = isDefined(updateMode) ? updateMode : newForm ? true : false;
        switch (operation) {
            case 'insert':
                if (newForm) {
                    /*if new form is to be shown after insert, skip the highlight of the row*/
                    this.table.gridfirstrowselect = false;
                    this.table.initiateSelectItem(this.table.getNavigationTargetBySortInfo(), response, true);
                }
                else {
                    /*The new row would always be inserted at the end of all existing records. Hence navigate to the last page and highlight the inserted row.*/
                    this.table.initiateSelectItem(this.table.getNavigationTargetBySortInfo(), response);
                }
                break;
            case 'update':
                /*The updated row would be found in the current page itself. Hence simply highlight the row in the current page.*/
                if (newForm) {
                    this.table.gridfirstrowselect = false;
                    this.table.initiateSelectItem('current', response, true);
                }
                else {
                    this.table.initiateSelectItem('current', response);
                }
                break;
            case 'delete':
                this.table.onRecordDelete();
                break;
        }
        this.table.updateVariable();
        if (this.isLayoutDialog) {
            /*if new form is to be shown after update or insert, don't close the dialog*/
            if (newForm) {
                if (operation === 'insert') {
                    this.form.new();
                }
                else if (operation === 'update') {
                    this.form.edit();
                }
            }
            else {
                this.closeDialog();
            }
        }
    };
    LiveTableComponent.prototype.showErrorMessage = function () {
        this.app.notifyApp(this.appLocale.LABEL_ACCESS_DENIED, 'error');
    };
    // In dialog mode, on form render call the queued functions
    LiveTableComponent.prototype.onFormReady = function (form) {
        var _this = this;
        this.form = form;
        setTimeout(function () {
            triggerFn(_this.$queue.pop());
        });
    };
    var LiveTableComponent_1;
    LiveTableComponent.initializeProps = registerProps();
    LiveTableComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: AbstractDialogService },
        { type: App },
        { type: String, decorators: [{ type: Attribute, args: ['formlayout',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['dialogid',] }] }
    ]; };
    __decorate([
        ContentChild(TableComponent),
        __metadata("design:type", TableComponent)
    ], LiveTableComponent.prototype, "table", void 0);
    LiveTableComponent = LiveTableComponent_1 = __decorate([
        Component({
            selector: '[wmLiveTable]',
            template: "<ng-content></ng-content>",
            providers: [
                provideAsWidgetRef(LiveTableComponent_1)
            ]
        }),
        __param(5, Attribute('formlayout')),
        __param(6, Attribute('dialogid')),
        __metadata("design:paramtypes", [Injector,
            ElementRef,
            ChangeDetectorRef,
            AbstractDialogService,
            App, String, String])
    ], LiveTableComponent);
    return LiveTableComponent;
}(StylableComponent));

var components = [
    LiveTableComponent
];
var LiveTableModule = /** @class */ (function () {
    function LiveTableModule() {
    }
    LiveTableModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], LiveTableModule);
    return LiveTableModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { LiveTableComponent, LiveTableModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
