import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { EventEmitter, Injector, SkipSelf, Inject, Output, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { PaginationModule as PaginationModule$1 } from 'ngx-bootstrap/pagination';
import { register, PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, DEBOUNCE_TIMES, styler, getOrderByExpr, WidgetRef, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { debounce, isDefined, AppConstants, DataSource, triggerFn, $appDigest, $watch, switchClass } from '@wm/core';

var registerProps = function () {
    register('wm-pagination', new Map([
        ['boundarylinks', __assign({ value: false }, PROP_BOOLEAN)],
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['directionlinks', __assign({ value: true }, PROP_BOOLEAN)],
        ['forceellipses', __assign({ value: true }, PROP_BOOLEAN)],
        ['maxResults', PROP_NUMBER],
        ['maxsize', __assign({ value: 5 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['navigation', __assign({ value: 'Basic' }, PROP_STRING)],
        ['navigationalign', __assign({ value: 'left' }, PROP_STRING)],
        ['navigationsize', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['showrecordcount', PROP_BOOLEAN],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
    ]));
};

var DEFAULT_CLS = 'app-datanavigator clearfix';
var WIDGET_CONFIG = { widgetType: 'wm-pagination', hostClass: DEFAULT_CLS };
var sizeClasses = {
    'Pager': {
        'small': 'pager-sm',
        'large': 'pager-lg'
    },
    'Basic': {
        'small': 'pagination-sm',
        'large': 'pagination-lg'
    },
    'Classic': {
        'small': 'pagination-sm',
        'large': 'pagination-lg'
    }
};
var unsupportedStatePersistenceTypes = ['On-Demand', 'Scroll'];
var PaginationComponent = /** @class */ (function (_super) {
    __extends(PaginationComponent, _super);
    function PaginationComponent(inj, parent) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.parent = parent;
        _this.resultEmitter = new EventEmitter();
        _this.maxResultsEmitter = new EventEmitter();
        _this.dn = {
            currentPage: 1
        };
        _this.pageCount = 0;
        _this.isDisableNext = true;
        _this.isDisablePrevious = true;
        _this.isDisableFirst = true;
        _this.isDisableLast = true;
        _this._debouncedApplyDataset = debounce(function () { return _this.widget.dataset = _this.dataset; }, DEBOUNCE_TIMES.PAGINATION_DEBOUNCE_TIME);
        _this._debouncedPageChanged = debounce(function (event) {
            var currentPage = event && event.page;
            // Do not call goToPage if page has not changed
            if (currentPage !== _this.dn.currentPage) {
                var inst = _this.parent || _this;
                _this.dn.currentPage = currentPage;
                inst.invokeEventCallback('paginationchange', { $event: undefined, $index: _this.dn.currentPage });
                _this.goToPage();
            }
        }, DEBOUNCE_TIMES.PAGINATION_DEBOUNCE_TIME);
        styler(_this.nativeElement, _this);
        return _this;
    }
    PaginationComponent_1 = PaginationComponent;
    PaginationComponent.prototype.setResult = function (result) {
        // TODO: Emit event only if result is changed
        this.result = result;
        this.resultEmitter.emit(this.result);
    };
    // Update navigationClass based on navigation and navigationSize props
    PaginationComponent.prototype.updateNavSize = function () {
        var sizeCls = sizeClasses[this.navcontrols];
        if (sizeCls && this.navigationsize) {
            this.navigationClass = sizeCls[this.navigationsize];
        }
        else {
            this.navigationClass = '';
        }
    };
    // Function to reset the paging values to default.
    PaginationComponent.prototype.resetPageNavigation = function () {
        this.pageCount = 0;
        this.dn.currentPage = 1;
        this.dataSize = 0;
    };
    /*Function to calculate the paging values.*/
    PaginationComponent.prototype.calculatePagingValues = function () {
        this.pageCount = (this.dataSize > this.maxResults) ? (Math.ceil(this.dataSize / this.maxResults)) : (this.dataSize < 0 ? 0 : 1);
        this.dn.currentPage = this.dn.currentPage || 1;
    };
    /*Function to set default values to the paging parameters*/
    PaginationComponent.prototype.setDefaultPagingValues = function (dataSize, maxResults, currentPage) {
        /*If neither 'dataSize' nor 'maxResults' is set, then set default values to the paging parameters.*/
        if (!dataSize && !maxResults) {
            this.pageCount = 1;
            this.dn.currentPage = 1;
            this.maxResults = dataSize;
            this.dataSize = dataSize;
        }
        else { /*Else, set the specified values and recalculate paging parameters.*/
            this.maxResults = maxResults || this.maxResults;
            this.dataSize = isDefined(dataSize) ? dataSize : this.dataSize;
            this.dn.currentPage = currentPage || this.dn.currentPage;
            this.calculatePagingValues();
        }
        this.maxResultsEmitter.emit(this.maxResults);
    };
    /*Function to check the dataSize and manipulate the navigator accordingly.*/
    PaginationComponent.prototype.checkDataSize = function (dataSize, numberOfElements, size) {
        /*If the dataSize is -1 or Integer.MAX_VALUE( which is 2147483647), then the total number of records is not known.
         * Hence,
         * 1. Hide the 'Total Record Count'.
         * 2. Disable the 'GoToLastPage' link as the page number of the last page is not known.*/
        if (dataSize === -1 || dataSize === AppConstants.INT_MAX_VALUE) {
            this.prevshowrecordcount = this.showrecordcount;
            this.isDisableLast = true;
            this.isDisableCount = true;
            this.showrecordcount = false;
            // If number of records in current page is less than the max records size, this is the last page. So disable next button.
            if (numberOfElements < size) {
                this.isDisableNext = true;
            }
        }
        else {
            this.isDisableCount = false;
            this.showrecordcount = this.prevshowrecordcount || this.showrecordcount;
        }
    };
    /*Function to disable navigation based on the current and total pages.*/
    PaginationComponent.prototype.disableNavigation = function () {
        var isCurrentPageFirst = (this.dn.currentPage === 1), isCurrentPageLast = (this.dn.currentPage === this.pageCount);
        this.isDisableFirst = this.isDisablePrevious = isCurrentPageFirst;
        this.isDisableNext = this.isDisableLast = isCurrentPageLast;
        this.isDisableCurrent = isCurrentPageFirst && isCurrentPageLast;
    };
    /*Function to check if the variable bound to the data-navigator has paging.*/
    PaginationComponent.prototype.isDataSourceHasPaging = function () {
        return this.datasource && this.datasource.execute(DataSource.Operation.IS_PAGEABLE);
    };
    // Set the result for client side pagination
    PaginationComponent.prototype.setNonPageableData = function (newVal) {
        var dataSize, maxResults, currentPage, startIndex;
        dataSize = _.isArray(newVal) ? newVal.length : (_.isEmpty(newVal) ? 0 : 1);
        maxResults = (this.options && this.options.maxResults) || dataSize;
        // For static variable, keep the current page. For other variables without pagination reset the page to 1
        if (this.datasource && this.datasource.execute(DataSource.Operation.IS_API_AWARE)) {
            currentPage = 1;
        }
        else {
            currentPage = this.dn.currentPage || 1;
        }
        this.setDefaultPagingValues(dataSize, maxResults, currentPage);
        this.disableNavigation();
        startIndex = (this.dn.currentPage - 1) * this.maxResults;
        this.setResult(_.isArray(newVal) ? newVal.slice(startIndex, startIndex + this.maxResults) : newVal);
    };
    /*Function to set the values needed for pagination*/
    PaginationComponent.prototype.setPagingValues = function (newVal) {
        var dataSize, maxResults, currentPage, dataSource;
        var variableOptions = {};
        // Store the data in __fullData. This is used for client side searching witvah out modifying the actual dataset.
        this.__fullData = newVal;
        /*Check for sanity*/
        if (this.binddataset) {
            dataSource = this.datasource || {};
            variableOptions = dataSource._options || {};
            /*Check for number of elements in the data set*/
            if (newVal) {
                if (this.isDataSourceHasPaging()) {
                    this.pagination = this.datasource.execute(DataSource.Operation.GET_PAGING_OPTIONS) || {};
                    // If "filterFields" and "sortOptions" have been set, then set them so that the filters can be retained while fetching data upon page navigation.
                    this.filterFields = variableOptions.filterFields || {};
                    this.sortOptions = variableOptions.orderBy ||
                        (_.isArray(this.pagination.sort) ? getOrderByExpr(this.pagination.sort) : '');
                    dataSize = this.pagination.totalElements;
                    maxResults = this.pagination.size;
                    if (this.pagination.numberOfElements > 0) {
                        if (isDefined(this.pagination.number)) { // number is page number received from backend
                            this.dn.currentPage = this.pagination.number + 1;
                        }
                        currentPage = this.dn.currentPage || 1;
                    }
                    else {
                        currentPage = 1;
                    }
                    /* Sending pageCount undefined to calculate it again for query.*/
                    this.setDefaultPagingValues(dataSize, maxResults, currentPage);
                    this.disableNavigation();
                    this.checkDataSize(dataSize, this.pagination.numberOfElements, this.pagination.size);
                    this.setResult(newVal);
                }
                else if (!_.isString(newVal)) {
                    this.setNonPageableData(newVal);
                }
            }
            else {
                this.setResult(newVal);
                this.resetPageNavigation();
            }
        }
        else {
            if (newVal && !_.isString(newVal)) {
                this.setNonPageableData(newVal);
            }
        }
    };
    /*Function to check if the current page is the first page*/
    PaginationComponent.prototype.isFirstPage = function () {
        return (this.dn.currentPage === 1 || !this.dn.currentPage);
    };
    /*Function to check if the current page is the last page*/
    PaginationComponent.prototype.isLastPage = function () {
        return (this.dn.currentPage === this.pageCount);
    };
    /*Function to navigate to the last page*/
    PaginationComponent.prototype.goToLastPage = function (isRefresh, event, callback) {
        if (!this.isLastPage()) {
            this.dn.currentPage = this.pageCount;
            this.goToPage(event, callback);
        }
        else if (isRefresh) {
            this.goToPage(event, callback);
        }
    };
    /*Function to navigate to the first page*/
    PaginationComponent.prototype.goToFirstPage = function (isRefresh, event, callback) {
        if (!this.isFirstPage()) {
            this.dn.currentPage = 1;
            this.goToPage(event, callback);
        }
        else if (isRefresh) {
            this.goToPage(event, callback);
        }
    };
    /*Function to navigate to the current page*/
    PaginationComponent.prototype.goToPage = function (event, callback) {
        this.firstRow = (this.dn.currentPage - 1) * this.maxResults;
        var mode = this.parent.statePersistence.computeMode(this.statehandler);
        if (mode && mode.toLowerCase() !== 'none' && (this.parent.widgetType === 'wm-table' || this.parent.widgetType === 'wm-list')) {
            this.parent._selectedItemsExist = true;
            if (this.isFirstPage()) {
                this.parent.statePersistence.removeWidgetState(this.parent, 'pagination');
            }
            else {
                if (unsupportedStatePersistenceTypes.indexOf(this.parent.navigation) < 0) {
                    this.parent.statePersistence.setWidgetState(this.parent, { pagination: this.dn.currentPage });
                }
                else if (this.parent.widgetType === 'wm-list') {
                    console.warn('Retain State handling on Widget ' + this.parent.name + ' is not supported for current pagination type.');
                }
            }
        }
        // call getPageData after checking state config so that Static Variables can use it
        this.getPageData(event, callback);
    };
    /*Function to be invoked after the data of the page has been fetched.*/
    PaginationComponent.prototype.onPageDataReady = function (event, data, callback) {
        this.disableNavigation();
        this.invokeSetRecord(event, data);
        triggerFn(callback);
    };
    /*Function to get data for the current page*/
    PaginationComponent.prototype.getPageData = function (event, callback) {
        var _this = this;
        var data, startIndex;
        if (this.isDataSourceHasPaging()) {
            this.datasource.execute(DataSource.Operation.LIST_RECORDS, {
                'page': this.dn.currentPage,
                'filterFields': this.filterFields,
                'orderBy': this.sortOptions,
                'matchMode': 'anywhereignorecase'
            }).then(function (response) {
                _this.onPageDataReady(event, response && response.data, callback);
                $appDigest();
            }, function (error) {
                // If error is undefined, do not show any message as this may be discarded request
                if (error) {
                    // TODO: Handle Error
                    // wmToaster.show('error', 'ERROR', 'Unable to get data of page -' + this.dn.currentPage + ':' + error);
                }
            });
        }
        else {
            startIndex = (this.dn.currentPage - 1) * this.maxResults;
            data = _.isArray(this.__fullData) ? this.__fullData.slice(startIndex, startIndex + this.maxResults) : this.__fullData;
            this.setResult(data);
            this.onPageDataReady(event, data, callback);
        }
    };
    PaginationComponent.prototype.invokeSetRecord = function (event, data) {
        // Trigger the event handler if exists.
        var pageInfo = {
            currentPage: this.dn.currentPage,
            size: this.maxResults,
            totalElements: this.dataSize,
            totalPages: this.pageCount
        };
        if (this.parent) {
            this.parent.invokeEventCallback('setrecord', { $event: event, $data: data, $index: this.dn.currentPage, pageInfo: pageInfo, data: data });
        }
        else {
            this.invokeEventCallback('setrecord', { $event: event, $data: data, $index: this.dn.currentPage, pageInfo: pageInfo, data: data });
        }
    };
    /*Function to validate the page input.
     In case of invalid input, navigate to the appropriate page; also return false.
     In case of valid input, return true.*/
    PaginationComponent.prototype.validateCurrentPage = function (event, callback) {
        /*If the value entered is greater than the last page number or invalid value, then highlighting the field showing error.*/
        if (event && (isNaN(this.dn.currentPage) || this.dn.currentPage <= 0 || (this.pageCount && (this.dn.currentPage > this.pageCount || _.isNull(this.dn.currentPage))))) {
            $(event.target).closest('a').addClass('ng-invalid');
            return false;
        }
        return true;
    };
    PaginationComponent.prototype.onModelChange = function (event) {
        if (!this.validateCurrentPage(event)) {
            return;
        }
        this.goToPage(event);
    };
    PaginationComponent.prototype.onKeyDown = function (event) {
        var targetEle = $(event.target).closest('a');
        if (event.code === 'KeyE') {
            targetEle.addClass('ng-invalid');
            return false;
        }
        targetEle.removeClass('ng-invalid');
        return true;
    };
    PaginationComponent.prototype.pageChanged = function (event) {
        this._debouncedPageChanged(event);
    };
    /*Function to navigate to the respective pages.*/
    PaginationComponent.prototype.navigatePage = function (index, event, isRefresh, callback) {
        this.invokeEventCallback('paginationchange', { $event: undefined, $index: this.dn.currentPage });
        // Convert the current page to a valid page number.
        this.dn.currentPage = +this.dn.currentPage;
        switch (index) {
            case 'first':
                this.goToFirstPage(isRefresh, event, callback);
                return;
            case 'prev':
                /*Return if already on the first page.*/
                if (this.isFirstPage() || !this.validateCurrentPage(event, callback)) {
                    return;
                }
                /*Decrement the current page by 1.*/
                this.dn.currentPage -= 1;
                break;
            case 'next':
                /*Return if already on the last page.*/
                if (this.isLastPage() || !this.validateCurrentPage(event, callback)) {
                    return;
                }
                /*Increment the current page by 1.*/
                this.dn.currentPage += 1;
                break;
            case 'last':
                this.goToLastPage(isRefresh, event, callback);
                return;
            default:
                break;
        }
        /*Navigate to the current page.*/
        this.goToPage(event, callback);
    };
    PaginationComponent.prototype.setBindDataSet = function (binddataset, parent, dataSource, dataset, binddatasource, datasetBoundExpr, statehandler) {
        var _this = this;
        var parts = binddataset.split('.');
        var bindPagingOptions;
        this.statehandler = statehandler;
        if (parts[0] === 'Variables' || parts[0] === 'Widgets') {
            bindPagingOptions = parts[0] + "." + parts[1] + ".pagination";
        }
        if (!binddatasource && dataset && !datasetBoundExpr) {
            this.dataset = dataset;
            this._debouncedApplyDataset();
            return;
        }
        this.binddataset = binddataset;
        setTimeout(function () {
            // watching for dataset changes even when dataset is bound to "item." (i.e. by checking for datasetBoundExpr)
            // item context is passed as dataSource here
            _this.registerDestroyListener($watch(binddataset, parent, datasetBoundExpr ? dataSource : {}, function (nv) {
                if (nv) {
                    _this.dataset = nv;
                    _this._debouncedApplyDataset();
                }
                else {
                    // WMS-19285: Setting dataset on proxy only to handle propertychange handler call for undefined cases
                    _this.widget.dataset = nv;
                }
            }));
            // Register a watch on paging options. Call dataset property change handler even if paging options changes to reflect pagination state
            if (!bindPagingOptions) {
                return;
            }
            _this.registerDestroyListener($watch(bindPagingOptions, parent, {}, function () { return _this._debouncedApplyDataset(); }));
        });
        // apply datasource only when dataset is not bound to "item.FIELD"
        if (!datasetBoundExpr) {
            this.datasource = dataSource;
        }
    };
    // Set the datasource of pagination from the parent widget
    PaginationComponent.prototype.setDataSource = function (dataSource) {
        this.datasource = dataSource;
    };
    PaginationComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'dataset') {
            var data = void 0;
            if (this.parent && this.parent.onDataNavigatorDataSetChange) {
                data = this.parent.onDataNavigatorDataSetChange(nv);
            }
            else {
                data = nv;
            }
            this.setPagingValues(data);
        }
        else if (key === 'navigation') {
            if (nv === 'Advanced') { // Support for older projects where navigation type was advanced instead of clasic
                this.navigation = 'Classic';
            }
            this.updateNavSize();
            this.navcontrols = nv;
        }
        else if (key === 'navigationsize') {
            this.updateNavSize();
        }
        else if (key === 'navigationalign') {
            switchClass(this.nativeElement, "text-" + nv, "text-" + ov);
        }
        else if (key === 'maxResults') {
            this.setPagingValues(this.dataset);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    PaginationComponent.prototype.ngAfterViewInit = function () {
        var paginationElem = this.nativeElement;
        paginationElem.onclick = function (event) {
            event.stopPropagation();
        };
    };
    var PaginationComponent_1;
    PaginationComponent.initializeProps = registerProps();
    PaginationComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [WidgetRef,] }] }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], PaginationComponent.prototype, "resultEmitter", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], PaginationComponent.prototype, "maxResultsEmitter", void 0);
    PaginationComponent = PaginationComponent_1 = __decorate([
        Component({
            selector: '[wmPagination]',
            template: "\n<ul class=\"pagination advanced {{navigationClass}}\" *ngIf=\"navcontrols === 'Classic'\">\n    <li [ngClass]=\"{'disabled':isDisableFirst}\">\n        <a [title]=\"appLocale.LABEL_FIRST\" name=\"first\" href=\"javascript:void(0);\" aria-label=\"First\"\n           (click)=\"navigatePage('first', $event)\">\n            <span aria-hidden=\"true\"><i class=\"wi wi-first-page\"></i></span>\n            <span class=\"sr-only\">{{appLocale.LABEL_FIRST}}</span>\n        </a>\n    </li>\n    <li [ngClass]=\"{'disabled':isDisablePrevious}\">\n        <a [title]=\"appLocale.LABEL_PREVIOUS\" name=\"prev\" href=\"javascript:void(0);\" aria-label=\"Previous\"\n           (click)=\"navigatePage('prev', $event)\">\n            <span aria-hidden=\"true\"><i class=\"wi wi-chevron-left\"></i></span>\n            <span class=\"sr-only\">{{appLocale.LABEL_PREVIOUS}}</span>\n        </a>\n    </li>\n    <li class=\"pagecount disabled\">\n        <a><input type=\"number\" [disabled]=\"isDisableCurrent\" [(ngModel)]=\"dn.currentPage\"\n                  (keydown)=\"onKeyDown($event)\" (change)=\"onModelChange($event)\" class=\"form-control\"/></a>\n    </li>\n    <li class=\"disabled\">\n        <a [hidden]=\"isDisableCount\"> / {{pageCount}}</a>\n    </li>\n    <li [ngClass]=\"{'disabled':isDisableNext}\">\n        <a [title]=\"appLocale.LABEL_NEXT\" name=\"next\" href=\"javascript:void(0);\" aria-label=\"Next\"\n           (click)=\"navigatePage('next', $event)\">\n            <span aria-hidden=\"true\"><i class=\"wi wi-chevron-right\"></i></span>\n            <span class=\"sr-only\">{{appLocale.LABEL_NEXT}}</span>\n        </a>\n    </li>\n    <li [ngClass]=\"{'disabled':isDisableLast}\">\n        <a [title]=\"appLocale.LABEL_LAST\" name=\"last\" href=\"javascript:void(0);\" aria-label=\"Last\"\n           (click)=\"navigatePage('last', $event)\">\n            <span aria-hidden=\"true\"><i class=\"wi wi-last-page\"></i></span>\n            <span class=\"sr-only\">{{appLocale.LABEL_LAST}}</span>\n        </a>\n    </li>\n    <li *ngIf=\"showrecordcount\" class=\"totalcount disabled\">\n        <a>{{appLocale.LABEL_TOTAL_RECORDS}}: {{dataSize}}</a>\n    </li>\n</ul>\n<ul class=\"pager {{navigationClass}}\" *ngIf=\"navcontrols === 'Pager'\">\n    <li class=\"previous\" [ngClass]=\"{'disabled':isDisablePrevious}\">\n        <a href=\"javascript:void(0);\" (click)=\"navigatePage('prev', $event)\" aria-label=\"Previous\">\n            <span aria-hidden=\"true\"><i class=\"wi wi-chevron-left\"></i></span>\n            {{appLocale.LABEL_PREVIOUS}}\n            <span class=\"sr-only\">{{appLocale.LABEL_PREVIOUS}}</span>\n        </a>\n    </li>\n    <li class=\"next\" [ngClass]=\"{'disabled':isDisableNext}\">\n        <a href=\"javascript:void(0);\" (click)=\"navigatePage('next', $event)\" aria-label=\"Next\">\n            {{appLocale.LABEL_NEXT}}\n            <span aria-hidden=\"true\"><i class=\"wi wi-chevron-right\"></i></span>\n            <span class=\"sr-only\">{{appLocale.LABEL_NEXT}}</span>\n        </a>\n    </li>\n</ul>\n\n<pagination class=\"pagination basic\" [ngClass]=\"navigationClass\" *ngIf=\"navcontrols === 'Basic'\"\n            [itemsPerPage]=\"maxResults\" [totalItems]=\"dataSize\"\n            [ngModel]=\"dn.currentPage\" (pageChanged)=\"pageChanged($event)\"\n            [boundaryLinks]=\"boundarylinks\" [maxSize]=\"maxsize\"\n            [directionLinks]=\"directionlinks\" previousText=\".\" nextText=\".\" firstText=\".\" lastText=\".\"></pagination>\n\n<ul *ngIf=\"navcontrols === 'Basic' && showrecordcount\" class=\"pagination\">\n    <li class=\"totalcount disabled basiccount\"><a>{{appLocale.LABEL_TOTAL_RECORDS}}: {{dataSize}}</a></li>\n</ul>\n",
            providers: [
                provideAsWidgetRef(PaginationComponent_1)
            ]
        }),
        __param(1, SkipSelf()), __param(1, Inject(WidgetRef)),
        __metadata("design:paramtypes", [Injector, Object])
    ], PaginationComponent);
    return PaginationComponent;
}(StylableComponent));

var components = [
    PaginationComponent
];
var PaginationModule = /** @class */ (function () {
    function PaginationModule() {
    }
    PaginationModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                FormsModule,
                PaginationModule$1,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], PaginationModule);
    return PaginationModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { PaginationComponent, PaginationModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
