import { __assign, __extends, __spread, __awaiter, __generator, __decorate, __metadata, __param } from 'tslib';
import { Injector, Optional, Attribute, NgZone, ViewChild, ElementRef, ContentChildren, QueryList, ViewContainerRef, ContentChild, TemplateRef, HostListener, Component, Self, Inject, Directive, SkipSelf, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormBuilder, NG_VALUE_ACCESSOR, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { TooltipModule } from 'ngx-bootstrap/tooltip';
import { BsDropdownModule } from 'ngx-bootstrap/dropdown';
import { register, PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, getConditionalClasses, styler, extractDataSourceName, EDIT_MODE, getRowOperationsColumn, transformData, prepareFieldDefs, getOrderByExpr, TrustAsPipe, provideAs, provideAsWidgetRef, StylableComponent, refreshDataSource, getMatchModeTypesMap, getMatchModeMsgs, isDataSetWidget, BaseComponent, setHeaderConfigForTable, BaseFieldValidations, applyFilterOnField, getDistinctFieldProperties, getDistinctValues, getWatchIdentifier, fetchRelatedFieldData, getDistinctValuesForField, getEditModeWidget, getDataTableFilterWidget, getDefaultValue, Context, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { InputModule } from '@wm/components/input';
import { MenuModule } from '@wm/components/navigation/menu';
import { ListComponent, ListModule } from '@wm/components/data/list';
import { PaginationComponent, PaginationModule } from '@wm/components/data/pagination';
import { closePopover, getClonedObject, $unwatch, $watch, $appDigest, triggerFn, DataSource, isDataSourceEqual, isDefined, $invokeWatchers, isMobile, extendProto, getValidJSON, getDatasourceFromExpr, $parseEvent, App, DynamicComponentRefProvider, StatePersistence, Viewport, extractCurrentItemExpr, AbstractDialogService, isNumberType, DataType, adjustContainerRightEdges, FormWidgetType, isDateTimeType, getDisplayDateTimeFormat, AppDefaults } from '@wm/core';
import { Subject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';

var registerProps = function () {
    register('wm-table', new Map([
        ['boundarylinks', __assign({ value: false }, PROP_BOOLEAN)],
        ['class', PROP_STRING],
        ['confirmdelete', __assign({ value: 'Are you sure you want to delete this?' }, PROP_STRING)],
        ['datasource', PROP_STRING],
        ['dataset', PROP_ANY],
        ['deletecanceltext', PROP_STRING],
        ['deleteoktext', PROP_STRING],
        ['deletemessage', __assign({ value: 'Record deleted successfully' }, PROP_STRING)],
        ['directionlinks', __assign({ value: true }, PROP_BOOLEAN)],
        ['editmode', PROP_STRING],
        ['errorstyle', __assign({ value: 'hover' }, PROP_STRING)],
        ['enablecolumnselection', __assign({ value: false }, PROP_BOOLEAN)],
        ['enablesort', __assign({ value: true }, PROP_BOOLEAN)],
        ['errormessage', PROP_STRING],
        ['exportformat', PROP_STRING],
        ['exportdatasize', __assign({ value: 100 }, PROP_NUMBER)],
        ['filtermode', PROP_STRING],
        ['filternullrecords', __assign({ value: true }, PROP_BOOLEAN)],
        ['forceellipses', __assign({ value: true }, PROP_BOOLEAN)],
        ['formposition', PROP_STRING],
        ['gridclass', __assign({ value: 'table-hover' }, PROP_STRING)],
        ['gridfirstrowselect', PROP_BOOLEAN],
        ['iconclass', PROP_STRING],
        ['insertmessage', __assign({ value: 'Record added successfully' }, PROP_STRING)],
        ['isdynamictable', PROP_BOOLEAN],
        ['loadingdatamsg', __assign({ value: 'Loading...' }, PROP_STRING)],
        ['loadingicon', __assign({ value: 'fa fa-circle-o-notch fa-spin' }, PROP_STRING)],
        ['maxsize', __assign({ value: 5 }, PROP_NUMBER)],
        ['multiselect', PROP_BOOLEAN],
        ['name', PROP_STRING],
        ['navigation', __assign({ value: 'Basic' }, PROP_STRING)],
        ['navigationalign', __assign({ value: 'left' }, PROP_STRING)],
        ['nodatamessage', __assign({ value: 'No data found.' }, PROP_STRING)],
        ['pagesize', PROP_NUMBER],
        ['radioselect', PROP_BOOLEAN],
        ['rowclass', PROP_STRING],
        ['rowngclass', PROP_STRING],
        ['searchlabel', __assign({ value: 'Search' }, PROP_STRING)],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['showheader', __assign({ value: true }, PROP_BOOLEAN)],
        ['shownewrow', __assign({ value: true }, PROP_BOOLEAN)],
        ['showrecordcount', PROP_BOOLEAN],
        ['showrowindex', PROP_BOOLEAN],
        ['spacing', __assign({ value: 'normal' }, PROP_STRING)],
        ['statehandler', __assign({ value: 'none' }, PROP_STRING)],
        ['subheading', PROP_STRING],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING],
        ['updatemessage', __assign({ value: 'Record updated successfully' }, PROP_STRING)]
    ]));
};

var DEFAULT_CLS = 'app-grid app-panel panel';
var WIDGET_CONFIG = { widgetType: 'wm-table', hostClass: DEFAULT_CLS };
var exportIconMapping = {
    EXCEL: 'fa fa-file-excel-o',
    CSV: 'fa fa-file-text-o'
};
var ROW_OPS_FIELD = 'rowOperations';
var noop = function () { };
var ɵ0 = noop;
var isInputBodyWrapper = function (target) {
    var classes = ['.dropdown-menu', '.uib-typeahead-match', '.modal-dialog', '.toast'];
    var isInput = false;
    classes.forEach(function (cls) {
        if (target.closest(cls).length) {
            isInput = true;
            return false;
        }
    });
    var attrs = ['bsdatepickerdaydecorator'];
    if (!isInput) {
        attrs.forEach(function (attr) {
            if (target[0].hasAttribute(attr)) {
                isInput = true;
                return false;
            }
        });
    }
    return isInput;
};
var ɵ1 = isInputBodyWrapper;
var TableComponent = /** @class */ (function (_super) {
    __extends(TableComponent, _super);
    function TableComponent(inj, fb, app, dynamicComponentProvider, statePersistence, viewport, parentList, binddataset, binddatasource, readonlygrid, ngZone, trustAsPipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.inj = inj;
        _this.fb = fb;
        _this.app = app;
        _this.dynamicComponentProvider = dynamicComponentProvider;
        _this.statePersistence = statePersistence;
        _this.viewport = viewport;
        _this.parentList = parentList;
        _this.binddataset = binddataset;
        _this.binddatasource = binddatasource;
        _this.readonlygrid = readonlygrid;
        _this.ngZone = ngZone;
        _this.trustAsPipe = trustAsPipe;
        _this.rowActionsCompiledTl = {};
        _this.rowFilterCompliedTl = {};
        _this.inlineCompliedTl = {};
        _this.inlineNewCompliedTl = {};
        _this.customExprCompiledTl = {};
        _this.customExprCompiledSummaryTl = {};
        _this.rowDefInstances = {};
        _this.rowDefMap = {};
        _this.rowExpansionActionTl = {};
        _this.columns = {};
        _this.formfields = {};
        _this.enablesort = true;
        _this.primaryKey = [];
        _this.selectedItems = [];
        _this.selectedItemChange = new Subject();
        _this.selectedItemChange$ = _this.selectedItemChange.asObservable();
        _this.actions = [];
        _this._actions = {
            'header': [],
            'footer': []
        };
        _this.exportOptions = [];
        _this.headerConfig = [];
        _this.items = [];
        _this.rowActions = [];
        _this.shownavigation = false;
        _this.documentClickBind = noop;
        _this.fieldDefs = [];
        _this.rowDef = {};
        _this.rowInstance = {};
        _this.fullFieldDefs = [];
        _this.applyProps = new Map();
        _this.redraw = _.debounce(_this._redraw, 150);
        _this.debouncedHandleLoading = _.debounce(_this.handleLoading, 350);
        // Filter and Sort Methods
        _this.rowFilter = {};
        _this._searchSortHandler = noop;
        _this.searchSortHandler = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            _this._searchSortHandler.apply(_this, args);
        };
        _this.onRowFilterChange = noop;
        _this.onFilterConditionSelect = noop;
        _this.showClearIcon = noop;
        _this.clearRowFilter = noop;
        _this.adjustContainer = noop;
        _this._pageLoad = true;
        _this.gridOptions = {
            data: [],
            colDefs: [],
            startRowIndex: 1,
            sortInfo: {
                field: '',
                direction: ''
            },
            filtermode: '',
            searchLabel: '',
            rowActions: [],
            headerConfig: [],
            rowClass: '',
            editmode: '',
            formPosition: '',
            isMobile: false,
            rowExpansionEnabled: false,
            rowDef: {
                position: '0'
            },
            name: '',
            messages: {
                selectField: 'Select Field'
            },
            securityUtils: {
                pipeTransform: {}
            },
            onDataRender: function () {
                _this.ngZone.run(function () {
                    if (_this.gridData.length) {
                        _this.invokeEventCallback('datarender', { $data: _this.gridData, data: _this.gridData });
                    }
                    // select rows selected in previous pages. (Not finding intersection of data and selecteditems as it will be heavy)
                    if (!_this.multiselect) {
                        _this.items.length = 0;
                    }
                    _this.callDataGridMethod('selectRows', _this.items);
                    _this.selectedItems = _this.callDataGridMethod('getSelectedRows');
                    _this.selectedItemChange.next(_this.selectedItems);
                    // On render, apply the filters set for query service variable
                    if (_this._isPageSearch && _this.filterInfo) {
                        _this.searchSortHandler(_this.filterInfo, undefined, 'search');
                    }
                });
            },
            onRowSelect: function (row, e) {
                _this.ngZone.run(function () {
                    _this.selectedItems = _this.callDataGridMethod('getSelectedRows');
                    _this.selectedItemChange.next(_this.selectedItems);
                    var rowData = _this.addRowIndex(row);
                    if (rowData.$index && _this.getConfiguredState() !== 'none' && _this.dataNavigator) {
                        var obj = { page: _this.dataNavigator.dn.currentPage, index: rowData.$index - 1 };
                        var widgetState = _this.statePersistence.getWidgetState(_this);
                        if (_.get(widgetState, 'selectedItem') && _this.multiselect) {
                            if (!_.some(widgetState.selectedItem, obj)) {
                                widgetState.selectedItem.push(obj);
                            }
                            _this.statePersistence.setWidgetState(_this, { 'selectedItem': widgetState.selectedItem });
                        }
                        else {
                            _this.statePersistence.setWidgetState(_this, { 'selectedItem': [obj] });
                        }
                    }
                    _this.invokeEventCallback('rowselect', { $data: rowData, $event: e, row: rowData });
                });
            },
            // assigns the items on capture phase of the click handler.
            assignSelectedItems: function (row, e) {
                _this.ngZone.run(function () {
                    /*
                     * in case of single select, update the items with out changing the reference.
                     * for multi select, keep old selected items in tact
                     */
                    if (_this.multiselect) {
                        if (_.findIndex(_this.items, row) === -1) {
                            _this.items.push(row);
                        }
                    }
                    else {
                        _this.items.length = 0;
                        _this.items.push(row);
                    }
                });
            },
            onRowDblClick: function (row, e) {
                var rowData = _this.addRowIndex(row);
                _this.invokeEventCallback('rowdblclick', { $data: rowData, $event: e, row: rowData });
            },
            onRowDeselect: function (row, e) {
                if (_this.multiselect) {
                    _this.ngZone.run(function () {
                        _this.items = _.pullAllWith(_this.items, [row], _.isEqual);
                        _this.selectedItems = _this.callDataGridMethod('getSelectedRows');
                        _this.invokeEventCallback('rowdeselect', { $data: row, $event: e, row: row });
                        var rowData = _this.addRowIndex(row);
                        if (_this.getConfiguredState() !== 'none') {
                            var obj_1 = { page: _this.dataNavigator.dn.currentPage, index: rowData.$index - 1 };
                            var widgetState = _this.statePersistence.getWidgetState(_this);
                            if (_.get(widgetState, 'selectedItem')) {
                                _.remove(widgetState.selectedItem, function (selectedItem) {
                                    return _.isEqual(selectedItem, obj_1);
                                });
                                _this.statePersistence.removeWidgetState(_this, 'selectedItem');
                                if (widgetState.selectedItem.length > 0) {
                                    _this.statePersistence.setWidgetState(_this, { 'selectedItem': widgetState.selectedItem });
                                }
                            }
                        }
                    });
                }
            },
            callOnRowDeselectEvent: function (row, e) {
                _this.items = _this.selectedItems = _this.callDataGridMethod('getSelectedRows');
                _this.invokeEventCallback('rowdeselect', { $data: row, $event: e, row: row });
            },
            callOnRowClickEvent: function (row, e) {
                // WMS-18774: If we invoke navigation action variable on rowclick callback, the navigation is triggering outside angular zone so adding ngZone.run
                _this.ngZone.run(function () {
                    // Call row click only if click is triggered by user
                    if (e && e.hasOwnProperty('originalEvent')) {
                        var rowData = _this.addRowIndex(row);
                        _this.invokeEventCallback('rowclick', { $data: rowData, $event: e, row: rowData });
                    }
                });
            },
            closePopover: closePopover,
            onColumnSelect: function (col, e) {
                _this.selectedColumns = _this.callDataGridMethod('getSelectedColumns');
                _this.invokeEventCallback('columnselect', { $data: col, $event: e });
            },
            onColumnDeselect: function (col, e) {
                _this.selectedColumns = _this.callDataGridMethod('getSelectedColumns');
                _this.invokeEventCallback('columndeselect', { $data: col, $event: e });
            },
            onHeaderClick: function (col, e) {
                // if onSort function is registered invoke it when the column header is clicked
                _this.invokeEventCallback('headerclick', { $event: e, $data: col, column: col });
            },
            onRowDelete: function (row, cancelRowDeleteCallback, e, callBack, options) {
                _this.ngZone.run(function () {
                    _this.deleteRecord(_.extend({}, options, { row: row, 'cancelRowDeleteCallback': cancelRowDeleteCallback, 'evt': e, 'callBack': callBack }));
                });
            },
            onRowInsert: function (row, e, callBack, options) {
                _this.insertRecord(_.extend({}, options, { row: row, event: e, 'callBack': callBack }));
            },
            beforeRowUpdate: function (row, eventName) {
                if (_this._liveTableParent) {
                    _this._liveTableParent.updateRow(row, eventName);
                }
                _this.prevData = getClonedObject(row);
            },
            afterRowUpdate: function (row, e, callBack, options) {
                _this.updateRecord(_.extend({}, options, { row: row, 'prevData': _this.prevData, 'event': e, 'callBack': callBack }));
            },
            onBeforeRowUpdate: function (row, e, options) {
                return _this.invokeEventCallback('beforerowupdate', { $event: e, $data: row, row: row, options: options });
            },
            onBeforeRowInsert: function (row, e, options) {
                return _this.invokeEventCallback('beforerowinsert', { $event: e, $data: row, row: row, options: options });
            },
            onBeforeRowDelete: function (row, e, options) {
                var rowData = _this.addRowIndex(row);
                return _this.invokeEventCallback('beforerowdelete', { $event: e, row: rowData, options: options });
            },
            onFormRender: function ($row, e, operation, alwaysNewRow) {
                var widget = alwaysNewRow ? 'inlineInstanceNew' : 'inlineInstance';
                setTimeout(function () {
                    _this.formWidgets = {};
                    _this.fieldDefs.forEach(function (col) {
                        if (col[widget]) {
                            _this.formWidgets[col.field] = col[widget];
                            // Adding formwidgets to the context for inlineinstance widget when fieldDefs were ready
                            if (col[widget].context) {
                                col[widget].context.formWidgets = _this.formWidgets;
                            }
                            _this.setDisabledOnField(operation, col, widget);
                        }
                    });
                    _this.invokeEventCallback('formrender', { $event: e, formWidgets: _this.formWidgets, $operation: operation });
                }, 250);
            },
            onBeforeFormRender: function (row, e, operation) {
                return _this.invokeEventCallback('beforeformrender', { $event: e, row: row, $operation: operation });
            },
            registerRowNgClassWatcher: function (rowData, index) {
                if (!_this.rowngclass) {
                    return;
                }
                var row = _this.getClonedRowObject(rowData);
                var watchName = _this.widgetId + "_rowNgClass_" + index;
                $unwatch(watchName);
                _this.registerDestroyListener($watch(_this.rowngclass, _this.viewParent, { row: row }, function (nv, ov) {
                    _this.callDataGridMethod('applyRowNgClass', getConditionalClasses(nv, ov), index);
                }, watchName));
            },
            registerColNgClassWatcher: function (rowData, colDef, rowIndex, colIndex) {
                if (!colDef['col-ng-class']) {
                    return;
                }
                var row = _this.getClonedRowObject(rowData);
                var watchName = _this.widgetId + "_colNgClass_" + rowIndex + "_" + colIndex;
                $unwatch(watchName);
                _this.registerDestroyListener($watch(colDef['col-ng-class'], _this.viewParent, { row: row }, function (nv, ov) {
                    _this.callDataGridMethod('applyColNgClass', getConditionalClasses(nv, ov), rowIndex, colIndex);
                }, watchName));
            },
            clearCustomExpression: function () {
                _this.customExprViewRef.clear();
                _this.customExprCompiledTl = {};
                _this.customExprCompiledSummaryTl = {};
            },
            clearRowDetailExpression: function () {
                _this.rowDetailViewRef.clear();
                _this.rowDefMap = {};
                _this.rowDefInstances = {};
            },
            generateCustomExpressions: function (rowData, index, summaryRow) {
                var row = _this.getClonedRowObject(rowData);
                var compileTemplate = function (tmpl) {
                    if (!tmpl) {
                        return;
                    }
                    var colDef = {};
                    var context = {
                        row: row,
                        colDef: colDef
                    };
                    _this.addEventsToContext(context);
                    var customExprView = _this.customExprViewRef.createEmbeddedView(tmpl, context);
                    var rootNode = customExprView.rootNodes[0];
                    var fieldName = rootNode.getAttribute('data-col-identifier');
                    _.extend(colDef, _this.columns[fieldName]);
                    if (!summaryRow) {
                        _this.customExprCompiledTl[fieldName + index] = rootNode;
                    }
                    else {
                        _this.customExprCompiledSummaryTl[fieldName + index] = rootNode;
                    }
                };
                if (_this.isdynamictable) {
                    _this.fieldDefs.forEach(function (col) {
                        compileTemplate(col.customExprTmpl);
                    });
                    return;
                }
                // For all the columns inside the table, generate the custom expression
                _this.customExprTmpl.forEach(compileTemplate.bind(_this));
            },
            generateRowExpansionCell: function (rowData, index) {
                var row = _this.getClonedRowObject(rowData);
                // For all the columns inside the table, generate the inline widget
                _this.rowExpansionActionTmpl.forEach(function (tmpl) {
                    _this.rowExpansionActionTl[index] = _this.rowExpansionActionViewRef.createEmbeddedView(tmpl, { row: row }).rootNodes;
                });
            },
            getRowExpansionAction: function (index) {
                return _this.rowExpansionActionTl[index];
            },
            generateRowDetailView: function ($event, rowData, rowId, $target, $overlay, callback) {
                var row = _this.getClonedRowObject(rowData);
                var rowDef = getClonedObject(_this.rowDef);
                if (_this.rowInstance.invokeEventCallback('beforerowexpand', { $event: $event, $data: rowDef, row: row }) === false) {
                    return;
                }
                if (!rowDef.content) {
                    return;
                }
                // Expand the row detail
                callback();
                // Row is already rendered. Return here
                if (_this.rowDefMap[rowId] && _this.rowDefMap[rowId].content === rowDef.content) {
                    _this.rowInstance.invokeEventCallback('rowexpand', { $event: $event, row: row, $data: _this.rowDefInstances[rowId] });
                    return;
                }
                _this.rowDefMap[rowId] = rowDef;
                $target.empty();
                $target.hide();
                $overlay.show();
                var context = {
                    row: row,
                    rowDef: rowDef,
                    containerLoad: function (widget) {
                        setTimeout(function () {
                            $overlay.hide();
                            $target.show();
                            _this.rowDefInstances[rowId] = widget;
                            _this.rowInstance.invokeEventCallback('rowexpand', { $event: $event, row: row, $data: widget });
                        }, 500);
                    }
                };
                var rootNode = _this.rowDetailViewRef.createEmbeddedView(_this.rowExpansionTmpl, context).rootNodes[0];
                $target[0].appendChild(rootNode);
                $appDigest();
            },
            onBeforeRowCollapse: function ($event, row, rowId) {
                return _this.rowInstance.invokeEventCallback('beforerowcollapse', { $event: $event, row: row, $data: _this.rowDefInstances[rowId] });
            },
            onRowCollapse: function ($event, row) {
                _this.rowInstance.invokeEventCallback('rowcollapse', { $event: $event, row: row });
            },
            getCustomExpression: function (fieldName, index, summaryRow) {
                var customExpression;
                if (!summaryRow) {
                    customExpression = _this.customExprCompiledTl[fieldName + index];
                }
                else {
                    customExpression = _this.customExprCompiledSummaryTl[fieldName + index];
                }
                return customExpression || '';
            },
            clearRowActions: function () {
                _this.rowActionsViewRef.clear();
                _this.rowActionsCompiledTl = {};
                _this.rowExpansionActionViewRef.clear();
                _this.rowExpansionActionTl = {};
            },
            generateRowActions: function (rowData, index) {
                var row = _this.getClonedRowObject(rowData);
                _this.rowActionsCompiledTl[index] = [];
                // For all the columns inside the table, generate the inline widget
                _this.rowActionTmpl.forEach(function (tmpl) {
                    var _a;
                    (_a = _this.rowActionsCompiledTl[index]).push.apply(_a, __spread(_this.rowActionsViewRef.createEmbeddedView(tmpl, { row: row }).rootNodes));
                });
            },
            getRowAction: function (index) {
                return _this.rowActionsCompiledTl[index];
            },
            generateInlineEditRow: function (rowData, alwaysNewRow) {
                var row = _this.getClonedRowObject(rowData);
                if (alwaysNewRow) {
                    // Clear the view container ref
                    _this.inlineEditNewViewRef.clear();
                    _this.inlineNewCompliedTl = {};
                    // For all the columns inside the table, generate the inline widget
                    _this.inlineWidgetNewTmpl.forEach(function (tmpl) {
                        var fieldName;
                        var context = {
                            row: row,
                            getControl: function () {
                                return _this.ngform.controls[fieldName + '_new'] || {};
                            },
                            getValidationMessage: function () {
                                return _this.columns[fieldName] && _this.columns[fieldName].validationmessage;
                            },
                            getPendingSpinnerStatusNew: function () {
                                return _this.columns[fieldName] && _this.columns[fieldName].showPendingSpinnerNew;
                            }
                        };
                        var rootNode = _this.inlineEditNewViewRef.createEmbeddedView(tmpl, context).rootNodes[0];
                        fieldName = rootNode.getAttribute('data-col-identifier');
                        _this.inlineNewCompliedTl[fieldName] = rootNode;
                    });
                    _this.clearForm(true);
                    return;
                }
                // Clear the view container ref
                _this.inlineEditViewRef.clear();
                _this.inlineCompliedTl = {};
                _this.clearForm();
                // For all the columns inside the table, generate the inline widget
                _this.inlineWidgetTmpl.forEach(function (tmpl) {
                    var fieldName;
                    var context = {
                        row: row,
                        getControl: function () {
                            return _this.ngform.controls[fieldName] || {};
                        },
                        getValidationMessage: function () {
                            return _this.columns[fieldName] && _this.columns[fieldName].validationmessage;
                        },
                        getPendingSpinnerStatus: function () {
                            return _this.columns[fieldName] && _this.columns[fieldName].showPendingSpinner;
                        }
                    };
                    var rootNode = _this.inlineEditViewRef.createEmbeddedView(tmpl, context).rootNodes[0];
                    fieldName = rootNode.getAttribute('data-col-identifier');
                    _this.inlineCompliedTl[fieldName] = rootNode;
                });
            },
            getInlineEditWidget: function (fieldName, value, alwaysNewRow) {
                if (alwaysNewRow) {
                    _this.gridOptions.setFieldValue(fieldName + '_new', value);
                    return _this.inlineNewCompliedTl[fieldName];
                }
                _this.gridOptions.setFieldValue(fieldName, value);
                return _this.inlineCompliedTl[fieldName];
            },
            setFieldValue: function (fieldName, value) {
                var control = _this.ngform.controls && _this.ngform.controls[fieldName];
                if (control) {
                    control.setValue(value);
                }
            },
            getFieldValue: function (fieldName) {
                var control = _this.ngform.controls && _this.ngform.controls[fieldName];
                return control && control.value;
            },
            generateFilterRow: function () {
                // Clear the view container ref
                _this.filterViewRef.clear();
                _this.rowFilterCompliedTl = {};
                // For all the columns inside the table, generate the compiled filter template
                _this.filterTmpl.forEach(function (tmpl) {
                    var rootNode = _this.filterViewRef.createEmbeddedView(tmpl, {
                        changeFn: _this.onRowFilterChange.bind(_this),
                        isDisabled: function (fieldName) {
                            return _this.emptyMatchModes.indexOf(_this.rowFilter[fieldName] && _this.rowFilter[fieldName].matchMode) > -1;
                        }
                    }).rootNodes[0];
                    _this.rowFilterCompliedTl[rootNode.getAttribute('data-col-identifier')] = rootNode;
                });
            },
            getFilterWidget: function (fieldName) {
                // Move the generated filter template to the filter row
                return _this.rowFilterCompliedTl[fieldName];
            },
            setGridEditMode: function (val) {
                _this.ngZone.run(function () {
                    _this.isGridEditMode = val;
                    $appDigest();
                });
            },
            setGridState: function (val) {
                _this.isLoading = val === 'loading';
            },
            noChangesDetected: function () {
                _this.toggleMessage(true, 'info', 'No Changes Detected', '');
            },
            // Function to redraw the widgets on resize of columns
            redrawWidgets: function () {
                _this.fieldDefs.forEach(function (col) {
                    triggerFn(col.inlineInstance && col.inlineInstance.redraw);
                    triggerFn(col.inlineInstanceNew && col.inlineInstanceNew.redraw);
                    triggerFn(col.filterInstance && col.filterInstance.redraw);
                });
            },
            searchHandler: _this.searchSortHandler.bind(_this),
            sortHandler: _this.searchSortHandler.bind(_this),
            timeoutCall: function (fn, delay) {
                setTimeout(fn, delay);
            },
            runInNgZone: function (fn) {
                _this.ngZone.run(fn);
            },
            safeApply: function () {
                $appDigest();
            },
            setTouched: function (name) {
                var ctrl = _this.ngform.controls[name];
                if (ctrl) {
                    ctrl.markAsTouched();
                }
            },
            clearForm: _this.clearForm.bind(_this),
            callLoadInlineWidgetData: function () {
                _this.fullFieldDefs.forEach(function (col) {
                    if (_.isUndefined(col.isDataSetBound)) {
                        triggerFn(col.loadInlineWidgetData && col.loadInlineWidgetData.bind(col));
                    }
                });
            },
            // Function to apply validators to Inline form controls
            applyValidations: function (col, alwaysNewRow) {
                if (alwaysNewRow) {
                    triggerFn(col.applyNewRowValidations && col.applyNewRowValidations.bind(col));
                }
                else {
                    triggerFn(col.applyValidations && col.applyValidations.bind(col));
                }
            },
            // Function to remove validators and set form state to untouched for inline form control
            removeValidations: function (alwaysNewRow) {
                _this.fullFieldDefs.forEach(function (col) {
                    if (!col.readonly && col.binding !== 'rowOperations') {
                        if (alwaysNewRow) {
                            triggerFn(col.removeNewRowValidations && col.removeNewRowValidations.bind(col));
                        }
                        else {
                            triggerFn(col.removeValidations && col.removeValidations.bind(col));
                        }
                    }
                });
            }
        };
        _this._selectedItemsExist = false;
        _this._onChange = function () { };
        _this._onTouched = function () { };
        styler(_this.nativeElement, _this);
        _this.ngform = fb.group({});
        _this.addEventsToContext(_this.context);
        var listenersToRemove = [
            // Updates pagination, filter, sort etc options for service and crud variables
            _this.app.subscribe('check-state-persistence-options', function (options) {
                var dataSourceName = _.get(_this.datasource, 'name');
                // in Prefabs, this.datasource is not resolved at the time of variable invocation, so additional check is required.
                if (!dataSourceName) {
                    dataSourceName = extractDataSourceName(_this.binddatasource);
                }
                if (_.get(options, 'variable.name') !== dataSourceName) {
                    return;
                }
                if (_this._pageLoad && _this.getConfiguredState() !== 'none') {
                    _this._pageLoad = false;
                    var widgetState = _this.statePersistence.getWidgetState(_this);
                    if (widgetState) {
                        options = _this.handleStateParams(widgetState, options);
                    }
                    else {
                        _this.setDataGridOption('selectFirstRow', _this.gridfirstrowselect);
                    }
                }
            }),
            // Show loading status based on the variable life cycle
            _this.app.subscribe('toggle-variable-state', function (options) {
                if (_this.datasource && _this.datasource.execute(DataSource.Operation.IS_API_AWARE) && isDataSourceEqual(options.variable, _this.datasource)) {
                    if (_this._pageLoad && _this.getConfiguredState() !== 'none') {
                        _this._pageLoad = false;
                        var widgetState = _this.statePersistence.getWidgetState(_this);
                        if (widgetState) {
                            options = _this.handleStateParams(widgetState, options);
                        }
                        else {
                            _this.setDataGridOption('selectFirstRow', _this.gridfirstrowselect);
                        }
                    }
                    isDefined(_this.variableInflight) ? _this.debouncedHandleLoading(options) : _this.handleLoading(options);
                }
            }),
            _this.app.subscribe('setup-cud-listener', function (param) {
                if (_this.name !== param) {
                    return;
                }
                _this._isDependent = true;
                _this.selectedItemChange$
                    .pipe(debounceTime(250))
                    .subscribe(_this.triggerWMEvent.bind(_this));
            })
        ];
        listenersToRemove.forEach(function (l) { return _this.registerDestroyListener(l); });
        _this.deleteoktext = _this.appLocale.LABEL_OK;
        _this.deletecanceltext = _this.appLocale.LABEL_CANCEL;
        return _this;
    }
    TableComponent_1 = TableComponent;
    Object.defineProperty(TableComponent.prototype, "gridData", {
        get: function () {
            return this._gridData || [];
        },
        set: function (newValue) {
            this._gridData = newValue;
            var startRowIndex = 0;
            var gridOptions;
            this._onChange(newValue);
            this._onTouched();
            if (isDefined(newValue)) {
                /*Setting the serial no's only when show navigation is enabled and data navigator is compiled
                 and its current page is set properly*/
                if (this.isNavigationEnabled() && this.dataNavigator.dn.currentPage) {
                    startRowIndex = ((this.dataNavigator.dn.currentPage - 1) * (this.dataNavigator.maxResults || 1)) + 1;
                    this.setDataGridOption('startRowIndex', startRowIndex);
                }
                /* If colDefs are available, but not already set on the datagrid, then set them.
                 * This will happen while switching from markup to design tab. */
                gridOptions = this.callDataGridMethod('getOptions');
                if (!gridOptions) {
                    return;
                }
                if (!gridOptions.colDefs.length && this.fieldDefs.length) {
                    this.setDataGridOption('colDefs', getClonedObject(this.fieldDefs));
                }
                // If data and colDefs are present, call on before data render event
                if (!this.isdynamictable && !_.isEmpty(newValue) && gridOptions.colDefs.length) {
                    this.invokeEventCallback('beforedatarender', { $data: newValue, $columns: this.columns, data: newValue, columns: this.columns });
                }
                this.setDataGridOption('data', getClonedObject(newValue));
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TableComponent.prototype, "selecteditem", {
        get: function () {
            if (this.multiselect) {
                return getClonedObject(this.items);
            }
            if (_.isEmpty(this.items)) {
                return {};
            }
            return getClonedObject(this.items[0]);
        },
        set: function (val) {
            // Select the rows in the table based on the new selected items passed
            this.items.length = 0;
            this.callDataGridMethod('selectRows', val);
        },
        enumerable: true,
        configurable: true
    });
    TableComponent.prototype.onKeyPress = function ($event) {
        if ($event.which === 13) {
            this.invokeEventCallback('enterkeypress', { $event: $event, $data: this.gridData });
        }
    };
    TableComponent.prototype.getConfiguredState = function () {
        var mode = this.statePersistence.computeMode(this.statehandler);
        return mode && mode.toLowerCase();
    };
    TableComponent.prototype.handleStateParams = function (widgetState, options) {
        var _this = this;
        if (_.get(widgetState, 'selectedItem')) {
            this._selectedItemsExist = true;
        }
        options.options = options.options || {};
        if (_.get(widgetState, 'pagination')) {
            options.options.page = widgetState.pagination;
            if (_.get(widgetState, 'sort')) {
                this.sortStateHandler(widgetState);
                options.options.orderBy = _.get(widgetState, 'sort.field') + ' ' + _.get(widgetState, 'sort.direction');
            }
            if (_.get(widgetState, 'search')) {
                setTimeout(function () {
                    _this.searchStateHandler(widgetState);
                }, 500);
                options.options.filterFields = this.getFilterFields(widgetState.search);
            }
        }
        else {
            options.options.page = 1;
            if (_.get(widgetState, 'search')) {
                setTimeout(function () {
                    _this.searchStateHandler(widgetState);
                }, 500);
                options.options.filterFields = this.getFilterFields(widgetState.search);
            }
            if (_.get(widgetState, 'sort')) {
                this.sortStateHandler(widgetState);
                options.options.orderBy = _.get(widgetState, 'sort.field') + ' ' + _.get(widgetState, 'sort.direction');
            }
        }
        return options;
    };
    TableComponent.prototype.triggerWMEvent = function (newVal) {
        if (this.editmode === 'dialog') {
            return;
        }
        $invokeWatchers(true);
        this.app.notify('wm-event', { eventName: 'selectedItemChange', widgetName: this.name, row: newVal, table: this });
    };
    TableComponent.prototype.sortStateHandler = function (widgetState) {
        var $gridElement = this.datagridElement;
        var $sortIcon = $gridElement.find('th[data-col-field="' + _.get(widgetState, 'sort.field') + '"] .sort-icon');
        if (_.get(widgetState, 'sort.direction') === 'asc' && $sortIcon.length) {
            $sortIcon.addClass('asc wi wi-long-arrow-up');
        }
        else if (_.get(widgetState, 'sort.direction') === 'desc' && $sortIcon.length) {
            $sortIcon.addClass('desc wi wi-long-arrow-down');
        }
    };
    TableComponent.prototype.searchStateHandler = function (widgetState) {
        var _this = this;
        if (_.isArray(widgetState.search)) {
            _.forEach(widgetState.search, function (filterObj) {
                if (_this.rowFilter[filterObj.field]) {
                    _this.rowFilter[filterObj.field].value = filterObj.value;
                    _this.rowFilter[filterObj.field].matchMode = filterObj.matchMode;
                    _this.rowFilter[filterObj.field].type = filterObj.type;
                    if ($(_this.rowFilterCompliedTl[filterObj.field]).length) {
                        var val = filterObj.type === 'integer' ? parseInt(filterObj.value) : filterObj.value;
                        $(_this.rowFilterCompliedTl[filterObj.field]).find('input').val(filterObj.value);
                    }
                }
            });
        }
        else {
            var $gridElement = this.datagridElement;
            $gridElement.find('[data-element="dgSearchText"]').val(widgetState.search.value);
            $gridElement.find('[data-element="dgFilterValue"]').val(widgetState.search.field);
        }
    };
    // removes empty values in headerConfig array.
    TableComponent.prototype.filterEmptyValues = function (item) {
        if (item.columns) {
            item.columns = item.columns.filter(this.filterEmptyValues.bind(this));
        }
        return item !== undefined;
    };
    TableComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        _super.prototype.ngAfterContentInit.call(this);
        this.headerConfig = this.headerConfig.filter(this.filterEmptyValues.bind(this));
        _.remove(this.fieldDefs, function (f) { return f === undefined; });
        var runModeInitialProperties = {
            showrowindex: 'showRowIndex',
            multiselect: 'multiselect',
            radioselect: 'showRadioColumn',
            filternullrecords: 'filterNullRecords',
            enablesort: 'enableSort',
            showheader: 'showHeader',
            enablecolumnselection: 'enableColumnSelection',
            shownewrow: 'showNewRow',
            gridfirstrowselect: 'selectFirstRow'
        };
        if (this._liveTableParent) {
            this.isPartOfLiveGrid = true;
        }
        if (this.readonlygrid || !this.editmode) {
            if (this.readonlygrid === 'true') {
                this.editmode = '';
            }
            else {
                if (this.isPartOfLiveGrid) {
                    this.editmode = this.isPartOfLiveGrid.formlayout === 'inline' ? EDIT_MODE.FORM : EDIT_MODE.DIALOG;
                }
                else {
                    this.editmode = this.readonlygrid ? EDIT_MODE.INLINE : '';
                }
            }
        }
        this.gridOptions.colDefs = this.fullFieldDefs;
        this.gridOptions.rowActions = this.rowActions;
        this.gridOptions.headerConfig = this.headerConfig;
        this.gridOptions.rowClass = this.rowclass;
        this.gridOptions.editmode = this.editmode;
        this.gridOptions.formPosition = this.formposition;
        this.gridOptions.filtermode = this.filtermode;
        this.gridOptions.searchLabel = this.searchlabel;
        this.gridOptions.isMobile = isMobile();
        this.gridOptions.name = this.name;
        this.gridOptions.securityUtils.pipeTransform = this.trustAsPipe;
        // When loadondemand property is enabled(deferload="true") and show is true, only the column titles of the datatable are rendered, the data(body of the datatable) is not at all rendered.
        // Because the griddata is setting before the datatable dom is rendered but we are sending empty data to the datatable.
        if (!_.isEmpty(this.gridData)) {
            this.gridOptions.data = getClonedObject(this.gridData);
        }
        this.gridOptions.messages = {
            'selectField': this.appLocale.MESSAGE_SELECT_FIELD
        };
        this.datagridElement = $(this._tableElement.nativeElement);
        this.gridElement = this.$element;
        this.$element.css({ 'position': 'relative' });
        _.forEach(runModeInitialProperties, function (value, key) {
            if (isDefined(_this[key])) {
                _this.gridOptions[value] = (_this[key] === 'true' || _this[key] === true);
            }
        });
        if (this.statehandler !== 'none') {
            this.gridOptions['selectFirstRow'] = false;
        }
        this.renderOperationColumns();
        this.gridOptions.colDefs = this.fieldDefs;
        this.datagridElement.datatable(this.gridOptions);
        this.callDataGridMethod('setStatus', 'loading', this.loadingdatamsg);
        this.applyProps.forEach(function (args) { return _this.callDataGridMethod.apply(_this, __spread(args)); });
        if (this.editmode === EDIT_MODE.QUICK_EDIT) {
            this.documentClickBind = this._documentClickBind.bind(this);
            document.addEventListener('click', this.documentClickBind);
        }
    };
    TableComponent.prototype.ngOnDestroy = function () {
        document.removeEventListener('click', this.documentClickBind);
        if (this.navigatorResultWatch) {
            this.navigatorResultWatch.unsubscribe();
        }
        if (this.navigatorMaxResultWatch) {
            this.navigatorMaxResultWatch.unsubscribe();
        }
        _super.prototype.ngOnDestroy.call(this);
    };
    TableComponent.prototype.addRowIndex = function (row) {
        var rowData = getClonedObject(row);
        var rowIndex = _.indexOf(this.gridOptions.data, row);
        if (rowIndex < 0) {
            return row;
        }
        rowData.$index = rowIndex + 1;
        rowData.$isFirst = rowData.$index === 1;
        rowData.$isLast = this.gridData.length === rowData.$index;
        return rowData;
    };
    TableComponent.prototype.addEventsToContext = function (context) {
        var _this = this;
        context.addNewRow = function () { return _this.addNewRow(); };
        context.deleteRow = function () { return _this.deleteRow(); };
        context.editRow = function () { return _this.editRow(); };
    };
    TableComponent.prototype.execute = function (operation, options) {
        if ([DataSource.Operation.IS_API_AWARE, DataSource.Operation.IS_PAGEABLE, DataSource.Operation.SUPPORTS_SERVER_FILTER].includes(operation)) {
            return false;
        }
        return this.datasource ? this.datasource.execute(operation, options) : {};
    };
    TableComponent.prototype.isNavigationEnabled = function () {
        return this.shownavigation && this.dataNavigator && this.dataNavigatorWatched;
    };
    TableComponent.prototype.getClonedRowObject = function (rowData) {
        var row = getClonedObject(rowData);
        row.getProperty = function (field) {
            return _.get(row, field);
        };
        row.$isFirst = row.$index === 1;
        row.$isLast = this.gridData.length === row.$index;
        delete row.$$index;
        delete row.$$pk;
        return row;
    };
    TableComponent.prototype.handleLoading = function (data) {
        this.variableInflight = data.active;
        // based on the active state and response toggling the 'loading data...' and 'no data found' messages.
        if (data.active) {
            this.callDataGridMethod('setStatus', 'loading', this.loadingdatamsg);
        }
        else {
            // If grid is in edit mode or grid has data, dont show the no data message
            if (!this.isGridEditMode && _.isEmpty(this.dataset)) {
                this.callDataGridMethod('setStatus', 'nodata', this.nodatamessage);
            }
            else {
                this.callDataGridMethod('setStatus', 'ready');
            }
        }
    };
    TableComponent.prototype.setDisabledOnField = function (operation, colDef, widgetType) {
        if (operation !== 'new' && colDef['primary-key'] && colDef.generator === 'assigned' && !colDef['related-entity-name'] && !colDef.period) {
            colDef[widgetType].disabled = true;
        }
    };
    TableComponent.prototype.resetFormControl = function (ctrl) {
        ctrl.markAsUntouched();
        ctrl.markAsPristine();
    };
    TableComponent.prototype.clearForm = function (newRow) {
        var _this = this;
        var ctrls = this.ngform.controls;
        _.keys(this.ngform.controls).forEach(function (key) {
            // If new row, clear the controls in the new row. Else, clear the controls in edit row
            if (!key.endsWith('_filter') && ((key.endsWith('_new') && newRow) || (!key.endsWith('_new') && !newRow))) {
                ctrls[key].setValue('');
                _this.resetFormControl(ctrls[key]);
            }
        });
    };
    TableComponent.prototype.showFieldBasedOnScreenType = function (field) {
        var showField;
        if (isMobile() && this.viewport.isMobileType) {
            showField = field.mobileDisplay;
        }
        else if (this.viewport.isTabletType) {
            showField = field.tabletDisplay;
        }
        else {
            showField = field.pcDisplay;
        }
        return showField;
    };
    /* Check whether it is non-empty row. */
    TableComponent.prototype.isEmptyRecord = function (record) {
        var _this = this;
        var properties = Object.keys(record);
        var data, isDisplayed;
        return properties.every(function (prop, index) {
            data = record[prop];
            /* If fieldDefs are missing, show all columns in data. */
            isDisplayed = (_this.fieldDefs.length && isDefined(_this.fieldDefs[index]) && _this.showFieldBasedOnScreenType(_this.fieldDefs[index])) || true;
            /*Validating only the displayed fields*/
            if (isDisplayed) {
                return (data === null || data === undefined || data === '');
            }
            return true;
        });
    };
    /* Function to remove the empty data. */
    TableComponent.prototype.removeEmptyRecords = function (serviceData) {
        var _this = this;
        var allRecords = serviceData;
        var filteredData = [];
        if (allRecords && allRecords.length) {
            /*Comparing and pushing the non-empty data columns*/
            filteredData = allRecords.filter(function (record) {
                return record && !_this.isEmptyRecord(record);
            });
        }
        return filteredData;
    };
    TableComponent.prototype.setGridData = function (serverData) {
        var data = this.filternullrecords ? this.removeEmptyRecords(serverData) : serverData;
        if (!this.variableInflight) {
            if (data && data.length === 0) {
                this.callDataGridMethod('setStatus', 'nodata', this.nodatamessage);
            }
            else {
                this.callDataGridMethod('setStatus', 'ready');
            }
        }
        this.gridData = data;
    };
    TableComponent.prototype.setDataGridOption = function (optionName, newVal, forceSet) {
        if (forceSet === void 0) { forceSet = false; }
        if (!this.datagridElement || !this.datagridElement.datatable || !this.datagridElement.datatable('instance')) {
            return;
        }
        var option = {};
        if (isDefined && (!_.isEqual(newVal, this.gridOptions[optionName]) || forceSet)) {
            option[optionName] = newVal;
            this.datagridElement.datatable('option', option);
            this.gridOptions[optionName] = newVal;
        }
    };
    TableComponent.prototype.callDataGridMethod = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!this.datagridElement || !this.datagridElement.datatable('instance')) {
            this.applyProps.set(args[1], args);
            return; // If datagrid is not initiliazed or destroyed, return here
        }
        return this.datagridElement.datatable.apply(this.datagridElement, args);
    };
    TableComponent.prototype.renderOperationColumns = function () {
        var rowActionCol, insertPosition;
        var rowOperationsColumn = getRowOperationsColumn(), config = {
            'name': rowOperationsColumn.field,
            'field': rowOperationsColumn.field,
            'isPredefined': true
        };
        // Return if no fieldDefs are present
        if (!this.fieldDefs.length) {
            return;
        }
        rowActionCol = _.find(this.fullFieldDefs, { 'field': ROW_OPS_FIELD, type: 'custom' }); // Check if column is fetched from markup
        _.remove(this.fieldDefs, { type: 'custom', field: ROW_OPS_FIELD }); // Removing operations column
        _.remove(this.headerConfig, { field: rowOperationsColumn.field });
        /*Add the column for row operations only if at-least one operation has been enabled.*/
        if (this.rowActions.length) {
            if (rowActionCol) { // If column is present in markup, push the column or push the default column
                insertPosition = rowActionCol.rowactionsposition ? _.toNumber(rowActionCol.rowactionsposition) : this.fieldDefs.length;
                this.fieldDefs.splice(insertPosition, 0, rowActionCol);
                if (insertPosition === 0) {
                    this.headerConfig.unshift(config);
                }
                else {
                    this.headerConfig.push(config);
                }
            }
            else {
                this.fieldDefs.push(rowOperationsColumn);
                this.headerConfig.push(config);
            }
        }
        this.setDataGridOption('headerConfig', this.headerConfig);
    };
    TableComponent.prototype.enablePageNavigation = function () {
        var _this = this;
        if (this.dataset && this.binddataset && this.dataNavigator) {
            /*Check for sanity*/
            this.dataNavigatorWatched = true;
            if (this.navigatorResultWatch) {
                this.navigatorResultWatch.unsubscribe();
            }
            /*Register a watch on the "result" property of the "dataNavigator" so that the paginated data is displayed in the live-list.*/
            this.navigatorResultWatch = this.dataNavigator.resultEmitter.subscribe(function (newVal) {
                /* Check for sanity. */
                if (isDefined(newVal)) {
                    // Watch will not be triggered if dataset and new value are equal. So trigger the property change handler manually
                    // This happens in case, if dataset is directly updated.
                    if (_.isEqual(_this.dataset, newVal)) {
                        _this.watchVariableDataSet(newVal);
                    }
                    else {
                        if (_.isArray(newVal)) {
                            _this.widget.dataset = [].concat(newVal);
                        }
                        else if (_.isObject(newVal)) {
                            _this.widget.dataset = _.extend({}, newVal);
                        }
                        else {
                            _this.widget.dataset = newVal;
                        }
                    }
                }
                else {
                    _this.widget.dataset = undefined;
                }
            }, true);
            /*De-register the watch if it is exists */
            if (this.navigatorMaxResultWatch) {
                this.navigatorMaxResultWatch.unsubscribe();
            }
            /*Register a watch on the "maxResults" property of the "dataNavigator" so that the "pageSize" is displayed in the live-list.*/
            this.navigatorMaxResultWatch = this.dataNavigator.maxResultsEmitter.subscribe(function (newVal) {
                _this.pagesize = newVal;
            });
            // If dataset is a pageable object, data is present inside the content property
            this.__fullData = this.dataset;
            this.dataNavigator.widget.maxResults = this.pagesize || 5;
            this.dataNavigator.options = {
                maxResults: this.pagesize || 5
            };
            this.removePropertyBinding('dataset');
            this.dataNavigator.setBindDataSet(this.binddataset, this.viewParent, this.datasource, this.dataset, this.binddatasource, undefined, this.statehandler);
        }
    };
    TableComponent.prototype.resetPageNavigation = function () {
        /*Check for sanity*/
        if (this.dataNavigator) {
            this.dataNavigator.resetPageNavigation();
        }
    };
    TableComponent.prototype.isDataValid = function () {
        var error;
        var dataset = this.dataset || {};
        /*In case "data" contains "error" & "errorMessage", then display the error message in the grid.*/
        if (dataset.error) {
            error = dataset.error;
        }
        if (dataset.data && dataset.data.error) {
            if (dataset.data.errorMessage) {
                error = dataset.data.errorMessage;
            }
        }
        if (error) {
            this.setGridData([]);
            this.callDataGridMethod('setStatus', 'error', error);
            return false;
        }
        return true;
    };
    // Function to populate the grid with data.
    TableComponent.prototype.populateGridData = function (serviceData) {
        var _this = this;
        var data;
        serviceData = transformData(serviceData, this.name);
        // Apply filter and sort, if data is refreshed through Refresh data method
        if (!this.isNavigationEnabled() && this._isClientSearch) {
            data = getClonedObject(serviceData);
            data = this.getSearchResult(data, this.filterInfo);
            data = this.getSortResult(data, this.sortInfo);
            this.serverData = data;
        }
        else {
            this.serverData = serviceData;
        }
        // If fielddefs are not present, generate fielddefs from data
        // Removing fielddefs check because When loadondemand property is enabled(deferload="true"), the dataset propertychangehanlder is triggered first before the dom is getting rendered.
        // So at that time fielddefs length is zero, due to this the columns are created dynamically.
        if (this.isdynamictable) {
            this.createGridColumns(this.serverData);
        }
        else {
            this.setGridData(this.serverData);
        }
        if (this.getConfiguredState() !== 'none' && this._selectedItemsExist && serviceData.length) {
            var widgetState = this.statePersistence.getWidgetState(this);
            var currentPageItems = void 0;
            if (_.get(widgetState, 'selectedItem')) {
                currentPageItems = widgetState.selectedItem.filter(function (val) {
                    return val.page === _this.dataNavigator.dn.currentPage;
                });
                this._selectedItemsExist = false;
                // if an item is already selected, don't trigger onSelect event for it again
                if (currentPageItems.length && this.selecteditem.length !== widgetState.selectedItem.length) {
                    // don't reassign this.selecteditem if selected items already exist.
                    if (_.isArray(this.selecteditem)) {
                        currentPageItems.forEach(function (item) {
                            _this.selectItem(item.index, undefined);
                        });
                    }
                    else {
                        this.selecteditem = currentPageItems.map(function (val) { return val.index; });
                    }
                }
            }
        }
    };
    // Function to generate and compile the form fields from the metadata
    TableComponent.prototype.generateDynamicColumns = function (columns) {
        return __awaiter(this, void 0, void 0, function () {
            var tmpl, componentFactoryRef, component;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.fieldDefs = []; // empty the form fields
                        // empty the filter field template refs.
                        this.filterTmpl._results = [];
                        if (_.isEmpty(columns)) {
                            return [2 /*return*/];
                        }
                        tmpl = '';
                        columns.forEach(function (col) {
                            var attrsTmpl = '';
                            var customTmpl = '';
                            _.forEach(col, function (val, key) {
                                if (!_.isUndefined(val) && val !== '') {
                                    // If custom expression is present, keep it inside table column. Else, keep as attribute
                                    if (key === 'customExpression') {
                                        customTmpl = val;
                                    }
                                    else {
                                        attrsTmpl += " " + key + "=\"" + val + "\"";
                                    }
                                }
                            });
                            tmpl += "<wm-table-column " + attrsTmpl + " tableName=\"" + _this.name + "\">" + customTmpl + "</wm-table-column>";
                        });
                        this.dynamicTableRef.clear();
                        if (!this._dynamicContext) {
                            this._dynamicContext = Object.create(this.viewParent);
                            this._dynamicContext[this.getAttr('wmTable')] = this;
                        }
                        this.noOfColumns = columns.length;
                        return [4 /*yield*/, this.dynamicComponentProvider.getComponentFactoryRef('app-table-dynamic-' + this.widgetId, tmpl, {
                                noCache: true,
                                transpile: true
                            })];
                    case 1:
                        componentFactoryRef = _a.sent();
                        component = this.dynamicTableRef.createComponent(componentFactoryRef, 0, this.inj);
                        extendProto(component.instance, this._dynamicContext);
                        this.$element.find('.dynamic-table-container')[0].appendChild(component.location.nativeElement);
                        return [2 /*return*/];
                }
            });
        });
    };
    TableComponent.prototype.prepareColDefs = function (data) {
        var _this = this;
        var defaultFieldDefs;
        var properties;
        var columnIndex = 0;
        var headerIndex = 0;
        this.fieldDefs = [];
        this.headerConfig = [];
        this.columns = {};
        /* if properties map is existed then fetch the column configuration for all nested levels using util function */
        properties = data;
        /*call utility function to prepare fieldDefs for grid against given data (A MAX OF 10 COLUMNS ONLY)*/
        defaultFieldDefs = prepareFieldDefs(properties);
        /*append additional properties*/
        _.forEach(defaultFieldDefs, function (columnDef) {
            columnDef.binding = columnDef.field;
            columnDef.caption = columnDef.displayName;
            columnDef.pcDisplay = true;
            columnDef.mobileDisplay = true;
            columnDef.tabletDisplay = true;
            columnDef.searchable = true;
            columnDef.type = 'string';
            // Fix for [WMS-19668] and [WMS-19669]- Adding index and headerIndex for Dynamic DB columns.
            // Note: we don't have column groups for Dynamic DB
            columnDef.index = columnIndex;
            columnDef.headerIndex = headerIndex;
            columnIndex++;
            headerIndex++;
        });
        defaultFieldDefs.forEach(function (col) {
            _this.columns[col.field] = col;
        });
        this.invokeEventCallback('beforedatarender', { $data: data, $columns: this.columns, data: data, columns: this.columns });
        defaultFieldDefs = [];
        // Apply the changes made by the user
        _.forEach(this.columns, function (val) {
            defaultFieldDefs.push(val);
        });
        this.generateDynamicColumns(defaultFieldDefs);
    };
    TableComponent.prototype.createGridColumns = function (data) {
        /* this call back function receives the data from the variable */
        /* check whether data is valid or not */
        var dataValid = data && !data.error;
        /*if the data is type json object, make it an array of the object*/
        if (dataValid && !_.isArray(data)) {
            data = [data];
        }
        /* if the data is empty, show nodatamessage */
        if (_.isEmpty(data)) {
            this.setGridData(data);
            return;
        }
        if (!dataValid) {
            return;
        }
        /* if new columns to be rendered, prepare default fieldDefs for the data provided*/
        this.prepareColDefs(data);
        this.serverData = data;
        this.setGridData(this.serverData);
    };
    TableComponent.prototype.getSortExpr = function () {
        var sortExp;
        var pagingOptions;
        if (this.datasource && this.datasource.execute(DataSource.Operation.IS_PAGEABLE)) {
            pagingOptions = this.datasource.execute(DataSource.Operation.GET_PAGING_OPTIONS);
            sortExp = _.isEmpty(pagingOptions) ? '' : getOrderByExpr(pagingOptions.sort);
        }
        return sortExp || '';
    };
    TableComponent.prototype.watchVariableDataSet = function (newVal) {
        var result;
        // State handling for static variables
        if (_.get(this.datasource, 'category') === 'wm.Variable' && this._pageLoad && this.getConfiguredState() !== 'none') {
            var widgetState = this.statePersistence.getWidgetState(this);
            this._pageLoad = false;
            if (_.get(widgetState, 'selectedItem')) {
                this._selectedItemsExist = true;
            }
            else {
                this.setDataGridOption('selectFirstRow', this.gridfirstrowselect);
            }
            if (_.get(widgetState, 'search')) {
                this.searchStateHandler(widgetState);
                this.searchSortHandler(widgetState.search, undefined, 'search', true);
            }
            if (_.get(widgetState, 'sort')) {
                this.searchSortHandler(widgetState.sort, undefined, 'sort', true);
                this.sortStateHandler(widgetState);
            }
            if (_.get(widgetState, 'pagination')) {
                this.dataNavigator.pageChanged({ page: widgetState.pagination }, true);
            }
        }
        // After the setting the watch on navigator, dataset is triggered with undefined. In this case, return here.
        if (this.dataNavigatorWatched && _.isUndefined(newVal) && this.__fullData) {
            return;
        }
        // If variable is in loading state, show loading icon
        if (this.variableInflight) {
            this.callDataGridMethod('setStatus', 'loading', this.loadingdatamsg);
        }
        result = getValidJSON(newVal);
        // Converting newval to object if it is an Object that comes as a string "{"data" : 1}"
        if (result) {
            newVal = result;
        }
        var sortExp = this.getSortExpr();
        var sortExpArr = sortExp.split(' ');
        if (sortExp && sortExpArr.length) {
            this.sortInfo = {
                direction: sortExpArr[1],
                field: sortExpArr[0]
            };
        }
        /*Return if data is invalid.*/
        if (!this.isDataValid()) {
            return;
        }
        // If value is empty or in studio mode, dont enable the navigation
        if (newVal) {
            if (this.shownavigation && !this.dataNavigatorWatched) {
                this.enablePageNavigation();
                return;
            }
        }
        else {
            this.resetPageNavigation();
            /*for run mode, disabling the loader and showing no data found message if dataset is not valid*/
            if (!this.variableInflight) {
                this.callDataGridMethod('setStatus', 'nodata', this.nodatamessage);
            }
            this.setDataGridOption('selectFirstRow', this.gridfirstrowselect);
        }
        if (!this.isNavigationEnabled() && newVal) {
            this.checkFiltersApplied(this.getSortExpr());
        }
        if (!_.isObject(newVal) || newVal === '' || (newVal && newVal.dataValue === '')) {
            if (!this.variableInflight) {
                // If variable has finished loading and resultSet is empty, ender empty data
                this.setGridData([]);
            }
            return;
        }
        if (newVal) {
            this.populateGridData(newVal);
        }
    };
    TableComponent.prototype.onDataSourceChange = function () {
        this.fieldDefs.forEach(function (col) {
            triggerFn(col.onDataSourceChange && col.onDataSourceChange.bind(col));
        });
    };
    TableComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        var enableNewRow;
        switch (key) {
            case 'datasource':
                this.watchVariableDataSet(this.dataset);
                this.onDataSourceChange();
                break;
            case 'dataset':
                if (this.binddatasource && !this.datasource) {
                    return;
                }
                // if table is inside list then table dataset will be set as "item.XXX" and there is no datasource.
                // So extracting datasource from the datset bound expression.
                if (this.parentList && !this.datasource && _.startsWith(this.binddataset, 'item')) {
                    this.datasource = getDatasourceFromExpr(this.widget.$attrs.get('datasetboundexpr'), this);
                }
                // for Static Variables with Retain State enabled, prevent Table from rendering more than once
                if (!(_.get(this.datasource, 'category') === 'wm.Variable' && this._pageLoad && this.getConfiguredState() !== 'none')) {
                    this.watchVariableDataSet(nv);
                }
                break;
            case 'filtermode':
                this.setDataGridOption('filtermode', nv);
                break;
            case 'searchlabel':
                this.setDataGridOption('searchLabel', nv);
                break;
            case 'navigation':
                if (nv === 'Advanced') { // Support for older projects where navigation type was advanced instead of clasic
                    this.navigation = 'Classic';
                    return;
                }
                if (nv !== 'None') {
                    this.shownavigation = true;
                }
                this.navControls = nv;
                break;
            case 'gridfirstrowselect':
                this.setDataGridOption('selectFirstRow', nv);
                break;
            case 'gridclass':
                this.callDataGridMethod('option', 'cssClassNames.grid', nv);
                break;
            case 'nodatamessage':
                this.callDataGridMethod('option', 'dataStates.nodata', nv);
                break;
            case 'loadingdatamsg':
                this.callDataGridMethod('option', 'dataStates.loading', nv);
                break;
            case 'loadingicon':
                this.callDataGridMethod('option', 'loadingicon', nv);
                break;
            case 'spacing':
                this.callDataGridMethod('option', 'spacing', nv);
                if (nv === 'condensed') {
                    this.navigationSize = 'small';
                }
                else {
                    this.navigationSize = '';
                }
                break;
            case 'exportformat':
                this.exportOptions = [];
                if (nv) {
                    // Populate options for export drop down menu
                    _.forEach(_.split(nv, ','), function (type) {
                        _this.exportOptions.push({
                            label: type,
                            icon: exportIconMapping[type]
                        });
                    });
                }
                break;
            case 'shownewrow':
                // Enable new row if shownew is true or addNewRow buton is present
                enableNewRow = nv || _.some(this.actions, function (act) { return _.includes(act.action, 'addNewRow()'); });
                this.callDataGridMethod('option', 'actionsEnabled.new', enableNewRow);
                break;
            case 'show':
                if (nv) {
                    this.invokeEventCallback('show');
                }
                else {
                    this.invokeEventCallback('hide');
                }
            default:
                _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    TableComponent.prototype.onStyleChange = function (key, nv, ov) {
        switch (key) {
            case 'width':
                this.callDataGridMethod('setGridDimensions', 'width', nv);
                break;
            case 'height':
                this.callDataGridMethod('setGridDimensions', 'height', nv);
                break;
        }
        _super.prototype.onStyleChange.call(this, key, nv, ov);
    };
    TableComponent.prototype.populateActions = function () {
        var _this = this;
        this._actions.header = [];
        this._actions.footer = [];
        _.forEach(this.actions, function (action) {
            if (_.includes(action.position, 'header')) {
                _this._actions.header.push(action);
            }
            if (_.includes(action.position, 'footer')) {
                _this._actions.footer.push(action);
            }
        });
    };
    // this method will render the filter row.
    TableComponent.prototype.renderDynamicFilterColumn = function (filteTemRef) {
        // For dynamic table manually pushing the filtertemplateRef as templateRef will not be available prior.
        if (this.isdynamictable) {
            this.filterTmpl._results.push(filteTemRef);
        }
    };
    TableComponent.prototype.registerColumns = function (tableColumn, colIndex) {
        var _this = this;
        if (isMobile() && this.viewport.isMobileType) {
            if (!tableColumn.mobileDisplay) {
                return;
            }
        }
        else if (this.viewport.isTabletType) {
            if (!tableColumn.tabletDisplay) {
                return;
            }
        }
        else {
            if (!tableColumn.pcDisplay) {
                return;
            }
        }
        if (tableColumn['primary-key']) {
            this.primaryKey.push(tableColumn.field);
        }
        this.fieldDefs[colIndex] = tableColumn;
        var colCount = this.fieldDefs.length;
        this.fullFieldDefs.push(tableColumn);
        this.rowFilter[tableColumn.field] = {
            value: undefined
        };
        this.fieldDefs.forEach(function (col) {
            _this.columns[col.field] = col;
        });
        // If dynamic datatable and last column, pass the columns to jquery datatable
        if (this.isdynamictable && colCount === this.noOfColumns) {
            this.renderOperationColumns();
            this.setDataGridOption('colDefs', this.fieldDefs);
        }
    };
    TableComponent.prototype.registerFormField = function (name, formField) {
        this.formfields[name] = formField;
    };
    TableComponent.prototype.registerActions = function (tableAction) {
        this.actions.push(tableAction);
        this.populateActions();
    };
    TableComponent.prototype.registerRow = function (tableRow, rowInstance) {
        this.rowDef = tableRow;
        this.rowInstance = rowInstance;
        this.callDataGridMethod('option', 'cssClassNames.rowExpandIcon', this.rowDef.expandicon);
        this.callDataGridMethod('option', 'cssClassNames.rowCollapseIcon', this.rowDef.collapseicon);
        this.gridOptions.rowExpansionEnabled = true;
        this.gridOptions.rowDef = this.rowDef;
    };
    TableComponent.prototype.registerRowActions = function (tableRowAction) {
        this.rowActions.push(tableRowAction);
    };
    TableComponent.prototype.selectItem = function (item, data) {
        /* server is not updating immediately, so set the server data to success callback data */
        if (data) {
            this.serverData = data;
        }
        if (_.isObject(item)) {
            item = _.omitBy(item, function (value) {
                return _.isArray(value) && _.isEmpty(value);
            });
        }
        this.callDataGridMethod('selectRow', item, true);
    };
    /* deselect the given item*/
    TableComponent.prototype.deselectItem = function (item) {
        this.callDataGridMethod('deselectRow', item);
    };
    TableComponent.prototype.onDataNavigatorDataSetChange = function (nv) {
        var data;
        this.__fullData = nv;
        this.checkFiltersApplied(this.getSortExpr());
        if (this._isClientSearch) {
            data = getClonedObject(this.__fullData);
            if (_.isObject(data) && !_.isArray(data)) {
                data = [data];
            }
            data = this.getSearchResult(data, this.filterInfo);
            data = this.getSortResult(data, this.sortInfo);
            return data;
        }
        return nv;
    };
    TableComponent.prototype.toggleMessage = function (show, type, msg, header) {
        if (show && msg) {
            this.app.notifyApp(msg, type, header);
        }
    };
    TableComponent.prototype.export = function ($item) {
        var filterFields;
        var sortOptions = _.isEmpty(this.sortInfo) ? '' : this.sortInfo.field + ' ' + this.sortInfo.direction;
        var columns = {};
        var isValid;
        var requestData;
        this.fieldDefs.forEach(function (fieldDef) {
            // Do not add the row operation actions column to the exported file.
            if (fieldDef.field === ROW_OPS_FIELD || !fieldDef.show) {
                return;
            }
            var option = {
                'header': fieldDef.displayName
            };
            // If column has exportexpression, then send form the expression as required by backend.
            // otherwise send the field name.
            if (fieldDef.exportexpression) {
                option.expression = fieldDef.exportexpression;
            }
            else {
                option.field = fieldDef.field;
            }
            columns[fieldDef.field] = option;
        });
        filterFields = this.getFilterFields(this.filterInfo);
        requestData = {
            matchMode: 'anywhereignorecase',
            filterFields: filterFields,
            orderBy: sortOptions,
            exportType: $item.label,
            logicalOp: 'AND',
            exportSize: this.exportdatasize,
            columns: columns
        };
        isValid = this.invokeEventCallback('beforeexport', { $data: requestData });
        if (isValid === false) {
            return;
        }
        requestData.fields = _.values(requestData.columns);
        this.datasource.execute(DataSource.Operation.DOWNLOAD, { data: requestData });
    };
    TableComponent.prototype.expandRow = function (rowId) {
        this.callDataGridMethod('expandRow', rowId);
    };
    TableComponent.prototype.collapseRow = function (rowId) {
        this.callDataGridMethod('collapseRow', rowId);
    };
    TableComponent.prototype._documentClickBind = function (event) {
        var $target = event.target;
        // If click triggered from same grid or a dialog, do not save the row
        if (this.$element[0].contains($target) || event.target.doctype || isInputBodyWrapper($($target))) {
            return;
        }
        this.callDataGridMethod('saveRow');
    };
    TableComponent.prototype._redraw = function (forceRender) {
        var _this = this;
        if (forceRender) {
            this.datagridElement.datatable(this.gridOptions);
        }
        else {
            setTimeout(function () {
                _this.callDataGridMethod('setColGroupWidths');
                _this.callDataGridMethod('addOrRemoveScroll');
            });
        }
    };
    TableComponent.prototype.invokeActionEvent = function ($event, expression) {
        var fn = $parseEvent(expression);
        fn(this.viewParent, Object.assign(this.context, { $event: $event }));
    };
    // change and blur events are added from the template
    TableComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName !== 'select') {
            _super.prototype.handleEvent.call(this, this.nativeElement, eventName, callback, locals);
        }
    };
    TableComponent.prototype.triggerUploadEvent = function ($event, eventName, fieldName, row) {
        var params = { $event: $event, row: row };
        if (!this.columns[fieldName]) {
            return;
        }
        if (eventName === 'change') {
            params.newVal = $event.target.files;
            params.oldVal = this.columns[fieldName]._oldUploadVal;
            this.columns[fieldName]._oldUploadVal = params.newVal;
        }
        this.columns[fieldName].invokeEventCallback(eventName, params);
    };
    TableComponent.prototype.registerFormWidget = function () { };
    // Form control accessor methods. This will be used for table inside form
    TableComponent.prototype.writeValue = function () { };
    TableComponent.prototype.registerOnChange = function (fn) {
        this._onChange = fn;
    };
    TableComponent.prototype.registerOnTouched = function (fn) {
        this._onTouched = fn;
    };
    TableComponent.prototype.ngOnDetach = function () {
        _super.prototype.ngOnDetach.call(this);
        this._pageLoad = true;
    };
    var TableComponent_1;
    TableComponent.initializeProps = registerProps();
    TableComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: FormBuilder },
        { type: App },
        { type: DynamicComponentRefProvider },
        { type: StatePersistence },
        { type: Viewport },
        { type: ListComponent, decorators: [{ type: Optional }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['datasource.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['readonlygrid',] }] },
        { type: NgZone },
        { type: TrustAsPipe }
    ]; };
    __decorate([
        ViewChild(PaginationComponent, { static: true }),
        __metadata("design:type", Object)
    ], TableComponent.prototype, "dataNavigator", void 0);
    __decorate([
        ViewChild('datagridElement', { static: true }),
        __metadata("design:type", ElementRef)
    ], TableComponent.prototype, "_tableElement", void 0);
    __decorate([
        ContentChildren('rowActionTmpl'),
        __metadata("design:type", QueryList)
    ], TableComponent.prototype, "rowActionTmpl", void 0);
    __decorate([
        ViewChild('rowActionsView', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "rowActionsViewRef", void 0);
    __decorate([
        ContentChildren('filterTmpl', { descendants: true }),
        __metadata("design:type", QueryList)
    ], TableComponent.prototype, "filterTmpl", void 0);
    __decorate([
        ViewChild('multiColumnFilterView', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "filterViewRef", void 0);
    __decorate([
        ContentChildren('inlineWidgetTmpl', { descendants: true }),
        __metadata("design:type", QueryList)
    ], TableComponent.prototype, "inlineWidgetTmpl", void 0);
    __decorate([
        ViewChild('inlineEditView', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "inlineEditViewRef", void 0);
    __decorate([
        ContentChildren('inlineWidgetTmplNew', { descendants: true }),
        __metadata("design:type", QueryList)
    ], TableComponent.prototype, "inlineWidgetNewTmpl", void 0);
    __decorate([
        ViewChild('inlineEditNewView', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "inlineEditNewViewRef", void 0);
    __decorate([
        ContentChildren('customExprTmpl', { descendants: true }),
        __metadata("design:type", QueryList)
    ], TableComponent.prototype, "customExprTmpl", void 0);
    __decorate([
        ViewChild('customExprView', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "customExprViewRef", void 0);
    __decorate([
        ContentChildren('rowExpansionActionTmpl'),
        __metadata("design:type", QueryList)
    ], TableComponent.prototype, "rowExpansionActionTmpl", void 0);
    __decorate([
        ContentChild('rowExpansionTmpl'),
        __metadata("design:type", TemplateRef)
    ], TableComponent.prototype, "rowExpansionTmpl", void 0);
    __decorate([
        ViewChild('rowDetailView', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "rowDetailViewRef", void 0);
    __decorate([
        ViewChild('rowExpansionActionView', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "rowExpansionActionViewRef", void 0);
    __decorate([
        ViewChild('dynamicTable', { static: true, read: ViewContainerRef }),
        __metadata("design:type", ViewContainerRef)
    ], TableComponent.prototype, "dynamicTableRef", void 0);
    __decorate([
        HostListener('keypress', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], TableComponent.prototype, "onKeyPress", null);
    TableComponent = TableComponent_1 = __decorate([
        Component({
            selector: '[wmTable]',
            template: "<div class=\"panel-heading\" *ngIf=\"title || subheading || iconclass || exportOptions.length || _actions.header.length\">\n    <h3 class=\"panel-title\">\n        <div class=\"pull-left\">\n            <i class=\"app-icon panel-icon {{iconclass}}\" *ngIf=\"iconclass\"></i>\n        </div>\n        <div class=\"pull-left\">\n            <div class=\"heading\" [innerHTML]=\"title | trustAs: 'html'\"></div>\n            <div class=\"description\" [innerHTML]=\"subheading | trustAs: 'html'\"></div>\n        </div>\n        <div class=\"panel-actions app-datagrid-actions\" *ngIf=\"exportOptions.length || _actions.header.length\">\n            <ng-container *ngFor=\"let btn of _actions.header\"\n                          [ngTemplateOutlet]=\"btn.widgetType === 'button' ? buttonRef : anchorRef\" [ngTemplateOutletContext]=\"{btn:btn}\">\n            </ng-container>\n            <div wmMenu dropdown caption.bind=\"appLocale.LABEL_EXPORT\" *ngIf=\"exportOptions.length\"  autoclose=\"always\" attr.name=\"{{name}}-export\"\n                dataset.bind=\"exportOptions\" select.event=\"export($item)\" menuposition=\"down,left\"></div>\n        </div>\n    </h3>\n</div>\n<div class=\"app-datagrid\" #datagridElement></div>\n<div class=\"panel-footer clearfix\" [hidden]=\"!_actions.footer.length && (!shownavigation || dataNavigator?.dataSize <= pagesize)\">\n    <div class=\"app-datagrid-paginator\" [hidden]=\"!dataNavigator?.dataSize || !shownavigation || dataNavigator?.dataSize <= pagesize\">\n        <nav wmPagination show.bind=\"shownavigation\" navigationalign.bind=\"navigationalign\"\n             navigationsize.bind=\"navigationSize\"\n             navigation.bind=\"navControls\" showrecordcount.bind=\"showrecordcount\" maxsize.bind=\"maxsize\"\n             boundarylinks.bind=\"boundarylinks\"\n             forceellipses.bind=\"forceellipses\" directionlinks.bind=\"directionlinks\"></nav>\n    </div>\n    <div class=\"app-datagrid-actions\" *ngIf=\"_actions.footer.length\">\n        <ng-container *ngFor=\"let btn of _actions.footer\"\n                      [ngTemplateOutlet]=\"btn.widgetType === 'button' ? buttonRef : anchorRef\" [ngTemplateOutletContext]=\"{btn:btn}\">\n        </ng-container>\n    </div>\n</div>\n\n<ng-template #buttonRef let-btn=\"btn\">\n    <button wmButton caption.bind=\"btn.displayName\" show.bind=\"btn.show\" class.bind=\"btn.class\"  iconclass.bind=\"btn.iconclass\" (click)=\"invokeActionEvent($event, btn.action)\"\n            [ngClass]=\"{'btn-sm': spacing === 'condensed', 'disabled-new': btn.key === 'addNewRow' && (isGridEditMode || isLoading)}\"\n            type=\"button\" shortcutkey.bind=\"btn.shortcutkey\" tabindex.bind=\"btn.tabindex\" hint.bind=\"btn.title\" disabled.bind=\"btn.disabled\" conditionalclass.bind=\"btn.conditionalclass\" conditionalstyle.bind=\"btn.conditionalstyle\"></button>\n</ng-template>\n\n<ng-template #anchorRef let-btn=\"btn\">\n    <a wmAnchor caption.bind=\"btn.displayName\" show.bind=\"btn.show\" class.bind=\"btn.class\"  iconclass.bind=\"btn.iconclass\" (click)=\"invokeActionEvent($event, btn.action)\"\n            [ngClass]=\"{'btn-sm': spacing === 'condensed', 'disabled-new': btn.key === 'addNewRow' && (isGridEditMode || isLoading)}\"\n            shortcutkey.bind=\"btn.shortcutkey\" tabindex.bind=\"btn.tabindex\" hint.bind=\"btn.title\"\n            hyperlink.bind=\"btn.hyperlink\" target.bind=\"btn.target\" conditionalclass.bind=\"btn.conditionalclass\" conditionalstyle.bind=\"btn.conditionalstyle\"></a>\n</ng-template>\n\n<div hidden>\n    <ng-container #multiColumnFilterView></ng-container>\n\n    <ng-container #inlineEditView></ng-container>\n\n    <ng-container #inlineEditNewView></ng-container>\n\n    <ng-container #rowActionsView></ng-container>\n\n    <ng-container #rowExpansionActionView></ng-container>\n\n    <ng-container #customExprView></ng-container>\n\n    <ng-container #dynamicTable></ng-container>\n\n    <ng-container #rowDetailView></ng-container>\n\n    <div class=\"dynamic-table-container\"></div>\n</div>\n",
            providers: [
                provideAs(TableComponent_1, NG_VALUE_ACCESSOR, true),
                provideAsWidgetRef(TableComponent_1)
            ]
        }),
        __param(6, Optional()),
        __param(7, Attribute('dataset.bind')),
        __param(8, Attribute('datasource.bind')),
        __param(9, Attribute('readonlygrid')),
        __metadata("design:paramtypes", [Injector,
            FormBuilder,
            App,
            DynamicComponentRefProvider,
            StatePersistence,
            Viewport,
            ListComponent, Object, Object, Object, NgZone,
            TrustAsPipe])
    ], TableComponent);
    return TableComponent;
}(StylableComponent));

var OPERATION = {
    'NEW': 'new',
    'EDIT': 'edit',
    'DELETE': 'delete'
};
var TableCUDDirective = /** @class */ (function () {
    function TableCUDDirective(table, dialogService, app) {
        this.table = table;
        this.dialogService = dialogService;
        this.app = app;
        table.initiateSelectItem = this.initiateSelectItem.bind(this);
        table.updateVariable = this.updateVariable.bind(this);
        table.updateRecord = this.updateRecord.bind(this);
        table.deleteRecord = this.deleteRecord.bind(this);
        table.insertRecord = this.insertRecord.bind(this);
        table.editRow = this.editRow.bind(this);
        table.addNewRow = this.addNewRow.bind(this);
        table.addRow = this.addNewRow.bind(this);
        table.deleteRow = this.deleteRow.bind(this);
        table.onRecordDelete = this.onRecordDelete.bind(this);
        table.hideEditRow = this.hideEditRow.bind(this);
        table.saveRow = this.saveRow.bind(this);
        table.cancelRow = this.cancelRow.bind(this);
    }
    TableCUDDirective.prototype.selectItemOnSuccess = function (row, skipSelectItem, callBack) {
        var _this = this;
        /*$timeout is used so that by then $is.dataset has the updated value.
         * Selection of the item is done in the callback of page navigation so that the item that needs to be selected actually exists in the grid.*/
        /*Do not select the item if skip selection item is specified*/
        setTimeout(function () {
            if (!skipSelectItem) {
                _this.table.selectItem(row, _this.table.dataset);
            }
            triggerFn(callBack);
        }, 250);
    };
    TableCUDDirective.prototype.initiateSelectItem = function (index, row, skipSelectItem, isStaticVariable, callBack) {
        var _this = this;
        /*index === "last" indicates that an insert operation has been successfully performed and navigation to the last page is required.
         * Hence increment the "dataSize" by 1.*/
        if (index === 'last') {
            if (!isStaticVariable) {
                this.table.dataNavigator.dataSize += 1;
            }
            /*Update the data in the current page in the grid after insert/update operations.*/
            if (!this.table.isNavigationEnabled()) {
                index = 'current';
            }
        }
        /*Re-calculate the paging values like pageCount etc that could change due to change in the dataSize.*/
        this.table.dataNavigator.calculatePagingValues();
        /* updating pagination fulldata*/
        this.table.dataNavigator.__fullData = this.table.__fullData;
        this.table.dataNavigator.navigatePage(index, null, true, function () {
            if (_this.table.isNavigationEnabled() || isStaticVariable) {
                _this.selectItemOnSuccess(row, skipSelectItem, callBack);
            }
        });
    };
    TableCUDDirective.prototype.updateVariable = function (row, callBack) {
        var _this = this;
        var dataSource = this.table.datasource;
        if (!dataSource) {
            return;
        }
        dataSource.execute(DataSource.Operation.FETCH_DISTINCT_VALUES);
        if (!this.table.isNavigationEnabled()) {
            var sortInfo = this.table.sortInfo;
            var sortOptions = sortInfo && sortInfo.direction ? (sortInfo.field + ' ' + sortInfo.direction) : '';
            refreshDataSource(dataSource, {
                page: 1,
                filterFields: this.table.getFilterFields(this.table.filterInfo),
                orderBy: sortOptions,
                matchMode: 'anywhereignorecase'
            }).then(function () {
                $appDigest();
                _this.selectItemOnSuccess(row, true, callBack);
            });
        }
    };
    TableCUDDirective.prototype.insertSuccessHandler = function (response, options) {
        /*Display appropriate error message in case of error.*/
        if (response.error) {
            this.table.invokeEventCallback('error', { $event: options.event, $operation: OPERATION.NEW, $data: response.error });
            if (!this.table.onError) {
                this.table.toggleMessage(true, 'error', this.table.errormessage || response.error);
            }
            triggerFn(options.error, response);
        }
        else {
            if (options.event) {
                var row = $(options.event.target).closest('tr');
                this.table.callDataGridMethod('hideRowEditMode', row);
            }
            if (!this.table.onRowinsert) {
                this.table.toggleMessage(true, 'success', this.table.insertmessage);
            }
            if (this.table.datasource.execute(DataSource.Operation.SUPPORTS_CRUD)) {
                this.table.initiateSelectItem(this.table.getNavigationTargetBySortInfo(), response, undefined, false, options.callBack);
                this.updateVariable(response, options.callBack);
            }
            else if (!this.table.datasource.execute(DataSource.Operation.IS_API_AWARE)) {
                this.table.initiateSelectItem(this.table.getNavigationTargetBySortInfo(), response, undefined, false, options.callBack);
            }
            triggerFn(options.success, response);
            this.table.invokeEventCallback('rowinsert', { $event: options.event, $data: response, row: response });
        }
    };
    TableCUDDirective.prototype.generatePath = function (binddataset) {
        var path, index;
        var dataBoundExpr = this.table.widget.$attrs.get('datasetboundexpr');
        if (_.startsWith(binddataset, 'item') && dataBoundExpr) {
            if (_.startsWith(dataBoundExpr, 'Widgets.')) {
                dataBoundExpr = extractCurrentItemExpr(dataBoundExpr, this.table);
            }
            var parentListItems = this.table.$element.parents('.app-list-item');
            var indexArr = _.map(parentListItems, function (item) { return _.parseInt($(item).attr('listitemindex')); });
            for (var i = indexArr.length - 1; i >= 0; i--) {
                dataBoundExpr = dataBoundExpr.replace('$i', indexArr[i]);
            }
            index = _.last(indexArr);
            path = dataBoundExpr.replace(/^Variables\..*\.dataSet([^.])*(\.|$)/g, '');
        }
        else {
            // if we have dataset as "Variables.staticVar1.dataSet[1].details" then pass index as 1.
            var regEx = /^Variables\..*\.dataSet([\[][0-9]])/g;
            if (regEx.test(binddataset)) {
                index = _.parseInt(binddataset.replace(/^Variables\..*\.dataSet([\[])/g, ''));
            }
            path = binddataset.replace(/^Variables\..*\.dataSet([^.])*(\.|$)/g, '');
        }
        return { path: path, index: index };
    };
    TableCUDDirective.prototype.insertRecord = function (options) {
        var _this = this;
        var dataSource = this.table.datasource;
        if (!dataSource) {
            return;
        }
        var currentPageNum = dataSource.pagination && dataSource.pagination.number + 1;
        var dataObject = {
            row: options.row,
            skipNotification: true,
            period: options.period
        };
        if (dataSource.execute(DataSource.Operation.SUPPORTS_CRUD) || !dataSource.execute(DataSource.Operation.IS_API_AWARE) || dataSource.category === 'wm.CrudVariable') {
            if (!dataSource.execute(DataSource.Operation.IS_API_AWARE)) {
                var extraOptions = this.generatePath(this.table.binddataset);
                dataSource.execute(DataSource.Operation.ADD_ITEM, { item: options.row, path: extraOptions.path, parentIndex: extraOptions.index });
                this.insertSuccessHandler(options.row, options);
                return;
            }
            dataSource.execute(DataSource.Operation.INSERT_RECORD, dataObject).then(function (response) {
                _this.insertSuccessHandler(response, options);
                if (dataSource.category === 'wm.CrudVariable') {
                    dataSource.execute(DataSource.Operation.LIST_RECORDS, {
                        'skipToggleState': true,
                        'operation': 'list',
                        'page': currentPageNum
                    });
                }
            }, function (error) {
                _this.table.invokeEventCallback('error', { $event: options.event, $operation: OPERATION.NEW, $data: error });
                if (!_this.table.onError) {
                    _this.table.toggleMessage(true, 'error', _this.table.errormessage || error);
                }
                triggerFn(options.error, error);
                triggerFn(options.callBack, undefined, true);
            });
        }
        else {
            this.table.invokeEventCallback('rowinsert', { $event: options.event, row: options.row });
        }
    };
    TableCUDDirective.prototype.updateSuccessHandler = function (response, options) {
        /*Display appropriate error message in case of error.*/
        if (response.error) {
            this.table.invokeEventCallback('error', { $event: options.event, $operation: OPERATION.EDIT, $data: response.error });
            /*disable readonly and show the appropriate error*/
            if (!this.table.onError) {
                this.table.toggleMessage(true, 'error', this.table.errormessage || response.error);
            }
            triggerFn(options.error, response);
        }
        else {
            if (options.event) {
                var row = $(options.event.target).closest('tr');
                this.table.callDataGridMethod('hideRowEditMode', row);
            }
            if (!this.table.onRowupdate) {
                this.table.toggleMessage(true, 'success', this.table.updatemessage);
            }
            if (this.table.datasource.execute(DataSource.Operation.SUPPORTS_CRUD)) {
                this.table.initiateSelectItem('current', response, undefined, false, options.callBack);
                this.updateVariable(response, options.callBack);
            }
            else if (!this.table.datasource.execute(DataSource.Operation.IS_API_AWARE)) {
                this.table.initiateSelectItem('current', response, undefined, false, options.callBack);
            }
            triggerFn(options.success, response);
            this.table.invokeEventCallback('rowupdate', { $event: options.event, $data: response, row: response });
        }
    };
    TableCUDDirective.prototype.updateRecord = function (options) {
        var _this = this;
        var dataSource = this.table.datasource;
        if (!dataSource) {
            return;
        }
        var currentPageNum = dataSource.pagination && dataSource.pagination.number + 1;
        var dataObject = {
            row: options.row,
            prevData: options.prevData,
            skipNotification: true,
            period: options.period
        };
        if (dataSource.execute(DataSource.Operation.SUPPORTS_CRUD) || !dataSource.execute(DataSource.Operation.IS_API_AWARE) || dataSource.category === 'wm.CrudVariable') {
            if (!dataSource.execute(DataSource.Operation.IS_API_AWARE)) {
                var extraOptions = this.generatePath(this.table.binddataset);
                dataSource.execute(DataSource.Operation.SET_ITEM, { prevItem: options.prevData, item: options.row, path: extraOptions.path, parentIndex: extraOptions.index });
                this.updateSuccessHandler(options.row, options);
                return;
            }
            dataSource.execute(DataSource.Operation.UPDATE_RECORD, dataObject).then(function (response) {
                _this.updateSuccessHandler(response, options);
                if (dataSource.category === 'wm.CrudVariable') {
                    dataSource.execute(DataSource.Operation.LIST_RECORDS, {
                        'skipToggleState': true,
                        'operation': 'list',
                        'page': currentPageNum
                    });
                }
            }, function (error) {
                _this.table.invokeEventCallback('error', { $event: options.event, $operation: OPERATION.EDIT, $data: error });
                if (!_this.table.onError) {
                    _this.table.toggleMessage(true, 'error', _this.table.errormessage || error);
                }
                triggerFn(options.error, error);
                triggerFn(options.callBack, undefined, true);
            });
        }
        else {
            this.table.invokeEventCallback('rowupdate', { $event: options.event, row: options.row });
        }
    };
    TableCUDDirective.prototype.onRecordDelete = function (callBack) {
        var index;
        /*Check for sanity*/
        if (this.table.dataNavigator) {
            this.table.dataNavigator.dataSize -= 1;
            this.table.dataNavigator.calculatePagingValues();
            /*If the current page does not contain any records due to deletion, then navigate to the previous page.*/
            index = this.table.dataNavigator.pageCount < this.table.dataNavigator.dn.currentPage ? 'prev' : undefined;
            /* updating pagination fulldata*/
            this.table.dataNavigator.__fullData = this.table.__fullData;
            this.table.dataNavigator.navigatePage(index, null, true, function () {
                setTimeout(function () {
                    triggerFn(callBack);
                }, undefined, false);
            });
        }
    };
    TableCUDDirective.prototype.deleteSuccessHandler = function (row, response, evt, callBack) {
        /* check the response whether the data successfully deleted or not , if any error occurred show the
         * corresponding error , other wise remove the row from grid */
        if (response && response.error) {
            if (!this.table.onError) {
                this.table.toggleMessage(true, 'error', this.table.errormessage || response.error);
            }
            return;
        }
        this.onRecordDelete(callBack);
        if (this.table.datasource.execute(DataSource.Operation.SUPPORTS_CRUD)) {
            this.updateVariable(row, callBack);
        }
        if (!this.table.onRowdelete) {
            this.table.toggleMessage(true, 'success', this.table.deletemessage);
        }
        // custom EventHandler for row deleted event
        this.table.invokeEventCallback('rowdelete', { $event: evt, $data: row, row: row });
        this.table.invokeEventCallback('rowdeleted', { $event: evt, $data: row, row: row });
    };
    TableCUDDirective.prototype.deleteFn = function (options) {
        var _this = this;
        var dataSource = this.table.datasource;
        var isLastPageElement, currentPageNum;
        if (!dataSource) {
            return;
        }
        if (dataSource.category === 'wm.CrudVariable') {
            this.triggerWMEvent('resetEditMode');
            if (dataSource.pagination) {
                isLastPageElement = dataSource.pagination.last && dataSource.pagination.numberOfElements === 1;
                currentPageNum = isLastPageElement ? dataSource.pagination.number : dataSource.pagination.number + 1;
            }
        }
        if (dataSource.execute(DataSource.Operation.SUPPORTS_CRUD) || !dataSource.execute(DataSource.Operation.IS_API_AWARE) || this.table._isDependent || dataSource.category === 'wm.CrudVariable') {
            if (!dataSource.execute(DataSource.Operation.IS_API_AWARE)) {
                var extraOptions = this.generatePath(this.table.binddataset);
                dataSource.execute(DataSource.Operation.REMOVE_ITEM, { item: options.row, path: extraOptions.path, parentIndex: extraOptions.index });
                this.deleteSuccessHandler(options.row, undefined, options.evt, options.callBack);
                return;
            }
            dataSource.execute(DataSource.Operation.DELETE_RECORD, {
                row: options.row,
                skipNotification: true,
                period: options.period
            }).then(function (response) {
                _this.deleteSuccessHandler(response, options.row, options.evt, options.callBack);
                if (dataSource.category === 'wm.CrudVariable') {
                    //this.triggerWMEvent('rerender', dataSource);
                    dataSource.execute(DataSource.Operation.LIST_RECORDS, {
                        'skipToggleState': true,
                        'operation': 'list',
                        'page': currentPageNum
                    });
                }
            }, function (error) {
                triggerFn(options.callBack, undefined, true);
                _this.table.invokeEventCallback('error', { $event: options.evt, $operation: OPERATION.DELETE, $data: error });
                if (!_this.table.onError) {
                    _this.table.toggleMessage(true, 'error', _this.table.errormessage || error);
                }
            });
        }
        else {
            this.table.invokeEventCallback('rowdelete', { $event: options.evt, row: options.row });
        }
        triggerFn(options.cancelRowDeleteCallback);
    };
    TableCUDDirective.prototype.deleteRecord = function (options) {
        var _this = this;
        if (!this.table.confirmdelete) {
            this.deleteFn(options);
            triggerFn(options.cancelRowDeleteCallback);
            return;
        }
        this.dialogService.showAppConfirmDialog({
            title: this.app.appLocale.MESSAGE_DELETE_RECORD || 'Delete Record',
            iconclass: 'wi wi-delete fa-lg',
            message: this.table.confirmdelete,
            oktext: this.table.deleteoktext,
            canceltext: this.table.deletecanceltext,
            onOk: function () {
                _this.deleteFn(options);
                _this.dialogService.closeAppConfirmDialog();
            },
            onCancel: function () {
                triggerFn(options.cancelRowDeleteCallback);
                _this.dialogService.closeAppConfirmDialog();
            },
            onOpen: function () {
                // Focus the cancel button on open
                $('.cancel-action').focus();
            },
            onEscape: function () {
                triggerFn(options.cancelRowDeleteCallback);
            }
        });
    };
    TableCUDDirective.prototype.editRow = function (evt) {
        var _this = this;
        var row;
        if (this.table._isDependent) {
            this.triggerWMEvent('update');
            return;
        }
        if (evt && evt.target) {
            this.table.callDataGridMethod('toggleEditRow', evt, { 'selectRow': true, action: 'edit' });
        }
        else {
            // For live form, call the update function with selected item
            if (this.table.editmode === 'form' || this.table.editmode === 'dialog') {
                row = evt || this.table.selectedItems[0];
                this.table.gridOptions.beforeRowUpdate(row);
            }
            else {
                // Wait for the selected item to get updated
                setTimeout(function () {
                    row = _this.table.datagridElement.find('tr.active');
                    if (row.length) {
                        _this.table.callDataGridMethod('toggleEditRow', undefined, { $row: row, action: 'edit' });
                    }
                });
            }
        }
    };
    TableCUDDirective.prototype.addNewRow = function () {
        if (!this.table.isGridEditMode) { // If grid is already in edit mode, do not add new row
            this.table.callDataGridMethod('addNewRow');
            if (this.table._liveTableParent) {
                this.table._liveTableParent.addNewRow();
            }
            else if (this.table._isDependent) {
                this.triggerWMEvent('insert');
                //this.table.callDataGridMethod('addNewRow');
            }
        }
    };
    TableCUDDirective.prototype.triggerWMEvent = function (eventName, dataSource) {
        $invokeWatchers(true);
        this.app.notify('wm-event', { eventName: eventName, widgetName: this.table.name, row: this.table.selecteditem, dataSource: dataSource });
    };
    TableCUDDirective.prototype.deleteRow = function (evt) {
        var _this = this;
        var row;
        if (evt && evt.target) {
            this.table.callDataGridMethod('deleteRowAndUpdateSelectAll', evt);
        }
        else {
            // Wait for the selected item to get updated
            setTimeout(function () {
                row = evt || _this.table.selectedItems[0];
                _this.deleteRecord({ row: row });
            });
        }
    };
    // Function to hide the edited row
    TableCUDDirective.prototype.hideEditRow = function () {
        var $row = this.table.datagridElement.find('tr.row-editing');
        if ($row.length) {
            this.table.callDataGridMethod('hideRowEditMode', $row);
        }
    };
    // Function to save the row
    TableCUDDirective.prototype.saveRow = function () {
        this.table.callDataGridMethod('saveRow');
    };
    // Function to cancel the edit
    TableCUDDirective.prototype.cancelRow = function () {
        var $row = this.table.datagridElement.find('tr.row-editing');
        if ($row.length) {
            this.table.callDataGridMethod('cancelEdit', $row);
        }
    };
    TableCUDDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [TableComponent,] }] },
        { type: AbstractDialogService },
        { type: App }
    ]; };
    TableCUDDirective = __decorate([
        Directive({
            selector: '[wmTableCUD]'
        }),
        __param(0, Self()), __param(0, Inject(TableComponent)),
        __metadata("design:paramtypes", [Object, AbstractDialogService,
            App])
    ], TableCUDDirective);
    return TableCUDDirective;
}());

var emptyMatchModes = ['null', 'empty', 'nullorempty', 'isnotnull', 'isnotempty'];
// Get search value based on the time
var getSearchValue = function (value, type) {
    if (!isDefined(value) || value === '') {
        return undefined;
    }
    if (isNumberType(type)) {
        return _.toNumber(value);
    }
    if (type === DataType.DATETIME) {
        return moment(value).valueOf();
    }
    return _.toString(value).toLowerCase();
};
var ɵ0$1 = getSearchValue;
// Filter the data based on the search value and conditions
var getFilteredData = function (data, searchObj) {
    var searchVal = getSearchValue(searchObj.value, searchObj.type);
    var currentVal;
    // Return whole data if
    // - search value is undefined and matchmode is not an empty matchmode type
    // - search value is null and datatype is number. Null can not be compared with numeric values[WMS-19275].
    if ((!isDefined(searchVal) && !_.includes(emptyMatchModes, searchObj.matchMode)) || (isNumberType(searchObj.type) && _.isNull(searchObj.value))) {
        return data;
    }
    data = data.filter(function (obj) {
        var isExists;
        if (searchObj.field) {
            currentVal = getSearchValue(_.get(obj, searchObj.field), searchObj.type);
        }
        else {
            currentVal = _.values(obj).join(' ').toLowerCase(); // If field is not there, search on all the columns
        }
        switch (searchObj.matchMode) {
            case 'start':
                isExists = _.startsWith(currentVal, searchVal);
                break;
            case 'end':
                isExists = _.endsWith(currentVal, searchVal);
                break;
            case 'exact':
                isExists = _.isEqual(currentVal, searchVal);
                break;
            case 'notequals':
                isExists = !_.isEqual(currentVal, searchVal);
                break;
            case 'null':
                isExists = _.isNull(currentVal);
                break;
            case 'isnotnull':
                isExists = !_.isNull(currentVal);
                break;
            case 'empty':
                isExists = _.isEmpty(currentVal);
                break;
            case 'isnotempty':
                isExists = !_.isEmpty(currentVal);
                break;
            case 'nullorempty':
                isExists = _.isNull(currentVal) || _.isEmpty(currentVal);
                break;
            case 'lessthan':
                isExists = currentVal < searchVal;
                break;
            case 'lessthanequal':
                isExists = currentVal <= searchVal;
                break;
            case 'greaterthan':
                isExists = currentVal > searchVal;
                break;
            case 'greaterthanequal':
                isExists = currentVal >= searchVal;
                break;
            default:
                isExists = isNumberType(searchObj.type) ? _.isEqual(currentVal, searchVal) : _.includes(currentVal, searchVal);
                break;
        }
        return isExists;
    });
    return data;
};
var ɵ1$1 = getFilteredData;
// Set the filter fields as required by datasource
var setFilterFields = function (filterFields, searchObj) {
    var field = searchObj && searchObj.field;
    /*Set the filter options only when a field/column has been selected.*/
    if (field) {
        filterFields[field] = {
            'value': searchObj.value,
            'logicalOp': 'AND'
        };
        if (searchObj.matchMode) {
            filterFields[field].matchMode = searchObj.matchMode;
        }
    }
};
var ɵ2 = setFilterFields;
// Transform filter fields from array to object having field names as keys
var transformFilterField = function (userFilters, filterField) {
    if (filterField.field) {
        userFilters[filterField.field] = {
            value: filterField.value,
            matchMode: filterField.matchMode,
            type: filterField.type
        };
    }
};
var ɵ3 = transformFilterField;
var TableFilterSortDirective = /** @class */ (function () {
    function TableFilterSortDirective(table) {
        this.table = table;
        table._searchSortHandler = this.searchSortHandler.bind(this);
        table.getSearchResult = this.getSearchResult.bind(this);
        table.getSortResult = this.getSortResult.bind(this);
        table.checkFiltersApplied = this.checkFiltersApplied.bind(this);
        table.getFilterFields = this.getFilterFields.bind(this);
        table.onRowFilterChange = this.onRowFilterChange.bind(this);
        table.onFilterConditionSelect = this.onFilterConditionSelect.bind(this);
        table.showClearIcon = this.showClearIcon.bind(this);
        table.clearRowFilter = this.clearRowFilter.bind(this);
        table.matchModeTypesMap = getMatchModeTypesMap();
        table.matchModeMsgs = getMatchModeMsgs(table.appLocale);
        table.emptyMatchModes = emptyMatchModes;
        table.getNavigationTargetBySortInfo = this.getNavigationTargetBySortInfo.bind(this);
        table.refreshData = this.refreshData.bind(this);
        table.clearFilter = this.clearFilter.bind(this);
        table.adjustContainer = this.adjustContainer.bind(this);
    }
    TableFilterSortDirective.prototype.adjustContainer = function (fieldName) {
        var _this = this;
        setTimeout(function () {
            adjustContainerRightEdges($('bs-dropdown-container'), _this.table.rowFilterCompliedTl[fieldName], _this.table, $('bs-dropdown-container .dropdown-menu'));
        });
    };
    // Get first or last page based on sort info of primary key
    TableFilterSortDirective.prototype.getNavigationTargetBySortInfo = function () {
        return this.table.sortInfo && this.table.sortInfo.direction === 'desc' &&
            _.includes(this.table.primaryKey, this.table.sortInfo.field) ? 'first' : 'last';
    };
    // Get the filter fields as required by datasource
    TableFilterSortDirective.prototype.getFilterFields = function (searchObj) {
        var filterFields = {};
        if (_.isArray(searchObj)) {
            _.forEach(searchObj, function (obj) {
                setFilterFields(filterFields, obj);
            });
        }
        else {
            setFilterFields(filterFields, searchObj);
        }
        return filterFields;
    };
    // Reset the sort based on sort returned by the call
    TableFilterSortDirective.prototype.resetSortStatus = function (variableSort) {
        var gridSortString;
        if (!_.isEmpty(this.table.sortInfo) && this.table.datasource) {
            gridSortString = this.table.sortInfo.field + ' ' + this.table.sortInfo.direction;
            variableSort = this.table.datasource.execute(DataSource.Operation.GET_OPTIONS).orderBy || variableSort;
            if (variableSort) { // If multiple order by fields are present, compare with the first one
                variableSort = _.head(_.split(variableSort, ','));
            }
            if (gridSortString !== variableSort) {
                this.table.callDataGridMethod('resetSortIcons');
                this.table.sortInfo = {};
                this.table.setDataGridOption('sortInfo', {});
            }
        }
    };
    // Clear the all the filters applied
    TableFilterSortDirective.prototype.clearFilter = function (skipFilter) {
        var $gridElement;
        this.table.filterInfo = {};
        if (this.table.filtermode === 'multicolumn') {
            this.table.fieldDefs.forEach(function (col) {
                if (col.resetFilter) {
                    col.resetFilter();
                }
            });
            if (!skipFilter) {
                this.table.onRowFilterChange();
            }
        }
        else if (this.table.filtermode === 'search') {
            $gridElement = this.table.datagridElement;
            $gridElement.find('[data-element="dgSearchText"]').val('');
            $gridElement.find('[data-element="dgFilterValue"]').val('');
            if (!skipFilter) {
                $gridElement.find('.app-search-button').trigger('click');
            }
        }
    };
    // Check the filters applied and remove if dat does not contain any filters
    TableFilterSortDirective.prototype.checkFiltersApplied = function (variableSort) {
        if (!this.table.datasource) {
            return;
        }
        if (this.table.datasource.execute(DataSource.Operation.SUPPORTS_SERVER_FILTER)) {
            if (_.isEmpty(this.table.datasource.execute(DataSource.Operation.GET_OPTIONS).filterFields)) {
                this.clearFilter(true);
            }
            this.resetSortStatus(variableSort);
            return;
        }
        if (this.table.datasource.execute(DataSource.Operation.IS_PAGEABLE)) {
            this.resetSortStatus(variableSort);
        }
    };
    // Returns data filtered using searchObj
    TableFilterSortDirective.prototype.getSearchResult = function (data, searchObj) {
        if (!searchObj) {
            return data;
        }
        if (_.isArray(searchObj)) {
            searchObj.forEach(function (obj) {
                data = getFilteredData(data, obj);
            });
        }
        else {
            data = getFilteredData(data, searchObj);
        }
        return data;
    };
    // Returns data sorted using sortObj
    TableFilterSortDirective.prototype.getSortResult = function (data, sortObj) {
        if (sortObj && sortObj.direction) {
            data = _.orderBy(data, sortObj.field, sortObj.direction);
        }
        return data;
    };
    // This method handles the client side sort and search
    TableFilterSortDirective.prototype.handleClientSideSortSearch = function (searchSortObj, e, type) {
        this.table._isClientSearch = true;
        var data;
        data = this.table.isNavigationEnabled() ? getClonedObject(this.table.__fullData) : getClonedObject(this.table.dataset);
        if (type === 'search') {
            this.table.filterInfo = searchSortObj;
        }
        else {
            this.table.sortInfo = searchSortObj;
        }
        if (_.isObject(data) && !_.isArray(data)) {
            data = [data];
        }
        /*Both the functions return same 'data' if arguments are undefined*/
        data = this.getSearchResult(data, this.table.filterInfo);
        data = this.getSortResult(data, this.table.sortInfo);
        this.table.serverData = data;
        if (type === 'sort') {
            // Calling 'onSort' event
            this.table.invokeEventCallback('sort', { $event: e, $data: {
                    data: this.table.serverData,
                    sortDirection: this.table.sortInfo.direction,
                    colDef: this.table.columns[this.table.sortInfo.field]
                } });
        }
        if (this.table.isNavigationEnabled()) {
            // Reset the page number to 1
            this.table.dataNavigator.dn.currentPage = 1;
            this.table.dataNavigator.setPagingValues(data);
        }
        else {
            this.table.setGridData(this.table.serverData);
        }
    };
    // This method handles the search for pageable datasource
    TableFilterSortDirective.prototype.handleSinglePageSearch = function (searchObj) {
        this.table._isPageSearch = true;
        var data = getClonedObject(this.table.gridData);
        var $rows = this.table.datagridElement.find('tbody tr.app-datagrid-row');
        this.table.filterInfo = searchObj;
        data = this.getSearchResult(data, searchObj);
        // Compared the filtered data and original data, to show or hide the rows
        _.forEach(this.table.gridData, function (value, index) {
            var $row = $($rows[index]);
            if (_.find(data, function (obj) { return _.isEqual(obj, value); })) {
                $row.show();
            }
            else {
                $row.hide();
            }
        });
        if (data && data.length) {
            this.table.callDataGridMethod('setStatus', 'ready');
            // Select the first row after the search for single select
            if (this.table.gridfirstrowselect && !this.table.multiselect) {
                this.table.callDataGridMethod('selectFirstRow', true, true);
            }
        }
        else {
            this.table.callDataGridMethod('setStatus', 'nodata', this.table.nodatamessage);
            this.table.selecteditem = undefined;
        }
        this.table.callDataGridMethod('updateSelectAllCheckboxState');
    };
    // This method handles the search for server side variables
    TableFilterSortDirective.prototype.handleServerSideSearch = function (searchObj) {
        var _this = this;
        this.table.filterInfo = searchObj;
        if (!this.table.datasource) {
            return;
        }
        var sortInfo = this.table.sortInfo;
        var sortOptions = sortInfo && sortInfo.direction ? (sortInfo.field + ' ' + sortInfo.direction) : '';
        var filterFields = this.getFilterFields(searchObj);
        refreshDataSource(this.table.datasource, {
            page: 1,
            filterFields: filterFields,
            orderBy: sortOptions
        }).then(function () {
            $appDigest();
        }, function () {
            _this.table.toggleMessage(true, 'error', _this.table.nodatamessage);
        });
    };
    // This method handles the sort for server side variables
    TableFilterSortDirective.prototype.handleSeverSideSort = function (sortObj, e, statePersistenceTriggered) {
        var _this = this;
        // Update the sort info for passing to datagrid
        this.table.gridOptions.sortInfo.field = sortObj.field;
        this.table.gridOptions.sortInfo.direction = sortObj.direction;
        this.table.sortInfo = getClonedObject(sortObj);
        var sortOptions = sortObj && sortObj.direction ? (sortObj.field + ' ' + sortObj.direction) : '';
        var filterFields = this.getFilterFields(this.table.filterInfo);
        if (!statePersistenceTriggered && this.table.getConfiguredState() !== 'none') {
            this.table.statePersistence.removeWidgetState(this.table, 'pagination');
        }
        refreshDataSource(this.table.datasource, {
            page: 1,
            filterFields: filterFields,
            orderBy: sortOptions
        }).then(function (response) {
            $appDigest();
            var data = (response && response.data) ? response.data : response;
            _this.table.invokeEventCallback('sort', { $event: e, $data: {
                    data: data,
                    sortDirection: sortObj.direction,
                    colDef: _this.table.columns[sortObj.field]
                } });
        });
    };
    TableFilterSortDirective.prototype.searchHandler = function (searchSortObj, e, type, statePersistenceTriggered) {
        var obj;
        if (_.isArray(searchSortObj)) {
            obj = searchSortObj.filter(function (searchObject) {
                return searchObject.matchMode !== undefined && searchObject.value !== undefined;
            });
        }
        else {
            obj = { field: searchSortObj.field, value: searchSortObj.value, type: searchSortObj.type };
        }
        if (this.table.getConfiguredState() !== 'none') {
            if ((_.isArray(searchSortObj) && obj.length) || (searchSortObj.value && searchSortObj.field)) {
                this.table.statePersistence.removeWidgetState(this.table, 'search');
                this.table.statePersistence.setWidgetState(this.table, { search: obj });
            }
            else {
                this.table.statePersistence.removeWidgetState(this.table, 'search');
            }
            if (!statePersistenceTriggered) {
                this.table.statePersistence.removeWidgetState(this.table, 'pagination');
                this.table.statePersistence.removeWidgetState(this.table, 'selectedItem');
            }
        }
        var filterFields = getClonedObject(searchSortObj);
        var dataSource = this.table.datasource;
        if (!dataSource) {
            return;
        }
        var output;
        var userFilters = {};
        // Transform filter fields from array to object having field names as keys
        if (_.isArray(filterFields)) {
            filterFields.forEach(function (filterField) {
                transformFilterField(userFilters, filterField);
            });
        }
        else {
            transformFilterField(userFilters, filterFields);
        }
        output = this.table.invokeEventCallback('beforefilter', { $event: e, $data: userFilters, columns: userFilters });
        // If callback returns false, don't trigger the filter call
        if (output === false) {
            return;
        }
        // Transform back the filter fields from object to array
        filterFields = [];
        _.forEach(userFilters, function (val, key) {
            filterFields.push({
                field: key,
                matchMode: val.matchMode,
                type: val.type,
                value: val.value
            });
        });
        if (dataSource.execute(DataSource.Operation.SUPPORTS_SERVER_FILTER)) {
            this.handleServerSideSearch(filterFields);
            return;
        }
        if (dataSource.execute(DataSource.Operation.IS_API_AWARE) && dataSource.execute(DataSource.Operation.IS_PAGEABLE)) {
            this.handleSinglePageSearch(filterFields);
        }
        else {
            this.handleClientSideSortSearch(filterFields, e, type);
        }
    };
    TableFilterSortDirective.prototype.sortHandler = function (searchSortObj, e, type, statePersistenceTriggered) {
        var dataSource = this.table.datasource;
        if (!dataSource) {
            return;
        }
        if (!statePersistenceTriggered && this.table.getConfiguredState() !== 'none') {
            var obj = { direction: searchSortObj.direction, field: searchSortObj.field };
            if (searchSortObj.direction) {
                this.table.statePersistence.setWidgetState(this.table, { sort: obj });
            }
            else {
                this.table.statePersistence.removeWidgetState(this.table, 'sort');
            }
            this.table.statePersistence.removeWidgetState(this.table, 'selectedItem');
        }
        if (dataSource.execute(DataSource.Operation.IS_PAGEABLE)) {
            this.handleSeverSideSort(searchSortObj, e, statePersistenceTriggered);
        }
        else {
            this.handleClientSideSortSearch(searchSortObj, e, type);
        }
    };
    // This method is triggered by jquery table
    TableFilterSortDirective.prototype.searchSortHandler = function (searchSortObj, e, type, statePersistenceTriggered) {
        if (type === 'search') {
            this.searchHandler(searchSortObj, e, type, statePersistenceTriggered);
        }
        else {
            this.sortHandler(searchSortObj, e, type, statePersistenceTriggered);
        }
    };
    // Method to show/hide clear icon in multi column filter
    TableFilterSortDirective.prototype.showClearIcon = function (fieldName) {
        var value = this.table.rowFilter[fieldName] && this.table.rowFilter[fieldName].value;
        return isDefined(value) && value !== '' && value !== null;
    };
    // Method clear the filter value in multi column filter
    TableFilterSortDirective.prototype.clearRowFilter = function (fieldName) {
        if (this.table.rowFilter && this.table.rowFilter[fieldName]) {
            this.table.columns[fieldName].resetFilter();
            this.onRowFilterChange(fieldName);
        }
    };
    // This method is triggered on select of condition in multi column filter
    TableFilterSortDirective.prototype.onFilterConditionSelect = function (fieldName, condition) {
        var _this = this;
        this.table.rowFilter[fieldName] = this.table.rowFilter[fieldName] || {};
        this.table.rowFilter[fieldName].matchMode = condition;
        // For empty match modes, clear off the value and call filter
        if (_.includes(this.table.emptyMatchModes, condition)) {
            this.table.columns[fieldName].resetFilter();
            this.table.onRowFilterChange();
        }
        else {
            // If value is present, call the filter. Else, focus on the field
            if (isDefined(this.table.rowFilter[fieldName].value) && this.table.rowFilter[fieldName].value !== '') {
                this.table.onRowFilterChange();
            }
            else {
                setTimeout(function () {
                    _this.table.columns[fieldName].filterInstance.focus();
                });
            }
        }
    };
    // Method to get the updated values when filter on field is changed for multicolumn filter
    TableFilterSortDirective.prototype.getFilterOnFieldValues = function (filterDef) {
        var _this = this;
        if (!this.table.datasource || !this.table.datasource.execute(DataSource.Operation.SUPPORTS_DISTINCT_API)) {
            return;
        }
        var fieldName = filterDef.field;
        var formFields = this.table.fullFieldDefs;
        var filterOnFields = _.filter(formFields, { 'filteronfilter': fieldName });
        var newVal = _.get(this.table.rowFilter, [fieldName, 'value']);
        // Loop over the fields for which the current field is filter on field
        _.forEach(filterOnFields, function (filterField) {
            var filterOn = filterField.filteronfilter;
            var filterKey = filterField.field;
            var filterFields = {};
            var filterWidget = filterField.filterwidget;
            if (!isDataSetWidget(filterWidget) || filterOn === filterKey || filterField.isFilterDataSetBound) {
                return;
            }
            filterFields[filterOn] = (isDefined(newVal) && newVal !== '' && newVal !== null) ? { 'value': newVal } : {};
            if (filterWidget === FormWidgetType.AUTOCOMPLETE && filterField.filterInstance.dataoptions) {
                filterField.filterInstance.dataoptions.filterFields = filterFields;
            }
            else {
                _this.table.datasource.execute(DataSource.Operation.GET_DISTINCT_DATA_BY_FIELDS, {
                    'fields': filterKey,
                    'filterFields': filterFields
                }).then(function (data) {
                    filterField.filterdataset = data.data;
                });
            }
        });
    };
    // This method is triggered on value change in multi column filter
    TableFilterSortDirective.prototype.onRowFilterChange = function (fieldName) {
        var _this = this;
        var searchObj = [];
        var field = _.find(this.table.fullFieldDefs, { 'field': fieldName });
        // Convert row filters to a search object and call search handler
        _.forEach(this.table.rowFilter, function (value, key) {
            if ((isDefined(value.value) && value.value !== '') || _.includes(_this.table.emptyMatchModes, value.matchMode)) {
                if (field && key === field.field) {
                    value.type = value.type || field.type;
                    value.matchMode = value.matchMode || _.get(_this.table.matchModeTypesMap[value.type], 0);
                }
                searchObj.push({
                    field: key,
                    value: value.value,
                    matchMode: value.matchMode,
                    type: value.type
                });
            }
        });
        this.table.gridOptions.searchHandler(searchObj, undefined, 'search');
        // If field is passed, update any filter on field values if present
        if (field) {
            this.getFilterOnFieldValues(field);
        }
    };
    TableFilterSortDirective.prototype.refreshData = function (isSamePage) {
        if (!this.table.datasource) {
            return;
        }
        var page = isSamePage ? this.table.dataNavigator.dn.currentPage : 1;
        var sortInfo = this.table.sortInfo;
        var sortOptions = sortInfo && sortInfo.direction ? (sortInfo.field + ' ' + sortInfo.direction) : '';
        var filterFields = this.getFilterFields(this.table.filterInfo);
        refreshDataSource(this.table.datasource, {
            page: page,
            filterFields: filterFields,
            orderBy: sortOptions
        });
    };
    TableFilterSortDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [TableComponent,] }] }
    ]; };
    TableFilterSortDirective = __decorate([
        Directive({
            selector: '[wmTableFilterSort]'
        }),
        __param(0, Self()), __param(0, Inject(TableComponent)),
        __metadata("design:paramtypes", [Object])
    ], TableFilterSortDirective);
    return TableFilterSortDirective;
}());

var registerProps$1 = function () {
    register('wm-table-action', new Map([
        ['accessroles', PROP_STRING],
        ['action', PROP_STRING],
        ['caption', PROP_STRING],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['disabled', __assign({ value: false }, PROP_BOOLEAN)],
        ['display-name', PROP_STRING],
        ['hyperlink', PROP_STRING],
        ['icon', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['key', PROP_STRING],
        ['position', __assign({ value: 'footer' }, PROP_STRING)],
        ['shortcutkey', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING],
        ['target', PROP_STRING],
        ['widget-type', __assign({ value: 'button' }, PROP_STRING)]
    ]));
};

var WIDGET_CONFIG$1 = { widgetType: 'wm-table-action', hostClass: '' };
var TableActionDirective = /** @class */ (function (_super) {
    __extends(TableActionDirective, _super);
    function TableActionDirective(inj, table) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        _this.table = table;
        return _this;
    }
    TableActionDirective_1 = TableActionDirective;
    TableActionDirective.prototype.populateAction = function () {
        this.buttonDef = {
            key: this.key,
            displayName: this['display-name'] || this.caption || '',
            show: this.show,
            class: this.class || '',
            iconclass: this.iconclass || '',
            title: _.isUndefined(this.title) ? (this['display-name'] || '') : this.title,
            action: this.action,
            accessroles: this.accessroles,
            shortcutkey: this.shortcutkey,
            disabled: this.disabled,
            tabindex: this.tabindex,
            icon: this.icon,
            position: this.position,
            widgetType: this['widget-type'],
            hyperlink: this.hyperlink,
            target: this.target || '',
            conditionalclass: this.conditionalclass || '',
            conditionalstyle: this.conditionalstyle || {}
        };
        this._propsInitialized = true;
    };
    TableActionDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.populateAction();
        this.table.registerActions(this.buttonDef);
    };
    TableActionDirective.prototype.onPropertyChange = function (key, nv) {
        if (!this._propsInitialized) {
            return;
        }
        if (key === 'display-name') {
            this.buttonDef.displayName = nv;
        }
        else {
            this.buttonDef[key] = nv;
        }
    };
    var TableActionDirective_1;
    TableActionDirective.initializeProps = registerProps$1();
    TableActionDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: TableComponent, decorators: [{ type: Optional }] }
    ]; };
    TableActionDirective = TableActionDirective_1 = __decorate([
        Directive({
            selector: '[wmTableAction]',
            providers: [
                provideAsWidgetRef(TableActionDirective_1)
            ]
        }),
        __param(1, Optional()),
        __metadata("design:paramtypes", [Injector, TableComponent])
    ], TableActionDirective);
    return TableActionDirective;
}(BaseComponent));

var registerProps$2 = function () {
    register('wm-table-column', new Map([
        ['accessroles', PROP_STRING],
        ['autoclose', __assign({ value: 'outsideClick' }, PROP_STRING)],
        ['backgroundcolor', PROP_STRING],
        ['binding', PROP_STRING],
        ['caption', PROP_STRING],
        ['checkedvalue', PROP_STRING],
        ['col-class', PROP_STRING],
        ['col-ng-class', PROP_STRING],
        ['currencypattern', PROP_STRING],
        ['customExpression', PROP_BOOLEAN],
        ['dataentrymode', __assign({ value: 'default' }, PROP_STRING)],
        ['datafield', PROP_STRING],
        ['dataoptions', PROP_ANY],
        ['dataset', PROP_ANY],
        ['datasource', PROP_STRING],
        ['datepattern', PROP_STRING],
        ['debouncetime', __assign({ value: 250 }, PROP_NUMBER)],
        ['defaultvalue', PROP_STRING],
        ['disabled', PROP_BOOLEAN],
        ['displayfield', PROP_STRING],
        ['displaylabel', PROP_STRING],
        ['editdatepattern', PROP_STRING],
        ['edit-widget-type', PROP_STRING],
        ['editinputtype', PROP_STRING],
        ['exportexpression', PROP_STRING],
        ['filterdataset', PROP_STRING],
        ['filterdatafield', PROP_STRING],
        ['filterdisplayfield', PROP_STRING],
        ['filterdisplaylabel', PROP_STRING],
        ['filterexpressions', __assign({ value: '{}' }, PROP_STRING)],
        ['filtersearchkey', PROP_STRING],
        ['filter-on', PROP_STRING],
        ['filteronfilter', PROP_STRING],
        ['filterplaceholder', PROP_STRING],
        ['filterwidget', PROP_STRING],
        ['filterinputtype', PROP_STRING],
        ['formatpattern', PROP_STRING],
        ['custompipeformat', PROP_STRING],
        ['fractionsize', PROP_STRING],
        ['generator', PROP_STRING],
        ['limit', PROP_NUMBER],
        ['matchmode', PROP_STRING],
        ['maxchars', PROP_NUMBER],
        ['maxvalue', PROP_NUMBER],
        ['step', PROP_NUMBER],
        ['mobiledisplay', __assign({ value: true }, PROP_BOOLEAN)],
        ['minvalue', PROP_NUMBER],
        ['maxdate', PROP_STRING],
        ['mindate', PROP_STRING],
        ['excludedays', PROP_STRING],
        ['excludedates', PROP_STRING],
        ['maxtime', PROP_STRING],
        ['mintime', PROP_STRING],
        ['pcdisplay', __assign({ value: true }, PROP_BOOLEAN)],
        ['period', PROP_BOOLEAN],
        ['placeholder', PROP_STRING],
        ['prefix', PROP_STRING],
        ['primary-key', PROP_BOOLEAN],
        ['readonly', PROP_BOOLEAN],
        ['regexp', PROP_ANY],
        ['related-entity-name', PROP_STRING],
        ['required', PROP_BOOLEAN],
        ['rowactionsposition', PROP_STRING],
        ['searchkey', PROP_STRING],
        ['searchable', __assign({ value: true }, PROP_BOOLEAN)],
        ['suffix', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['showdropdownon', __assign({ value: 'default' }, PROP_STRING)],
        ['sortable', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabletdisplay', __assign({ value: true }, PROP_BOOLEAN)],
        ['textalignment', __assign({ value: 'left' }, PROP_STRING)],
        ['textcolor', PROP_STRING],
        ['type', __assign({ value: 'string' }, PROP_STRING)],
        ['uncheckedvalue', PROP_STRING],
        ['validationmessage', PROP_STRING],
        ['widget-type', PROP_STRING],
        ['width', PROP_STRING],
    ]));
};

var registerProps$3 = function () {
    register('wm-table-column-group', new Map([
        ['backgroundcolor', PROP_STRING],
        ['caption', PROP_STRING],
        ['col-class', PROP_STRING],
        ['name', PROP_STRING],
        ['textalignment', __assign({ value: 'center' }, PROP_STRING)]
    ]));
};

var WIDGET_CONFIG$2 = { widgetType: 'wm-table-column-group', hostClass: '' };
var TableColumnGroupDirective = /** @class */ (function (_super) {
    __extends(TableColumnGroupDirective, _super);
    function TableColumnGroupDirective(inj, group, table) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$2) || this;
        _this.group = group;
        _this.table = table;
        _this.config = {};
        return _this;
    }
    TableColumnGroupDirective_1 = TableColumnGroupDirective;
    TableColumnGroupDirective.prototype.populateConfig = function () {
        this.config = {
            field: this.name,
            displayName: this.caption || '',
            columns: [],
            isGroup: true,
            accessroles: this.accessroles,
            textAlignment: this.textalignment || 'center',
            backgroundColor: this.backgroundcolor,
            class: this['col-class']
        };
    };
    TableColumnGroupDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'caption') {
            this.config.displayName = nv || '';
            this.table.callDataGridMethod('setColumnProp', this.config.field, 'displayName', nv, true);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    TableColumnGroupDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.populateConfig();
        // when group is inside another group (i.e. sub-level column-group) then consider the col index and not header index.
        var headerIndex = parseInt(this.getAttr('headerIndex'), 10);
        var colIndex = parseInt(this.getAttr('index'), 10);
        var fieldName = this.group && this.group.name;
        setHeaderConfigForTable(this.table.headerConfig, this.config, fieldName, fieldName ? colIndex : headerIndex);
    };
    var TableColumnGroupDirective_1;
    TableColumnGroupDirective.initializeProps = registerProps$3();
    TableColumnGroupDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: TableColumnGroupDirective, decorators: [{ type: SkipSelf }, { type: Optional }] },
        { type: TableComponent, decorators: [{ type: Optional }] }
    ]; };
    TableColumnGroupDirective = TableColumnGroupDirective_1 = __decorate([
        Directive({
            selector: '[wmTableColumnGroup]',
            providers: [
                provideAsWidgetRef(TableColumnGroupDirective_1)
            ]
        }),
        __param(1, SkipSelf()), __param(1, Optional()),
        __param(2, Optional()),
        __metadata("design:paramtypes", [Injector,
            TableColumnGroupDirective,
            TableComponent])
    ], TableColumnGroupDirective);
    return TableColumnGroupDirective;
}(BaseComponent));

var WIDGET_CONFIG$3 = { widgetType: 'wm-table-column', hostClass: '' };
var inlineWidgetProps = ['datafield', 'displayfield', 'placeholder', 'searchkey', 'matchmode', 'displaylabel',
    'checkedvalue', 'uncheckedvalue', 'showdropdownon', 'dataentrymode', 'autoclose', 'dataset'];
var validationProps = ['maxchars', 'regexp', 'minvalue', 'maxvalue', 'step', 'required', 'mindate', 'maxdate',
    'excludedates', 'excludedays', 'mintime', 'maxtime'];
inlineWidgetProps = __spread(inlineWidgetProps, validationProps);
var FieldDef = /** @class */ (function () {
    function FieldDef(widget) {
        this.widget = widget;
    }
    FieldDef.prototype.focus = function () {
        this.widget.focus();
    };
    FieldDef.prototype.setProperty = function (prop, newval) {
        // Get the scope of the current editable widget and set the value
        prop = prop === 'value' ? 'datavalue' : prop;
        this.widget[prop] = newval;
    };
    FieldDef.prototype.getProperty = function (prop) {
        prop = prop === 'value' ? 'datavalue' : prop;
        return this.widget[prop];
    };
    return FieldDef;
}());
var TableColumnDirective = /** @class */ (function (_super) {
    __extends(TableColumnDirective, _super);
    function TableColumnDirective(inj, appDefaults, table, group, bindfilterdataset, binddataset) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$3) || this;
        _this.appDefaults = appDefaults;
        _this.table = table;
        _this.group = group;
        _this.bindfilterdataset = bindfilterdataset;
        _this.binddataset = binddataset;
        _this.syncValidators = [];
        _this.asyncValidators = [];
        _this.observeOnFields = [];
        _this.aggregate = {
            sum: function () {
                return _.sum(_this._getColumnData());
            },
            average: function (precision) {
                if (precision === void 0) { precision = 2; }
                return _.round(_.mean(_this._getColumnData()), precision);
            },
            count: function () {
                return _this._getColumnData().length;
            },
            minimum: function () {
                return _.min(_this._getColumnData());
            },
            maximum: function () {
                return _.max(_this._getColumnData());
            },
            percent: function (value, precision) {
                if (precision === void 0) { precision = 2; }
                return _.round((_.sum(_this._getColumnData()) / value) * 100, precision);
            }
        };
        _this.notifyForFields = [];
        return _this;
    }
    TableColumnDirective_1 = TableColumnDirective;
    Object.defineProperty(TableColumnDirective.prototype, "dataoptions", {
        get: function () {
            return this._dataoptions;
        },
        set: function (options) {
            this._dataoptions = options;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TableColumnDirective.prototype, "datasource", {
        get: function () {
            return this._datasource;
        },
        set: function (ds) {
            this._datasource = ds;
        },
        enumerable: true,
        configurable: true
    });
    TableColumnDirective.prototype.ngOnInit = function () {
        var _this = this;
        _super.prototype.ngOnInit.call(this);
        // Set the default values and register with table
        this.populateFieldDef();
        // when column is inside a group then consider the col index and not header index.
        var colIndex = parseInt(this.getAttr('index'), 10);
        var headerIndex = parseInt(this.getAttr('headerIndex'), 10);
        var fieldName = this.group && this.group.name;
        // Register column with header config to create group structure
        setHeaderConfigForTable(this.table.headerConfig, {
            field: this.field,
            displayName: this.displayName
        }, fieldName, fieldName ? colIndex : headerIndex);
        this.table.registerColumns(this.widget, colIndex);
        this._isRowFilter = this.table.filtermode === 'multicolumn' && this.searchable;
        this._isInlineEditable = !this.readonly && (this.table.editmode !== EDIT_MODE.DIALOG && this.table.editmode !== EDIT_MODE.FORM);
        this._isNewEditableRow = this._isInlineEditable && this.table.editmode === EDIT_MODE.QUICK_EDIT && this.table.shownewrow;
        this.setUpControls();
        this._propsInitialized = true;
        // Inline control status change subscriber
        if (this.getFormControl()) {
            var onStatusChangeSubscription_1 = this.getFormControl().statusChanges
                .pipe(debounceTime(100))
                .subscribe(function (status) { return _this.onStatusChange(status, 'inlineInstance'); });
            this.registerDestroyListener(function () { return onStatusChangeSubscription_1.unsubscribe(); });
            // Instantiate custom validators class for inline edit form control
            this.fieldValidations = new BaseFieldValidations(this, this['inlineInstance'], this.editWidgetType, this.getFormControl(), this.table, 'inlineInstance');
        }
        // Quick edit new row control status change subscriber
        if (this._checkNewEditableRowControl()) {
            var onStatusChangeSubscription_new_1 = this.getFormControl('_new').statusChanges
                .pipe(debounceTime(100))
                .subscribe(function (status) { return _this.onStatusChange(status, 'inlineInstanceNew'); });
            this.registerDestroyListener(function () { return onStatusChangeSubscription_new_1.unsubscribe(); });
            // Instantiate custom validators class for Quick edit newrow form control
            this.fieldValidations_new = new BaseFieldValidations(this, this['inlineInstanceNew'], this.editWidgetType, this.getFormControl('_new'), this.table, 'inlineInstanceNew');
        }
    };
    TableColumnDirective.prototype.ngAfterContentInit = function () {
        var _this = this;
        if (this._isRowFilter) {
            // Listen on the inner row filter widget and setup the widget
            var s1_1 = this._filterInstances.changes.subscribe(function (val) {
                _this.filterInstance = val.first && val.first.widget;
                _this.setUpFilterWidget();
            });
            this.registerDestroyListener(function () { return s1_1.unsubscribe(); });
        }
        if (this._isInlineEditable) {
            var s2_1 = this._inlineInstances.changes.subscribe(function (val) {
                // Listen on the inner inline widget and setup the widget
                _this.inlineInstance = val.first && val.first.widget;
                _this.fieldValidations.formwidget = val.first;
                _this.table.registerFormField(_this.binding, new FieldDef(_this.inlineInstance));
                _this.setUpInlineWidget('inlineInstance');
            });
            this.registerDestroyListener(function () { return s2_1.unsubscribe(); });
            if (this._isNewEditableRow) {
                var s3_1 = this._inlineInstancesNew.changes.subscribe(function (val) {
                    // Listen on the inner inline widget and setup the widget
                    _this.inlineInstanceNew = val.first && val.first.widget;
                    _this.fieldValidations_new.formwidget = val.first;
                    _this.setUpInlineWidget('inlineInstanceNew');
                });
                this.registerDestroyListener(function () { return s3_1.unsubscribe(); });
            }
        }
        this.registerReadyStateListener(function () {
            _this.key = _this.field || _this.binding;
        });
        _super.prototype.ngAfterContentInit.call(this);
        if (this.table.isdynamictable) {
            this.table.inlineWidgetTmpl._results.push(this.inlineWidthTempRef);
        }
    };
    TableColumnDirective.prototype.ngAfterViewInit = function () {
        // manually listing the table column templateRef as templateRef will not be available prior.
        if (this.filterTemplateRef) {
            this.table.renderDynamicFilterColumn(this.filterTemplateRef);
        }
    };
    // Apply default|sync|async|prop validators for inline form control
    TableColumnDirective.prototype.applyValidations = function () {
        if (!this.getFormControl()) {
            return;
        }
        if (this.syncValidators.length > 0) {
            this.fieldValidations.setValidators(this.syncValidators);
        }
        this.fieldValidations.setUpValidators();
        if (this.asyncValidators.length > 0) {
            this.fieldValidations.setAsyncValidators(this.asyncValidators);
        }
        if (this.observeOnFields.length > 0) {
            this.fieldValidations.observeOn(this.observeOnFields, 'columns');
        }
    };
    // Remove validators for the inline widget and set form to untouched
    TableColumnDirective.prototype.removeValidations = function () {
        this.table.ngform.markAsUntouched();
        var control = this.getFormControl();
        if (!control) {
            return;
        }
        control.clearValidators();
        control.clearAsyncValidators();
        control.updateValueAndValidity();
    };
    // Apply default|sync|async|prop validators for QuickEdit new row form control
    TableColumnDirective.prototype.applyNewRowValidations = function () {
        if (!this._checkNewEditableRowControl()) {
            return;
        }
        if (this.syncValidators.length > 0) {
            this.fieldValidations_new.setValidators(this.syncValidators);
        }
        this.fieldValidations_new.setUpValidators();
        if (this.asyncValidators.length > 0) {
            this.fieldValidations_new.setAsyncValidators(this.asyncValidators);
        }
        if (this.observeOnFields.length > 0) {
            this.fieldValidations_new.observeOn(this.observeOnFields, 'columns');
        }
    };
    // Remove validators for the QuickEdit new row widget and set form to untouched
    TableColumnDirective.prototype.removeNewRowValidations = function () {
        this.table.ngform.markAsUntouched();
        if (!this._checkNewEditableRowControl()) {
            return;
        }
        var control = this.getFormControl('_new');
        control.clearValidators();
        control.clearAsyncValidators();
        control.updateValueAndValidity();
    };
    TableColumnDirective.prototype.addFormControl = function (suffix) {
        var ctrlName = suffix ? (this.binding + suffix) : this.binding;
        this.table.ngform.addControl(ctrlName, this.table.fb.control(''));
    };
    TableColumnDirective.prototype.getFormControl = function (suffix) {
        var ctrlName = suffix ? (this.binding + suffix) : this.binding;
        return this.table.ngform.controls[ctrlName];
    };
    TableColumnDirective.prototype._checkNewEditableRowControl = function () {
        return this._isNewEditableRow && this.getFormControl('_new');
    };
    // Setup the inline edit and filter widget
    TableColumnDirective.prototype.setUpControls = function () {
        var _this = this;
        if (this._isInlineEditable) {
            if (this.editWidgetType === FormWidgetType.UPLOAD) {
                return;
            }
            this.addFormControl();
            var control = this.getFormControl();
            if (control) {
                var onValueChangeSubscription_1 = control.valueChanges
                    .pipe(debounceTime(200))
                    .subscribe(function (value) { return _this.onValueChange(value, 'inlineInstance'); });
                this.registerDestroyListener(function () { return onValueChangeSubscription_1.unsubscribe(); });
            }
            if (this._isNewEditableRow) {
                this.addFormControl('_new');
                var newControl = this.getFormControl('_new');
                if (newControl) {
                    var onNewValueChangeSubscription_1 = newControl.valueChanges
                        .pipe(debounceTime(200))
                        .subscribe(function (value) { return _this.onValueChange(value, 'inlineInstanceNew'); });
                    this.registerDestroyListener(function () { return onNewValueChangeSubscription_1.unsubscribe(); });
                }
            }
        }
        if (this._isRowFilter) {
            this.addFormControl('_filter');
            this.filterControl = this.getFormControl('_filter');
            if (this.filterControl) {
                var onFilterValueSubscription_1 = this.filterControl.valueChanges.subscribe(this.onFilterValueChange.bind(this));
                this.registerDestroyListener(function () { return onFilterValueSubscription_1.unsubscribe(); });
            }
        }
    };
    // Reset the row filter value
    TableColumnDirective.prototype.resetFilter = function () {
        if (this.filterControl) {
            this.filterControl.setValue('');
        }
        if (this.filterwidget === FormWidgetType.AUTOCOMPLETE) {
            this.filterInstance.query = '';
            this.filterInstance.queryModel = '';
        }
    };
    // On field value change, propagate event to parent form
    TableColumnDirective.prototype.onFilterValueChange = function (val) {
        this.table.rowFilter[this.field].value = val;
    };
    Object.defineProperty(TableColumnDirective.prototype, "datavalue", {
        // Get of the active from control
        get: function () {
            return this[this.activeControlType] && this[this.activeControlType].datavalue;
        },
        set: function (val) {
            var control = this.getFormControl(this.activeControlType === 'inlineInstanceNew' ? '_new' : undefined);
            if (!control) {
                return;
            }
            if (control && this.editWidgetType !== FormWidgetType.UPLOAD) {
                control.setValue(val);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TableColumnDirective.prototype, "value", {
        get: function () {
            return this.datavalue;
        },
        set: function (val) {
            this.datavalue = val;
        },
        enumerable: true,
        configurable: true
    });
    // On field value change, apply cascading filter and set validation message
    TableColumnDirective.prototype.onValueChange = function (val, widget) {
        if (val !== null) {
            applyFilterOnField(this.table.datasource, this.widget, this.table.fieldDefs, val, {
                widget: 'edit-widget-type'
            });
        }
        if (this.table.ngform.touched) {
            this.activeControlType = widget;
            if (widget === 'inlineInstance') {
                this.notifyChanges();
                this.fieldValidations.setCustomValidationMessage();
            }
            else if (this._isNewEditableRow) {
                this.notifyChanges('_new');
                this.fieldValidations_new.setCustomValidationMessage();
            }
        }
    };
    TableColumnDirective.prototype.loadFilterData = function () {
        var _this = this;
        // If filterdataset is not bound, get the data implicitly
        if (this._isRowFilter && isDataSetWidget(this.filterwidget) && !this.bindfilterdataset) {
            // For live variable, get the data using distinct API
            if (this.table.datasource.execute(DataSource.Operation.SUPPORTS_DISTINCT_API)) {
                // check for related entity columns
                if (this.relatedEntityName) {
                    this.widget['is-related'] = true;
                    this.widget['lookup-type'] = this.relatedEntityName;
                    this.widget['lookup-field'] = _.last(_.split(this.field, '.'));
                }
                if (this.filterwidget === FormWidgetType.AUTOCOMPLETE) {
                    this.filterInstance.dataoptions = getDistinctFieldProperties(this.table.datasource, this);
                    this.filterInstance.datasource = this.table.datasource;
                }
                else {
                    getDistinctValues(this.table.datasource, this.widget, 'filterwidget').then(function (res) {
                        _this._filterDataSet = res.data;
                        _this.setFilterWidgetDataSet();
                    });
                }
            }
            else {
                // For other datasources, get the data from datasource bound to table
                this.registerDestroyListener($watch(this.table.binddataset, this.viewParent, {}, function (nv) { return _this.widget.filterdataset = nv; }, getWatchIdentifier(this.widgetId, 'filterdataset')));
            }
        }
    };
    TableColumnDirective.prototype.loadInlineWidgetData = function () {
        // If dataset is not bound, get the data implicitly
        if (isDataSetWidget(this['edit-widget-type']) && !this.binddataset && !this.readonly) {
            var dataSource = this.table.datasource;
            if (this['related-entity-name'] && this['primary-key']) {
                // Fetch the data for the related fields
                this.isDataSetBound = true;
                var bindings = _.split(this.binding, '.');
                this.showPendingSpinner = true;
                fetchRelatedFieldData(dataSource, this.widget, {
                    relatedField: _.head(bindings),
                    datafield: _.last(bindings),
                    widget: 'edit-widget-type'
                });
            }
            else if (dataSource.execute(DataSource.Operation.SUPPORTS_DISTINCT_API)) {
                getDistinctValuesForField(dataSource, this.widget, {
                    widget: 'edit-widget-type'
                });
            }
        }
    };
    // On table datasource change, get the data for row filters
    TableColumnDirective.prototype.onDataSourceChange = function () {
        this.loadFilterData();
        if (this.table.editmode === EDIT_MODE.QUICK_EDIT) {
            this.loadInlineWidgetData();
        }
    };
    // Set the data on the row filter widget
    TableColumnDirective.prototype.setFilterWidgetDataSet = function () {
        if (this.filterInstance) {
            this.filterInstance.dataset = this._filterDataSet;
        }
    };
    // Set the props on the row filter widget
    TableColumnDirective.prototype.setUpFilterWidget = function () {
        var _this = this;
        this.filterInstance.registerReadyStateListener(function () {
            if (isDataSetWidget(_this.filterwidget)) {
                // if binding is department.deptId then field will be deptId
                var field = _.last(_this.binding.split('.'));
                _this.filterInstance.dataset = _this._filterDataSet;
                _this.filterInstance.datafield = _this.filterdatafield || field;
                _this.filterInstance.displayfield = _this.filterdisplayfield || field;
                if (_this.filterwidget === FormWidgetType.AUTOCOMPLETE) {
                    _this.filterInstance.displaylabel = _this.filterdisplaylabel || field;
                    _this.filterInstance.searchkey = _this.filtersearchkey || field;
                }
            }
            if (_this.filterwidget === FormWidgetType.TIME) {
                _this.filterInstance.timepattern = _this.appDefaults.timeFormat || 'hh:mm:ss a';
            }
            _this.filterInstance.placeholder = _this.filterplaceholder || '';
        });
    };
    // Notifies all the dependent validation controls incase of any changes
    TableColumnDirective.prototype.notifyChanges = function (quickEdit) {
        var _this = this;
        _.forEach(this.notifyForFields, function (field) {
            if (quickEdit && _this._isNewEditableRow) {
                field.fieldValidations_new.validate();
            }
            else {
                field.fieldValidations.validate();
            }
        });
    };
    // Watches control for dependent validation changes
    TableColumnDirective.prototype.observeOn = function (fields) {
        this.observeOnFields = _.cloneDeep(fields);
    };
    // Sets the default validators inline and quickedit new row using props
    TableColumnDirective.prototype.setUpValidators = function () {
        this.fieldValidations.setUpValidators();
        if (this._checkNewEditableRowControl()) {
            this.fieldValidations_new.setUpValidators();
        }
    };
    // Sets the Async validators on the inline and quickedit new row form control
    TableColumnDirective.prototype.setAsyncValidators = function (validators) {
        this.asyncValidators = _.cloneDeep(validators);
    };
    // Sets the default/custom validators on the inline and quickedit new row form control
    TableColumnDirective.prototype.setValidators = function (validators) {
        this.syncValidators = _.cloneDeep(validators);
    };
    TableColumnDirective.prototype.boundFn = function (fn) {
        return fn();
    };
    // Shows spinner for async validators based on status
    TableColumnDirective.prototype.onStatusChange = function (status, type) {
        this['showPendingSpinner' + (type === 'inlineInstance' ? '' : 'New')] = (status === 'PENDING');
    };
    /* Summary Row Logic start */
    TableColumnDirective.prototype.setSummaryRowData = function (data) {
        var newData = (data instanceof Array) ? data : [data];
        this._invokeSummaryRowData(newData);
    };
    TableColumnDirective.prototype._invokeSummaryRowData = function (data) {
        var _this = this;
        _.forEach(data, function (item, index) {
            var content = item;
            if (content instanceof Promise) {
                content.then(function (res) {
                    _this.table.callDataGridMethod('setSummaryRowDef', _this.key, res, index, true);
                });
            }
            else if (content.value && content.value instanceof Promise) {
                var contentData_1 = content;
                content.value.then(function (res) {
                    contentData_1.value = res;
                    _this.table.callDataGridMethod('setSummaryRowDef', _this.key, contentData_1, index, true);
                });
            }
            _this.table.callDataGridMethod('setSummaryRowDef', _this.key, content, index);
        });
    };
    TableColumnDirective.prototype._getColumnData = function () {
        return _.map(this.table.dataset, this.binding);
    };
    /* Summary Row Logic end */
    // Set the props on the inline edit widget
    TableColumnDirective.prototype.setInlineWidgetProp = function (widget, prop, nv) {
        if (prop === 'datepattern' && this.editWidgetType === FormWidgetType.TIME) {
            prop = 'timepattern';
        }
        if (this[widget] && isDefined(nv)) {
            this[widget][prop] = nv;
        }
    };
    // Initialize the inline edit widget
    TableColumnDirective.prototype.setUpInlineWidget = function (widget) {
        var _this = this;
        this[widget].registerReadyStateListener(function () {
            if (isDataSetWidget(_this['edit-widget-type'])) {
                _this[widget].dataset = _this.dataset;
            }
            inlineWidgetProps.forEach(function (key) {
                _this.setInlineWidgetProp(widget, key, _this[key]);
            });
            _this[widget].datasource = _this._datasource;
            _this[widget].dataoptions = _this._dataoptions;
            _this.setInlineWidgetProp(widget, 'datepattern', _this.editdatepattern);
        });
    };
    TableColumnDirective.prototype.getStyleDef = function () {
        return "width: " + (this.width || '') + "; background-color: " + (this.backgroundcolor || '') + "; color: " + (this.textcolor || '') + ";";
    };
    TableColumnDirective.prototype.populateFieldDef = function () {
        this.width = this.width === 'px' ? '' : (this.width || '');
        this.field = this.binding;
        this.displayName = this.caption || '';
        this.pcDisplay = this.pcdisplay;
        this.mobileDisplay = this.mobiledisplay;
        this.tabletDisplay = this.tabletdisplay;
        this.textAlignment = this.textalignment;
        this.backgroundColor = this.backgroundcolor;
        this.textColor = this.textcolor;
        this.primaryKey = this['primary-key'];
        this.relatedEntityName = this['related-entity-name'];
        this.style = this.getStyleDef();
        this.class = this['col-class'];
        this.ngclass = this['col-ng-class'];
        this.formatpattern = this.formatpattern === 'toNumber' ? 'numberToString' : this.formatpattern;
        this.searchable = (this.type === 'blob' || this.type === 'clob') ? false : this.searchable;
        this.limit = this.limit ? +this.limit : undefined;
        this.editWidgetType = this['edit-widget-type'] = this['edit-widget-type'] || getEditModeWidget(this);
        this.filterOn = this['filter-on'];
        this.readonly = isDefined(this.getAttr('readonly')) ? this.getAttr('readonly') === 'true' : (this['related-entity-name'] ? !this['primary-key'] : _.includes(['identity', 'uniqueid', 'sequence'], this.generator));
        this.filterwidget = this.filterwidget || getDataTableFilterWidget(this.type || 'string');
        this.isFilterDataSetBound = !!this.bindfilterdataset;
        this.defaultvalue = getDefaultValue(this.defaultvalue, this.type, this.editWidgetType);
        // For date time data types, if date pattern is not specified, set the app format or default format
        if (isDateTimeType(this.type) && this.formatpattern === 'toDate' && !this.datepattern) {
            var defaultFormat = getDisplayDateTimeFormat(this.type);
            if (this.type === DataType.DATE) {
                this.datepattern = this.appDefaults.dateFormat || defaultFormat;
            }
            else if (this.type === DataType.TIME) {
                this.datepattern = this.appDefaults.timeFormat || defaultFormat;
            }
            else if (this.type === DataType.TIMESTAMP || this.type === DataType.DATETIME) {
                this.datepattern = this.appDefaults.dateTimeFormat || defaultFormat;
            }
        }
    };
    TableColumnDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (!this._propsInitialized) {
            return;
        }
        switch (key) {
            case 'caption':
                this.displayName = nv || '';
                this.setProperty('displayName', this.displayName);
                break;
            case 'defaultvalue':
                this.defaultvalue = getDefaultValue(this.defaultvalue, this.type, this.editWidgetType);
                break;
            case 'show':
                this.table.redraw(true);
                break;
            case 'filterdataset':
                this._filterDataSet = nv;
                this.setFilterWidgetDataSet();
                break;
            case 'editdatepattern':
                this.setInlineWidgetProp('inlineInstance', 'datepattern', nv);
                this.setInlineWidgetProp('inlineInstanceNew', 'datepattern', nv);
                break;
            default:
                if (inlineWidgetProps.includes(key)) {
                    this.setInlineWidgetProp('inlineInstance', key, nv);
                    this.setInlineWidgetProp('inlineInstanceNew', key, nv);
                }
                break;
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    TableColumnDirective.prototype.setProperty = function (property, nv) {
        this[property] = nv;
        switch (property) {
            case 'displayName':
                this.table.callDataGridMethod('setColumnProp', this.field, property, nv);
                break;
            default:
                this.table.redraw(true);
        }
    };
    var TableColumnDirective_1;
    TableColumnDirective.initializeProps = registerProps$2();
    TableColumnDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: AppDefaults },
        { type: TableComponent, decorators: [{ type: Optional }] },
        { type: TableColumnGroupDirective, decorators: [{ type: Optional }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['filterdataset.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] }
    ]; };
    __decorate([
        ContentChildren('filterWidget'),
        __metadata("design:type", Object)
    ], TableColumnDirective.prototype, "_filterInstances", void 0);
    __decorate([
        ContentChildren('inlineWidget'),
        __metadata("design:type", Object)
    ], TableColumnDirective.prototype, "_inlineInstances", void 0);
    __decorate([
        ContentChildren('inlineWidgetNew'),
        __metadata("design:type", Object)
    ], TableColumnDirective.prototype, "_inlineInstancesNew", void 0);
    __decorate([
        ContentChild('customExprTmpl'),
        __metadata("design:type", Object)
    ], TableColumnDirective.prototype, "customExprTmpl", void 0);
    __decorate([
        ContentChild('inlineWidgetTmpl'),
        __metadata("design:type", Object)
    ], TableColumnDirective.prototype, "inlineWidthTempRef", void 0);
    __decorate([
        ContentChild('filterTmpl'),
        __metadata("design:type", TemplateRef)
    ], TableColumnDirective.prototype, "filterTemplateRef", void 0);
    TableColumnDirective = TableColumnDirective_1 = __decorate([
        Directive({
            selector: '[wmTableColumn]',
            providers: [
                provideAsWidgetRef(TableColumnDirective_1)
            ]
        }),
        __param(2, Optional()),
        __param(3, Optional()),
        __param(4, Attribute('filterdataset.bind')),
        __param(5, Attribute('dataset.bind')),
        __metadata("design:paramtypes", [Injector,
            AppDefaults,
            TableComponent,
            TableColumnGroupDirective, Object, Object])
    ], TableColumnDirective);
    return TableColumnDirective;
}(BaseComponent));

var registerProps$4 = function () {
    register('wm-table-row', new Map([
        ['class', PROP_STRING],
        ['closeothers', __assign({ value: true }, PROP_BOOLEAN)],
        ['content', PROP_STRING],
        ['columnwidth', __assign({ value: '30px' }, PROP_STRING)],
        ['collapseicon', PROP_STRING],
        ['expandicon', PROP_STRING],
        ['height', PROP_STRING],
        ['position', __assign({ value: '0' }, PROP_STRING)]
    ]));
};

var WIDGET_CONFIG$4 = { widgetType: 'wm-table-row', hostClass: '' };
var TableRowDirective = /** @class */ (function (_super) {
    __extends(TableRowDirective, _super);
    function TableRowDirective(inj, table) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$4) || this;
        _this.table = table;
        return _this;
    }
    TableRowDirective_1 = TableRowDirective;
    TableRowDirective.prototype.populateConfig = function () {
        this.config = {
            closeothers: this.closeothers,
            content: this.content,
            columnwidth: this.columnwidth,
            expandicon: this.expandicon,
            collapseicon: this.collapseicon,
            height: this.height,
            position: this.position
        };
    };
    TableRowDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.populateConfig();
        this.table.registerRow(this.config, this);
    };
    TableRowDirective.prototype.onPropertyChange = function (key, nv) {
        if (key === 'content' && this.config) {
            this.config.content = this.content;
        }
    };
    var TableRowDirective_1;
    TableRowDirective.initializeProps = registerProps$4();
    TableRowDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: TableComponent, decorators: [{ type: Optional }, { type: SkipSelf }] }
    ]; };
    TableRowDirective = TableRowDirective_1 = __decorate([
        Directive({
            selector: '[wmTableRow]',
            providers: [
                provideAsWidgetRef(TableRowDirective_1)
            ]
        }),
        __param(1, Optional()), __param(1, SkipSelf()),
        __metadata("design:paramtypes", [Injector,
            TableComponent])
    ], TableRowDirective);
    return TableRowDirective;
}(BaseComponent));

var registerProps$5 = function () {
    register('wm-table-row-action', new Map([
        ['accessroles', PROP_STRING],
        ['action', PROP_STRING],
        ['caption', PROP_STRING],
        ['class ', __assign({ value: 'btn-secondary' }, PROP_STRING)],
        ['disabled', __assign({ value: false }, PROP_BOOLEAN)],
        ['display-name', PROP_STRING],
        ['hyperlink', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['key', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['target', PROP_STRING],
        ['title', PROP_STRING],
        ['widget-type', __assign({ value: 'button' }, PROP_STRING)]
    ]));
};

var WIDGET_CONFIG$5 = { widgetType: 'wm-table-row-action', hostClass: '' };
var ɵ0$2 = {};
var TableRowActionDirective = /** @class */ (function (_super) {
    __extends(TableRowActionDirective, _super);
    function TableRowActionDirective(inj, table, contexts) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$5) || this;
        _this.table = table;
        contexts[0].editRow = function (evt) { return _this.table.editRow(evt); };
        contexts[0].deleteRow = function (evt) { return _this.table.deleteRow(evt); };
        contexts[0].addNewRow = function (evt) { return _this.table.addNewRow(evt); };
        return _this;
    }
    TableRowActionDirective_1 = TableRowActionDirective;
    TableRowActionDirective.prototype.getTitle = function () {
        return _.isUndefined(this.title) ? (this['display-name'] || '') : this.title;
    };
    TableRowActionDirective.prototype.populateAction = function () {
        this.buttonDef = {
            key: this.key,
            displayName: this['display-name'] || this.caption || '',
            show: this.show,
            class: this.class || '',
            iconclass: this.iconclass || '',
            title: this.getTitle(),
            action: this.action,
            accessroles: this.accessroles,
            disabled: this.disabled,
            tabindex: this.tabindex ? +this.tabindex : undefined,
            widgetType: this['widget-type'] || 'button',
            hyperlink: this.hyperlink,
            target: this.target || '',
            conditionalclass: this.conditionalclass || '',
            conditionalstyle: this.conditionalstyle || {}
        };
    };
    TableRowActionDirective.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.populateAction();
        this.table.registerRowActions(this.buttonDef);
    };
    var TableRowActionDirective_1;
    TableRowActionDirective.initializeProps = registerProps$5();
    TableRowActionDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: TableComponent, decorators: [{ type: Optional }] },
        { type: Array, decorators: [{ type: Self }, { type: Inject, args: [Context,] }] }
    ]; };
    TableRowActionDirective = TableRowActionDirective_1 = __decorate([
        Directive({
            selector: '[wmTableRowAction]',
            providers: [
                provideAsWidgetRef(TableRowActionDirective_1),
                { provide: Context, useValue: ɵ0$2, multi: true }
            ]
        }),
        __param(1, Optional()),
        __param(2, Self()), __param(2, Inject(Context)),
        __metadata("design:paramtypes", [Injector,
            TableComponent,
            Array])
    ], TableRowActionDirective);
    return TableRowActionDirective;
}(BaseComponent));

var components = [
    TableComponent,
    TableCUDDirective,
    TableFilterSortDirective,
    TableActionDirective,
    TableColumnDirective,
    TableColumnGroupDirective,
    TableRowDirective,
    TableRowActionDirective
];
var TableModule = /** @class */ (function () {
    function TableModule() {
    }
    TableModule = __decorate([
        NgModule({
            imports: [
                BasicModule,
                BsDropdownModule,
                CommonModule,
                FormsModule,
                ReactiveFormsModule,
                InputModule,
                ListModule,
                MenuModule,
                PaginationModule,
                TooltipModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], TableModule);
    return TableModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { TableComponent, TableModule, ɵ0, ɵ1, registerProps as ɵa, TableCUDDirective as ɵb, TableFilterSortDirective as ɵc, TableActionDirective as ɵd, registerProps$1 as ɵe, TableColumnDirective as ɵf, TableColumnGroupDirective as ɵg, registerProps$3 as ɵh, registerProps$2 as ɵi, TableRowDirective as ɵj, registerProps$4 as ɵk, TableRowActionDirective as ɵl, registerProps$5 as ɵm };
//# sourceMappingURL=index.js.map
