import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Attribute, ViewChild, TemplateRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, provideAsWidgetRef, provideAsDialogRef, WmComponentsModule } from '@wm/components/base';
import { BaseDialog, DialogModule } from '@wm/components/dialogs';
import { InputModule } from '@wm/components/input';
import { toBoolean } from '@wm/core';

const registerProps = () => {
    register('wm-alertdialog', new Map([
        ['alerttype', Object.assign({ value: 'error' }, PROP_STRING)],
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', { value: true, PROP_BOOLEAN }],
        ['iconclass', { value: 'wi wi-warning', PROP_STRING }],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['message', Object.assign({ value: 'Am an alert box!' }, PROP_STRING)],
        ['modal', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['oktext', Object.assign({ value: 'OK' }, PROP_STRING)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['title', Object.assign({ value: 'Alert' }, PROP_STRING)]
    ]));
};

var AlertDialogComponent_1;
const DIALOG_CLS = 'app-dialog modal-dialog app-alert-dialog';
const WIDGET_INFO = { widgetType: 'wm-alertdialog' };
let AlertDialogComponent = AlertDialogComponent_1 = class AlertDialogComponent extends BaseDialog {
    constructor(inj, dialogClass, modal, closable) {
        if (modal === null || modal === undefined) {
            modal = false;
        }
        if (closable === null || closable === undefined) {
            closable = true;
        }
        // setting the backdrop to 'static' will not close the dialog on backdrop click
        const backdrop = toBoolean(modal) ? 'static' : true;
        super(inj, WIDGET_INFO, {
            class: `${DIALOG_CLS} ${dialogClass || ''}`,
            backdrop,
            keyboard: !toBoolean(modal)
        });
    }
    getTemplateRef() {
        return this.dialogTemplate;
    }
    /**
     * Click event handler for the ok button
     * invokes on-ok event callback
     * @param {Event} $event
     */
    onOk($event) {
        this.invokeEventCallback('ok', { $event });
    }
    ngOnInit() {
        super.ngOnInit();
        this.register(this.viewParent);
    }
};
AlertDialogComponent.initializeProps = registerProps();
AlertDialogComponent.ctorParameters = () => [
    { type: Injector },
    { type: String, decorators: [{ type: Attribute, args: ['class',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['modal',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['closable',] }] }
];
__decorate([
    ViewChild('dialogTemplate', { static: true }),
    __metadata("design:type", TemplateRef)
], AlertDialogComponent.prototype, "dialogTemplate", void 0);
AlertDialogComponent = AlertDialogComponent_1 = __decorate([
    Component({
        selector: 'div[wmAlertDialog]',
        template: "<ng-template #dialogTemplate>\n    <div wmDialogHeader [closable]=\"closable\"\n         [iconclass]=\"iconclass\"\n         [iconurl]=\"iconurl\"\n         [iconwidth]=\"iconwidth\"\n         [iconheight]=\"iconheight\"\n         [iconmargin]=\"iconmargin\"\n         [heading]=\"title\"></div>\n    <div wmDialogBody>\n        <p class=\"app-dialog-message text-{{alerttype}}\" [attr.aria-describedby]=\"message\" [textContent]=\"message\"></p>\n    </div>\n    <div wmDialogFooter>\n        <button wmButton class=\"btn-primary ok-action\" caption.bind=\"oktext\" aria-label=\"Submit button\" (click)=\"onOk($event)\"></button>\n    </div>\n</ng-template>\n",
        providers: [
            provideAsWidgetRef(AlertDialogComponent_1),
            provideAsDialogRef(AlertDialogComponent_1)
        ]
    }),
    __param(1, Attribute('class')),
    __param(2, Attribute('modal')),
    __param(3, Attribute('closable')),
    __metadata("design:paramtypes", [Injector, String, Object, Object])
], AlertDialogComponent);

const components = [
    AlertDialogComponent
];
let AlertDialogModule = class AlertDialogModule {
};
AlertDialogModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            DialogModule,
            InputModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], AlertDialogModule);

/**
 * Generated bundle index. Do not edit.
 */

export { AlertDialogModule, AlertDialogComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
