import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Attribute, ViewChild, TemplateRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, provideAsWidgetRef, provideAsDialogRef, WmComponentsModule } from '@wm/components/base';
import { BaseDialog, DialogModule } from '@wm/components/dialogs';
import { InputModule } from '@wm/components/input';
import { toBoolean } from '@wm/core';

var registerProps = function () {
    register('wm-alertdialog', new Map([
        ['alerttype', __assign({ value: 'error' }, PROP_STRING)],
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', { value: true, PROP_BOOLEAN: PROP_BOOLEAN }],
        ['iconclass', { value: 'wi wi-warning', PROP_STRING: PROP_STRING }],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['message', __assign({ value: 'Am an alert box!' }, PROP_STRING)],
        ['modal', __assign({ value: false }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['oktext', __assign({ value: 'OK' }, PROP_STRING)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', __assign({ value: 'Alert' }, PROP_STRING)]
    ]));
};

var DIALOG_CLS = 'app-dialog modal-dialog app-alert-dialog';
var WIDGET_INFO = { widgetType: 'wm-alertdialog' };
var AlertDialogComponent = /** @class */ (function (_super) {
    __extends(AlertDialogComponent, _super);
    function AlertDialogComponent(inj, dialogClass, modal, closable) {
        var _this = this;
        if (modal === null || modal === undefined) {
            modal = false;
        }
        if (closable === null || closable === undefined) {
            closable = true;
        }
        // setting the backdrop to 'static' will not close the dialog on backdrop click
        var backdrop = toBoolean(modal) ? 'static' : true;
        _this = _super.call(this, inj, WIDGET_INFO, {
            class: DIALOG_CLS + " " + (dialogClass || ''),
            backdrop: backdrop,
            keyboard: !toBoolean(modal)
        }) || this;
        return _this;
    }
    AlertDialogComponent_1 = AlertDialogComponent;
    AlertDialogComponent.prototype.getTemplateRef = function () {
        return this.dialogTemplate;
    };
    /**
     * Click event handler for the ok button
     * invokes on-ok event callback
     * @param {Event} $event
     */
    AlertDialogComponent.prototype.onOk = function ($event) {
        this.invokeEventCallback('ok', { $event: $event });
    };
    AlertDialogComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.register(this.viewParent);
    };
    var AlertDialogComponent_1;
    AlertDialogComponent.initializeProps = registerProps();
    AlertDialogComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: String, decorators: [{ type: Attribute, args: ['class',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['modal',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['closable',] }] }
    ]; };
    __decorate([
        ViewChild('dialogTemplate', { static: true }),
        __metadata("design:type", TemplateRef)
    ], AlertDialogComponent.prototype, "dialogTemplate", void 0);
    AlertDialogComponent = AlertDialogComponent_1 = __decorate([
        Component({
            selector: 'div[wmAlertDialog]',
            template: "<ng-template #dialogTemplate>\n    <div wmDialogHeader [closable]=\"closable\"\n         [iconclass]=\"iconclass\"\n         [iconurl]=\"iconurl\"\n         [iconwidth]=\"iconwidth\"\n         [iconheight]=\"iconheight\"\n         [iconmargin]=\"iconmargin\"\n         [heading]=\"title\"></div>\n    <div wmDialogBody>\n        <p class=\"app-dialog-message text-{{alerttype}}\" [attr.aria-describedby]=\"message\" [textContent]=\"message\"></p>\n    </div>\n    <div wmDialogFooter>\n        <button wmButton class=\"btn-primary ok-action\" caption.bind=\"oktext\" aria-label=\"Submit button\" (click)=\"onOk($event)\"></button>\n    </div>\n</ng-template>\n",
            providers: [
                provideAsWidgetRef(AlertDialogComponent_1),
                provideAsDialogRef(AlertDialogComponent_1)
            ]
        }),
        __param(1, Attribute('class')),
        __param(2, Attribute('modal')),
        __param(3, Attribute('closable')),
        __metadata("design:paramtypes", [Injector, String, Object, Object])
    ], AlertDialogComponent);
    return AlertDialogComponent;
}(BaseDialog));

var components = [
    AlertDialogComponent
];
var AlertDialogModule = /** @class */ (function () {
    function AlertDialogModule() {
    }
    AlertDialogModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                DialogModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], AlertDialogModule);
    return AlertDialogModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { AlertDialogModule, AlertDialogComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
