import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Attribute, ViewChild, TemplateRef, HostListener, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, provideAsWidgetRef, provideAsDialogRef, WmComponentsModule } from '@wm/components/base';
import { BaseDialog, DialogModule } from '@wm/components/dialogs';
import { InputModule } from '@wm/components/input';
import { toBoolean } from '@wm/core';

var registerProps = function () {
    register('wm-confirmdialog', new Map([
        ['animation', PROP_STRING],
        ['canceltext', __assign({ value: 'CANCEL' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['closable', { value: true, PROP_BOOLEAN: PROP_BOOLEAN }],
        ['iconclass', { value: 'wi wi-done', PROP_STRING: PROP_STRING }],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['message', __assign({ value: 'I am confirm box!' }, PROP_STRING)],
        ['modal', __assign({ value: false }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['oktext', __assign({ value: 'OK' }, PROP_STRING)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', __assign({ value: 'Confirm' }, PROP_STRING)]
    ]));
};

var DIALOG_CLS = 'app-dialog modal-dialog app-confirm-dialog';
var WIDGET_INFO = { widgetType: 'wm-confirmdialog' };
var ConfirmDialogComponent = /** @class */ (function (_super) {
    __extends(ConfirmDialogComponent, _super);
    function ConfirmDialogComponent(inj, dialogClass, modal, closable) {
        var _this = this;
        if (modal === null || modal === undefined) {
            modal = false;
        }
        if (closable === null || closable === undefined) {
            closable = true;
        }
        // setting the backdrop to 'static' will not close the dialog on backdrop click
        var backdrop = 'static';
        _this = _super.call(this, inj, WIDGET_INFO, {
            class: DIALOG_CLS + " " + (dialogClass || ''),
            backdrop: backdrop,
            keyboard: !toBoolean(modal)
        }) || this;
        return _this;
    }
    ConfirmDialogComponent_1 = ConfirmDialogComponent;
    ConfirmDialogComponent.prototype.onEscape = function ($event) { };
    ;
    ConfirmDialogComponent.prototype.getTemplateRef = function () {
        return this.dialogTemplate;
    };
    /**
     * Click event handler for the ok button
     * invokes on-ok event callback
     * @param {Event} $event
     */
    ConfirmDialogComponent.prototype.onOk = function ($event) {
        this.invokeEventCallback('ok', { $event: $event });
    };
    /**
     * Click event handler for the cancel button
     * invokes on-cancel event callback
     * @param {Event} $event
     */
    ConfirmDialogComponent.prototype.onCancel = function ($event) {
        this.invokeEventCallback('cancel', { $event: $event });
    };
    ConfirmDialogComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.register(this.viewParent);
    };
    var ConfirmDialogComponent_1;
    ConfirmDialogComponent.initializeProps = registerProps();
    ConfirmDialogComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: String, decorators: [{ type: Attribute, args: ['class',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['modal',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['closable',] }] }
    ]; };
    __decorate([
        ViewChild('dialogTemplate', { static: true }),
        __metadata("design:type", TemplateRef)
    ], ConfirmDialogComponent.prototype, "dialogTemplate", void 0);
    __decorate([
        HostListener('window:keydown.esc', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [KeyboardEvent]),
        __metadata("design:returntype", void 0)
    ], ConfirmDialogComponent.prototype, "onEscape", null);
    ConfirmDialogComponent = ConfirmDialogComponent_1 = __decorate([
        Component({
            selector: 'div[wmConfirmDialog]',
            template: "<ng-template #dialogTemplate>\n    <div wmDialogHeader [closable]=\"closable\"\n         [iconclass]=\"iconclass\"\n         [iconurl]=\"iconurl\"\n         [iconwidth]=\"iconwidth\"\n         [iconheight]=\"iconheight\"\n         [iconmargin]=\"iconmargin\"\n         [heading]=\"title\"></div>\n    <div wmDialogBody>\n        <p class=\"app-dialog-message {{messageclass}}\" [attr.aria-describedby]=\"message\">{{message}}</p>\n    </div>\n    <div wmDialogFooter>\n        <button wmButton class=\"btn-default cancel-action\" caption.bind=\"canceltext\" aria-label=\"Cancel button\" (click)=\"onCancel($event)\"></button>\n        <button wmButton class=\"btn-primary ok-action\" caption.bind=\"oktext\" aria-label=\"Submit button\" (click)=\"onOk($event)\"></button>\n    </div>\n</ng-template>\n",
            providers: [
                provideAsWidgetRef(ConfirmDialogComponent_1),
                provideAsDialogRef(ConfirmDialogComponent_1)
            ]
        }),
        __param(1, Attribute('class')),
        __param(2, Attribute('modal')),
        __param(3, Attribute('closable')),
        __metadata("design:paramtypes", [Injector, String, Object, Object])
    ], ConfirmDialogComponent);
    return ConfirmDialogComponent;
}(BaseDialog));

var components = [
    ConfirmDialogComponent
];
var ConfirmDialogModule = /** @class */ (function () {
    function ConfirmDialogModule() {
    }
    ConfirmDialogModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                DialogModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], ConfirmDialogModule);
    return ConfirmDialogModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ConfirmDialogModule, ConfirmDialogComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
