import { __decorate, __metadata, __param, __assign, __extends, __spread } from 'tslib';
import { ElementRef, Inject, HostBinding, Directive, Injector, Input, Component, NgModule } from '@angular/core';
import { BsModalService, ModalModule } from 'ngx-bootstrap/modal';
import { addClass, setCSS, setAttr, findRootContainer, AbstractDialogService, toBoolean } from '@wm/core';
import { DialogRef, register, PROP_STRING, PROP_BOOLEAN, provideAsWidgetRef, BaseComponent, WmComponentsModule } from '@wm/components/base';
import { Router, NavigationStart } from '@angular/router';
import { CommonModule } from '@angular/common';

var DEFAULT_CLS = 'app-dialog-body modal-body';
var DialogBodyDirective = /** @class */ (function () {
    function DialogBodyDirective(elRef, dialogRef, bsModal) {
        var _this = this;
        this.dialogRef = dialogRef;
        addClass(elRef.nativeElement, DEFAULT_CLS);
        var subscription = bsModal.onShown.subscribe(function () {
            var dialogRoot = $(elRef.nativeElement).closest('.app-dialog')[0];
            var width = _this.dialogRef.width;
            var height = _this.dialogRef.height;
            if (height) {
                setTimeout(function (e) {
                    setCSS($(elRef.nativeElement)[0], 'height', height);
                });
            }
            if (dialogRoot) {
                if (width) {
                    setCSS(dialogRoot, 'width', width);
                }
                setAttr(dialogRoot, 'tabindex', _this.dialogRef.tabindex);
                setAttr(dialogRoot, 'name', _this.dialogRef.name);
            }
            subscription.unsubscribe();
        });
    }
    DialogBodyDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: undefined, decorators: [{ type: Inject, args: [DialogRef,] }] },
        { type: BsModalService }
    ]; };
    __decorate([
        HostBinding('style.height'),
        __metadata("design:type", Object)
    ], DialogBodyDirective.prototype, "height", void 0);
    DialogBodyDirective = __decorate([
        Directive({
            selector: 'div[wmDialogBody]',
        }),
        __param(1, Inject(DialogRef)),
        __metadata("design:paramtypes", [ElementRef, Object, BsModalService])
    ], DialogBodyDirective);
    return DialogBodyDirective;
}());

var registerProps = function () {
    register('wm-dialogfooter', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var WIDGET_INFO = {
    widgetType: 'wm-dialogfooter',
    hostClass: 'app-dialog-footer modal-footer'
};
var DialogFooterDirective = /** @class */ (function (_super) {
    __extends(DialogFooterDirective, _super);
    function DialogFooterDirective(inj) {
        return _super.call(this, inj, WIDGET_INFO) || this;
    }
    DialogFooterDirective_1 = DialogFooterDirective;
    var DialogFooterDirective_1;
    DialogFooterDirective.initializeProps = registerProps();
    DialogFooterDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    DialogFooterDirective = DialogFooterDirective_1 = __decorate([
        Directive({
            selector: 'div[wmDialogFooter]',
            providers: [
                provideAsWidgetRef(DialogFooterDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], DialogFooterDirective);
    return DialogFooterDirective;
}(BaseComponent));

var eventsRegistered = false;
var invokeOpenedCallback = function (ref) {
    if (ref) {
        setTimeout(function () {
            var root = findRootContainer(ref.$element);
            // if page styles have to be applied to dialog then dialog has to be child of page element.
            if (root) {
                $('body > modal-container > div').wrap('<' + root + '/>');
            }
            ref.invokeEventCallback('opened', { $event: { type: 'opened' } });
        });
    }
};
var ɵ0 = invokeOpenedCallback;
var invokeClosedCallback = function (ref) {
    if (ref) {
        ref.invokeEventCallback('close');
        ref.dialogRef = undefined;
    }
};
var ɵ1 = invokeClosedCallback;
var BaseDialog = /** @class */ (function (_super) {
    __extends(BaseDialog, _super);
    function BaseDialog(inj, widgetConfig, modalOptions) {
        var _this = _super.call(this, inj, widgetConfig) || this;
        _this.modalOptions = modalOptions;
        _this.dialogService = inj.get(AbstractDialogService);
        _this.bsModal = inj.get(BsModalService);
        var router = inj.get(Router);
        var subscriptions = [
            _this.bsModal.onShown.subscribe(function () {
                var ref = _this.dialogService.getLastOpenedDialog();
                if (ref === _this) {
                    // Always get the reference of last pushed dialog in the array for calling onOpen callback
                    invokeOpenedCallback(ref);
                }
            }),
            _this.bsModal.onHidden.subscribe(function (closeReason) {
                var ref = closeReason === 'esc' || closeReason === 'backdrop-click' ? _this.dialogService.getLastOpenedDialog() : _this.dialogService.getDialogRefFromClosedDialogs();
                if (ref === _this) {
                    // remove the dialog reference from opened dialogs and closed dialogs
                    _this.dialogService.removeFromOpenedDialogs(ref);
                    _this.dialogService.removeFromClosedDialogs(ref);
                    invokeClosedCallback(ref);
                    if (ref.closeCallBackFn) {
                        ref.closeCallBackFn();
                    }
                }
            }),
            router.events.subscribe(function (e) {
                if (e instanceof NavigationStart) {
                    _this.close();
                }
            })
        ];
        _this.registerDestroyListener(function () {
            _this.dialogService.removeFromOpenedDialogs(_this);
            _this.dialogService.removeFromClosedDialogs(_this);
            subscriptions.forEach(function (s) { return s.unsubscribe(); });
        });
        return _this;
    }
    /**
     * Opens the dialog
     * Subscribe to the onShown event emitter of bsModal and trigger on-opened event callback
     */
    BaseDialog.prototype.open = function (initState) {
        // remove the popovers in the page to avoid the overlap with dialog
        // closePopover(this.$element); Commenting this line because it is causing regression(if we have dialog inside popover as partail content, then the dialog close is not working because on closing the popover the partial get destroyed.)
        var _this = this;
        // do not open the dialog again if it is already opened
        var duplicateDialogCheck = function (openedDialog) {
            return openedDialog === _this;
        };
        if (this.dialogService.getOpenedDialogs().some(duplicateDialogCheck)) {
            return;
        }
        this.dialogService.addToOpenedDialogs(this);
        // extend the context with the initState
        Object.assign(this.context, initState);
        this.dialogRef = this.bsModal.show(this.getTemplateRef(), this.modalOptions);
    };
    /**
     * closes the dialog
     * invokes the on-close event callback
     */
    BaseDialog.prototype.close = function () {
        // remove the popovers in the page to avoid the overlap with dialog
        // closePopover(this.$element); Commenting this line because it is causing regression(if we have dialog inside popover as partail content, then the dialog close is not working because on closing the popover the partial get destroyed.)
        if (this.dialogRef) {
            this.dialogService.addToClosedDialogs(this);
            this.dialogRef.hide();
        }
    };
    /**
     * Register the dialog with the dialog service for programmatic access
     */
    BaseDialog.prototype.register = function (scope) {
        // add scope along with name in the dialogRefsCollection Map while registering dialog
        // So that 2 dialogs having same name on different pages won't be overridden.
        this.dialogService.register(this.name, this, scope);
    };
    /**
     * De Register the dialog with the dialog service after dialog destruction
     */
    BaseDialog.prototype.deRegister = function (scope) {
        this.dialogService.deRegister(this.name, scope);
    };
    BaseDialog.prototype.onPropertyChange = function (key, nv, ov) {
        // ignore the class attribute.
        // Prevent the framework from setting the class on the host element.
        if (key === 'class' || key === 'name' || key === 'tabindex') {
            return;
        }
        else if (key === 'animation') {
            this.modalOptions.class = this.modalOptions.class.replace(' animated ' + ov, '');
            if (nv) {
                this.modalOptions.class = this.modalOptions.class + ' animated ' + nv;
            }
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    BaseDialog.prototype.ngOnDestroy = function () {
        this.close();
        this.deRegister(this.viewParent);
        _super.prototype.ngOnDestroy.call(this);
    };
    return BaseDialog;
}(BaseComponent));

var DEFAULT_CLS$1 = 'app-dialog-header modal-header';
var DEFAULT_ICON_DIMENSIONS = '21px';
var DialogHeaderComponent = /** @class */ (function () {
    function DialogHeaderComponent(elRef, dialogRef) {
        this.dialogRef = dialogRef;
        this.iconwidth = DEFAULT_ICON_DIMENSIONS;
        this.iconheight = DEFAULT_ICON_DIMENSIONS;
        this.closable = true;
        addClass(elRef.nativeElement, DEFAULT_CLS$1);
    }
    Object.defineProperty(DialogHeaderComponent.prototype, "isClosable", {
        get: function () {
            return toBoolean(this.closable);
        },
        enumerable: true,
        configurable: true
    });
    DialogHeaderComponent.prototype.closeDialog = function () {
        this.dialogRef.close();
    };
    DialogHeaderComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: BaseDialog, decorators: [{ type: Inject, args: [DialogRef,] }] }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DialogHeaderComponent.prototype, "iconwidth", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DialogHeaderComponent.prototype, "iconheight", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DialogHeaderComponent.prototype, "iconmargin", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DialogHeaderComponent.prototype, "iconclass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DialogHeaderComponent.prototype, "iconurl", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DialogHeaderComponent.prototype, "closable", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DialogHeaderComponent.prototype, "heading", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DialogHeaderComponent.prototype, "subheading", void 0);
    DialogHeaderComponent = __decorate([
        Component({
            selector: 'div[wmDialogHeader]',
            template: "<button *ngIf=\"isClosable\" aria-label=\"Close\" class=\"app-dialog-close close\" (click)=\"closeDialog();\" title=\"Close\">\n    <span aria-hidden=\"true\">&times;</span>\n</button>\n<h4 class=\"app-dialog-title modal-title\">\n    <i [ngClass]=\"iconclass\" [ngStyle]=\"{width: iconwidth, height: iconheight, margin: iconmargin}\"  *ngIf=\"iconclass && !iconurl\"></i>\n    <img data-identifier=\"img\" [src]=\"iconurl | image\" *ngIf=\"iconurl\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin}\"/>\n    <span class=\"dialog-heading\" [textContent]=\"heading\"></span>\n    <span class=\"dialog-sub-heading\" *ngIf=\"subheading\" [title]=\"subheading\" [textContent]=\"subheading\"></span>\n</h4>"
        }),
        __param(1, Inject(DialogRef)),
        __metadata("design:paramtypes", [ElementRef, BaseDialog])
    ], DialogHeaderComponent);
    return DialogHeaderComponent;
}());

var components = [
    DialogBodyDirective,
    DialogFooterDirective,
    DialogHeaderComponent
];
var DialogModule = /** @class */ (function () {
    function DialogModule() {
    }
    DialogModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                ModalModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], DialogModule);
    return DialogModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { BaseDialog, DialogBodyDirective, DialogFooterDirective, DialogHeaderComponent, DialogModule, ɵ0, ɵ1, registerProps as ɵa };
//# sourceMappingURL=index.js.map
