import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Attribute, Self, Inject, ViewChild, TemplateRef, ContentChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, PROP_ANY, Context, provideAsWidgetRef, provideAsDialogRef, WmComponentsModule } from '@wm/components/base';
import { BaseDialog, DialogModule } from '@wm/components/dialogs';
import { InputModule } from '@wm/components/input';
import { toBoolean } from '@wm/core';

var registerProps = function () {
    register('wm-dialog', new Map([
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', { value: true, PROP_BOOLEAN: PROP_BOOLEAN }],
        ['iconclass', { value: 'wi wi-file-text', PROP_STRING: PROP_STRING }],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['modal', __assign({ value: true }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['showheader', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', __assign({ value: 'Information' }, PROP_STRING)],
        ['eventsource', PROP_ANY]
    ]));
};

var DIALOG_CLS = 'app-dialog modal-dialog';
var WIDGET_INFO = { widgetType: 'wm-dialog' };
var ɵ0 = {};
var DialogComponent = /** @class */ (function (_super) {
    __extends(DialogComponent, _super);
    function DialogComponent(inj, dialogClass, modal, closable, contexts) {
        var _this = this;
        if (modal === null || modal === undefined) {
            modal = true;
        }
        if (closable === null || closable === undefined) {
            closable = true;
        }
        // contexts[0] will refer to the self context provided by this component
        contexts[0].closeDialog = function () { return _this.close(); };
        // setting the backdrop to 'static' will not close the dialog on backdrop click
        var backdrop = toBoolean(modal) ? 'static' : true;
        _this = _super.call(this, inj, WIDGET_INFO, {
            class: DIALOG_CLS + " " + (dialogClass || ''),
            backdrop: backdrop,
            keyboard: !toBoolean(modal)
        }) || this;
        return _this;
    }
    DialogComponent_1 = DialogComponent;
    DialogComponent.prototype.getTemplateRef = function () {
        return this.dialogTemplate;
    };
    DialogComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.register(this.viewParent);
    };
    var DialogComponent_1;
    DialogComponent.initializeProps = registerProps();
    DialogComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: String, decorators: [{ type: Attribute, args: ['class',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['modal',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['closable',] }] },
        { type: Array, decorators: [{ type: Self }, { type: Inject, args: [Context,] }] }
    ]; };
    __decorate([
        ViewChild('dialogTemplate', { static: true, read: TemplateRef }),
        __metadata("design:type", TemplateRef)
    ], DialogComponent.prototype, "dialogTemplate", void 0);
    __decorate([
        ContentChild('dialogBody', /* TODO: add static flag */ { read: TemplateRef }),
        __metadata("design:type", TemplateRef)
    ], DialogComponent.prototype, "dialogBody", void 0);
    __decorate([
        ContentChild('dialogFooter', /* TODO: add static flag */ { read: TemplateRef }),
        __metadata("design:type", TemplateRef)
    ], DialogComponent.prototype, "dialogFooter", void 0);
    DialogComponent = DialogComponent_1 = __decorate([
        Component({
            selector: 'div[wmDialog]',
            template: "<ng-template #dialogTemplate>\n    <div wmDialogHeader [closable]=\"closable\"\n         [iconclass]=\"iconclass\"\n         [iconurl]=\"iconurl\"\n         [iconwidth]=\"iconwidth\"\n         [iconheight]=\"iconheight\"\n         [iconmargin]=\"iconmargin\"\n         [heading]=\"title\"\n         *ngIf=\"showheader\"\n    ></div>\n    <div wmDialogBody>\n        <ng-container *ngTemplateOutlet=\"dialogBody\"></ng-container>\n    </div>\n    <ng-container *ngTemplateOutlet=\"dialogFooter\"></ng-container>\n</ng-template>",
            providers: [
                provideAsWidgetRef(DialogComponent_1),
                provideAsDialogRef(DialogComponent_1),
                { provide: Context, useValue: ɵ0, multi: true }
            ]
        }),
        __param(1, Attribute('class')),
        __param(2, Attribute('modal')),
        __param(3, Attribute('closable')),
        __param(4, Self()), __param(4, Inject(Context)),
        __metadata("design:paramtypes", [Injector, String, Object, Object, Array])
    ], DialogComponent);
    return DialogComponent;
}(BaseDialog));

var components = [
    DialogComponent
];
var DesignDialogModule = /** @class */ (function () {
    function DesignDialogModule() {
    }
    DesignDialogModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                DialogModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], DesignDialogModule);
    return DesignDialogModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { DesignDialogModule, DialogComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
