import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Attribute, ViewChild, TemplateRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, provideAsWidgetRef, provideAsDialogRef, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { BaseDialog, DialogModule } from '@wm/components/dialogs';
import { InputModule } from '@wm/components/input';
import { toBoolean } from '@wm/core';

var registerProps = function () {
    register('wm-iframedialog', new Map([
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', { value: true, PROP_BOOLEAN: PROP_BOOLEAN }],
        ['encodeurl', __assign({ value: false }, PROP_BOOLEAN)],
        ['height', __assign({ value: '400px' }, PROP_STRING)],
        ['iconclass', { value: 'wi wi-globe', PROP_STRING: PROP_STRING }],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['modal', __assign({ value: false }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['oktext', __assign({ value: 'OK' }, PROP_STRING)],
        ['showactions', __assign({ value: true }, PROP_BOOLEAN)],
        ['showheader', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', __assign({ value: 'External Content' }, PROP_STRING)],
        ['url', __assign({ value: '//www.wavemaker.com' }, PROP_STRING)]
    ]));
};

var DIALOG_CLS = 'app-dialog modal-dialog app-iframe-dialog';
var WIDGET_INFO = { widgetType: 'wm-iframedialog' };
var IframeDialogComponent = /** @class */ (function (_super) {
    __extends(IframeDialogComponent, _super);
    function IframeDialogComponent(inj, dialogClass, modal, closable) {
        var _this = this;
        if (modal === null || modal === undefined) {
            modal = false;
        }
        if (closable === null || closable === undefined) {
            closable = true;
        }
        // setting the backdrop to 'static' will not close the dialog on backdrop click
        var backdrop = toBoolean(modal) ? 'static' : true;
        _this = _super.call(this, inj, WIDGET_INFO, {
            class: DIALOG_CLS + " " + (dialogClass || ''),
            backdrop: backdrop,
            keyboard: !toBoolean(modal)
        }) || this;
        return _this;
    }
    IframeDialogComponent_1 = IframeDialogComponent;
    IframeDialogComponent.prototype.getTemplateRef = function () {
        return this.dialogTemplate;
    };
    /**
     * Click event handler for the ok button
     * invokes on-ok event callback
     * @param {Event} $event
     */
    IframeDialogComponent.prototype.onOk = function ($event) {
        this.invokeEventCallback('ok', { $event: $event });
    };
    IframeDialogComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.register(this.viewParent);
    };
    var IframeDialogComponent_1;
    IframeDialogComponent.initializeProps = registerProps();
    IframeDialogComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: String, decorators: [{ type: Attribute, args: ['class',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['modal',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['closable',] }] }
    ]; };
    __decorate([
        ViewChild('dialogTemplate', { static: true }),
        __metadata("design:type", TemplateRef)
    ], IframeDialogComponent.prototype, "dialogTemplate", void 0);
    IframeDialogComponent = IframeDialogComponent_1 = __decorate([
        Component({
            selector: 'div[wmIframeDialog]',
            template: "<ng-template #dialogTemplate>\n    <div wmDialogHeader [closable]=\"closable\"\n         [iconclass]=\"iconclass\"\n         [iconurl]=\"iconurl\"\n         [iconwidth]=\"iconwidth\"\n         [iconheight]=\"iconheight\"\n         [iconmargin]=\"iconmargin\"\n         [heading]=\"title\"\n         *ngIf=\"showheader\"\n    ></div>\n    <div wmDialogBody>\n        <div wmIframe encodeurl.bind=\"encodeurl\" iframesrc.bind=\"url\" height.bind=\"height\" width.bind=\"width\" hint.bind=\"hint\"></div>\n    </div>\n    <div wmDialogFooter *ngIf=\"showactions\">\n        <button wmButton class=\"btn-primary ok-action\" caption.bind=\"oktext\" aria-label=\"Submit button\" (click)=\"onOk($event)\"></button>\n    </div>\n</ng-template>\n",
            providers: [
                provideAsWidgetRef(IframeDialogComponent_1),
                provideAsDialogRef(IframeDialogComponent_1)
            ]
        }),
        __param(1, Attribute('class')),
        __param(2, Attribute('modal')),
        __param(3, Attribute('closable')),
        __metadata("design:paramtypes", [Injector, String, Object, Object])
    ], IframeDialogComponent);
    return IframeDialogComponent;
}(BaseDialog));

var components = [
    IframeDialogComponent
];
var IframeDialogModule = /** @class */ (function () {
    function IframeDialogModule() {
    }
    IframeDialogModule = __decorate([
        NgModule({
            imports: [
                BasicModule,
                CommonModule,
                DialogModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], IframeDialogModule);
    return IframeDialogModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { IframeDialogModule, IframeDialogComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
