import { __decorate, __metadata, __param } from 'tslib';
import { Self, Inject, ContentChildren, QueryList, ContentChild, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Context, DialogRef, MessageComponent, WmComponentsModule } from '@wm/components/base';
import { DialogModule } from '@wm/components/dialogs';
import { DesignDialogModule } from '@wm/components/dialogs/design-dialog';
import { InputModule } from '@wm/components/input';
import { $appDigest } from '@wm/core';
import { FormComponent } from '@wm/components/data/form';

let LoginDialogDirective = class LoginDialogDirective {
    constructor(contexts, dialogRef) {
        this.contexts = contexts;
        this.dialogRef = dialogRef;
        this.contexts[0].doLogin = () => this.doLogin();
    }
    hideMsg() {
        if (this.msgCmp) {
            this.msgCmp.hideMessage();
        }
    }
    showError(msg) {
        if (this.msgCmp) {
            this.msgCmp.showMessage(msg, 'error');
        }
    }
    showLoading() {
        if (this.msgCmp) {
            this.msgCmp.showMessage('Loading...', 'loading');
        }
    }
    onSuccess() {
        this.hideMsg();
    }
    onError(error) {
        this.showError(error);
    }
    getLoginDetails() {
        return this.formCmp.first.dataoutput;
    }
    doLogin() {
        const loginInfo = this.getLoginDetails();
        const ds = this.dialogRef.eventsource;
        if (ds && loginInfo['j_username'] && loginInfo['j_password']) {
            this.showLoading();
            ds.invoke({ loginInfo: loginInfo }, this.onSuccess.bind(this), this.onError.bind(this));
        }
    }
    ngAfterViewInit() {
        // On login dialog open we wait till the form loads and then assign the enter
        // key event to the textbox in form.
        this.dialogOpenSubscription = this.dialogRef.bsModal.onShown.subscribe(() => {
            setTimeout(() => {
                // On enter key press submit the login form in Login Dialog
                this.dialogRef.$element.find('.app-textbox').keypress((evt) => {
                    if (evt.which === 13) {
                        evt.stopPropagation();
                        /**
                         * As this function is getting executed outside of angular context,
                         * trigger a digest cycle and then trigger doLogin method, So that the bindings in the loginVariable will be evaluated property
                         */
                        $appDigest();
                        this.doLogin();
                    }
                });
            });
        });
    }
    ngOnDestroy() {
        this.dialogOpenSubscription.unsubscribe();
    }
};
LoginDialogDirective.ctorParameters = () => [
    { type: Array, decorators: [{ type: Self }, { type: Inject, args: [Context,] }] },
    { type: DialogRef, decorators: [{ type: Self }] }
];
__decorate([
    ContentChildren(FormComponent),
    __metadata("design:type", QueryList)
], LoginDialogDirective.prototype, "formCmp", void 0);
__decorate([
    ContentChild(MessageComponent),
    __metadata("design:type", MessageComponent)
], LoginDialogDirective.prototype, "msgCmp", void 0);
LoginDialogDirective = __decorate([
    Directive({
        selector: '[wmDialog][wmLoginDialog]'
    }),
    __param(0, Self()), __param(0, Inject(Context)),
    __param(1, Self()),
    __metadata("design:paramtypes", [Array,
        DialogRef])
], LoginDialogDirective);

const components = [
    LoginDialogDirective
];
let LoginDialogModule = class LoginDialogModule {
};
LoginDialogModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            DesignDialogModule,
            DialogModule,
            InputModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], LoginDialogModule);

/**
 * Generated bundle index. Do not edit.
 */

export { LoginDialogModule, LoginDialogDirective as ɵa };
//# sourceMappingURL=index.js.map
