import { __decorate, __metadata, __param, __spread } from 'tslib';
import { Self, Inject, ContentChildren, QueryList, ContentChild, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Context, DialogRef, MessageComponent, WmComponentsModule } from '@wm/components/base';
import { DialogModule } from '@wm/components/dialogs';
import { DesignDialogModule } from '@wm/components/dialogs/design-dialog';
import { InputModule } from '@wm/components/input';
import { $appDigest } from '@wm/core';
import { FormComponent } from '@wm/components/data/form';

var LoginDialogDirective = /** @class */ (function () {
    function LoginDialogDirective(contexts, dialogRef) {
        var _this = this;
        this.contexts = contexts;
        this.dialogRef = dialogRef;
        this.contexts[0].doLogin = function () { return _this.doLogin(); };
    }
    LoginDialogDirective.prototype.hideMsg = function () {
        if (this.msgCmp) {
            this.msgCmp.hideMessage();
        }
    };
    LoginDialogDirective.prototype.showError = function (msg) {
        if (this.msgCmp) {
            this.msgCmp.showMessage(msg, 'error');
        }
    };
    LoginDialogDirective.prototype.showLoading = function () {
        if (this.msgCmp) {
            this.msgCmp.showMessage('Loading...', 'loading');
        }
    };
    LoginDialogDirective.prototype.onSuccess = function () {
        this.hideMsg();
    };
    LoginDialogDirective.prototype.onError = function (error) {
        this.showError(error);
    };
    LoginDialogDirective.prototype.getLoginDetails = function () {
        return this.formCmp.first.dataoutput;
    };
    LoginDialogDirective.prototype.doLogin = function () {
        var loginInfo = this.getLoginDetails();
        var ds = this.dialogRef.eventsource;
        if (ds && loginInfo['j_username'] && loginInfo['j_password']) {
            this.showLoading();
            ds.invoke({ loginInfo: loginInfo }, this.onSuccess.bind(this), this.onError.bind(this));
        }
    };
    LoginDialogDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        // On login dialog open we wait till the form loads and then assign the enter
        // key event to the textbox in form.
        this.dialogOpenSubscription = this.dialogRef.bsModal.onShown.subscribe(function () {
            setTimeout(function () {
                // On enter key press submit the login form in Login Dialog
                _this.dialogRef.$element.find('.app-textbox').keypress(function (evt) {
                    if (evt.which === 13) {
                        evt.stopPropagation();
                        /**
                         * As this function is getting executed outside of angular context,
                         * trigger a digest cycle and then trigger doLogin method, So that the bindings in the loginVariable will be evaluated property
                         */
                        $appDigest();
                        _this.doLogin();
                    }
                });
            });
        });
    };
    LoginDialogDirective.prototype.ngOnDestroy = function () {
        this.dialogOpenSubscription.unsubscribe();
    };
    LoginDialogDirective.ctorParameters = function () { return [
        { type: Array, decorators: [{ type: Self }, { type: Inject, args: [Context,] }] },
        { type: DialogRef, decorators: [{ type: Self }] }
    ]; };
    __decorate([
        ContentChildren(FormComponent),
        __metadata("design:type", QueryList)
    ], LoginDialogDirective.prototype, "formCmp", void 0);
    __decorate([
        ContentChild(MessageComponent),
        __metadata("design:type", MessageComponent)
    ], LoginDialogDirective.prototype, "msgCmp", void 0);
    LoginDialogDirective = __decorate([
        Directive({
            selector: '[wmDialog][wmLoginDialog]'
        }),
        __param(0, Self()), __param(0, Inject(Context)),
        __param(1, Self()),
        __metadata("design:paramtypes", [Array,
            DialogRef])
    ], LoginDialogDirective);
    return LoginDialogDirective;
}());

var components = [
    LoginDialogDirective
];
var LoginDialogModule = /** @class */ (function () {
    function LoginDialogModule() {
    }
    LoginDialogModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                DesignDialogModule,
                DialogModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], LoginDialogModule);
    return LoginDialogModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { LoginDialogModule, LoginDialogDirective as ɵa };
//# sourceMappingURL=index.js.map
