import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Attribute, ViewChild, TemplateRef, ContentChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, provideAsWidgetRef, provideAsDialogRef, WmComponentsModule } from '@wm/components/base';
import { BaseDialog, DialogModule } from '@wm/components/dialogs';
import { InputModule } from '@wm/components/input';
import { toBoolean, App } from '@wm/core';

const registerProps = () => {
    register('wm-partialdialog', new Map([
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', { value: true, PROP_BOOLEAN }],
        ['content', PROP_STRING],
        ['iconclass', { value: 'wi wi-file', PROP_STRING }],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['modal', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['oktext', Object.assign({ value: 'OK' }, PROP_STRING)],
        ['showactions', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['title', Object.assign({ value: 'Page Content' }, PROP_STRING)]
    ]));
};

var PartialDialogComponent_1;
const DIALOG_CLS = 'app-dialog modal-dialog app-page-dialog';
const WIDGET_INFO = { widgetType: 'wm-partialdialog' };
let PartialDialogComponent = PartialDialogComponent_1 = class PartialDialogComponent extends BaseDialog {
    constructor(inj, app, dialogClass, modal, closable) {
        if (modal === null || modal === undefined) {
            modal = false;
        }
        if (closable === null || closable === undefined) {
            closable = true;
        }
        // setting the backdrop to 'static' will not close the dialog on backdrop click
        const backdrop = toBoolean(modal) ? 'static' : true;
        super(inj, WIDGET_INFO, {
            class: `${DIALOG_CLS} ${dialogClass || ''}`,
            backdrop,
            keyboard: !toBoolean(modal)
        });
        this.app = app;
    }
    getTemplateRef() {
        return this.dialogTemplate;
    }
    /**
     * Click event handler for the ok button
     * invokes on-ok event callback
     * @param {Event} $event
     */
    onOk($event) {
        this.invokeEventCallback('ok', { $event });
    }
    ngOnInit() {
        super.ngOnInit();
        this.register(this.viewParent);
    }
    setPartialLoadListener() {
        const cancelSubscription = this.app.subscribe('partialLoaded', () => {
            const parEle = this.partialRef.nativeElement;
            let partialScope;
            if (parEle) {
                partialScope = parEle.widget;
                this.Widgets = partialScope.Widgets;
                this.Variables = partialScope.Variables;
                this.Actions = partialScope.Actions;
            }
            cancelSubscription();
        });
    }
    open(initState) {
        super.open(initState);
        // WMS-19410 - Set the partial load listener to access widgets, variables and actions of page dialog
        this.setPartialLoadListener();
    }
};
PartialDialogComponent.initializeProps = registerProps();
PartialDialogComponent.ctorParameters = () => [
    { type: Injector },
    { type: App },
    { type: String, decorators: [{ type: Attribute, args: ['class',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['modal',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['closable',] }] }
];
__decorate([
    ViewChild('dialogTemplate', { static: true }),
    __metadata("design:type", TemplateRef)
], PartialDialogComponent.prototype, "dialogTemplate", void 0);
__decorate([
    ContentChild(TemplateRef),
    __metadata("design:type", TemplateRef)
], PartialDialogComponent.prototype, "dialogContent", void 0);
__decorate([
    ContentChild('partial'),
    __metadata("design:type", Object)
], PartialDialogComponent.prototype, "partialRef", void 0);
PartialDialogComponent = PartialDialogComponent_1 = __decorate([
    Component({
        selector: 'div[wmPartialDialog]',
        template: "<ng-template #dialogTemplate>\n    <div wmDialogHeader [closable]=\"closable\"\n         [iconclass]=\"iconclass\"\n         [iconurl]=\"iconurl\"\n         [iconwidth]=\"iconwidth\"\n         [iconheight]=\"iconheight\"\n         [iconmargin]=\"iconmargin\"\n         [heading]=\"title\"\n    ></div>\n    <div wmDialogBody>\n        <ng-container *ngTemplateOutlet=\"dialogContent\"></ng-container>\n    </div>\n    <div wmDialogFooter *ngIf=\"showactions\">\n        <button wmButton class=\"btn-primary ok-action\" caption.bind=\"oktext\" aria-label=\"Submit button\" (click)=\"onOk($event)\"></button>\n    </div>\n</ng-template>\n",
        providers: [
            provideAsWidgetRef(PartialDialogComponent_1),
            provideAsDialogRef(PartialDialogComponent_1)
        ]
    }),
    __param(2, Attribute('class')),
    __param(3, Attribute('modal')),
    __param(4, Attribute('closable')),
    __metadata("design:paramtypes", [Injector,
        App, String, Object, Object])
], PartialDialogComponent);

const components = [
    PartialDialogComponent
];
let PartialDialogModule = class PartialDialogModule {
};
PartialDialogModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            DialogModule,
            InputModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], PartialDialogModule);

/**
 * Generated bundle index. Do not edit.
 */

export { PartialDialogModule, PartialDialogComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
