import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Attribute, ViewChild, TemplateRef, ContentChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, provideAsWidgetRef, provideAsDialogRef, WmComponentsModule } from '@wm/components/base';
import { BaseDialog, DialogModule } from '@wm/components/dialogs';
import { InputModule } from '@wm/components/input';
import { toBoolean, App } from '@wm/core';

var registerProps = function () {
    register('wm-partialdialog', new Map([
        ['animation', PROP_STRING],
        ['class', PROP_STRING],
        ['closable', { value: true, PROP_BOOLEAN: PROP_BOOLEAN }],
        ['content', PROP_STRING],
        ['iconclass', { value: 'wi wi-file', PROP_STRING: PROP_STRING }],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['modal', __assign({ value: false }, PROP_BOOLEAN)],
        ['name', PROP_STRING],
        ['oktext', __assign({ value: 'OK' }, PROP_STRING)],
        ['showactions', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', __assign({ value: 'Page Content' }, PROP_STRING)]
    ]));
};

var DIALOG_CLS = 'app-dialog modal-dialog app-page-dialog';
var WIDGET_INFO = { widgetType: 'wm-partialdialog' };
var PartialDialogComponent = /** @class */ (function (_super) {
    __extends(PartialDialogComponent, _super);
    function PartialDialogComponent(inj, app, dialogClass, modal, closable) {
        var _this = this;
        if (modal === null || modal === undefined) {
            modal = false;
        }
        if (closable === null || closable === undefined) {
            closable = true;
        }
        // setting the backdrop to 'static' will not close the dialog on backdrop click
        var backdrop = toBoolean(modal) ? 'static' : true;
        _this = _super.call(this, inj, WIDGET_INFO, {
            class: DIALOG_CLS + " " + (dialogClass || ''),
            backdrop: backdrop,
            keyboard: !toBoolean(modal)
        }) || this;
        _this.app = app;
        return _this;
    }
    PartialDialogComponent_1 = PartialDialogComponent;
    PartialDialogComponent.prototype.getTemplateRef = function () {
        return this.dialogTemplate;
    };
    /**
     * Click event handler for the ok button
     * invokes on-ok event callback
     * @param {Event} $event
     */
    PartialDialogComponent.prototype.onOk = function ($event) {
        this.invokeEventCallback('ok', { $event: $event });
    };
    PartialDialogComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.register(this.viewParent);
    };
    PartialDialogComponent.prototype.setPartialLoadListener = function () {
        var _this = this;
        var cancelSubscription = this.app.subscribe('partialLoaded', function () {
            var parEle = _this.partialRef.nativeElement;
            var partialScope;
            if (parEle) {
                partialScope = parEle.widget;
                _this.Widgets = partialScope.Widgets;
                _this.Variables = partialScope.Variables;
                _this.Actions = partialScope.Actions;
            }
            cancelSubscription();
        });
    };
    PartialDialogComponent.prototype.open = function (initState) {
        _super.prototype.open.call(this, initState);
        // WMS-19410 - Set the partial load listener to access widgets, variables and actions of page dialog
        this.setPartialLoadListener();
    };
    var PartialDialogComponent_1;
    PartialDialogComponent.initializeProps = registerProps();
    PartialDialogComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: App },
        { type: String, decorators: [{ type: Attribute, args: ['class',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['modal',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['closable',] }] }
    ]; };
    __decorate([
        ViewChild('dialogTemplate', { static: true }),
        __metadata("design:type", TemplateRef)
    ], PartialDialogComponent.prototype, "dialogTemplate", void 0);
    __decorate([
        ContentChild(TemplateRef),
        __metadata("design:type", TemplateRef)
    ], PartialDialogComponent.prototype, "dialogContent", void 0);
    __decorate([
        ContentChild('partial'),
        __metadata("design:type", Object)
    ], PartialDialogComponent.prototype, "partialRef", void 0);
    PartialDialogComponent = PartialDialogComponent_1 = __decorate([
        Component({
            selector: 'div[wmPartialDialog]',
            template: "<ng-template #dialogTemplate>\n    <div wmDialogHeader [closable]=\"closable\"\n         [iconclass]=\"iconclass\"\n         [iconurl]=\"iconurl\"\n         [iconwidth]=\"iconwidth\"\n         [iconheight]=\"iconheight\"\n         [iconmargin]=\"iconmargin\"\n         [heading]=\"title\"\n    ></div>\n    <div wmDialogBody>\n        <ng-container *ngTemplateOutlet=\"dialogContent\"></ng-container>\n    </div>\n    <div wmDialogFooter *ngIf=\"showactions\">\n        <button wmButton class=\"btn-primary ok-action\" caption.bind=\"oktext\" aria-label=\"Submit button\" (click)=\"onOk($event)\"></button>\n    </div>\n</ng-template>\n",
            providers: [
                provideAsWidgetRef(PartialDialogComponent_1),
                provideAsDialogRef(PartialDialogComponent_1)
            ]
        }),
        __param(2, Attribute('class')),
        __param(3, Attribute('modal')),
        __param(4, Attribute('closable')),
        __metadata("design:paramtypes", [Injector,
            App, String, Object, Object])
    ], PartialDialogComponent);
    return PartialDialogComponent;
}(BaseDialog));

var components = [
    PartialDialogComponent
];
var PartialDialogModule = /** @class */ (function () {
    function PartialDialogModule() {
    }
    PartialDialogModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                DialogModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], PartialDialogModule);
    return PartialDialogModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { PartialDialogModule, PartialDialogComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
