import { __assign, __extends, __spread, __decorate, __metadata, __param } from 'tslib';
import { Injector, Attribute, ViewChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, register, registerFormWidget, styler, getConditionalClasses, getUniqObjsByDataField, ALLFIELDS, configureDnD, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { SearchComponent, SearchModule } from '@wm/components/basic/search';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { FormWidgetType, isDefined, debounce, $unwatch, $watch, $appDigest, toBoolean, isAppleProduct } from '@wm/core';
import { DatasetAwareFormComponent } from '@wm/components/input';

var chipsProps = new Map([
    ['allowonlyselect', PROP_BOOLEAN],
    ['autofocus', PROP_BOOLEAN],
    ['chipclass', PROP_STRING],
    ['class', PROP_STRING],
    ['compareby', PROP_STRING],
    ['datafield', PROP_STRING],
    ['dataoptions', PROP_ANY],
    ['dataset', __assign({ value: 'Option 1, Option 2, Option 3' }, PROP_ANY)],
    ['datasource', PROP_ANY],
    ['datavalue', PROP_ANY],
    ['datavaluesource', PROP_ANY],
    ['disabled', PROP_BOOLEAN],
    ['displayexpression', PROP_STRING],
    ['displayfield', PROP_STRING],
    ['displayimagesrc', PROP_STRING],
    ['enablereorder', PROP_BOOLEAN],
    ['inputposition', __assign({ value: 'last' }, PROP_STRING)],
    ['inputwidth', __assign({ value: 'default' }, PROP_STRING)],
    ['limit', PROP_NUMBER],
    ['loadingdatamsg', __assign({ value: 'Loading items...' }, PROP_STRING)],
    ['matchmode', PROP_STRING],
    ['maxsize', PROP_NUMBER],
    ['minchars', __assign({ value: 1 }, PROP_NUMBER)],
    ['name', PROP_STRING],
    ['navsearchbar', __assign({ value: false }, PROP_BOOLEAN)],
    ['orderby', PROP_STRING],
    ['placeholder', __assign({ value: 'Type here..' }, PROP_STRING)],
    ['query', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['searchkey', PROP_STRING],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['showsearchicon', PROP_BOOLEAN],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['type', __assign({ value: 'search' }, PROP_STRING)],
    ['debouncetime', __assign({ value: 250 }, PROP_NUMBER)],
]);
var registerProps = function () {
    register('wm-chips', chipsProps);
    registerFormWidget(FormWidgetType.CHIPS, new Map(chipsProps));
};

var WIDGET_CONFIG = {
    widgetType: 'wm-chips',
    hostClass: 'app-chips nav nav-pills list-inline'
};
var ChipsComponent = /** @class */ (function (_super) {
    __extends(ChipsComponent, _super);
    function ChipsComponent(inj, bindDisplayField, bindDisplayExpr, bindDisplayImgSrc, bindDataField, bindDataSet, bindChipclass) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.bindDisplayField = bindDisplayField;
        _this.bindDisplayExpr = bindDisplayExpr;
        _this.bindDisplayImgSrc = bindDisplayImgSrc;
        _this.bindDataField = bindDataField;
        _this.bindDataSet = bindDataSet;
        _this.bindChipclass = bindChipclass;
        _this.chipsList = [];
        _this.maxSizeReached = 'Max size reached';
        _this._unsubscribeDv = false;
        styler(_this.nativeElement, _this);
        // set the showsearchicon as false by default.
        if (!isDefined(_this.showsearchicon)) {
            _this.showsearchicon = false;
        }
        _this.multiple = true;
        _this.nextItemIndex = 0; // default chip index
        _this._debounceUpdateQueryModel = debounce(function (val) {
            _this.updateQueryModel(val).then(function () {
                if (_this.bindChipclass) {
                    _.forEach(_this.chipsList, function (item, index) {
                        _this.registerChipItemClass(item, index);
                    });
                }
            });
        }, 150);
        var datasetSubscription = _this.dataset$.subscribe(function () {
            _this.searchComponent.dataset = _this.dataset;
            // clearing the last results when dataset is reassigned
            _this.searchComponent._lastResult = undefined;
            _this.nextItemIndex = _this.datasetItems.length;
            _this._debounceUpdateQueryModel(_this.datavalue || _this.toBeProcessedDatavalue);
        });
        _this.registerDestroyListener(function () { return datasetSubscription.unsubscribe(); });
        var datavalueSubscription = _this.datavalue$.subscribe(function (val) {
            // update queryModel only when parentRef is available.
            if (!_this._unsubscribeDv) {
                _this.chipsList = [];
                // if the datafield is ALLFILEDS do not fetch the records
                // update the query model with the values we have
                _this._debounceUpdateQueryModel(val);
            }
        });
        _this.registerDestroyListener(function () { return datavalueSubscription.unsubscribe(); });
        return _this;
    }
    ChipsComponent_1 = ChipsComponent;
    Object.defineProperty(ChipsComponent.prototype, "datasource", {
        // getter setter is added to pass the datasource to searchcomponent.
        get: function () {
            return this._datasource;
        },
        set: function (nv) {
            this._datasource = nv;
            this.searchComponent.datasource = nv;
            this._debounceUpdateQueryModel(this.datavalue || this.toBeProcessedDatavalue);
        },
        enumerable: true,
        configurable: true
    });
    ChipsComponent.prototype.ngOnInit = function () {
        var _this = this;
        _super.prototype.ngOnInit.call(this);
        this.searchComponent.multiple = true;
        this.searchComponent.binddisplayimagesrc = this.bindDisplayImgSrc;
        this.searchComponent.displayimagesrc = this.displayimagesrc;
        this.searchComponent.binddisplaylabel = this.bindDisplayExpr;
        this.searchComponent.displaylabel = this.displayfield;
        this.searchComponent.datafield = this.bindDataField || this.datafield;
        this.searchComponent.binddataset = this.bindDataSet;
        this.searchComponent.dataset = this.dataset;
        this.searchComponent.searchkey = this.searchkey;
        this.searchComponent.limit = this.limit;
        this.searchComponent.debouncetime = this.debouncetime;
        this.searchComponent.matchmode = this.matchmode;
        this.getTransformedData = function (val, isCustom) {
            return _this.searchComponent.getTransformedData([val], _this.nextItemIndex++, isCustom);
        };
    };
    ChipsComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        if (this.enablereorder) {
            this.configureDnD();
        }
    };
    /**
     * This method returns the evaluated class expression.
     * @param $index index of the chip
     * @param item chip object containing the key, value, label
     * @returns {any} evaluated class expression value
     */
    ChipsComponent.prototype.registerChipItemClass = function (item, $index) {
        var _this = this;
        if (this.bindChipclass) {
            var watchName = this.widgetId + "_chipItemClass_" + $index;
            $unwatch(watchName);
            this.registerDestroyListener($watch(this.bindChipclass, this.viewParent, { item: item, $index: $index }, function (nv, ov) {
                _this.applyItemClass(getConditionalClasses(nv, ov), $index);
            }, watchName));
        }
    };
    ChipsComponent.prototype.applyItemClass = function (val, index) {
        var chipItem = this.nativeElement.querySelectorAll('.chip-item').item(index);
        $(chipItem).removeClass(val.toRemove).addClass(val.toAdd);
    };
    ChipsComponent.prototype.removeDuplicates = function () {
        this.chipsList = getUniqObjsByDataField(this.chipsList, this.datafield, this.displayfield || this.displaylabel);
    };
    // This method updates the queryModel.
    // default call to get the default data can be done only when defaultQuery is true.
    ChipsComponent.prototype.updateQueryModel = function (data) {
        var _this = this;
        var promises = [];
        if (!data) {
            this.chipsList = [];
            return Promise.resolve();
        }
        // clone the data as the updations on data will change the datavalue.
        var dataValue = _.clone(data);
        var prevChipsList = this.chipsList;
        this.chipsList = [];
        // update the model when model has items more than maxsize
        if (this.maxsize && dataValue.length > this.maxsize) {
            this._modelByValue = dataValue = _.slice(dataValue, 0, this.maxsize);
            data = dataValue;
        }
        var searchQuery = [];
        /**
         * For each value in datavalue,
         * 1. check whether value is in datasetItems, if item is found, addd to the chipsList.
         * 2. else make a default query to the filter and get the record.
         * 3. In step 2, if datavalue is not ALLFIELDS, then make a query. Extract the chipsList from the query response.
         * 4. If there is no response for the value and allowonlyselect is true, remove the value from the datavalue. So that datavalue just contains the valid values.
         * 5. In step 2, if datavalue is ALLFIELDS and value is object, then just prepare the datasetItem from the value.
         * 6. If value is not object and allowonlyselect is false, then create a customModel and replace this value with customModel and prepare datasetItem from this value
         */
        dataValue.forEach(function (val, i) {
            var itemFound = _.find(_this.datasetItems, function (item) {
                return _.isObject(item.value) ? _.isEqual(item.value, val) : _.toString(item.value) === _.toString(val);
            });
            if (itemFound) {
                _this.chipsList.push(itemFound);
            }
            else if (_this.datafield !== ALLFIELDS) {
                searchQuery.push(val);
            }
            else if (_this.datafield === ALLFIELDS) {
                var dataObj = void 0, isCustom = false;
                if (!_.isObject(val)) {
                    dataObj = _this.createCustomDataModel(val);
                    isCustom = true;
                    if (dataObj) {
                        data.splice(i, 1, dataObj);
                    }
                }
                else {
                    // if custom chips is already generated, val will be object as {'dataField_val': 'entered_val'}
                    // Hence check this val in previous chipList and assign the iscustom flag
                    var prevChipObj = prevChipsList.find(function (obj) {
                        return _.isEqual(obj.value, val);
                    });
                    if (prevChipObj) {
                        isCustom = prevChipObj.iscustom;
                    }
                }
                dataObj = dataObj || val;
                var transformedData = _this.getTransformedData(dataObj, isCustom);
                var chipObj = transformedData[0];
                if (isCustom) {
                    chipObj.iscustom = isCustom;
                }
                _this.chipsList.push(chipObj);
            }
        });
        // make default query with all the values and if response for the value is not in datavalue then add a custom chip object.
        if (searchQuery.length) {
            promises.push(this.getDefaultModel(searchQuery, this.nextItemIndex)
                .then(function (response) {
                _this.chipsList = _this.chipsList.concat(response || []);
                dataValue.forEach(function (val, i) {
                    var isExists = _.find(_this.chipsList, function (obj) {
                        return obj.value.toString() === val.toString();
                    });
                    if (!isExists) {
                        if (_this.allowonlyselect) {
                            var index = data.indexOf(val);
                            if (index > -1) {
                                data.splice(index, 1);
                            }
                            return;
                        }
                        var transformedData = _this.getTransformedData(val, true);
                        var chipObj = transformedData[0];
                        chipObj.iscustom = true;
                        _this.chipsList.push(chipObj);
                    }
                });
            }));
        }
        // default chip data is adding focus on to the search input. Hence this flag helps not to focus.
        this.resetSearchModel(true);
        return Promise.all(promises).then(function () {
            _this._modelByValue = data;
            _this.removeDuplicates();
            _this.updateMaxSize();
            $appDigest();
        });
    };
    ChipsComponent.prototype.resetSearchModel = function (defaultQuery) {
        this._unsubscribeDv = true;
        // clear search will empty the query model and gets the data when minchars is 0 (i.e. autocomplete) on focus
        // defaultQuery flag is set when widget is not active. This will only load the autocomplete dropup with minchars as 0 when widget is focused/active
        this.searchComponent.clearSearch(undefined, !this.minchars && !defaultQuery);
        this._unsubscribeDv = false;
    };
    // Triggerred when typeahead option is selected by enter keypress.
    ChipsComponent.prototype.onSelect = function ($event) {
        if (!this.searchComponent.liElements.length) {
            this.addItem($event);
        }
    };
    // Add the newItem to the list
    ChipsComponent.prototype.addItem = function ($event, widget) {
        var searchComponent = widget;
        var allowAdd;
        var chipObj;
        if (searchComponent && isDefined(searchComponent.datavalue) && searchComponent.queryModel !== '') {
            if (!searchComponent.query || !_.trim(searchComponent.query)) {
                return;
            }
            chipObj = searchComponent.queryModel;
        }
        else {
            if (this.allowonlyselect) {
                return;
            }
            var dataObj = void 0;
            if (this.datafield === ALLFIELDS) {
                if (!_.isObject(this.searchComponent.query) && _.trim(this.searchComponent.query)) {
                    dataObj = this.createCustomDataModel(this.searchComponent.query);
                    // return if the custom chip is empty
                    if (!dataObj) {
                        this.resetSearchModel();
                        return;
                    }
                }
            }
            var data = dataObj || _.trim(this.searchComponent.query);
            if (data) {
                var transformedData = this.getTransformedData(data, true);
                chipObj = transformedData[0];
                chipObj.iscustom = true;
            }
        }
        if (!isDefined(chipObj) || chipObj === '') {
            return;
        }
        allowAdd = this.invokeEventCallback('beforeadd', { $event: $event, newItem: chipObj });
        if (isDefined(allowAdd) && !toBoolean(allowAdd)) {
            return;
        }
        if (this.isDuplicate(chipObj)) {
            this.resetSearchModel();
            return;
        }
        this.registerChipItemClass(chipObj, this.chipsList.length);
        this.chipsList.push(chipObj);
        if (!this.datavalue) {
            this._modelByValue = [chipObj.value];
        }
        else {
            this._modelByValue = __spread(this._modelByValue, [chipObj.value]);
        }
        this._unsubscribeDv = true;
        this.invokeOnTouched();
        this.invokeOnChange(this._modelByValue, $event || {}, true);
        this.invokeEventCallback('add', { $event: $event, $item: chipObj });
        this.updateMaxSize();
        // reset input box when item is added.
        this.resetSearchModel();
        // stop the event to not to call the submit event on enter press.
        if ($event && ($event.key === 'Enter' || $event.keyCode === 13)) {
            this.stopEvent($event);
        }
    };
    // Prepare datavalue object from a string(junk) value when datafield is allFields.
    ChipsComponent.prototype.createCustomDataModel = function (val) {
        var key = this.displayfield || (this.datafield !== ALLFIELDS ? this.datafield : undefined);
        if (key) {
            var customObj = {};
            customObj[key] = val;
            return customObj;
        }
    };
    // Check if newItem already exists
    ChipsComponent.prototype.isDuplicate = function (item) {
        if (this.datafield === ALLFIELDS) {
            return _.findIndex(this.chipsList, { value: item.value }) > -1;
        }
        return _.findIndex(this.chipsList, { key: item.key }) > -1;
    };
    // Check if max size is reached
    ChipsComponent.prototype.updateMaxSize = function () {
        this.saturate = this.maxsize > 0 && this.chipsList.length === this.maxsize;
    };
    // Makes call to searchComponent to filter the dataSource based on the query.
    ChipsComponent.prototype.getDefaultModel = function (query, index) {
        this.nextItemIndex++;
        return this.searchComponent.getDataSource(query, true, index)
            .then(function (response) {
            return _.filter(query, function (queryVal) {
                _.find(response, { value: queryVal });
            });
        });
    };
    ChipsComponent.prototype.handleChipClick = function ($event, chip) {
        if (this.readonly) {
            return;
        }
        $event.currentTarget.focus();
        this.invokeEventCallback('chipclick', { $event: $event, $item: chip });
    };
    ChipsComponent.prototype.handleChipFocus = function ($event, chip) {
        if (this.readonly) {
            return;
        }
        chip.active = true;
        this.invokeEventCallback('chipselect', { $event: $event, $item: chip });
    };
    // To avoid form submit on pressing enter key
    ChipsComponent.prototype.stopEvent = function ($event) {
        $event.stopPropagation();
        $event.preventDefault();
    };
    ChipsComponent.prototype.onTextDelete = function ($event) {
        if (isAppleProduct) {
            this.onInputClear($event);
        }
    };
    ChipsComponent.prototype.onInputClear = function ($event) {
        if (!this.chipsList || !this.chipsList.length || this.searchComponent.query) {
            return;
        }
        this.$element.find('li.chip-item > a.app-chip').last().focus();
        this.stopEvent($event);
    };
    ChipsComponent.prototype.onBackspace = function ($event, $item, $index) {
        if (this.readonly) {
            return;
        }
        this.removeItem($event, $item, $index, true);
    };
    ChipsComponent.prototype.onDelete = function ($event, $item, $index) {
        if (this.readonly) {
            return;
        }
        this.removeItem($event, $item, $index);
    };
    ChipsComponent.prototype.onArrowLeft = function ($item, $index) {
        if (this.readonly) {
            return;
        }
        // On left arrow click when search input query is empty.
        if (!this.searchComponent.query && !isDefined($index) && !isDefined($item)) {
            this.$element.find('li.chip-item > a.app-chip').last().focus();
            return;
        }
        if ($index > 0) {
            this.$element.find('li.chip-item > a.app-chip').get($index - 1).focus();
        }
        else {
            this.focusSearchBox();
        }
    };
    ChipsComponent.prototype.onArrowRight = function ($item, $index) {
        if (this.readonly) {
            return;
        }
        // On right arrow click when search input query is empty.
        if (!this.searchComponent.query && !isDefined($index) && !isDefined($item)) {
            this.$element.find('li.chip-item > a.app-chip').first().focus();
            return;
        }
        if ($index < (this.chipsList.length - 1)) {
            this.$element.find('li.chip-item > a.app-chip').get($index + 1).focus();
        }
        else {
            this.focusSearchBox();
        }
    };
    // focus search box.
    ChipsComponent.prototype.focusSearchBox = function () {
        this.$element.find('.app-chip-input > input.app-textbox').focus();
    };
    // Remove the item from list
    ChipsComponent.prototype.removeItem = function ($event, item, index, canFocus) {
        var _this = this;
        $event.stopPropagation();
        var indexes = _.isArray(index) ? index : [index];
        var focusIndex = _.max(indexes);
        var items = _.reduce(indexes, function (result, i) {
            result.push(_this.chipsList[i]);
            return result;
        }, []);
        // prevent deletion if the before-remove event callback returns false
        var allowRemove = this.invokeEventCallback('beforeremove', { $event: $event, $item: items.length === 1 ? items[0] : items });
        if (isDefined(allowRemove) && !toBoolean(allowRemove)) {
            return;
        }
        var prevDatavalue = _.clone(this.datavalue);
        // focus next chip after deletion.
        // if there are no chips in the list focus search box
        setTimeout(function () {
            var chipsLength = _this.chipsList.length;
            var $chipsList = _this.$element.find('li.chip-item > a.app-chip');
            if (!chipsLength || !canFocus) {
                _this.focusSearchBox();
            }
            else if ((chipsLength - 1) < focusIndex) {
                // if focus index is greater than chips length select last chip
                $chipsList.get(chipsLength - 1).focus();
            }
            else {
                // manually set the succeeding chip as active if there is a chip next to the current chip.
                _this.chipsList[focusIndex].active = true;
                $chipsList.get(focusIndex).focus();
            }
        });
        var pulledItems = _.pullAt(this.chipsList, indexes);
        pulledItems.forEach(function (datasetItem) {
            _this._modelByValue = _.filter(_this._modelByValue, function (val) {
                return !(_.isObject(val) ? _.isEqual(val, datasetItem.value) : _.toString(val) === _.toString(datasetItem.value));
            });
        });
        this._unsubscribeDv = false;
        this.invokeOnChange(this._modelByValue, $event, true);
        this.invokeEventCallback('change', { $event: $event, newVal: this.datavalue, oldVal: prevDatavalue });
        this.updateMaxSize();
        this.invokeEventCallback('remove', { $event: $event, $item: items.length === 1 ? items[0] : items });
    };
    /**
     * Swaps items in an array if provided with indexes.
     * @param data :- array to be swapped
     * @param newIndex :- new index for the element to be placed
     * @param currentIndex :- the current index of the element.
     */
    ChipsComponent.prototype.swapElementsInArray = function (data, newIndex, currentIndex) {
        var draggedItem = _.pullAt(data, currentIndex)[0];
        data.splice(newIndex, 0, draggedItem);
    };
    /**
     * Cancels the reorder by reseting the elements to the original position.
     */
    ChipsComponent.prototype.resetReorder = function () {
        this.$element.removeData('oldIndex');
    };
    ChipsComponent.prototype.invokeOnBeforeServiceCall = function (inputData) {
        this.invokeEventCallback('beforeservicecall', { inputData: inputData });
    };
    ChipsComponent.prototype.handleEvent = function (node, eventName, eventCallback, locals) {
        if (eventName === 'remove' || eventName === 'beforeremove' || eventName === 'chipselect'
            || eventName === 'chipclick' || eventName === 'add' || eventName === 'reorder' || eventName === 'change') {
            return;
        }
        _super.prototype.handleEvent.call(this, node, eventName, eventCallback, locals);
    };
    // Configures the reordable feature in chips widgets.
    ChipsComponent.prototype.configureDnD = function () {
        var options = {
            items: '> li:not(.app-chip-search)',
            placeholder: 'chip-placeholder'
        };
        configureDnD(this.$element, options, this.onReorderStart.bind(this), this.update.bind(this));
    };
    // Triggered on drag start while reordering.
    ChipsComponent.prototype.onReorderStart = function (evt, ui) {
        var helper = ui.helper;
        // increasing the width of the dragged item by 1
        helper.width(helper.width() + 1);
        this.$element.data('oldIndex', ui.item.index() - (this.inputposition === 'first' ? 1 : 0));
    };
    // updates the chipsList and datavalue on reorder.
    ChipsComponent.prototype.update = function ($event, ui) {
        var changedItem, newIndex, oldIndex;
        // Get the index of the item at position before drag and after the reorder.
        newIndex = ui.item.index() - (this.inputposition === 'first' ? 1 : 0);
        oldIndex = this.$element.data('oldIndex');
        newIndex = this.chipsList.length === newIndex ? newIndex - 1 : newIndex;
        changedItem = {
            oldIndex: oldIndex,
            newIndex: newIndex,
            item: this.chipsList[oldIndex]
        };
        if (newIndex === oldIndex) {
            this.resetReorder();
            return;
        }
        changedItem.item = this.chipsList[oldIndex];
        var allowReorder = this.invokeEventCallback('beforereorder', { $event: $event, $data: this.chipsList, $changedItem: changedItem });
        if (isDefined(allowReorder) && toBoolean(allowReorder) === false) {
            this.resetReorder();
            return;
        }
        // modify the chipsList and datavalue after the reorder.
        this.swapElementsInArray(this.chipsList, newIndex, oldIndex);
        this.swapElementsInArray(this._modelByValue, newIndex, oldIndex);
        changedItem.item = this.chipsList[newIndex];
        this.chipsList = __spread(this.chipsList);
        this.resetReorder();
        this.invokeEventCallback('reorder', { $event: $event, $data: this.chipsList, $changedItem: changedItem });
    };
    ChipsComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'tabindex') {
            return;
        }
        if (key === 'dataoptions') {
            this.searchComponent.dataoptions = nv;
        }
        if (key === 'datafield') {
            this.searchComponent.datafield = this.datafield;
        }
        if (key === 'displayfield') {
            this.searchComponent.displaylabel = this.displayfield;
        }
        if (key === 'displayexpression') {
            this.searchComponent.binddisplaylabel = this.binddisplayexpression ? this.binddisplayexpression : this.displayexpression;
        }
        if (key === 'displayimagesrc') {
            this.searchComponent.binddisplayimagesrc = this.binddisplayimagesrc ? this.binddisplayimagesrc : this.displayimagesrc;
        }
        if (key === 'limit') {
            this.searchComponent.limit = this.limit;
        }
        if (key === 'matchmode') {
            this.searchComponent.matchmode = this.matchmode;
        }
        if (key === 'enablereorder') {
            if (this.$element.hasClass('ui-sortable')) {
                this.$element.sortable('option', 'disabled', !nv);
            }
            else if (nv) {
                this.configureDnD();
            }
        }
        if (key === 'readonly') {
            if (nv) {
                this.$element.addClass('readonly');
            }
            else {
                this.$element.removeClass('readonly');
            }
        }
        if (key === 'inputposition') {
            var $inputEl = this.$element.find('li.app-chip-search');
            if (nv === 'first') {
                this.$element.prepend($inputEl);
            }
            else {
                this.$element.append($inputEl);
            }
        }
        if (key === 'autofocus' && nv) {
            // setting the autofocus on the input once after dom is updated
            setTimeout(function () {
                var $chipsList = _this.$element.find('.app-chip-input > input.app-textbox');
                if ($chipsList && $chipsList.length) {
                    _this.focusSearchBox();
                }
            });
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    var ChipsComponent_1;
    ChipsComponent.initializeProps = registerProps();
    ChipsComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: undefined, decorators: [{ type: Attribute, args: ['displayfield.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['displayexpression.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['displayimagesrc.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['datafield.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['dataset.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['chipclass.bind',] }] }
    ]; };
    __decorate([
        ViewChild(SearchComponent, /* TODO: add static flag */ { static: true }),
        __metadata("design:type", SearchComponent)
    ], ChipsComponent.prototype, "searchComponent", void 0);
    ChipsComponent = ChipsComponent_1 = __decorate([
        Component({
            selector: '[wmChips]',
            template: "<li class=\"chip-item {{chipclass && !bindChipclass ? chipclass : ''}}\" role=\"option\" *ngFor=\"let item of chipsList; let $index = index;\"\n    [ngClass]=\"{'active': item.active, 'disabled': disabled}\">\n    <a class=\"app-chip\" href=\"javascript:void(0);\" tabindex=\"-1\"\n       (click)=\"handleChipClick($event, item)\"\n       (keydown.delete)=\"onDelete($event, item, $index)\"\n       (keydown.backspace)=\"onBackspace($event, item, $index)\"\n       (keydown.arrowleft)=\"onArrowLeft(item, $index)\"\n       (keydown.arrowright)=\"onArrowRight(item, $index)\"\n       (focus)=\"handleChipFocus($event, item)\"\n       (blur)=\"readonly ? 0 : item.active = false\"\n       [ngClass]=\"{'chip-duplicate bg-danger': item.isDuplicate, 'chip-picture': item.imgSrc}\">\n        <img data-identifier=\"img\" alt=\"Chip Image\" class=\"button-image-icon\" [src]=\"item.imgSrc\" *ngIf=\"item.imgSrc\"/>\n        <span class=\"app-chip-title\" [textContent]=\"item.label\" [title]=\"item.label\"></span>\n        <button type=\"button\" tabindex=\"-1\" arial-label=\"Clear Button\" class=\"btn btn-transparent\" (click)=\"removeItem($event, item, $index);\" *ngIf=\"!readonly\">\n            <i class=\"app-icon wi wi-close\"></i>\n        </button>\n    </a>\n</li>\n<li class=\"app-chip-search\" [ngClass]=\"{'full-width': inputwidth === 'full'}\">\n    <div #search wmSearch class=\"app-chip-input\"\n         name=\"app-chip-search\"\n         submit.event=\"addItem($event, widget)\"\n         beforeservicecall.event=\"invokeOnBeforeServiceCall(inputData)\"\n         disabled.bind=\"disabled || readonly || saturate\"\n         datafield.bind=\"datafield\"\n         allowonlyselect.bind=\"allowonlyselect\"\n         searchkey.bind=\"searchkey\"\n         orderby.bind=\"orderby\"\n         placeholder.bind=\"saturate ? maxSizeReached : placeholder\"\n         showsearchicon.bind=\"showsearchicon\"\n         tabindex.bind=\"tabindex\"\n         minchars.bind=\"minchars\"\n         debouncetime.bind=\"debouncetime\"\n         matchmode.bind=\"matchmode\"\n         (keydown.enter)=\"onSelect($event)\"\n         (keydown.delete)=\"onTextDelete($event)\"\n         (keydown.backspace)=\"onInputClear($event)\"\n         (keydown.arrowleft)=\"onArrowLeft()\"\n         (keydown.arrowright)=\"onArrowRight()\">\n    </div>\n</li>\n",
            providers: [
                provideAs(ChipsComponent_1, NG_VALUE_ACCESSOR, true),
                provideAsWidgetRef(ChipsComponent_1)
            ]
        }),
        __param(1, Attribute('displayfield.bind')),
        __param(2, Attribute('displayexpression.bind')),
        __param(3, Attribute('displayimagesrc.bind')),
        __param(4, Attribute('datafield.bind')),
        __param(5, Attribute('dataset.bind')),
        __param(6, Attribute('chipclass.bind')),
        __metadata("design:paramtypes", [Injector, Object, Object, Object, Object, Object, Object])
    ], ChipsComponent);
    return ChipsComponent;
}(DatasetAwareFormComponent));

var components = [
    ChipsComponent
];
var ChipsModule = /** @class */ (function () {
    function ChipsModule() {
    }
    ChipsModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                SearchModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], ChipsModule);
    return ChipsModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ChipsModule, ChipsComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
