import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ViewChild, ElementRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NgModel, NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { ColorPickerDirective, ColorPickerModule as ColorPickerModule$1 } from 'ngx-color-picker';
import { PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, register, registerFormWidget, styler, AUTOCLOSE_TYPE, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { FormWidgetType, addClass, removeClass } from '@wm/core';
import { BaseFormCustomComponent } from '@wm/components/input';

var colorPickerProps = new Map([
    ['autoclose', __assign({ value: 'outsideClick' }, PROP_STRING)],
    ['class', PROP_STRING],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Select Color' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
]);
var registerProps = function () {
    register('wm-colorpicker', colorPickerProps);
    registerFormWidget(FormWidgetType.COLORPICKER, new Map(colorPickerProps));
};

var DEFAULT_CLS = 'input-group app-colorpicker';
var WIDGET_CONFIG = {
    widgetType: 'wm-colorpicker',
    hostClass: DEFAULT_CLS,
    displayType: 'inline-block'
};
var ColorPickerComponent = /** @class */ (function (_super) {
    __extends(ColorPickerComponent, _super);
    function ColorPickerComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        styler(_this.nativeElement, _this);
        return _this;
    }
    ColorPickerComponent_1 = ColorPickerComponent;
    // To remove space occupied by colorpicker when it is closed
    ColorPickerComponent.prototype.colorPickerToggleChange = function (isOpen) {
        var colorPickerContainer = this.nativeElement.querySelector(".color-picker");
        (!isOpen) ? addClass(colorPickerContainer, 'hidden') : removeClass(colorPickerContainer, 'hidden');
    };
    ColorPickerComponent.prototype.colorPickerOpen = function (value) {
        var _this = this;
        // Whenever autoclose property is set to 'always', adding the onclick listener to the colorPicker container to close the picker.
        if (this.autoclose === AUTOCLOSE_TYPE.ALWAYS) {
            var colorPickerContainer = this.nativeElement.querySelector(".color-picker");
            colorPickerContainer.onclick = function () { return _this.cpDirective.closeDialog(); };
        }
    };
    // This mehtod is used to show/open the colorpicker popup.
    ColorPickerComponent.prototype.open = function () {
        this.cpDirective.openDialog();
    };
    // This mehtod is used to hide/close the colorpicker popup.
    ColorPickerComponent.prototype.close = function () {
        this.cpDirective.closeDialog();
    };
    // change and blur events are added from the template
    ColorPickerComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName !== 'change' && eventName !== 'blur') {
            _super.prototype.handleEvent.call(this, this.inputEl.nativeElement, eventName, callback, locals);
        }
    };
    ColorPickerComponent.prototype.handleChange = function (newVal) {
        this.invokeOnChange(this.datavalue, { type: 'change' }, this.ngModel.valid);
    };
    ColorPickerComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        if (key === 'required') {
            /* WMS-18269 | Update Angular about the required attr value change */
            this._onChange(this.datavalue);
            return;
        }
        if (key === 'datavalue') {
            this._onChange(nv);
            return;
        }
        if (key === 'autoclose') {
            this.outsideclick = (nv === AUTOCLOSE_TYPE.OUTSIDECLICK || nv === AUTOCLOSE_TYPE.ALWAYS) ? true : false;
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    var ColorPickerComponent_1;
    ColorPickerComponent.initializeProps = registerProps();
    ColorPickerComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], ColorPickerComponent.prototype, "ngModel", void 0);
    __decorate([
        ViewChild('input', { static: true, read: ElementRef }),
        __metadata("design:type", ElementRef)
    ], ColorPickerComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(ColorPickerDirective),
        __metadata("design:type", ColorPickerDirective)
    ], ColorPickerComponent.prototype, "cpDirective", void 0);
    ColorPickerComponent = ColorPickerComponent_1 = __decorate([
        Component({
            selector: '[wmColorPicker]',
            template: "<input class=\"form-control app-textbox\" aria-label=\"Enter the color code\" aria-haspopup=\"true\" aria-expanded=\"false\"\n       #input\n       focus-target\n       [attr.name]=\"name\"\n       [(colorPicker)]=\"datavalue\"\n       (colorPickerChange)=\"handleChange($event);\"\n       (cpToggleChange)=\"colorPickerToggleChange($event)\"\n       [cpPosition]=\"'bottom'\"\n       [cpCloseClickOutside]=\"outsideclick\"\n       (colorPickerOpen)=\"colorPickerOpen($event)\"\n       [attr.placeholder]=\"placeholder\"\n       [disabled]=\"readonly || disabled\"\n       [required]=\"required\"\n       [tabindex]=\"tabindex\"\n       [(ngModel)]=\"datavalue\"\n       (ngModelChange)=\"handleChange($event);\"\n       [ngModelOptions]=\"{updateOn: 'change'}\"\n       (blur)=\"invokeOnTouched($event)\"\n       [attr.accesskey]=\"shortcutkey\">\n<span class=\"input-group-addon colorpicker-container\">\n    <i class=\"colored-box\" [style.backgroundColor]=\"datavalue\">&nbsp;</i>\n</span>\n",
            providers: [
                provideAs(ColorPickerComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(ColorPickerComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(ColorPickerComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], ColorPickerComponent);
    return ColorPickerComponent;
}(BaseFormCustomComponent));

var components = [
    ColorPickerComponent
];
var ColorPickerModule = /** @class */ (function () {
    function ColorPickerModule() {
    }
    ColorPickerModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                ColorPickerModule$1,
                FormsModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], ColorPickerModule);
    return ColorPickerModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ColorPickerModule, ColorPickerComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
