import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ViewChild, ElementRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NgModel, NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, register, registerFormWidget, TrailingZeroDecimalPipe, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { NumberLocale, InputModule } from '@wm/components/input';
import { FormWidgetType, CURRENCY_INFO, AppDefaults, AbstractI18nService } from '@wm/core';

var currencyProps = new Map([
    ['class', PROP_STRING],
    ['currency', PROP_STRING],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['maxvalue', PROP_NUMBER],
    ['minvalue', PROP_NUMBER],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Enter value' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['step', __assign({ value: 1 }, PROP_NUMBER)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['trailingzero', __assign({ value: false }, PROP_BOOLEAN)]
]);
var registerProps = function () {
    register('wm-currency', currencyProps);
    registerFormWidget(FormWidgetType.CURRENCY, new Map(currencyProps));
};

var DEFAULT_CLS = 'input-group app-currency';
var WIDGET_CONFIG = {
    widgetType: 'wm-currency',
    hostClass: DEFAULT_CLS
};
var CurrencyComponent = /** @class */ (function (_super) {
    __extends(CurrencyComponent, _super);
    function CurrencyComponent(inj, appDefaults, i18nService, trailingZeroDecimalPipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG, i18nService, trailingZeroDecimalPipe) || this;
        _this.appDefaults = appDefaults;
        _this.currencySymbol = CURRENCY_INFO[_this.appDefaults.currencyCode || 'USD'].symbol;
        return _this;
    }
    CurrencyComponent_1 = CurrencyComponent;
    CurrencyComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'currency') {
            this.currencySymbol = CURRENCY_INFO[this.currency || this.appDefaults.currencyCode].symbol;
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var CurrencyComponent_1;
    CurrencyComponent.initializeProps = registerProps();
    CurrencyComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: AppDefaults },
        { type: AbstractI18nService },
        { type: TrailingZeroDecimalPipe }
    ]; };
    __decorate([
        ViewChild(NgModel, { static: true }),
        __metadata("design:type", NgModel)
    ], CurrencyComponent.prototype, "ngModel", void 0);
    __decorate([
        ViewChild('input', { static: true, read: ElementRef }),
        __metadata("design:type", ElementRef)
    ], CurrencyComponent.prototype, "inputEl", void 0);
    CurrencyComponent = CurrencyComponent_1 = __decorate([
        Component({
            selector: '[wmCurrency]',
            template: "<span class=\"input-group-addon\" [textContent]=\"currencySymbol\"></span>\n<input class=\"form-control app-textbox app-currency-input\"\n       #input\n       role=\"input\"\n       type=\"text\"\n       focus-target\n       [autofocus]=\"autofocus\"\n       [readonly]=\"readonly\"\n       [disabled]=\"disabled\"\n       [required]=\"required\"\n       [pattern]=\"regexp\"\n       [attr.name]=\"name\"\n       [attr.aria-label]=\"name\"\n       [attr.tabindex]=\"tabindex\"\n       [attr.placeholder]=\"placeholder\"\n       [attr.accesskey]=\"shortcutkey\"\n       [ngModel]=\"displayValue\"\n       [ngModelOptions]=\"ngModelOptions\"\n\n       (blur)=\"handleBlur($event)\"\n       (ngModelChange)=\"datavalue=$event\"\n       (keydown)=\"validateInputEntry($event)\"\n       (keydown.enter)=\"onEnter($event)\"\n       (keydown.ArrowUp)=\"onArrowPress($event, 'UP')\"\n       (keydown.ArrowDown)=\"onArrowPress($event, 'DOWN')\">",
            providers: [
                provideAs(CurrencyComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(CurrencyComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(CurrencyComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, AppDefaults, AbstractI18nService, TrailingZeroDecimalPipe])
    ], CurrencyComponent);
    return CurrencyComponent;
}(NumberLocale));

var components = [
    CurrencyComponent
];
var CurrencyModule = /** @class */ (function () {
    function CurrencyModule() {
    }
    CurrencyModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                FormsModule,
                InputModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], CurrencyModule);
    return CurrencyModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { CurrencyModule, CurrencyComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
