import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, HostBinding, Component, HostListener, Directive, Attribute, ViewChild, ElementRef, ContentChildren, NgModule } from '@angular/core';
import { getLocaleNumberSymbol, NumberSymbol, CommonModule } from '@angular/common';
import { FormControlName, NgModel, NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule } from '@angular/forms';
import { IMaskModule } from 'angular-imask';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, DISPLAY_TYPE, styler, provideAsWidgetRef, StylableComponent, APPLY_STYLES_TYPE, registerFormWidget, provideAs, extractDataAsArray, ALLFIELDS, getOrderedDataset, convertDataToObject, transformDataWithKeys, transformFormData, getUniqObjsByDataField, groupData, handleHeaderClick, toggleAllHeaders, ToDatePipe, WidgetRef, TrailingZeroDecimalPipe, WmComponentsModule } from '@wm/components/base';
import { DataSource, FormWidgetType, toggleClass, isDefined, debounce, $appDigest, isEqualWithFields, toBoolean, noop, setListClass, AppDefaults, switchClass, addForIdAttributes, addClass, AbstractI18nService, setAttr, removeAttr, setCSS } from '@wm/core';
import { Subject } from 'rxjs';

var registerProps = function () {
    register('wm-button', new Map([
        ['animation', PROP_STRING],
        ['badgevalue', PROP_STRING],
        ['caption', PROP_STRING],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['conditionalstyle', PROP_ANY],
        ['disabled', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconposition', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['name', PROP_STRING],
        ['shortcutkey', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['type', PROP_STRING]
    ]));
};

var DEFAULT_CLS = 'btn app-button';
var WIDGET_CONFIG = {
    widgetType: 'wm-button',
    hostClass: DEFAULT_CLS,
    displayType: DISPLAY_TYPE.INLINE_BLOCK
};
var ButtonComponent = /** @class */ (function (_super) {
    __extends(ButtonComponent, _super);
    function ButtonComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        styler(_this.nativeElement, _this);
        return _this;
    }
    ButtonComponent_1 = ButtonComponent;
    var ButtonComponent_1;
    ButtonComponent.initializeProps = registerProps();
    ButtonComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        HostBinding('type'),
        __metadata("design:type", String)
    ], ButtonComponent.prototype, "type", void 0);
    __decorate([
        HostBinding('tabIndex'),
        __metadata("design:type", Number)
    ], ButtonComponent.prototype, "tabindex", void 0);
    __decorate([
        HostBinding('disabled'),
        __metadata("design:type", Boolean)
    ], ButtonComponent.prototype, "disabled", void 0);
    __decorate([
        HostBinding('attr.accesskey'),
        __metadata("design:type", String)
    ], ButtonComponent.prototype, "shortcutkey", void 0);
    __decorate([
        HostBinding('attr.icon-position'),
        __metadata("design:type", String)
    ], ButtonComponent.prototype, "iconposition", void 0);
    ButtonComponent = ButtonComponent_1 = __decorate([
        Component({
            selector: 'button[wmButton]',
            template: "<img data-identifier=\"img\" alt=\"button image\" class=\"button-image-icon\" [src]=\"iconurl | image\" *ngIf=\"iconurl\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin}\"/>\n<i class=\"app-icon {{iconclass}}\" aria-hidden=\"true\" [ngStyle]=\"{width:iconwidth, height:iconheight, margin:iconmargin, fontSize:iconwidth}\" *ngIf=\"iconclass\"></i>\n<span class=\"sr-only\" *ngIf=\"iconclass\">{{caption | trustAs:'html'}} {{appLocale.LABEL_ICON}}</span>\n<span class=\"btn-caption\" [innerHTML]=\"caption | trustAs:'html'\"></span>\n<ng-content select=\".caret\"></ng-content>\n<span *ngIf=\"badgevalue\" class=\"badge pull-right\" [textContent]=\"badgevalue\"></span>",
            providers: [
                provideAsWidgetRef(ButtonComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], ButtonComponent);
    return ButtonComponent;
}(StylableComponent));

var registerProps$1 = function () {
    register('wm-buttongroup', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['vertical', PROP_BOOLEAN]
    ]));
};

var DEFAULT_CLS$1 = 'btn-group app-button-group';
var WIDGET_CONFIG$1 = {
    widgetType: 'wm-buttongroup',
    hostClass: DEFAULT_CLS$1
};
var ButtonGroupDirective = /** @class */ (function (_super) {
    __extends(ButtonGroupDirective, _super);
    function ButtonGroupDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    ButtonGroupDirective_1 = ButtonGroupDirective;
    ButtonGroupDirective.prototype.handleClick = function (event) {
        var $target = $(event.target).closest('.app-button');
        this.$element.find('.app-button').removeClass('selected');
        $target.addClass('selected');
    };
    var ButtonGroupDirective_1;
    ButtonGroupDirective.initializeProps = registerProps$1();
    ButtonGroupDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        HostBinding('class.btn-group-vertical'),
        __metadata("design:type", Boolean)
    ], ButtonGroupDirective.prototype, "vertical", void 0);
    __decorate([
        HostListener('click', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], ButtonGroupDirective.prototype, "handleClick", null);
    ButtonGroupDirective = ButtonGroupDirective_1 = __decorate([
        Directive({
            selector: '[wmButtonGroup]',
            providers: [
                provideAsWidgetRef(ButtonGroupDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], ButtonGroupDirective);
    return ButtonGroupDirective;
}(StylableComponent));

var BaseFormComponent = /** @class */ (function (_super) {
    __extends(BaseFormComponent, _super);
    function BaseFormComponent(inj, config, initPromise) {
        var _this = _super.call(this, inj, config, initPromise) || this;
        _this.inj = inj;
        _this.binddatavalue = _this.$element.attr('datavalue.bind');
        return _this;
    }
    /**
     * Responsible for updating the variable bound to the widget's datavalue property.
     * @param value
     */
    BaseFormComponent.prototype.updateBoundVariable = function (value) {
        var binddatavalue = this.binddatavalue;
        // return if the variable bound is not static.
        if (this.datavaluesource && this.datavaluesource.execute(DataSource.Operation.IS_API_AWARE)) {
            return;
        }
        else if (this.datavaluesource && !this.datavaluesource.twoWayBinding) {
            return;
        }
        // return if widget is bound.
        if (!binddatavalue || binddatavalue.startsWith('Widgets.') || binddatavalue.startsWith('itemRef.currentItemWidgets')) {
            return;
        }
        binddatavalue = binddatavalue.replace(/\[\$i\]/g, '[0]');
        // In case of list widget context will be the listItem.
        if (_.has(this.context, binddatavalue.split('.')[0])) {
            _.set(this.context, binddatavalue, value);
        }
        else {
            _.set(this.viewParent, binddatavalue, value);
        }
    };
    BaseFormComponent.prototype.invokeOnChange = function (value, $event) {
        // invoke the event callback
        if ($event) {
            if (this.datavalue !== this.prevDatavalue) {
                this.updateBoundVariable(value);
                this.invokeEventCallback('change', {
                    $event: $event,
                    newVal: value,
                    oldVal: this.prevDatavalue
                });
            }
        }
        // update the previous value
        this.prevDatavalue = value;
    };
    BaseFormComponent.prototype.updatePrevDatavalue = function (val) {
        this.prevDatavalue = val;
    };
    BaseFormComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        // [WMS-18892]- stopping event propagation [for ArrowLeft, ArrowUp, ArrowRight, ArrowDown actions]  when form widgets or form is inside list widget
        var parentElemList = $(this.nativeElement).parents();
        if (parentElemList.closest('[wmList]').length) {
            this.$element.keydown(function ($event) {
                if ($event.keyCode === 37 || $event.keyCode === 38 || $event.keyCode === 39 || $event.keyCode === 40) {
                    $event.stopPropagation();
                }
            });
        }
    };
    return BaseFormComponent;
}(StylableComponent));

/*
 * WMS-18269
 * custom 'required' validator for components,
 * as the bound values are resolved at runtime
 * Called on 'required' property change
 *
 * Note: Component needs NG_VALIDATOR provider for this to work
 */
function isValidValue(val) {
    switch (typeof val) {
        case 'object': return (!!val && (!!val.length || !!Object.keys(val).length));
        case 'number': return (!!val || val === 0);
        default: return !!val;
    }
}
function validateRequiredBind(required) {
    return function (control) {
        return required
            ? (isValidValue(control.value)
                ? null
                : { required: true })
            : null;
    };
}
var BaseFormCustomComponent = /** @class */ (function (_super) {
    __extends(BaseFormCustomComponent, _super);
    function BaseFormCustomComponent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onChange = function () { };
        _this._onTouched = function () { };
        return _this;
    }
    BaseFormCustomComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this._formControl = this.inj.get(FormControlName, null);
    };
    BaseFormCustomComponent.prototype.registerOnChange = function (fn) {
        this._onChange = fn;
    };
    BaseFormCustomComponent.prototype.registerOnTouched = function (fn) {
        this._onTouched = fn;
    };
    BaseFormCustomComponent.prototype.writeValue = function (value) {
        if (this.isDestroyed) {
            return;
        }
        if (this._formControl) {
            this.datavalue = value;
            this.onPropertyChange('datavalue', value);
            this.updatePrevDatavalue(value);
        }
        /*
         * WMS:18246
         * Call onChange on default value, so that the Component Model is updated
         * Do only When the Model Value is different from datavalue
         */
        (value !== this.datavalue) && this._onChange(this.datavalue);
    };
    BaseFormCustomComponent.prototype.invokeOnChange = function (value, $event, valid) {
        // let the angular know about the change
        this._onChange(value);
        if (valid) {
            _super.prototype.invokeOnChange.call(this, value, $event);
        }
    };
    BaseFormCustomComponent.prototype.invokeOnTouched = function ($event) {
        this._onTouched();
        if ($event) {
            this.invokeEventCallback('blur', { $event: $event });
        }
    };
    BaseFormCustomComponent.prototype.invokeOnFocus = function ($event) {
        this.invokeEventCallback('focus', { $event: $event });
    };
    /* WMS-18269 */
    BaseFormCustomComponent.prototype.validate = function (control) {
        return this['show'] ? validateRequiredBind(this['required'])(control) : null;
    };
    return BaseFormCustomComponent;
}(BaseFormComponent));

var checkboxProps = new Map([
    ['caption', PROP_STRING],
    ['class', PROP_STRING],
    ['conditionalclass', PROP_ANY],
    ['conditionalstyle', PROP_ANY],
    ['checkedvalue', __assign({ value: true }, PROP_STRING)],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['name', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['uncheckedvalue', __assign({ value: false }, PROP_STRING)]
]);
var registerProps$2 = function () {
    register('wm-checkbox', checkboxProps);
    registerFormWidget(FormWidgetType.CHECKBOX, checkboxProps);
    registerFormWidget(FormWidgetType.TOGGLE, checkboxProps);
};

var DEFAULT_CLS$2 = 'app-checkbox checkbox';
var WIDGET_CONFIG$2 = {
    widgetType: 'wm-checkbox',
    hostClass: DEFAULT_CLS$2
};
/*
 * try to convert the chekedvalue and unchecked values to boolean/number
 */
var unStringify = function (val, defaultVal) {
    if (val === null) {
        return defaultVal;
    }
    if (val === true || val === 'true') {
        return true;
    }
    if (val === false || val === 'false') {
        return false;
    }
    var number = parseInt(val, 10);
    if (!isNaN(number)) {
        return number;
    }
    return val;
};
var ɵ0 = unStringify;
var CheckboxComponent = /** @class */ (function (_super) {
    __extends(CheckboxComponent, _super);
    function CheckboxComponent(inj, checkedVal, uncheckedVal, type) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$2) || this;
        _this._caption = '&nbsp';
        _this._checkedvalue = unStringify(checkedVal, true);
        _this._uncheckedvalue = unStringify(uncheckedVal, false);
        // if the type of the checkbox is toggle update the related classes on the host node
        toggleClass(_this.nativeElement, 'app-toggle', type === 'toggle');
        return _this;
    }
    CheckboxComponent_1 = CheckboxComponent;
    Object.defineProperty(CheckboxComponent.prototype, "datavalue", {
        // if the checkbox is checked, return checkedvalue else return uncheckedvalue
        get: function () {
            return isDefined(this.proxyModel) ? (this.proxyModel ? this._checkedvalue : this._uncheckedvalue) : undefined;
        },
        // when the datavalue is set, update the checked state
        set: function (v) {
            this.proxyModel = (isDefined(v) && v !== '') ? v === this._checkedvalue : undefined;
            this.updatePrevDatavalue(this.datavalue);
        },
        enumerable: true,
        configurable: true
    });
    CheckboxComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        if (key === 'required') {
            this._onChange(this.datavalue);
            return;
        }
        if (key === 'caption') {
            if (!isDefined(nv) || nv === '') {
                this._caption = '&nbsp;';
            }
            else {
                this._caption = nv;
            }
        }
        else if (key === 'checkedvalue') {
            this._checkedvalue = unStringify(nv, true);
        }
        else if (key === 'uncheckedvalue') {
            this._uncheckedvalue = unStringify(nv, false);
        }
        else if (key === 'datavalue') {
            this.datavalue = unStringify(nv);
            this._onChange(this.datavalue);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    // change and blur events are handled from template
    CheckboxComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName !== 'change' && eventName !== 'blur') {
            // applying tap (Hammer event) on the label as the event handler is not getting triggered on the input.
            var $el = eventName === 'tap' ? this.nativeElement.querySelector('label') : this.checkboxEl.nativeElement;
            _super.prototype.handleEvent.call(this, $el, eventName, callback, locals);
        }
    };
    CheckboxComponent.prototype.handleChange = function (newVal) {
        this.invokeOnChange(this.datavalue, { type: 'change' }, this.ngModel.valid);
    };
    CheckboxComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.nativeElement.querySelector('label'), this);
    };
    var CheckboxComponent_1;
    CheckboxComponent.initializeProps = registerProps$2();
    CheckboxComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: undefined, decorators: [{ type: Attribute, args: ['checkedvalue',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['uncheckedvalue',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['type',] }] }
    ]; };
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], CheckboxComponent.prototype, "ngModel", void 0);
    __decorate([
        ViewChild('checkbox', { static: true, read: ElementRef }),
        __metadata("design:type", ElementRef)
    ], CheckboxComponent.prototype, "checkboxEl", void 0);
    CheckboxComponent = CheckboxComponent_1 = __decorate([
        Component({
            selector: '[wmCheckbox]',
            template: "<label [ngClass]=\"{'unchecked': !proxyModel, 'disabled': (disabled || readonly), 'required': (required && _caption)}\" role=\"button\">\n    <input type=\"checkbox\" aria-describedby=\"checkbox\"\n           #checkbox\n           [attr.name]=\"name\"\n           focus-target\n           [(ngModel)]=\"proxyModel\"\n           [readonly]=\"readonly\"\n           [required]=\"required\"\n           [disabled]=\"disabled || readonly\"\n           [attr.accesskey]=\"shortcutkey\"\n           [tabindex]=\"tabindex\"\n           (blur)=\"invokeOnTouched($event)\"\n           (ngModelChange)=\"handleChange($event)\">\n    <span class=\"caption\" [innerHtml]=\"_caption\"></span>\n    <img alt=\"Checkbox Image\" src=\"data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==\" class=\"switch\"/>\n</label>\n<input type=\"hidden\" class=\"ng-hide model-holder\" [disabled]=\"disabled\" [value]=\"proxyModel\">\n",
            providers: [
                provideAs(CheckboxComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(CheckboxComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(CheckboxComponent_1)
            ]
        }),
        __param(1, Attribute('checkedvalue')),
        __param(2, Attribute('uncheckedvalue')),
        __param(3, Attribute('type')),
        __metadata("design:paramtypes", [Injector, Object, Object, Object])
    ], CheckboxComponent);
    return CheckboxComponent;
}(BaseFormCustomComponent));

var DatasetAwareFormComponent = /** @class */ (function (_super) {
    __extends(DatasetAwareFormComponent, _super);
    function DatasetAwareFormComponent(inj, WIDGET_CONFIG) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.datasetItems = [];
        _this.acceptsArray = false; // set to true if proxyModel on widget accepts array type.
        _this.dataset$ = new Subject();
        _this.datavalue$ = new Subject();
        _this.allowempty = true;
        _this._debounceDatavalueUpdation = _.debounce(function (values) {
            // if no item is found in datasetItems, wait untill the dataset updates by preserving the datavalue in toBeProcessedDatavalue.
            if (!isDefined(_this._modelByKey) || (_.isArray(_this._modelByKey) && !_this._modelByKey.length)) {
                _this.toBeProcessedDatavalue = values;
                _this._modelByValue = '';
            }
            else if (isDefined(_this.toBeProcessedDatavalue)) {
                // obtain the first array value when multiple is set to false.
                // set the modelByValue only when undefined.
                if (!isDefined(_this._modelByValue)) {
                    _this._modelByValue = (!_this.multiple && _.isArray(_this.toBeProcessedDatavalue)) ? _this.toBeProcessedDatavalue[0] : _this.toBeProcessedDatavalue;
                }
                _this.toBeProcessedDatavalue = undefined;
            }
            _this.initDisplayValues();
        }, 150);
        _this.binddisplayexpression = _this.nativeElement.getAttribute('displayexpression.bind');
        _this.binddisplayimagesrc = _this.nativeElement.getAttribute('displayimagesrc.bind');
        _this.binddisplaylabel = _this.nativeElement.getAttribute('displaylabel.bind');
        _this._debouncedInitDatasetItems = debounce(function () {
            _this.initDatasetItems();
            $appDigest();
        }, 150);
        return _this;
    }
    Object.defineProperty(DatasetAwareFormComponent.prototype, "modelByKey", {
        get: function () {
            return this._modelByKey;
        },
        // triggers on ngModel change. This function extracts the datavalue value.
        set: function (val) {
            this.selectByKey(val);
            // invoke on datavalue change.
            this.invokeOnChange(this._modelByValue);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DatasetAwareFormComponent.prototype, "datavalue", {
        get: function () {
            return this._modelByValue;
        },
        // triggers on setting the datavalue. This function extracts the model value.
        set: function (val) {
            if (this.multiple) {
                val = extractDataAsArray(val);
            }
            this._modelByValue = val;
            this.selectByValue(val);
            // changes on the datavalue can be subscribed using listenToDatavalue
            this.datavalue$.next(val);
            // invoke on datavalue change.
            this.invokeOnChange(val, undefined, true);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * This function sets the _datavalue value from the model and sets the selected flag when item is found.
     * Here model is the value obtained from ngModel.
     * @param keys represent the model.
     */
    DatasetAwareFormComponent.prototype.selectByKey = function (keys) {
        var _this = this;
        this.resetDatasetItems();
        if (!this.datasetItems.length) {
            return;
        }
        if (this.multiple && !_.isArray(keys)) {
            keys = [keys];
        }
        // Set the _modelByKey to the modified keys.
        this._modelByKey = keys;
        if (this.multiple) {
            this._modelByValue = [];
            keys.forEach(function (key) {
                var itemByKey = _.find(_this.datasetItems, function (item) {
                    // not triple equal, as the instance type can be different.
                    // only value comparison should be done.
                    return _.toString(item.key) === _.toString(key);
                });
                if (itemByKey) {
                    itemByKey.selected = true;
                    _this._modelByValue = __spread(_this._modelByValue, [itemByKey.value]);
                }
            });
        }
        else {
            this._modelByValue = '';
            var itemByKey = _.find(this.datasetItems, function (item) {
                // not triple equal, as the instance type can be different.
                // only value comparison should be done.
                return _.toString(item.key) === _.toString(keys);
            });
            if (itemByKey) {
                itemByKey.selected = true;
                this._modelByValue = itemByKey.value;
            }
        }
        this.initDisplayValues();
    };
    /**
     * This function sets the _model value from the datavalue (selectedvalues) and sets the selected flag when item is found.
     * datavalue is the default value or a value representing the displayField (for suppose: object in case of ALLFIELDS).
     * If acceptsArray is true, the model always accepts an array.
     * For example, select always accepts model as array whether multiple select is true or false.
     * @param values represent the datavalue.
     */
    DatasetAwareFormComponent.prototype.selectByValue = function (values) {
        var _this = this;
        this.resetDatasetItems();
        // if datavalue is not defined or empty then set the model as undefined.
        if (!isDefined(values) || values === '' || _.isNull(values) || (values instanceof Array && !values.length)) {
            this._modelByKey = undefined;
            // do not return when allowempty is set to true.
            if (!this.allowempty || !isDefined(values)) {
                return;
            }
        }
        if (this.acceptsArray && !_.isArray(values)) {
            values = this.allowempty ? [values] : extractDataAsArray(values);
        }
        // preserve the datavalue if datasetItems are empty.
        if (!this.datasetItems.length && isDefined(values)) {
            this.toBeProcessedDatavalue = values;
            return;
        }
        var filterField = this.datafield === ALLFIELDS ? 'dataObject' : 'key';
        if (_.isArray(values)) {
            this._modelByKey = [];
            values.forEach(function (val) {
                var itemByValue = _.find(_this.datasetItems, function (item) {
                    if (filterField === 'dataObject') {
                        if (_this.compareby && _this.compareby.length) {
                            return isEqualWithFields(item[filterField], val, _this.compareby);
                        }
                    }
                    return (_.isObject(item.value) ? _.isEqual(item.value, val) : (_.toString(item.value)).toLowerCase() === (_.toString(val)).toLowerCase());
                });
                if (itemByValue) {
                    itemByValue.selected = true;
                    _this._modelByKey.push(itemByValue.key);
                }
            });
        }
        else {
            this._modelByKey = undefined;
            var itemByValue = _.find(this.datasetItems, function (item) {
                if (filterField === 'dataObject') {
                    if (_this.compareby && _this.compareby.length) {
                        return isEqualWithFields(item[filterField], values, _this.compareby);
                    }
                }
                return (_.isObject(item.value) ? _.isEqual(item.value, values) : (_.toString(item.value)).toLowerCase() === (_.toString(values)).toLowerCase());
            });
            if (itemByValue) {
                itemByValue.selected = true;
                this._modelByKey = itemByValue.key;
            }
        }
        // delaying the datavalue update as the widgets in liveform are having datavalue as undefined and not the default provided value
        // because datavalue is updated later when new dataset is available.
        this._debounceDatavalueUpdation(values);
    };
    // Updates the displayValue property.
    DatasetAwareFormComponent.prototype.initDisplayValues = function () {
        var displayValues = [];
        this.datasetItems.forEach(function (item) {
            if (item.selected) {
                displayValues.push(item.label);
            }
        });
        this.displayValue = this.multiple ? displayValues : displayValues[0] || '';
    };
    // This function parses the dataset and extracts the displayOptions from parsed dataset.
    DatasetAwareFormComponent.prototype.initDatasetItems = function () {
        if (!this.dataset || _.isEmpty(this.dataset)) {
            this.datasetItems = [];
            // notify the dataset listeners
            this.dataset$.next(this.datasetItems);
            return;
        }
        // convert any dataset to the object format.
        var orderedDataset = getOrderedDataset(convertDataToObject(this.dataset), this.orderby);
        if (this.usekeys) {
            this.datasetItems = transformDataWithKeys(orderedDataset);
        }
        else {
            var displayOptions = transformFormData(this.viewParent, orderedDataset, this.datafield, {
                displayField: this.displayfield || this.displaylabel,
                displayExpr: this.displayexpression,
                bindDisplayExpr: this.binddisplayexpression || this.binddisplaylabel,
                bindDisplayImgSrc: this.binddisplayimagesrc,
                displayImgSrc: this.displayimagesrc
            });
            // get the unique objects out of the extracted data. Notify change in datasetItems using [...datasetItems] notation
            this.datasetItems = __spread(getUniqObjsByDataField(displayOptions, this.datafield, this.displayfield || this.displaylabel, toBoolean(this.allowempty)));
        }
        this.postDatasetItemsInit();
    };
    // Once the datasetItems are ready, set the proxyModel by using datavalue.
    DatasetAwareFormComponent.prototype.postDatasetItemsInit = function () {
        if (this.datasetItems.length && !this._defaultQueryInvoked) {
            // use the latest of toBeProcessedDatavalue, datavalue
            var _datavalue = _.isEmpty(this.toBeProcessedDatavalue) ? this.datavalue : this.toBeProcessedDatavalue;
            this.selectByValue(_datavalue);
        }
        // notify the dataset listeners
        this.dataset$.next(this.datasetItems);
    };
    // Reset the selected flag on datasetItems to false.
    DatasetAwareFormComponent.prototype.resetDatasetItems = function () {
        this.datasetItems.forEach(function (item) { return item.selected = false; });
    };
    DatasetAwareFormComponent.prototype.setTemplate = function (partialName) {
        this.content = partialName;
    };
    DatasetAwareFormComponent.prototype.onPropertyChange = function (key, nv, ov) {
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
        switch (key) {
            case 'dataset':
            case 'datafield':
            case 'displayfield':
            case 'displaylabel':
            case 'displayexpression':
            case 'orderby':
            case 'usekeys':
                this._debouncedInitDatasetItems();
                break;
            case 'required':
            case 'datavalue':
                this._onChange(this.datavalue);
                break;
        }
    };
    return DatasetAwareFormComponent;
}(BaseFormCustomComponent));

var checkboxsetProps = new Map([
    ['class', PROP_STRING],
    ['collapsible', PROP_BOOLEAN],
    ['compareby', PROP_STRING],
    ['datafield', PROP_STRING],
    ['dataset', __assign({ value: 'Option 1, Option 2, Option 3' }, PROP_ANY)],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['dateformat', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['displayexpression', PROP_STRING],
    ['displayfield', PROP_STRING],
    ['displayValue', PROP_STRING],
    ['groupby', PROP_STRING],
    ['itemclass', __assign({ value: '' }, PROP_STRING)],
    ['itemsperrow', __assign({ value: 'xs-1 sm-1 md-1 lg-1' }, PROP_STRING)],
    ['listclass', __assign({ value: '' }, PROP_STRING)],
    ['match', PROP_STRING],
    ['name', PROP_STRING],
    ['orderby', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['showcount', PROP_BOOLEAN],
    ['showindevice', __assign({ displayType: 'inline-block', value: 'all' }, PROP_STRING)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['usekeys', PROP_BOOLEAN]
]);
var registerProps$3 = function () {
    register('wm-checkboxset', checkboxsetProps);
    registerFormWidget(FormWidgetType.CHECKBOXSET, new Map(checkboxsetProps));
};

var DEFAULT_CLS$3 = 'app-checkboxset list-group inline';
var WIDGET_CONFIG$3 = { widgetType: 'wm-checkboxset', hostClass: DEFAULT_CLS$3 };
var CheckboxsetComponent = /** @class */ (function (_super) {
    __extends(CheckboxsetComponent, _super);
    function CheckboxsetComponent(inj, groupby, appDefaults, datePipe) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$3) || this;
        _this.groupby = groupby;
        _this.appDefaults = appDefaults;
        _this.datePipe = datePipe;
        _this.layout = '';
        styler(_this.nativeElement, _this);
        _this.multiple = true;
        _this.handleHeaderClick = noop;
        return _this;
    }
    CheckboxsetComponent_1 = CheckboxsetComponent;
    CheckboxsetComponent.prototype.onCheckboxLabelClick = function ($event, key) {
        if (!$($event.target).is('input')) {
            return;
        }
        // construct the _model from the checked elements.
        var inputElements = this.nativeElement.querySelectorAll('input:checked');
        var keys = [];
        _.forEach(inputElements, function ($el) {
            keys.push($el.value);
        });
        this.modelByKey = keys;
        this.invokeOnTouched();
        // invoke on datavalue change.
        this.invokeOnChange(this.datavalue, $event || {}, true);
    };
    // change and blur events are added from the template
    CheckboxsetComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName === 'click') {
            this.eventManager.addEventListener(node, eventName, function (e) {
                if (!$(e.target).is('input')) {
                    return;
                }
                locals.$event = e;
                return callback();
            });
        }
        else if (!_.includes(['change'], eventName)) {
            _super.prototype.handleEvent.call(this, node, eventName, callback, locals);
        }
    };
    CheckboxsetComponent.prototype.getGroupedData = function () {
        return this.datasetItems.length ? groupData(this, convertDataToObject(this.datasetItems), this.groupby, this.match, this.orderby, this.dateformat, this.datePipe, 'dataObject', this.appDefaults) : [];
    };
    CheckboxsetComponent.prototype.datasetSubscription = function () {
        var _this = this;
        var datasetSubscription = this.dataset$.subscribe(function () {
            _this.groupedData = _this.getGroupedData();
        });
        this.registerDestroyListener(function () { return datasetSubscription.unsubscribe(); });
    };
    CheckboxsetComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        if (key === 'required') {
            this._onChange(this.datavalue);
            return;
        }
        if (key === 'itemsperrow') {
            setListClass(this);
        }
        else if (key === 'groupby' || key === 'match') {
            this.datasetSubscription();
            // If groupby is set, get the groupedData from the datasetItems.
            this.groupedData = this.getGroupedData();
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    CheckboxsetComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        if (this.groupby) {
            this.datasetSubscription();
            // If groupby is set, get the groupedData from the datasetItems.
            this.groupedData = this.getGroupedData();
        }
        // adding the handler for header click and toggle headers.
        if (this.groupby && this.collapsible) {
            this.handleHeaderClick = handleHeaderClick;
            this.toggleAllHeaders = toggleAllHeaders.bind(undefined, this);
        }
    };
    var CheckboxsetComponent_1;
    CheckboxsetComponent.initializeProps = registerProps$3();
    CheckboxsetComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: String, decorators: [{ type: Attribute, args: ['groupby',] }] },
        { type: AppDefaults },
        { type: ToDatePipe }
    ]; };
    CheckboxsetComponent = CheckboxsetComponent_1 = __decorate([
        Component({
            selector: '[wmCheckboxset]',
            exportAs: 'wmCheckboxset',
            template: "<ng-template [ngIf]=\"!groupby\" [ngIfElse]=\"groupedListTemplate\">\n    <li [ngClass]=\"['checkbox', 'app-checkbox', itemclass, itemsPerRowClass]\"\n        [class.active]=\"item.selected\"\n        *ngFor=\"let item of datasetItems; let i = index\"\n        [wmItemTemplate]=\"content\" [userComponentParams]=\"item\" partialContainer\n        (click)=\"onCheckboxLabelClick($event, item.key)\">\n        <!-- Default item template -->\n        <label *ngIf=\"!content\" class=\"app-checkboxset-label\" [ngClass]=\"{'disabled':disabled || readonly}\" [title]=\"item.label\">\n            <input [name]=\"'checkboxset_' + widgetId\" type=\"checkbox\" aria-label=\"checkbox group\"\n                   [tabindex]=\"tabindex\"\n                   [disabled]=\"disabled || readonly\" [attr.data-attr-index]=\"i\"\n                   [value]=\"item.key\" [tabindex]=\"tabindex\" [checked]=\"item.selected\"/>\n            <span class=\"caption\" [textContent]=\"item.label\"></span>\n        </label>\n        <!-- Custom partial template -->\n        <label *ngIf=\"content\" class=\"app-checkboxset-label\" [ngClass]=\"{'disabled':disabled || readonly}\" >\n            <input [name]=\"'checkboxset_' + widgetId\" type=\"checkbox\" aria-label=\"checkbox group\"\n                   [tabindex]=\"tabindex\"\n                   [disabled]=\"disabled || readonly\" [attr.data-attr-index]=\"i\"\n                   [value]=\"item.key\" [tabindex]=\"tabindex\" [checked]=\"item.selected\"/>\n            <span class=\"caption customTemplate\" partial-container-target></span>\n        </label>\n    </li>\n</ng-template>\n<input [disabled]=\"disabled || readonly\" hidden class=\"model-holder\">\n<div *ngIf=\"readonly || disabled\" class=\"readonly-wrapper\"></div>\n\n<!-- This template will be displayed when groupby is specified. -->\n<ng-template #groupedListTemplate>\n    <li *ngFor=\"let groupObj of groupedData\" class=\"app-list-item-group\">\n        <ul class=\"item-group\">\n            <li class=\"list-group-header\" (click)=\"handleHeaderClick($event)\" [title]=\"groupObj.key\" [ngClass]=\"{'collapsible-content': collapsible}\">\n                <h4 class=\"group-title\">{{groupObj.key}}\n                    <div class=\"header-action\">\n                        <i class=\"app-icon wi action wi-chevron-up\" *ngIf=\"collapsible\" title=\"{{appLocale.LABEL_COLLAPSE}}/{{appLocale.LABEL_EXPAND}}\"></i>\n                        <span *ngIf=\"showcount\" class=\"label label-default\" [textContent]=\"groupObj.data.length\"></span>\n                    </div>\n                </h4>\n            </li>\n            <li *ngFor=\"let item of groupObj.data; let i = index;\"\n                [ngClass]=\"['checkbox', 'app-checkbox', 'group-list-item', itemclass, itemsPerRowClass]\"\n                [class.active]=\"item.selected\"\n                [wmItemTemplate]=\"content\" [userComponentParams]=\"item\" partialContainer\n                (click)=\"onCheckboxLabelClick($event, item)\">\n                <!-- Default item template -->\n                <label *ngIf=\"!content\" class=\"app-checkboxset-label\" [ngClass]=\"{'disabled':disabled || readonly}\" [title]=\"item.label\">\n                    <input [name]=\"'checkboxset_' + widgetId\" type=\"checkbox\" aria-label=\"checkbox group\"\n                           [tabindex]=\"tabindex\"\n                           [disabled]=\"disabled || readonly\" [attr.data-attr-index]=\"i\"\n                           [value]=\"item.key\" [tabindex]=\"tabindex\" [checked]=\"item.selected\"/>\n                    <span class=\"caption\" [textContent]=\"item.label\"></span>\n                </label>\n                <!-- Custom partial template -->\n                <label *ngIf=\"content\" class=\"app-checkboxset-label\" [ngClass]=\"{'disabled':disabled || readonly}\" >\n                    <input [name]=\"'checkboxset_' + widgetId\" type=\"checkbox\" aria-label=\"checkbox group\"\n                           [tabindex]=\"tabindex\"\n                           [disabled]=\"disabled || readonly\" [attr.data-attr-index]=\"i\"\n                           [value]=\"item.key\" [tabindex]=\"tabindex\" [checked]=\"item.selected\"/>\n                    <span class=\"caption\" partial-container-target></span>\n                </label>\n            </li>\n        </ul>\n    </li>\n</ng-template>\n",
            providers: [
                provideAs(CheckboxsetComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(CheckboxsetComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(CheckboxsetComponent_1)
            ]
        }),
        __param(1, Attribute('groupby')),
        __metadata("design:paramtypes", [Injector, String, AppDefaults,
            ToDatePipe])
    ], CheckboxsetComponent);
    return CheckboxsetComponent;
}(DatasetAwareFormComponent));

var registerProps$4 = function () {
    register('wm-form-group', new Map([
        ['captionposition', PROP_STRING],
        ['name', PROP_STRING],
        ['required', PROP_BOOLEAN],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$4 = 'form-group app-composite-widget clearfix';
var WIDGET_CONFIG$4 = {
    widgetType: 'wm-form-group',
    hostClass: DEFAULT_CLS$4
};
var CAPTION_POSITION = {
    left: 'caption-left',
    right: 'caption-right',
    top: 'caption-top'
};
var CompositeDirective = /** @class */ (function (_super) {
    __extends(CompositeDirective, _super);
    function CompositeDirective(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$4) || this;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        return _this;
    }
    CompositeDirective_1 = CompositeDirective;
    /**
     * this is onPropertyChange handler for the form-group component
     * @param key
     * @param nv
     * @param ov
     */
    CompositeDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'captionposition') {
            switchClass(this.nativeElement, CAPTION_POSITION[nv], CAPTION_POSITION[ov]);
        }
        else if (key === 'required') {
            this.required = nv;
            this.assignRequiredToSubComponents();
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    /**
     * this method assigns the required on the component/directive based on the required attribute of the form-group
     */
    CompositeDirective.prototype.assignRequiredToSubComponents = function () {
        var _this = this;
        if (this.required && this.componentRefs) {
            setTimeout(function () {
                _this.componentRefs.forEach(function (componentRef) { return componentRef.widget.required = true; });
            }, 50);
        }
    };
    CompositeDirective.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        addForIdAttributes(this.nativeElement);
        this.assignRequiredToSubComponents();
    };
    var CompositeDirective_1;
    CompositeDirective.initializeProps = registerProps$4();
    CompositeDirective.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ContentChildren(WidgetRef, { descendants: true }),
        __metadata("design:type", Object)
    ], CompositeDirective.prototype, "componentRefs", void 0);
    CompositeDirective = CompositeDirective_1 = __decorate([
        Directive({
            selector: 'div[wmComposite]',
            providers: [
                provideAsWidgetRef(CompositeDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], CompositeDirective);
    return CompositeDirective;
}(StylableComponent));

var numberProps = new Map([
    ['autofocus', PROP_BOOLEAN],
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['localefilter', PROP_STRING],
    ['minvalue', PROP_NUMBER],
    ['maxvalue', PROP_NUMBER],
    ['name', PROP_STRING],
    ['numberfilter', PROP_STRING],
    ['placeholder', __assign({ value: 'Enter number' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['regexp', PROP_STRING],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['step', __assign({ value: 1 }, PROP_NUMBER)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['trailingzero', __assign({ value: false }, PROP_BOOLEAN)],
    ['updateon', PROP_STRING]
]);
var registerProps$5 = function () {
    register('wm-number', numberProps);
    registerFormWidget(FormWidgetType.NUMBER, new Map(numberProps));
};

var BaseInput = /** @class */ (function (_super) {
    __extends(BaseInput, _super);
    function BaseInput(inj, config) {
        var _this = _super.call(this, inj, config) || this;
        // possible values for ngModelOptions are 'blur' and 'change'
        // default is 'blur'
        _this.ngModelOptions = {
            updateOn: ''
        };
        var updateOn = _this.nativeElement.getAttribute('updateon') || 'blur';
        updateOn = updateOn === 'default' ? 'change' : updateOn;
        _this.ngModelOptions.updateOn = updateOn;
        return _this;
    }
    BaseInput.prototype.onPropertyChange = function (key, nv, ov) {
        // set the class on the input element
        if (key === 'tabindex') {
            return;
        }
        if (key === 'required') {
            /* WMS-18269 | Update Angular about the required attr value change */
            this._onChange(this.datavalue);
        }
        if (key === 'class') {
            if (this.inputEl.nativeElement) {
                switchClass(this.inputEl.nativeElement, nv, ov);
            }
        }
        else if (key === 'datavalue') {
            // update the oldDataValue when the datavalue is modified programmatically
            this.updatePrevDatavalue(nv);
            this._onChange(this.datavalue);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    // invoke the change callback
    BaseInput.prototype.handleChange = function (newValue) {
        if (this.autotrim && this.datavalue && _.isString(this.datavalue)) {
            this.datavalue = this.datavalue.trim();
        }
        this.invokeOnChange(this.datavalue, { type: 'change' }, this.ngModel.valid);
    };
    // Change event is registered from the template, Prevent the framework from registering one more event
    BaseInput.prototype.handleEvent = function (node, eventName, eventCallback, locals) {
        if (eventName !== 'change' && eventName !== 'blur') {
            _super.prototype.handleEvent.call(this, this.inputEl.nativeElement, eventName, eventCallback, locals);
        }
    };
    // invoke the blur callback
    BaseInput.prototype.handleBlur = function ($event) {
        this.invokeOnTouched($event);
    };
    // Update the model on enter key press
    BaseInput.prototype.flushViewChanges = function (val) {
        this.ngModel.update.next(val);
        $appDigest();
    };
    BaseInput.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        // add the class on the input element
        if (this.class) {
            addClass(this.inputEl.nativeElement, this.class);
        }
        styler(this.inputEl.nativeElement, this);
    };
    return BaseInput;
}(BaseFormCustomComponent));

var NumberLocale = /** @class */ (function (_super) {
    __extends(NumberLocale, _super);
    function NumberLocale(inj, config, i18nService, trailingZeroDecimalPipe) {
        var _this = _super.call(this, inj, config) || this;
        _this.trailingZeroDecimalPipe = trailingZeroDecimalPipe;
        _this.isDefaultQuery = true;
        _this.decimalValue = '';
        _this.selectedLocale = i18nService.getSelectedLocale();
        _this.DECIMAL = getLocaleNumberSymbol(_this.localefilter || _this.selectedLocale, NumberSymbol.Decimal);
        _this.GROUP = getLocaleNumberSymbol(_this.localefilter || _this.selectedLocale, NumberSymbol.Group);
        _this.numberfilter = '1.0-16';
        _this.resetValidations();
        return _this;
    }
    Object.defineProperty(NumberLocale.prototype, "datavalue", {
        // returns the actual model value of the widget.
        get: function () {
            return this.proxyModel;
        },
        // Setter for the datavalue.
        set: function (value) {
            // set text value to null if data value is empty.
            if (_.includes([null, undefined, ''], value)) {
                var input = this.inputEl.nativeElement;
                this.displayValue = input.value = this.proxyModel = null;
                this.resetValidations();
                this._onChange();
                return;
            }
            // if the widget has default value and if we change the locale, the value should be in selected locale format.
            if (this.isDefaultQuery) {
                var isLocalizedNumber = _.isString(value) && _.includes(value, this.DECIMAL);
                var parts = isLocalizedNumber ? value.split(this.DECIMAL) : _.isString(value) && value.split('.');
                this.decimalValue = parts[1] || '';
                value = isLocalizedNumber ? value : this.transformNumber(value);
            }
            // get a valid number form the text.
            var model = this.parseNumber(value.toString());
            // if the number is valid or if number is not in range update the model value.
            if (this.isValid(model)) {
                this.proxyModel = model;
                // update the display value in the text box.
                this.updateDisplayText();
                this.handleChange(model);
            }
            else {
                this.displayValue = value.toString();
                this.proxyModel = null;
                this.handleChange(null);
            }
        },
        enumerable: true,
        configurable: true
    });
    // resets all the flags related to the widget's validation.
    NumberLocale.prototype.resetValidations = function () {
        this.isInvalidNumber = false;
        this.numberNotInRange = false;
    };
    /**
     * Adds validations for the number before updating the widget model. like validating min and max value for the widget.
     * @param {number} val number to be validated
     * @returns {number}
     */
    NumberLocale.prototype.isValid = function (val) {
        // id number is infinite then consider it as invalid value
        if (_.isNaN(val) || !_.isFinite(val) || (!Number.isInteger(this.step) &&
            this.countDecimals(val) > this.countDecimals(this.step))) {
            this.isInvalidNumber = true;
            return false;
        }
        if (val !== this.getValueInRange(val)) {
            this.numberNotInRange = true;
            return true;
        }
        this.resetValidations();
        return true;
    };
    /**
     * returns a valid number by validating the minimum and maximum values.
     * @param {number} value
     * @returns {number}
     */
    NumberLocale.prototype.getValueInRange = function (value) {
        if (!_.isNaN(this.minvalue) && value < this.minvalue) {
            this.validateType = 'minvalue';
            return this.minvalue;
        }
        if (!_.isNaN(this.maxvalue) && value > this.maxvalue) {
            this.validateType = 'maxvalue';
            return this.maxvalue;
        }
        return value;
    };
    /**
     * convert number to localized number using angular decimal pipe. eg 10,00,000 or 1,000,000
     * @param number
     * @returns {string}
     */
    NumberLocale.prototype.transformNumber = function (number) {
        return this.trailingZeroDecimalPipe.transform(number, this.selectedLocale, this.numberfilter, this.localefilter, this.trailingzero, this.decimalValue);
    };
    /**
     * resets the cursor position in the text box.
     * @param {number} value cursor position index form left to right.
     */
    NumberLocale.prototype.resetCursorPosition = function (value) {
        var input = this.inputEl.nativeElement;
        // position of the cursor should be given form right to left.
        var position = input.value.length - value;
        position = position < 0 ? 0 : position;
        // set the cursor position in the text box.
        input.setSelectionRange(position, position);
    };
    /**
     * Method parses the Localized number(string) to a valid number.
     * if the string dose not result to a valid number then returns NaN.
     * @param {string} val Localized number.
     * @returns {number}
     */
    NumberLocale.prototype.parseNumber = function (val) {
        // splits string into two parts. decimal and number.
        var parts = val.split(this.DECIMAL);
        if (!parts.length) {
            return null;
        }
        if (parts.length > 2) {
            return NaN;
        }
        // If number have decimal point and not have a decimal value then return.
        if (parts[1] === '') {
            return NaN;
        }
        // replaces all group separators form the number.
        var number = Number(parts[0].split(this.GROUP).join(''));
        var decimal = Number("0." + (parts[1] || 0));
        if (Number.isNaN(number) || Number.isNaN(decimal)) {
            return NaN;
        }
        // if the number is negative then calculate the number as number - decimal
        // Ex: number = -123 and decimal = 0.45 then number - decimal = -123-045 = -123.45
        return number >= 0 ? number + decimal : number - decimal;
    };
    // updates the widgets text value.
    NumberLocale.prototype.updateDisplayText = function () {
        var input = this.inputEl.nativeElement;
        var position = input.selectionStart;
        var preValue = input.value;
        if (!this.isDefaultQuery) {
            var parts = preValue.split(this.DECIMAL);
            this.decimalValue = parts[1] || '';
        }
        this.displayValue = input.value = this.transformNumber(this.proxyModel);
        // in safari browser, setSelectionRange will focus the input by default, which may invoke the focus event on widget.
        // Hence preventing the setSelectionRange when default value is set i.e. widget is not focused.
        if (this.updateon === 'default' && !this.isDefaultQuery) {
            this.resetCursorPosition(preValue.length - position);
        }
    };
    /**
     * returns the number of decimal places a number have.
     * @param value: number
     * @returns {number}
     */
    NumberLocale.prototype.countDecimals = function (value) {
        if ((value % 1) !== 0) {
            var decimalValue = value.toString().split('.')[1];
            return decimalValue && decimalValue.length;
        }
        return 0;
    };
    /**
     * handles the arrow press event. Increases or decreases the number. triggered fom the template
     * @param $event keyboard event.
     * @param key identifier to increase or decrease the number.
     */
    NumberLocale.prototype.onArrowPress = function ($event, key) {
        $event.preventDefault();
        if (this.readonly || this.step === 0) {
            return;
        }
        var proxyModel = this.proxyModel;
        var value;
        // if the number is not in range and when arrow buttons are pressed need to get appropriate number value.
        if (this.numberNotInRange) {
            var inputValue = this.parseNumber(this.inputEl.nativeElement.value);
            // take the textbox value as current model if the value is valid.
            if (!_.isNaN(inputValue)) {
                value = this.getValueInRange(inputValue);
                proxyModel = inputValue;
                this.resetValidations();
            }
        }
        else {
            if (_.isUndefined(proxyModel) || _.isNull(proxyModel)) {
                proxyModel = value = this.getValueInRange((this.minvalue || 0));
                this.resetValidations();
            }
            else {
                value = this.getValueInRange(proxyModel + (key === 'UP' ? this.step : -this.step));
            }
        }
        if ((key === 'UP' && proxyModel <= value) || (key === 'DOWN' && proxyModel >= value)) {
            var decimalRoundValue = Math.max(this.countDecimals(proxyModel), this.countDecimals(this.step));
            // update the modelProxy.
            this.proxyModel = _.round(value, decimalRoundValue);
            this.updateDisplayText();
            this.handleChange(this.proxyModel);
        }
    };
    /**
     * method is called from the from widget. to check whether the value entered is valid or not.
     * @returns {object}
     */
    NumberLocale.prototype.validate = function (c) {
        if (this.isInvalidNumber) {
            this.validateType = '';
            return {
                invalidNumber: {
                    valid: false
                },
            };
        }
        if (this.numberNotInRange) {
            return {
                numberNotInRange: {
                    valid: false
                },
            };
        }
        this.validateType = '';
        /* WMS-18269 | Extending the existing validation for 'required' */
        if (this['show'] && this['required']) {
            return (!!c.value || c.value === 0) ? null : { required: true };
        }
        return null;
    };
    NumberLocale.prototype.validateInputEntry = function ($event) {
        this.isDefaultQuery = false;
        // allow actions if control key is pressed or if backspace is pressed. (for Mozilla).
        if ($event.ctrlKey || _.includes(['Backspace', 'ArrowRight', 'ArrowLeft', 'Tab', 'Enter'], $event.key)) {
            return;
        }
        var validity = new RegExp("^[\\d\\s-,.e+" + this.GROUP + this.DECIMAL + "]$", 'i');
        var inputValue = $event.target.value;
        // validates if user entered an invalid character.
        if (!validity.test($event.key)) {
            return false;
        }
        // a decimal value can be entered only once in the input.
        if (_.includes(inputValue, this.DECIMAL) && $event.key === this.DECIMAL) {
            return false;
        }
        // 'e' can be entered only once in the input.
        if (_.intersection(_.toArray(inputValue), ['e', 'E']).length && _.includes('eE', $event.key)) {
            return false;
        }
        if ((_.includes(inputValue, '+') || _.includes(inputValue, '-')) && ($event.key === '+' || $event.key === '-')) {
            return false;
        }
    };
    NumberLocale.prototype.onEnter = function ($event) {
        this.datavalue = $event.target.value;
    };
    NumberLocale.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'minvalue' || key === 'maxvalue') {
            this.isValid(nv);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    return NumberLocale;
}(BaseInput));

var WIDGET_CONFIG$5 = {
    widgetType: 'wm-number',
    hostClass: 'app-input-wrapper'
};
var NumberComponent = /** @class */ (function (_super) {
    __extends(NumberComponent, _super);
    function NumberComponent(inj, i18nService, trailingZeroDecimalPipe) {
        return _super.call(this, inj, WIDGET_CONFIG$5, i18nService, trailingZeroDecimalPipe) || this;
    }
    NumberComponent_1 = NumberComponent;
    var NumberComponent_1;
    NumberComponent.initializeProps = registerProps$5();
    NumberComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: AbstractI18nService },
        { type: TrailingZeroDecimalPipe }
    ]; };
    __decorate([
        ViewChild('input', { static: true }),
        __metadata("design:type", ElementRef)
    ], NumberComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(NgModel, { static: true }),
        __metadata("design:type", NgModel)
    ], NumberComponent.prototype, "ngModel", void 0);
    NumberComponent = NumberComponent_1 = __decorate([
        Component({
            selector: '[wmNumber]',
            template: "<input class=\"form-control app-textbox app-number-input\"\n       inputmode=\"numeric\"\n       focus-target\n       [attr.name]=\"name\"\n       role=\"input\"\n       type=\"text\"\n       [ngModel]=\"displayValue\"\n       [readonly]=\"readonly\"\n       [disabled]=\"disabled\"\n       [pattern]=\"regexp\"\n       [attr.tabindex]=\"tabindex\"\n       [attr.placeholder]=\"placeholder\"\n       [attr.accesskey]=\"shortcutkey\"\n       [autofocus]=\"autofocus\"\n       (ngModelChange)=\"datavalue=$event\"\n       (blur)=\"handleBlur($event)\"\n       [ngModelOptions]=\"ngModelOptions\"\n       [required]=\"required\"\n       (keypress)=\"validateInputEntry($event)\"\n       (keydown.enter)=\"onEnter($event)\"\n       (keydown.ArrowUp)=\"onArrowPress($event, 'UP')\"\n       (keydown.ArrowDown)=\"onArrowPress($event, 'DOWN')\"\n       #input>",
            providers: [
                provideAs(NumberComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(NumberComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(NumberComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, AbstractI18nService, TrailingZeroDecimalPipe])
    ], NumberComponent);
    return NumberComponent;
}(NumberLocale));

var radiosetProps = new Map([
    ['class', PROP_STRING],
    ['compareby', PROP_STRING],
    ['datafield', PROP_STRING],
    ['dataset', __assign({ value: 'Option 1, Option 2, Option 3' }, PROP_ANY)],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_ANY],
    ['disabled', PROP_BOOLEAN],
    ['displayexpression', PROP_STRING],
    ['displayfield', PROP_STRING],
    ['displayValue', PROP_STRING],
    ['itemclass', __assign({ value: '' }, PROP_STRING)],
    ['itemsperrow', __assign({ value: 'xs-1 sm-1 md-1 lg-1' }, PROP_STRING)],
    ['listclass', __assign({ value: '' }, PROP_STRING)],
    ['name', PROP_STRING],
    ['orderby', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['showindevice', __assign({ displayType: 'inline-block', value: 'all' }, PROP_STRING)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['usekeys', PROP_BOOLEAN]
]);
var registerProps$6 = function () {
    register('wm-radioset', radiosetProps);
    registerFormWidget(FormWidgetType.RADIOSET, new Map(radiosetProps));
};

var DEFAULT_CLS$5 = 'app-radioset list-group inline';
var WIDGET_CONFIG$6 = { widgetType: 'wm-radioset', hostClass: DEFAULT_CLS$5 };
var RadiosetComponent = /** @class */ (function (_super) {
    __extends(RadiosetComponent, _super);
    function RadiosetComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$6) || this;
        _this.layout = '';
        styler(_this.nativeElement, _this);
        _this.multiple = false;
        return _this;
    }
    RadiosetComponent_1 = RadiosetComponent;
    /**
     * On click of the option, update the datavalue
     */
    RadiosetComponent.prototype.onRadioLabelClick = function ($event, key) {
        if (!$($event.target).is('input')) {
            return;
        }
        this.modelByKey = key;
        this.invokeOnTouched();
        // invoke on datavalue change.
        this.invokeOnChange(this.datavalue, $event || {}, true);
    };
    // change and blur events are added from the template
    RadiosetComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName === 'click') {
            this.eventManager.addEventListener(node, eventName, function (e) {
                if (!$(e.target).is('input')) {
                    return;
                }
                locals.$event = e;
                return callback();
            });
        }
        else if (!_.includes(['change'], eventName)) {
            _super.prototype.handleEvent.call(this, node, eventName, callback, locals);
        }
    };
    RadiosetComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        if (key === 'itemsperrow') {
            setListClass(this);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var RadiosetComponent_1;
    RadiosetComponent.initializeProps = registerProps$6();
    RadiosetComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    RadiosetComponent = RadiosetComponent_1 = __decorate([
        Component({
            selector: '[wmRadioset]',
            exportAs: 'wmRadioset',
            template: "<li [ngClass]=\"['radio', 'app-radio', itemclass, itemsPerRowClass]\"\n    [class.active]=\"item.selected\"\n    *ngFor=\"let item of datasetItems;let i = index\"\n    [wmItemTemplate]=\"content\" [userComponentParams]=\"item\" partialContainer\n    (click)=\"onRadioLabelClick($event, item.key)\">\n    <!-- Default item template -->\n    <label *ngIf=\"!content\" class=\"app-radioset-label\"\n           [ngClass]=\"{'disabled':disabled || readonly}\"\n           [title]=\"item.label\">\n        <input [name]=\"'radioset_' + widgetId\" type=\"radio\" [attr.aria-checked]=\"item.selected\" [attr.data-attr-index]=\"i\"\n               [value]=\"item.key\" [disabled]=\"disabled || readonly\" [tabindex]=\"tabindex\" [checked]=\"item.selected\"/>\n        <span class=\"caption\" [textContent]=\"item.label\"></span>\n    </label>\n    <!-- Custom partial template -->\n    <label *ngIf=\"content\" class=\"app-radioset-label\"\n           [ngClass]=\"{'disabled':disabled || readonly}\"\n           [title]=\"item.label\">\n        <input [name]=\"'radioset_' + widgetId\" type=\"radio\" [attr.aria-checked]=\"item.selected\" [attr.data-attr-index]=\"i\"\n               [value]=\"item.key\" [disabled]=\"disabled || readonly\" [tabindex]=\"tabindex\" [checked]=\"item.selected\"/>\n        <span class=\"caption customTemplate\" partial-container-target></span>\n    </label>\n</li>\n\n<input [disabled]=\"disabled || readonly\" hidden class=\"model-holder\">\n<div *ngIf=\"readonly || disabled\" aria-readonly=\"true\" class=\"readonly-wrapper\"></div>\n",
            providers: [
                provideAs(RadiosetComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(RadiosetComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(RadiosetComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], RadiosetComponent);
    return RadiosetComponent;
}(DatasetAwareFormComponent));

var selectProps = new Map([
    ['autofocus', PROP_BOOLEAN],
    ['class', __assign({ value: '' }, PROP_STRING)],
    ['compareby', PROP_STRING],
    ['datafield', PROP_STRING],
    ['dataset', PROP_ANY],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['displayexpression', PROP_STRING],
    ['displayfield', PROP_STRING],
    ['displayValue', PROP_STRING],
    ['hint', PROP_STRING],
    ['multiple', __assign({ value: false }, PROP_BOOLEAN)],
    ['name', PROP_STRING],
    ['orderby', PROP_STRING],
    ['placeholder', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['usekeys', PROP_BOOLEAN]
]);
var registerProps$7 = function () {
    register('wm-select', selectProps);
    registerFormWidget(FormWidgetType.SELECT, new Map(selectProps));
    registerFormWidget(FormWidgetType.TYPEAHEAD, new Map(selectProps));
};

var WIDGET_CONFIG$7 = { widgetType: 'wm-select', hostClass: 'app-select-wrapper' };
var SelectComponent = /** @class */ (function (_super) {
    __extends(SelectComponent, _super);
    function SelectComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$7) || this;
        _this.acceptsArray = true;
        return _this;
    }
    SelectComponent_1 = SelectComponent;
    Object.defineProperty(SelectComponent.prototype, "datasource", {
        set: function (ds) {
            if (ds && ds.execute && ds.execute(DataSource.Operation.IS_BOUND_TO_LOCALE)) {
                this.datavalue = ds.execute(DataSource.Operation.GET_DEFAULT_LOCALE);
            }
        },
        enumerable: true,
        configurable: true
    });
    SelectComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.selectEl.nativeElement, this);
    };
    // Change event is registered from the template, Prevent the framework from registering one more event
    SelectComponent.prototype.handleEvent = function (node, eventName, eventCallback, locals) {
        if (!_.includes(['blur', 'change'], eventName)) {
            _super.prototype.handleEvent.call(this, this.selectEl.nativeElement, eventName, eventCallback, locals);
        }
    };
    SelectComponent.prototype.onSelectValueChange = function ($event) {
        if (this.readonly) {
            if (this.placeholder) {
                this.selectEl.nativeElement.value = this.placeholder;
            }
            else {
                this.selectEl.nativeElement.value = '';
            }
            this.datavalue = this.prevDatavalue;
            return;
        }
        this.invokeOnTouched();
        // invoke on datavalue change.
        this.invokeOnChange(this.datavalue, $event || {}, true);
    };
    SelectComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'required') {
            this._onChange(this.datavalue);
            return;
        }
        if (key === 'class' || key === 'tabindex') {
            return;
        }
        else if (key === 'readonly') {
            (nv === true) ? setAttr(this.selectEl.nativeElement, 'readonly', 'readonly') : removeAttr(this.selectEl.nativeElement, 'readonly');
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    var SelectComponent_1;
    SelectComponent.initializeProps = registerProps$7();
    SelectComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('select', { static: true, read: ElementRef }),
        __metadata("design:type", ElementRef)
    ], SelectComponent.prototype, "selectEl", void 0);
    SelectComponent = SelectComponent_1 = __decorate([
        Component({
            selector: 'wm-select',
            template: "<select role=\"input\" aria-haspopup=\"true\" aria-expanded=\"false\" #select\n        focus-target\n        [ngClass]=\"['app-select form-control', class]\"\n        [disabled]=\"disabled\"\n        [required]=\"required\"\n        [tabindex]=\"tabindex\"\n        [(ngModel)]=\"modelByKey\"\n        [multiple]=\"multiple\"\n        (change)=\"onSelectValueChange($event)\"\n        (blur)=\"invokeOnTouched($event)\"\n        [attr.name]=\"name\"\n        [autofocus]=\"autofocus\"\n>\n  <option selected value=\"undefined\" [textContent]=\"placeholder\" [hidden]=\"!placeholder\" ></option>\n  <option\n          *ngFor=\"let item of datasetItems\"\n          [value]=\"item.key\" [selected]=\"item.selected\" [textContent]=\"item.label\"></option>\n</select>\n",
            providers: [
                provideAs(SelectComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(SelectComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(SelectComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], SelectComponent);
    return SelectComponent;
}(DatasetAwareFormComponent));

var switchProps = new Map([
    ['class', PROP_STRING],
    ['compareby', PROP_STRING],
    ['datafield', PROP_STRING],
    ['dataset', __assign({ value: 'yes, no, maybe' }, PROP_ANY)],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['displayexpression', PROP_STRING],
    ['displayfield', PROP_STRING],
    ['hint', PROP_STRING],
    ['iconclass', PROP_STRING],
    ['name', PROP_STRING],
    ['orderby', PROP_STRING],
    ['required', PROP_BOOLEAN],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
]);
var registerProps$8 = function () {
    register('wm-switch', switchProps);
    registerFormWidget(FormWidgetType.SWITCH, new Map(switchProps));
    registerFormWidget(FormWidgetType.TOGGLE, new Map(switchProps));
};

var DEFAULT_CLS$6 = 'app-switch';
var WIDGET_CONFIG$8 = { widgetType: 'wm-switch', hostClass: DEFAULT_CLS$6 };
var SwitchComponent = /** @class */ (function (_super) {
    __extends(SwitchComponent, _super);
    function SwitchComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$8) || this;
        _this.options = [];
        _this._debounceSetSelectedValue = debounce(function (val) {
            _this.setSelectedValue();
            _this.updateHighlighter(val);
            // only for default value trigger app digest to apply the selectedItem
            if (val) {
                $appDigest();
            }
        }, 200);
        var datasetSubscription = _this.dataset$.subscribe(function () { return _this.updateSwitchOptions(); });
        _this.registerDestroyListener(function () { return datasetSubscription.unsubscribe(); });
        var datavalueSubscription = _this.datavalue$.subscribe(function () {
            _this._debounceSetSelectedValue(true);
        });
        _this.registerDestroyListener(function () { return datavalueSubscription.unsubscribe(); });
        return _this;
    }
    SwitchComponent_1 = SwitchComponent;
    SwitchComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.nativeElement, this);
    };
    SwitchComponent.prototype.onStyleChange = function (key, nv, ov) {
        if (key === 'height') {
            setCSS(this.nativeElement, 'overflow', nv ? 'auto' : '');
        }
        else {
            _super.prototype.onStyleChange.call(this, key, nv, ov);
        }
    };
    // This function sets the selectedItem by either using compareby fields or selected flag on datasetItems.
    SwitchComponent.prototype.setSelectedValue = function () {
        if (isDefined(this.datavalue) || isDefined(this.toBeProcessedDatavalue)) {
            this.selectedItem = _.find(this.datasetItems, { selected: true });
            return;
        }
        // If no value is provided, set first value as default if options are available else set -1 ie no selection
        this.selectOptAtIndex(0);
    };
    // set the css for switch overlay element.
    // set the selected index from the datasetItems and highlight the datavalue on switch.
    SwitchComponent.prototype.updateSwitchOptions = function () {
        if (this.datasetItems.length) {
            this.btnwidth = (100 / this.datasetItems.length);
            setCSS(this.nativeElement.querySelector('.app-switch-overlay'), 'width', this.btnwidth + '%');
        }
        this._debounceSetSelectedValue(true);
    };
    // This function animates the highlighted span on to the selected value.
    SwitchComponent.prototype.updateHighlighter = function (skipAnimation) {
        var handler = $(this.nativeElement).find('span.app-switch-overlay');
        this.setSelectedValue();
        var left, index = this.selectedItem ? _.findIndex(this.datasetItems, { key: this.selectedItem.key }) : -1;
        if (index === undefined || index === null) {
            index = -1;
        }
        left = index * this.btnwidth;
        if (skipAnimation) {
            handler.css('left', left + '%');
        }
        else {
            handler.animate({
                left: left + '%'
            }, 300);
        }
    };
    SwitchComponent.prototype.selectOptAtIndex = function ($index) {
        if (!this.datasetItems.length) {
            return;
        }
        var opt = this.datasetItems[$index];
        this._modelByValue = opt.value;
    };
    // Triggered on selected the option from the switch.
    // set the index and highlight the default value. Invoke onchange event handler.
    SwitchComponent.prototype.selectOpt = function ($event, $index, option) {
        $event.preventDefault();
        if (this.disabled) {
            return;
        }
        this.modelByKey = option.key;
        this.invokeOnTouched();
        if (this.selectedItem && $index === _.findIndex(this.datasetItems, { key: this.selectedItem.key })) {
            if (this.datasetItems.length === 2) {
                $index = $index === 1 ? 0 : 1;
            }
            else {
                return;
            }
        }
        this.selectedItem = this.datasetItems[$index];
        this.updateHighlighter();
        // invoke on datavalue change.
        this.invokeOnChange(this.datavalue, $event || {}, true);
        $appDigest();
    };
    SwitchComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'disabled' && !toBoolean(nv)) {
            this.nativeElement.removeAttribute(key);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    var SwitchComponent_1;
    SwitchComponent.initializeProps = registerProps$8();
    SwitchComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    SwitchComponent = SwitchComponent_1 = __decorate([
        Component({
            selector: '[wmSwitch]',
            template: "<div role=\"group\" aria-label=\"button switch\" class=\"btn-group btn-group-justified\">\n    <a *ngFor=\"let opt of datasetItems; let $index = index;\"\n       [title]=\"opt.label\" focus-target\n       href=\"javascript:void(0);\"\n       class=\"btn btn-default\"\n       [name]=\"'wm-switch-' + opt.key\"\n       [ngClass]=\"{'selected': opt.selected, 'disabled': disabled}\"\n       (click)=\"selectOpt($event, $index, opt)\"\n    >\n        <i *ngIf=\"opt.dataObject && opt.dataObject[iconclass]\" aria-hidden=\"true\" [ngClass]=\"['app-icon', opt.dataObject[iconclass] || opt['icon']]\"></i>\n        <span class=\"caption\" [textContent]=\"opt[displayfield] || opt.label\"></span>\n    </a>\n</div>\n<span [title]=\"selectedItem ? selectedItem.label : modelByKey\"\n      class=\"btn btn-primary app-switch-overlay switch-handle\">\n    <i *ngIf=\"iconclass\"\n       class=\"app-icon {{(selectedItem && selectedItem.dataObject) && selectedItem.dataObject[iconclass]}}\"></i>\n    {{selectedItem ? selectedItem.label : modelByKey}}\n</span>\n<input [name]=\"name\" class=\"model-holder ng-hide\" [disabled]=\"disabled\" [value]=\"modelByKey\" [required]=\"required\">\n",
            providers: [
                provideAs(SwitchComponent_1, NG_VALUE_ACCESSOR, true),
                provideAsWidgetRef(SwitchComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], SwitchComponent);
    return SwitchComponent;
}(DatasetAwareFormComponent));

var inputCalendarTypeProps = new Map([
    ['autocomplete', __assign({ value: true }, PROP_BOOLEAN)],
    ['autotrim', __assign({ value: true }, PROP_BOOLEAN)],
    ['autofocus', PROP_BOOLEAN],
    ['class', PROP_STRING],
    ['datasource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['maxvalue', PROP_NUMBER],
    ['minvalue', PROP_NUMBER],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Enter text' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['step', PROP_NUMBER],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['type', PROP_STRING],
    ['updateon', PROP_STRING]
]);
var registerProps$9 = function () {
    register('wm-input-calendar', inputCalendarTypeProps);
};

var WIDGET_CONFIG$9 = {
    widgetType: 'wm-input-calendar',
    hostClass: 'app-input-wrapper'
};
var InputCalendarComponent = /** @class */ (function (_super) {
    __extends(InputCalendarComponent, _super);
    function InputCalendarComponent(inj) {
        return _super.call(this, inj, WIDGET_CONFIG$9) || this;
    }
    InputCalendarComponent_1 = InputCalendarComponent;
    var InputCalendarComponent_1;
    InputCalendarComponent.initializeProps = registerProps$9();
    InputCalendarComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('input', { static: true }),
        __metadata("design:type", ElementRef)
    ], InputCalendarComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], InputCalendarComponent.prototype, "ngModel", void 0);
    InputCalendarComponent = InputCalendarComponent_1 = __decorate([
        Component({
            selector: 'wm-input[type="date"], wm-input[type="datetime-local"], wm-input[type="month"], wm-input[type="time"], wm-input[type="week"]',
            template: "<input class=\"form-control app-textbox\"\n       focus-target\n       role=\"input\"\n       [type]=\"type\"\n       [attr.name]=\"name\"\n       [(ngModel)]=\"datavalue\"\n       [readonly]=\"readonly\"\n       [required]=\"required\"\n       [disabled]=\"disabled\"\n       [min]=\"minvalue\"\n       [max]=\"maxvalue\"\n       [step]=\"step\"\n       [attr.tabindex]=\"tabindex\"\n       [attr.placeholder]=\"placeholder\"\n       [attr.accesskey]=\"shortcutkey\"\n       [autofocus]=\"autofocus\"\n       (blur)=\"handleBlur($event)\"\n       (ngModelChange)=\"handleChange($event)\"\n       [autocomplete]=\"autocomplete ? 'on' : 'off'\"\n       #input>\n",
            providers: [
                provideAs(InputCalendarComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(InputCalendarComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(InputCalendarComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], InputCalendarComponent);
    return InputCalendarComponent;
}(BaseInput));

var inputColorTypeProps = new Map([
    ['autocomplete', __assign({ value: true }, PROP_BOOLEAN)],
    ['autofocus', PROP_BOOLEAN],
    ['autotrim', __assign({ value: true }, PROP_BOOLEAN)],
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Enter text' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['type', PROP_STRING],
    ['updateon', PROP_STRING]
]);
var registerProps$a = function () {
    register('wm-input-color', inputColorTypeProps);
};

var WIDGET_CONFIG$a = {
    widgetType: 'wm-input-color',
    hostClass: 'app-input-wrapper'
};
var InputColorComponent = /** @class */ (function (_super) {
    __extends(InputColorComponent, _super);
    function InputColorComponent(inj) {
        return _super.call(this, inj, WIDGET_CONFIG$a) || this;
    }
    InputColorComponent_1 = InputColorComponent;
    var InputColorComponent_1;
    InputColorComponent.initializeProps = registerProps$a();
    InputColorComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('input', { static: true }),
        __metadata("design:type", ElementRef)
    ], InputColorComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], InputColorComponent.prototype, "ngModel", void 0);
    InputColorComponent = InputColorComponent_1 = __decorate([
        Component({
            selector: 'wm-input[type="color"]',
            template: "<input class=\"form-control app-textbox\"\n       focus-target\n       type=\"color\"\n       role=\"input\"\n       [attr.name]=\"name\"\n       [(ngModel)]=\"datavalue\"\n       [readonly]=\"readonly\"\n       [required]=\"required\"\n       [disabled]=\"disabled\"\n       [maxlength]=\"maxchars\"\n       [attr.tabindex]=\"tabindex\"\n       [attr.placeholder]=\"placeholder\"\n       [attr.accesskey]=\"shortcutkey\"\n       [autofocus]=\"autofocus\"\n       (blur)=\"handleBlur($event)\"\n       (ngModelChange)=\"handleChange($event)\"\n       #input>",
            providers: [
                provideAs(InputColorComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(InputColorComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(InputColorComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], InputColorComponent);
    return InputColorComponent;
}(BaseInput));

var inputEmailTypeProps = new Map([
    ['autocomplete', __assign({ value: true }, PROP_BOOLEAN)],
    ['autofocus', PROP_BOOLEAN],
    ['autotrim', __assign({ value: true }, PROP_BOOLEAN)],
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['maxchars', PROP_NUMBER],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Enter text' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['regexp', PROP_STRING],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['type', PROP_STRING],
    ['updateon', PROP_STRING]
]);
var registerProps$b = function () {
    register('wm-input-email', inputEmailTypeProps);
};

var WIDGET_CONFIG$b = {
    widgetType: 'wm-input-email',
    hostClass: 'app-input-wrapper'
};
var InputEmailComponent = /** @class */ (function (_super) {
    __extends(InputEmailComponent, _super);
    function InputEmailComponent(inj) {
        return _super.call(this, inj, WIDGET_CONFIG$b) || this;
    }
    InputEmailComponent_1 = InputEmailComponent;
    var InputEmailComponent_1;
    InputEmailComponent.initializeProps = registerProps$b();
    InputEmailComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('input', { static: true }),
        __metadata("design:type", ElementRef)
    ], InputEmailComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], InputEmailComponent.prototype, "ngModel", void 0);
    InputEmailComponent = InputEmailComponent_1 = __decorate([
        Component({
            selector: 'wm-input[type="email"]',
            template: "<input class=\"form-control app-textbox\"\n       focus-target\n       role=\"input\"\n       type=\"email\"\n       [attr.name]=\"name\"\n       [(ngModel)]=\"datavalue\"\n       [ngModelOptions]=\"ngModelOptions\"\n       [readonly]=\"readonly\"\n       [required]=\"required\"\n       [disabled]=\"disabled\"\n       [maxlength]=\"maxchars\"\n       [pattern]=\"regexp\"\n       [attr.tabindex]=\"tabindex\"\n       [attr.placeholder]=\"placeholder\"\n       [attr.accesskey]=\"shortcutkey\"\n       [autofocus]=\"autofocus\"\n       (blur)=\"handleBlur($event)\"\n       (ngModelChange)=\"handleChange($event)\"\n       [autocomplete]=\"autocomplete ? 'on' : 'off'\"\n       (keyup.enter)=\"flushViewChanges(input.value)\"\n       email\n       #input>",
            providers: [
                provideAs(InputEmailComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(InputEmailComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(InputEmailComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], InputEmailComponent);
    return InputEmailComponent;
}(BaseInput));

var inputNumberTypeProps = new Map([
    ['autocomplete', __assign({ value: true }, PROP_BOOLEAN)],
    ['autofocus', PROP_BOOLEAN],
    ['autotrim', __assign({ value: true }, PROP_BOOLEAN)],
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['maxchars', PROP_NUMBER],
    ['maxvalue', PROP_NUMBER],
    ['minvalue', PROP_NUMBER],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Enter text' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['regexp', PROP_STRING],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['step', PROP_NUMBER],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['type', PROP_STRING],
    ['updateon', PROP_STRING]
]);
var registerProps$c = function () {
    register('wm-input-number', inputNumberTypeProps);
};

var WIDGET_CONFIG$c = {
    widgetType: 'wm-input-number',
    hostClass: 'app-input-wrapper'
};
var InputNumberComponent = /** @class */ (function (_super) {
    __extends(InputNumberComponent, _super);
    function InputNumberComponent(inj) {
        return _super.call(this, inj, WIDGET_CONFIG$c) || this;
    }
    InputNumberComponent_1 = InputNumberComponent;
    InputNumberComponent.prototype.onArrowPress = function ($event) {
        if (this.step === 0) {
            $event.preventDefault();
        }
    };
    InputNumberComponent.prototype.validateInputEntry = function ($event) {
        if ($event.key === 'e' && $event.target.value.indexOf($event.key) !== -1) {
            return false;
        }
    };
    var InputNumberComponent_1;
    InputNumberComponent.initializeProps = registerProps$c();
    InputNumberComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('input', { static: true }),
        __metadata("design:type", ElementRef)
    ], InputNumberComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], InputNumberComponent.prototype, "ngModel", void 0);
    InputNumberComponent = InputNumberComponent_1 = __decorate([
        Component({
            selector: 'wm-input[type="number"]',
            template: "<input class=\"form-control app-textbox\"\n       [ngClass]=\"{'step-hidden': step === 0}\"\n       focus-target\n       role=\"input\"\n       type=\"number\"\n       [attr.name]=\"name\"\n       [(ngModel)]=\"datavalue\"\n       [ngModelOptions]=\"ngModelOptions\"\n       [readonly]=\"readonly\"\n       [required]=\"required\"\n       [disabled]=\"disabled\"\n       [maxlength]=\"maxchars\"\n       [min]=\"minvalue\"\n       [max]=\"maxvalue\"\n       [step]=\"step\"\n       [attr.tabindex]=\"tabindex\"\n       [attr.placeholder]=\"placeholder\"\n       [attr.accesskey]=\"shortcutkey\"\n       [autofocus]=\"autofocus\"\n       (blur)=\"handleBlur($event)\"\n       (ngModelChange)=\"handleChange($event)\"\n       [autocomplete]=\"autocomplete ? 'on' : 'off'\"\n       (keyup.enter)=\"flushViewChanges(input.value)\"\n       (keydown.ArrowUp)=\"onArrowPress($event)\"\n       (keydown.ArrowDown)=\"onArrowPress($event)\"\n       (keypress)=\"validateInputEntry($event)\"\n       #input>",
            providers: [
                provideAs(InputNumberComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(InputNumberComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(InputNumberComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], InputNumberComponent);
    return InputNumberComponent;
}(BaseInput));

var inputTextTypeProps = new Map([
    ['autocomplete', __assign({ value: true }, PROP_BOOLEAN)],
    ['autofocus', PROP_BOOLEAN],
    ['autotrim', __assign({ value: true }, PROP_BOOLEAN)],
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['displayformat', PROP_STRING],
    ['hint', PROP_STRING],
    ['maxchars', PROP_NUMBER],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Enter text' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['regexp', PROP_STRING],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['type', __assign({ value: 'text' }, PROP_STRING)],
    ['updateon', PROP_STRING]
]);
var registerProps$d = function () {
    register('wm-input-text', inputTextTypeProps);
    registerFormWidget(FormWidgetType.PASSWORD, new Map(inputTextTypeProps));
    var formTextProps = new Map(inputTextTypeProps);
    var mergeTextProps = function (typeProps) {
        typeProps.forEach(function (v, k) { return formTextProps.set(k, v); });
    };
    mergeTextProps(inputCalendarTypeProps);
    mergeTextProps(inputColorTypeProps);
    mergeTextProps(inputEmailTypeProps);
    mergeTextProps(inputNumberTypeProps);
    registerFormWidget(FormWidgetType.TEXT, new Map(formTextProps));
};

var WIDGET_CONFIG$d = {
    widgetType: 'wm-input-text',
    hostClass: 'app-input-wrapper'
};
var InputTextComponent = /** @class */ (function (_super) {
    __extends(InputTextComponent, _super);
    function InputTextComponent(inj) {
        return _super.call(this, inj, WIDGET_CONFIG$d) || this;
    }
    InputTextComponent_1 = InputTextComponent;
    /* Define the property change handler. This function will be triggered when there is a change in the widget property */
    InputTextComponent.prototype.onPropertyChange = function (key, nv, ov) {
        /*Monitoring changes for styles or properties and accordingly handling respective changes.*/
        switch (key) {
            case 'displayformat':
                this.maskVal = this.displayformat;
                break;
            default:
                _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    Object.defineProperty(InputTextComponent.prototype, "mask", {
        get: function () {
            if (this.displayformat) {
                return {
                    mask: this.maskVal,
                    lazy: false,
                    definitions: {
                        '9': /\d/,
                        'A': /[a-zA-Z]/,
                        'a': /[a-z]/,
                        '*': /\w/
                    }
                };
            }
            else {
                return false;
            }
        },
        enumerable: true,
        configurable: true
    });
    var InputTextComponent_1;
    InputTextComponent.initializeProps = registerProps$d();
    InputTextComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('input', { static: true }),
        __metadata("design:type", ElementRef)
    ], InputTextComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], InputTextComponent.prototype, "ngModel", void 0);
    InputTextComponent = InputTextComponent_1 = __decorate([
        Component({
            selector: 'wm-input[type="text"], wm-input:not([type]), wm-input[type="password"], wm-input[type="search"], wm-input[type="tel"], wm-input[type="url"]',
            template: "<input class=\"form-control app-textbox\"\n       focus-target\n       role=\"input\"\n       [type]=\"type\"\n       [attr.name]=\"name\"\n       [(ngModel)]=\"datavalue\"\n       [imask]=\"mask\"\n       [unmask]=\"true\"\n       [ngModelOptions]=\"ngModelOptions\"\n       [readonly]=\"readonly\"\n       [required]=\"required\"\n       [disabled]=\"disabled\"\n       [maxlength]=\"maxchars\"\n       [pattern]=\"regexp\"\n       [attr.tabindex]=\"tabindex\"\n       [attr.placeholder]=\"placeholder\"\n       [attr.accesskey]=\"shortcutkey\"\n       [autofocus]=\"autofocus\"\n       (ngModelChange)=\"handleChange($event)\"\n       (blur)=\"handleBlur($event)\"\n       [autocomplete]=\"autocomplete ? 'on' : 'off'\"\n       (keyup.enter)=\"flushViewChanges(input.value)\"\n       #input>\n",
            providers: [
                provideAs(InputTextComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(InputTextComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(InputTextComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], InputTextComponent);
    return InputTextComponent;
}(BaseInput));

var textareaProps = new Map([
    ['autofocus', PROP_BOOLEAN],
    ['autotrim', __assign({ value: true }, PROP_BOOLEAN)],
    ['class', PROP_STRING],
    ['datavaluesource', PROP_ANY],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['maxchars', PROP_NUMBER],
    ['name', PROP_STRING],
    ['placeholder', __assign({ value: 'Place your text' }, PROP_STRING)],
    ['readonly', PROP_BOOLEAN],
    ['required', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
    ['updateon', PROP_STRING]
]);
var registerProps$e = function () {
    register('wm-textarea', textareaProps);
    registerFormWidget(FormWidgetType.TEXTAREA, new Map(textareaProps));
};

var WIDGET_CONFIG$e = {
    widgetType: 'wm-textarea',
    hostClass: 'app-input-wrapper'
};
var TextareaComponent = /** @class */ (function (_super) {
    __extends(TextareaComponent, _super);
    function TextareaComponent(inj) {
        return _super.call(this, inj, WIDGET_CONFIG$e) || this;
    }
    TextareaComponent_1 = TextareaComponent;
    var TextareaComponent_1;
    TextareaComponent.initializeProps = registerProps$e();
    TextareaComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild('textarea', { static: true }),
        __metadata("design:type", ElementRef)
    ], TextareaComponent.prototype, "inputEl", void 0);
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], TextareaComponent.prototype, "ngModel", void 0);
    TextareaComponent = TextareaComponent_1 = __decorate([
        Component({
            selector: 'wm-textarea',
            template: "<textarea class=\"form-control app-textarea\"\n          focus-target\n          role=\"input\"\n          [attr.name]=\"name\"\n          [(ngModel)]=\"datavalue\"\n          [ngModelOptions]=\"ngModelOptions\"\n          [readonly]=\"readonly\"\n          [required]=\"required\"\n          [disabled]=\"disabled\"\n          [maxlength]=\"maxchars\"\n          [attr.tabindex]=\"tabindex\"\n          [attr.placeholder]=\"placeholder\"\n          [attr.accesskey]=\"shortcutkey\"\n          [autofocus]=\"autofocus\"\n          (blur)=\"handleBlur($event)\"\n          (ngModelChange)=\"handleChange($event)\"\n          #textarea\n></textarea>",
            providers: [
                provideAs(TextareaComponent_1, NG_VALUE_ACCESSOR, true),
                provideAs(TextareaComponent_1, NG_VALIDATORS, true),
                provideAsWidgetRef(TextareaComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], TextareaComponent);
    return TextareaComponent;
}(BaseInput));

var components = [
    ButtonComponent,
    ButtonGroupDirective,
    CheckboxComponent,
    CheckboxsetComponent,
    CompositeDirective,
    NumberComponent,
    RadiosetComponent,
    SelectComponent,
    SwitchComponent,
    InputCalendarComponent,
    InputColorComponent,
    InputEmailComponent,
    InputNumberComponent,
    InputTextComponent,
    TextareaComponent
];
var InputModule = /** @class */ (function () {
    function InputModule() {
    }
    InputModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                FormsModule,
                WmComponentsModule,
                IMaskModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], InputModule);
    return InputModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { BaseFormComponent, BaseFormCustomComponent, ButtonComponent, DatasetAwareFormComponent, InputModule, NumberLocale, registerProps as ɵa, ButtonGroupDirective as ɵb, InputTextComponent as ɵba, registerProps$d as ɵbb, TextareaComponent as ɵbc, registerProps$e as ɵbd, registerProps$1 as ɵc, CheckboxComponent as ɵd, registerProps$2 as ɵe, CheckboxsetComponent as ɵf, registerProps$3 as ɵg, CompositeDirective as ɵh, registerProps$4 as ɵi, NumberComponent as ɵj, BaseInput as ɵk, registerProps$5 as ɵl, RadiosetComponent as ɵm, registerProps$6 as ɵn, SelectComponent as ɵo, registerProps$7 as ɵp, SwitchComponent as ɵq, registerProps$8 as ɵr, InputCalendarComponent as ɵs, registerProps$9 as ɵt, InputColorComponent as ɵu, registerProps$a as ɵv, InputEmailComponent as ɵw, registerProps$b as ɵx, InputNumberComponent as ɵy, registerProps$c as ɵz };
//# sourceMappingURL=index.js.map
