import { __decorate, __param, __metadata } from 'tslib';
import { Injector, Attribute, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_NUMBER, PROP_BOOLEAN, registerFormWidget, StylableComponent, styler, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { Subject } from 'rxjs';
import { FormWidgetType, isImageFile, isAudioFile, isVideoFile, DataSource, App } from '@wm/core';

const registerProps = () => {
    register('wm-fileupload', new Map([
        ['caption', PROP_STRING],
        ['name', PROP_STRING],
        ['tabindex', PROP_NUMBER],
        ['multiple', PROP_BOOLEAN],
        ['fileuploadmessage', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['disabled', PROP_BOOLEAN],
        ['contenttype', PROP_STRING],
        ['maxfilesize', PROP_NUMBER],
        ['iconclass', Object.assign({ value: 'wi wi-file-upload' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['uploadpath', Object.assign({}, PROP_STRING)],
        ['datasource', PROP_STRING],
        ['selectedFiles', PROP_STRING],
        ['destination', PROP_STRING],
        ['filelistheight', PROP_NUMBER],
    ]));
    registerFormWidget(FormWidgetType.UPLOAD, new Map([
        ['disabled', PROP_BOOLEAN],
        ['extensions', PROP_STRING],
        ['filetype', PROP_STRING],
        ['multiple', PROP_BOOLEAN],
        ['readonly', PROP_BOOLEAN],
        ['required', PROP_BOOLEAN]
    ]));
};

var FileUploadComponent_1;
const DEFAULT_CLS = 'app-fileupload';
const WIDGET_CONFIG = {
    widgetType: 'wm-fileupload',
    hostClass: DEFAULT_CLS
};
let FileUploadComponent = FileUploadComponent_1 = class FileUploadComponent extends StylableComponent {
    constructor(inj, app, onSelectEvt) {
        super(inj, WIDGET_CONFIG);
        this.app = app;
        this.onSelectEvt = onSelectEvt;
        this.selectedFiles = [];
        this.selectedFolders = [];
        this.fileTransfers = {};
        this.caption = 'Upload';
        this.formName = '';
        this.DEFAULT_CAPTIONS = {
            MULTIPLE_SELECT: 'Drop your files here.',
            SELECT: 'Select'
        };
        this.DEVICE_CONTENTTYPES = {
            IMAGE: 'image',
            VIDEO: 'video',
            AUDIO: 'audio',
            FILES: 'files'
        };
        this.FILESIZE_MB = 1048576;
        // parentPrefabScope = element.closest('.app-prefab').isolateScope(),
        this.CONSTANT_FILE_SERVICE = 'FileService';
        this.uploadData = {
            file: undefined,
            uploadPath: undefined
        };
        this.chooseFilter = '';
        this.fileUploadMessage = 'Drop your files here or click here to browse';
        this.uploadedFiles = {
            fileName: '',
            path: '',
            length: '',
            status: ''
        };
        this.uploadUrl = 'services';
        // styler(this.nativeElement, this);
    }
    /*_hasOnSuccessEvt = WM.isDefined(attrs.onSuccess);
     _hasOnErrorEvt = WM.isDefined(attrs.onError);*/
    // Checking if the selected file is valid for the choosen filter type
    isValidFile(filename, contenttype, extensionName, isMobileType) {
        let isValid, contentTypes;
        if (!contenttype) {
            return true;
        }
        contentTypes = _.toLower(contenttype).split(',');
        if (_.includes(contentTypes, 'image/*') || (_.includes(contentTypes, 'image') && isMobileType)) {
            isValid = isImageFile(filename);
            // If one of the content type chosen is image and user uploads image it is valid file
            if (isValid) {
                return isValid;
            }
        }
        if (_.includes(contentTypes, 'audio/*') || (_.includes(contentTypes, 'audio') && isMobileType)) {
            isValid = isAudioFile(filename);
            // If one of the content type chosen is audio/* and user uploads audio it is valid file
            if (isValid) {
                return isValid;
            }
        }
        if (_.includes(contentTypes, 'video/*') || (_.includes(contentTypes, 'video') && isMobileType)) {
            isValid = isVideoFile(filename);
            // If one of the content type chosen is video/* and user uploads video it is valid file
            if (isValid) {
                return isValid;
            }
        }
        /*content type and the uploaded file extension should be same*/
        if (_.includes(contentTypes, '.' + _.toLower(extensionName))) {
            isValid = true;
        }
        return isValid;
    }
    /* this return the array of files which are having the file size not more than maxfilesize and filters based on contenttype */
    getValidFiles($files) {
        const validFiles = [];
        const errorFiles = [];
        const MAXFILEUPLOAD_SIZE = parseFloat(this.maxfilesize) * this.FILESIZE_MB || this.FILESIZE_MB;
        const MAX_FILE_UPLOAD_FORMATTED_SIZE = (this.maxfilesize || '1') + 'MB';
        // if contenttype is files for mobile projects.
        if (this.chooseFilter === this.DEVICE_CONTENTTYPES.FILES) {
            this.chooseFilter = '';
        }
        _.forEach($files, (file) => {
            /* check for the file content type before uploading */
            if (!this.isValidFile(file.name, this.chooseFilter, this.getFileExtension(file.name), this._isMobileType)) {
                const msg = `${this.appLocale.LABEL_FILE_EXTENTION_VALIDATION_MESSAGE} ${this.chooseFilter}`;
                this.handleErrorFiles('INVALID_FILE_EXTENSION', msg, file, errorFiles);
                return;
            }
            if (file.size > MAXFILEUPLOAD_SIZE) {
                const msg = `${this.appLocale.LABEL_FILE_EXCEED_VALIDATION_MESSAGE} ${MAX_FILE_UPLOAD_FORMATTED_SIZE}`;
                this.handleErrorFiles('INVALID_FILE_SIZE', msg, file, errorFiles);
                return;
            }
            validFiles.push(file);
        });
        return {
            validFiles: validFiles,
            errorFiles: errorFiles
        };
    }
    handleErrorFiles(key, msg, file, errorFiles) {
        // Check whether the error callback exist or not. If it exists then dont show taoster message
        if (!this.hasEventCallback('error')) {
            this.app.notifyApp(msg, 'Error');
        }
        const error = {
            key: key,
            message: msg
        };
        file.error = error;
        errorFiles.push(file);
    }
    /*Overwrite the caption only if they are default*/
    getCaption(caption, isMultiple, isMobileType) {
        if (_.includes(this.DEFAULT_CAPTIONS, caption)) {
            return isMultiple && !isMobileType ? this.DEFAULT_CAPTIONS.MULTIPLE_SELECT : this.DEFAULT_CAPTIONS.SELECT;
        }
        return caption;
    }
    /* change server path based on user option */
    changeServerUploadPath(path) {
        this.selectedUploadTypePath = path;
    }
    /* this function returns the fileextension */
    getFileExtension(fileName) {
        if (fileName && _.includes(fileName, '.')) {
            return fileName.substring(fileName.lastIndexOf('.') + 1);
        }
        return 'file';
    }
    /**
     * Calls select Event
     * @param $event
     * @param $files
     */
    onSelectEventCall($event, $files) {
        this.selectedFiles = $files;
        setTimeout(() => {
            this.invokeEventCallback('select', {
                $event: $.extend($event.$files || {}, $files),
                selectedFiles: $files
            });
        });
    }
    onFileElemClick() {
        //The file upload widget value should be set to null to reupload the same file.
        this.$element.find('.file-input')[0].value = null;
    }
    /*this function to append upload status dom elements to widget */
    onFileSelect($event, $files) {
        let beforeSelectVal;
        const files = this.getValidFiles($files);
        $files = files.validFiles;
        // If the user has previously tried uploading folders using drop, and then uploading using click method we clear the folders dom
        if ($event.type === "change") {
            this.selectedFolders = [];
        }
        // Trigger error callback event if any invalid file found.
        if (!_.isEmpty(files.errorFiles)) {
            this.invokeEventCallback('error', {
                $event,
                files: files.errorFiles
            });
        }
        // Make call if there are valid files else no call is made
        if ($files.length) {
            this.progressObservable = new Subject();
            // EVENT: ON_BEFORE_SELECT
            beforeSelectVal = this.invokeEventCallback('beforeselect', {
                $event: $.extend($event.$files || {}, $files),
                files: $files
            });
            if (this.datasource) {
                this.datasource._progressObservable = this.progressObservable;
                this.datasource._progressObservable.asObservable().subscribe((progressObj) => {
                    _.forEach(this.selectedFiles, (file) => {
                        if (file.name === progressObj.fileName) {
                            file.progress = progressObj.progress;
                            if (file.progress === 100) {
                                file.status = 'success';
                            }
                            else {
                                file.status = progressObj.status;
                                if (progressObj.errMsg) {
                                    file.errMsg = progressObj.errMsg;
                                }
                            }
                        }
                    });
                });
            }
            else {
                this.selectedFiles = $files;
            }
            if (beforeSelectVal !== false) {
                // EVENT: ON_SELECT
                this.onSelectEventCall($event, $files);
            }
        }
    }
    // Prevent default behavior (Prevent file from being opened)
    dragOverHandler($event) {
        $event.preventDefault();
        $event.stopPropagation();
    }
    // Get the file data if it exists and call the onFileSelect function
    onFileDrop($event) {
        $event.preventDefault();
        let listOfFiles = [];
        this.selectedFolders = [];
        const filesData = $event.dataTransfer.items;
        if (filesData.length > 0) {
            // Use DataTransferItemList interface to access the file(s)
            for (var i = 0; i < filesData.length; i++) {
                // Get all the files and push them into an array
                if (filesData[i].webkitGetAsEntry().isFile) {
                    listOfFiles.push(filesData[i].getAsFile());
                }
                // If a selected item is directory push the folder element into an array and display error message by adding to dom
                else if (filesData[i].webkitGetAsEntry().isDirectory) {
                    this.selectedFolders.push(filesData[i].getAsFile());
                }
            }
            this.onFileSelect($event, listOfFiles);
        }
    }
    /**
     * Aborts a file upload request
     * @param $file, the file for which the request is to be aborted
     */
    abortFileUpload($file) {
        this.datasource.execute(DataSource.Operation.CANCEL, $file);
    }
    /* Define the property change handler. This function will be triggered when there is a change in the widget property */
    onPropertyChange(key, nv, ov) {
        /*Monitoring changes for styles or properties and accordingly handling respective changes.*/
        switch (key) {
            case 'uploadpath':
                // TODO Srinivas: why do we need uploadpath
                this.changeServerUploadPath(nv);
                break;
            case 'contenttype':
                this.chooseFilter = nv.split(' ').join(',');
                break;
            case 'multiple':
                this.formName = this.name + (this.multiple ? '-multiple-fileupload' : '-single-fileupload');
                this.caption = this.getCaption(this.caption, this.multiple, this._isMobileType);
                break;
            case 'fileuploadmessage':
                this.fileUploadMessage = nv;
        }
        super.onPropertyChange(key, nv, ov);
    }
    ngOnInit() {
        super.ngOnInit();
    }
    ngAfterViewInit() {
        styler(this.nativeElement.querySelector('.app-button, .drop-box'), this);
    }
};
FileUploadComponent.initializeProps = registerProps();
FileUploadComponent.ctorParameters = () => [
    { type: Injector },
    { type: App },
    { type: undefined, decorators: [{ type: Attribute, args: ['select.event',] }] }
];
FileUploadComponent = FileUploadComponent_1 = __decorate([
    Component({
        selector: '[wmFileUpload]',
        template: "<div class=\"app-fileupload\" init-widget>\n    <!-- drag and drop files UI in web -->\n    <div class=\"app-multi-file-upload\" *ngIf=\"!_isMobileType && multiple\">\n        <div class=\"drop-box\" (dragover)=\"dragOverHandler($event)\" (drop)=\"onFileDrop($event)\" >\n            <i class=\"{{iconclass}}\"></i>\n            <div class=\"message\">\n                <label [innerHtml]=\"caption\" class=\"caption\"></label>\n                <form class=\"form-horizontal\" name=\"{{formName}}\">\n                    <input class=\"file-input\" aria-label=\"Select File\" type=\"file\" name=\"files\" *ngIf=\"multiple\" [disabled]=\"disabled\" [accept]=\"chooseFilter\" (click)=\"onFileElemClick()\" (change)=\"onFileSelect($event, $event.target.files)\" multiple >\n                    <a href=\"javascript:void(0);\" class=\"app-anchor\">{{fileUploadMessage}}</a>\n                </form>\n            </div>\n        </div>\n    </div>\n    <!-- single file upload in web and single , multiple file upload UI in mobile runmode -->\n    <div class=\"app-single-file-upload\" *ngIf=\"!_isCordova && (!multiple || _isMobileType)\">\n        <div class=\"app-button-wrapper\">\n            <form class=\"form-horizontal\" name=\"{{formName}}\">\n                <!-- support for file upload in Mobileapp in its runmode (Web) -->\n                <input class=\"file-input\" aria-label=\"Select files\" type=\"file\" name=\"files\" *ngIf=\"multiple\" [disabled]=\"disabled\" [accept]=\"chooseFilter\" (click)=\"onFileElemClick()\" (change)=\"onFileSelect($event, $event.target.files)\" multiple>\n                <input class=\"file-input\" aria-label=\"Select Files\" type=\"file\" name=\"files\" *ngIf=\"!multiple\" [disabled]=\"disabled\" [accept]=\"chooseFilter\" (click)=\"onFileElemClick()\" (change)=\"onFileSelect($event, $event.target.files)\" >\n                <button focus-target class=\"app-button btn btn-default\">\n                    <i class=\"{{iconclass}}\" aria-hidden=\"true\"></i>\n                    <span class=\"caption\">{{caption}}</span>\n                </button>\n            </form>\n        </div>\n        <div class=\"app-files-upload-status single\"></div>\n    </div>\n    <!-- support for file upload in Mobile Application (device) -->\n    <button *ngIf=\"_isCordova\" focus-target class=\"app-button btn btn-default\" (click)=\"openFileSelector()\" [disabled]=\"disabled\">\n        <i class=\"{{iconclass}}\"></i>\n        <span class=\"caption\">{{caption}}</span>\n    </button>\n    <!-- list of selectedfiles UI -->\n    <ul class=\"list-group file-upload\" [ngStyle]=\"{height: filelistheight, overflow: overflow}\" *ngIf=\"selectedFiles.length > 0\" >\n        <div *ngFor=\"let ft of selectedFiles\">\n            <li class=\"list-group-item file-upload-status\" *ngIf=\"ft.status !== 'abort'\">\n                <div class=\"media upload-file-list\">\n                    <div class=\"media-left media-middle file-icon {{getFileExtension(ft.name) | fileIconClass}}\" title=\"{{getFileExtension(ft.name)}}\"></div>\n                    <div class=\"media-body media-middle file-details\">\n                        <p class=\"uploaddetails\">\n                            <label class=\"upload-title\">{{ft.name}}</label><br/>\n                            <span class=\"filesize\" *ngIf=\"ft.fileLength  !== 0\">{{ft.size | filesize:0}}</span>\n                        </p>\n                        <div class=\"progress\" *ngIf=\"ft.status === 'onProgress'\">\n                            <div class=\"progress-bar progress-bar-striped progress-bar-info\" [ngStyle]=\"{width: (ft.progress +'%')}\"></div>\n                        </div>\n                    </div>\n                    <div class=\"media-right media-middle\" *ngIf=\"ft.status === 'onProgress' || ft.status === 'queued'\">\n                        <a class=\"btn btn-transparent file-upload-stop\" type=\"button\" (click)=\"abortFileUpload(ft)\">\n                            <i class=\"wi wi-close\"></i>\n                        </a>\n                    </div>\n                    <div class=\"media-right media-middle\"  *ngIf=\"ft.status === 'success'\">\n                        <span class=\"status-icon {{ft.status | stateClass }}\"></span>\n                    </div>\n                    <div class=\"media-right media-middle\"  *ngIf=\"ft.status === 'error'\" title=\"{{ft.errMsg || this.appLocale.MESSAGE_FILE_UPLOAD_FAILED}}\">\n                        <span class=\"status-icon {{ft.status | stateClass }}\"></span>\n                    </div>\n                </div>\n            </li>\n        </div>\n    </ul>\n    <!-- list of selectedfolders UI to show the error in title -->\n    <ul class=\"list-group file-upload\" [ngStyle]=\"{height: filelistheight, overflow: overflow}\" title=\"{{this.appLocale.MESSAGE_FOLDER_UPLOADS_NOT_ALLOWED}}\" *ngIf=\"selectedFolders.length > 0\" >\n        <li *ngFor=\"let ft of selectedFolders\" class=\"list-group-item file-upload-status\">\n            <div class=\"media upload-file-list\">\n                <div class=\"media-left media-middle file-icon fa fa-folder-o\"></div>\n                <div class=\"media-body media-middle file-details\">\n                    <p class=\"uploaddetails\">\n                        <label class=\"upload-title\">{{ft.name}}</label><br/>\n                    </p>\n                </div>\n                <div class=\"media-right media-middle\" >\n                    <span class=\"status-icon  wi wi-cancel text-danger\"></span>\n                </div>\n            </div>\n        </li>\n    </ul>\n</div>\n",
        providers: [
            provideAsWidgetRef(FileUploadComponent_1)
        ]
    }),
    __param(2, Attribute('select.event')),
    __metadata("design:paramtypes", [Injector, App, Object])
], FileUploadComponent);

const components = [
    FileUploadComponent
];
let FileUploadModule = class FileUploadModule {
};
FileUploadModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], FileUploadModule);

/**
 * Generated bundle index. Do not edit.
 */

export { FileUploadComponent, FileUploadModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
