import { __decorate, __metadata } from 'tslib';
import { Injector, ViewChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NgModel, NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, register, registerFormWidget, styler, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { BaseFormCustomComponent } from '@wm/components/input';
import { FormWidgetType } from '@wm/core';

const sliderProps = new Map([
    ['class', PROP_STRING],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['maxvalue', PROP_NUMBER],
    ['minvalue', PROP_NUMBER],
    ['name', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['step', PROP_NUMBER],
    ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)]
]);
const registerProps = () => {
    register('wm-slider', sliderProps);
    registerFormWidget(FormWidgetType.SLIDER, new Map(sliderProps));
};

var SliderComponent_1;
const DEFAULT_CLS = 'app-slider slider';
const WIDGET_CONFIG = {
    widgetType: 'wm-slider',
    hostClass: DEFAULT_CLS
};
let SliderComponent = SliderComponent_1 = class SliderComponent extends BaseFormCustomComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG);
        styler(this.nativeElement, this);
    }
    // change and blur events are added from the template
    handleEvent(node, eventName, callback, locals) {
        if (eventName !== 'change' && eventName !== 'blur') {
            super.handleEvent(node, eventName, callback, locals);
        }
    }
    handleChange(newVal) {
        this.invokeOnChange(this.datavalue, { type: 'change' }, this.ngModel.valid);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        super.onPropertyChange(key, nv, ov);
    }
};
SliderComponent.initializeProps = registerProps();
SliderComponent.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ViewChild(NgModel),
    __metadata("design:type", NgModel)
], SliderComponent.prototype, "ngModel", void 0);
SliderComponent = SliderComponent_1 = __decorate([
    Component({
        selector: '[wmSlider]',
        template: "<span class=\"app-slider-value pull-left\" [attr.aria-valuemin]=\"minvalue\" [textContent]=\"minvalue\"></span>\n<span class=\"app-slider-value pull-right\" [attr.aria-valuemax]=\"maxvalue\" [textContent]=\"maxvalue\"></span>\n<input class=\"range-input\" aria-label=\"slider range\" aria-orientation=\"horizontal\"\n       focus-target\n       type=\"range\" [title]=\"datavalue\"\n       [attr.min]=\"minvalue\"\n       [attr.max]=\"maxvalue\"\n       [attr.step]=\"step\"\n       [(ngModel)]=\"datavalue\"\n       [disabled]=\"disabled\"\n       [attr.accesskey]=\"shortcutkey\"\n       [tabindex]=\"tabindex\"\n       (blur)=\"invokeOnTouched($event)\"\n       (ngModelChange)=\"handleChange($event)\"\n       [attr.name]=\"name\"\n       [tabindex]=\"tabindex\"/>\n<div *ngIf=\"readonly || disabled\" aria-readonly=\"true\" class=\"readonly-wrapper\"></div>",
        providers: [
            provideAs(SliderComponent_1, NG_VALUE_ACCESSOR, true),
            provideAsWidgetRef(SliderComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], SliderComponent);

const components = [
    SliderComponent
];
let SliderModule = class SliderModule {
};
SliderModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            FormsModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], SliderModule);

/**
 * Generated bundle index. Do not edit.
 */

export { SliderModule, SliderComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
