import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ViewChild, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NgModel, NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { PROP_STRING, PROP_BOOLEAN, PROP_NUMBER, register, registerFormWidget, styler, provideAs, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { BaseFormCustomComponent } from '@wm/components/input';
import { FormWidgetType } from '@wm/core';

var sliderProps = new Map([
    ['class', PROP_STRING],
    ['datavalue', PROP_STRING],
    ['disabled', PROP_BOOLEAN],
    ['hint', PROP_STRING],
    ['maxvalue', PROP_NUMBER],
    ['minvalue', PROP_NUMBER],
    ['name', PROP_STRING],
    ['readonly', PROP_BOOLEAN],
    ['shortcutkey', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ['step', PROP_NUMBER],
    ['tabindex', __assign({ value: 0 }, PROP_NUMBER)]
]);
var registerProps = function () {
    register('wm-slider', sliderProps);
    registerFormWidget(FormWidgetType.SLIDER, new Map(sliderProps));
};

var DEFAULT_CLS = 'app-slider slider';
var WIDGET_CONFIG = {
    widgetType: 'wm-slider',
    hostClass: DEFAULT_CLS
};
var SliderComponent = /** @class */ (function (_super) {
    __extends(SliderComponent, _super);
    function SliderComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        styler(_this.nativeElement, _this);
        return _this;
    }
    SliderComponent_1 = SliderComponent;
    // change and blur events are added from the template
    SliderComponent.prototype.handleEvent = function (node, eventName, callback, locals) {
        if (eventName !== 'change' && eventName !== 'blur') {
            _super.prototype.handleEvent.call(this, node, eventName, callback, locals);
        }
    };
    SliderComponent.prototype.handleChange = function (newVal) {
        this.invokeOnChange(this.datavalue, { type: 'change' }, this.ngModel.valid);
    };
    SliderComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    var SliderComponent_1;
    SliderComponent.initializeProps = registerProps();
    SliderComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ViewChild(NgModel),
        __metadata("design:type", NgModel)
    ], SliderComponent.prototype, "ngModel", void 0);
    SliderComponent = SliderComponent_1 = __decorate([
        Component({
            selector: '[wmSlider]',
            template: "<span class=\"app-slider-value pull-left\" [attr.aria-valuemin]=\"minvalue\" [textContent]=\"minvalue\"></span>\n<span class=\"app-slider-value pull-right\" [attr.aria-valuemax]=\"maxvalue\" [textContent]=\"maxvalue\"></span>\n<input class=\"range-input\" aria-label=\"slider range\" aria-orientation=\"horizontal\"\n       focus-target\n       type=\"range\" [title]=\"datavalue\"\n       [attr.min]=\"minvalue\"\n       [attr.max]=\"maxvalue\"\n       [attr.step]=\"step\"\n       [(ngModel)]=\"datavalue\"\n       [disabled]=\"disabled\"\n       [attr.accesskey]=\"shortcutkey\"\n       [tabindex]=\"tabindex\"\n       (blur)=\"invokeOnTouched($event)\"\n       (ngModelChange)=\"handleChange($event)\"\n       [attr.name]=\"name\"\n       [tabindex]=\"tabindex\"/>\n<div *ngIf=\"readonly || disabled\" aria-readonly=\"true\" class=\"readonly-wrapper\"></div>",
            providers: [
                provideAs(SliderComponent_1, NG_VALUE_ACCESSOR, true),
                provideAsWidgetRef(SliderComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], SliderComponent);
    return SliderComponent;
}(BaseFormCustomComponent));

var components = [
    SliderComponent
];
var SliderModule = /** @class */ (function () {
    function SliderModule() {
    }
    SliderModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                FormsModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], SliderModule);
    return SliderModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { SliderModule, SliderComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
