import { __decorate, __param, __metadata } from 'tslib';
import { Injector, Attribute, Component, NgModule } from '@angular/core';
import { Location, CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, DatasetAwareNavComponent, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { MenuModule } from '@wm/components/navigation/menu';
import { Router } from '@angular/router';
import { getUrlParams, getRouteNameFromLink, openLink } from '@wm/core';

const registerProps = () => {
    register('wm-breadcrumb', new Map([
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['itemlabel', PROP_STRING],
        ['itemid', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemclass', PROP_STRING],
        ['itemtarget', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var BreadcrumbComponent_1;
const DEFAULT_CLS = 'breadcrumb app-breadcrumb';
const WIDGET_CONFIG = { widgetType: 'wm-breadcrumb', hostClass: DEFAULT_CLS };
let BreadcrumbComponent = BreadcrumbComponent_1 = class BreadcrumbComponent extends DatasetAwareNavComponent {
    constructor(inj, route, location, beforeNavigateCB) {
        super(inj, WIDGET_CONFIG);
        this.route = route;
        this.location = location;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
        this.disableMenuContext = !!beforeNavigateCB;
    }
    /**
     * Gets the first path found based on the key provided inside info Object.
     * @param info - Info object which has properties key(Active Page Name) and isPathFound[boolean] is set true if path found.
     * @param children - a child Object form children Array.
     * @param path - final path.
     * @returns {*|Array}: returns array of objects which represents the final path.
     */
    getPath(info, children, path = []) {
        children.forEach((child) => {
            // return if path already found.
            if (info.isPathFound) {
                return path;
            }
            // if key is matched set path found to true and return the path.
            if (child.id === info.key) {
                info.isPathFound = true;
                // only push the child object by omiting the children within it.
                path.push(child);
                return path;
            }
            // if the node has children make a recursive call.
            if (child.children.length) {
                path.push(child);
                this.getPath(info, child.children, path);
                // if path is not found in that node pop the node.
                if (!info.isPathFound) {
                    path.pop();
                }
            }
        });
        // return the path.
        return path;
    }
    getCurrentRoute() {
        return this.location.path().substr(1).split('?')[0];
    }
    // over rides resetNode function, generating path for the breadcrumb.
    resetNodes() {
        super.resetNodes();
        // get path only if the widget have id property.
        if (this.itemid || this.binditemid) {
            this.nodes = this.getPath({ key: this.getCurrentRoute(), isPathFound: false }, this.nodes);
        }
    }
    onItemClick($event, $item) {
        $event.preventDefault();
        const locals = { $item: $item.value, $event };
        const canNavigate = !(this.invokeEventCallback('beforenavigate', locals) === false);
        const linkTarget = $item.target;
        let itemLink = $item.link;
        if (itemLink && canNavigate) {
            if (itemLink.startsWith('#/') && (!linkTarget || linkTarget === '_self')) {
                const queryParams = getUrlParams(itemLink);
                itemLink = getRouteNameFromLink(itemLink);
                this.route.navigate([itemLink], { queryParams });
            }
            else {
                openLink(itemLink, linkTarget);
            }
        }
    }
};
BreadcrumbComponent.initializeProps = registerProps();
BreadcrumbComponent.ctorParameters = () => [
    { type: Injector },
    { type: Router },
    { type: Location },
    { type: String, decorators: [{ type: Attribute, args: ['beforenavigate.event',] }] }
];
BreadcrumbComponent = BreadcrumbComponent_1 = __decorate([
    Component({
        selector: '[wmBreadcrumb]',
        template: "<li *ngFor=\"let item of nodes; let last = last; let index = index\" [ngClass]=\"{active: last}\" class=\"{{item.class}}\">\n    <i class=\"{{item.icon}}\"></i>\n    <a [title]=\"item.label\" href=\"javascript:void(0)\" [wmNavigationControl]=\"item.link\" [disableMenuContext]=\"disableMenuContext || !!item.action\" (click)=\"onItemClick($event, item)\" *ngIf=\"!last\" [textContent]=\"item.label\"></a>\n    <label *ngIf=\"last\" [textContent]=\"item.label\"></label>\n</li>\n",
        providers: [
            provideAsWidgetRef(BreadcrumbComponent_1)
        ]
    }),
    __param(3, Attribute('beforenavigate.event')),
    __metadata("design:paramtypes", [Injector,
        Router,
        Location, String])
], BreadcrumbComponent);

const components = [
    BreadcrumbComponent
];
let BreadcrumbModule = class BreadcrumbModule {
};
BreadcrumbModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            MenuModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], BreadcrumbModule);

/**
 * Generated bundle index. Do not edit.
 */

export { BreadcrumbModule, BreadcrumbComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
