import { __assign, __extends, __decorate, __param, __metadata, __spread } from 'tslib';
import { Injector, Attribute, Component, NgModule } from '@angular/core';
import { Location, CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, DatasetAwareNavComponent, WmComponentsModule } from '@wm/components/base';
import { MenuModule } from '@wm/components/navigation/menu';
import { Router } from '@angular/router';
import { getUrlParams, getRouteNameFromLink, openLink } from '@wm/core';

var registerProps = function () {
    register('wm-breadcrumb', new Map([
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['itemlabel', PROP_STRING],
        ['itemid', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemclass', PROP_STRING],
        ['itemtarget', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS = 'breadcrumb app-breadcrumb';
var WIDGET_CONFIG = { widgetType: 'wm-breadcrumb', hostClass: DEFAULT_CLS };
var BreadcrumbComponent = /** @class */ (function (_super) {
    __extends(BreadcrumbComponent, _super);
    function BreadcrumbComponent(inj, route, location, beforeNavigateCB) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.route = route;
        _this.location = location;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        _this.disableMenuContext = !!beforeNavigateCB;
        return _this;
    }
    BreadcrumbComponent_1 = BreadcrumbComponent;
    /**
     * Gets the first path found based on the key provided inside info Object.
     * @param info - Info object which has properties key(Active Page Name) and isPathFound[boolean] is set true if path found.
     * @param children - a child Object form children Array.
     * @param path - final path.
     * @returns {*|Array}: returns array of objects which represents the final path.
     */
    BreadcrumbComponent.prototype.getPath = function (info, children, path) {
        var _this = this;
        if (path === void 0) { path = []; }
        children.forEach(function (child) {
            // return if path already found.
            if (info.isPathFound) {
                return path;
            }
            // if key is matched set path found to true and return the path.
            if (child.id === info.key) {
                info.isPathFound = true;
                // only push the child object by omiting the children within it.
                path.push(child);
                return path;
            }
            // if the node has children make a recursive call.
            if (child.children.length) {
                path.push(child);
                _this.getPath(info, child.children, path);
                // if path is not found in that node pop the node.
                if (!info.isPathFound) {
                    path.pop();
                }
            }
        });
        // return the path.
        return path;
    };
    BreadcrumbComponent.prototype.getCurrentRoute = function () {
        return this.location.path().substr(1).split('?')[0];
    };
    // over rides resetNode function, generating path for the breadcrumb.
    BreadcrumbComponent.prototype.resetNodes = function () {
        _super.prototype.resetNodes.call(this);
        // get path only if the widget have id property.
        if (this.itemid || this.binditemid) {
            this.nodes = this.getPath({ key: this.getCurrentRoute(), isPathFound: false }, this.nodes);
        }
    };
    BreadcrumbComponent.prototype.onItemClick = function ($event, $item) {
        $event.preventDefault();
        var locals = { $item: $item.value, $event: $event };
        var canNavigate = !(this.invokeEventCallback('beforenavigate', locals) === false);
        var linkTarget = $item.target;
        var itemLink = $item.link;
        if (itemLink && canNavigate) {
            if (itemLink.startsWith('#/') && (!linkTarget || linkTarget === '_self')) {
                var queryParams = getUrlParams(itemLink);
                itemLink = getRouteNameFromLink(itemLink);
                this.route.navigate([itemLink], { queryParams: queryParams });
            }
            else {
                openLink(itemLink, linkTarget);
            }
        }
    };
    var BreadcrumbComponent_1;
    BreadcrumbComponent.initializeProps = registerProps();
    BreadcrumbComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: Router },
        { type: Location },
        { type: String, decorators: [{ type: Attribute, args: ['beforenavigate.event',] }] }
    ]; };
    BreadcrumbComponent = BreadcrumbComponent_1 = __decorate([
        Component({
            selector: '[wmBreadcrumb]',
            template: "<li *ngFor=\"let item of nodes; let last = last; let index = index\" [ngClass]=\"{active: last}\" class=\"{{item.class}}\">\n    <i class=\"{{item.icon}}\"></i>\n    <a [title]=\"item.label\" href=\"javascript:void(0)\" [wmNavigationControl]=\"item.link\" [disableMenuContext]=\"disableMenuContext || !!item.action\" (click)=\"onItemClick($event, item)\" *ngIf=\"!last\" [textContent]=\"item.label\"></a>\n    <label *ngIf=\"last\" [textContent]=\"item.label\"></label>\n</li>\n",
            providers: [
                provideAsWidgetRef(BreadcrumbComponent_1)
            ]
        }),
        __param(3, Attribute('beforenavigate.event')),
        __metadata("design:paramtypes", [Injector,
            Router,
            Location, String])
    ], BreadcrumbComponent);
    return BreadcrumbComponent;
}(DatasetAwareNavComponent));

var components = [
    BreadcrumbComponent
];
var BreadcrumbModule = /** @class */ (function () {
    function BreadcrumbModule() {
    }
    BreadcrumbModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                MenuModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], BreadcrumbModule);
    return BreadcrumbModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { BreadcrumbModule, BreadcrumbComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
