import { __decorate, __param, __metadata } from 'tslib';
import { Injector, ChangeDetectorRef, Attribute, Component, Self, Optional, HostListener, ElementRef, Input, ContentChild, Directive, NgModule, ViewChildren, QueryList } from '@angular/core';
import { CommonModule } from '@angular/common';
import { BsDropdownDirective, BsDropdownModule } from 'ngx-bootstrap/dropdown';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, DatasetAwareNavComponent, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, PROP_NUMBER, isActiveNavItem, AUTOCLOSE_TYPE, StylableComponent, WmComponentsModule, BaseContainerComponent } from '@wm/components/base';
import { AnchorComponent, BasicModule } from '@wm/components/basic';
import { InputModule } from '@wm/components/input';
import { Router } from '@angular/router';
import { triggerItemAction, addClass, removeClass, UserDefinedExecutionContext, App, $appDigest, setAttr } from '@wm/core';

const registerProps = () => {
    register('wm-nav', new Map([
        ['autoclose', Object.assign({ value: 'always' }, PROP_STRING)],
        ['autoopen', Object.assign({ value: 'never' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['iconposition', Object.assign({ value: 'left' }, PROP_STRING)],
        ['isactive', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemclass', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemaction', PROP_STRING],
        ['itembadge', PROP_STRING],
        ['itemtarget', PROP_STRING],
        ['layout', PROP_STRING],
        ['name', PROP_STRING],
        ['orderby', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['type', PROP_STRING],
        ['userrole', PROP_STRING]
    ]));
};

var NavComponent_1;
const DEFAULT_CLS = 'nav app-nav';
const WIDGET_CONFIG = { widgetType: 'wm-nav', hostClass: DEFAULT_CLS };
const NavClassMap = {
    pills: 'nav-pills',
    tabs: 'nav-tabs',
    navbar: 'navbar-nav'
};
let NavComponent = NavComponent_1 = class NavComponent extends DatasetAwareNavComponent {
    constructor(inj, cdRef, route, userDefinedExecutionContext, app, selectEventCB) {
        super(inj, WIDGET_CONFIG);
        this.cdRef = cdRef;
        this.route = route;
        this.userDefinedExecutionContext = userDefinedExecutionContext;
        this.app = app;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
        this.disableMenuContext = !!selectEventCB;
        this.pageScope = this.viewParent;
        // For selecting the item on load
        const datasetSubscription = this.nodes$.subscribe(() => {
            if (!_.isEmpty(this.nodes)) {
                let itemFound = false;
                const getItem = (nodes, isMenuWidget) => {
                    _.forEach(nodes, (item) => {
                        if (itemFound) {
                            return;
                        }
                        if (item.isactive) {
                            itemFound = true;
                            this.selecteditem = isMenuWidget ? _.omit(item, ['children', 'value']) : item.value;
                            this.invokeEventCallback('select', { $event: {}, $item: item.value });
                            // Trigger the action associated with active item
                            triggerItemAction(this, item);
                            // _selected is used to add active class for nav item. If we have children inside nav widget then it is not required.
                            if (!isMenuWidget) {
                                item._selected = true;
                            }
                            return false;
                        }
                        if (!_.isEmpty(item.children)) {
                            getItem(item.children, 'menu');
                        }
                    });
                };
                getItem(this.nodes);
            }
        });
        this.registerDestroyListener(() => datasetSubscription.unsubscribe());
    }
    get activePageName() {
        return this.app.activePageName;
    }
    setNavType(type) {
        addClass(this.nativeElement, NavClassMap[type]);
    }
    setNavLayout(layout) {
        addClass(this.nativeElement, `nav-${layout}`);
    }
    onNavSelect($event, item, liRef) {
        $event.preventDefault();
        const selectedItem = _.find(this.nodes, '_selected');
        if (selectedItem) {
            delete selectedItem._selected;
        }
        if (this.activeNavLINode) {
            removeClass(this.activeNavLINode, 'active');
        }
        this.activeNavLINode = liRef;
        addClass(liRef, 'active');
        this.selecteditem = item.value;
        this.invokeEventCallback('select', { $event, $item: item.value });
        // Trigger the action associated with active item
        triggerItemAction(this, item);
    }
    ngOnInit() {
        super.ngOnInit();
        this.setNavType(this.type);
        this.setNavLayout(this.layout);
    }
    /**
     * invoked from the menu widget when a menu item is selected.
     * @param $event
     * @param widget
     * @param $item
     */
    onMenuItemSelect($event, widget, $item) {
        this.selecteditem = _.omit($item, ['children', 'value']);
        this.invokeEventCallback('select', { $event, $item: this.selecteditem });
    }
};
NavComponent.initializeProps = registerProps();
NavComponent.ctorParameters = () => [
    { type: Injector },
    { type: ChangeDetectorRef },
    { type: Router },
    { type: UserDefinedExecutionContext },
    { type: App },
    { type: undefined, decorators: [{ type: Attribute, args: ['select.event',] }] }
];
NavComponent = NavComponent_1 = __decorate([
    Component({
        selector: '[wmNav]',
        template: "<ng-template #menuRef let-item=\"item\" let-index=\"index\">\n    <div wmMenu dropdown\n         type=\"anchor\"\n         autoclose.bind=\"autoclose\"\n         iconclass.bind=\"item.icon\"\n         autoopen.bind=\"autoopen\"\n         caption.bind=\"item.label\"\n         dataset.bind=\"item.children\"\n         iconposition.bind=\"iconposition\"\n         select.event=\"onMenuItemSelect($event, widget, $item)\"></div>\n</ng-template>\n\n<ng-template #anchorRef let-item=\"item\" let-index=\"index\" let-liRef=\"liRef\">\n    <a wmAnchor\n       [disableMenuContext]=\"disableMenuContext || !!item.action\"\n       [wmNavigationControl]=\"item.link\"\n       caption.bind=\"item.label\"\n       iconclass.bind=\"item.icon\"\n       badgevalue.bind=\"item.badge\"\n       iconposition.bind=\"iconposition\"\n       click.event=\"onNavSelect($event, item, liRef)\"></a>\n</ng-template>\n\n<li class=\"app-nav-item {{item.class}}\" *ngFor=\"let item of nodes; let index = index;\" #liRef [ngClass]=\"{active: item._selected || (!isactive && item.link === '#/' + activePageName)}\">\n    <ng-container [ngTemplateOutlet]=\"anchorRef\" [ngTemplateOutletContext]=\"{item: item, index:index, liRef: liRef}\" *ngIf=\"!item.children.length\"></ng-container>\n    <ng-container [ngTemplateOutlet]=\"menuRef\" [ngTemplateOutletContext]=\"{item: item, index:index}\"  *ngIf=\"item.children.length\"></ng-container>\n</li>\n\n<ng-content *ngIf=\"!nodes.length\" select=\"[wmNavItem]\"></ng-content>\n",
        providers: [
            provideAsWidgetRef(NavComponent_1)
        ]
    }),
    __param(5, Attribute('select.event')),
    __metadata("design:paramtypes", [Injector,
        ChangeDetectorRef,
        Router,
        UserDefinedExecutionContext,
        App, Object])
], NavComponent);

const registerProps$1 = () => {
    register('wm-menu', new Map([
        ['accessroles', Object.assign({ value: 'Everyone' }, PROP_STRING)],
        ['animateitems', PROP_STRING],
        ['autoclose', Object.assign({ value: 'always' }, PROP_STRING)],
        ['autoopen', Object.assign({ value: 'never' }, PROP_STRING)],
        ['caption', PROP_STRING],
        ['class', PROP_STRING],
        ['dataset', Object.assign({ value: 'Menu Item 1, Menu Item 2, Menu Item 3' }, PROP_ANY)],
        ['hint', Object.assign({ value: '' }, PROP_STRING)],
        ['isactive', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconposition', Object.assign({ value: 'left' }, PROP_STRING)],
        ['itemaction', PROP_STRING],
        ['itemchildren', PROP_STRING],
        ['itemclass', PROP_STRING],
        ['itemicon', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['itemtarget', PROP_STRING],
        ['linktarget', Object.assign({ value: '_self' }, PROP_STRING)],
        ['menuclass', PROP_STRING],
        ['menulayout', PROP_STRING],
        ['menuposition', PROP_STRING],
        ['name', PROP_STRING],
        ['orderby', PROP_STRING],
        ['shortcutkey', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['type', Object.assign({ value: 'menu' }, PROP_STRING)],
        ['userrole', PROP_STRING]
    ]));
};

var MenuComponent_1;
const KEYBOARD_MOVEMENTS = {
    MOVE_UP: 'UP-ARROW',
    MOVE_LEFT: 'LEFT-ARROW',
    MOVE_RIGHT: 'RIGHT-ARROW',
    MOVE_DOWN: 'DOWN-ARROW',
    ON_ENTER: 'ENTER',
    ON_TAB: 'TAB',
    ON_SHIFT_TAB: 'SHIFT-TAB',
    ON_ESCAPE: 'ESC'
};
const MENU_POSITION = {
    UP_LEFT: 'up,left',
    UP_RIGHT: 'up,right',
    DOWN_LEFT: 'down,left',
    DOWN_RIGHT: 'down,right',
    INLINE: 'inline'
};
const POSITION = {
    DOWN_RIGHT: 'down,right',
    DOWN_LEFT: 'down,left',
    UP_RIGHT: 'up,right',
    UP_LEFT: 'up,left',
    INLINE: 'inline'
};
const CARET_CLS = {
    UP: 'fa-caret-up',
    DOWN: 'fa-caret-down'
};
const PULL_CLS = {
    LEFT: 'pull-left',
    RIGHT: 'pull-right'
};
const AUTO_OPEN = {
    NEVER: 'never',
    ACTIVE_PAGE: 'activepage',
    ALWAYS: 'always'
};
const WIDGET_CONFIG$1 = { widgetType: 'wm-menu', hostClass: 'dropdown app-menu' };
let MenuComponent = MenuComponent_1 = class MenuComponent extends DatasetAwareNavComponent {
    constructor(inj, route, userDefinedExecutionContext, bsDropdown, parentNav, selectEventCB) {
        super(inj, WIDGET_CONFIG$1);
        this.route = route;
        this.userDefinedExecutionContext = userDefinedExecutionContext;
        this.bsDropdown = bsDropdown;
        this.parentNav = parentNav;
        this.selectEventCB = selectEventCB;
        this.menuCaret = 'fa-caret-down';
        this._selectFirstItem = false;
        if (parentNav) {
            this.disableMenuContext = !!parentNav.disableMenuContext;
        }
        else {
            this.disableMenuContext = !!selectEventCB;
        }
        // For selecting the item on load
        const datasetSubscription = this.nodes$.subscribe(() => {
            if (!_.isEmpty(this.nodes)) {
                // If menu widget is inside nav widget then dont check for item isactive property because these will be handled form nav widget.
                if (this.parentNav) {
                    return;
                }
                let itemFound = false;
                const getItem = (nodes) => {
                    _.forEach(nodes, (item) => {
                        if (itemFound) {
                            return;
                        }
                        if (item.isactive) {
                            itemFound = true;
                            this.onMenuItemSelect({ $event: {}, $item: item });
                            // Trigger the action associated with active item
                            triggerItemAction(this, item);
                            return false;
                        }
                        if (!_.isEmpty(item.children)) {
                            getItem(item.children);
                        }
                    });
                };
                getItem(this.nodes);
            }
        });
        this.registerDestroyListener(() => datasetSubscription.unsubscribe());
    }
    onShow() {
        if (this._selectFirstItem) {
            setTimeout(() => {
                this.$element.find('> ul[wmmenudropdown] li.app-menu-item').first().find('> a').focus();
            });
        }
        $appDigest();
    }
    onHide() {
        this.$element.find('>.dropdown-toggle').focus();
        this.$element.find('li').removeClass('open');
        this._selectFirstItem = false;
        $appDigest();
    }
    onKeyDown($event, eventAction) {
        const KEY_MOVEMENTS = _.clone(KEYBOARD_MOVEMENTS);
        if (this.menuposition === MENU_POSITION.UP_RIGHT) {
            KEY_MOVEMENTS.MOVE_UP = 'DOWN-ARROW';
            KEY_MOVEMENTS.MOVE_DOWN = 'UP-ARROW';
        }
        else if (this.menuposition === MENU_POSITION.UP_LEFT) {
            KEY_MOVEMENTS.MOVE_UP = 'DOWN-ARROW';
            KEY_MOVEMENTS.MOVE_DOWN = 'UP-ARROW';
            KEY_MOVEMENTS.MOVE_LEFT = 'RIGHT-ARROW';
            KEY_MOVEMENTS.MOVE_RIGHT = 'LEFT-ARROW';
        }
        else if (this.menuposition === MENU_POSITION.DOWN_LEFT) {
            KEY_MOVEMENTS.MOVE_LEFT = 'RIGHT-ARROW';
            KEY_MOVEMENTS.MOVE_RIGHT = 'LEFT-ARROW';
        }
        if (_.includes([KEY_MOVEMENTS.MOVE_DOWN, KEY_MOVEMENTS.MOVE_RIGHT], eventAction)) {
            if (!this.bsDropdown.isOpen) {
                this._selectFirstItem = true;
                this.bsDropdown.show();
            }
            else {
                this.$element.find('> ul[wmmenudropdown] li.app-menu-item').first().find('> a').focus();
            }
        }
        else if (eventAction === KEY_MOVEMENTS.ON_ENTER) {
            this.bsDropdown.toggle(true);
        }
        else if (_.includes([KEY_MOVEMENTS.MOVE_UP, KEY_MOVEMENTS.MOVE_LEFT], eventAction)) {
            this.bsDropdown.hide();
        }
        $event.preventDefault();
    }
    /**
     * returns true if the menu has link to the current page.
     * @param nodes
     */
    hasLinkToCurrentPage(nodes) {
        return nodes.some(node => {
            if (isActiveNavItem(node.link, this.route.url)) {
                return true;
            }
            if (node.children) {
                return this.hasLinkToCurrentPage(node.children);
            }
        });
    }
    resetNodes() {
        super.resetNodes();
        // open the menu if any of its menu items has link to current page and if autoopen value is 'active page'
        if ((this.autoopen === AUTO_OPEN.ACTIVE_PAGE && this.hasLinkToCurrentPage(this.nodes)) || this.autoopen === AUTO_OPEN.ALWAYS) {
            this.bsDropdown.show();
        }
    }
    ngOnInit() {
        super.ngOnInit();
        this.setMenuPosition();
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        if (key === 'autoclose') {
            this.bsDropdown.autoClose = nv !== AUTOCLOSE_TYPE.DISABLED;
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    setMenuPosition() {
        switch (this.menuposition) {
            case POSITION.DOWN_RIGHT:
                removeClass(this.nativeElement, 'dropup');
                this.menualign = PULL_CLS.LEFT;
                this.menuCaret = CARET_CLS.DOWN;
                break;
            case POSITION.DOWN_LEFT:
                removeClass(this.nativeElement, 'dropup');
                this.menualign = PULL_CLS.RIGHT;
                this.menuCaret = CARET_CLS.DOWN;
                break;
            case POSITION.UP_LEFT:
                addClass(this.nativeElement, 'dropup');
                this.menualign = PULL_CLS.RIGHT;
                this.menuCaret = CARET_CLS.UP;
                break;
            case POSITION.UP_RIGHT:
                addClass(this.nativeElement, 'dropup');
                this.menualign = PULL_CLS.LEFT;
                this.menuCaret = CARET_CLS.UP;
                break;
            case POSITION.INLINE:
                this.menualign = 'dropinline-menu';
                break;
        }
    }
    onMenuItemSelect(args) {
        const { $event } = args;
        const $item = args.$item.value;
        this.invokeEventCallback('select', { $event, $item });
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        styler(this.nativeElement.querySelector('.dropdown-toggle'), this);
    }
};
MenuComponent.initializeProps = registerProps$1();
MenuComponent.ctorParameters = () => [
    { type: Injector },
    { type: Router },
    { type: UserDefinedExecutionContext },
    { type: BsDropdownDirective, decorators: [{ type: Self }, { type: Optional }] },
    { type: NavComponent, decorators: [{ type: Optional }] },
    { type: String, decorators: [{ type: Attribute, args: ['select.event',] }] }
];
__decorate([
    HostListener('onShown'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MenuComponent.prototype, "onShow", null);
__decorate([
    HostListener('onHidden'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MenuComponent.prototype, "onHide", null);
__decorate([
    HostListener('keydown.arrowup', ['$event', '"UP-ARROW"']),
    HostListener('keydown.arrowdown', ['$event', '"DOWN-ARROW"']),
    HostListener('keydown.arrowright', ['$event', '"RIGHT-ARROW"']),
    HostListener('keydown.arrowleft', ['$event', '"LEFT-ARROW"']),
    HostListener('keydown.enter', ['$event', '"ENTER"']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], MenuComponent.prototype, "onKeyDown", null);
MenuComponent = MenuComponent_1 = __decorate([
    Component({
        selector: '[wmMenu]',
        template: "<ng-template #menuTemplate>\n    <button wmButton\n            dropdownToggle\n            aria-haspopup=\"true\"\n            aria-expanded=\"false\"\n\n            class=\"btn app-button dropdown-toggle {{menuclass}}\"\n            hint.bind=\"hint\"\n            shortcutkey.bind=\"shortcutkey\"\n            tabindex.bind=\"tabindex\"\n            caption.bind=\"caption\"\n            iconclass.bind=\"iconclass\"\n            iconposition.bind=\"iconposition\">\n        <span class=\"pull-right caret fa {{menuCaret}}\"></span>\n    </button>\n</ng-template>\n\n<ng-template #innerTemplate>\n    <i class=\"app-icon {{iconclass}}\"></i>\n    <span class=\"caption\" [textContent]=\"caption\"></span>\n    <span class=\"pull-right caret fa {{menuCaret}}\"></span>\n</ng-template>\n\n<ng-container *ngIf=\"type === 'anchor'; else menuTemplate\">\n    <a wmAnchor\n       dropdownToggle\n\n       href=\"javascript:void(0);\"\n       role=\"button\"\n       aria-haspopup=\"true\"\n       aria-expanded=\"false\"\n\n       hint.bind=\"hint\"\n       class=\"dropdown-toggle {{menuclass}}\"\n       shortcutkey.bind=\"shortcutkey\"\n       tabindex.bind=\"tabindex\"\n       caption.bind=\"caption\"\n       iconclass.bind=\"iconclass\"\n       iconposition.bind=\"iconposition\"\n    >\n        <span class=\"pull-right caret fa {{menuCaret}}\"></span>\n    </a>\n</ng-container>\n\n<ul wmMenuDropdown [items]=\"nodes\" [ngClass]=\"menulayout\" class=\"icon-position-{{iconposition}}\" *dropdownMenu aria-labelledby=\"dropdownmenu\"></ul>\n",
        providers: [
            provideAsWidgetRef(MenuComponent_1)
        ]
    }),
    __param(3, Self()), __param(3, Optional()),
    __param(4, Optional()),
    __param(5, Attribute('select.event')),
    __metadata("design:paramtypes", [Injector,
        Router,
        UserDefinedExecutionContext,
        BsDropdownDirective,
        NavComponent, String])
], MenuComponent);

const animationClasses = {
    scale: {
        'name': 'wmScaleInLeft',
        'down,right': 'wmScaleInLeft',
        'down,left': 'wmScaleInRight',
        'up,right': 'wmScaleInTopLeft',
        'up,left': 'wmScaleInTopRight'
    },
    fade: {
        'name': 'fadeIn',
        'down,right': 'fadeIn',
        'down,left': 'fadeIn',
        'up,right': 'fadeIn',
        'up,left': 'fadeIn'
    },
    slide: {
        'name': 'wmSlideInDown',
        'down,right': 'wmSlideInDown',
        'down,left': 'wmSlideInDown',
        'up,right': 'wmSlideInUp',
        'up,left': 'wmSlideInUp'
    }
};
const DEFAULT_CLS$1 = 'dropdown-menu';
let MenuDropdownComponent = class MenuDropdownComponent {
    constructor(elRef, menuRef) {
        this.menuRef = menuRef;
        this.nativeElement = elRef.nativeElement;
        addClass(this.nativeElement, DEFAULT_CLS$1);
    }
    ngAfterViewInit() {
        const animateItems = this.menuRef.animateitems;
        let animationClass = '';
        if (animateItems) {
            animationClass = `animated ${(animationClasses[animateItems][this.menuRef.menuposition] || animationClasses[animateItems].name)}`;
        }
        addClass(this.nativeElement, `dropdown-menu ${this.menuRef.menualign} ${animationClass}`, true);
    }
};
MenuDropdownComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: MenuComponent }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], MenuDropdownComponent.prototype, "items", void 0);
MenuDropdownComponent = __decorate([
    Component({
        selector: 'ul[wmMenuDropdown]',
        template: "<li wmMenuDropdownItem *ngFor=\"let item of items\"\n    [item]=\"item\"\n    [ngClass]=\"{disabled: item.disabled, 'dropdown-submenu': item.children.length > 0}\"\n    class=\"{{item.class}}\">\n</li>"
    }),
    __metadata("design:paramtypes", [ElementRef, MenuComponent])
], MenuDropdownComponent);

const menuAlignClass = {
    'pull-right': 'fa-caret-left',
    'dropinline-menu': 'fa-caret-down',
    'pull-left': 'fa-caret-right'
};
const MENU_LAYOUT_TYPE = {
    HORIZONTAL: 'horizontal',
    VERTICAL: 'vertical'
};
let MenuDropdownItemComponent = class MenuDropdownItemComponent {
    constructor(menuRef, userDefinedExecutionContext, parentNav, elRef) {
        this.menuRef = menuRef;
        this.userDefinedExecutionContext = userDefinedExecutionContext;
        this.parentNav = parentNav;
        this.onSelect = ($event, item) => {
            if (this.nativeElement !== $($event.target).closest('.app-menu-item').get(0)) {
                return;
            }
            // prevent event event propagation if auto close is outside click.
            if (this.menuRef.autoclose === 'outsideClick') {
                $event.stopPropagation();
            }
            $event.preventDefault();
            const args = { $event, $item: item };
            this.menuRef.onMenuItemSelect(args);
            const selectedItem = _.clone(item);
            selectedItem.target = selectedItem.target || this.menuRef.linktarget;
            // Trigger the action associated with active item
            triggerItemAction(this, selectedItem);
        };
        this.nativeElement = elRef.nativeElement;
        addClass(this.nativeElement, 'app-menu-item');
        this.menualign = menuAlignClass[this.menuRef.menualign] || menuAlignClass['pull-left'];
    }
    ngOnInit() {
        // add active class to the item only if it is in nav component.
        if (this.parentNav) {
            if (isActiveNavItem(this.item.link, this.menuRef.route.url)) {
                // add active class to the li, if the menu item's link is same as the current page name.
                addClass(this.nativeElement, 'active');
            }
        }
    }
    getInitialKeyMovements() {
        const KEY_MOVEMENTS = _.clone(KEYBOARD_MOVEMENTS);
        if (this.menuRef.menulayout === MENU_LAYOUT_TYPE.HORIZONTAL) {
            KEY_MOVEMENTS.MOVE_UP = 'LEFT-ARROW';
            KEY_MOVEMENTS.MOVE_LEFT = 'UP-ARROW';
            KEY_MOVEMENTS.MOVE_RIGHT = 'DOWN-ARROW';
            KEY_MOVEMENTS.MOVE_DOWN = 'RIGHT-ARROW';
        }
        else {
            if (this.menuRef.menuposition === MENU_POSITION.DOWN_LEFT || this.menuRef.menuposition === MENU_POSITION.UP_LEFT) {
                KEY_MOVEMENTS.MOVE_LEFT = 'RIGHT-ARROW';
                KEY_MOVEMENTS.MOVE_RIGHT = 'LEFT-ARROW';
            }
            else if (this.menuRef.menuposition === 'inline') {
                KEY_MOVEMENTS.MOVE_UP = 'LEFT-ARROW';
                KEY_MOVEMENTS.MOVE_LEFT = 'UP-ARROW';
                KEY_MOVEMENTS.MOVE_RIGHT = 'DOWN-ARROW';
                KEY_MOVEMENTS.MOVE_DOWN = 'RIGHT-ARROW';
            }
        }
        return KEY_MOVEMENTS;
    }
    onKeyDown($event, eventAction) {
        const $li = $(this.nativeElement);
        const $ul = $(this.nativeElement).closest('ul.dropdown-menu');
        const $parentUl = this.menuRef.$element.find('> ul.dropdown-menu');
        const ARROW_KEYS = ['LEFT-ARROW', 'RIGHT-ARROW', 'UP-ARROW', 'DOWN-ARROW'];
        const KEY_MOVEMENTS = this.getInitialKeyMovements();
        if (_.includes(ARROW_KEYS, eventAction)) {
            // preventing from page scroll when up/down arrow is pressed, in case of menu is opened.
            $event.preventDefault();
        }
        if ((eventAction === KEY_MOVEMENTS.ON_TAB && $parentUl.children().last()[0] === this.nativeElement)
            || (eventAction === KEY_MOVEMENTS.ON_SHIFT_TAB && $parentUl.children().first()[0] === this.nativeElement)
            || eventAction === KEY_MOVEMENTS.ON_ESCAPE) {
            /*closing all the children elements when
            * 1. Tab is clicked on the last $element
            * 2. Shift Tab is clicked on the first $element
            * 3. When Escape key is clicked*/
            $event.preventDefault();
            this.menuRef.bsDropdown.hide();
        }
        else if ((eventAction === KEY_MOVEMENTS.ON_ENTER && !this.item.link) || eventAction === KEY_MOVEMENTS.MOVE_RIGHT) {
            // when there is no link for the menu, on enter open the inner child elements and focus the first $element
            $event.stopPropagation();
            if (this.item.children.length) {
                $li.toggleClass('open');
                $li.find('li').first().find('> a').focus();
            }
            else {
                $li.find('> a').focus();
            }
        }
        else if (eventAction === KEY_MOVEMENTS.MOVE_LEFT
            || (eventAction === KEY_MOVEMENTS.ON_TAB && $ul.children().last()[0] === this.nativeElement)
            || (eventAction === KEY_MOVEMENTS.ON_SHIFT_TAB && $ul.children().first()[0] === this.nativeElement)) {
            if ($parentUl[0] !== $ul[0]) {
                const $parentItem = $ul.parent();
                $parentItem.toggleClass('open').find('li.open').removeClass('open');
                $parentItem.find('> a').focus();
                $event.preventDefault();
                $event.stopPropagation();
            }
        }
        else if (eventAction === KEY_MOVEMENTS.MOVE_UP) {
            if ($parentUl[0] !== $ul[0] || $parentUl.find('> li').first()[0] !== this.nativeElement) {
                $event.stopPropagation();
                $li.prev().find('> a').focus();
            }
        }
        else if (eventAction === KEY_MOVEMENTS.MOVE_DOWN) {
            $event.stopPropagation();
            if ($parentUl.find('> li').last()[0] === this.nativeElement
                && (this.menuRef.menulayout !== MENU_LAYOUT_TYPE.HORIZONTAL
                    && this.menuRef.menuposition === MENU_POSITION.UP_RIGHT
                    || this.menuRef.menuposition === MENU_POSITION.UP_LEFT)) {
                this.menuRef.bsDropdown.hide();
            }
            else {
                $li.next().find('> a').focus();
            }
        }
        else if (eventAction === KEY_MOVEMENTS.ON_ENTER) {
            this.onSelect($event, this.item);
        }
        else {
            $event.stopPropagation();
        }
    }
};
MenuDropdownItemComponent.ctorParameters = () => [
    { type: MenuComponent },
    { type: UserDefinedExecutionContext },
    { type: NavComponent, decorators: [{ type: Optional }] },
    { type: ElementRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], MenuDropdownItemComponent.prototype, "item", void 0);
__decorate([
    HostListener('keydown.tab', ['$event', '"TAB"']),
    HostListener('keydown.shift.tab', ['$event', '"SHIFT-TAB"']),
    HostListener('keydown.escape', ['$event', '"ESC"']),
    HostListener('keydown.enter', ['$event', '"ENTER"']),
    HostListener('keydown.arrowup', ['$event', '"UP-ARROW"']),
    HostListener('keydown.arrowdown', ['$event', '"DOWN-ARROW"']),
    HostListener('keydown.arrowright', ['$event', '"RIGHT-ARROW"']),
    HostListener('keydown.arrowleft', ['$event', '"LEFT-ARROW"']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], MenuDropdownItemComponent.prototype, "onKeyDown", null);
__decorate([
    HostListener('click', ['$event', 'item']),
    __metadata("design:type", Object)
], MenuDropdownItemComponent.prototype, "onSelect", void 0);
MenuDropdownItemComponent = __decorate([
    Component({
        selector: 'li[wmMenuDropdownItem]',
        template: "<a href=\"javascript:void(0);\" [title]=\"item.label\" [wmNavigationControl]=\"item.link\" [disableMenuContext]=\"menuRef.disableMenuContext || !!item.action\">\n    <span *ngIf=\"item.children.length\" class=\"pull-right fa caret {{menualign}}\"></span>\n    <i class=\"app-icon {{item.icon}}\"></i>\n    <span class=\"anchor-caption\">{{item.label}}</span>\n\n</a>\n<ng-container [ngTemplateOutlet]=\"nestedMenuDropdown\" *ngIf=\"item.children.length\" [ngTemplateOutletContext]=\"{item: item}\"></ng-container>\n\n<ng-template #nestedMenuDropdown let-item=\"item\">\n    <ul wmMenuDropdown [items]=\"item.children\"></ul>\n</ng-template>\n"
    }),
    __param(2, Optional()),
    __metadata("design:paramtypes", [MenuComponent,
        UserDefinedExecutionContext,
        NavComponent,
        ElementRef])
], MenuDropdownItemComponent);

const registerProps$2 = () => {
    register('wm-nav-item', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var NavItemDirective_1;
const DEFAULT_CLS$2 = 'app-nav-item';
const WIDGET_CONFIG$2 = { widgetType: 'wm-nav-item', hostClass: DEFAULT_CLS$2 };
let NavItemDirective = NavItemDirective_1 = class NavItemDirective extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG$2);
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
    }
    makeActive() {
        const parentNode = this.nativeElement.parentNode;
        $(parentNode).find('> li.active').removeClass('active');
        if (this.isAttached) {
            addClass(this.nativeElement, 'active');
        }
    }
    ngAfterViewInit() {
        if (this.innerLink) {
            this.innerLink.onActive(() => this.makeActive());
        }
    }
    ngOnDetach() {
        removeClass(this.nativeElement, 'active', true);
        super.ngOnDetach();
    }
};
NavItemDirective.initializeProps = registerProps$2();
NavItemDirective.ctorParameters = () => [
    { type: Injector }
];
__decorate([
    ContentChild(AnchorComponent),
    __metadata("design:type", Object)
], NavItemDirective.prototype, "innerLink", void 0);
__decorate([
    HostListener('click'),
    HostListener('keydown.enter'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], NavItemDirective.prototype, "makeActive", null);
NavItemDirective = NavItemDirective_1 = __decorate([
    Directive({
        selector: '[wmNavItem]',
        providers: [
            provideAsWidgetRef(NavItemDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], NavItemDirective);

const disableContextMenu = ($event) => {
    $event.preventDefault();
};
let NavigationControlDirective = class NavigationControlDirective {
    constructor(eleRef) {
        this.nativeElement = eleRef.nativeElement;
    }
    set wmNavigationControl(val) {
        this._link = val;
        if (val && !this.disableMenuContext) {
            setAttr(this.nativeElement, 'href', val);
            this.nativeElement.removeEventListener('contextmenu', disableContextMenu);
        }
        else {
            setAttr(this.nativeElement, 'href', 'javascript:void(0)');
            this.nativeElement.addEventListener('contextmenu', disableContextMenu);
        }
    }
};
NavigationControlDirective.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], NavigationControlDirective.prototype, "disableMenuContext", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], NavigationControlDirective.prototype, "wmNavigationControl", null);
NavigationControlDirective = __decorate([
    Directive({ selector: '[wmNavigationControl]' }),
    __metadata("design:paramtypes", [ElementRef])
], NavigationControlDirective);

const components = [
    MenuComponent,
    MenuDropdownComponent,
    MenuDropdownItemComponent,
    NavigationControlDirective,
    NavComponent,
    NavItemDirective
];
let MenuModule = class MenuModule {
};
MenuModule = __decorate([
    NgModule({
        imports: [
            BasicModule,
            BsDropdownModule,
            CommonModule,
            InputModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: [
            MenuComponent,
            MenuDropdownComponent
        ]
    })
], MenuModule);

const menuProps = ['itemlabel', 'itemicon', 'itemlink', 'itemaction', 'itemchildren', 'userrole'];
let MenuAdapterComponent = class MenuAdapterComponent extends BaseContainerComponent {
    constructor(inj, WIDGET_CONFIG) {
        super(inj, WIDGET_CONFIG);
        this.pageScope = this.viewParent;
        this.binditemlabel = this.nativeElement.getAttribute('itemlabel.bind');
        this.binditemicon = this.nativeElement.getAttribute('itemicon.bind');
        this.binditemaction = this.nativeElement.getAttribute('itemaction.bind');
        this.binditemlink = this.nativeElement.getAttribute('itemlink.bind');
        this.binduserrole = this.nativeElement.getAttribute('userrole.bind');
        this.binditemchildren = this.nativeElement.getAttribute('itemchildren.bind');
    }
    onPropertyChange(key, nv, ov) {
        if (_.includes(menuProps, key) && this.menuRef) {
            this.menuRef.itemlabel = nv;
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        const subscriber = this.menuRefQL.changes.subscribe((menuRefQL) => {
            if (menuRefQL.first) {
                this.menuRef = menuRefQL.first;
                menuProps.forEach((prop) => {
                    const bindProp = `bind${prop}`;
                    if (this[bindProp]) {
                        this.menuRef[bindProp] = this[bindProp];
                    }
                    this.menuRef[prop] = this[prop];
                });
                subscriber.unsubscribe();
            }
        });
    }
};
MenuAdapterComponent.ctorParameters = () => [
    null,
    null
];
__decorate([
    ViewChildren(MenuComponent),
    __metadata("design:type", QueryList)
], MenuAdapterComponent.prototype, "menuRefQL", void 0);
MenuAdapterComponent = __decorate([
    Directive(),
    __metadata("design:paramtypes", [Object, Object])
], MenuAdapterComponent);

/**
 * Generated bundle index. Do not edit.
 */

export { MenuAdapterComponent, MenuModule, NavComponent, NavItemDirective, MenuComponent as ɵa, registerProps as ɵb, registerProps$1 as ɵc, MenuDropdownComponent as ɵd, MenuDropdownItemComponent as ɵe, NavigationControlDirective as ɵf, registerProps$2 as ɵg };
//# sourceMappingURL=index.js.map
