import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Inject, ViewChild, ElementRef, ContentChild, TemplateRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PopoverDirective, PopoverModule as PopoverModule$1 } from 'ngx-bootstrap/popover';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, StylableComponent, AUTOCLOSE_TYPE, styler, getContainerTargetClass, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { EVENT_MANAGER_PLUGINS } from '@angular/platform-browser';
import { adjustContainerPosition, adjustContainerRightEdges, findRootContainer, setCSSFromObj, addClass, setAttr, App } from '@wm/core';

const registerProps = () => {
    register('wm-popover', new Map([
        ['autoclose', Object.assign({ value: 'outsideClick' }, PROP_STRING)],
        ['badgevalue', PROP_STRING],
        ['caption', Object.assign({ value: 'Link' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['content', PROP_STRING],
        ['contentanimation', PROP_STRING],
        ['contentsource', Object.assign({ value: 'partial' }, PROP_STRING)],
        ['encodeurl', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['hyperlink', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconposition', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['interaction', Object.assign({ value: 'click' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['popoverarrow', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['popoverheight', PROP_STRING],
        ['popoverplacement', Object.assign({ value: 'bottom' }, PROP_STRING)],
        ['popoverwidth', PROP_STRING],
        ['shortcutkey', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', Object.assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING],
        ['adaptiveposition', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var PopoverComponent_1;
const DEFAULT_CLS = 'app-popover-wrapper';
const WIDGET_CONFIG = {
    widgetType: 'wm-popover',
    hostClass: DEFAULT_CLS
};
const eventsMap = {
    click: 'click',
    hover: 'mouseenter:click',
    default: 'click mouseenter'
};
let activePopover;
let PopoverComponent = PopoverComponent_1 = class PopoverComponent extends StylableComponent {
    constructor(inj, app, evtMngrPlugins) {
        super(inj, WIDGET_CONFIG);
        this.app = app;
        this.isOpen = false;
        this.canPopoverOpen = true;
        // KeyEventsPlugin
        this.keyEventPlugin = evtMngrPlugins[1];
        this.popoverContainerCls = `app-popover-${this.widgetId}`;
    }
    // This mehtod is used to show/open the popover. This refers to the same method showPopover.
    open() {
        this.showPopover();
    }
    // This mehtod is used to hide/close the popover.
    close() {
        this.isOpen = false;
    }
    // Trigger on hiding popover
    onHidden() {
        this.invokeEventCallback('hide', { $event: { type: 'hide' } });
    }
    setFocusToPopoverLink() {
        setTimeout(() => this.anchorRef.nativeElement.focus(), 10);
    }
    adjustPopoverArrowPosition(popoverElem, popoverLeftShift) {
        this.bsPopoverDirective._popover._ngZone.onStable.subscribe(() => {
            popoverElem.find('.popover-arrow').css('left', popoverLeftShift + 'px');
        });
    }
    calculatePopoverPostion(element) {
        const popoverElem = $(element);
        let popoverLeft = popoverElem.offset().left;
        const popoverWidth = popoverElem[0].offsetWidth;
        const viewPortWidth = $(window).width();
        const parentDimesion = this.anchorRef.nativeElement.getBoundingClientRect();
        // Adjusting popover position, if it is not visible at left side
        if (popoverLeft < 0) {
            adjustContainerPosition(popoverElem, this.nativeElement, this.bsPopoverDirective._popover);
            const popoverLeftShift = 4;
            const arrowLeftShift = (parentDimesion.left + (parentDimesion.width / 2)) - popoverLeftShift;
            this.adjustPopoverArrowPosition(popoverElem, arrowLeftShift);
        }
        // Adjusting popover position, if it is not visible at right side
        if (popoverLeft + popoverWidth > viewPortWidth) {
            adjustContainerRightEdges(popoverElem, this.nativeElement, this.bsPopoverDirective._popover);
            setTimeout(() => {
                popoverLeft = popoverElem.offset().left;
                const popoverLeftAdjust = (popoverLeft + popoverWidth) - viewPortWidth;
                const popoverLeftShift = popoverLeft - popoverLeftAdjust;
                let arrowLeftShift;
                if (popoverLeft <= 100) {
                    arrowLeftShift = (parentDimesion.left + (parentDimesion.width / 2)) - popoverLeft;
                }
                else {
                    arrowLeftShift = (parentDimesion.left + (parentDimesion.width / 2)) - popoverLeftShift;
                }
                if (viewPortWidth > 500) {
                    arrowLeftShift += 30;
                }
                this.adjustPopoverArrowPosition(popoverElem, arrowLeftShift);
            });
        }
    }
    // Trigger on showing popover
    onShown() {
        const root = findRootContainer(this.$element);
        // if page styles have to be applied to popover then popover has to be child of page element.
        if (root) {
            $('body > popover-container').wrap('<' + root + '/>');
        }
        if (activePopover && activePopover.isOpen) {
            activePopover.isOpen = false;
        }
        activePopover = this;
        activePopover.isOpen = true;
        const popoverContainer = document.querySelector(`.${this.popoverContainerCls}`);
        setCSSFromObj(popoverContainer, {
            height: this.popoverheight,
            minWidth: this.popoverwidth,
            width: this.popoverwidth
        });
        if (!this.popoverarrow) {
            addClass(popoverContainer.querySelector('.arrow'), 'hidden');
        }
        if (this.interaction === 'hover' || this.interaction === 'default') {
            // do not use addEventListener here
            // attaching the event this way will override the existing event handlers
            popoverContainer.onmouseenter = () => clearTimeout(this.closePopoverTimeout);
            popoverContainer.onmouseleave = () => this.hidePopover();
            this.anchorRef.nativeElement.onmouseenter = () => clearTimeout(this.closePopoverTimeout);
            this.anchorRef.nativeElement.onmouseleave = () => this.hidePopover();
        }
        const deRegister = this.eventManager.addEventListener(popoverContainer, 'keydown.esc', () => {
            this.isOpen = false;
            this.setFocusToPopoverLink();
            deRegister();
        });
        const popoverStartBtn = popoverContainer.querySelector('.popover-start');
        const popoverEndBtn = popoverContainer.querySelector('.popover-end');
        popoverStartBtn.onkeydown = (event) => {
            const action = this.keyEventPlugin.constructor.getEventFullKey(event);
            // Check for Shift+Tab key
            if (action === 'shift.tab') {
                this.bsPopoverDirective.hide();
                this.setFocusToPopoverLink();
            }
        };
        popoverEndBtn.onkeydown = (event) => {
            const action = this.keyEventPlugin.constructor.getEventFullKey(event);
            // Check for Tab key
            if (action === 'tab') {
                this.bsPopoverDirective.hide();
                this.setFocusToPopoverLink();
            }
        };
        //Whenever autoclose property is set to 'always', adding the onclick listener to the popover container to close the popover.
        if (this.autoclose === AUTOCLOSE_TYPE.ALWAYS) {
            popoverContainer.onclick = () => this.close();
        }
        setAttr(popoverContainer, 'tabindex', 0);
        setTimeout(() => popoverStartBtn.focus(), 50);
        // Adjusting popover position if the popover placement is top or bottom
        setTimeout(() => {
            if (!this.adaptiveposition) {
                this.calculatePopoverPostion(popoverContainer);
            }
            // triggering onload and onshow events after popover content has rendered
            this.triggerPopoverEvents();
        });
    }
    triggerPopoverEvents() {
        if (this.contentsource === 'partial') {
            const cancelSubscription = this.app.subscribe('partialLoaded', (data) => {
                const parEle = this.partialRef.nativeElement;
                let partialScope;
                if (parEle) {
                    partialScope = parEle.widget;
                    this.Widgets = partialScope.Widgets;
                    this.Variables = partialScope.Variables;
                    this.Actions = partialScope.Actions;
                    this.invokeEventCallback('load');
                    this.invokeEventCallback('show', { $event: { type: 'show' } });
                }
                cancelSubscription();
            });
        }
        else {
            this.Widgets = this.viewParent.Widgets;
            this.Variables = this.viewParent.Variables;
            this.Actions = this.viewParent.Actions;
            this.invokeEventCallback('show', { $event: { type: 'show' } });
        }
    }
    hidePopover() {
        this.closePopoverTimeout = setTimeout(() => this.isOpen = false, 500);
    }
    showPopover() {
        this.bsPopoverDirective.show();
    }
    onPopoverAnchorKeydown($event) {
        // if there is no content available, the popover should not open through enter key. So checking whether the canPopoverOpen flag is true or not.
        if (!this.canPopoverOpen) {
            return;
        }
        const action = this.keyEventPlugin.constructor.getEventFullKey(event);
        if (action === 'enter') {
            $event.stopPropagation();
            this.showPopover();
        }
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'class' || key === 'tabindex') {
            return;
        }
        if (key === 'contentsource') {
            // if there is no partial content available, the popover should not open
            if (this.contentsource === 'partial' && !this.content) {
                this.canPopoverOpen = false;
            }
        }
        if (key === 'content' && nv) {
            this.canPopoverOpen = true;
        }
        if (key === 'autoclose') {
            this.outsideclick = (nv === AUTOCLOSE_TYPE.OUTSIDECLICK || nv === AUTOCLOSE_TYPE.ALWAYS) ? true : false;
        }
        super.onPropertyChange(key, nv, ov);
    }
    ngOnInit() {
        super.ngOnInit();
        this.event = eventsMap[this.interaction];
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        styler(this.anchorRef.nativeElement, this);
        this.containerTarget = getContainerTargetClass(this.nativeElement);
        let parentElemPopover = $(this.nativeElement).parents();
        if (parentElemPopover.closest('[wmTable]').length ||
            parentElemPopover.closest('[wmtabs]').length ||
            parentElemPopover.closest('modal-container').length) {
            this.adaptiveposition = false;
        }
    }
};
PopoverComponent.initializeProps = registerProps();
PopoverComponent.ctorParameters = () => [
    { type: Injector },
    { type: App },
    { type: undefined, decorators: [{ type: Inject, args: [EVENT_MANAGER_PLUGINS,] }] }
];
__decorate([
    ViewChild(PopoverDirective),
    __metadata("design:type", Object)
], PopoverComponent.prototype, "bsPopoverDirective", void 0);
__decorate([
    ViewChild('anchor', { static: true }),
    __metadata("design:type", ElementRef)
], PopoverComponent.prototype, "anchorRef", void 0);
__decorate([
    ContentChild(TemplateRef),
    __metadata("design:type", Object)
], PopoverComponent.prototype, "popoverTemplate", void 0);
__decorate([
    ContentChild('partial'),
    __metadata("design:type", Object)
], PopoverComponent.prototype, "partialRef", void 0);
PopoverComponent = PopoverComponent_1 = __decorate([
    Component({
        selector: 'wm-popover',
        template: "<a [popover]=\"popoverTemplate\"\n    [popoverContext]=\"context\"\n    [popoverTitle]=\"title\"\n    [placement]=\"popoverplacement\"\n    [outsideClick]=\"outsideclick\"\n    [triggers]=\"event\"\n    (onShown)=\"onShown()\"\n    (onHidden)=\"onHidden()\"\n    containerClass=\"app-popover animated {{contentanimation}} {{popoverContainerCls}}\"\n    [container]=\"containerTarget || 'body'\"\n    [isOpen]=\"isOpen\"\n    [tabindex]=\"tabindex\"\n    [name]=\"name\"\n    [class]=\"class\"\n    [ngClass]=\"{'disable-popover': !canPopoverOpen}\"\n    [adaptivePosition]=\"adaptiveposition\"\n    #anchor\n    name.bind=\"name\"\n    wmAnchor\n    animation.bind=\"animation\"\n    badgevalue.bind=\"badgevalue\"\n    caption.bind=\"caption\"\n    encodeurl.bind=\"encodeurl\"\n    hint.bind=\"hint\"\n    hyperlink.bind=\"hyperlink\"\n    iconurl.bind=\"iconurl\"\n    iconwidth.bind=\"iconwidth\"\n    iconheight.bind=\"iconheight\"\n    iconmargin.bind=\"iconmargin\"\n    iconclass.bind=\"iconclass\"\n    iconposition.bind=\"iconposition\"\n    shortcutkey.bind=\"shortcutkey\"\n    tabindex.bind=\"tabindex\"\n    (keydown)=\"onPopoverAnchorKeydown($event)\">\n</a>\n",
        providers: [
            provideAsWidgetRef(PopoverComponent_1)
        ]
    }),
    __param(2, Inject(EVENT_MANAGER_PLUGINS)),
    __metadata("design:paramtypes", [Injector, App, Object])
], PopoverComponent);

const components = [
    PopoverComponent
];
let PopoverModule = class PopoverModule {
};
PopoverModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            BasicModule,
            PopoverModule$1,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], PopoverModule);

/**
 * Generated bundle index. Do not edit.
 */

export { PopoverModule, PopoverComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
