import { __assign, __extends, __decorate, __metadata, __param, __spread } from 'tslib';
import { Injector, Inject, ViewChild, ElementRef, ContentChild, TemplateRef, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PopoverDirective, PopoverModule as PopoverModule$1 } from 'ngx-bootstrap/popover';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, PROP_NUMBER, AUTOCLOSE_TYPE, styler, getContainerTargetClass, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { BasicModule } from '@wm/components/basic';
import { EVENT_MANAGER_PLUGINS } from '@angular/platform-browser';
import { adjustContainerPosition, adjustContainerRightEdges, findRootContainer, setCSSFromObj, addClass, setAttr, App } from '@wm/core';

var registerProps = function () {
    register('wm-popover', new Map([
        ['autoclose', __assign({ value: 'outsideClick' }, PROP_STRING)],
        ['badgevalue', PROP_STRING],
        ['caption', __assign({ value: 'Link' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['conditionalclass', PROP_ANY],
        ['content', PROP_STRING],
        ['contentanimation', PROP_STRING],
        ['contentsource', __assign({ value: 'partial' }, PROP_STRING)],
        ['encodeurl', PROP_BOOLEAN],
        ['hint', PROP_STRING],
        ['hyperlink', PROP_STRING],
        ['iconclass', PROP_STRING],
        ['iconheight', PROP_STRING],
        ['iconmargin', PROP_STRING],
        ['iconposition', PROP_STRING],
        ['iconurl', PROP_STRING],
        ['iconwidth', PROP_STRING],
        ['interaction', __assign({ value: 'click' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['popoverarrow', __assign({ value: true }, PROP_BOOLEAN)],
        ['popoverheight', PROP_STRING],
        ['popoverplacement', __assign({ value: 'bottom' }, PROP_STRING)],
        ['popoverwidth', PROP_STRING],
        ['shortcutkey', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['tabindex', __assign({ value: 0 }, PROP_NUMBER)],
        ['title', PROP_STRING],
        ['adaptiveposition', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS = 'app-popover-wrapper';
var WIDGET_CONFIG = {
    widgetType: 'wm-popover',
    hostClass: DEFAULT_CLS
};
var eventsMap = {
    click: 'click',
    hover: 'mouseenter:click',
    default: 'click mouseenter'
};
var activePopover;
var PopoverComponent = /** @class */ (function (_super) {
    __extends(PopoverComponent, _super);
    function PopoverComponent(inj, app, evtMngrPlugins) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.app = app;
        _this.isOpen = false;
        _this.canPopoverOpen = true;
        // KeyEventsPlugin
        _this.keyEventPlugin = evtMngrPlugins[1];
        _this.popoverContainerCls = "app-popover-" + _this.widgetId;
        return _this;
    }
    PopoverComponent_1 = PopoverComponent;
    // This mehtod is used to show/open the popover. This refers to the same method showPopover.
    PopoverComponent.prototype.open = function () {
        this.showPopover();
    };
    // This mehtod is used to hide/close the popover.
    PopoverComponent.prototype.close = function () {
        this.isOpen = false;
    };
    // Trigger on hiding popover
    PopoverComponent.prototype.onHidden = function () {
        this.invokeEventCallback('hide', { $event: { type: 'hide' } });
    };
    PopoverComponent.prototype.setFocusToPopoverLink = function () {
        var _this = this;
        setTimeout(function () { return _this.anchorRef.nativeElement.focus(); }, 10);
    };
    PopoverComponent.prototype.adjustPopoverArrowPosition = function (popoverElem, popoverLeftShift) {
        this.bsPopoverDirective._popover._ngZone.onStable.subscribe(function () {
            popoverElem.find('.popover-arrow').css('left', popoverLeftShift + 'px');
        });
    };
    PopoverComponent.prototype.calculatePopoverPostion = function (element) {
        var _this = this;
        var popoverElem = $(element);
        var popoverLeft = popoverElem.offset().left;
        var popoverWidth = popoverElem[0].offsetWidth;
        var viewPortWidth = $(window).width();
        var parentDimesion = this.anchorRef.nativeElement.getBoundingClientRect();
        // Adjusting popover position, if it is not visible at left side
        if (popoverLeft < 0) {
            adjustContainerPosition(popoverElem, this.nativeElement, this.bsPopoverDirective._popover);
            var popoverLeftShift = 4;
            var arrowLeftShift = (parentDimesion.left + (parentDimesion.width / 2)) - popoverLeftShift;
            this.adjustPopoverArrowPosition(popoverElem, arrowLeftShift);
        }
        // Adjusting popover position, if it is not visible at right side
        if (popoverLeft + popoverWidth > viewPortWidth) {
            adjustContainerRightEdges(popoverElem, this.nativeElement, this.bsPopoverDirective._popover);
            setTimeout(function () {
                popoverLeft = popoverElem.offset().left;
                var popoverLeftAdjust = (popoverLeft + popoverWidth) - viewPortWidth;
                var popoverLeftShift = popoverLeft - popoverLeftAdjust;
                var arrowLeftShift;
                if (popoverLeft <= 100) {
                    arrowLeftShift = (parentDimesion.left + (parentDimesion.width / 2)) - popoverLeft;
                }
                else {
                    arrowLeftShift = (parentDimesion.left + (parentDimesion.width / 2)) - popoverLeftShift;
                }
                if (viewPortWidth > 500) {
                    arrowLeftShift += 30;
                }
                _this.adjustPopoverArrowPosition(popoverElem, arrowLeftShift);
            });
        }
    };
    // Trigger on showing popover
    PopoverComponent.prototype.onShown = function () {
        var _this = this;
        var root = findRootContainer(this.$element);
        // if page styles have to be applied to popover then popover has to be child of page element.
        if (root) {
            $('body > popover-container').wrap('<' + root + '/>');
        }
        if (activePopover && activePopover.isOpen) {
            activePopover.isOpen = false;
        }
        activePopover = this;
        activePopover.isOpen = true;
        var popoverContainer = document.querySelector("." + this.popoverContainerCls);
        setCSSFromObj(popoverContainer, {
            height: this.popoverheight,
            minWidth: this.popoverwidth,
            width: this.popoverwidth
        });
        if (!this.popoverarrow) {
            addClass(popoverContainer.querySelector('.arrow'), 'hidden');
        }
        if (this.interaction === 'hover' || this.interaction === 'default') {
            // do not use addEventListener here
            // attaching the event this way will override the existing event handlers
            popoverContainer.onmouseenter = function () { return clearTimeout(_this.closePopoverTimeout); };
            popoverContainer.onmouseleave = function () { return _this.hidePopover(); };
            this.anchorRef.nativeElement.onmouseenter = function () { return clearTimeout(_this.closePopoverTimeout); };
            this.anchorRef.nativeElement.onmouseleave = function () { return _this.hidePopover(); };
        }
        var deRegister = this.eventManager.addEventListener(popoverContainer, 'keydown.esc', function () {
            _this.isOpen = false;
            _this.setFocusToPopoverLink();
            deRegister();
        });
        var popoverStartBtn = popoverContainer.querySelector('.popover-start');
        var popoverEndBtn = popoverContainer.querySelector('.popover-end');
        popoverStartBtn.onkeydown = function (event) {
            var action = _this.keyEventPlugin.constructor.getEventFullKey(event);
            // Check for Shift+Tab key
            if (action === 'shift.tab') {
                _this.bsPopoverDirective.hide();
                _this.setFocusToPopoverLink();
            }
        };
        popoverEndBtn.onkeydown = function (event) {
            var action = _this.keyEventPlugin.constructor.getEventFullKey(event);
            // Check for Tab key
            if (action === 'tab') {
                _this.bsPopoverDirective.hide();
                _this.setFocusToPopoverLink();
            }
        };
        //Whenever autoclose property is set to 'always', adding the onclick listener to the popover container to close the popover.
        if (this.autoclose === AUTOCLOSE_TYPE.ALWAYS) {
            popoverContainer.onclick = function () { return _this.close(); };
        }
        setAttr(popoverContainer, 'tabindex', 0);
        setTimeout(function () { return popoverStartBtn.focus(); }, 50);
        // Adjusting popover position if the popover placement is top or bottom
        setTimeout(function () {
            if (!_this.adaptiveposition) {
                _this.calculatePopoverPostion(popoverContainer);
            }
            // triggering onload and onshow events after popover content has rendered
            _this.triggerPopoverEvents();
        });
    };
    PopoverComponent.prototype.triggerPopoverEvents = function () {
        var _this = this;
        if (this.contentsource === 'partial') {
            var cancelSubscription_1 = this.app.subscribe('partialLoaded', function (data) {
                var parEle = _this.partialRef.nativeElement;
                var partialScope;
                if (parEle) {
                    partialScope = parEle.widget;
                    _this.Widgets = partialScope.Widgets;
                    _this.Variables = partialScope.Variables;
                    _this.Actions = partialScope.Actions;
                    _this.invokeEventCallback('load');
                    _this.invokeEventCallback('show', { $event: { type: 'show' } });
                }
                cancelSubscription_1();
            });
        }
        else {
            this.Widgets = this.viewParent.Widgets;
            this.Variables = this.viewParent.Variables;
            this.Actions = this.viewParent.Actions;
            this.invokeEventCallback('show', { $event: { type: 'show' } });
        }
    };
    PopoverComponent.prototype.hidePopover = function () {
        var _this = this;
        this.closePopoverTimeout = setTimeout(function () { return _this.isOpen = false; }, 500);
    };
    PopoverComponent.prototype.showPopover = function () {
        this.bsPopoverDirective.show();
    };
    PopoverComponent.prototype.onPopoverAnchorKeydown = function ($event) {
        // if there is no content available, the popover should not open through enter key. So checking whether the canPopoverOpen flag is true or not.
        if (!this.canPopoverOpen) {
            return;
        }
        var action = this.keyEventPlugin.constructor.getEventFullKey(event);
        if (action === 'enter') {
            $event.stopPropagation();
            this.showPopover();
        }
    };
    PopoverComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'class' || key === 'tabindex') {
            return;
        }
        if (key === 'contentsource') {
            // if there is no partial content available, the popover should not open
            if (this.contentsource === 'partial' && !this.content) {
                this.canPopoverOpen = false;
            }
        }
        if (key === 'content' && nv) {
            this.canPopoverOpen = true;
        }
        if (key === 'autoclose') {
            this.outsideclick = (nv === AUTOCLOSE_TYPE.OUTSIDECLICK || nv === AUTOCLOSE_TYPE.ALWAYS) ? true : false;
        }
        _super.prototype.onPropertyChange.call(this, key, nv, ov);
    };
    PopoverComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.event = eventsMap[this.interaction];
    };
    PopoverComponent.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        styler(this.anchorRef.nativeElement, this);
        this.containerTarget = getContainerTargetClass(this.nativeElement);
        var parentElemPopover = $(this.nativeElement).parents();
        if (parentElemPopover.closest('[wmTable]').length ||
            parentElemPopover.closest('[wmtabs]').length ||
            parentElemPopover.closest('modal-container').length) {
            this.adaptiveposition = false;
        }
    };
    var PopoverComponent_1;
    PopoverComponent.initializeProps = registerProps();
    PopoverComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: App },
        { type: undefined, decorators: [{ type: Inject, args: [EVENT_MANAGER_PLUGINS,] }] }
    ]; };
    __decorate([
        ViewChild(PopoverDirective),
        __metadata("design:type", Object)
    ], PopoverComponent.prototype, "bsPopoverDirective", void 0);
    __decorate([
        ViewChild('anchor', { static: true }),
        __metadata("design:type", ElementRef)
    ], PopoverComponent.prototype, "anchorRef", void 0);
    __decorate([
        ContentChild(TemplateRef),
        __metadata("design:type", Object)
    ], PopoverComponent.prototype, "popoverTemplate", void 0);
    __decorate([
        ContentChild('partial'),
        __metadata("design:type", Object)
    ], PopoverComponent.prototype, "partialRef", void 0);
    PopoverComponent = PopoverComponent_1 = __decorate([
        Component({
            selector: 'wm-popover',
            template: "<a [popover]=\"popoverTemplate\"\n    [popoverContext]=\"context\"\n    [popoverTitle]=\"title\"\n    [placement]=\"popoverplacement\"\n    [outsideClick]=\"outsideclick\"\n    [triggers]=\"event\"\n    (onShown)=\"onShown()\"\n    (onHidden)=\"onHidden()\"\n    containerClass=\"app-popover animated {{contentanimation}} {{popoverContainerCls}}\"\n    [container]=\"containerTarget || 'body'\"\n    [isOpen]=\"isOpen\"\n    [tabindex]=\"tabindex\"\n    [name]=\"name\"\n    [class]=\"class\"\n    [ngClass]=\"{'disable-popover': !canPopoverOpen}\"\n    [adaptivePosition]=\"adaptiveposition\"\n    #anchor\n    name.bind=\"name\"\n    wmAnchor\n    animation.bind=\"animation\"\n    badgevalue.bind=\"badgevalue\"\n    caption.bind=\"caption\"\n    encodeurl.bind=\"encodeurl\"\n    hint.bind=\"hint\"\n    hyperlink.bind=\"hyperlink\"\n    iconurl.bind=\"iconurl\"\n    iconwidth.bind=\"iconwidth\"\n    iconheight.bind=\"iconheight\"\n    iconmargin.bind=\"iconmargin\"\n    iconclass.bind=\"iconclass\"\n    iconposition.bind=\"iconposition\"\n    shortcutkey.bind=\"shortcutkey\"\n    tabindex.bind=\"tabindex\"\n    (keydown)=\"onPopoverAnchorKeydown($event)\">\n</a>\n",
            providers: [
                provideAsWidgetRef(PopoverComponent_1)
            ]
        }),
        __param(2, Inject(EVENT_MANAGER_PLUGINS)),
        __metadata("design:paramtypes", [Injector, App, Object])
    ], PopoverComponent);
    return PopoverComponent;
}(StylableComponent));

var components = [
    PopoverComponent
];
var PopoverModule = /** @class */ (function () {
    function PopoverModule() {
    }
    PopoverModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                BasicModule,
                PopoverModule$1,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], PopoverModule);
    return PopoverModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { PopoverModule, PopoverComponent as ɵa, registerProps as ɵb };
//# sourceMappingURL=index.js.map
