import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, Directive, Component, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_BOOLEAN, PROP_STRING, updateDeviceView, provideAsWidgetRef, StylableComponent, styler, PROP_NUMBER, APPLY_STYLES_TYPE, PullToRefresh, WmComponentsModule } from '@wm/components/base';
import { Title } from '@angular/platform-browser';
import { EventNotifier, Viewport, isMobileApp, switchClass, App } from '@wm/core';

var registerProps = function () {
    register('wm-page', new Map([
        ['cache', __assign({ value: false }, PROP_BOOLEAN)],
        ['refreshdataonattach', __assign({ value: true }, PROP_BOOLEAN)],
        ['pagetitle', PROP_STRING]
    ]));
};

var DEFAULT_CLS = 'app-page container';
var WIDGET_CONFIG = { widgetType: 'wm-page', hostClass: DEFAULT_CLS };
var PageDirective = /** @class */ (function (_super) {
    __extends(PageDirective, _super);
    function PageDirective(inj, titleService, viewport) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.titleService = titleService;
        _this.viewport = viewport;
        _this._eventNotifier = new EventNotifier(false);
        _this.refreshdataonattach = true;
        return _this;
    }
    PageDirective_1 = PageDirective;
    PageDirective.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'pagetitle') {
            this.titleService.setTitle(nv);
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    /**
     * A child component can notify page using this method. Notified event will be passed to
     * subscribed children only after page initialization.
     *
     * @param {string} eventName
     * @param data
     */
    PageDirective.prototype.notify = function (eventName) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        this._eventNotifier.notify.apply(this._eventNotifier, arguments);
    };
    /**
     * The main purpose of this function is to provide communication between page children objects.
     * Child component can subscribe for an event that will be emitted by another child component.
     *
     * @param eventName
     * @param {(data: any) => void} callback
     * @returns {any}
     */
    PageDirective.prototype.subscribe = function (eventName, callback) {
        return this._eventNotifier.subscribe(eventName, callback);
    };
    PageDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        setTimeout(function () {
            _this._eventNotifier.start();
            updateDeviceView(_this.nativeElement, _this.getAppInstance().isTabletApplicationType);
        }, 1);
        this.registerDestroyListener(this.viewport.subscribe("resize" /* RESIZE */, function (args) {
            _this.invokeEventCallback('resize', { $event: args.$event, widget: _this, data: args.data });
        }));
        this.registerDestroyListener(this.viewport.subscribe("orientationchange" /* ORIENTATION_CHANGE */, function (args) {
            _this.invokeEventCallback('orientationchange', { $event: args.$event, widget: _this, data: args.data });
        }));
    };
    PageDirective.prototype.ngOnAttach = function () {
        this.titleService.setTitle(this.pagetitle);
        this.invokeEventCallback('attach', { widget: this });
        this.notify('attach', {
            refreshData: this.refreshdataonattach
        });
    };
    PageDirective.prototype.ngOnDetach = function () {
        this.invokeEventCallback('detach', { widget: this });
        this.notify('detach');
    };
    PageDirective.prototype.ngOnDestroy = function () {
        this.invokeEventCallback('destroy', { widget: this });
        this._eventNotifier.destroy();
    };
    var PageDirective_1;
    PageDirective.initializeProps = registerProps();
    PageDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: Title },
        { type: Viewport }
    ]; };
    PageDirective = PageDirective_1 = __decorate([
        Directive({
            selector: '[wmPage]',
            providers: [
                provideAsWidgetRef(PageDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, Title, Viewport])
    ], PageDirective);
    return PageDirective;
}(StylableComponent));

var registerProps$1 = function () {
    register('wm-content', new Map([
        ['name', PROP_STRING],
        ['class', PROP_STRING]
    ]));
};

var DEFAULT_CLS$1 = 'app-content clearfix';
var WIDGET_CONFIG$1 = { widgetType: 'wm-content', hostClass: DEFAULT_CLS$1 };
var ContentComponent = /** @class */ (function (_super) {
    __extends(ContentComponent, _super);
    function ContentComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        styler(_this.nativeElement, _this);
        return _this;
    }
    ContentComponent_1 = ContentComponent;
    var ContentComponent_1;
    ContentComponent.initializeProps = registerProps$1();
    ContentComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    ContentComponent = ContentComponent_1 = __decorate([
        Component({
            selector: '[wmContent]',
            template: "<div class=\"row app-content-row clearfix\">\n    <ng-content></ng-content>\n</div>",
            providers: [
                provideAsWidgetRef(ContentComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], ContentComponent);
    return ContentComponent;
}(StylableComponent));

var registerProps$2 = function () {
    register('wm-page-content', new Map([
        ['class', PROP_STRING],
        ['columnwidth', PROP_NUMBER],
        ['content', PROP_STRING],
        ['name', PROP_STRING],
        ['pulltorefresh', __assign({ value: isMobileApp() }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$2 = 'app-page-content app-content-column';
var WIDGET_CONFIG$2 = { widgetType: 'wm-page-content', hostClass: DEFAULT_CLS$2 };
var PageContentComponent = /** @class */ (function (_super) {
    __extends(PageContentComponent, _super);
    function PageContentComponent(inj, app) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$2) || this;
        _this.app = app;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        _this.registerDestroyListener(_this.app.subscribe('pullToRefresh:enable', function () {
            _this.childPullToRefresh = true;
            _this.initPullToRefresh();
        }));
        return _this;
    }
    PageContentComponent_1 = PageContentComponent;
    PageContentComponent.prototype.onPropertyChange = function (key, nv, ov) {
        var _this = this;
        if (key === 'columnwidth') {
            switchClass(this.nativeElement, "col-md-" + nv + " col-sm-" + nv, "col-md-" + ov + " col-sm-" + ov);
        }
        else if (key === 'pulltorefresh' && nv) {
            // creating instance after timeout as the smoothscroll styles where getting added on pull refresh-container
            setTimeout(function () {
                _this.initPullToRefresh();
            });
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    // when list component is ready, pulltorefresh instance is created and this appends pullToRefresh element on the page content.
    PageContentComponent.prototype.initPullToRefresh = function () {
        var _this = this;
        var hasPullToRefreshEvent = this.hasEventCallback('pulltorefresh');
        if (!this.pullToRefreshIns && (this.childPullToRefresh || hasPullToRefreshEvent) && this.pulltorefresh) {
            this.pullToRefreshIns = new PullToRefresh($(this.nativeElement), this.app, function () {
                if (hasPullToRefreshEvent) {
                    _this.invokeEventCallback('pulltorefresh');
                }
                else {
                    _this.app.notify('pulltorefresh');
                }
            });
            this.registerDestroyListener(function () {
                if (_this.pullToRefreshIns.cancelSubscription) {
                    _this.pullToRefreshIns.cancelSubscription();
                }
            });
        }
    };
    var PageContentComponent_1;
    PageContentComponent.initializeProps = registerProps$2();
    PageContentComponent.ctorParameters = function () { return [
        { type: Injector },
        { type: App }
    ]; };
    PageContentComponent = PageContentComponent_1 = __decorate([
        Component({
            selector: '[wmPageContent]',
            template: "<ng-content></ng-content>",
            providers: [
                provideAsWidgetRef(PageContentComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, App])
    ], PageContentComponent);
    return PageContentComponent;
}(StylableComponent));

var components = [
    PageDirective,
    ContentComponent,
    PageContentComponent
];
var PageModule = /** @class */ (function () {
    function PageModule() {
    }
    PageModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], PageModule);
    return PageModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { PageDirective, PageModule, registerProps as ɵa, ContentComponent as ɵb, registerProps$1 as ɵc, PageContentComponent as ɵd, registerProps$2 as ɵe };
//# sourceMappingURL=index.js.map
