import { __decorate, __metadata } from 'tslib';
import { Injector, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_NUMBER, StylableComponent, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { NavigationStart, Router } from '@angular/router';
import { $appDigest, addClass, switchClass, removeClass, toggleClass, App } from '@wm/core';
import { PageDirective } from '@wm/components/page';
import { SwipeAnimation } from '@swipey';

const registerProps = () => {
    register('wm-left-panel', new Map([
        ['animation', Object.assign({ value: 'slide-in' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['columnwidth', Object.assign({ value: 3 }, PROP_NUMBER)],
        ['content', PROP_STRING],
        ['gestures', Object.assign({ value: 'on' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['xscolumnwidth', Object.assign({ value: 10 }, PROP_NUMBER)]
    ]));
};

class LeftPanelAnimator extends SwipeAnimation {
    constructor(leftPanel) {
        super();
        this.leftPanel = leftPanel;
        this.init(this.leftPanel.$ele, this.leftPanel.$page);
    }
    bounds(e, length) {
        let offset = 0;
        this.setGesturesEnabled(this.leftPanel.isGesturesEnabled());
        if (!this._width) {
            this._pageContainerWidth = this.leftPanel.$page.width();
            this._leftPanelWidth = this.leftPanel.$ele.width();
            this._maxX = this._leftPanelWidth / this._pageContainerWidth * 100;
            this._width = this.leftPanel.animation === AnimationType.SLIDE_IN ? this._pageContainerWidth : this._leftPanelWidth;
        }
        this._expanded = this.leftPanel.expanded;
        if (this._expanded) {
            return {
                'center': this._leftPanelWidth,
                'lower': -(this._leftPanelWidth - offset)
            };
        }
        if (this.leftPanel.app.isTabletApplicationType) {
            offset = 53.32;
        }
        if (e && (e.pageX || (e.touches && e.touches[0].pageX)) > offset + 100) {
            return {
                strict: true
            };
        }
        return {
            'center': 0,
            'upper': this._leftPanelWidth - offset
        };
    }
    context() {
        return {
            'w': this._width,
            'pageW': this._pageContainerWidth,
            'leftW': this._leftPanelWidth,
            'maxX': this._maxX,
            'limit': (min, v, max) => {
                if (v < min) {
                    return min;
                }
                if (v > max) {
                    return max;
                }
                return v;
            }
        };
    }
    animation() {
        this._$animatedElements = this.leftPanel.$ele;
        if (this.leftPanel.animation === AnimationType.SLIDE_IN) {
            this._$animatedElements = this._$animatedElements.add(this.leftPanel.$page);
            if (this.leftPanel.app.isTabletApplicationType) {
                return [
                    {
                        'target': this.leftPanel.$ele,
                        'css': {
                            'transform': 'translate3d(${{ limit(-100, -($d * 100 / leftW), 0) + \'%\' }}, 0, 0)'
                        }
                    },
                    {
                        'target': this.leftPanel.$page,
                        'css': {
                            'transform': 'translate3d(${{ (($d) * 100 / pageW) + \'%\' }}, 0, 0)',
                            'width': '${{ (pageW - $d) + \'px\' }}',
                            'z-index': 101
                        }
                    }
                ];
            }
            return [
                {
                    'target': this.leftPanel.$ele,
                    'css': {
                        'transform': 'translate3d(-100%, 0, 0)',
                        'opacity': 1,
                        'z-index': 101
                    }
                },
                {
                    'target': this.leftPanel.$page,
                    'css': {
                        'transform': 'translate3d(${{ limit( 0, ((($D + $d) * 100 / w)), maxX ) + \'%\' }}, 0, 0)',
                        'opacity': 1,
                        'z-index': 101
                    }
                }
            ];
        }
        else {
            return {
                'transform': 'translate3d(${{ limit( -100, ((($D + $d) * 100 / w) - 100), 0 ) + \'%\'}}, 0, 0)',
                'opacity': 1,
                'z-index': 101
            };
        }
    }
    onLower() {
        this._expanded = false;
        this.leftPanel.collapse();
        this.resetTransition();
        $appDigest();
    }
    onUpper() {
        this._expanded = true;
        this.leftPanel.expand();
        this.resetTransition();
        $appDigest();
    }
    resetTransition() {
        if (this._$animatedElements) {
            /*
             * This timeout is for preventing UI flicker at the end of animation.
             */
            setTimeout(() => {
                this._$animatedElements.css({
                    'transform': '',
                    'opacity': '',
                    'z-index': '',
                    'width': ''
                });
            }, 100);
        }
    }
}

var LeftPanelDirective_1;
const DEFAULT_CLS = 'app-left-panel left-panel-collapsed';
const WIDGET_CONFIG = {
    widgetType: 'wm-left-panel',
    hostClass: DEFAULT_CLS
};
var AnimationType;
(function (AnimationType) {
    AnimationType["SLIDE_IN"] = "slide-in";
    AnimationType["SLIDE_OVER"] = "slide-over";
})(AnimationType || (AnimationType = {}));
let LeftPanelDirective = LeftPanelDirective_1 = class LeftPanelDirective extends StylableComponent {
    constructor(app, page, inj, router) {
        super(inj, WIDGET_CONFIG);
        this.app = app;
        this.page = page;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.CONTAINER);
        this.$ele = this.$element;
        this.$page = page.$element;
        addClass(this.$page[0], 'left-panel-collapsed-container');
        if (this.columnwidth) {
            this.setLeftPanelWidth(['md', 'sm'], this.columnwidth);
        }
        if (this.app.isTabletApplicationType) {
            addClass(this.nativeElement, 'wm-tablet-app-left-panel');
        }
        const onRouteChange = router.events.subscribe(e => {
            if (e instanceof NavigationStart) {
                this.collapse();
            }
        });
        this.registerDestroyListener(() => onRouteChange.unsubscribe());
    }
    collapse() {
        addClass(this.nativeElement, 'swipee-transition');
        switchClass(this.nativeElement, 'left-panel-collapsed', 'left-panel-expanded');
        this.expanded = false;
        switchClass(this.$page[0], 'left-panel-collapsed-container', 'left-panel-expanded-container');
        if (this.animation === AnimationType.SLIDE_IN) {
            this.setPageWidthAndPosition(['md', 'sm'], null, this.columnwidth);
            this.setPageWidthAndPosition(['xs'], null, this.xscolumnwidth);
        }
        if (this._destroyCollapseActionListener) {
            this._destroyCollapseActionListener();
        }
        this.page.notify('wmLeftPanel:collapse');
    }
    expand() {
        removeClass(this.nativeElement, 'swipee-transition');
        switchClass(this.nativeElement, 'left-panel-expanded', 'left-panel-collapsed');
        this.expanded = true;
        if (!(this.app.isTabletApplicationType && this.animation === AnimationType.SLIDE_IN)) {
            this._destroyCollapseActionListener = this.listenForCollapseAction();
        }
        switchClass(this.$page[0], 'left-panel-expanded-container', 'left-panel-collapsed-container');
        if (this.animation === AnimationType.SLIDE_IN) {
            this.setPageWidthAndPosition(['md', 'sm'], this.columnwidth);
            this.setPageWidthAndPosition(['xs'], this.xscolumnwidth);
        }
        this.page.notify('wmLeftPanel:expand');
    }
    isGesturesEnabled() {
        return this.gestures === 'on';
    }
    isVisible() {
        return this.expanded;
    }
    onPropertyChange(key, nv, ov) {
        switch (key) {
            case 'animation':
                if (nv === AnimationType.SLIDE_IN) {
                    removeClass(this.$page[0], 'slide-over-left-panel-container');
                    addClass(this.$page[0], 'slide-in-left-panel-container');
                    this.setPageWidthAndPosition(['md', 'sm'], this.columnwidth);
                    this.setPageWidthAndPosition(['xs'], this.xscolumnwidth);
                }
                else if (nv === AnimationType.SLIDE_OVER) {
                    removeClass(this.$page[0], 'slide-in-left-panel-container');
                    addClass(this.$page[0], 'slide-over-left-panel-container');
                }
                this._leftPanelAnimator = new LeftPanelAnimator(this);
                switchClass(this.nativeElement, nv, ov);
                break;
            case 'columnwidth':
                this.setLeftPanelWidth(['md', 'sm'], nv, ov);
                if (this.animation === AnimationType.SLIDE_IN) {
                    this.setPageWidthAndPosition(['md', 'sm'], nv, ov);
                }
                break;
            case 'expanded':
                toggleClass(this.nativeElement, 'left-panel-expanded', nv);
                toggleClass(this.nativeElement, 'left-panel-collapsed', !nv);
                break;
            case 'xscolumnwidth':
                this.setLeftPanelWidth(['xs'], nv, ov);
                if (this.animation === AnimationType.SLIDE_IN) {
                    this.setPageWidthAndPosition(['xs'], nv, ov);
                }
                break;
            default:
                super.onPropertyChange(key, nv, ov);
        }
    }
    toggle() {
        if (this.app.isTabletApplicationType) {
            setTimeout(() => {
                this.expanded ? this.collapse() : this.expand();
            }, 50);
        }
        else {
            this.$ele.swipeAnimation(this.expanded ? 'gotoLower' : 'gotoUpper');
        }
    }
    listenForCollapseAction() {
        const eventName = 'click.leftNavToggle';
        let skipEvent = false;
        this.$ele.on(eventName, () => {
            skipEvent = true;
        });
        this.$page.on(eventName, () => {
            if (!skipEvent) {
                this.toggle();
            }
            skipEvent = false;
        });
        return () => {
            this.$ele.off(eventName);
            this.$page.off(eventName);
        };
    }
    setLeftPanelWidth(devices, newVal, oldVal) {
        devices.forEach(device => {
            if (newVal) {
                addClass(this.nativeElement, `col-${device}-${newVal}`);
            }
            if (oldVal) {
                removeClass(this.nativeElement, `col-${device}-${oldVal}`);
            }
        });
    }
    setPageWidthAndPosition(devices, newVal, oldVal) {
        devices.forEach(device => {
            if (newVal) {
                addClass(this.$page[0], `left-panel-container-${device}-${12 - newVal}`);
            }
            if (oldVal) {
                removeClass(this.$page[0], `left-panel-container-${device}-${12 - oldVal}`);
            }
        });
    }
};
LeftPanelDirective.initializeProps = registerProps();
LeftPanelDirective.ctorParameters = () => [
    { type: App },
    { type: PageDirective },
    { type: Injector },
    { type: Router }
];
LeftPanelDirective = LeftPanelDirective_1 = __decorate([
    Directive({
        selector: '[wmLeftPanel]',
        providers: [
            provideAsWidgetRef(LeftPanelDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [App, PageDirective, Injector, Router])
], LeftPanelDirective);

const components = [
    LeftPanelDirective
];
let LeftPanelModule = class LeftPanelModule {
};
LeftPanelModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], LeftPanelModule);

/**
 * Generated bundle index. Do not edit.
 */

export { AnimationType, LeftPanelDirective, LeftPanelModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
