import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { register, PROP_STRING, PROP_NUMBER, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { NavigationStart, Router } from '@angular/router';
import { $appDigest, addClass, switchClass, removeClass, toggleClass, App } from '@wm/core';
import { PageDirective } from '@wm/components/page';
import { SwipeAnimation } from '@swipey';

var registerProps = function () {
    register('wm-left-panel', new Map([
        ['animation', __assign({ value: 'slide-in' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['columnwidth', __assign({ value: 3 }, PROP_NUMBER)],
        ['content', PROP_STRING],
        ['gestures', __assign({ value: 'on' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['xscolumnwidth', __assign({ value: 10 }, PROP_NUMBER)]
    ]));
};

var LeftPanelAnimator = /** @class */ (function (_super) {
    __extends(LeftPanelAnimator, _super);
    function LeftPanelAnimator(leftPanel) {
        var _this = _super.call(this) || this;
        _this.leftPanel = leftPanel;
        _this.init(_this.leftPanel.$ele, _this.leftPanel.$page);
        return _this;
    }
    LeftPanelAnimator.prototype.bounds = function (e, length) {
        var offset = 0;
        this.setGesturesEnabled(this.leftPanel.isGesturesEnabled());
        if (!this._width) {
            this._pageContainerWidth = this.leftPanel.$page.width();
            this._leftPanelWidth = this.leftPanel.$ele.width();
            this._maxX = this._leftPanelWidth / this._pageContainerWidth * 100;
            this._width = this.leftPanel.animation === AnimationType.SLIDE_IN ? this._pageContainerWidth : this._leftPanelWidth;
        }
        this._expanded = this.leftPanel.expanded;
        if (this._expanded) {
            return {
                'center': this._leftPanelWidth,
                'lower': -(this._leftPanelWidth - offset)
            };
        }
        if (this.leftPanel.app.isTabletApplicationType) {
            offset = 53.32;
        }
        if (e && (e.pageX || (e.touches && e.touches[0].pageX)) > offset + 100) {
            return {
                strict: true
            };
        }
        return {
            'center': 0,
            'upper': this._leftPanelWidth - offset
        };
    };
    LeftPanelAnimator.prototype.context = function () {
        return {
            'w': this._width,
            'pageW': this._pageContainerWidth,
            'leftW': this._leftPanelWidth,
            'maxX': this._maxX,
            'limit': function (min, v, max) {
                if (v < min) {
                    return min;
                }
                if (v > max) {
                    return max;
                }
                return v;
            }
        };
    };
    LeftPanelAnimator.prototype.animation = function () {
        this._$animatedElements = this.leftPanel.$ele;
        if (this.leftPanel.animation === AnimationType.SLIDE_IN) {
            this._$animatedElements = this._$animatedElements.add(this.leftPanel.$page);
            if (this.leftPanel.app.isTabletApplicationType) {
                return [
                    {
                        'target': this.leftPanel.$ele,
                        'css': {
                            'transform': 'translate3d(${{ limit(-100, -($d * 100 / leftW), 0) + \'%\' }}, 0, 0)'
                        }
                    },
                    {
                        'target': this.leftPanel.$page,
                        'css': {
                            'transform': 'translate3d(${{ (($d) * 100 / pageW) + \'%\' }}, 0, 0)',
                            'width': '${{ (pageW - $d) + \'px\' }}',
                            'z-index': 101
                        }
                    }
                ];
            }
            return [
                {
                    'target': this.leftPanel.$ele,
                    'css': {
                        'transform': 'translate3d(-100%, 0, 0)',
                        'opacity': 1,
                        'z-index': 101
                    }
                },
                {
                    'target': this.leftPanel.$page,
                    'css': {
                        'transform': 'translate3d(${{ limit( 0, ((($D + $d) * 100 / w)), maxX ) + \'%\' }}, 0, 0)',
                        'opacity': 1,
                        'z-index': 101
                    }
                }
            ];
        }
        else {
            return {
                'transform': 'translate3d(${{ limit( -100, ((($D + $d) * 100 / w) - 100), 0 ) + \'%\'}}, 0, 0)',
                'opacity': 1,
                'z-index': 101
            };
        }
    };
    LeftPanelAnimator.prototype.onLower = function () {
        this._expanded = false;
        this.leftPanel.collapse();
        this.resetTransition();
        $appDigest();
    };
    LeftPanelAnimator.prototype.onUpper = function () {
        this._expanded = true;
        this.leftPanel.expand();
        this.resetTransition();
        $appDigest();
    };
    LeftPanelAnimator.prototype.resetTransition = function () {
        var _this = this;
        if (this._$animatedElements) {
            /*
             * This timeout is for preventing UI flicker at the end of animation.
             */
            setTimeout(function () {
                _this._$animatedElements.css({
                    'transform': '',
                    'opacity': '',
                    'z-index': '',
                    'width': ''
                });
            }, 100);
        }
    };
    return LeftPanelAnimator;
}(SwipeAnimation));

var DEFAULT_CLS = 'app-left-panel left-panel-collapsed';
var WIDGET_CONFIG = {
    widgetType: 'wm-left-panel',
    hostClass: DEFAULT_CLS
};
var AnimationType;
(function (AnimationType) {
    AnimationType["SLIDE_IN"] = "slide-in";
    AnimationType["SLIDE_OVER"] = "slide-over";
})(AnimationType || (AnimationType = {}));
var LeftPanelDirective = /** @class */ (function (_super) {
    __extends(LeftPanelDirective, _super);
    function LeftPanelDirective(app, page, inj, router) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.app = app;
        _this.page = page;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.CONTAINER);
        _this.$ele = _this.$element;
        _this.$page = page.$element;
        addClass(_this.$page[0], 'left-panel-collapsed-container');
        if (_this.columnwidth) {
            _this.setLeftPanelWidth(['md', 'sm'], _this.columnwidth);
        }
        if (_this.app.isTabletApplicationType) {
            addClass(_this.nativeElement, 'wm-tablet-app-left-panel');
        }
        var onRouteChange = router.events.subscribe(function (e) {
            if (e instanceof NavigationStart) {
                _this.collapse();
            }
        });
        _this.registerDestroyListener(function () { return onRouteChange.unsubscribe(); });
        return _this;
    }
    LeftPanelDirective_1 = LeftPanelDirective;
    LeftPanelDirective.prototype.collapse = function () {
        addClass(this.nativeElement, 'swipee-transition');
        switchClass(this.nativeElement, 'left-panel-collapsed', 'left-panel-expanded');
        this.expanded = false;
        switchClass(this.$page[0], 'left-panel-collapsed-container', 'left-panel-expanded-container');
        if (this.animation === AnimationType.SLIDE_IN) {
            this.setPageWidthAndPosition(['md', 'sm'], null, this.columnwidth);
            this.setPageWidthAndPosition(['xs'], null, this.xscolumnwidth);
        }
        if (this._destroyCollapseActionListener) {
            this._destroyCollapseActionListener();
        }
        this.page.notify('wmLeftPanel:collapse');
    };
    LeftPanelDirective.prototype.expand = function () {
        removeClass(this.nativeElement, 'swipee-transition');
        switchClass(this.nativeElement, 'left-panel-expanded', 'left-panel-collapsed');
        this.expanded = true;
        if (!(this.app.isTabletApplicationType && this.animation === AnimationType.SLIDE_IN)) {
            this._destroyCollapseActionListener = this.listenForCollapseAction();
        }
        switchClass(this.$page[0], 'left-panel-expanded-container', 'left-panel-collapsed-container');
        if (this.animation === AnimationType.SLIDE_IN) {
            this.setPageWidthAndPosition(['md', 'sm'], this.columnwidth);
            this.setPageWidthAndPosition(['xs'], this.xscolumnwidth);
        }
        this.page.notify('wmLeftPanel:expand');
    };
    LeftPanelDirective.prototype.isGesturesEnabled = function () {
        return this.gestures === 'on';
    };
    LeftPanelDirective.prototype.isVisible = function () {
        return this.expanded;
    };
    LeftPanelDirective.prototype.onPropertyChange = function (key, nv, ov) {
        switch (key) {
            case 'animation':
                if (nv === AnimationType.SLIDE_IN) {
                    removeClass(this.$page[0], 'slide-over-left-panel-container');
                    addClass(this.$page[0], 'slide-in-left-panel-container');
                    this.setPageWidthAndPosition(['md', 'sm'], this.columnwidth);
                    this.setPageWidthAndPosition(['xs'], this.xscolumnwidth);
                }
                else if (nv === AnimationType.SLIDE_OVER) {
                    removeClass(this.$page[0], 'slide-in-left-panel-container');
                    addClass(this.$page[0], 'slide-over-left-panel-container');
                }
                this._leftPanelAnimator = new LeftPanelAnimator(this);
                switchClass(this.nativeElement, nv, ov);
                break;
            case 'columnwidth':
                this.setLeftPanelWidth(['md', 'sm'], nv, ov);
                if (this.animation === AnimationType.SLIDE_IN) {
                    this.setPageWidthAndPosition(['md', 'sm'], nv, ov);
                }
                break;
            case 'expanded':
                toggleClass(this.nativeElement, 'left-panel-expanded', nv);
                toggleClass(this.nativeElement, 'left-panel-collapsed', !nv);
                break;
            case 'xscolumnwidth':
                this.setLeftPanelWidth(['xs'], nv, ov);
                if (this.animation === AnimationType.SLIDE_IN) {
                    this.setPageWidthAndPosition(['xs'], nv, ov);
                }
                break;
            default:
                _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    LeftPanelDirective.prototype.toggle = function () {
        var _this = this;
        if (this.app.isTabletApplicationType) {
            setTimeout(function () {
                _this.expanded ? _this.collapse() : _this.expand();
            }, 50);
        }
        else {
            this.$ele.swipeAnimation(this.expanded ? 'gotoLower' : 'gotoUpper');
        }
    };
    LeftPanelDirective.prototype.listenForCollapseAction = function () {
        var _this = this;
        var eventName = 'click.leftNavToggle';
        var skipEvent = false;
        this.$ele.on(eventName, function () {
            skipEvent = true;
        });
        this.$page.on(eventName, function () {
            if (!skipEvent) {
                _this.toggle();
            }
            skipEvent = false;
        });
        return function () {
            _this.$ele.off(eventName);
            _this.$page.off(eventName);
        };
    };
    LeftPanelDirective.prototype.setLeftPanelWidth = function (devices, newVal, oldVal) {
        var _this = this;
        devices.forEach(function (device) {
            if (newVal) {
                addClass(_this.nativeElement, "col-" + device + "-" + newVal);
            }
            if (oldVal) {
                removeClass(_this.nativeElement, "col-" + device + "-" + oldVal);
            }
        });
    };
    LeftPanelDirective.prototype.setPageWidthAndPosition = function (devices, newVal, oldVal) {
        var _this = this;
        devices.forEach(function (device) {
            if (newVal) {
                addClass(_this.$page[0], "left-panel-container-" + device + "-" + (12 - newVal));
            }
            if (oldVal) {
                removeClass(_this.$page[0], "left-panel-container-" + device + "-" + (12 - oldVal));
            }
        });
    };
    var LeftPanelDirective_1;
    LeftPanelDirective.initializeProps = registerProps();
    LeftPanelDirective.ctorParameters = function () { return [
        { type: App },
        { type: PageDirective },
        { type: Injector },
        { type: Router }
    ]; };
    LeftPanelDirective = LeftPanelDirective_1 = __decorate([
        Directive({
            selector: '[wmLeftPanel]',
            providers: [
                provideAsWidgetRef(LeftPanelDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [App, PageDirective, Injector, Router])
    ], LeftPanelDirective);
    return LeftPanelDirective;
}(StylableComponent));

var components = [
    LeftPanelDirective
];
var LeftPanelModule = /** @class */ (function () {
    function LeftPanelModule() {
    }
    LeftPanelModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], LeftPanelModule);
    return LeftPanelModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { AnimationType, LeftPanelDirective, LeftPanelModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
