import { __decorate, __param, __metadata } from 'tslib';
import { Injector, ElementRef, ChangeDetectorRef, Attribute, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_STRING, PROP_BOOLEAN, StylableComponent, styler, register, PROP_TYPE, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { setCSS, Viewport } from '@wm/core';

const prefabProps = new Map([
    ['animation', PROP_STRING],
    ['class', Object.assign({ value: '' }, PROP_STRING)],
    ['name', PROP_STRING],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
]);

var PrefabDirective_1;
const DEFAULT_CLS = 'app-prefab';
const registeredPropsSet = new Set();
let PrefabDirective = PrefabDirective_1 = class PrefabDirective extends StylableComponent {
    constructor(inj, elRef, cdr, prefabName) {
        const widgetType = `wm-prefab-${prefabName}`;
        const WIDGET_CONFIG = { widgetType, hostClass: DEFAULT_CLS };
        super(inj, WIDGET_CONFIG, new Promise(res => this.propsReady = res));
        this.prefabName = prefabName;
        this.widgetType = widgetType;
        this.name = elRef.nativeElement.getAttribute('name');
        styler(this.nativeElement, this);
        // Call on property change on name to set name attribute on element.
        this.registerReadyStateListener(() => {
            super.onPropertyChange('name', this.name);
        });
    }
    onStyleChange(key, nv, ov) {
        if (key === 'height') {
            setCSS(this.nativeElement, 'overflow', 'auto');
        }
    }
    setProps(config) {
        if (!config || !config.properties) {
            return;
        }
        if (!registeredPropsSet.has(this.widgetType)) {
            register(this.widgetType, this.prepareProps(config.properties));
        }
        this.propsReady();
    }
    handleEvent() {
        // do not call the super;
        // prevent events from getting registered
    }
    prepareProps(props = {}) {
        const propsMap = new Map(prefabProps);
        Object.entries(props).forEach(([k, v]) => {
            let type = PROP_TYPE.STRING;
            if (v.type === 'boolean') {
                type = PROP_TYPE.BOOLEAN;
            }
            else if (v.type === 'number') {
                type = PROP_TYPE.NUMBER;
            }
            else if (v.type !== 'string') {
                type = v.type;
            }
            // Do not set the 'bind:*' values
            propsMap.set(k, { type, value: _.startsWith(v.value, 'bind:') ? undefined : v.value });
        });
        registeredPropsSet.add(this.widgetType);
        return propsMap;
    }
    ngOnDestroy() {
        this.invokeEventCallback('destroy', { widget: this });
    }
};
PrefabDirective.ctorParameters = () => [
    { type: Injector },
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: undefined, decorators: [{ type: Attribute, args: ['prefabname',] }] }
];
PrefabDirective = PrefabDirective_1 = __decorate([
    Directive({
        selector: 'section[wmPrefab]',
        providers: [
            provideAsWidgetRef(PrefabDirective_1)
        ]
    }),
    __param(3, Attribute('prefabname')),
    __metadata("design:paramtypes", [Injector, ElementRef, ChangeDetectorRef, Object])
], PrefabDirective);

const registerProps = () => {
    register('wm-prefab-container', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ]));
};

var PrefabContainerDirective_1;
const DEFAULT_CLS$1 = 'app-prefab-container full-height';
const WIDGET_CONFIG = {
    widgetType: 'wm-prefab-container',
    hostClass: DEFAULT_CLS$1
};
let PrefabContainerDirective = PrefabContainerDirective_1 = class PrefabContainerDirective extends StylableComponent {
    constructor(inj, viewport) {
        super(inj, WIDGET_CONFIG);
        this.viewport = viewport;
        styler(this.nativeElement, this);
        this.registerDestroyListener(this.viewport.subscribe("resize" /* RESIZE */, data => this.callback('resize', data)));
        this.registerDestroyListener(this.viewport.subscribe("orientationchange" /* ORIENTATION_CHANGE */, data => this.callback('orientationchange', data)));
    }
    callback(eventName, locals) {
        locals = _.assign({ widget: this }, locals);
        this.invokeEventCallback(eventName, locals);
    }
    ngOnAttach() {
        this.callback('attach');
    }
    ngOnDetach() {
        this.callback('detach');
    }
    ngOnDestroy() {
        this.callback('destroy');
    }
};
PrefabContainerDirective.initializeProps = registerProps();
PrefabContainerDirective.ctorParameters = () => [
    { type: Injector },
    { type: Viewport }
];
PrefabContainerDirective = PrefabContainerDirective_1 = __decorate([
    Directive({
        selector: '[wmPrefabContainer]',
        providers: [
            provideAsWidgetRef(PrefabContainerDirective_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector, Viewport])
], PrefabContainerDirective);

const components = [
    PrefabDirective,
    PrefabContainerDirective
];
let PrefabModule = class PrefabModule {
};
PrefabModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], PrefabModule);

/**
 * Generated bundle index. Do not edit.
 */

export { PrefabContainerDirective, PrefabDirective, PrefabModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
