import { __assign, __extends, __read, __decorate, __param, __metadata, __spread } from 'tslib';
import { Injector, ElementRef, ChangeDetectorRef, Attribute, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PROP_STRING, PROP_BOOLEAN, styler, register, PROP_TYPE, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { setCSS, Viewport } from '@wm/core';

var prefabProps = new Map([
    ['animation', PROP_STRING],
    ['class', __assign({ value: '' }, PROP_STRING)],
    ['name', PROP_STRING],
    ['show', __assign({ value: true }, PROP_BOOLEAN)]
]);

var DEFAULT_CLS = 'app-prefab';
var registeredPropsSet = new Set();
var PrefabDirective = /** @class */ (function (_super) {
    __extends(PrefabDirective, _super);
    function PrefabDirective(inj, elRef, cdr, prefabName) {
        var _this = this;
        var widgetType = "wm-prefab-" + prefabName;
        var WIDGET_CONFIG = { widgetType: widgetType, hostClass: DEFAULT_CLS };
        _this = _super.call(this, inj, WIDGET_CONFIG, new Promise(function (res) { return _this.propsReady = res; })) || this;
        _this.prefabName = prefabName;
        _this.widgetType = widgetType;
        _this.name = elRef.nativeElement.getAttribute('name');
        styler(_this.nativeElement, _this);
        // Call on property change on name to set name attribute on element.
        _this.registerReadyStateListener(function () {
            _super.prototype.onPropertyChange.call(_this, 'name', _this.name);
        });
        return _this;
    }
    PrefabDirective_1 = PrefabDirective;
    PrefabDirective.prototype.onStyleChange = function (key, nv, ov) {
        if (key === 'height') {
            setCSS(this.nativeElement, 'overflow', 'auto');
        }
    };
    PrefabDirective.prototype.setProps = function (config) {
        if (!config || !config.properties) {
            return;
        }
        if (!registeredPropsSet.has(this.widgetType)) {
            register(this.widgetType, this.prepareProps(config.properties));
        }
        this.propsReady();
    };
    PrefabDirective.prototype.handleEvent = function () {
        // do not call the super;
        // prevent events from getting registered
    };
    PrefabDirective.prototype.prepareProps = function (props) {
        if (props === void 0) { props = {}; }
        var propsMap = new Map(prefabProps);
        Object.entries(props).forEach(function (_a) {
            var _b = __read(_a, 2), k = _b[0], v = _b[1];
            var type = PROP_TYPE.STRING;
            if (v.type === 'boolean') {
                type = PROP_TYPE.BOOLEAN;
            }
            else if (v.type === 'number') {
                type = PROP_TYPE.NUMBER;
            }
            else if (v.type !== 'string') {
                type = v.type;
            }
            // Do not set the 'bind:*' values
            propsMap.set(k, { type: type, value: _.startsWith(v.value, 'bind:') ? undefined : v.value });
        });
        registeredPropsSet.add(this.widgetType);
        return propsMap;
    };
    PrefabDirective.prototype.ngOnDestroy = function () {
        this.invokeEventCallback('destroy', { widget: this });
    };
    var PrefabDirective_1;
    PrefabDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: undefined, decorators: [{ type: Attribute, args: ['prefabname',] }] }
    ]; };
    PrefabDirective = PrefabDirective_1 = __decorate([
        Directive({
            selector: 'section[wmPrefab]',
            providers: [
                provideAsWidgetRef(PrefabDirective_1)
            ]
        }),
        __param(3, Attribute('prefabname')),
        __metadata("design:paramtypes", [Injector, ElementRef, ChangeDetectorRef, Object])
    ], PrefabDirective);
    return PrefabDirective;
}(StylableComponent));

var registerProps = function () {
    register('wm-prefab-container', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
    ]));
};

var DEFAULT_CLS$1 = 'app-prefab-container full-height';
var WIDGET_CONFIG = {
    widgetType: 'wm-prefab-container',
    hostClass: DEFAULT_CLS$1
};
var PrefabContainerDirective = /** @class */ (function (_super) {
    __extends(PrefabContainerDirective, _super);
    function PrefabContainerDirective(inj, viewport) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.viewport = viewport;
        styler(_this.nativeElement, _this);
        _this.registerDestroyListener(_this.viewport.subscribe("resize" /* RESIZE */, function (data) { return _this.callback('resize', data); }));
        _this.registerDestroyListener(_this.viewport.subscribe("orientationchange" /* ORIENTATION_CHANGE */, function (data) { return _this.callback('orientationchange', data); }));
        return _this;
    }
    PrefabContainerDirective_1 = PrefabContainerDirective;
    PrefabContainerDirective.prototype.callback = function (eventName, locals) {
        locals = _.assign({ widget: this }, locals);
        this.invokeEventCallback(eventName, locals);
    };
    PrefabContainerDirective.prototype.ngOnAttach = function () {
        this.callback('attach');
    };
    PrefabContainerDirective.prototype.ngOnDetach = function () {
        this.callback('detach');
    };
    PrefabContainerDirective.prototype.ngOnDestroy = function () {
        this.callback('destroy');
    };
    var PrefabContainerDirective_1;
    PrefabContainerDirective.initializeProps = registerProps();
    PrefabContainerDirective.ctorParameters = function () { return [
        { type: Injector },
        { type: Viewport }
    ]; };
    PrefabContainerDirective = PrefabContainerDirective_1 = __decorate([
        Directive({
            selector: '[wmPrefabContainer]',
            providers: [
                provideAsWidgetRef(PrefabContainerDirective_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector, Viewport])
    ], PrefabContainerDirective);
    return PrefabContainerDirective;
}(StylableComponent));

var components = [
    PrefabDirective,
    PrefabContainerDirective
];
var PrefabModule = /** @class */ (function () {
    function PrefabModule() {
    }
    PrefabModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], PrefabModule);
    return PrefabModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { PrefabContainerDirective, PrefabDirective, PrefabModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
