import { __decorate, __metadata, __param } from 'tslib';
import { Inject, HostListener, Directive, ElementRef, Component, EventEmitter, Input, Output, ɵɵdefineInjectable, ɵɵinject, Injectable, Self, Injector, NgModule } from '@angular/core';
import { WidgetRef, register, StylableComponent, styler, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { HttpClient } from '@angular/common/http';
import { File } from '@ionic-native/file';
import { FileOpener } from '@ionic-native/file-opener';
import { addClass, setCSS, hasCordova, noop, removeClass, isIos, $appDigest, convertToBlob, AbstractSpinnerService, isSpotcues, transformFileURI, App, setAttr, removeAttr } from '@wm/core';
import { DeviceService, DeviceFileDownloadService, DeviceFileCacheService, NetworkService } from '@wm/mobile/core';
import { CommonModule } from '@angular/common';
import { ProgressModule } from '@wm/components/basic/progress';

let AnchorDirective = class AnchorDirective {
    constructor(widget) {
        this.widget = widget;
    }
    onClick() {
        let href = this.widget.hyperlink;
        if (href) {
            if (href.indexOf(':') >= 0 && !(_.startsWith(href, 'http://') || _.startsWith(href, 'https://'))) {
                return;
            }
            else if (_.startsWith(href, '#')) {
                window.location.href = window.location.origin + window.location.pathname + href;
                return;
            }
            else if (_.startsWith(href, '//')) {
                href = 'https:' + href;
            }
            if (window['cordova']) {
                window.open(href, this.widget.target);
            }
        }
    }
};
AnchorDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [WidgetRef,] }] }
];
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AnchorDirective.prototype, "onClick", null);
AnchorDirective = __decorate([
    Directive({
        selector: '[wmAnchor]'
    }),
    __param(0, Inject(WidgetRef)),
    __metadata("design:paramtypes", [Object])
], AnchorDirective);

const DEFAULT_CLS = 'app-update-dialog modal fade in hidden';
const AUTO_UPDATE_FILENAME = 'app-auto-update.apk';
let AppUpdateComponent = class AppUpdateComponent {
    constructor(deviceService, fileDownloadService, elRef, file, fileOpener, http) {
        this.deviceService = deviceService;
        this.fileDownloadService = fileDownloadService;
        this.elRef = elRef;
        this.file = file;
        this.fileOpener = fileOpener;
        this.http = http;
        this.downloadProgress = 0;
        this.downloading = false;
        this.message = 'There is an update available. Would you like to update the app?';
        addClass(this.elRef.nativeElement, DEFAULT_CLS);
        setCSS(this.elRef.nativeElement, 'display', 'block');
        if (hasCordova()) {
            this.getBuildMeta().then(buildMeta => {
                if (buildMeta && buildMeta.buildMode === 'DEVELOPMENT_MODE') {
                    this.file.removeFile(cordova.file.externalApplicationStorageDirectory, AUTO_UPDATE_FILENAME).catch(noop);
                    this.checkForUpdate()
                        .then(this.getUserConfirmation.bind(this), noop);
                }
            });
        }
    }
    cancel() {
        addClass(this.elRef.nativeElement, 'hidden');
    }
    installLatestVersion() {
        const apkFile = cordova.file.externalApplicationStorageDirectory + AUTO_UPDATE_FILENAME, downloadLink = `${this._buildMeta.host}/appBuild/rest/mobileBuilds/android/download?token=${this._buildMeta.token}&buildNumber=${this._buildMeta.latestBuildNumber}&fileName=${AUTO_UPDATE_FILENAME}&releaseVersion=${this._buildMeta.platformVersion}`, progressObserver = { next: e => { this.downloadProgress = (e.loaded / e.total) * 100; }, error: null, complete: null };
        this.fileDownloadService.download(downloadLink, false, cordova.file.externalApplicationStorageDirectory, AUTO_UPDATE_FILENAME, progressObserver).then(() => {
            this.fileOpener.open(apkFile, 'application/vnd.android.package-archive');
        }, function () {
            this.message = 'Failed to download latest version.';
        });
        this.message = `Downloading the latest version [${this._buildMeta.latestVersion}].`;
        this.downloading = true;
    }
    checkForUpdate() {
        return new Promise((resolve, reject) => {
            this.http.get(`${this._buildMeta.host}/appBuild/rest/mobileBuilds/latest_build?token=${this._buildMeta.token}&releaseVersion=${this._buildMeta.platformVersion}`)
                .toPromise()
                .then((response) => {
                const latestBuildNumber = response.success.body.buildNumber, latestVersion = response.success.body.version;
                if (this._buildMeta.buildNumber < latestBuildNumber) {
                    this._buildMeta.latestBuildNumber = latestBuildNumber;
                    this._buildMeta.latestVersion = latestVersion;
                    resolve(latestBuildNumber);
                }
                else {
                    reject();
                }
            }, reject);
        });
    }
    getBuildMeta() {
        if (!this._buildMeta) {
            return this.file.readAsText(cordova.file.applicationDirectory + 'www/', 'build_meta.json')
                .then(data => {
                this._buildMeta = JSON.parse(data);
                return this._buildMeta;
            }, noop);
        }
        return Promise.resolve(this._buildMeta);
    }
    getUserConfirmation() {
        this.downloadProgress = 0;
        this.downloading = false;
        this.message = 'There is an update available. Would you like to update the app?';
        removeClass(this.elRef.nativeElement, 'hidden');
    }
};
AppUpdateComponent.ctorParameters = () => [
    { type: DeviceService },
    { type: DeviceFileDownloadService },
    { type: ElementRef },
    { type: File },
    { type: FileOpener },
    { type: HttpClient }
];
AppUpdateComponent = __decorate([
    Component({
        selector: '[wmAppUpdate]',
        template: "<div class=\"modal-dialog\">\n    <div class=\"modal-content\"> \n        <div class=\"modal-body\"> \n            <span [textContent]=\"message\"></span>\n            <div class=\"progress\" [hidden]=\"!downloading\">\n                <div class=\"progress-bar\" [style.width.%]=\"downloadProgress\"></div>\n            </div>\n        </div>\n        <div class=\"modal-footer\"> \n            <button type=\"button\" class=\"btn btn-default\" data-dismiss=\"modal\" (click)=\"cancel()\">Skip update</button>\n            <button type=\"button\" class=\"btn btn-primary\" [hidden]=\"downloading\" (click)=\"installLatestVersion()\">Update</button>\n        </div>\n    </div>\n</div>"
    }),
    __metadata("design:paramtypes", [DeviceService,
        DeviceFileDownloadService,
        ElementRef,
        File,
        FileOpener,
        HttpClient])
], AppUpdateComponent);

let FileBrowserComponent = class FileBrowserComponent {
    constructor(deviceService) {
        this.deviceService = deviceService;
        this.selectedFiles = [];
        this.submitEmitter = new EventEmitter();
    }
    getFileExtension(fileName) {
        const extIndex = fileName ? fileName.lastIndexOf('.') : -1;
        if (extIndex > 0) {
            return fileName.substring(extIndex + 1);
        }
        return '';
    }
    ngOnDestroy() {
        if (this.backButtonListenerDeregister) {
            this.backButtonListenerDeregister();
        }
    }
    onFileClick(file) {
        if (file.isFile) {
            if (file.isSelected) {
                this.deselectFile(file);
            }
            else {
                this.selectFile(file);
            }
        }
        else {
            this.goToFolder(file);
        }
    }
    set show(flag) {
        let rootDir = cordova.file.externalRootDirectory;
        this.isVisible = flag;
        if (flag) {
            if (!this.currentFolder) {
                if (isIos()) {
                    rootDir = cordova.file.documentsDirectory;
                }
                resolveLocalFileSystemURL(rootDir, root => this.goToFolder(root));
            }
            this.backButtonListenerDeregister = this.deviceService.onBackButtonTap(() => {
                if (this.isVisible) {
                    if (this.currentFolder.parent) {
                        this.onFileClick(this.currentFolder.parent);
                    }
                    else {
                        this.isVisible = false;
                    }
                    $appDigest();
                    return false;
                }
            });
        }
        else if (this.backButtonListenerDeregister) {
            this.backButtonListenerDeregister();
        }
    }
    submit() {
        const files = [];
        this.loadFileSize(this.selectedFiles).then(() => {
            _.forEach(this.selectedFiles, function (f) {
                f.isSelected = false;
                files.push({ path: f.nativeURL,
                    name: f.name,
                    size: f.size });
            });
            this.selectedFiles = [];
            this.isVisible = false;
            this.submitEmitter.next({ files: files });
        });
    }
    deselectFile(file) {
        _.remove(this.selectedFiles, file);
        file.isSelected = false;
    }
    goToFolder(folder) {
        if (!folder.files) {
            this.loadFolder(folder, this.fileTypeToSelect)
                .then(files => {
                folder.files = files;
                folder.parent = this.currentFolder;
                this.currentFolder = folder;
            });
        }
        else {
            this.currentFolder = folder;
        }
    }
    loadFileSize(files) {
        return Promise.all(files.map(f => {
            return new Promise((resolve, reject) => {
                f.file(o => {
                    f.size = o.size;
                    resolve();
                }, reject);
            });
        }));
    }
    loadFolder(folder, fileTypeToSelect) {
        return new Promise((resolve, reject) => {
            let fileTypeToShow;
            folder.createReader().readEntries((entries) => {
                if (!_.isEmpty(fileTypeToSelect)) {
                    fileTypeToShow = _.split(fileTypeToSelect, ',');
                    entries = _.filter(entries, e => {
                        return !e.isFile || _.findIndex(fileTypeToShow, ext => _.endsWith(e.name, '.' + ext)) >= 0;
                    });
                }
                resolve(_.sortBy(entries, e => (e.isFile ? '1_' : '0_') + e.name.toLowerCase()));
            }, reject);
        });
    }
    refreshFolder() {
        return this.loadFolder(this.currentFolder, this.fileTypeToSelect)
            .then(files => this.currentFolder.files = files);
    }
    selectFile(file) {
        if (!this.multiple && this.selectedFiles.length > 0) {
            this.selectedFiles[0].isSelected = false;
            this.selectedFiles = [];
        }
        this.selectedFiles.push(file);
        file.isSelected = true;
    }
};
FileBrowserComponent.ctorParameters = () => [
    { type: DeviceService }
];
__decorate([
    Input(),
    __metadata("design:type", String)
], FileBrowserComponent.prototype, "fileTypeToSelect", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], FileBrowserComponent.prototype, "multiple", void 0);
__decorate([
    Output('submit'),
    __metadata("design:type", Object)
], FileBrowserComponent.prototype, "submitEmitter", void 0);
FileBrowserComponent = __decorate([
    Component({
        selector: '[wmMobileFileBrowser]',
        template: "<div class=\"app-file-browser\" *ngIf=\"isVisible && currentFolder\">\n    <div class=\"modal-backdrop fade\" [class.in]=\"isVisible\"></div>\n    <div class=\"modal fade\" style=\"display: block;\" [class.in]=\"isVisible\" >\n        <div class=\"modal-dialog\">\n            <div class=\"modal-content\">\n                <div class=\"modal-header clearfix\">\n                    <h4 class=\"modal-title pull-left\">\n                        <span (click)=\"onFileClick(currentFolder.parent)\" [hidden]=\"!!!currentFolder.parent\">\n                            <i class=\"wi wi-long-arrow-left\"></i>\n                        </span>\n                     {{currentFolder.name}}\n                    </h4>\n                    <div class=\"selected-file-button pull-right\" (click)=\"refreshFolder()\">\n                        <i class=\"wi wi-refresh\"></i>\n                    </div>\n                </div>\n                <div class=\"modal-body\">\n                    <div class=\"file-info-box\" *ngFor=\"let file of currentFolder.files\">\n                        <div class=\"file-info\"  [class.bg-primary]=\"file.isSelected\" (click)=\"onFileClick(file)\">\n                            <i class=\"file-icon wi wi-folder\" *ngIf=\"!file.isFile\"></i>\n                            <i class=\"file-icon wi wi-file {{getFileExtension(file.name)}}\" *ngIf=\"file.isFile\"></i>\n                            <span class=\"file-name\">{{file.name}}</span>\n                        </div>\n                    </div>\n                </div>\n                <div class=\"modal-footer\">\n                    <button type=\"button\" class=\"btn btn-primary\" *ngIf=\"selectedFiles && selectedFiles.length > 0\" (click)=\"submit()\">\n                        Done <span class=\"badge badge-light\">{{selectedFiles.length}}</span>\n                    </button>\n                    <button type=\"button\" class=\"btn btn-default\" (click)=\"show = false;\">Close</button>\n                </div>\n            </div>\n        </div>\n    </div>\n</div>"
    }),
    __metadata("design:paramtypes", [DeviceService])
], FileBrowserComponent);

let FileSelectorService = class FileSelectorService {
    constructor(spinnerService) {
        this.spinnerService = spinnerService;
    }
    selectAudio(multiple = false) {
        var spinnerId = this.spinnerService.show("");
        return new Promise((resolve, reject) => {
            cordova.wavemaker.filePicker.selectAudio(multiple, resolve, reject);
        }).then(files => this.getFiles(files))
            .then((paths) => {
            this.spinnerService.hide(spinnerId);
            return paths;
        });
    }
    setFileBrowser(f) {
        this.fileBrowserComponent = f;
    }
    selectFiles(multiple = false, fileTypeToSelect) {
        var spinnerId = this.spinnerService.show("");
        return new Promise((resolve, reject) => {
            cordova.wavemaker.filePicker.selectFiles(multiple, resolve, reject);
        }).then(files => this.getFiles(files))
            .then((paths) => {
            this.spinnerService.hide(spinnerId);
            return paths;
        });
    }
    selectImages(multiple = false) {
        var spinnerId = this.spinnerService.show("");
        return new Promise((resolve, reject) => {
            cordova.wavemaker.filePicker.selectImage(multiple, resolve, reject);
        }).then(files => this.getFiles(files))
            .then((paths) => {
            this.spinnerService.hide(spinnerId);
            return paths;
        });
    }
    selectVideos(multiple = false) {
        var spinnerId = this.spinnerService.show("");
        return new Promise((resolve, reject) => {
            cordova.wavemaker.filePicker.selectVideo(multiple, resolve, reject);
        }).then(files => this.getFiles(files))
            .then((paths) => {
            this.spinnerService.hide(spinnerId);
            return paths;
        });
    }
    /**
     * Converts the file to blob using filepath
     * @param filePaths, array of file paths
     * @returns fileObj having name, path, content
     */
    getFiles(filePaths) {
        return Promise.all(_.map(filePaths, filePath => {
            if (filePath.indexOf('://') < 0) {
                filePath = 'file://' + filePath;
            }
            return convertToBlob(filePath);
        })).then(filesList => {
            return _.map(filesList, fileObj => {
                const path = fileObj.filepath;
                return {
                    name: path.split('/').pop(),
                    path: path,
                    content: fileObj.blob,
                    size: fileObj.blob.size
                };
            });
        });
    }
};
FileSelectorService.SERVICE_NAME = 'FileSelectorService';
FileSelectorService.ctorParameters = () => [
    { type: AbstractSpinnerService }
];
FileSelectorService.ɵprov = ɵɵdefineInjectable({ factory: function FileSelectorService_Factory() { return new FileSelectorService(ɵɵinject(AbstractSpinnerService)); }, token: FileSelectorService, providedIn: "root" });
FileSelectorService = __decorate([
    Injectable({ providedIn: 'root' }),
    __metadata("design:paramtypes", [AbstractSpinnerService])
], FileSelectorService);

const DEFAULT_IMAGE = 'resources/images/imagelists/default-image.png';
let ImageCacheDirective = class ImageCacheDirective {
    constructor(componentInstance, deviceFileCacheService, file) {
        this.componentInstance = componentInstance;
        this.deviceFileCacheService = deviceFileCacheService;
        this.file = file;
        this._isEnabled = false;
        this._lastUrl = '';
    }
    ngDoCheck() {
        if (!isSpotcues && this.componentInstance.imgSource
            && this.componentInstance.imgSource.startsWith('http')) {
            if (this._lastUrl !== this.componentInstance.imgSource) {
                this._lastUrl = this.componentInstance.imgSource;
                this.componentInstance.imgSource = DEFAULT_IMAGE;
                this.getLocalPath(this._lastUrl).then((localPath) => {
                    this._cacheUrl = transformFileURI(localPath);
                    this.componentInstance.imgSource = this._cacheUrl;
                });
            }
            else if (this._cacheUrl) {
                this.componentInstance.imgSource = this._cacheUrl;
            }
        }
    }
    set wmImageCache(val) {
        this._isEnabled = (hasCordova() && val === 'true');
    }
    getLocalPath(val) {
        if (hasCordova() && val && val.indexOf('{{') < 0 && val.startsWith('http')) {
            return this.deviceFileCacheService.getLocalPath(val, true, true, !this._isEnabled)
                .catch(noop);
        }
        return Promise.resolve(val);
    }
};
ImageCacheDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [WidgetRef,] }] },
    { type: DeviceFileCacheService },
    { type: File }
];
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], ImageCacheDirective.prototype, "wmImageCache", null);
ImageCacheDirective = __decorate([
    Directive({
        selector: '[wmImageCache]'
    }),
    __param(0, Self()), __param(0, Inject(WidgetRef)),
    __metadata("design:paramtypes", [Object, DeviceFileCacheService,
        File])
], ImageCacheDirective);

const registerProps = () => {
    register('wm-network-info-toaster', new Map([]));
};

var NetworkInfoToasterComponent_1;
const DEFAULT_CLS$1 = 'network-info-toaster';
const WIDGET_CONFIG = { widgetType: 'wm-network-info-toaster', hostClass: DEFAULT_CLS$1 };
var NetworkState;
(function (NetworkState) {
    NetworkState[NetworkState["CONNECTED"] = 1] = "CONNECTED";
    NetworkState[NetworkState["CONNECTING"] = 0] = "CONNECTING";
    NetworkState[NetworkState["SERVICE_AVAILABLE_BUT_NOT_CONNECTED"] = -1] = "SERVICE_AVAILABLE_BUT_NOT_CONNECTED";
    NetworkState[NetworkState["NETWORK_AVAIABLE_BUT_SERVICE_NOT_AVAILABLE"] = -2] = "NETWORK_AVAIABLE_BUT_SERVICE_NOT_AVAILABLE";
    NetworkState[NetworkState["NETWORK_NOT_AVAIABLE"] = -3] = "NETWORK_NOT_AVAIABLE";
})(NetworkState || (NetworkState = {}));
let NetworkInfoToasterComponent = NetworkInfoToasterComponent_1 = class NetworkInfoToasterComponent extends StylableComponent {
    constructor(networkService, app, inj) {
        super(inj, WIDGET_CONFIG);
        this.networkService = networkService;
        this.showMessage = false;
        this.isServiceConnected = false;
        this.isServiceAvailable = false;
        styler(this.$element, this);
        this.isServiceAvailable = this.networkService.isAvailable();
        this.isServiceConnected = this.networkService.isConnected();
        this._listenerDestroyer = app.subscribe('onNetworkStateChange', (data) => {
            this.renderMessage(data);
        });
        this.renderMessage();
    }
    renderMessage(data) {
        data = data || this.networkService.getState();
        const oldState = this.networkState;
        let autoHide = false;
        if (data.isConnected) {
            this.networkState = NetworkState.CONNECTED;
            autoHide = true;
        }
        else if (data.isConnecting) {
            this.networkState = NetworkState.CONNECTING;
        }
        else if (data.isServiceAvailable) {
            this.networkState = NetworkState.SERVICE_AVAILABLE_BUT_NOT_CONNECTED;
        }
        else if (data.isNetworkAvailable && !data.isServiceAvailable) {
            this.networkState = NetworkState.NETWORK_AVAIABLE_BUT_SERVICE_NOT_AVAILABLE;
        }
        else {
            this.networkState = NetworkState.NETWORK_NOT_AVAIABLE;
        }
        this.showMessage = (!(oldState === undefined && data.isConnected) && oldState !== this.networkState);
        if (autoHide && this.showMessage) {
            setTimeout(() => {
                this.showMessage = false;
                $appDigest();
            }, 5000);
        }
    }
    connect() {
        this.networkService.connect();
    }
    hideMessage() {
        this.showMessage = false;
    }
    ngOnDestroy() {
        this._listenerDestroyer();
        super.ngOnDestroy();
    }
};
NetworkInfoToasterComponent.initializeProps = registerProps();
NetworkInfoToasterComponent.ctorParameters = () => [
    { type: NetworkService },
    { type: App },
    { type: Injector }
];
NetworkInfoToasterComponent = NetworkInfoToasterComponent_1 = __decorate([
    Component({
        selector: '[wmNetworkInfoToaster]',
        template: "<div class=\"network-info-toaster-content\" [ngSwitch]=\"networkState\" *ngIf=\"showMessage\">\n    <div class=\"info\" *ngSwitchCase=\"-3\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_NETWORK_NOT_AVAILABLE\"></label>\n       <button class=\"btn btn-default hide-btn\" (click)=\"hideMessage()\" [textContent]=\"appLocale.LABEL_HIDE_NETWORK_INFO\"></button>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"-2\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_NOT_AVAILABLE\"></label>\n       <button class=\"btn btn-default hide-btn\" (click)=\"hideMessage()\" [textContent]=\"appLocale.LABEL_HIDE_NETWORK_INFO\"></button>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"-1\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_AVAILABLE\"></label>\n       <button class=\"btn btn-default hide-btn\" (click)=\"hideMessage()\" [textContent]=\"appLocale.LABEL_HIDE_NETWORK_INFO\"></button>\n       <button class=\"btn btn-primary connect-btn\" (click)=\"connect()\" [textContent]=\"appLocale.LABEL_CONNECT_TO_SERVICE\"></button>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"0\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_CONNECTING\"></label>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"1\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_CONNECTED\"></label>\n    </div>\n</div>",
        providers: [
            provideAsWidgetRef(NetworkInfoToasterComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [NetworkService, App, Injector])
], NetworkInfoToasterComponent);

var ProcessManagerComponent_1;
const MAX_PROCESS = 3;
let ProcessManagerComponent = ProcessManagerComponent_1 = class ProcessManagerComponent {
    constructor(el) {
        this.el = el;
        this.isVisible = true;
        this.instances = [];
        this.queue = [];
        addClass(this.el.nativeElement, 'app-global-progress-bar modal default');
    }
    createInstance(name, min = 0, max = 100) {
        const instance = {
            max: max,
            min: min,
            name: name,
            onStop: null,
            progressLabel: '',
            show: min !== max,
            stopButtonLabel: 'Cancel',
            value: 0
        };
        const api = {
            get: (propertyName) => instance[propertyName],
            set: (propertyName, propertyValue) => this.setInstaceProperty(instance, propertyName, propertyValue),
            destroy: () => this.removeInstance(instance)
        };
        return this.addToQueue(instance).then(() => api);
    }
    getVisibleInstances() {
        return this.instances.filter(i => i.show);
    }
    ngDoCheck() {
        const hasInstancesToShow = !!this.instances.find(i => i.show);
        if (this.isVisible && !hasInstancesToShow) {
            setAttr(this.el.nativeElement, 'hidden', 'true');
            this.isVisible = false;
        }
        else if (!this.isVisible && hasInstancesToShow) {
            removeAttr(this.el.nativeElement, 'hidden');
            this.isVisible = true;
        }
    }
    addToQueue(instance) {
        return new Promise(resolve => {
            this.queue.push(() => {
                if (this.instances.length < MAX_PROCESS) {
                    this.instances.push(instance);
                    resolve(instance);
                }
                else {
                    return false;
                }
            });
            this.flushQueue();
        });
    }
    flushQueue() {
        if (this.queue.length > 0 && this.queue[0]() !== false) {
            this.queue.shift();
            this.flushQueue();
        }
    }
    removeInstance(instance) {
        return new Promise(resolve => {
            setTimeout(() => {
                _.remove(this.instances, instance);
                this.flushQueue();
                resolve();
            }, 1000);
        });
    }
    setInstaceProperty(instance, propertyName, propertyValue) {
        if (propertyName === 'value') {
            if (instance.value >= instance.max) {
                propertyValue = instance.max;
            }
            instance.value = propertyValue;
            instance.progressLabel = instance.value + '/' + instance.max;
        }
        else if (propertyName === 'onStop' && _.isFunction(propertyValue)) {
            instance.onStop = () => {
                propertyValue();
                return this.removeInstance(instance);
            };
        }
        else {
            instance[propertyName] = propertyValue;
        }
        instance.show = instance.min !== instance.max;
    }
};
ProcessManagerComponent.ctorParameters = () => [
    { type: ElementRef }
];
ProcessManagerComponent = ProcessManagerComponent_1 = __decorate([
    Component({
        selector: '[wmProcessManager]',
        template: "<ng-container *ngIf=\"instances.length > 0\">\n   <div class=\"modal-dialog app-dialog\">\n       <div class=\"modal-content\">\n           <ul class=\"instance-list list-unstyled\">\n               <li *ngFor=\"let instance of getVisibleInstances(instances); index as i\" class=\"instance-list-item\">\n                   <div class=\"row\">\n                       <div class=\"col-xs-8\">\n                           <label class=\"app-global-progress-bar-name h6\">{{instance.name}}</label>\n                       </div>\n                       <div class=\"col-xs-4 app-global-progress-bar-progress-label-col\">\n                           <label class=\"app-global-progress-bar-progress-label h6\">\n                                   {{instance.progressLabel}}</label>\n                       </div>\n                   </div>\n                   <ng-template [ngTemplateOutlet]=\"progressTemplate\" [ngTemplateOutletContext]=\"{instance: instance}\"></ng-template>\n                   <button class=\"btn btn-secondary pull-right stop-btn\" *ngIf=\"instance.onStop\" (click)=\"instance.onStop();\">\n                       {{instance.stopButtonLabel}}\n                   </button>\n                   <div style=\"clear: both;\"></div>\n               </li>\n               <li class=\"instance-list-item\" *ngIf=\"queue.length > 0\">\n                   <label class=\"global-progress-bar-ui-primary-label h6\">\n                       ({{queue.length}}) queued\n                   </label>\n               </li>\n           </ul>\n       </div>\n   </div>\n</ng-container>\n<ng-template #progressTemplate let-instance=\"instance\">\n    <div wmProgressBar minvalue.bind=\"instance.min\" maxvalue.bind=\"instance.max\" datavalue.bind=\"instance.value\"></div>\n</ng-template>",
        providers: [
            provideAsWidgetRef(ProcessManagerComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [ElementRef])
], ProcessManagerComponent);

let ProcessManagementService = class ProcessManagementService {
    setUIComponent(processManagerComponent) {
        this.processManagerComponent = processManagerComponent;
    }
    /**
     * Returns a promise that will be resolved when an instance is available. At max, 3 instances can only run
     * in parallel and rest has to wait till a process is completed.
     *
     * A progress instance has the following properties.
     *
     *   1) min {number} minimum value, default value is 0 </br>
     *   2) max {number} maximum value, default value is 100 </br>
     *   3) value {number} progress value </br>
     *   4) progressLabel {string} process name </br>
     *   5) stopButtonLabel {string} label for stop button, default value is 'Cancel' </br>
     *   6) onStop {function} function to invoke when stop button is clicked. </br>
     *
     * A progress instance has 3 methods </br>
     *   1) set(property, value) -- sets value to the corresponding property </br>
     *   2) get(property) -- returns property value </br>
     *   3) destroy() -- closes the instance. </br>
     *
     * A progress instance will get auto closed when value and max are equal or when destroy method is called.
     *
     * @param {string} name name of the process whose progress is going to be shown
     * @param {number} min minimum value
     * @param {number} max maximum value
     *
     * @returns {object} a promise
     */
    createInstance(name, min, max) {
        if (!this.processManagerComponent) {
            return Promise.reject('ProcessManagerComponent is missing');
        }
        return this.processManagerComponent.createInstance(name, min, max);
    }
};
ProcessManagementService.SERVICE_NAME = 'ProcessManagementService';
ProcessManagementService.ɵprov = ɵɵdefineInjectable({ factory: function ProcessManagementService_Factory() { return new ProcessManagementService(); }, token: ProcessManagementService, providedIn: "root" });
ProcessManagementService = __decorate([
    Injectable({ providedIn: 'root' })
], ProcessManagementService);

const components = [
    AnchorDirective,
    AppUpdateComponent,
    FileBrowserComponent,
    ImageCacheDirective,
    NetworkInfoToasterComponent,
    ProcessManagerComponent
];
let BasicModule = class BasicModule {
};
BasicModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            ProgressModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], BasicModule);

/**
 * Generated bundle index. Do not edit.
 */

export { AnchorDirective, AppUpdateComponent, BasicModule, FileBrowserComponent, FileSelectorService, ImageCacheDirective, NetworkInfoToasterComponent, NetworkState, ProcessManagementService, ProcessManagerComponent, registerProps as ɵa };
//# sourceMappingURL=index.js.map
