import { __decorate, __metadata, __param, __extends, __spread } from 'tslib';
import { Inject, HostListener, Directive, ElementRef, Component, EventEmitter, Input, Output, ɵɵdefineInjectable, ɵɵinject, Injectable, Self, Injector, NgModule } from '@angular/core';
import { WidgetRef, register, styler, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { HttpClient } from '@angular/common/http';
import { File } from '@ionic-native/file';
import { FileOpener } from '@ionic-native/file-opener';
import { addClass, setCSS, hasCordova, noop, removeClass, isIos, $appDigest, convertToBlob, AbstractSpinnerService, isSpotcues, transformFileURI, App, setAttr, removeAttr } from '@wm/core';
import { DeviceService, DeviceFileDownloadService, DeviceFileCacheService, NetworkService } from '@wm/mobile/core';
import { CommonModule } from '@angular/common';
import { ProgressModule } from '@wm/components/basic/progress';

var AnchorDirective = /** @class */ (function () {
    function AnchorDirective(widget) {
        this.widget = widget;
    }
    AnchorDirective.prototype.onClick = function () {
        var href = this.widget.hyperlink;
        if (href) {
            if (href.indexOf(':') >= 0 && !(_.startsWith(href, 'http://') || _.startsWith(href, 'https://'))) {
                return;
            }
            else if (_.startsWith(href, '#')) {
                window.location.href = window.location.origin + window.location.pathname + href;
                return;
            }
            else if (_.startsWith(href, '//')) {
                href = 'https:' + href;
            }
            if (window['cordova']) {
                window.open(href, this.widget.target);
            }
        }
    };
    AnchorDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [WidgetRef,] }] }
    ]; };
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], AnchorDirective.prototype, "onClick", null);
    AnchorDirective = __decorate([
        Directive({
            selector: '[wmAnchor]'
        }),
        __param(0, Inject(WidgetRef)),
        __metadata("design:paramtypes", [Object])
    ], AnchorDirective);
    return AnchorDirective;
}());

var DEFAULT_CLS = 'app-update-dialog modal fade in hidden';
var AUTO_UPDATE_FILENAME = 'app-auto-update.apk';
var AppUpdateComponent = /** @class */ (function () {
    function AppUpdateComponent(deviceService, fileDownloadService, elRef, file, fileOpener, http) {
        var _this = this;
        this.deviceService = deviceService;
        this.fileDownloadService = fileDownloadService;
        this.elRef = elRef;
        this.file = file;
        this.fileOpener = fileOpener;
        this.http = http;
        this.downloadProgress = 0;
        this.downloading = false;
        this.message = 'There is an update available. Would you like to update the app?';
        addClass(this.elRef.nativeElement, DEFAULT_CLS);
        setCSS(this.elRef.nativeElement, 'display', 'block');
        if (hasCordova()) {
            this.getBuildMeta().then(function (buildMeta) {
                if (buildMeta && buildMeta.buildMode === 'DEVELOPMENT_MODE') {
                    _this.file.removeFile(cordova.file.externalApplicationStorageDirectory, AUTO_UPDATE_FILENAME).catch(noop);
                    _this.checkForUpdate()
                        .then(_this.getUserConfirmation.bind(_this), noop);
                }
            });
        }
    }
    AppUpdateComponent.prototype.cancel = function () {
        addClass(this.elRef.nativeElement, 'hidden');
    };
    AppUpdateComponent.prototype.installLatestVersion = function () {
        var _this = this;
        var apkFile = cordova.file.externalApplicationStorageDirectory + AUTO_UPDATE_FILENAME, downloadLink = this._buildMeta.host + "/appBuild/rest/mobileBuilds/android/download?token=" + this._buildMeta.token + "&buildNumber=" + this._buildMeta.latestBuildNumber + "&fileName=" + AUTO_UPDATE_FILENAME + "&releaseVersion=" + this._buildMeta.platformVersion, progressObserver = { next: function (e) { _this.downloadProgress = (e.loaded / e.total) * 100; }, error: null, complete: null };
        this.fileDownloadService.download(downloadLink, false, cordova.file.externalApplicationStorageDirectory, AUTO_UPDATE_FILENAME, progressObserver).then(function () {
            _this.fileOpener.open(apkFile, 'application/vnd.android.package-archive');
        }, function () {
            this.message = 'Failed to download latest version.';
        });
        this.message = "Downloading the latest version [" + this._buildMeta.latestVersion + "].";
        this.downloading = true;
    };
    AppUpdateComponent.prototype.checkForUpdate = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.http.get(_this._buildMeta.host + "/appBuild/rest/mobileBuilds/latest_build?token=" + _this._buildMeta.token + "&releaseVersion=" + _this._buildMeta.platformVersion)
                .toPromise()
                .then(function (response) {
                var latestBuildNumber = response.success.body.buildNumber, latestVersion = response.success.body.version;
                if (_this._buildMeta.buildNumber < latestBuildNumber) {
                    _this._buildMeta.latestBuildNumber = latestBuildNumber;
                    _this._buildMeta.latestVersion = latestVersion;
                    resolve(latestBuildNumber);
                }
                else {
                    reject();
                }
            }, reject);
        });
    };
    AppUpdateComponent.prototype.getBuildMeta = function () {
        var _this = this;
        if (!this._buildMeta) {
            return this.file.readAsText(cordova.file.applicationDirectory + 'www/', 'build_meta.json')
                .then(function (data) {
                _this._buildMeta = JSON.parse(data);
                return _this._buildMeta;
            }, noop);
        }
        return Promise.resolve(this._buildMeta);
    };
    AppUpdateComponent.prototype.getUserConfirmation = function () {
        this.downloadProgress = 0;
        this.downloading = false;
        this.message = 'There is an update available. Would you like to update the app?';
        removeClass(this.elRef.nativeElement, 'hidden');
    };
    AppUpdateComponent.ctorParameters = function () { return [
        { type: DeviceService },
        { type: DeviceFileDownloadService },
        { type: ElementRef },
        { type: File },
        { type: FileOpener },
        { type: HttpClient }
    ]; };
    AppUpdateComponent = __decorate([
        Component({
            selector: '[wmAppUpdate]',
            template: "<div class=\"modal-dialog\">\n    <div class=\"modal-content\"> \n        <div class=\"modal-body\"> \n            <span [textContent]=\"message\"></span>\n            <div class=\"progress\" [hidden]=\"!downloading\">\n                <div class=\"progress-bar\" [style.width.%]=\"downloadProgress\"></div>\n            </div>\n        </div>\n        <div class=\"modal-footer\"> \n            <button type=\"button\" class=\"btn btn-default\" data-dismiss=\"modal\" (click)=\"cancel()\">Skip update</button>\n            <button type=\"button\" class=\"btn btn-primary\" [hidden]=\"downloading\" (click)=\"installLatestVersion()\">Update</button>\n        </div>\n    </div>\n</div>"
        }),
        __metadata("design:paramtypes", [DeviceService,
            DeviceFileDownloadService,
            ElementRef,
            File,
            FileOpener,
            HttpClient])
    ], AppUpdateComponent);
    return AppUpdateComponent;
}());

var FileBrowserComponent = /** @class */ (function () {
    function FileBrowserComponent(deviceService) {
        this.deviceService = deviceService;
        this.selectedFiles = [];
        this.submitEmitter = new EventEmitter();
    }
    FileBrowserComponent.prototype.getFileExtension = function (fileName) {
        var extIndex = fileName ? fileName.lastIndexOf('.') : -1;
        if (extIndex > 0) {
            return fileName.substring(extIndex + 1);
        }
        return '';
    };
    FileBrowserComponent.prototype.ngOnDestroy = function () {
        if (this.backButtonListenerDeregister) {
            this.backButtonListenerDeregister();
        }
    };
    FileBrowserComponent.prototype.onFileClick = function (file) {
        if (file.isFile) {
            if (file.isSelected) {
                this.deselectFile(file);
            }
            else {
                this.selectFile(file);
            }
        }
        else {
            this.goToFolder(file);
        }
    };
    Object.defineProperty(FileBrowserComponent.prototype, "show", {
        set: function (flag) {
            var _this = this;
            var rootDir = cordova.file.externalRootDirectory;
            this.isVisible = flag;
            if (flag) {
                if (!this.currentFolder) {
                    if (isIos()) {
                        rootDir = cordova.file.documentsDirectory;
                    }
                    resolveLocalFileSystemURL(rootDir, function (root) { return _this.goToFolder(root); });
                }
                this.backButtonListenerDeregister = this.deviceService.onBackButtonTap(function () {
                    if (_this.isVisible) {
                        if (_this.currentFolder.parent) {
                            _this.onFileClick(_this.currentFolder.parent);
                        }
                        else {
                            _this.isVisible = false;
                        }
                        $appDigest();
                        return false;
                    }
                });
            }
            else if (this.backButtonListenerDeregister) {
                this.backButtonListenerDeregister();
            }
        },
        enumerable: true,
        configurable: true
    });
    FileBrowserComponent.prototype.submit = function () {
        var _this = this;
        var files = [];
        this.loadFileSize(this.selectedFiles).then(function () {
            _.forEach(_this.selectedFiles, function (f) {
                f.isSelected = false;
                files.push({ path: f.nativeURL,
                    name: f.name,
                    size: f.size });
            });
            _this.selectedFiles = [];
            _this.isVisible = false;
            _this.submitEmitter.next({ files: files });
        });
    };
    FileBrowserComponent.prototype.deselectFile = function (file) {
        _.remove(this.selectedFiles, file);
        file.isSelected = false;
    };
    FileBrowserComponent.prototype.goToFolder = function (folder) {
        var _this = this;
        if (!folder.files) {
            this.loadFolder(folder, this.fileTypeToSelect)
                .then(function (files) {
                folder.files = files;
                folder.parent = _this.currentFolder;
                _this.currentFolder = folder;
            });
        }
        else {
            this.currentFolder = folder;
        }
    };
    FileBrowserComponent.prototype.loadFileSize = function (files) {
        return Promise.all(files.map(function (f) {
            return new Promise(function (resolve, reject) {
                f.file(function (o) {
                    f.size = o.size;
                    resolve();
                }, reject);
            });
        }));
    };
    FileBrowserComponent.prototype.loadFolder = function (folder, fileTypeToSelect) {
        return new Promise(function (resolve, reject) {
            var fileTypeToShow;
            folder.createReader().readEntries(function (entries) {
                if (!_.isEmpty(fileTypeToSelect)) {
                    fileTypeToShow = _.split(fileTypeToSelect, ',');
                    entries = _.filter(entries, function (e) {
                        return !e.isFile || _.findIndex(fileTypeToShow, function (ext) { return _.endsWith(e.name, '.' + ext); }) >= 0;
                    });
                }
                resolve(_.sortBy(entries, function (e) { return (e.isFile ? '1_' : '0_') + e.name.toLowerCase(); }));
            }, reject);
        });
    };
    FileBrowserComponent.prototype.refreshFolder = function () {
        var _this = this;
        return this.loadFolder(this.currentFolder, this.fileTypeToSelect)
            .then(function (files) { return _this.currentFolder.files = files; });
    };
    FileBrowserComponent.prototype.selectFile = function (file) {
        if (!this.multiple && this.selectedFiles.length > 0) {
            this.selectedFiles[0].isSelected = false;
            this.selectedFiles = [];
        }
        this.selectedFiles.push(file);
        file.isSelected = true;
    };
    FileBrowserComponent.ctorParameters = function () { return [
        { type: DeviceService }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileBrowserComponent.prototype, "fileTypeToSelect", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], FileBrowserComponent.prototype, "multiple", void 0);
    __decorate([
        Output('submit'),
        __metadata("design:type", Object)
    ], FileBrowserComponent.prototype, "submitEmitter", void 0);
    FileBrowserComponent = __decorate([
        Component({
            selector: '[wmMobileFileBrowser]',
            template: "<div class=\"app-file-browser\" *ngIf=\"isVisible && currentFolder\">\n    <div class=\"modal-backdrop fade\" [class.in]=\"isVisible\"></div>\n    <div class=\"modal fade\" style=\"display: block;\" [class.in]=\"isVisible\" >\n        <div class=\"modal-dialog\">\n            <div class=\"modal-content\">\n                <div class=\"modal-header clearfix\">\n                    <h4 class=\"modal-title pull-left\">\n                        <span (click)=\"onFileClick(currentFolder.parent)\" [hidden]=\"!!!currentFolder.parent\">\n                            <i class=\"wi wi-long-arrow-left\"></i>\n                        </span>\n                     {{currentFolder.name}}\n                    </h4>\n                    <div class=\"selected-file-button pull-right\" (click)=\"refreshFolder()\">\n                        <i class=\"wi wi-refresh\"></i>\n                    </div>\n                </div>\n                <div class=\"modal-body\">\n                    <div class=\"file-info-box\" *ngFor=\"let file of currentFolder.files\">\n                        <div class=\"file-info\"  [class.bg-primary]=\"file.isSelected\" (click)=\"onFileClick(file)\">\n                            <i class=\"file-icon wi wi-folder\" *ngIf=\"!file.isFile\"></i>\n                            <i class=\"file-icon wi wi-file {{getFileExtension(file.name)}}\" *ngIf=\"file.isFile\"></i>\n                            <span class=\"file-name\">{{file.name}}</span>\n                        </div>\n                    </div>\n                </div>\n                <div class=\"modal-footer\">\n                    <button type=\"button\" class=\"btn btn-primary\" *ngIf=\"selectedFiles && selectedFiles.length > 0\" (click)=\"submit()\">\n                        Done <span class=\"badge badge-light\">{{selectedFiles.length}}</span>\n                    </button>\n                    <button type=\"button\" class=\"btn btn-default\" (click)=\"show = false;\">Close</button>\n                </div>\n            </div>\n        </div>\n    </div>\n</div>"
        }),
        __metadata("design:paramtypes", [DeviceService])
    ], FileBrowserComponent);
    return FileBrowserComponent;
}());

var FileSelectorService = /** @class */ (function () {
    function FileSelectorService(spinnerService) {
        this.spinnerService = spinnerService;
    }
    FileSelectorService.prototype.selectAudio = function (multiple) {
        var _this = this;
        if (multiple === void 0) { multiple = false; }
        var spinnerId = this.spinnerService.show("");
        return new Promise(function (resolve, reject) {
            cordova.wavemaker.filePicker.selectAudio(multiple, resolve, reject);
        }).then(function (files) { return _this.getFiles(files); })
            .then(function (paths) {
            _this.spinnerService.hide(spinnerId);
            return paths;
        });
    };
    FileSelectorService.prototype.setFileBrowser = function (f) {
        this.fileBrowserComponent = f;
    };
    FileSelectorService.prototype.selectFiles = function (multiple, fileTypeToSelect) {
        var _this = this;
        if (multiple === void 0) { multiple = false; }
        var spinnerId = this.spinnerService.show("");
        return new Promise(function (resolve, reject) {
            cordova.wavemaker.filePicker.selectFiles(multiple, resolve, reject);
        }).then(function (files) { return _this.getFiles(files); })
            .then(function (paths) {
            _this.spinnerService.hide(spinnerId);
            return paths;
        });
    };
    FileSelectorService.prototype.selectImages = function (multiple) {
        var _this = this;
        if (multiple === void 0) { multiple = false; }
        var spinnerId = this.spinnerService.show("");
        return new Promise(function (resolve, reject) {
            cordova.wavemaker.filePicker.selectImage(multiple, resolve, reject);
        }).then(function (files) { return _this.getFiles(files); })
            .then(function (paths) {
            _this.spinnerService.hide(spinnerId);
            return paths;
        });
    };
    FileSelectorService.prototype.selectVideos = function (multiple) {
        var _this = this;
        if (multiple === void 0) { multiple = false; }
        var spinnerId = this.spinnerService.show("");
        return new Promise(function (resolve, reject) {
            cordova.wavemaker.filePicker.selectVideo(multiple, resolve, reject);
        }).then(function (files) { return _this.getFiles(files); })
            .then(function (paths) {
            _this.spinnerService.hide(spinnerId);
            return paths;
        });
    };
    /**
     * Converts the file to blob using filepath
     * @param filePaths, array of file paths
     * @returns fileObj having name, path, content
     */
    FileSelectorService.prototype.getFiles = function (filePaths) {
        return Promise.all(_.map(filePaths, function (filePath) {
            if (filePath.indexOf('://') < 0) {
                filePath = 'file://' + filePath;
            }
            return convertToBlob(filePath);
        })).then(function (filesList) {
            return _.map(filesList, function (fileObj) {
                var path = fileObj.filepath;
                return {
                    name: path.split('/').pop(),
                    path: path,
                    content: fileObj.blob,
                    size: fileObj.blob.size
                };
            });
        });
    };
    FileSelectorService.SERVICE_NAME = 'FileSelectorService';
    FileSelectorService.ctorParameters = function () { return [
        { type: AbstractSpinnerService }
    ]; };
    FileSelectorService.ɵprov = ɵɵdefineInjectable({ factory: function FileSelectorService_Factory() { return new FileSelectorService(ɵɵinject(AbstractSpinnerService)); }, token: FileSelectorService, providedIn: "root" });
    FileSelectorService = __decorate([
        Injectable({ providedIn: 'root' }),
        __metadata("design:paramtypes", [AbstractSpinnerService])
    ], FileSelectorService);
    return FileSelectorService;
}());

var DEFAULT_IMAGE = 'resources/images/imagelists/default-image.png';
var ImageCacheDirective = /** @class */ (function () {
    function ImageCacheDirective(componentInstance, deviceFileCacheService, file) {
        this.componentInstance = componentInstance;
        this.deviceFileCacheService = deviceFileCacheService;
        this.file = file;
        this._isEnabled = false;
        this._lastUrl = '';
    }
    ImageCacheDirective.prototype.ngDoCheck = function () {
        var _this = this;
        if (!isSpotcues && this.componentInstance.imgSource
            && this.componentInstance.imgSource.startsWith('http')) {
            if (this._lastUrl !== this.componentInstance.imgSource) {
                this._lastUrl = this.componentInstance.imgSource;
                this.componentInstance.imgSource = DEFAULT_IMAGE;
                this.getLocalPath(this._lastUrl).then(function (localPath) {
                    _this._cacheUrl = transformFileURI(localPath);
                    _this.componentInstance.imgSource = _this._cacheUrl;
                });
            }
            else if (this._cacheUrl) {
                this.componentInstance.imgSource = this._cacheUrl;
            }
        }
    };
    Object.defineProperty(ImageCacheDirective.prototype, "wmImageCache", {
        set: function (val) {
            this._isEnabled = (hasCordova() && val === 'true');
        },
        enumerable: true,
        configurable: true
    });
    ImageCacheDirective.prototype.getLocalPath = function (val) {
        if (hasCordova() && val && val.indexOf('{{') < 0 && val.startsWith('http')) {
            return this.deviceFileCacheService.getLocalPath(val, true, true, !this._isEnabled)
                .catch(noop);
        }
        return Promise.resolve(val);
    };
    ImageCacheDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Self }, { type: Inject, args: [WidgetRef,] }] },
        { type: DeviceFileCacheService },
        { type: File }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], ImageCacheDirective.prototype, "wmImageCache", null);
    ImageCacheDirective = __decorate([
        Directive({
            selector: '[wmImageCache]'
        }),
        __param(0, Self()), __param(0, Inject(WidgetRef)),
        __metadata("design:paramtypes", [Object, DeviceFileCacheService,
            File])
    ], ImageCacheDirective);
    return ImageCacheDirective;
}());

var registerProps = function () {
    register('wm-network-info-toaster', new Map([]));
};

var DEFAULT_CLS$1 = 'network-info-toaster';
var WIDGET_CONFIG = { widgetType: 'wm-network-info-toaster', hostClass: DEFAULT_CLS$1 };
var NetworkState;
(function (NetworkState) {
    NetworkState[NetworkState["CONNECTED"] = 1] = "CONNECTED";
    NetworkState[NetworkState["CONNECTING"] = 0] = "CONNECTING";
    NetworkState[NetworkState["SERVICE_AVAILABLE_BUT_NOT_CONNECTED"] = -1] = "SERVICE_AVAILABLE_BUT_NOT_CONNECTED";
    NetworkState[NetworkState["NETWORK_AVAIABLE_BUT_SERVICE_NOT_AVAILABLE"] = -2] = "NETWORK_AVAIABLE_BUT_SERVICE_NOT_AVAILABLE";
    NetworkState[NetworkState["NETWORK_NOT_AVAIABLE"] = -3] = "NETWORK_NOT_AVAIABLE";
})(NetworkState || (NetworkState = {}));
var NetworkInfoToasterComponent = /** @class */ (function (_super) {
    __extends(NetworkInfoToasterComponent, _super);
    function NetworkInfoToasterComponent(networkService, app, inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.networkService = networkService;
        _this.showMessage = false;
        _this.isServiceConnected = false;
        _this.isServiceAvailable = false;
        styler(_this.$element, _this);
        _this.isServiceAvailable = _this.networkService.isAvailable();
        _this.isServiceConnected = _this.networkService.isConnected();
        _this._listenerDestroyer = app.subscribe('onNetworkStateChange', function (data) {
            _this.renderMessage(data);
        });
        _this.renderMessage();
        return _this;
    }
    NetworkInfoToasterComponent_1 = NetworkInfoToasterComponent;
    NetworkInfoToasterComponent.prototype.renderMessage = function (data) {
        var _this = this;
        data = data || this.networkService.getState();
        var oldState = this.networkState;
        var autoHide = false;
        if (data.isConnected) {
            this.networkState = NetworkState.CONNECTED;
            autoHide = true;
        }
        else if (data.isConnecting) {
            this.networkState = NetworkState.CONNECTING;
        }
        else if (data.isServiceAvailable) {
            this.networkState = NetworkState.SERVICE_AVAILABLE_BUT_NOT_CONNECTED;
        }
        else if (data.isNetworkAvailable && !data.isServiceAvailable) {
            this.networkState = NetworkState.NETWORK_AVAIABLE_BUT_SERVICE_NOT_AVAILABLE;
        }
        else {
            this.networkState = NetworkState.NETWORK_NOT_AVAIABLE;
        }
        this.showMessage = (!(oldState === undefined && data.isConnected) && oldState !== this.networkState);
        if (autoHide && this.showMessage) {
            setTimeout(function () {
                _this.showMessage = false;
                $appDigest();
            }, 5000);
        }
    };
    NetworkInfoToasterComponent.prototype.connect = function () {
        this.networkService.connect();
    };
    NetworkInfoToasterComponent.prototype.hideMessage = function () {
        this.showMessage = false;
    };
    NetworkInfoToasterComponent.prototype.ngOnDestroy = function () {
        this._listenerDestroyer();
        _super.prototype.ngOnDestroy.call(this);
    };
    var NetworkInfoToasterComponent_1;
    NetworkInfoToasterComponent.initializeProps = registerProps();
    NetworkInfoToasterComponent.ctorParameters = function () { return [
        { type: NetworkService },
        { type: App },
        { type: Injector }
    ]; };
    NetworkInfoToasterComponent = NetworkInfoToasterComponent_1 = __decorate([
        Component({
            selector: '[wmNetworkInfoToaster]',
            template: "<div class=\"network-info-toaster-content\" [ngSwitch]=\"networkState\" *ngIf=\"showMessage\">\n    <div class=\"info\" *ngSwitchCase=\"-3\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_NETWORK_NOT_AVAILABLE\"></label>\n       <button class=\"btn btn-default hide-btn\" (click)=\"hideMessage()\" [textContent]=\"appLocale.LABEL_HIDE_NETWORK_INFO\"></button>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"-2\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_NOT_AVAILABLE\"></label>\n       <button class=\"btn btn-default hide-btn\" (click)=\"hideMessage()\" [textContent]=\"appLocale.LABEL_HIDE_NETWORK_INFO\"></button>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"-1\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_AVAILABLE\"></label>\n       <button class=\"btn btn-default hide-btn\" (click)=\"hideMessage()\" [textContent]=\"appLocale.LABEL_HIDE_NETWORK_INFO\"></button>\n       <button class=\"btn btn-primary connect-btn\" (click)=\"connect()\" [textContent]=\"appLocale.LABEL_CONNECT_TO_SERVICE\"></button>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"0\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_CONNECTING\"></label>\n    </div>\n    <div class=\"info\" *ngSwitchCase=\"1\">\n       <label class=\"message\" [textContent]=\"appLocale.MESSAGE_SERVICE_CONNECTED\"></label>\n    </div>\n</div>",
            providers: [
                provideAsWidgetRef(NetworkInfoToasterComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [NetworkService, App, Injector])
    ], NetworkInfoToasterComponent);
    return NetworkInfoToasterComponent;
}(StylableComponent));

var MAX_PROCESS = 3;
var ProcessManagerComponent = /** @class */ (function () {
    function ProcessManagerComponent(el) {
        this.el = el;
        this.isVisible = true;
        this.instances = [];
        this.queue = [];
        addClass(this.el.nativeElement, 'app-global-progress-bar modal default');
    }
    ProcessManagerComponent_1 = ProcessManagerComponent;
    ProcessManagerComponent.prototype.createInstance = function (name, min, max) {
        var _this = this;
        if (min === void 0) { min = 0; }
        if (max === void 0) { max = 100; }
        var instance = {
            max: max,
            min: min,
            name: name,
            onStop: null,
            progressLabel: '',
            show: min !== max,
            stopButtonLabel: 'Cancel',
            value: 0
        };
        var api = {
            get: function (propertyName) { return instance[propertyName]; },
            set: function (propertyName, propertyValue) { return _this.setInstaceProperty(instance, propertyName, propertyValue); },
            destroy: function () { return _this.removeInstance(instance); }
        };
        return this.addToQueue(instance).then(function () { return api; });
    };
    ProcessManagerComponent.prototype.getVisibleInstances = function () {
        return this.instances.filter(function (i) { return i.show; });
    };
    ProcessManagerComponent.prototype.ngDoCheck = function () {
        var hasInstancesToShow = !!this.instances.find(function (i) { return i.show; });
        if (this.isVisible && !hasInstancesToShow) {
            setAttr(this.el.nativeElement, 'hidden', 'true');
            this.isVisible = false;
        }
        else if (!this.isVisible && hasInstancesToShow) {
            removeAttr(this.el.nativeElement, 'hidden');
            this.isVisible = true;
        }
    };
    ProcessManagerComponent.prototype.addToQueue = function (instance) {
        var _this = this;
        return new Promise(function (resolve) {
            _this.queue.push(function () {
                if (_this.instances.length < MAX_PROCESS) {
                    _this.instances.push(instance);
                    resolve(instance);
                }
                else {
                    return false;
                }
            });
            _this.flushQueue();
        });
    };
    ProcessManagerComponent.prototype.flushQueue = function () {
        if (this.queue.length > 0 && this.queue[0]() !== false) {
            this.queue.shift();
            this.flushQueue();
        }
    };
    ProcessManagerComponent.prototype.removeInstance = function (instance) {
        var _this = this;
        return new Promise(function (resolve) {
            setTimeout(function () {
                _.remove(_this.instances, instance);
                _this.flushQueue();
                resolve();
            }, 1000);
        });
    };
    ProcessManagerComponent.prototype.setInstaceProperty = function (instance, propertyName, propertyValue) {
        var _this = this;
        if (propertyName === 'value') {
            if (instance.value >= instance.max) {
                propertyValue = instance.max;
            }
            instance.value = propertyValue;
            instance.progressLabel = instance.value + '/' + instance.max;
        }
        else if (propertyName === 'onStop' && _.isFunction(propertyValue)) {
            instance.onStop = function () {
                propertyValue();
                return _this.removeInstance(instance);
            };
        }
        else {
            instance[propertyName] = propertyValue;
        }
        instance.show = instance.min !== instance.max;
    };
    var ProcessManagerComponent_1;
    ProcessManagerComponent.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    ProcessManagerComponent = ProcessManagerComponent_1 = __decorate([
        Component({
            selector: '[wmProcessManager]',
            template: "<ng-container *ngIf=\"instances.length > 0\">\n   <div class=\"modal-dialog app-dialog\">\n       <div class=\"modal-content\">\n           <ul class=\"instance-list list-unstyled\">\n               <li *ngFor=\"let instance of getVisibleInstances(instances); index as i\" class=\"instance-list-item\">\n                   <div class=\"row\">\n                       <div class=\"col-xs-8\">\n                           <label class=\"app-global-progress-bar-name h6\">{{instance.name}}</label>\n                       </div>\n                       <div class=\"col-xs-4 app-global-progress-bar-progress-label-col\">\n                           <label class=\"app-global-progress-bar-progress-label h6\">\n                                   {{instance.progressLabel}}</label>\n                       </div>\n                   </div>\n                   <ng-template [ngTemplateOutlet]=\"progressTemplate\" [ngTemplateOutletContext]=\"{instance: instance}\"></ng-template>\n                   <button class=\"btn btn-secondary pull-right stop-btn\" *ngIf=\"instance.onStop\" (click)=\"instance.onStop();\">\n                       {{instance.stopButtonLabel}}\n                   </button>\n                   <div style=\"clear: both;\"></div>\n               </li>\n               <li class=\"instance-list-item\" *ngIf=\"queue.length > 0\">\n                   <label class=\"global-progress-bar-ui-primary-label h6\">\n                       ({{queue.length}}) queued\n                   </label>\n               </li>\n           </ul>\n       </div>\n   </div>\n</ng-container>\n<ng-template #progressTemplate let-instance=\"instance\">\n    <div wmProgressBar minvalue.bind=\"instance.min\" maxvalue.bind=\"instance.max\" datavalue.bind=\"instance.value\"></div>\n</ng-template>",
            providers: [
                provideAsWidgetRef(ProcessManagerComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], ProcessManagerComponent);
    return ProcessManagerComponent;
}());

var ProcessManagementService = /** @class */ (function () {
    function ProcessManagementService() {
    }
    ProcessManagementService.prototype.setUIComponent = function (processManagerComponent) {
        this.processManagerComponent = processManagerComponent;
    };
    /**
     * Returns a promise that will be resolved when an instance is available. At max, 3 instances can only run
     * in parallel and rest has to wait till a process is completed.
     *
     * A progress instance has the following properties.
     *
     *   1) min {number} minimum value, default value is 0 </br>
     *   2) max {number} maximum value, default value is 100 </br>
     *   3) value {number} progress value </br>
     *   4) progressLabel {string} process name </br>
     *   5) stopButtonLabel {string} label for stop button, default value is 'Cancel' </br>
     *   6) onStop {function} function to invoke when stop button is clicked. </br>
     *
     * A progress instance has 3 methods </br>
     *   1) set(property, value) -- sets value to the corresponding property </br>
     *   2) get(property) -- returns property value </br>
     *   3) destroy() -- closes the instance. </br>
     *
     * A progress instance will get auto closed when value and max are equal or when destroy method is called.
     *
     * @param {string} name name of the process whose progress is going to be shown
     * @param {number} min minimum value
     * @param {number} max maximum value
     *
     * @returns {object} a promise
     */
    ProcessManagementService.prototype.createInstance = function (name, min, max) {
        if (!this.processManagerComponent) {
            return Promise.reject('ProcessManagerComponent is missing');
        }
        return this.processManagerComponent.createInstance(name, min, max);
    };
    ProcessManagementService.SERVICE_NAME = 'ProcessManagementService';
    ProcessManagementService.ɵprov = ɵɵdefineInjectable({ factory: function ProcessManagementService_Factory() { return new ProcessManagementService(); }, token: ProcessManagementService, providedIn: "root" });
    ProcessManagementService = __decorate([
        Injectable({ providedIn: 'root' })
    ], ProcessManagementService);
    return ProcessManagementService;
}());

var components = [
    AnchorDirective,
    AppUpdateComponent,
    FileBrowserComponent,
    ImageCacheDirective,
    NetworkInfoToasterComponent,
    ProcessManagerComponent
];
var BasicModule = /** @class */ (function () {
    function BasicModule() {
    }
    BasicModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                ProgressModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], BasicModule);
    return BasicModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { AnchorDirective, AppUpdateComponent, BasicModule, FileBrowserComponent, FileSelectorService, ImageCacheDirective, NetworkInfoToasterComponent, NetworkState, ProcessManagementService, ProcessManagerComponent, registerProps as ɵa };
//# sourceMappingURL=index.js.map
