import { __decorate, __metadata } from 'tslib';
import { Injector, Component, NgModule } from '@angular/core';
import { register, PROP_STRING, PROP_BOOLEAN, StylableComponent, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, PROP_NUMBER, WmComponentsModule } from '@wm/components/base';
import { setCSSFromObj, setCSS, isNumber } from '@wm/core';
import { CommonModule } from '@angular/common';

const registerProps = () => {
    register('wm-segmented-control', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var SegmentedControlComponent_1;
const DEFAULT_CLS = 'app-segmented-control';
const WIDGET_CONFIG = { widgetType: 'wm-segmented-control', hostClass: DEFAULT_CLS };
let SegmentedControlComponent = SegmentedControlComponent_1 = class SegmentedControlComponent extends StylableComponent {
    constructor(inj) {
        super(inj, WIDGET_CONFIG);
        this.contents = [];
        this.currentSelectedIndex = 0;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
    }
    addContent(content) {
        this.contents.push(content);
    }
    goToNext() {
        this.showContent(this.currentSelectedIndex + 1);
    }
    goToPrev() {
        this.showContent(this.currentSelectedIndex - 1);
    }
    ngAfterViewInit() {
        this._$container = this.$element.find('>.app-segments-container');
        const childEls = this._$container.find('>.list-inline >li');
        const maxWidth = `${this.contents.length * 100}%`;
        setCSSFromObj(this._$container[0], {
            maxWidth: maxWidth,
            width: maxWidth,
            'white-space': 'nowrap',
            transition: 'transform 0.2s linear'
        });
        const width = `${100 / this.contents.length}%`;
        for (const child of Array.from(childEls)) {
            setCSS(child, 'width', width);
        }
        this.showContent(0, undefined, true);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    removeContent(content) {
        const index = this.contents.findIndex(c => {
            return c === content;
        });
        if (index >= 0) {
            this.contents.splice(index, 1);
            if (index < this.contents.length) {
                this.showContent(index);
            }
            else if (this.contents.length > 0) {
                this.showContent(0);
            }
        }
    }
    showContent(content, $event, defaultLoad) {
        let index;
        let selectedContent;
        if (isNumber(content)) {
            index = content;
            if (this.contents.length) {
                selectedContent = this.contents[index];
            }
        }
        else {
            selectedContent = content;
            index = this.contents.findIndex(c => {
                return c === content;
            });
        }
        if (selectedContent) {
            selectedContent.loadContent(defaultLoad);
        }
        if (index < 0 || index >= this.contents.length) {
            return;
        }
        if ($event) {
            $event.stopPropagation();
        }
        const eventData = {
            $old: this.currentSelectedIndex,
            $new: index
        };
        this.currentSelectedIndex = index;
        this.invokeEventCallback('beforesegmentchange', eventData);
        setCSS(this._$container[0], 'transform', `translate3d(${-1 * index / this.contents.length * 100}%, 0, 0)`);
        this.invokeEventCallback('segmentchange', eventData);
    }
};
SegmentedControlComponent.initializeProps = registerProps();
SegmentedControlComponent.ctorParameters = () => [
    { type: Injector }
];
SegmentedControlComponent = SegmentedControlComponent_1 = __decorate([
    Component({
        selector: '[wmSegmentedControl]',
        template: "<div class=\"app-segments-container\"> \n    <ul class=\"list-inline\">\n        <ng-content></ng-content>\n    </ul>\n    </div> \n<div class=\"btn-group btn-group-justified\"> \n    <a class=\"btn btn-default\" *ngFor=\"let content of contents; index as i\"\n       [ngClass]=\"{'active btn-primary' : i == currentSelectedIndex}\"\n       (click)=\"showContent(i, $event);\">\n        <i class=\"app-icon\" [ngClass]=\"content.iconclass\"></i>{{content.caption}}\n    </a> \n</div> ",
        providers: [
            provideAsWidgetRef(SegmentedControlComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Injector])
], SegmentedControlComponent);

const registerProps$1 = () => {
    register('wm-segment-content', new Map([
        ['caption', Object.assign({ value: '' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['iconclass', Object.assign({ value: '' }, PROP_STRING)],
        ['loadmode', PROP_STRING],
        ['loaddelay', Object.assign({ value: 10 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var SegmentContentComponent_1;
const DEFAULT_CLS$1 = 'app-segment-content clearfix';
const WIDGET_CONFIG$1 = { widgetType: 'wm-segment-content', hostClass: DEFAULT_CLS$1 };
let SegmentContentComponent = SegmentContentComponent_1 = class SegmentContentComponent extends StylableComponent {
    constructor(segmentedControl, inj) {
        super(inj, WIDGET_CONFIG$1);
        this.segmentedControl = segmentedControl;
        this.compile = false;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        segmentedControl.addContent(this);
    }
    ngAfterViewInit() {
        // load the content on demand when loadmode is not specified
        if (!this.loadmode) {
            this.loadContent(true);
        }
    }
    navigate() {
        this.segmentedControl.showContent(this);
    }
    // sets the compile flag to load the content
    _loadContent() {
        if (!this.compile) {
            this.compile = true;
            this.invokeEventCallback('ready');
        }
    }
    loadContent(defaultLoad) {
        if (this.loadmode === 'after-delay' || defaultLoad) {
            setTimeout(this._loadContent.bind(this), defaultLoad ? 0 : this.loaddelay);
        }
        else {
            this._loadContent();
        }
    }
};
SegmentContentComponent.initializeProps = registerProps$1();
SegmentContentComponent.ctorParameters = () => [
    { type: SegmentedControlComponent },
    { type: Injector }
];
SegmentContentComponent = SegmentContentComponent_1 = __decorate([
    Component({
        selector: '[wmSegmentContent]',
        template: "<ng-content *ngIf=\"compile\"></ng-content>",
        providers: [
            provideAsWidgetRef(SegmentContentComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [SegmentedControlComponent, Injector])
], SegmentContentComponent);

const components = [
    SegmentedControlComponent,
    SegmentContentComponent
];
let SegmentedControlModule = class SegmentedControlModule {
};
SegmentedControlModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], SegmentedControlModule);

/**
 * Generated bundle index. Do not edit.
 */

export { SegmentContentComponent, SegmentedControlComponent, SegmentedControlModule, registerProps as ɵa, registerProps$1 as ɵb };
//# sourceMappingURL=index.js.map
