import { __assign, __extends, __values, __decorate, __metadata, __spread } from 'tslib';
import { Injector, Component, NgModule } from '@angular/core';
import { register, PROP_STRING, PROP_BOOLEAN, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, StylableComponent, PROP_NUMBER, WmComponentsModule } from '@wm/components/base';
import { setCSSFromObj, setCSS, isNumber } from '@wm/core';
import { CommonModule } from '@angular/common';

var registerProps = function () {
    register('wm-segmented-control', new Map([
        ['class', PROP_STRING],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS = 'app-segmented-control';
var WIDGET_CONFIG = { widgetType: 'wm-segmented-control', hostClass: DEFAULT_CLS };
var SegmentedControlComponent = /** @class */ (function (_super) {
    __extends(SegmentedControlComponent, _super);
    function SegmentedControlComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.contents = [];
        _this.currentSelectedIndex = 0;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        return _this;
    }
    SegmentedControlComponent_1 = SegmentedControlComponent;
    SegmentedControlComponent.prototype.addContent = function (content) {
        this.contents.push(content);
    };
    SegmentedControlComponent.prototype.goToNext = function () {
        this.showContent(this.currentSelectedIndex + 1);
    };
    SegmentedControlComponent.prototype.goToPrev = function () {
        this.showContent(this.currentSelectedIndex - 1);
    };
    SegmentedControlComponent.prototype.ngAfterViewInit = function () {
        var e_1, _a;
        this._$container = this.$element.find('>.app-segments-container');
        var childEls = this._$container.find('>.list-inline >li');
        var maxWidth = this.contents.length * 100 + "%";
        setCSSFromObj(this._$container[0], {
            maxWidth: maxWidth,
            width: maxWidth,
            'white-space': 'nowrap',
            transition: 'transform 0.2s linear'
        });
        var width = 100 / this.contents.length + "%";
        try {
            for (var _b = __values(Array.from(childEls)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var child = _c.value;
                setCSS(child, 'width', width);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        this.showContent(0, undefined, true);
    };
    SegmentedControlComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'tabindex') {
            return;
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    SegmentedControlComponent.prototype.removeContent = function (content) {
        var index = this.contents.findIndex(function (c) {
            return c === content;
        });
        if (index >= 0) {
            this.contents.splice(index, 1);
            if (index < this.contents.length) {
                this.showContent(index);
            }
            else if (this.contents.length > 0) {
                this.showContent(0);
            }
        }
    };
    SegmentedControlComponent.prototype.showContent = function (content, $event, defaultLoad) {
        var index;
        var selectedContent;
        if (isNumber(content)) {
            index = content;
            if (this.contents.length) {
                selectedContent = this.contents[index];
            }
        }
        else {
            selectedContent = content;
            index = this.contents.findIndex(function (c) {
                return c === content;
            });
        }
        if (selectedContent) {
            selectedContent.loadContent(defaultLoad);
        }
        if (index < 0 || index >= this.contents.length) {
            return;
        }
        if ($event) {
            $event.stopPropagation();
        }
        var eventData = {
            $old: this.currentSelectedIndex,
            $new: index
        };
        this.currentSelectedIndex = index;
        this.invokeEventCallback('beforesegmentchange', eventData);
        setCSS(this._$container[0], 'transform', "translate3d(" + -1 * index / this.contents.length * 100 + "%, 0, 0)");
        this.invokeEventCallback('segmentchange', eventData);
    };
    var SegmentedControlComponent_1;
    SegmentedControlComponent.initializeProps = registerProps();
    SegmentedControlComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    SegmentedControlComponent = SegmentedControlComponent_1 = __decorate([
        Component({
            selector: '[wmSegmentedControl]',
            template: "<div class=\"app-segments-container\"> \n    <ul class=\"list-inline\">\n        <ng-content></ng-content>\n    </ul>\n    </div> \n<div class=\"btn-group btn-group-justified\"> \n    <a class=\"btn btn-default\" *ngFor=\"let content of contents; index as i\"\n       [ngClass]=\"{'active btn-primary' : i == currentSelectedIndex}\"\n       (click)=\"showContent(i, $event);\">\n        <i class=\"app-icon\" [ngClass]=\"content.iconclass\"></i>{{content.caption}}\n    </a> \n</div> ",
            providers: [
                provideAsWidgetRef(SegmentedControlComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], SegmentedControlComponent);
    return SegmentedControlComponent;
}(StylableComponent));

var registerProps$1 = function () {
    register('wm-segment-content', new Map([
        ['caption', __assign({ value: '' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['content', PROP_STRING],
        ['iconclass', __assign({ value: '' }, PROP_STRING)],
        ['loadmode', PROP_STRING],
        ['loaddelay', __assign({ value: 10 }, PROP_NUMBER)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS$1 = 'app-segment-content clearfix';
var WIDGET_CONFIG$1 = { widgetType: 'wm-segment-content', hostClass: DEFAULT_CLS$1 };
var SegmentContentComponent = /** @class */ (function (_super) {
    __extends(SegmentContentComponent, _super);
    function SegmentContentComponent(segmentedControl, inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG$1) || this;
        _this.segmentedControl = segmentedControl;
        _this.compile = false;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        segmentedControl.addContent(_this);
        return _this;
    }
    SegmentContentComponent_1 = SegmentContentComponent;
    SegmentContentComponent.prototype.ngAfterViewInit = function () {
        // load the content on demand when loadmode is not specified
        if (!this.loadmode) {
            this.loadContent(true);
        }
    };
    SegmentContentComponent.prototype.navigate = function () {
        this.segmentedControl.showContent(this);
    };
    // sets the compile flag to load the content
    SegmentContentComponent.prototype._loadContent = function () {
        if (!this.compile) {
            this.compile = true;
            this.invokeEventCallback('ready');
        }
    };
    SegmentContentComponent.prototype.loadContent = function (defaultLoad) {
        if (this.loadmode === 'after-delay' || defaultLoad) {
            setTimeout(this._loadContent.bind(this), defaultLoad ? 0 : this.loaddelay);
        }
        else {
            this._loadContent();
        }
    };
    var SegmentContentComponent_1;
    SegmentContentComponent.initializeProps = registerProps$1();
    SegmentContentComponent.ctorParameters = function () { return [
        { type: SegmentedControlComponent },
        { type: Injector }
    ]; };
    SegmentContentComponent = SegmentContentComponent_1 = __decorate([
        Component({
            selector: '[wmSegmentContent]',
            template: "<ng-content *ngIf=\"compile\"></ng-content>",
            providers: [
                provideAsWidgetRef(SegmentContentComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [SegmentedControlComponent, Injector])
    ], SegmentContentComponent);
    return SegmentContentComponent;
}(StylableComponent));

var components = [
    SegmentedControlComponent,
    SegmentContentComponent
];
var SegmentedControlModule = /** @class */ (function () {
    function SegmentedControlModule() {
    }
    SegmentedControlModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], SegmentedControlModule);
    return SegmentedControlModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { SegmentContentComponent, SegmentedControlComponent, SegmentedControlModule, registerProps as ɵa, registerProps$1 as ɵb };
//# sourceMappingURL=index.js.map
