import { __decorate, __metadata, __assign, __extends, __spread } from 'tslib';
import { Input, Directive, Injector, ContentChild, TemplateRef, Component, NgModule } from '@angular/core';
import { provideAsWidgetRef, register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, styler, APPLY_STYLES_TYPE, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { addClass, removeClass, $appDigest, isObject, isArray, isString, $parseExpr } from '@wm/core';
import { CommonModule } from '@angular/common';
import { BasicModule as BasicModule$1 } from '@wm/mobile/components/basic';
import { MobileNavbarModule } from '@wm/mobile/components/page/mobile-navbar';
import { PageModule } from '@wm/components/page';
import { BasicModule } from '@wm/components/basic';

var MediaListItemDirective = /** @class */ (function () {
    function MediaListItemDirective() {
    }
    MediaListItemDirective_1 = MediaListItemDirective;
    Object.defineProperty(MediaListItemDirective.prototype, "wmMediaListItem", {
        set: function (val) {
            this.item = val;
        },
        enumerable: true,
        configurable: true
    });
    var MediaListItemDirective_1;
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], MediaListItemDirective.prototype, "wmMediaListItem", null);
    MediaListItemDirective = MediaListItemDirective_1 = __decorate([
        Directive({
            selector: '[wmMediaListItem]',
            providers: [
                provideAsWidgetRef(MediaListItemDirective_1)
            ]
        })
    ], MediaListItemDirective);
    return MediaListItemDirective;
}());

var registerProps = function () {
    register('wm-media-list', new Map([
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['layout', __assign({ value: 'Single-row' }, PROP_STRING)],
        ['mediaurl', PROP_STRING],
        ['name', PROP_STRING],
        ['offline', __assign({ value: true }, PROP_BOOLEAN)],
        ['show', __assign({ value: true }, PROP_BOOLEAN)],
        ['thumbnailheight', __assign({ value: '100pt' }, PROP_STRING)],
        ['thumbnailwidth', __assign({ value: '100pt' }, PROP_STRING)],
        ['thumbnailurl', PROP_STRING],
    ]));
};

var DEFAULT_CLS = 'app-medialist';
var WIDGET_CONFIG = { widgetType: 'wm-media-list', hostClass: DEFAULT_CLS };
var Layout;
(function (Layout) {
    Layout["SINGLE_ROW"] = "Single-row";
    Layout["MULTI_ROW"] = "Multi-row";
})(Layout || (Layout = {}));
var MediaListComponent = /** @class */ (function (_super) {
    __extends(MediaListComponent, _super);
    function MediaListComponent(inj) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.selectedMediaIndex = -1;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        return _this;
    }
    MediaListComponent_1 = MediaListComponent;
    MediaListComponent.prototype.appendToBody = function () {
        var _this = this;
        if (!this.$fullScreenEle) {
            setTimeout(function () {
                _this.$fullScreenEle = _this.$element.find('>.app-media-fullscreen');
                _this.$fullScreenEle.appendTo('body');
            }, 100);
        }
        return true;
    };
    MediaListComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'dataset') {
            this.onDataChange(nv);
        }
        else if (key === 'layout') {
            if (nv === Layout.SINGLE_ROW) {
                addClass(this.nativeElement, 'singlerow');
            }
            else {
                removeClass(this.nativeElement, 'singlerow');
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    MediaListComponent.prototype.exitFullScreen = function () {
        this.selectedMediaIndex = -1;
        this.$fullScreenEle.appendTo(this.$element);
        this.$fullScreenEle = null;
        $appDigest();
    };
    MediaListComponent.prototype.getPicTitle = function () {
        return this.selectedMediaIndex + 1 + '/' + this.fieldDefs.length;
    };
    MediaListComponent.prototype.showFullScreen = function (i) {
        this.selectedMediaIndex = i;
        $appDigest();
    };
    MediaListComponent.prototype.showNext = function () {
        if (this.selectedMediaIndex < this.fieldDefs.length - 1) {
            this.selectedMediaIndex++;
            $appDigest();
        }
    };
    // Returns the field value (src) from the fieldDefs
    MediaListComponent.prototype.getSrc = function (field) {
        if (field) {
            return this.fieldDefs[this.selectedMediaIndex][field];
        }
        return '';
    };
    MediaListComponent.prototype.showPrev = function () {
        if (this.selectedMediaIndex > 0) {
            this.selectedMediaIndex--;
            $appDigest();
        }
    };
    MediaListComponent.prototype.onDataChange = function (nv) {
        if (nv) {
            if (isObject(nv) && !isArray(nv)) {
                nv = [nv];
            }
            if (!this.binddataset) {
                if (isString(nv)) {
                    nv = nv.split(',');
                }
            }
            if (isArray(nv)) {
                this.updateFieldDefs(nv);
            }
        }
        else {
            this.updateFieldDefs([]);
        }
    };
    /** With given data, creates media list items*/
    MediaListComponent.prototype.updateFieldDefs = function (data) {
        var _this = this;
        this.fieldDefs = data;
        data.forEach(function (field) {
            field.mediaUrlVal = $parseExpr(_this.mediaurl)(field);
            field.thumbnailUrlVal = $parseExpr(_this.thumbnailurl)(field);
        });
        this.fieldDefs = data;
    };
    /**
     * used to track list items by Index.
     * @param {number} index value of the list item
     * @returns {number} index.
     */
    MediaListComponent.prototype.listTrackByFn = function (index) {
        return index;
    };
    var MediaListComponent_1;
    MediaListComponent.initializeProps = registerProps();
    MediaListComponent.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    __decorate([
        ContentChild('mediaListTemplate'),
        __metadata("design:type", TemplateRef)
    ], MediaListComponent.prototype, "mediaListTemplate", void 0);
    MediaListComponent = MediaListComponent_1 = __decorate([
        Component({
            selector: '[wmMediaList]',
            template: "<ul class=\"list-unstyled list-inline app-media-thumbnail\">\n    <li *ngFor=\"let item of fieldDefs; index as index;\" [ngStyle]=\"{width: thumbnailwidth, height: thumbnailheight}\" (click)=\"showFullScreen(index)\">\n        <div class=\"thumbnail\">\n            <ng-container [ngTemplateOutlet]=\"imgTemplate\"\n                          [ngTemplateOutletContext]=\"{src: item.thumbnailUrlVal, class: 'thumbnail-image'}\">\n            </ng-container>\n            <div class=\"thumbnail-details\">\n                <ng-container [ngTemplateOutlet]=\"mediaListTemplate\" [ngTemplateOutletContext]=\"{item:item, index:index}\" [wmMediaListItem]=\"item\">\n                </ng-container>\n            </div>\n        </div>\n    </li>\n</ul>\n<div class=\"app-media-fullscreen\" *ngIf=\"selectedMediaIndex >= 0 && appendToBody()\">\n    <header wmMobileNavbar\n            backbutton=\"true\"\n            backbtnclick.event=\"exitFullScreen()\"\n            showLeftnavbtn=\"false\"\n            backbuttoniconclass.event=\"wi wi-chevron-left\"\n            title.bind=\"getPicTitle()\">\n    </header>\n    <div wmContent>\n        <div wmPageContent>\n            <div class=\"media-content\">\n                <div class=\"image-container\"  (swipeleft)=\"showNext()\" (swiperight)=\"showPrev()\">\n                    <ng-container [ngTemplateOutlet]=\"imgTemplate\"\n                                  [ngTemplateOutletContext]=\"{field: 'mediaUrlVal', class: 'center-block'}\">\n                    </ng-container>\n                    <a class=\"app-media-fullscreen-nav-control left\" [hidden]=\"!(selectedMediaIndex > 0)\" (click)=\"showPrev()\">\n                        <i class=\"wi wi-chevron-left\"></i>\n                    </a>\n                    <a class=\"app-media-fullscreen-nav-control right\" [hidden]=\"!(selectedMediaIndex < fieldDefs.length-1)\" (click)=\"showNext()\">\n                        <i class=\"wi wi-chevron-right\"></i>\n                    </a>\n                </div>\n            </div>\n        </div>\n    </div>\n</div>\n\n<ng-template #imgTemplate let-src=\"src\" let-classname=\"class\" let-field=\"field\">\n    <img wmPicture class=\"{{classname}}\" picturesource.bind=\"src || getSrc(field)\" wmImageCache=\"{{offline}}\" />\n</ng-template>\n",
            providers: [
                provideAsWidgetRef(MediaListComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Injector])
    ], MediaListComponent);
    return MediaListComponent;
}(StylableComponent));

var components = [
    MediaListItemDirective,
    MediaListComponent
];
var MediaListModule = /** @class */ (function () {
    function MediaListModule() {
    }
    MediaListModule = __decorate([
        NgModule({
            imports: [
                BasicModule,
                CommonModule,
                BasicModule$1,
                MobileNavbarModule,
                PageModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], MediaListModule);
    return MediaListModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { MediaListComponent, MediaListItemDirective, MediaListModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
