import { __decorate, __metadata } from 'tslib';
import { Injector, HostListener, Component, NgModule } from '@angular/core';
import { BarcodeScanner } from '@ionic-native/barcode-scanner';
import { register, PROP_STRING, PROP_BOOLEAN, StylableComponent, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { hasCordova } from '@wm/core';
import { CommonModule } from '@angular/common';

const registerProps = () => {
    register('wm-barcodescanner', new Map([
        ['barcodeformat', Object.assign({ value: 'ALL' }, PROP_STRING)],
        ['caption', Object.assign({ value: '' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['iconclass', Object.assign({ value: 'glyphicon glyphicon-barcode' }, PROP_STRING)],
        ['iconsize', Object.assign({ value: '2em' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var BarcodeScannerComponent_1;
const DEFAULT_CLS = 'btn app-barcode';
const WIDGET_CONFIG = { widgetType: 'wm-barcodescanner', hostClass: DEFAULT_CLS };
let BarcodeScannerComponent = BarcodeScannerComponent_1 = class BarcodeScannerComponent extends StylableComponent {
    constructor(scanner, inj) {
        super(inj, WIDGET_CONFIG);
        this.scanner = scanner;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
    }
    openBarcodescanner($event) {
        this.scan().then(text => {
            this.datavalue = text;
            this.invokeEventCallback('success', { $event });
        });
    }
    scan() {
        let options;
        if (hasCordova()) {
            if (this.barcodeformat && this.barcodeformat !== 'ALL') {
                options = {
                    formats: this.barcodeformat
                };
            }
            return this.scanner.scan(options)
                .then(data => data.text);
        }
        return Promise.resolve('BAR_CODE');
    }
};
BarcodeScannerComponent.initializeProps = registerProps();
BarcodeScannerComponent.ctorParameters = () => [
    { type: BarcodeScanner },
    { type: Injector }
];
__decorate([
    HostListener('click', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], BarcodeScannerComponent.prototype, "openBarcodescanner", null);
BarcodeScannerComponent = BarcodeScannerComponent_1 = __decorate([
    Component({
        selector: 'button[wmBarcodescanner]',
        template: "<i [ngClass]=\"iconclass\" [ngStyle]=\"{'font-size': iconsize}\"></i>\n<span class=\"btn-caption\" [textContent]=\"caption\"></span>",
        providers: [
            provideAsWidgetRef(BarcodeScannerComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [BarcodeScanner, Injector])
], BarcodeScannerComponent);

const components = [
    BarcodeScannerComponent
];
let BarcodeScannerModule = class BarcodeScannerModule {
};
BarcodeScannerModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], BarcodeScannerModule);

/**
 * Generated bundle index. Do not edit.
 */

export { BarcodeScannerComponent, BarcodeScannerModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
