import { __decorate, __metadata } from 'tslib';
import { Injector, ElementRef, ChangeDetectorRef, HostListener, Component, NgModule } from '@angular/core';
import { Camera } from '@ionic-native/camera';
import { MediaCapture } from '@ionic-native/media-capture';
import { register, PROP_BOOLEAN, PROP_STRING, PROP_NUMBER, StylableComponent, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { hasCordova, convertToBlob } from '@wm/core';
import { CommonModule } from '@angular/common';

const registerProps = () => {
    register('wm-camera', new Map([
        ['allowedit', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['capturetype', Object.assign({ value: 'IMAGE' }, PROP_STRING)],
        ['caption', Object.assign({ value: '' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['correctorientation', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['datavalue', Object.assign({ value: '' }, PROP_STRING)],
        ['iconclass', Object.assign({ value: 'wi wi-photo-camera' }, PROP_STRING)],
        ['iconsize', Object.assign({ value: '2em' }, PROP_STRING)],
        ['imagequality', Object.assign({ value: 80 }, PROP_NUMBER)],
        ['imageencodingtype', Object.assign({ value: 'JPEG' }, PROP_STRING)],
        ['imagetargetwidth', PROP_NUMBER],
        ['imagetargetheight', PROP_NUMBER],
        ['localFile', Object.assign({ value: '' }, PROP_STRING)],
        ['localFilePath', Object.assign({ value: '' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['savetogallery', Object.assign({ value: false }, PROP_BOOLEAN)],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var CameraComponent_1;
const DEFAULT_CLS = 'btn app-camera';
const WIDGET_CONFIG = { widgetType: 'wm-camera', hostClass: DEFAULT_CLS };
var CAPTURE_TYPE;
(function (CAPTURE_TYPE) {
    CAPTURE_TYPE["IMAGE"] = "IMAGE";
    CAPTURE_TYPE["PNG"] = "PNG";
})(CAPTURE_TYPE || (CAPTURE_TYPE = {}));
var ENCODING_TYPE;
(function (ENCODING_TYPE) {
    ENCODING_TYPE["JPEG"] = "JPEG";
    ENCODING_TYPE["PNG"] = "PNG";
})(ENCODING_TYPE || (ENCODING_TYPE = {}));
let CameraComponent = CameraComponent_1 = class CameraComponent extends StylableComponent {
    constructor(camera, mediaCapture, inj, elRef, cdr) {
        super(inj, WIDGET_CONFIG);
        this.camera = camera;
        this.mediaCapture = mediaCapture;
        styler(this.nativeElement, this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
    }
    openCamera($event) {
        if (hasCordova()) {
            if (this.capturetype === CAPTURE_TYPE.IMAGE) {
                this._cameraOptions = {
                    quality: this.imagequality,
                    destinationType: 1,
                    sourceType: 1,
                    allowEdit: this.allowedit,
                    correctOrientation: this.correctorientation,
                    encodingType: this.imageencodingtype === ENCODING_TYPE.JPEG ? 0 : 1,
                    saveToPhotoAlbum: this.savetogallery,
                    targetWidth: this.imagetargetwidth,
                    targetHeight: this.imagetargetheight
                };
                // start camera
                this.camera.getPicture(this._cameraOptions)
                    .then(path => this.updateModel($event, path));
            }
            else {
                this._cameraOptions = {
                    limit: 1
                };
                // start video capture
                this.mediaCapture.captureVideo(this._cameraOptions)
                    .then(mediaFiles => this.updateModel($event, mediaFiles[0].fullPath));
            }
        }
        else {
            this.invokeEventCallback('success', { $event });
        }
    }
    updateModel($event, value) {
        value = (value.startsWith('file://') ? '' : 'file://') + value;
        this.localFilePath = this.datavalue = value;
        convertToBlob(value)
            .then(result => {
            this.localFile = result.blob;
            this.invokeEventCallback('success', { $event, localFilePath: this.localFilePath, localFile: this.localFile });
        }, () => {
            this.localFile = undefined;
        });
    }
};
CameraComponent.initializeProps = registerProps();
CameraComponent.ctorParameters = () => [
    { type: Camera },
    { type: MediaCapture },
    { type: Injector },
    { type: ElementRef },
    { type: ChangeDetectorRef }
];
__decorate([
    HostListener('click', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CameraComponent.prototype, "openCamera", null);
CameraComponent = CameraComponent_1 = __decorate([
    Component({
        selector: 'button[wmCamera]',
        template: "<i [ngClass]=\"iconclass\" [ngStyle]=\"{'font-size': iconsize}\"></i>",
        providers: [
            provideAsWidgetRef(CameraComponent_1)
        ]
    }),
    __metadata("design:paramtypes", [Camera, MediaCapture, Injector, ElementRef, ChangeDetectorRef])
], CameraComponent);

const components = [
    CameraComponent
];
let CameraModule = class CameraModule {
};
CameraModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], CameraModule);

/**
 * Generated bundle index. Do not edit.
 */

export { CAPTURE_TYPE, CameraComponent, CameraModule, ENCODING_TYPE, registerProps as ɵa };
//# sourceMappingURL=index.js.map
