import { __assign, __extends, __decorate, __metadata, __spread } from 'tslib';
import { Injector, ElementRef, ChangeDetectorRef, HostListener, Component, NgModule } from '@angular/core';
import { Camera } from '@ionic-native/camera';
import { MediaCapture } from '@ionic-native/media-capture';
import { register, PROP_BOOLEAN, PROP_STRING, PROP_NUMBER, styler, APPLY_STYLES_TYPE, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { hasCordova, convertToBlob } from '@wm/core';
import { CommonModule } from '@angular/common';

var registerProps = function () {
    register('wm-camera', new Map([
        ['allowedit', __assign({ value: false }, PROP_BOOLEAN)],
        ['capturetype', __assign({ value: 'IMAGE' }, PROP_STRING)],
        ['caption', __assign({ value: '' }, PROP_STRING)],
        ['class', PROP_STRING],
        ['correctorientation', __assign({ value: false }, PROP_BOOLEAN)],
        ['datavalue', __assign({ value: '' }, PROP_STRING)],
        ['iconclass', __assign({ value: 'wi wi-photo-camera' }, PROP_STRING)],
        ['iconsize', __assign({ value: '2em' }, PROP_STRING)],
        ['imagequality', __assign({ value: 80 }, PROP_NUMBER)],
        ['imageencodingtype', __assign({ value: 'JPEG' }, PROP_STRING)],
        ['imagetargetwidth', PROP_NUMBER],
        ['imagetargetheight', PROP_NUMBER],
        ['localFile', __assign({ value: '' }, PROP_STRING)],
        ['localFilePath', __assign({ value: '' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['savetogallery', __assign({ value: false }, PROP_BOOLEAN)],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS = 'btn app-camera';
var WIDGET_CONFIG = { widgetType: 'wm-camera', hostClass: DEFAULT_CLS };
var CAPTURE_TYPE;
(function (CAPTURE_TYPE) {
    CAPTURE_TYPE["IMAGE"] = "IMAGE";
    CAPTURE_TYPE["PNG"] = "PNG";
})(CAPTURE_TYPE || (CAPTURE_TYPE = {}));
var ENCODING_TYPE;
(function (ENCODING_TYPE) {
    ENCODING_TYPE["JPEG"] = "JPEG";
    ENCODING_TYPE["PNG"] = "PNG";
})(ENCODING_TYPE || (ENCODING_TYPE = {}));
var CameraComponent = /** @class */ (function (_super) {
    __extends(CameraComponent, _super);
    function CameraComponent(camera, mediaCapture, inj, elRef, cdr) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.camera = camera;
        _this.mediaCapture = mediaCapture;
        styler(_this.nativeElement, _this, APPLY_STYLES_TYPE.SCROLLABLE_CONTAINER);
        return _this;
    }
    CameraComponent_1 = CameraComponent;
    CameraComponent.prototype.openCamera = function ($event) {
        var _this = this;
        if (hasCordova()) {
            if (this.capturetype === CAPTURE_TYPE.IMAGE) {
                this._cameraOptions = {
                    quality: this.imagequality,
                    destinationType: 1,
                    sourceType: 1,
                    allowEdit: this.allowedit,
                    correctOrientation: this.correctorientation,
                    encodingType: this.imageencodingtype === ENCODING_TYPE.JPEG ? 0 : 1,
                    saveToPhotoAlbum: this.savetogallery,
                    targetWidth: this.imagetargetwidth,
                    targetHeight: this.imagetargetheight
                };
                // start camera
                this.camera.getPicture(this._cameraOptions)
                    .then(function (path) { return _this.updateModel($event, path); });
            }
            else {
                this._cameraOptions = {
                    limit: 1
                };
                // start video capture
                this.mediaCapture.captureVideo(this._cameraOptions)
                    .then(function (mediaFiles) { return _this.updateModel($event, mediaFiles[0].fullPath); });
            }
        }
        else {
            this.invokeEventCallback('success', { $event: $event });
        }
    };
    CameraComponent.prototype.updateModel = function ($event, value) {
        var _this = this;
        value = (value.startsWith('file://') ? '' : 'file://') + value;
        this.localFilePath = this.datavalue = value;
        convertToBlob(value)
            .then(function (result) {
            _this.localFile = result.blob;
            _this.invokeEventCallback('success', { $event: $event, localFilePath: _this.localFilePath, localFile: _this.localFile });
        }, function () {
            _this.localFile = undefined;
        });
    };
    var CameraComponent_1;
    CameraComponent.initializeProps = registerProps();
    CameraComponent.ctorParameters = function () { return [
        { type: Camera },
        { type: MediaCapture },
        { type: Injector },
        { type: ElementRef },
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        HostListener('click', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], CameraComponent.prototype, "openCamera", null);
    CameraComponent = CameraComponent_1 = __decorate([
        Component({
            selector: 'button[wmCamera]',
            template: "<i [ngClass]=\"iconclass\" [ngStyle]=\"{'font-size': iconsize}\"></i>",
            providers: [
                provideAsWidgetRef(CameraComponent_1)
            ]
        }),
        __metadata("design:paramtypes", [Camera, MediaCapture, Injector, ElementRef, ChangeDetectorRef])
    ], CameraComponent);
    return CameraComponent;
}(StylableComponent));

var components = [
    CameraComponent
];
var CameraModule = /** @class */ (function () {
    function CameraModule() {
    }
    CameraModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], CameraModule);
    return CameraModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { CAPTURE_TYPE, CameraComponent, CameraModule, ENCODING_TYPE, registerProps as ɵa };
//# sourceMappingURL=index.js.map
