import { __decorate, __metadata, __param } from 'tslib';
import { Injector, Attribute, ViewChild, Component, NgModule } from '@angular/core';
import { App, AbstractNavigationService } from '@wm/core';
import { PROP_BOOLEAN, PROP_STRING, PROP_ANY, PROP_NUMBER, register, BaseComponent, getImageUrl, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { PageDirective } from '@wm/components/page';
import { SearchComponent, SearchModule } from '@wm/components/basic/search';
import { DeviceService } from '@wm/mobile/core';
import { CommonModule } from '@angular/common';

const navbarProps = new Map([
    ['backbutton', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['backbuttoniconclass', Object.assign({ value: 'wi wi-back' }, PROP_STRING)],
    ['backbuttonlabel', Object.assign({ value: '' }, PROP_STRING)],
    ['backgroundattachment', PROP_STRING],
    ['backgroundcolor', PROP_STRING],
    ['backgroundgradient', PROP_STRING],
    ['backgroundimage', PROP_STRING],
    ['backgroundposition', PROP_STRING],
    ['backgroundrepeat', PROP_STRING],
    ['backgroundsize', PROP_STRING],
    ['class', PROP_STRING],
    ['datafield', Object.assign({ value: 'All Fields' }, PROP_STRING)],
    ['dataset', PROP_ANY],
    ['datasource', PROP_STRING],
    ['datavalue', PROP_STRING],
    ['debouncetime', Object.assign({ value: 250 }, PROP_NUMBER)],
    ['defaultview', PROP_STRING],
    ['displayimagesrc', PROP_STRING],
    ['displaylabel', PROP_STRING],
    ['imgsrc', PROP_STRING],
    ['query', Object.assign({ value: '' }, PROP_STRING)],
    ['leftnavpaneliconclass', Object.assign({ value: 'wi wi-menu' }, PROP_STRING)],
    ['matchmode', PROP_STRING],
    ['name', PROP_STRING],
    ['readonlySearchBar', PROP_BOOLEAN],
    ['searchbutton', Object.assign({ value: false }, PROP_BOOLEAN)],
    ['searchbuttoniconclass', Object.assign({ value: 'wi wi-search' }, PROP_STRING)],
    ['searchbuttonlabel', Object.assign({ value: '' }, PROP_STRING)],
    ['show', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['showLeftnavbtn', Object.assign({ value: true }, PROP_BOOLEAN)],
    ['searchkey', PROP_STRING],
    ['searchplaceholder', Object.assign({ value: 'Search' }, PROP_STRING)],
    ['showSearchbar', PROP_BOOLEAN],
    ['title', PROP_STRING]
]);
const registerProps = () => {
    register('wm-mobile-navbar', navbarProps);
};

var MobileNavbarComponent_1;
const DEFAULT_CLS = 'app-mobile-header app-mobile-navbar';
const WIDGET_CONFIG = { widgetType: 'wm-mobile-navbar', hostClass: DEFAULT_CLS };
let MobileNavbarComponent = MobileNavbarComponent_1 = class MobileNavbarComponent extends BaseComponent {
    constructor(app, page, deviceService, navigationService, inj, backbtnClickEvt) {
        super(inj, WIDGET_CONFIG);
        this.page = page;
        this.deviceService = deviceService;
        this.navigationService = navigationService;
        this.backbtnClickEvt = backbtnClickEvt;
        this._isReady = false;
        page.subscribe('wmLeftPanel:ready', (leftNavPanel) => {
            if (this.showLeftnavbtn) {
                this.leftNavPanel = leftNavPanel;
            }
        });
        this._backBtnListenerDestroyer = deviceService.onBackButtonTap($event => {
            if (this._isReady) {
                if (this.backbtnClickEvt) {
                    this.invokeEventCallback('backbtnclick', { $event });
                    return false;
                }
            }
        });
        this.binddisplaylabel = this.nativeElement.getAttribute('displaylabel.bind');
        setTimeout(() => this._isReady = true, 1000);
    }
    // getter setter is added to pass the datasource to searchcomponent.
    get datasource() {
        return this._datasource;
    }
    set datasource(nv) {
        this._datasource = nv;
        this.searchComponent.datasource = nv;
    }
    ngAfterViewInit() {
        this.searchComponent.binddisplayimagesrc = this.binddisplayimagesrc;
        this.searchComponent.displayimagesrc = this.binddisplayimagesrc ? this.binddisplayimagesrc : this.displayimagesrc;
        this.searchComponent.displaylabel = this.displaylabel;
        this.searchComponent.binddisplaylabel = this.binddisplaylabel;
        this.searchComponent.datafield = this.datafield;
        this.searchComponent.binddataset = this.binddataset;
        this.searchComponent.dataset = this.dataset;
        this.searchComponent.searchkey = this.searchkey;
        this.searchComponent.datasource = this.datasource;
        this.searchComponent.matchmode = this.matchmode;
    }
    goBack($event) {
        /**
         * TODO: while trying navigating from details page to edit page in wavereads, app is navigating
         * as details -> editPage -> details. For now, keeping this callback to react after 1 second.
         */
        this.deviceService.executeBackTapListeners($event);
    }
    ngOnDestroy() {
        this._backBtnListenerDestroyer();
        super.ngOnDestroy();
    }
    onPropertyChange(key, nv, ov) {
        if (this.searchComponent) {
            if (key === 'datafield') {
                this.searchComponent.datafield = this.datafield;
            }
            if (key === 'displaylabel') {
                this.searchComponent.displaylabel = this.displaylabel;
            }
        }
        if (key === 'imgsrc') {
            this.imagesrc = getImageUrl(nv);
        }
        else if (key === 'dataset') {
            // $is._dataset = nv;
        }
        else if (key === 'defaultview') {
            this.showSearchbar = (nv === 'searchview');
        }
        else if (key === 'datavalue') {
            this.query = nv;
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    onSubmission($event) {
        this.invokeEventCallback('search', { $event });
    }
    // switches the view based on defaultview
    switchView(view) {
        this.showSearchbar = (view !== 'actionview');
    }
    // goto previous view or page
    goBacktoPreviousView($event) {
        if (this.defaultview === 'actionview') {
            // switches the view from search to action or action to search.
            this.switchView('actionview');
        }
        else {
            // goes back to the previous visited page.
            this.goBack($event);
        }
    }
    onSelect($event, widget, selectedValue) {
        this.datavalue = selectedValue;
        this.query = widget.query;
        this.invokeEventCallback('change', {
            $event,
            newVal: selectedValue,
            oldVal: widget.prevDatavalue
        });
    }
    onClear() {
        this.datavalue = '';
        this.query = '';
    }
};
MobileNavbarComponent.initializeProps = registerProps();
MobileNavbarComponent.ctorParameters = () => [
    { type: App },
    { type: PageDirective },
    { type: DeviceService },
    { type: AbstractNavigationService },
    { type: Injector },
    { type: undefined, decorators: [{ type: Attribute, args: ['backbtnclick.event',] }] }
];
__decorate([
    ViewChild(SearchComponent, { static: true }),
    __metadata("design:type", SearchComponent)
], MobileNavbarComponent.prototype, "searchComponent", void 0);
MobileNavbarComponent = MobileNavbarComponent_1 = __decorate([
    Component({
        selector: 'header[wmMobileNavbar]',
        template: "<nav class=\"navbar\" *ngIf=\"!showSearchbar\">\n    <div class=\"mobile-navbar-left\">\n        <ul class=\"nav navbar-nav navbar-left\">\n            <li *ngIf=\"leftNavPanel\" >\n                <a (click)=\"leftNavPanel.toggle();\"  href=\"javascript: void(0);\">\n                    <i [ngClass]=\"leftnavpaneliconclass\"></i>\n                    </a>\n                </li>\n            <li *ngIf=\"backbutton\">\n                <a class=\"btn-back\" type=\"button\" (click)=\"goBack($event)\"  href=\"javascript: void(0);\">\n                    <i [ngClass]=\"backbuttoniconclass\"></i><span [innerText]=\"backbuttonlabel\"></span>\n                </a>\n            </li>\n        </ul>\n    </div>\n    <div class=\"mobile-navbar-center\">\n        <div class=\"navbar-header\">\n            <h1 class=\"navbar-brand\">\n                <img data-identifier=\"img\" class=\"brand-image\" [alt]=\"title\" width=\"32\" height=\"32\" *ngIf=\"imgsrc\" [src]=\"imgsrc\"/>\n                <span class=\"title\" [innerText]=\"title || ''\"></span>\n                </h1>\n            </div>\n        </div>\n    <div class=\"mobile-navbar-right\">\n        <ul class=\"nav navbar-nav navbar-right\">\n            <li>\n                <ng-content></ng-content>\n            </li>\n            <li *ngIf=\"searchbutton\">\n                <a class=\"btn-search btn-transparent\" type=\"button\" (click)=\"showSearchbar = true\"  href=\"javascript: void(0);\">\n                    <i [ngClass]=\"searchbuttoniconclass\"></i><span [innerText]=\"searchbuttonlabel\"></span>\n                </a>\n            </li>\n        </ul>\n    </div>\n</nav>\n<nav class=\"navbar searchbar\" [hidden]=\"!showSearchbar\">\n    <div class=\"mobile-navbar-left\">\n        <ul class=\"nav navbar-nav navbar-left\">\n            <li>\n                <a class=\"btn-back\" type=\"button\" (click)=\"goBacktoPreviousView($event)\" href=\"javascript:void(0);\">\n                    <i [ngClass]=\"backbuttoniconclass\"></i>\n                </a>\n            </li>\n        </ul>\n    </div>\n    <div class=\"mobile-navbar-center search-container\">\n        <div wmSearch query.bind=\"query\"\n             clearsearch.event=\"onClear();\"\n             select.event=\"onSelect($event, widget, selectedValue)\"\n             searchkey.bind=\"searchkey\"\n             dataset.bind=\"dataset\"\n             displayimagesrc.bind=\"displayimagesrc\"\n             datavalue.bind=\"datavalue\"\n             submit.event=\"onSubmission($event, widget, value)\"\n             placeholder.bind=\"searchplaceholder\"\n             navsearchbar=\"true\"\n             readonly.bind=\"readonlySearchBar\">\n        </div>\n    </div>\n    <div class=\"mobile-navbar-right\">\n        <ul class=\"nav navbar-nav navbar-right\">\n            <li>\n                <a class=\"btn-cancel btn-transparent\" type=\"button\" (click)=\"showSearchbar = false;\"  href=\"javascript: void(0);\"> Cancel </a>\n            </li>\n        </ul>\n    </div>\n</nav>",
        providers: [
            provideAsWidgetRef(MobileNavbarComponent_1)
        ]
    }),
    __param(5, Attribute('backbtnclick.event')),
    __metadata("design:paramtypes", [App,
        PageDirective,
        DeviceService,
        AbstractNavigationService,
        Injector, Object])
], MobileNavbarComponent);

const components = [
    MobileNavbarComponent
];
let MobileNavbarModule = class MobileNavbarModule {
};
MobileNavbarModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            SearchModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], MobileNavbarModule);

/**
 * Generated bundle index. Do not edit.
 */

export { MobileNavbarComponent, MobileNavbarModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
