import { __decorate, __param, __metadata } from 'tslib';
import { Injector, Attribute, Component, NgModule } from '@angular/core';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, StylableComponent, styler, getEvaluatedData, provideAsWidgetRef, WmComponentsModule } from '@wm/components/base';
import { PageDirective } from '@wm/components/page';
import { CommonModule } from '@angular/common';

const registerProps = () => {
    register('wm-mobile-tabbar', new Map([
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['itemicon', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['morebuttoniconclass', Object.assign({ value: 'wi wi-more-horiz' }, PROP_STRING)],
        ['morebuttonlabel', Object.assign({ value: 'more' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', Object.assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var MobileTabbarComponent_1;
const DEFAULT_CLS = 'app-tabbar app-top-nav';
const WIDGET_CONFIG = { widgetType: 'wm-mobile-tabbar', hostClass: DEFAULT_CLS };
let MobileTabbarComponent = MobileTabbarComponent_1 = class MobileTabbarComponent extends StylableComponent {
    constructor(page, inj, binditemlabel, binditemicon, binditemlink) {
        super(inj, WIDGET_CONFIG);
        this.page = page;
        this.binditemlabel = binditemlabel;
        this.binditemicon = binditemicon;
        this.binditemlink = binditemlink;
        this.tabItems = [];
        this.layout = {};
        this._layouts = [
            { minwidth: 2048, max: 12 },
            { minwidth: 1024, max: 10 },
            { minwidth: 768, max: 7 },
            { minwidth: 480, max: 5 },
            { minwidth: 0, max: 4 }
        ];
        styler(this.nativeElement, this);
        page.notify('wmMobileTabbar:ready', this);
    }
    onPropertyChange(key, nv, ov) {
        if (key === 'dataset') {
            if (nv) {
                this.tabItems = this.getTabItems(nv);
            }
        }
        else {
            super.onPropertyChange(key, nv, ov);
        }
    }
    ngAfterViewInit() {
        setTimeout(() => {
            this.layout = this.getSuitableLayout();
            $(window).on('resize.tabbar', _.debounce(() => this.layout = this.getSuitableLayout(), 20));
        });
        super.ngAfterViewInit();
    }
    ngOnDestroy() {
        $(window).off('.tabbar');
        super.ngOnDestroy();
    }
    // triggered on item selection
    onItemSelect($event, selectedItem) {
        this.invokeEventCallback('select', { $event, $item: selectedItem.value || selectedItem.label });
    }
    getItems(itemArray) {
        return itemArray.map(item => ({
            label: item,
            icon: 'wi wi-' + item
        }));
    }
    getSuitableLayout() {
        const avaiableWidth = $(this.nativeElement).parent().width();
        return this._layouts.find(l => avaiableWidth >= l.minwidth);
    }
    getTabItems(value) {
        if (_.isArray(value)) {
            if (_.isObject(value[0])) {
                return value.map(item => {
                    const link = getEvaluatedData(item, { expression: 'itemlink', 'bindExpression': this.binditemlink }, this.viewParent) || item.link;
                    const activePageName = window.location.hash.substr(2);
                    return {
                        label: getEvaluatedData(item, { expression: 'itemlabel', bindExpression: this.binditemlabel }, this.viewParent) || item.label,
                        icon: getEvaluatedData(item, { expression: 'itemicon', bindExpression: this.binditemicon }, this.viewParent) || item.icon,
                        link: link,
                        active: _.includes([activePageName, '#' + activePageName, '#/' + activePageName], link)
                    };
                });
            }
            else {
                return this.getItems(value);
            }
        }
        else if (_.isString(value)) {
            return this.getItems(value.split(','));
        }
    }
};
MobileTabbarComponent.initializeProps = registerProps();
MobileTabbarComponent.ctorParameters = () => [
    { type: PageDirective },
    { type: Injector },
    { type: undefined, decorators: [{ type: Attribute, args: ['itemlabel.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['itemicon.bind',] }] },
    { type: undefined, decorators: [{ type: Attribute, args: ['itemicon.bind',] }] }
];
MobileTabbarComponent = MobileTabbarComponent_1 = __decorate([
    Component({
        selector: 'div[wmMobileTabbar]',
        template: "<nav class=\"navbar navbar-default\">\n    <ul class=\"tab-items nav navbar-nav\">\n        <li class=\"tab-item\" *ngFor=\"let item of tabItems; index as i\" [class.hidden]=\"!(tabItems.length === layout.max || i < layout.max)\" >\n            <a [class.active]=\"item.active\" [href]=\"(item.link || 'javascript:void(0)')| trustAs: 'resource'\" (click)=\"onItemSelect($event, item)\">\n                <i class=\"app-icon\" [ngClass]=\"item.icon\"></i><label>{{item.label}}</label>\n            </a>\n        </li>\n        <li class=\"menu-items dropdown\" [class.hidden]=\"tabItems.length <= layout.max\" [ngClass]=\"{dropup : position === bottom}\">\n            <a (click)=\"showMoreMenu = !showMoreMenu\" href=\"javascript:void(0)\">\n                <i class=\"app-icon {{morebuttoniconclass}}\"></i><label>{{morebuttonlabel}}</label>\n            </a>\n            <ul class=\"dropdown-menu dropdown-menu-right\" [ngClass]=\"{'nav navbar-nav' : menutype === thumbnail}\" *ngIf=\"showMoreMenu\">\n                <li role=\"menuitem\" class=\"menu-item\" *ngFor=\"let item of tabItems;index as i\" [class.hidden]=\"i < layout.max\">\n                    <a [ngClass]=\"{active : item.active}\" [href]=\"(item.link || 'javascript:void(0)')| trustAs: 'resource'\" (click)=\"onItemSelect($event, item)\" class=\"dropdown-item\">\n                        <i class=\"app-icon\" [ngClass]=\"item.icon\"></i><label>{{item.label}}</label>\n                    </a>\n                </li>\n            </ul>\n        </li>\n    </ul>\n</nav>\n",
        providers: [
            provideAsWidgetRef(MobileTabbarComponent_1)
        ]
    }),
    __param(2, Attribute('itemlabel.bind')),
    __param(3, Attribute('itemicon.bind')),
    __param(4, Attribute('itemicon.bind')),
    __metadata("design:paramtypes", [PageDirective,
        Injector, Object, Object, Object])
], MobileTabbarComponent);

const components = [
    MobileTabbarComponent
];
let TabBarModule = class TabBarModule {
};
TabBarModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            WmComponentsModule
        ],
        declarations: [...components],
        exports: [...components],
        entryComponents: []
    })
], TabBarModule);

/**
 * Generated bundle index. Do not edit.
 */

export { MobileTabbarComponent, TabBarModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
