import { __assign, __extends, __decorate, __param, __metadata, __spread } from 'tslib';
import { Injector, Attribute, Component, NgModule } from '@angular/core';
import { register, PROP_STRING, PROP_ANY, PROP_BOOLEAN, styler, getEvaluatedData, provideAsWidgetRef, StylableComponent, WmComponentsModule } from '@wm/components/base';
import { PageDirective } from '@wm/components/page';
import { CommonModule } from '@angular/common';

var registerProps = function () {
    register('wm-mobile-tabbar', new Map([
        ['class', PROP_STRING],
        ['dataset', PROP_ANY],
        ['itemicon', PROP_STRING],
        ['itemlabel', PROP_STRING],
        ['itemlink', PROP_STRING],
        ['morebuttoniconclass', __assign({ value: 'wi wi-more-horiz' }, PROP_STRING)],
        ['morebuttonlabel', __assign({ value: 'more' }, PROP_STRING)],
        ['name', PROP_STRING],
        ['show', __assign({ value: true }, PROP_BOOLEAN)]
    ]));
};

var DEFAULT_CLS = 'app-tabbar app-top-nav';
var WIDGET_CONFIG = { widgetType: 'wm-mobile-tabbar', hostClass: DEFAULT_CLS };
var MobileTabbarComponent = /** @class */ (function (_super) {
    __extends(MobileTabbarComponent, _super);
    function MobileTabbarComponent(page, inj, binditemlabel, binditemicon, binditemlink) {
        var _this = _super.call(this, inj, WIDGET_CONFIG) || this;
        _this.page = page;
        _this.binditemlabel = binditemlabel;
        _this.binditemicon = binditemicon;
        _this.binditemlink = binditemlink;
        _this.tabItems = [];
        _this.layout = {};
        _this._layouts = [
            { minwidth: 2048, max: 12 },
            { minwidth: 1024, max: 10 },
            { minwidth: 768, max: 7 },
            { minwidth: 480, max: 5 },
            { minwidth: 0, max: 4 }
        ];
        styler(_this.nativeElement, _this);
        page.notify('wmMobileTabbar:ready', _this);
        return _this;
    }
    MobileTabbarComponent_1 = MobileTabbarComponent;
    MobileTabbarComponent.prototype.onPropertyChange = function (key, nv, ov) {
        if (key === 'dataset') {
            if (nv) {
                this.tabItems = this.getTabItems(nv);
            }
        }
        else {
            _super.prototype.onPropertyChange.call(this, key, nv, ov);
        }
    };
    MobileTabbarComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        setTimeout(function () {
            _this.layout = _this.getSuitableLayout();
            $(window).on('resize.tabbar', _.debounce(function () { return _this.layout = _this.getSuitableLayout(); }, 20));
        });
        _super.prototype.ngAfterViewInit.call(this);
    };
    MobileTabbarComponent.prototype.ngOnDestroy = function () {
        $(window).off('.tabbar');
        _super.prototype.ngOnDestroy.call(this);
    };
    // triggered on item selection
    MobileTabbarComponent.prototype.onItemSelect = function ($event, selectedItem) {
        this.invokeEventCallback('select', { $event: $event, $item: selectedItem.value || selectedItem.label });
    };
    MobileTabbarComponent.prototype.getItems = function (itemArray) {
        return itemArray.map(function (item) { return ({
            label: item,
            icon: 'wi wi-' + item
        }); });
    };
    MobileTabbarComponent.prototype.getSuitableLayout = function () {
        var avaiableWidth = $(this.nativeElement).parent().width();
        return this._layouts.find(function (l) { return avaiableWidth >= l.minwidth; });
    };
    MobileTabbarComponent.prototype.getTabItems = function (value) {
        var _this = this;
        if (_.isArray(value)) {
            if (_.isObject(value[0])) {
                return value.map(function (item) {
                    var link = getEvaluatedData(item, { expression: 'itemlink', 'bindExpression': _this.binditemlink }, _this.viewParent) || item.link;
                    var activePageName = window.location.hash.substr(2);
                    return {
                        label: getEvaluatedData(item, { expression: 'itemlabel', bindExpression: _this.binditemlabel }, _this.viewParent) || item.label,
                        icon: getEvaluatedData(item, { expression: 'itemicon', bindExpression: _this.binditemicon }, _this.viewParent) || item.icon,
                        link: link,
                        active: _.includes([activePageName, '#' + activePageName, '#/' + activePageName], link)
                    };
                });
            }
            else {
                return this.getItems(value);
            }
        }
        else if (_.isString(value)) {
            return this.getItems(value.split(','));
        }
    };
    var MobileTabbarComponent_1;
    MobileTabbarComponent.initializeProps = registerProps();
    MobileTabbarComponent.ctorParameters = function () { return [
        { type: PageDirective },
        { type: Injector },
        { type: undefined, decorators: [{ type: Attribute, args: ['itemlabel.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['itemicon.bind',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['itemicon.bind',] }] }
    ]; };
    MobileTabbarComponent = MobileTabbarComponent_1 = __decorate([
        Component({
            selector: 'div[wmMobileTabbar]',
            template: "<nav class=\"navbar navbar-default\">\n    <ul class=\"tab-items nav navbar-nav\">\n        <li class=\"tab-item\" *ngFor=\"let item of tabItems; index as i\" [class.hidden]=\"!(tabItems.length === layout.max || i < layout.max)\" >\n            <a [class.active]=\"item.active\" [href]=\"(item.link || 'javascript:void(0)')| trustAs: 'resource'\" (click)=\"onItemSelect($event, item)\">\n                <i class=\"app-icon\" [ngClass]=\"item.icon\"></i><label>{{item.label}}</label>\n            </a>\n        </li>\n        <li class=\"menu-items dropdown\" [class.hidden]=\"tabItems.length <= layout.max\" [ngClass]=\"{dropup : position === bottom}\">\n            <a (click)=\"showMoreMenu = !showMoreMenu\" href=\"javascript:void(0)\">\n                <i class=\"app-icon {{morebuttoniconclass}}\"></i><label>{{morebuttonlabel}}</label>\n            </a>\n            <ul class=\"dropdown-menu dropdown-menu-right\" [ngClass]=\"{'nav navbar-nav' : menutype === thumbnail}\" *ngIf=\"showMoreMenu\">\n                <li role=\"menuitem\" class=\"menu-item\" *ngFor=\"let item of tabItems;index as i\" [class.hidden]=\"i < layout.max\">\n                    <a [ngClass]=\"{active : item.active}\" [href]=\"(item.link || 'javascript:void(0)')| trustAs: 'resource'\" (click)=\"onItemSelect($event, item)\" class=\"dropdown-item\">\n                        <i class=\"app-icon\" [ngClass]=\"item.icon\"></i><label>{{item.label}}</label>\n                    </a>\n                </li>\n            </ul>\n        </li>\n    </ul>\n</nav>\n",
            providers: [
                provideAsWidgetRef(MobileTabbarComponent_1)
            ]
        }),
        __param(2, Attribute('itemlabel.bind')),
        __param(3, Attribute('itemicon.bind')),
        __param(4, Attribute('itemicon.bind')),
        __metadata("design:paramtypes", [PageDirective,
            Injector, Object, Object, Object])
    ], MobileTabbarComponent);
    return MobileTabbarComponent;
}(StylableComponent));

var components = [
    MobileTabbarComponent
];
var TabBarModule = /** @class */ (function () {
    function TabBarModule() {
    }
    TabBarModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                WmComponentsModule
            ],
            declarations: __spread(components),
            exports: __spread(components),
            entryComponents: []
        })
    ], TabBarModule);
    return TabBarModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { MobileTabbarComponent, TabBarModule, registerProps as ɵa };
//# sourceMappingURL=index.js.map
