import { LocalDBManagementService } from './local-db-management.service';
import { LocalDBStore } from '../models/local-db-store';
export declare class LocalDbService {
    private localDBManagementService;
    static readonly SERVICE_NAME = "LocalDbService";
    private searchTableData;
    private searchTableDataWithQuery;
    private getDistinctDataByFields;
    constructor(localDBManagementService: LocalDBManagementService);
    getStore(params: any): Promise<LocalDBStore>;
    /**
     * Method to insert data into the specified table. This modification will be added to offline change log.
     *
     * @param {object} params
     *                 Object containing name of the project & table data to be inserted.
     * @param {function=} successCallback
     *                    Callback function to be triggered on success.
     * @param {function=} failureCallback
     *                    Callback function to be triggered on failure.
     */
    insertTableData(params: any, successCallback?: any, failureCallback?: any, options?: {}): void;
    /**
     * Method to insert multi part data into the specified table. This modification will be added to offline change log.
     *
     * @param {object} params
     *                 Object containing name of the project & table data to be inserted.
     * @param {function=} successCallback
     *                    Callback function to be triggered on success.
     * @param {function=} failureCallback
     *                    Callback function to be triggered on failure.
     */
    insertMultiPartTableData(params: any, successCallback?: any, failureCallback?: any, options?: {}): void;
    /**
     * Method to update data in the specified table. This modification will be added to offline change log.
     *
     * @param {object} params
     *                 Object containing name of the project & table data to be updated.
     * @param {function=} successCallback
     *                    Callback function to be triggered on success.
     * @param {function=} failureCallback
     *                    Callback function to be triggered on failure.
     */
    updateTableData(params: any, successCallback?: any, failureCallback?: any): void;
    /**
     * Method to update multi part data in the specified table. This modification will be added to offline change log.
     *
     * @param {object} params
     *                 Object containing name of the project & table data to be updated.
     * @param {function=} successCallback
     *                    Callback function to be triggered on success.
     * @param {function=} failureCallback
     *                    Callback function to be triggered on failure.
     */
    updateMultiPartTableData(params: any, successCallback?: any, failureCallback?: any): void;
    /**
     * Method to delete data in the specified table. This modification will be added to offline change log.
     *
     * @param {object} params
     *                 Object containing name of the project & table data to be inserted.
     * @param {function=} successCallback
     *                    Callback function to be triggered on success.
     * @param {function=} failureCallback
     *                    Callback function to be triggered on failure.
     */
    deleteTableData(params: any, successCallback?: any, failureCallback?: any): void;
    /**
     * Method to read data from a specified table.
     *
     * @param {object} params
     *                 Object containing name of the project & table data to be inserted.
     * @param {function=} successCallback
     *                    Callback function to be triggered on success.
     * @param {function=} failureCallback
     *                    Callback function to be triggered on failure.
     */
    readTableData(params: any, successCallback?: any, failureCallback?: any): void;
    private escapeName;
    private getColumnName;
    private convertFieldNameToColumnName;
}
